<?php

namespace Easy_Query;

class Base extends Easy_Query {
	protected $the_query_order = array();
	protected $current_query = array();
	protected $where_groups = array();
	protected $prepared_params = array();
	protected $prepared_params_types = '';
	public $query_string = '';
	protected $current_where_group = FALSE;
	public function limit($limit = 10) {
		$this->current_query['limit'] = "LIMIT ".$limit;
	}
	public function offset($offset = 0) {
		$this->current_query['offset'] = "OFFSET ".$offset;
	}
	public function join($primary, $foriegn, $type = 'LEFT JOIN') {
		if(!isset($this->current_query['join'])) {
			$this->current_query['join'] = array();
		}
		list($foriegn_table, $foriegn_key) = explode('.', $foriegn);
		$join = $type.' '.$foriegn_table.' ON '.$primary.' = '.$foriegn;
		$this->current_query['join'][] = $join;
	}
	public function group_by($key = '') {
		$this->current_query['group_by'] = "GROUP BY ".$key;
	}
	public function order_by($key = '') {
		$this->current_query['order_by'] = "ORDER BY ".$key;
	}
	public function select($select = '') {
		$this->current_query['select'] = "SELECT ".$select." FROM";
	}
	public function group_start($relation = 'AND') {
		$this->current_where_group = count($this->where_groups);
		$this->where_groups[$this->current_where_group] = array();
		$this->where_groups[$this->current_where_group]['group_relation'] = $relation;
	}
	public function group_end() {
		$this->current_where_group = FALSE;
	}
	public function set_relation($relation = 'AND') {
		if(!isset($this->where_groups[$this->current_where_group]['relation'])) {
			$this->where_groups[$this->current_where_group]['relation'] = $relation;
		}
	}
	public function _where($arr = array(), $relation = 'AND') {
		if($this->current_where_group === FALSE) {
			$this->group_start();
		}

		if(!isset($this->where_groups[$this->current_where_group])) {
			$this->where_groups[$this->current_where_group] = array();
		}
		if(!isset($this->where_groups[$this->current_where_group]['relation'])) {
			$this->where_groups[$this->current_where_group]['relation'] = $relation;
		}
		$this->where_groups[$this->current_where_group][] = $arr;
	}
	public function where_in($key = '', $value = array(), $relation = 'AND') {
		$this->_where(array(
			'key' => $key,
			'value' => $value,
			'compare' => 'IN'
		), $relation);
	}
	public function where($key, $value, $compare = '=', $relation = 'AND') {
		$this->_where(array(
			'key' => $key,
			'value' => $value,
			'compare' => $compare
		), $relation);
	}
	protected function create_join_str() {
		if(isset($this->current_query['join'])) {
			$this->current_query['join'] = implode(' ', $this->current_query['join']);
		}
	}
	protected function prepare_params_type() {
		foreach ($this->prepared_params as $key => $value) {
			if(is_int($value)) {
				$this->prepared_params_types .= 'i';
			} else if(is_float($value)) {
				$this->prepared_params_types .= 'd';
			} else {
				$this->prepared_params_types .= 's';
			}
		}
	}
	protected function empty($val) {
		if($val === 0) {
			return FALSE;
		}
		return empty($val);
	}
	protected function prepare_where() {
		$prepared = array();
		$prepared_params = array();
		$prepared[] = 'WHERE';
		if(!empty($this->where_groups)) {
			foreach ($this->where_groups as $group_num => $group_values) {
				if($group_num !== 0) {
					$prepared[] = $group_values['group_relation'];
				}
				$child_relation = $group_values['relation'];
				unset($group_values['group_relation']);
				unset($group_values['relation']);
				$multi_where = array();
				foreach ($group_values as $num => $where) {
					if(!isset($where['compare'])) {
						$where['compare'] = '=';
					}
					$is_null = FALSE;
					if(is_array($where['value'])) {
						$in_values = array();
						foreach ($where['value'] as $wkey => $wvalue) {
							if(!$this->empty($wvalue)) {
								$in_values[] = '?';
								$prepared_params[] = $wvalue;
							} else {
								$is_null = TRUE;
							}
						}
						$placeholder = '('. implode(', ', $in_values) .')';
					} else {
						if(!$this->empty($where['value'])) {
							$placeholder = '?';
							$prepared_params[] = $where['value'];
						} else {
							$is_null = TRUE;
						}
					}
					if($is_null === TRUE && !is_array($where['value'])) {
						$_current = "(".$where['key']." = ''";
						$_current .= ' OR ';
						$_current .= $where['key']." IS NULL)";
						$multi_where[] = $_current;
					} else if($is_null === TRUE && is_array($where['value'])) {
						$_current = "";
						if(!empty($placeholder) && $placeholder !== '()') {
							$_current .= "(". $where['key']." ".$where['compare']." ".$placeholder;
							$_current .= " OR ";
						}
						$_current .= "(".$where['key']." = ''";
						$_current .= ' OR ';
						$_current .= $where['key']." IS NULL)";
						if(!empty($placeholder) && $placeholder !== '()') {
							$_current .= ")";
						}
						$multi_where[] = $_current;
					} else {
						$multi_where[] = $where['key']." ".$where['compare']." ".$placeholder;
					}
				}
				$prepared[] = '(' . implode(" ".$child_relation." ", $multi_where) . ')';
			}
		}
		$this->prepared_params = $prepared_params;
		$this->current_query['where'] = implode(' ', $prepared);
	}
	protected function fix_current_query_order() {
		$new_current_query = array();
		$order = $this->the_query_order;
		foreach ($order as $key => $value) {
			if(isset($this->current_query[$value])) {
				$new_current_query[$value] = $this->current_query[$value];
			}
		}
		$this->current_query = $new_current_query;
	}
	public function _reset() {
		parent::reset();
		$this->current_query = array();
		$this->where_groups = array();
		$this->prepared_params = array();
		$this->prepared_params_types = '';
		$this->query_string = '';
		$this->current_where_group = FALSE;
	}
}
