<?php

namespace Easy_Query;

class Easy_Query {
	public $db;
	public $current_statement;
	public function __construct() {
		$this->db = isset($GLOBALS['eq_mysqli']) ? $GLOBALS['eq_mysqli'] : null;
	}
	public function escape_string($str = '') {
		return $this->db->real_escape_string($str);
	}
	public function bind_params($type = '', $params = array()) {
		if(!isset($this->current_statement) || empty($this->current_statement) || empty($type) || empty($params)) {
			return;
		}
		if(strlen($type) !== count($params)) {
			exit('Number of paramters in argument #2 must match with value provided in the first argument #1.');
		}
		$transfer = array($type);
		foreach ($params as $key => $value) {
			$transfer[] = &$params[$key];
		}
		call_user_func_array(array($this->current_statement, 'bind_param'), $transfer);
	}
	public function reset() {
		$this->current_statement = NULL;
	}
	public function filter_sql_query($sql = '') {
		$filter = isset($GLOBALS['easy_query_filter_current_sql']) ? $GLOBALS['easy_query_filter_current_sql'] : '';
		if($filter && is_callable($filter)) {
			unset($GLOBALS['easy_query_filter_current_sql']);
			$sql = $filter($sql);
		}
		return $sql;
	}
	public function query($sql = '') {
		$this->reset();
		if(!$this->db) {
			return FALSE;
		}
		$sql = $this->filter_sql_query($sql);
		$this->current_statement = $this->db->prepare($sql);
		return isset($this->current_statement) && !empty($this->current_statement);
	}
	public function insert_id() {
		if(!isset($this->current_statement) || empty($this->current_statement)) {
			return FALSE;
		}
		if(property_exists($this->current_statement, 'insert_id')) {
			return $this->current_statement->insert_id;
		}
		return FALSE;
	}
	public function execute() {
		if(!isset($this->current_statement) || empty($this->current_statement)) {
			return FALSE;
		}
		$result = $this->current_statement->execute();
		return $result === 1 || $result === TRUE;
	}
	public function get_results() {
		if(!isset($this->current_statement) || empty($this->current_statement)) {
			return FALSE;
		}
		$result = $this->current_statement->get_result();
		if(!empty($result) && (int) $result->num_rows > 0) {
			$r = $result->fetch_all(MYSQLI_ASSOC);
		} else {
			$r = array();
		}
		if(is_object($result) && method_exists($result, 'free_result')) {
			$result->free_result();
		}
		return $r;
	}
}
