<?php

namespace Easy_Query;

class Insert extends Select {
	protected $required_quries_arr = array();
	protected $required_quries = array();
	protected $required_quries_values = array();
	protected $insert_binded_params = array();
	protected $insert_query_strings = array();
	protected $_primary_insert_table = '';
	public $last_insert_id = NULL;
	protected function create_insert_into_part($input) {
		$quries = array();
		foreach ($input as $table_name => $fieldsArr) {
			$fields = implode(', ', $fieldsArr);
			$quries[$table_name] = 'INSERT INTO '.$table_name.' ('.$fields.')';
		}
		$this->required_quries = $quries;
		return $quries;
	}
	protected function separate_query($fields) {
		$tables = array();
		foreach ($fields as $num => $field) {
			if(strpos($field, '.') !== FALSE) {
				list($table_name, $field) = explode('.', $field);
			} else {
				$table_name = $this->_primary_insert_table;
			}
			if(!isset($tables[$table_name])) {
				$tables[$table_name] = array();
			}
			$tables[$table_name][] = $field;
		}
		$this->required_quries_arr = $tables;
		return $this->create_insert_into_part($tables);
	}
	protected function prepare_insert_values($fields, $data) {
		$table_values = array();
		foreach ($data as $num => $row) {
			foreach ($fields as $_i => $field) {
				$value = isset($row[$field]) ? $row[$field] : NULL;
				if(strpos($field, '.') !== FALSE) {
					list($table_name, $field) = explode('.', $field);
				} else {
					$table_name = $this->_primary_insert_table;
				}
				if(!isset($table_values[$table_name])) {
					$table_values[$table_name] = array();
				}
				if(!isset($table_values[$table_name][$num])) {
					$table_values[$table_name][$num] = array();
				}
				$table_values[$table_name][$num][$field] = $value;
			}
		}
		$this->required_quries_values = $table_values;
	}
	protected function handle_multiple_insert_query($data) {
		$fields = array();
		foreach ($data as $key => $row) {
			foreach ($row as $field => $value) {
				if(array_search($field, $fields) === FALSE) {
					$fields[] = $field;
				}
			}
		}
		$this->prepare_insert_values($fields, $data);
		return $this->separate_query($fields);
	}
	protected function isNestedArray($data = array()) {
		foreach ($data as $key => $value) {
			if(is_string($value)) {
				return FALSE;
			}
		}
		return TRUE;
	}
	protected function single_mapCreateValues($table_name = '') {
		$mapedValues = array();
		$keyArr = $this->required_quries_arr[$table_name];
		$valArr = $this->required_quries_values[$table_name];
		foreach ($valArr as $num => $value) {
			$newValues = array();
			foreach ($keyArr as $_num => $field) {
				if(isset($value[$field])) {
					$newValues[] = $value[$field];
				} else {
					$newValues[] = NULL;
				}
			}
			$mapedValues[] = $newValues;
		}
		return $mapedValues;
	}
	protected function createPlaceHolders($table_name = '', $fields = array()) {
		if(!isset($this->insert_binded_params[$table_name])) {
			$this->insert_binded_params[$table_name] = array();
		}
		foreach ($fields as $key => $value) {
			$fields[$key] = '?';
			$this->insert_binded_params[$table_name][] = $value;
		}
		return $fields;
	}
	protected function createMapValuesSQL($tables = array()) {
		$sql_strings = array();
		$query_string = array();
		foreach ($tables as $table_name => $values) {
			if(!isset($sql_strings[$table_name])) {
				$sql_strings[$table_name] = array();
			}
			foreach ($values as $num => $fields) {
				$sql_strings[$table_name][] = implode(',', $this->createPlaceHolders($table_name, $fields));
			}
		}
		foreach ($sql_strings as $table_name => $values) {
			$query_string[$table_name] = '('.implode('), (', $values).')';
		}
		$this->insert_query_strings = $query_string;
		foreach ($this->insert_query_strings as $table_name => $str) {
			$this->insert_query_strings[$table_name] = $this->required_quries[$table_name] . ' VALUES '.$this->insert_query_strings[$table_name];
		}
	}
	protected function mapCreateValues() {
		$tables = array();
		foreach ($this->required_quries_arr as $table_name => $value) {
			$tables[$table_name] = $this->single_mapCreateValues($table_name);
		}
		$this->createMapValuesSQL($tables);
	}
	protected function get_params_types($params) {
		$sql_binded_param_type = '';
		foreach ($params as $key => $value) {
			if(is_int($value)) {
				$sql_binded_param_type .= 'i';
			} else if(is_float($value)) {
				$sql_binded_param_type .= 'd';
			} else {
				$sql_binded_param_type .= 's';
			}
		}
		return $sql_binded_param_type;
	}
	public function parse_params($params, $primary_insert = NULL) {
		foreach ($params as $key => $value) {
			if($value === '%insert_id%') {
				if($primary_insert === NULL) {
					throw new Exception("Error Processing SQL Insert.", 1);
				}
				$params[$key] = $primary_insert;
			}
		}
		return $params;
	}
	protected function _handle_insert($table_name = '', $data = array()) {
		$this->_primary_insert_table = $table_name;
		if(!$this->isNestedArray($data)) {
			$data = array($data);
		}
		$result = $this->handle_multiple_insert_query($data);
		$this->mapCreateValues();
		$primary_insert = NULL;
		foreach ($this->insert_query_strings as $table_name => $query) {
			$params = $this->parse_params($this->insert_binded_params[$table_name], $primary_insert);
			$param_types = $this->get_params_types($params);
			$this->query($query);
			$this->bind_params($param_types, $params);
			if($this->execute() === FALSE) {
				$this->reset();
				$this->_reset();
				return FALSE;
			}
			$this->last_insert_id = $this->insert_id();
			if($this->_primary_insert_table === $table_name) {
				$primary_insert = $this->last_insert_id;
			}
			$this->reset();
		}
		$this->_reset();
		return TRUE;
	}
	public function insert($table_name = '', $data = array()) {
		$result = array();
		$insert_ids = array();
		if(!$this->isNestedArray($data)) {
			$data = array($data);
			$result[] = $this->_handle_insert($table_name, $data);
			$insert_ids[] = $this->last_insert_id;
		} else {
			foreach ($data as $key => $row) {
				$result[] = $this->_handle_insert($table_name, array($row));
				$insert_ids[] = $this->last_insert_id;
			}
		}
		$this->last_insert_id = NULL;
		if(array_search(FALSE, $result) === FALSE) {
			return $insert_ids;
		}
		return FALSE;
	}
	public function _reset() {
		$this->required_quries_arr = array();
		$this->required_quries = array();
		$this->required_quries_values = array();
		$this->insert_binded_params = array();
		$this->insert_query_strings = array();
		$this->_primary_insert_table = '';
	}
}
