<?php

// Avoid compatibility issues with Other
// Plugins that might be using Same
// Functions / Class Name
namespace RepairPluginPro;

// Exit if trying to run the script directly
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

// verify_next_run_token()
// get_next_run_token()

class CRON_Jobs_Handler {
	public $maximum_token_wait_in_seconds = 30;
	public $loop_schedule_in_seconds = 3;
	public $last_token_time = NULL;
	public $future_time_in_seconds;
	public function __construct() {
		$this->last_token_time = rp_get_option('rp_cron_jobs_next_token', NULL);
	}
	public function verify_next_run_token() {
		// Has run token in GET or POST?
		// If yes, TRUE
		// else, FALSE
		return $_REQUEST['rp_run_token'] ?? FALSE;
	}
	protected function _get_next_run_token() {
		// 1. If have no last_token_time,
		// 2. Last Token is Old?
		// New token can be generated without verification
		// Create a new and return...
		if($this->last_token_time === NULL || $this->last_token_time < time()) {
			$this->last_token_time = time() + $this->loop_schedule_in_seconds;
			return $this->last_token_time;
		}

		$this->future_time_in_seconds = $this->last_token_time - time();

		// Check if future time + 5 seconds are greater
		// than 30???
		if(($this->future_time_in_seconds + $this->loop_schedule_in_seconds) > $this->maximum_token_wait_in_seconds) {
			// No need to generate the token...
			return FALSE;
		}
		return $this->last_token_time + $this->loop_schedule_in_seconds;
	}
	public function get_next_run_token() {
		$token = $this->_get_next_run_token();
		if($token !== FALSE) {
			$this->_save_next_run_token($token);
		}
		return $token;
	}
	protected function _save_next_run_token($token = '') {
		if(rp_get_option('rp_cron_jobs_next_token', FALSE) === FALSE) {
			rp_add_option('rp_cron_jobs_next_token', $token);
		} else {
			rp_update_option('rp_cron_jobs_next_token', $token);
		}
	}
}

function rp_cron_job_next() {
	$rp_cron_jobs = new CRON_Jobs_Handler();
	$token = $rp_cron_jobs->get_next_run_token();
	if($token !== FALSE) {
		$token = $token - time();
	}
	return $token;
}

function rp_cron_job_verifiy($data = array()) {
	$rp_cron_jobs = new CRON_Jobs_Handler();
	if($rp_cron_jobs->verify_next_run_token() === FALSE) {
		rp_cron_job_response($data);
	}
}

function rp_cron_job_response($data = array()) {
	$data['rp_try_after'] = rp_cron_job_next();
	print_r(json_encode($data));
	exit();
}

// rp_cron_job_verifiy(array(
// 	'error' => 'No cron token found!'
// ));

// rp_cron_job_response(array(
// 	'success' => 'Awesome my friend!'
// ));
