<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class Download_Languages {

	protected $fetch_url = 'https://base.repairplugin.com/Endpoints/fetch_languages';

	public function __construct($rpQuery = NULL) {

		$this->wpdb = $rpQuery;

		$this->fetch_url = \WP_REPAIR_BASE_SERVER_URL.'Endpoints/fetch_languages';

	}
	
	public function fetch_repair_base($gzip = TRUE, $names_only = FALSE) {
		$license_key = rp_get_option('Repairplugin_lic_Key', '');
		$license_email = rp_get_option('Repairplugin_lic_email', '');
		
		if(empty($license_key)) {
			return array();
		}

		$curl = curl_init( $this->fetch_url );

		if($gzip == TRUE) {
			curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
		}

		$payload = array();

		if($names_only === TRUE) {

			$payload = array('license_key' => $license_key, 'names_only' => 'yes');

			curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

		} else {

			if(isset($_POST['lang_uq_ids']) && !empty($_POST['lang_uq_ids']) && is_array($_POST['lang_uq_ids'])) {

				$lang_uq_ids = json_encode($_POST['lang_uq_ids']);

				$rp_base_class = new \RepairpluginBase(WP_REPAIR_PLUGIN_FILE);
				$license_app_version = $rp_base_class->getCurrentVersion();
				$domain = $rp_base_class->getDomain();

                $dr_uq_ids = json_encode(\rp_get_all_dr_uq_id_from_rs_default_repair_table());

                $da_uq_ids = json_encode(\rp_get_all_da_uq_id_from_rs_default_repair_attr_table());

				$payload = array('license_key' => $license_key, 'license_email' => $license_email, 'license_app_version' => $license_app_version, 'license_domain' => $domain, 'lang_uq_ids' => $lang_uq_ids, 'dr_uq_ids' => $dr_uq_ids, 'da_uq_ids' => $da_uq_ids);

				curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

			} else {

				$payload = array('license_key' => $license_key);

				curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

			}

		}

		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );

		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false );

		curl_setopt( $curl, CURLOPT_SSL_VERIFYHOST, false );

		$response = curl_exec( $curl );

		$info = curl_getinfo( $curl );

		curl_close( $curl );

		$data = array();

		if($info['http_code'] != '200') {
			return $data;
		}

		if(!empty($response)) {
			try {
				$data = json_decode($response);
			} catch (\Exception $e) {

			}
		}
		if(!empty($data) && property_exists($data, 'status') && ($data->status === TRUE || $data->status === 1)) {
			return $data;
		}
		return array();
	}

	public function prepare_need_to_add_data( $data = array() ) {

		$prepared_data = array();

		$prepared_data['languages'] = array();

		$prepared_data['translations'] = array();

		$prepared_data['repair_translations'] = array();

		$prepared_data['attr_translations'] = array();

		$alreadyDownloaded = \rp_get_uq_id_of_all_rs_languages();

		if( !empty( $data ) && is_object($data) ) {

			if( isset($data->languages) && !empty($data->languages) ) {

				foreach($data->languages as $language) {

					if( !in_array( $language->uq_id, $alreadyDownloaded ) ) {

						$prepared_data['languages'][] = $language;

					}

				}

			}

			if( isset($data->translations) && !empty($data->translations) ) {

				$prepared_data['translations'] = $data->translations;

			}

			if( isset($data->repair_translations) && !empty($data->repair_translations) ) {

				$prepared_data['repair_translations'] = $data->repair_translations;

			}

			if( isset($data->attr_translations) && !empty($data->attr_translations) ) {

				$prepared_data['attr_translations'] = $data->attr_translations;

			}

		}

		return $prepared_data;

	}

	public function add_new_languages( $languages = array() ) {

		if( !empty( $languages ) ) {

			$rs_languages = $this->wpdb->prefix . 'rs_languages';

			foreach( $languages as $language_row ) {

				// count all rs_languages rows

				$count = $this->wpdb->get_var( "SELECT COUNT(*) FROM `$rs_languages`" );

				$position = $count + 1;

				$this->wpdb->insert(

					$rs_languages,

					array(

						'key' => $language_row->key,

						'name' => $language_row->name,

						'alt_name' => $language_row->alt_name,

						'image' => $language_row->image,

						'position' => $position,

						'is_active' => $language_row->is_active,

						'uq_id' => $language_row->uq_id

					)

				);

			}

		}

	}

	public function add_new_translations( $translations = array() ) {

		if( !empty( $translations ) ) {

			$rs_translations = $this->wpdb->prefix . 'rs_translations';

			foreach( $translations as $translation_row ) {

				$this->wpdb->insert(

					$rs_translations,

					array(

						'language' => $translation_row->language,

						'key' => $translation_row->key,

						'value' => $translation_row->value

					)

				);

			}

		}

	}

	public function delete_existing_translations( $translations = array() ) {

		if( !empty( $translations ) ) {

			$rs_translations = $this->wpdb->prefix . 'rs_translations';

			foreach( $translations as $translation_row ) {

				$this->wpdb->query( "DELETE FROM $rs_translations WHERE `key` = '".rp_escape_sql($translation_row->key)."' AND `language` = '".rp_escape_sql($translation_row->language)."' " );

			}

		}

	}

	public function add_new_repair_translations( $repair_translations = array() ) {

		if( !empty( $repair_translations ) ) {

			$rs_dr_translations = $this->wpdb->prefix . 'rs_dr_translations';

			foreach($repair_translations as $rt_row) {

				// check if already exists
				$check = $this->wpdb->get_row( "SELECT * FROM $rs_dr_translations WHERE `key` = '".rp_escape_sql($rt_row->key)."' AND `uq_id` = '".rp_escape_sql($rt_row->uq_id)."' AND `language` = '".rp_escape_sql($rt_row->language)."' " );

				if( empty( $check ) ) {

					$this->wpdb->insert(

						$rs_dr_translations,
	
						array(
	
							'language' => $rt_row->language,
	
							'key' => $rt_row->key,
	
							'value' => $rt_row->value,
	
							'uq_id' => $rt_row->uq_id
	
						)
	
					);
	
					// create new if not already exists
					// \rp_create_matching_repair_translations_if_not_already($rt_row->key, $rt_row->uq_id, $rt_row->language, $rt_row->value);

				}

			}

		}

	}

	public function add_new_attr_translations( $attr_translations = array() ) {

		if( !empty( $attr_translations ) ) {

			$rs_dr_translations = $this->wpdb->prefix . 'rs_dr_translations';

			foreach($attr_translations as $rt_row) {

				// check if already exists
				$check = $this->wpdb->get_row( "SELECT * FROM `$rs_dr_translations` WHERE `key` = '".rp_escape_sql($rt_row->key)."' AND `uq_id` = '".rp_escape_sql($rt_row->uq_id)."' AND `language` = '".rp_escape_sql($rt_row->language)."' " );

				if( empty( $check ) ) {

					$this->wpdb->insert(

						$rs_dr_translations,
	
						array(
	
							'language' => $rt_row->language,
	
							'key' => $rt_row->key,
	
							'value' => $rt_row->value,
	
							'uq_id' => $rt_row->uq_id
	
						)
	
					);
	
					// create new if not already exists
					// \rp_create_matching_attr_translations_if_not_already($rt_row->key, $rt_row->uq_id, $rt_row->language, $rt_row->value);

				}

			}

		}

	}

	public function add_need_to_add_data( $data = array() ) {

		$this->wpdb->query("START TRANSACTION");

		if( isset( $data['languages'] ) ) {

			$this->add_new_languages( $data['languages'] );

		}

		if( isset( $data['translations'] ) ) {

			$this->add_new_translations( $data['translations'] );

		}

		if( isset( $data['repair_translations'] ) ) {

			$this->add_new_repair_translations( $data['repair_translations'] );

		}

		if( isset( $data['attr_translations'] ) ) {

			$this->add_new_attr_translations( $data['attr_translations'] );

		}

		$this->wpdb->query("COMMIT");

	}

	public function add_need_to_add_data_for_new_translations( $data = array() ) {

		$this->wpdb->query("START TRANSACTION");

		if( isset( $data['translations'] ) ) {

			$this->delete_existing_translations( $data['translations'] );

			$this->add_new_translations( $data['translations'] );

		}

		$this->wpdb->query("COMMIT");

	}

	public function save_handler($data = array()) {
		if(isset($_POST['lang_uq_ids'])) {
			if(!empty($data)) {
				
				$prepared_data = $this->prepare_need_to_add_data($data);

				if(!empty($prepared_data) && (!empty($prepared_data['languages']))) {

					\rp_add_or_update_option('rp_get_last_languages_downloaded_date', date('Y-m-d'));

					$this->add_need_to_add_data($prepared_data);

					echo json_encode(array('success' => 'new_downloads', 'counts' => array(

						'languages' => count($prepared_data['languages'])

					)));

					exit();
				}
				echo json_encode(array('success' => 'up_to_date_already'));
				exit();
			} else {
				echo json_encode(array('error' => 'Failed to fetch the languages from repair plugin server.'));
				exit();
			}
		}
	}

	public function save_handler_first_time($data = array()) {
		if(isset($_POST['lang_uq_ids'])) {
			if(!empty($data)) {
				
				$prepared_data = $this->prepare_need_to_add_data($data);

				if(!empty($prepared_data) && (!empty($prepared_data['languages']))) {

					\rp_add_or_update_option('rp_get_last_languages_downloaded_date', date('Y-m-d'));

					$this->add_need_to_add_data($prepared_data);
					
				}
			}
		}
	}

	public function filter_only_new_translations( $data = array(), $new_translation_keys = array() ) {

		$new_only = array();

		if( isset( $data['translations'] ) && !empty( $data['translations'] ) ) {

			foreach( $data['translations'] as $translation_row ) {

				if( array_search( $translation_row->key, $new_translation_keys ) !== false ) {

					$new_only[] = $translation_row;

				}

			}

		}

		return $new_only;

	}

	public function save_handler_new_translations($data = array(), $new_translation_keys = array()) {
		if(isset($_POST['lang_uq_ids'])) {
			if(!empty($data)) {
				
				$prepared_data = $this->prepare_need_to_add_data($data);

				if(!empty($prepared_data) && (!empty($prepared_data['translations']))) {

					$prepared_data['translations'] = $this->filter_only_new_translations($prepared_data, $new_translation_keys);

					if( !empty( $prepared_data['translations'] ) ) {

						$this->add_need_to_add_data_for_new_translations($prepared_data);

					}

					return true;
					
				}
			}
		}
		return false;
	}

}
