<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class Download_Libraries {

	protected $download_repairs = FALSE;

	protected $fetch_url = 'https://base.repairplugin.com/Endpoints/fetch_database';

	protected $category_table = '';

	protected $brand_table = '';

	protected $model_table = '';

	protected $dc_tablename = '';

	protected $dr_tablename = '';

	protected $da_tablename = '';

	protected $categories_data = array();

	protected $brands_data = array();

	protected $models_data = array();

	protected $default_repairs_data = array();

	protected $default_colors_data = array();

	protected $default_repair_attr_data = array();

	public function __construct($rpQuery = NULL) {

		$this->wpdb = $rpQuery;

		$this->fetch_url = \WP_REPAIR_BASE_SERVER_URL.'Endpoints/fetch_database';

		$this->set_table_names();

		$this->load_existing_data();

		$this->fix_post_array();

	}
	public function set_table_names() {

		$this->category_table = $this->wpdb->prefix . "rs_category";

		$this->brand_table = $this->wpdb->prefix . "rs_brand";

		$this->model_table = $this->wpdb->prefix . "rs_model";

		$this->dc_tablename = $this->wpdb->prefix . "rs_default_color";

		$this->dr_tablename = $this->wpdb->prefix . "rs_default_repair";

		$this->da_tablename = $this->wpdb->prefix . "rs_default_repair_attr";

	}
	public function fix_post_array() {
		if(isset($_POST) && !empty($_POST) && isset($_POST['the_dl_form'])) {
			$decoded = json_decode((stripslashes($_POST['the_dl_form'])), TRUE); 
			$_POST = array();
			$_POST['download_libraries'] = TRUE;
			$_POST['c_id'] = $decoded['c_id'];
			$_POST['b_id'] = $decoded['b_id'];
			$_POST['m_id'] = $decoded['m_id'];
		}
	}
	public function load_existing_data() {

		$this->categories_data = $this->get_existing_records($this->category_table, 'c_uq_id');

		$this->brands_data = $this->get_existing_records($this->brand_table, 'b_uq_id');

		$this->models_data = $this->get_existing_records($this->model_table, 'm_uq_id');

		$this->default_colors_data = $this->get_existing_records($this->dc_tablename, 'dc_uq_id');

		$this->default_repairs_data = $this->get_existing_records($this->dr_tablename, 'dr_uq_id');

		$this->default_repair_attr_data = $this->get_existing_records_for_repair_attrs($this->da_tablename);

	}
	public function save_last_update($data = array()) {
		$last_update = $data->current_time ?? date('Y-m-d H:i:s');
		if(rp_get_option('rp_download_models_last_update', FALSE) !== FALSE) {
			rp_update_option('rp_download_models_last_update', $last_update);
		} else {
			rp_add_option('rp_download_models_last_update', $last_update);
		}
	}
	public function save_handler($data = array()) {
		if(isset($_POST['download_libraries'])) {
			if(!empty($data)) {
				$prepared_data = $this->prepare_need_to_add_data($data);
				if(!empty($prepared_data) && (!empty($prepared_data['categories']) || !empty($prepared_data['brands']) || !empty($prepared_data['models']))) {
					$this->save_last_update($data);
					$this->add_need_to_add_data($prepared_data);
					\rp_process_colors_and_repair_for_model();
					// \rp_trigger_translations_update_for_all_dr_repairs();
					echo json_encode(array('success' => 'new_downloads', 'counts' => array(
						'categories' => count($prepared_data['categories']),
						'brands' => count($prepared_data['brands']),
						'models' => count($prepared_data['models'])
					)));
					exit();
				}
				echo json_encode(array('success' => 'up_to_date_already'));
				exit();
			} else {
				echo json_encode(array('error' => 'Failed to fetch the libraries from repair plugin server.'));
				exit();
			}
		}
	}
	public function get_existing_records($table_name = '', $column_key = '') {
		$col_keys = $column_key . ', '.str_replace('_uq', '', $column_key);
		// $rows = \Easy_Query\select_where($table_name, NULL, $col_keys);

		$rows = $this->wpdb->get_results("SELECT {$col_keys} FROM `{$table_name}`;");

		$ids = array();
		
		if(!empty($rows)) {
		
			foreach ($rows as $key => $value) {

				$value = (array) $value;
		
				$ids[$value[$column_key]] = $value[str_replace('_uq', '', $column_key)];
		
			}
		}
		return $ids;
	}
	public function get_existing_records_for_repair_attrs($table_name = '') {
		// $rows = \Easy_Query\select_where($table_name, NULL, 'a_uq_id, r_id_fk, a_id');
		$rows = $this->wpdb->get_results("SELECT da_uq_id, dr_id_fk, da_id FROM `{$table_name}`;");

		$ids = array();
		
		if(!empty($rows)) {
		
			foreach ($rows as $key => $value) {

				$value = (array) $value;
		
				$ids[$value['da_uq_id'].'-'.$value['dr_id_fk']] = array(

					'da_id' => $value['da_id'],

					'dr_id_fk' => $value['dr_id_fk']

				);
		
			}
		}
		
		return $ids;
	}
	public function decode_posted_data() {
		if(isset($_POST) && !empty($_POST) && isset($_POST['download_libraries'])) {

			$decoded = json_decode((stripslashes($_POST['download_libraries'])), TRUE); 

			$_POST = array();
			$_POST['download_libraries'] = TRUE;

			foreach ($decoded as $key => $value) {

				if(strpos($value['name'], '[]') !== FALSE) {

					$array_key = str_replace('[]', '', $value['name']);

					if(!isset($_POST[$array_key])) {

						$_POST[$array_key] = array();

					}
					$_POST[$array_key][] = $value['value'];

				} else {

					$_POST[$value['key']] = $value['value'];
				}

			}

		}
	}
	public function fetch_repair_base($gzip = TRUE, $names_only = FALSE) {
		$license_key = rp_get_option('Repairplugin_lic_Key', '');
		$license_email = rp_get_option('Repairplugin_lic_email', '');
		
		if(empty($license_key)) {
			return array();
		}

		$curl = curl_init( $this->fetch_url );

		if($gzip == TRUE) {
			curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
		}

		$payload = array();

		if($names_only === TRUE) {

			$last_updated = rp_get_option('rp_download_models_last_update', '');

			$payload = array('license_key' => $license_key, 'names_only' => 'yes', 'last_updated' => $last_updated);

			curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

		} else {

			if(isset($_POST['m_id']) && !empty($_POST['m_id']) && is_array($_POST['m_id'])) {

				$m_id = json_encode($_POST['m_id']);

				$rp_base_class = new \RepairpluginBase(WP_REPAIR_PLUGIN_FILE);
				$license_app_version = $rp_base_class->getCurrentVersion();
				$domain = $rp_base_class->getDomain();

				$payload = array('license_key' => $license_key, 'license_email' => $license_email, 'license_app_version' => $license_app_version, 'license_domain' => $domain, 'm_id' => $m_id);

				curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

			} else {

				$payload = array('license_key' => $license_key);

				curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

			}

		}

		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );

		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false );

		curl_setopt( $curl, CURLOPT_SSL_VERIFYHOST, false );

		$response = curl_exec( $curl );

		$info = curl_getinfo( $curl );

		curl_close( $curl );

		$data = array();

		if($info['http_code'] != '200') {
			return $data;
		}

		if(!empty($response)) {
			try {
				$data = json_decode($response);
			} catch (\Exception $e) {

			}
		}
		if(!empty($data) && property_exists($data, 'status') && ($data->status === TRUE || $data->status === 1)) {
			return $data;
		}
		return array();
	}
	public function _prepare_need_to_add_default_colors($fetched_colors = array()) {

		$need_to_add = array();

		$default_colors_data = $this->default_colors_data;

		foreach ($fetched_colors as $fetched_color) {

			$dc_uq_id = $fetched_color->dc_uq_id;

			if(isset($default_colors_data[$dc_uq_id])) {

				$dc_check = $default_colors_data[$dc_uq_id];

			} else {

				$dc_check = NULL;

			}

			if($dc_check === NULL) {

				$dc_newdata = array(

				'dc_id'=>$fetched_color->dc_id,

				'dc_name'=>$fetched_color->dc_name,

				'dc_code'=>$fetched_color->dc_code,

				'dc_uq_id'=>$dc_uq_id,

				);

				$need_to_add[] = $dc_newdata;

			}

		}

		return $need_to_add;

	}
	public function _get_all_fetched_repairs_uqid( $fetched_repairs = array() ) {

		$uqids = array();

		foreach ($fetched_repairs as $fetched_repair) {

			$uqids[] = $fetched_repair->dr_uq_id;

		}

		return $uqids;

	}
	public function _is_added_category( $c_id = '' ) {

		return !empty( $this->wpdb->get_row("SELECT * FROM {$this->category_table} WHERE c_id = '$c_id' LIMIT 1;") );

	}

	public function _is_adding_category( $c_id = '', $adding_categories = array() ) {

		if( empty( $adding_categories ) || !is_array( $adding_categories ) ) {

			return FALSE;

		}

		foreach ($adding_categories as $key => $value) {
			
			if( (int) $value['c_id'] === (int) $c_id ) {

				return TRUE;

			}

		}

		return FALSE;

	}

	public function _need_to_add_default_repairs_for_category( $c_id = '', $fetched_repairs = array(), $adding_categories = array() ) {

		if( !$this->_is_added_category( $c_id ) && !$this->_is_adding_category( $c_id, $adding_categories ) ) {

			return FALSE;

		}

		$rpQuery = $this->wpdb;

		$default_repair_table = $rpQuery->prefix. "rs_default_repair";

		$fetched_repairs_str ='\'' . join('\',\'', $this->_get_all_fetched_repairs_uqid($fetched_repairs)) . '\'';

		$result = $this->wpdb->get_row("SELECT * FROM {$default_repair_table} WHERE dr_category = '$c_id' AND dr_uq_id IN ({$fetched_repairs_str}) LIMIT 1;");

		if( empty( $result ) ) {

			return TRUE;

		}

		return FALSE;

	}
	public function _prepare_need_to_add_default_repairs($fetched_repairs = array(), $adding_categories = array()) {

		$need_to_add = array();

		$categories = array( 9, 10 );

		$rpQuery = $this->wpdb;

		$rs_category = $rpQuery->prefix . 'rs_category';

		$categories = $rpQuery->get_results("SELECT c_id FROM `$rs_category` WHERE `c_uq_id` IN ('category_560296', 'category_144588')");

		if( !empty($categories) ) {

			$new_categories = array();

			foreach($categories as $cat_row) {

				$new_categories[] = $cat_row->c_id;

			}

			$categories = $new_categories;

		} else {

			$categories = array();

		}

		$rs_category = $rpQuery->prefix . 'rs_category';

		$auto_increment_row = $rpQuery->get_row("SELECT `auto_increment` FROM INFORMATION_SCHEMA.TABLES WHERE table_name = '$rs_category' AND TABLE_SCHEMA = '".DB_NAME."' LIMIT 1;");

		$auto_increment = 1;

		if( !empty($auto_increment_row) ) {

			$auto_increment = $auto_increment_row->auto_increment;

		}

		foreach($adding_categories as $ac_index => $adding_cat_row) {

			if( in_array($adding_cat_row['c_uq_id'], array('category_560296', 'category_144588')) ) {

				$adding_cat_row['c_id'] = $auto_increment;

				$categories[] = $adding_cat_row['c_id'];

				$adding_categories[$ac_index] = $adding_cat_row;

				$auto_increment++;

			}

		}

		if( empty($categories) ) {

			return $need_to_add;

		}

		foreach ($categories as $c_id) {
			
			if( $this->_need_to_add_default_repairs_for_category( $c_id, $fetched_repairs, $adding_categories ) === TRUE ) {

				$position = 1;

				foreach ($fetched_repairs as $fetched_repair) {

			    	$dr_newdata = array(

			    		// 'dr_id'=>$fetched_repair->dr_id,

				        'dr_name'=>$fetched_repair->dr_name,

				        'dr_price'=>$fetched_repair->dr_price,

				        'dr_duration'=>$fetched_repair->dr_duration,

				        'dr_discription'=>$fetched_repair->dr_discription,

				        'dr_img'=>$fetched_repair->dr_img,

				        'dr_uq_id'=>$fetched_repair->dr_uq_id,

				        'dr_category'=>$c_id,

				        'dr_position'=>$position

				    );

					$need_to_add[] = $dr_newdata;

					$position++;

				}

			}

		}

		return $need_to_add;

	}
	public function _prepare_need_to_add_default_repair_attrs($fetched_repair_attrs = array()) {

		if($this->download_repairs === FALSE) {

			return array();

		}

		$need_to_add = array();

		$default_repair_attr_data = $this->default_repair_attr_data;

		foreach ($fetched_repair_attrs as $fetched_repair_attr) {

			$da_uq_id = $fetched_repair_attr->da_uq_id;

			$dr_id_fk = $fetched_repair_attr->dr_id_fk;	

			if(isset($default_repair_attr_data[$da_uq_id.'-'.$dr_id_fk])) {

				$da_check = $default_repair_attr_data[$da_uq_id.'-'.$dr_id_fk];

			} else {

				$da_check = NULL;

			}

			if($da_check === NULL) {

		    	$da_newdata = array(

		    		'da_id'=>$fetched_repair_attr->a_id,

			        'da_name'=>$fetched_repair_attr->da_name,

			        'da_price'=>$fetched_repair_attr->da_price,

			        'dr_id_fk'=>$dr_id_fk,

			        'da_uq_id'=>$da_uq_id

			    );

			    $need_to_add[] = $da_newdata;

			}

		}

		return $need_to_add;

	}
	public function prepare_need_to_add_categories_brands_models($fetched_data = array()) {
		$need_to_add_categories = array();
		$need_to_add_brands = array();
		$need_to_add_models = array();
		$c_id = isset($_POST['c_id']) ? $_POST['c_id'] : '';
		$b_id = isset($_POST['b_id']) ? $_POST['b_id'] : '';
		$m_id = isset($_POST['m_id']) ? $_POST['m_id'] : '';
		if(!empty($c_id)) {
			$fetched_data_c_key_pair = array();
			foreach ($fetched_data as $final) {
				$fetched_data_c_key_pair[$final->c_uq_id] = $final;
			}
			$cat_count = count($c_id);
			for ($i=0; $i < $cat_count; $i++) { 
				if(isset($fetched_data_c_key_pair[$c_id[$i]])) {
					$final = $fetched_data_c_key_pair[$c_id[$i]];
					if(isset($this->categories_data[$c_id[$i]])) {
						$c_check = $this->categories_data[$c_id[$i]];
					} else {
						$c_check = NULL;
					}
					if($c_check === NULL) {
				    	$c_newdata = array(
				    		'c_id' =>$final->c_id,
					        'c_name'=>$final->c_name,
					        'c_img'=>$final->c_img,
					        'c_uq_id'=>$final->c_uq_id
					    );
					    $need_to_add_categories[] = $c_newdata;
					}

					$fetched_data_b_key_pair = array();
					foreach ($final->brands as $brand) {
						$fetched_data_b_key_pair[$brand->b_uq_id] = $brand;
					}

					$brand_count = count($b_id);
					for ($b=0; $b < $brand_count; $b++) {
						if(isset($fetched_data_b_key_pair[$b_id[$b]])) {
							$brand = $fetched_data_b_key_pair[$b_id[$b]];
							if(isset($this->brands_data[$b_id[$b]])) {
								$b_check = $this->brands_data[$b_id[$b]];
							} else {
								$b_check = NULL;
							}
							if($b_check === NULL) {
								$b_newdata = array(
									// 'b_id'=>$brand->b_id,
									'b_name'=>$brand->b_name,
									'b_img'=>$brand->b_img,
									'b_uq_id'=>$brand->b_uq_id,
									'c_id_fk'=>$final->c_uq_id
								);
								$need_to_add_brands[] = $b_newdata;
							}

							$fetched_data_m_key_pair = array();
							foreach ($brand->models as $model) {
								$fetched_data_m_key_pair[$model->m_uq_id] = $model;
							}

							$model_count = count($m_id);

							for ($m=0; $m < $model_count; $m++) {
								if(isset($fetched_data_m_key_pair[$m_id[$m]])) {
									$model = $fetched_data_m_key_pair[$m_id[$m]];
									if(isset($this->models_data[$m_id[$m]])) {
										$m_check = $this->models_data[$m_id[$m]];
									} else {
										$m_check = null;
									}
									if($m_check === NULL) {
										$default_repair_ids = $model->repair_ids;
										if($this->download_repairs === FALSE) {
											$default_repair_ids = '';
										}
								    	$m_newdata = array(
								    		// 'm_id' =>$model->m_id,
									        'm_name'=>$model->m_name,
									        'm_img'=>$model->m_img,
									        'device_type'=>$final->c_uq_id,
									        'brand_id_fk'=>$brand->b_uq_id,
									        'm_code'=>$model->m_code,
									        'release_date'=>$model->release_date,
									        'm_uq_id'=>$model->m_uq_id,
									        'repair_ids'=>$default_repair_ids,
									        'color_ids'=>$model->color_ids
									    );
									    $need_to_add_models[] = $m_newdata; 
									}
								}
							}

						}
					}
				}
			}
		}
		return array(
			'categories' => $need_to_add_categories,
			'brands' => $need_to_add_brands,
			'models' => $need_to_add_models
		);
	}
	public function prepare_need_to_add_data($data = NULL) {
		if(empty($data)) {
			return array();
		}
		$final_data = $data->all_records->categories;
		$colors = $data->all_records->colors;
		$repairs = $data->all_records->repairs;
		$r_attr = $data->all_records->repair_attributes;
		if(!is_array($r_attr)) {
			$r_attr = array();
		}
		$need_to_add_colors = $this->_prepare_need_to_add_default_colors($colors);
		$need_to_add_repair_attrs = $this->_prepare_need_to_add_default_repair_attrs($r_attr);
		$prepared_data = $this->prepare_need_to_add_categories_brands_models($final_data);

		// $need_to_add_repairs = $this->_prepare_need_to_add_default_repairs($repairs, $prepared_data['categories']);
		$need_to_add_repairs = array();

		foreach($prepared_data['categories'] as $index => $cat_row) {

			if( isset($cat_row['c_id']) ) {

				unset( $cat_row['c_id'] );

			}

			$prepared_data['categories'][$index] = $cat_row;

		}

		$prepared_data['colors'] = $need_to_add_colors;
		$prepared_data['repairs'] = $need_to_add_repairs;
		$prepared_data['repair_attrs'] = $need_to_add_repair_attrs;

		return $prepared_data;
	}
	public function add_need_to_add_data($prepared_data = array()) {
		\Easy_Query\insert_transaction($this->dc_tablename, $prepared_data['colors']);
		\Easy_Query\insert_transaction($this->dr_tablename, $prepared_data['repairs']);
		\Easy_Query\insert_transaction($this->da_tablename, $prepared_data['repair_attrs']);
		\Easy_Query\insert_transaction($this->category_table, $prepared_data['categories']);
		$this->categories_data = $this->get_existing_records($this->category_table, 'c_uq_id');
		$prepared_data_for_brands = array();
		foreach ($prepared_data['brands'] as $key => $value) {
			if(isset($this->categories_data[$value['c_id_fk']])) {
				$value['c_id_fk'] = $this->categories_data[$value['c_id_fk']];
				$prepared_data_for_brands[] = $value;
			}
		}
		unset($prepared_data['brands']);
		\Easy_Query\insert_transaction($this->brand_table, $prepared_data_for_brands);

		$this->brands_data = $this->get_existing_records($this->brand_table, 'b_uq_id');
		$prepared_data_for_models = array();
		foreach ($prepared_data['models'] as $key => $value) {
			if(isset($this->categories_data[$value['device_type']])) {
				if(isset($this->brands_data[$value['brand_id_fk']])) {
					$value['device_type'] = $this->categories_data[$value['device_type']];
					$value['brand_id_fk'] = $this->brands_data[$value['brand_id_fk']];
					if($this->download_repairs === FALSE) {
						$value['repair_ids'] = rp_get_all_default_repair_ids_of_category($value['device_type'], TRUE);
					}
					$prepared_data_for_models[] = $value;
				}
			}
		}
		unset($prepared_data['models']);
		\Easy_Query\insert_transaction($this->model_table, $prepared_data_for_models);
		return TRUE;
	}
	public function add_to_done_models($done_models = array()) {
		$file = WP_REPAIR_PLUGIN_PATH.'cached/done_models.json';
		$already_done_models = $this->get_done_models();
		foreach ($done_models as $key => $value) {
			$already_done_models[] = $value;
		}
		file_put_contents($file, json_encode($already_done_models));
	}
	public function get_done_models() {
		$file = WP_REPAIR_PLUGIN_PATH.'cached/done_models.json';
		$already_done_models = array();
		if(file_exists($file)) {
			$already_done_models = json_decode(file_get_contents($file), TRUE);
		}
		return $already_done_models;
	}
	public function delete_done_models() {
		$file = WP_REPAIR_PLUGIN_PATH.'cached/done_models.json';
		if(file_exists($file)) {
			@unlink($file);
		}
	}
	protected $defaultColors = array();
	public function get_default_color_by_id($c = NULL) {
		if(isset($this->defaultColors[$c])) {
			return $this->defaultColors[$c];
		}
		return array();
	}
	public function load_default_color_by_id($default_color_table = '') {
		$rows = $this->wpdb->get_results("SELECT * FROM $default_color_table;");
		$this->defaultColors = array();
		if(!empty($rows)) {
			foreach ($rows as $key => $color) {
				$this->defaultColors[$color->dc_id] = $color;
			}
		}
		return $this->defaultColors;
	}
	protected $defaultRepairs = array();
	public function get_default_repair_by_id($r = NULL) {
		if(isset($this->defaultRepairs[$r])) {
			return $this->defaultRepairs[$r];
		}
		return array();
	}
	public function load_default_repair_by_id($default_repair_table = '') {
		$rows = $this->wpdb->get_results("SELECT * FROM $default_repair_table;");
		$this->defaultRepairs = array();
		if(!empty($rows)) {
			foreach ($rows as $key => $repair) {
				$this->defaultRepairs[$repair->dr_id] = $repair;
			}
		}
		return $this->defaultRepairs;
	}
	protected $defaultRepairsAttr = array();
	public function get_default_repair_attr_by_id($a = NULL) {
		if(isset($this->defaultRepairsAttr[$a])) {
			return $this->defaultRepairsAttr[$a];
		}
		return array();
	}
	public function load_default_repair_attr_by_id($default_repair_attr_table = '') {
		$rows = $this->wpdb->get_results("SELECT * FROM $default_repair_attr_table;");
		$this->defaultRepairsAttr = array();
		if(!empty($rows)) {
			foreach ($rows as $key => $repairAttr) {
				if(!isset($this->defaultRepairsAttr[$repairAttr->dr_id_fk])) {
					$this->defaultRepairsAttr[$repairAttr->dr_id_fk] = array();
				}
				$this->defaultRepairsAttr[$repairAttr->dr_id_fk][] = $repairAttr;
			}
		}
		return $this->defaultRepairsAttr;
	}
	protected $currentRepairs = array();
	public function get_current_repair_attr_by_id($r_m_id = NULL) {
		if(isset($this->currentRepairs[$r_m_id])) {
			return $this->currentRepairs[$r_m_id];
		}
		return array();
	}
	public function load_current_repair_attr_by_id($rs_repair = '') {
		$rows = $this->wpdb->get_results("SELECT * FROM $rs_repair;");
		$this->currentRepairs = array();
		if(!empty($rows)) {
			foreach ($rows as $key => $c_repair) {
				$this->currentRepairs[$c_repair->r_uq_id.$c_repair->m_id_fk] = $c_repair;
			}
		}
		return $this->currentRepairs;
	}

	public function rp_get_all_model_color_ids($m_id = NULL) {

	      if(empty($m_id)) {
	            return array();
	      }

	      $rpQuery = $this->wpdb;

	      $rs_model_color = $rpQuery->prefix . 'rs_model_color';

	      $colors = $rpQuery->get_results("SELECT mc_uq_id FROM `$rs_model_color` WHERE m_id_fk = '".rp_escape_sql($m_id)."'");

	      $ids = array();

	      if(!empty($colors)) {

	            foreach ($colors as $color) {
	                  
	                  $ids[] = trim($color->mc_uq_id);

	            }

	      }

	      return $ids;

	}
	
	public function update_repairs_and_colors_for_models() {
		rp_cron_job_verifiy(array(
	        'rp_token_error' => 'No token detected?'
	    ));
		$rpQuery = $this->wpdb;
		$model_table = $rpQuery->prefix. "rs_model";
		$default_repair_table = $rpQuery->prefix. "rs_default_repair";
		$dr_attr_table = $rpQuery->prefix. "rs_default_repair_attr";
		$r_attr_table  = $rpQuery->prefix. "rs_repair_attr";
		$default_color_table = $rpQuery->prefix. "rs_default_color";
		$modelcolor_table = $rpQuery->prefix. "rs_model_color";
		$rs_repair = $rpQuery->prefix . "rs_repair";
		$rs_localization = $rpQuery->prefix . "rs_localization";
		$brand_table = $rpQuery->prefix . "rs_brand";

		$this->load_default_color_by_id($default_color_table);
		$this->load_default_repair_by_id($default_repair_table);
		$this->load_default_repair_attr_by_id($dr_attr_table);
		$this->load_current_repair_attr_by_id($rs_repair);

		$models = $rpQuery->get_results("SELECT * FROM $model_table");
		$undone_models = $rpQuery->get_results("SELECT * FROM $model_table WHERE `repair_ids` != '' OR `color_ids` != ''");
		if(!empty($undone_models)) {
			$rpQuery->query('START TRANSACTION');
			$quries = 0;
			$already_done_models = count($models) - count($undone_models);
			$done_models = array();
			foreach ($undone_models as $key => $model) {
				$m_id = $model->m_id;
				$repair_ids = $model->repair_ids;
				if($repair_ids != null){
					$repair_idz = explode(',', $repair_ids);
					$repair_ids = '';
					foreach($repair_idz as $r){
						$m_repair    = $this->get_default_repair_by_id($r);
						$r_attr      = $this->get_default_repair_attr_by_id($r);
						$r_name        = $m_repair->dr_name;
						$r_price       = $m_repair->dr_price;
						$r_duration    = $m_repair->dr_duration;
						$r_duration_measure    = $m_repair->dr_duration_measure;
						$r_discription = $m_repair->dr_discription;
						$r_img         = $m_repair->dr_img;
						$is_active     = $m_repair->is_active;
						$r_uq_id       = $m_repair->dr_uq_id;

						$dr_check = $this->get_current_repair_attr_by_id($r_uq_id.$m_id);
						if(empty($dr_check)) {
							$dr_check = NULL;
						}

				        if($dr_check == null){
				            $dr_newdata = array(
				                'r_name'=>$r_name,
				                'r_price'=>$r_price,
				                'r_duration'=>$r_duration,
				                'r_duration_measure'=>$r_duration_measure,
				                'r_discription'=>$r_discription,
				                'r_img'=>$r_img,
				                'is_active'=>$is_active,
				                'r_uq_id'=>$r_uq_id,
				                'm_id_fk'=>$m_id,
				            );
				            $quries++;
				            $rpQuery->insert($rs_repair, $dr_newdata);
				            $r_id_fk = $rpQuery->insert_id;
							foreach($r_attr as $a){

								$ra_name   = $a->da_name;
								$ra_price  = $a->da_price;
								$is_active = $a->is_active;
								$a_uq_id   = $a->da_uq_id;

								$da_check = $rpQuery->get_results("SELECT * FROM $r_attr_table WHERE a_uq_id = '".rp_escape_sql($a_uq_id)."' AND r_id_fk = '".rp_escape_sql($r_id_fk)."'");
				                if($da_check == null){
				                	$quries++;
									$rpQuery->query("INSERT INTO $r_attr_table(a_name,a_price,r_id_fk,a_uq_id) VALUES('".rp_escape_sql($ra_name)."','".rp_escape_sql($ra_price)."','".rp_escape_sql($r_id_fk)."','".rp_escape_sql($a_uq_id)."')");
								}
							}
						}
					}
					$quries++;
					$rpQuery->query("UPDATE $model_table SET repair_ids='' WHERE m_id='".rp_escape_sql($m_id)."'");
				}
				$color_ids = $model->color_ids;
				if($color_ids != null){
					$all_model_colors = $this->rp_get_all_model_color_ids($m_id);
					$color_idz = explode(',', $color_ids);
					$color_ids = '';
					foreach($color_idz as $c){
						$m_color    = $this->get_default_color_by_id($c);

						if(!empty($m_color)) {
							$mc_id      = $m_color->dc_id;
							$color_name = $m_color->dc_name;
						    $color_code = $m_color->dc_code;
						    $mc_uq_id   = $m_color->dc_uq_id;

						    if(array_search(trim($mc_uq_id), $all_model_colors) === FALSE) {
						    	$all_model_colors[] = trim($mc_uq_id);
								$quries++;
								$rpQuery->query("INSERT INTO $modelcolor_table(color_name,color_code,mc_uq_id,m_id_fk) VALUES('".rp_escape_sql($color_name)."','".rp_escape_sql($color_code)."','".rp_escape_sql($mc_uq_id)."','".rp_escape_sql($m_id)."')");

						    }
						 	
						}

					}
					$quries++;
					$rpQuery->query("UPDATE $model_table SET color_ids='' WHERE m_id='".rp_escape_sql($m_id)."'");
				}
				$done_models[] = $model->m_id;
				$cond1 = (count($done_models) >= 80);
				$cond2 = (count($done_models) === count($undone_models));
				if($cond1 || $cond2) {
					$rpQuery->query('COMMIT');
					$m_d_count = count($done_models) + $already_done_models;
					$total_models = count($models);
					if($cond2 === TRUE) {
						$next_request = FALSE;
					} else {
						$next_request = TRUE;
					}
					$response = array(
						'next_request' => $next_request,
						'models_done_count' => $m_d_count,
						'total_models' => $total_models
					);
					rp_cron_job_response($response);
					return;
				}
			}
			$rpQuery->query('COMMIT');
			$m_d_count = count($done_models) + $already_done_models;
			$total_models = count($models);
			$response = array(
				'next_request' => FALSE,
				'models_done_count' => $m_d_count,
				'total_models' => $total_models
			);
			rp_cron_job_response($response);
			return;
		}
	}

	public function update_repairs_and_colors_for_single_model( $single_model_id = '' ) {

		$rpQuery = $this->wpdb;
		$model_table = $rpQuery->prefix. "rs_model";
		$default_repair_table = $rpQuery->prefix. "rs_default_repair";
		$dr_attr_table = $rpQuery->prefix. "rs_default_repair_attr";
		$r_attr_table  = $rpQuery->prefix. "rs_repair_attr";
		$default_color_table = $rpQuery->prefix. "rs_default_color";
		$modelcolor_table = $rpQuery->prefix. "rs_model_color";
		$rs_repair = $rpQuery->prefix . "rs_repair";
		$rs_localization = $rpQuery->prefix . "rs_localization";
		$brand_table = $rpQuery->prefix . "rs_brand";

		$this->load_default_color_by_id($default_color_table);
		$this->load_default_repair_by_id($default_repair_table);
		$this->load_default_repair_attr_by_id($dr_attr_table);
		$this->load_current_repair_attr_by_id($rs_repair);


		$undone_models = $rpQuery->get_results("SELECT * FROM $model_table WHERE m_id = '$single_model_id'");

		if(!empty($undone_models)) {

			$rpQuery->query('START TRANSACTION');

			foreach ($undone_models as $key => $model) {

				$m_id = $model->m_id;

				$repair_ids = $model->repair_ids;

				if( !empty( $repair_ids ) ) {

					$repair_idz = explode(',', $repair_ids);

					$repair_ids = '';

					foreach($repair_idz as $r){

						$m_repair    = $this->get_default_repair_by_id($r);

						$r_attr      = $this->get_default_repair_attr_by_id($r);

						$r_name        = $m_repair->dr_name;
						$r_price       = $m_repair->dr_price;
						$r_duration    = $m_repair->dr_duration;
						$r_duration_measure    = $m_repair->dr_duration_measure;
						$r_discription = $m_repair->dr_discription;
						$r_img         = $m_repair->dr_img;
						$is_active     = $m_repair->is_active;
						$r_uq_id       = $m_repair->dr_uq_id;

	                    $dr_newdata = array(
	                        'r_name'=>$r_name,
	                        'r_price'=>$r_price,
	                        'r_duration'=>$r_duration,
	                        'r_duration_measure'=>$r_duration_measure,
	                        'r_discription'=>$r_discription,
	                        'r_img'=>$r_img,
	                        'is_active'=>$is_active,
	                        'r_uq_id'=>$r_uq_id,
	                        'm_id_fk'=>$m_id,
	                    );

	                    $rpQuery->insert($rs_repair, $dr_newdata);

	                    $r_id_fk = $rpQuery->insert_id;

	        			if( !empty( $r_attr ) ) {

							foreach($r_attr as $a){

								$ra_name   = $a->da_name;
								$ra_price  = $a->da_price;
								$is_active = $a->is_active;
								$a_uq_id   = $a->da_uq_id;

								$rpQuery->query("INSERT INTO $r_attr_table(a_name,a_price,r_id_fk,a_uq_id) VALUES('".rp_escape_sql($ra_name)."','".rp_escape_sql($ra_price)."','".rp_escape_sql($r_id_fk)."','".rp_escape_sql($a_uq_id)."')");
							}

	        			}
					}

					$rpQuery->query("UPDATE $model_table SET repair_ids='' WHERE m_id='".rp_escape_sql($m_id)."'");

				}

			}

			$rpQuery->query('COMMIT');

		}

	}
}
