<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class Download_Repairs {

	protected $fetch_url = 'https://base.repairplugin.com/Endpoints/fetch_repairs';

	public $reallyAddedRepairs = 0;

	public $reallyAddedAttributes = 0;

	public function __construct($rpQuery = NULL) {

		$this->wpdb = $rpQuery;

		$this->fetch_url = \WP_REPAIR_BASE_SERVER_URL.'Endpoints/fetch_repairs';

		$this->fix_post_array();

	}

	public function fix_post_array() {
		if(isset($_POST) && !empty($_POST) && isset($_POST['the_dl_form'])) {
			$decoded = json_decode((stripslashes($_POST['the_dl_form'])), TRUE); 
			$_POST = array();
			$_POST['c_uq_ids'] = $decoded['c_id'];
			$_POST['r_uq_ids'] = $decoded['r_id'];
			$_POST['a_uq_ids'] = $decoded['a_id'];
		}
	}
	
	public function fetch_repair_base($gzip = TRUE, $names_only = FALSE) {
		$license_key = rp_get_option('Repairplugin_lic_Key', '');
		$license_email = rp_get_option('Repairplugin_lic_email', '');
		
		if(empty($license_key)) {
			return array();
		}

		$curl = curl_init( $this->fetch_url );

		if($gzip == TRUE) {
			curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
		}

		$payload = array();

		if($names_only === TRUE) {

			$payload = array(
				'license_key' => $license_key, 
				'names_only' => 'yes',
				'site_default_language' => ( \rp_get_default_language() )
			);

			curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

		} else {

			if(isset($_POST['r_uq_ids']) && !empty($_POST['r_uq_ids']) && is_array($_POST['r_uq_ids'])) {

				$r_uq_ids = json_encode($_POST['r_uq_ids']);

				if( isset($_POST['a_uq_ids']) && !empty($_POST['a_uq_ids']) && is_array($_POST['a_uq_ids']) ) {

					$a_uq_ids = json_encode($_POST['a_uq_ids']);
					
				} else {

					$a_uq_ids = json_encode(array());

				}

				$rp_base_class = new \RepairpluginBase(WP_REPAIR_PLUGIN_FILE);
				$license_app_version = $rp_base_class->getCurrentVersion();
				$domain = $rp_base_class->getDomain();

                $lang_uq_ids = json_encode(\rp_get_all_uq_id_from_rs_languages_table());

				$payload = array('license_key' => $license_key, 'license_email' => $license_email, 'license_app_version' => $license_app_version, 'license_domain' => $domain, 
				'r_uq_ids' => $r_uq_ids, 
				'a_uq_ids' => $a_uq_ids,
				'lang_uq_ids' => $lang_uq_ids);

				curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

			} else {

				$payload = array('license_key' => $license_key);

				curl_setopt( $curl, CURLOPT_POSTFIELDS, $payload );

			}

		}

		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );

		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false );

		curl_setopt( $curl, CURLOPT_SSL_VERIFYHOST, false );

		$response = curl_exec( $curl );

		$info = curl_getinfo( $curl );

		curl_close( $curl );

		$data = array();

		if($info['http_code'] != '200') {
			return $data;
		}

		if(!empty($response)) {
			try {
				$data = json_decode($response);
			} catch (\Exception $e) {

			}
		}
		if(!empty($data) && property_exists($data, 'status') && ($data->status === TRUE || $data->status === 1)) {
			return $data;
		}
		return array();
	}

	public function prepare_need_to_add_data( $data = array() ) {

		$prepared_data = array();

		$prepared_data['repairs'] = array();

		$prepared_data['repair_translations'] = array();

		$prepared_data['attr_translations'] = array();

		// prepare repairs & attributes
		if( !empty($data) && isset($data->categories) ) {

			foreach($data->categories as $cat) {

				$c_uq_id = $cat->c_uq_id;

				$c_id = \rp_get_category_id_by_c_uq_id($c_uq_id);

				foreach($cat->repairs as $repair_row) {

					$repair_row->dr_category = $c_id;

					$prepared_data['repairs'][] = $repair_row;
					
				}

			}

			if( isset($data->repair_translations) && !empty($data->repair_translations) ) {

				$prepared_data['repair_translations'] = $data->repair_translations;

			}

			if( isset($data->attr_translations) && !empty($data->attr_translations) ) {

				$prepared_data['attr_translations'] = $data->attr_translations;

			}

		}

		return $prepared_data;

	}

	public function add_new_default_repairs_and_attrs( $data = array() ) {

		if(isset($data['repairs']) && !empty( $data['repairs'] ) ) {

			$rs_default_repair = $this->wpdb->prefix . 'rs_default_repair';

			$rs_default_repair_attr = $this->wpdb->prefix . 'rs_default_repair_attr';

			foreach( $data['repairs'] as $repair_row ) {

				// checkc if dr_uq_id and dr_category already exists
				$dr_uq_id = $repair_row->dr_uq_id;

				$dr_category = $repair_row->dr_category;

				$check = $this->wpdb->get_row( "SELECT * FROM $rs_default_repair WHERE dr_uq_id = '$dr_uq_id' AND dr_category = '$dr_category' LIMIT 1" );

				if( !empty($check) ) {

					$newlyDownloaded = FALSE;

					$dr_id = $check->dr_id;

				} else {

					$newlyDownloaded = TRUE;

					$this->reallyAddedRepairs++;

					$this->wpdb->insert(

						$rs_default_repair,
	
						array(
	
							'dr_name' => $repair_row->dr_name,
	
							'dr_price' => $repair_row->dr_price,
	
							'dr_duration' => $repair_row->dr_duration,
	
							'dr_discription' => $repair_row->dr_discription,

							'dr_badge' => '',
	
							'dr_img' => $repair_row->dr_img,
	
							'is_active' => $repair_row->is_active,
	
							'dr_uq_id' => $repair_row->dr_uq_id,
	
							'dr_position' => $repair_row->dr_position,
	
							'is_free' => $repair_row->is_free,
	
							'dr_duration_measure' => $repair_row->dr_duration_measure,
	
							'dr_category' => $repair_row->dr_category
	
						)
	
					);
	
					// get insert id
					$dr_id = $this->wpdb->insert_id;

				}

				if( !empty( $repair_row->attrs ) ) {

					$repair_row->attrs = (array) $repair_row->attrs;

					foreach( $repair_row->attrs as $attr_row ) {

						$attr_row->dr_id_fk = $dr_id;

						$this->reallyAddedAttributes++;

						$this->wpdb->insert(

							$rs_default_repair_attr,
		
							array(
	
								'da_name' => $attr_row->da_name,

								'da_desc' => $attr_row->da_desc,

								'da_badge' => '',

								'da_duration' => $repair_row->dr_duration,

								'da_duration_measure' => $repair_row->dr_duration_measure,
	
								'da_price' => $attr_row->da_price,
	
								'is_active' => 1, // default is active
	
								'da_uq_id' => $attr_row->da_uq_id,
	
								'da_position' => $attr_row->da_position,
	
								'dr_id_fk' => $attr_row->dr_id_fk,
								
								'dr_category' => $repair_row->dr_category
		
							)
		
						);

						// get insert id
						$da_insert_id = $this->wpdb->insert_id;

						if( $newlyDownloaded === FALSE ) {

							// repair was already there, so we need to add this new attribute to all models
						
							\rp_add_this_new_default_repair_attr_to_all_models( $da_insert_id );

						}

					}

				}

				\rp_add_this_default_repair_to_all_models( $dr_id );

			}

		}

	}

	public function add_new_repair_translations( $repair_translations = array() ) {

		if( !empty( $repair_translations ) ) {

			$rs_dr_translations = $this->wpdb->prefix . 'rs_dr_translations';

			foreach($repair_translations as $rt_row) {

				// check if already exists
				$check = $this->wpdb->get_row( "SELECT * FROM $rs_dr_translations WHERE `key` = '".rp_escape_sql($rt_row->key)."' AND `uq_id` = '".rp_escape_sql($rt_row->uq_id)."' AND `language` = '".rp_escape_sql($rt_row->language)."' " );


				if( empty( $check ) ) {

					$this->wpdb->insert(

						$rs_dr_translations,
	
						array(
	
							'language' => $rt_row->language,
	
							'key' => $rt_row->key,
	
							'value' => $rt_row->value,
	
							'uq_id' => $rt_row->uq_id
	
						)
	
					);
	
					// create new if not already exists
					// \rp_create_matching_repair_translations_if_not_already($rt_row->key, $rt_row->uq_id, $rt_row->language, $rt_row->value);

				}

			}

		}

	}

	public function add_new_attr_translations( $attr_translations = array() ) {

		if( !empty( $attr_translations ) ) {

			$rs_dr_translations = $this->wpdb->prefix . 'rs_dr_translations';

			foreach($attr_translations as $rt_row) {

				// check if already exists
				$check = $this->wpdb->get_row( "SELECT * FROM $rs_dr_translations WHERE `key` = '".rp_escape_sql($rt_row->key)."' AND `uq_id` = '".rp_escape_sql($rt_row->uq_id)."' AND `language` = '".rp_escape_sql($rt_row->language)."' " );

				if( empty( $check ) ) {

					$this->wpdb->insert(

						$rs_dr_translations,
	
						array(
	
							'language' => $rt_row->language,
	
							'key' => $rt_row->key,
	
							'value' => $rt_row->value,
	
							'uq_id' => $rt_row->uq_id
	
						)
	
					);
	
					// create new if not already exists
					// \rp_create_matching_attr_translations_if_not_already($rt_row->key, $rt_row->uq_id, $rt_row->language, $rt_row->value);

				}

			}

		}

	}

	public function add_need_to_add_data( $data = array() ) {

		$this->wpdb->query("START TRANSACTION");

		$this->add_new_default_repairs_and_attrs($data);

		if( isset( $data['repair_translations'] ) ) {

			$this->add_new_repair_translations( $data['repair_translations'] );

		}

		if( isset( $data['attr_translations'] ) ) {

			$this->add_new_attr_translations( $data['attr_translations'] );

		}

		$this->wpdb->query("COMMIT");

	}

	public function get_added_repairs_count() {

		return $this->reallyAddedRepairs ?? 0;

	}

	public function get_added_attribute_count() {

		return $this->reallyAddedAttributes ?? 0;

	}

	public function save_handler($data = array()) {
		
		if(isset($_POST['r_uq_ids'])) {
			if(!empty($data)) {
				
				$prepared_data = $this->prepare_need_to_add_data($data);

				if(!empty($prepared_data) && (!empty($prepared_data['repairs']))) {

					\rp_add_or_update_option('rp_get_last_repairs_downloaded_date', date('Y-m-d'));

					$this->add_need_to_add_data($prepared_data);

					$attrs_count = $this->get_added_attribute_count();

					$repairs_count = $this->get_added_repairs_count();

					echo json_encode(array('success' => 'new_downloads', 'counts' => array(

						'repairs' => $repairs_count,

						'attrs' => $attrs_count

					)));

					exit();
				}
				echo json_encode(array('success' => 'up_to_date_already'));
				exit();
			} else {
				echo json_encode(array('error' => 'Failed to fetch the repairs from RepairPlugin server.'));
				exit();
			}
		}

    }
}
