<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class Permission {
	protected $permissionByRoles = array();
	protected $permission_name = 'manage_wp_repair_plugin';
	protected $disallowed = array('customer', 'subscriber');
	public function __construct() {
		$this->prepareRolesPermissions();
		$this->setDefaultIfNotAlready();
		$this->prepareRolesPermissions();
	}
	public function getAllRolesPermissions() {
		$allowedRoles = array();
		foreach ($this->permissionByRoles as $role_name => $role_info) {
			if(!in_array(strtolower($role_name), $this->disallowed)) {
				$allowedRoles[$role_name] = $role_info;
			}
		}
		ksort($allowedRoles);
		arsort($allowedRoles);
		return $allowedRoles;
	}
	public function setDefaultIfNotAlready() {
		if(!$this->has_permission('administrator')) {
			$this->add_permission('shop_manager');
			$this->add_permission('editor');
			$this->add_permission('administrator');
		}
	}
	public function prepareRolesPermissions() {
		global $wp_roles;
		$this->permissionByRoles = array();
		foreach ($wp_roles->roles as $role_name => $role_info) {
			$has_permission = FALSE;
			foreach ($role_info['capabilities'] as $permission_name => $bool) {
				if($permission_name === $this->permission_name && !empty($bool)) {
					$has_permission = TRUE;
				}
			}
			$this->permissionByRoles[$role_name] = $has_permission;
		}
	}
	public function has_permission($role = '') {
		return isset($this->permissionByRoles[$role]) && !empty($this->permissionByRoles[$role]);
	}
	public function add_permission($role = '') {
	    if($this->has_permission($role) === FALSE && !in_array($role, $this->disallowed)) {
    	    $role = get_role($role);
    	    if(isset($role) && !empty($role)) {
    	    	$role->add_cap($this->permission_name, true);
    	    }
	    }
	}
	public function remove_permission($role = '') {
	    if($this->has_permission($role) !== FALSE) {
    	    $role = get_role($role);
    		if(isset($role) && !empty($role)) {
    			$role->remove_cap($this->permission_name);
    		}
	    }
	}
}
