<?php

namespace RepairPluginPro;

use RepairPluginPro\RP_Request_Handler;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if( !class_exists( 'RepairPluginPro\RP_Apply_Lock' ) ) {

	class RP_Apply_Lock {

		public static $public_actions = array();

		public static $admin_pages = array();

		public static $forms = array();

		public static function need_to_do_lock_protection() {

			if( RP_Request_Handler::is_repair_plugin_admin_page( self::$admin_pages ) || RP_Request_Handler::is_ajax_request_for_repair_plugin( self::$forms ) ) {

				return TRUE;

			}

			return FALSE;

		}

		public static function is_ajax_request() {

			return trim( strtolower( $_SERVER['HTTP_X_REQUESTED_WITH'] ?? '' ) ) === 'xmlhttprequest';

		}

		public static function is_administrator() {

			$user_id = get_current_user_id();

			if( empty( $user_id ) ) {

				return FALSE;

			}

			$user = get_userdata( $user_id );

			$roles = ($user->roles ?? array());

			return in_array( 'administrator', $roles );

		}

		public static function user_has_repair_plugin_access() {

			if( self::is_administrator() ) {

				// Current user is administrator
				// So definitely has access!
				return TRUE;

			}

			if( is_user_logged_in() && current_user_can( 'manage_wp_repair_plugin' ) ) {

				// User is logged in + also has permission
				// to manage RepairPlugin
				return TRUE;

			}

			return FALSE;

		}

		public static function do_lock_protection() {

			if( self::need_to_do_lock_protection() && !self::user_has_repair_plugin_access() ) {

				// If need to do protection but user doesn't
				// have access to repair plugin, no need to
				// block them from entering!

				if( self::is_ajax_request() ) {

					// Send Response to Ajax!

					echo json_encode(array(

						'status' => false,

						'message' => 'Looks like you are unauthorised to take this action!'

					));

				} else {

					// Redirect the user
					// Looks like the page is accessed directly!

					wp_redirect( get_site_url() );

				}

				exit;

			}

		}

		public static function exclude_public_actions() {

			foreach (self::$forms as $index => $action) {

				if( !in_array( $action, self::$public_actions ) ) {

					continue;

				}

				unset( self::$forms[$index] );

			}

			self::$forms = array_values( self::$forms );

		}

		public static function init() {

			self::$admin_pages = RP_Request_Handler::$admin_pages;

			self::$forms = RP_Request_Handler::$forms;

			self::exclude_public_actions();

			self::do_lock_protection();

		}

	}

	// Run this as early as possible!
	add_action( 'init', array( 'RepairPluginPro\RP_Apply_Lock', 'init' ), -999 );

}
