<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if( !function_exists('rp_check_if_file_exists') ) {

	function rp_check_if_file_exists( $file ) {

		// avoid open_basedir restriction in effect.
		// The @ symbol will tell PHP to suppress any errors/warnings generated by these functions.
		if ( ! @file_exists( $file ) ) {
			return false;
		}

		return true;

	}

}

if( !class_exists('RepairPluginPro\RP_Background_Process') ) {

	class RP_Background_Process {

		protected $php_path;

		protected $plugin_path;

		public function __construct($plugin_path = '') {

			$this->plugin_path = $plugin_path;

			try {

				$this->php_path = $this->runBackgroundProcessTest();

			} catch (\Exception $e) {

				$this->php_path = FALSE;

			}

			if( $this->php_path === FALSE ) {

				$this->php_path = 'php';

			}

		}

		public function get_php_path() {
			return $this->php_path;
		}

		public function getPHPExecutableFromPath() {

			$ext_path = @ini_get("extension_dir");

			if(!empty($ext_path)) {

				$php_dir = str_replace('ext', '', $ext_path);

				$php_executable = str_replace('\\', '/', $php_dir) . 'php.exe'; 

				if(rp_check_if_file_exists($php_executable) && is_file($php_executable) && is_executable($php_executable)) {

					return $php_executable;

				}

				return FALSE;

			}

			return FALSE;

		}

		public function getPhpInterpreter() {

			$basename = basename(PHP_BINARY);

	// If the binary is 'php', 'php7', 'php7.3' etc, then assume it's a usable interpreter
			if ($basename === 'php' || preg_match('/^php\d+(?:\.\d+)*$/', $basename) || $basename === 'php.exe') {
				return PHP_BINARY;
			}

	// Otherwise, we might be running as mod_php, php-fpm, etc, where PHP_BINARY is not a
	// usable PHP interpreter.  Try to find one with the same version as the current one.

			$candidates = [
				'php' . PHP_MAJOR_VERSION . '.' . PHP_MINOR_VERSION . '.' . PHP_RELEASE_VERSION,
				'php' . PHP_MAJOR_VERSION . '.' . PHP_MINOR_VERSION,
				'php' . PHP_MAJOR_VERSION . PHP_MINOR_VERSION . PHP_RELEASE_VERSION,
				'php' . PHP_MAJOR_VERSION . PHP_MINOR_VERSION,
				'php' . PHP_MAJOR_VERSION
			];

			$envPath = (@getenv('PATH')) ?? '';
			$paths = $envPath !== '' ? explode(PATH_SEPARATOR, $envPath) : [];

			if (!in_array(PHP_BINDIR, $paths, true)) {
				$paths[] = PHP_BINDIR;
			}

			foreach ($candidates as $candidate) {
				foreach ($paths as $path) {

					$php_executable = $path . DIRECTORY_SEPARATOR . $candidate;

					if(rp_check_if_file_exists($php_executable)  && is_executable($php_executable)) {

						return $php_executable;

					}

					if(strpos($path, $candidate) !== FALSE) {

						$php_executable = $path . DIRECTORY_SEPARATOR . 'php.exe';

						if(rp_check_if_file_exists($php_executable)  && is_executable($php_executable)) {

							return $php_executable;

						}

					}

				}
			}

			$paths = array();
			$paths[] = '/usr/bin';
			$paths[] = '/usr/local/bin';

			foreach ($candidates as $candidate) {
				foreach ($paths as $path) {

					$php_executable = $path . DIRECTORY_SEPARATOR . $candidate;

					if(rp_check_if_file_exists($php_executable) && is_executable($php_executable)) {

						return $php_executable;

					}

					$php_executable = $path . DIRECTORY_SEPARATOR . 'ea-' . $candidate;

					if(rp_check_if_file_exists($php_executable) && is_executable($php_executable)) {

						return $php_executable;

					}

				}
			}    

			return FALSE;
		}

		public function canRunBackgroundProcess() {

			return !empty( $this->php_path );

		}

		public function tryDefaultPHP() {

			if(function_exists('exec') && @exec('echo EXEC') == 'EXEC') {

				$test_file = $this->plugin_path . 'quick_test.php';

				if(@exec('php '.$test_file) === '4') {

					return 'php';

				}

			}

			return FALSE;

		}

		public function runBackgroundProcessTest() {

			$php_path = $this->getPHPExecutableFromPath();

			if($php_path === FALSE) {

				$php_path = $this->getPhpInterpreter();

			}

			if($php_path === FALSE) {

				$php_path = $this->tryDefaultPHP();

			}

			if($php_path !== FALSE) {

				if(function_exists('exec') && @exec('echo EXEC') == 'EXEC') {

	// Exec is enabled!
					$test_file = $this->plugin_path . 'quick_test.php';

					if($php_path === 'php') {

						$exec_php_path = '';

					} else {

						$exec_php_path = $php_path;

					}

					if(@exec(trim($exec_php_path.' '.$test_file)) === '4') {

						return $exec_php_path;

					} else {

	// Unable to run PHP
						return FALSE;
					}

				} else {

	// Ecex is disabled!
					return FALSE;

				}

			} else {

				return FALSE;

			}

		}

		public function runTheJob($job_file = null, $inBackground = FALSE) {

			if( !$this->canRunBackgroundProcess() ) {

				return FALSE;

			}

			if(empty($job_file) || !rp_check_if_file_exists($job_file) || !is_file($job_file)) {

				return FALSE;

			}

			$background_str = $inBackground === TRUE ? ' &' : '';

			if (strtolower(substr(php_uname(), 0, 7)) == "windows"){

			    if($inBackground === TRUE) {

			    	pclose( popen('start /B '.$this->get_php_path().' '.$job_file, 'r') );

			    } else {

			    	exec($this->get_php_path().' '.$job_file);

			    }

			} else {

			    exec($this->get_php_path().' '. $job_file . ' > /dev/null 2>&1'.$background_str);  
			
			}

			return TRUE;

		}

	}

}