<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if( !class_exists( 'RepairPluginPro\RP_Cart' ) ) {

	class RP_Cart {

		// Right Now, When single_select is
		// set to TRUE, we assume 1 repair
		// can be added only once.
		private $single_select = TRUE;

		private $m_id = '';

		private $repairs = array();

		private $repair_attrs = array();

		private $upsales = array();

		private $coupon_info = NULL;

		private $rs_repair = '';

		private $rs_repair_attr = '';

		private $rs_upsales = '';

		private $rs_upsales_models = '';

		private $rs_coupons = '';

		private $rs_planning_and_discount = '';

		private $rpQuery = null;

		private $delivery_method = array();

		public function __construct( $m_id = '' ) {

			global $rpQuery;

			$this->m_id = $m_id;

			$this->rpQuery = $rpQuery;

			$this->rs_repair = $this->rpQuery->prefix . 'rs_repair';

			$this->rs_repair_attr = $this->rpQuery->prefix . 'rs_repair_attr';

			$this->rs_upsales = $this->rpQuery->prefix . 'rs_upsales';

			$this->rs_upsales_models = $this->rpQuery->prefix . 'rs_upsales_models';

			$this->rs_coupons = $this->rpQuery->prefix . 'rs_coupons';

			$this->rs_planning_and_discount = $this->rpQuery->prefix . 'rs_planning_and_discount';

		}

		public function get_coupon_info() {
			
			return $this->coupon_info;

		}

		public function get_added_repair_attr_by_repair_id( $repair_id = '' ) {

			foreach ($this->repair_attrs as $row) {

				if( ((int) $row->r_id_fk) === ((int) $repair_id) ) {

					return $row;

				}

			}

			return array();

		}

		public function get_added_upsale( $upsales_id = '' ) {

			foreach ($this->upsales as $row) {

				if( ((int) $row->upsales_id) === ((int) $upsales_id) ) {

					return $row;

				}

			}

			return array();

		}

		public function get_added_repair( $repair_id = '' ) {

			foreach ($this->repairs as $row) {

				if( ((int) $row->r_id) === ((int) $repair_id) ) {

					return $row;

				}

			}

			return array();

		}

		public function has_added_this_repair( $repair_id = '' ) {

			return ( !empty( $this->get_added_repair( $repair_id ) ) || !empty( $this->get_added_repair_attr_by_repair_id( $repair_id ) ) );

		}

		public function possible_adding_repair( $repair_id = '' ) {

			if( $this->single_select === TRUE ) {

				return $this->has_added_this_repair( $repair_id ) === FALSE;

			}

			return TRUE;

		}

		public function add_delivery_method( $delivery_method_name = '' ) {

			$translation_key = '';

			// remove one or more space with single space
			$delivery_method_name = preg_replace('/\s+/', ' ', $delivery_method_name);

			// trim
			$delivery_method_name = trim( $delivery_method_name );

			// strtolower
			$delivery_method_name = strtolower( $delivery_method_name );

			// replace space or - with _
			$delivery_method_name = preg_replace('/\s+/', '_', $delivery_method_name);

			$delivery_method_name = preg_replace('/-+/', '_', $delivery_method_name);

			if( $delivery_method_name == 'come_our_store' || $delivery_method_name == 'come_by_our_store' ) {

				$delivery_method_name = 'come_our_store';

				$translation_key = 'come_by_our_store';

			}

			if( $delivery_method_name == 'ship_device' || $delivery_method_name == 'ship_your_device' ) {

				$delivery_method_name = 'ship_device';

				$translation_key = 'ship_device';

			}

			if( $delivery_method_name == 'pickup_service' ) {

				$delivery_method_name = 'pickup_service';

				$translation_key = 'let_us_pickup_your_device';

			}

			if( $delivery_method_name == 'repair_on_location' || $delivery_method_name == 'repair_on_your_location' ) {

				$delivery_method_name = 'repair_on_location';

				$translation_key = 'repair_on_your_location';

			}

			$dm_cost_key = 'rp_dm_cost_' . $delivery_method_name;

			$dm_cost = rp_get_option($dm_cost_key, 0);

			if( $dm_cost == '' || $dm_cost == 0 || $dm_cost == 0.00 || $dm_cost < 0 ) {

				$dm_cost = 0;

			}

			$this->delivery_method = array(

				'translation_key' => $translation_key,

				'name' => $delivery_method_name,

				'cost' => $dm_cost

			);

			return TRUE;

		}

		public function delivery_method_translation_key() {

			return $this->delivery_method['translation_key'] ?? '';

		}

		public function delivery_method_name() {

			return $this->delivery_method['name'] ?? '';

		}

		public function delivery_method_cost() {

			return $this->delivery_method['cost'] ?? 0;

		}

		public function add_repair( $repair_id = '' ) {

			if( !$this->possible_adding_repair( $repair_id ) ) {

				// Only one allowed and that's already added!
				return TRUE;

			}

			$row = $this->rpQuery->get_row("SELECT * FROM {$this->rs_repair} WHERE r_id = ':r_id' AND m_id_fk = ':m_id_fk' AND is_active = '1' LIMIT 1;",

				array(

					'r_id' => $repair_id,

					'm_id_fk' => $this->m_id

				)

			);

			if( !empty( $row ) ) {

				$row = apply_filters('rp_filter_before_adding_repair_to_cart', $row);

				$this->repairs[] = $row;

				return TRUE;

			}

			return FALSE;

		}

		public function get_added_repair_attr( $a_id = '' ) {

			foreach ($this->repair_attrs as $row) {

				if( ((int) $row->a_id) === ((int) $a_id) ) {

					return $row;

				}

			}

			return array();

		}

		public function possible_adding_repair_attr( $r_id_fk = '' ) {

			if( $this->single_select === TRUE ) {

				return empty( $this->get_added_repair_attr_by_repair_id( $r_id_fk ) );

			}

			return TRUE;

		}

		public function exclude_from_repairs_array( $repair_id = '' ) {

			foreach ($this->repairs as $key => $row) {

				if( ((int) $row->r_id) === ((int) $repair_id) ) {

					unset( $this->repairs[$key] );

					$this->repairs = array_values( $this->repairs );

				}

			}

		}

		public function add_repair_attr( $a_id = '' ) {

			$attr_row = $this->rpQuery->get_row("SELECT * FROM {$this->rs_repair_attr} WHERE a_id = ':a_id' AND is_active = '1' LIMIT 1;",

				array(

					'a_id' => $a_id

				)

			);

			if( !empty( $attr_row ) ) {

				if( !$this->possible_adding_repair_attr( $attr_row->r_id_fk ) ) {

					// Only one allowed and that's already added!
					return TRUE;

				}

				$this->exclude_from_repairs_array( $attr_row->r_id_fk );

				$connection_row = $this->rpQuery->get_row("SELECT * FROM {$this->rs_repair} WHERE r_id = ':r_id' AND m_id_fk = ':m_id_fk' LIMIT 1;",

					array(

						'r_id' => $attr_row->r_id_fk,

						'm_id_fk' => $this->m_id

					)

				);

				if( !empty( $connection_row ) ) {

					$attr_row = apply_filters('rp_filter_before_adding_repair_attr_to_cart', $attr_row, $connection_row);

					$this->repair_attrs[] = $attr_row;

					return TRUE;

				}

			}

			return FALSE;

		}

		public function possible_adding_upsale( $upsales_id = '' ) {

			if( $this->single_select === TRUE ) {

				return empty( $this->get_added_upsale( $upsales_id ) );

			}

			return TRUE;

		}

		public function add_upsale( $upsales_id = '' ) {

			if( !$this->possible_adding_upsale( $upsales_id ) ) {

				return TRUE;

			}

			$upsale_row = $this->rpQuery->get_row("SELECT * FROM {$this->rs_upsales} WHERE upsales_id = ':upsales_id' AND is_active = '1' LIMIT 1;",

				array(

					'upsales_id' => $upsales_id

				)

			);

			if( !empty( $upsale_row ) ) {

				$connection_row = $this->rpQuery->get_row("SELECT * FROM {$this->rs_upsales_models} WHERE upsales_id_fk = ':upsales_id' AND m_id_fk = ':m_id_fk' LIMIT 1;",

					array(

						'upsales_id' => $upsales_id,

						'm_id_fk' => $this->m_id

					)

				);

				if( !empty( $connection_row ) ) {

					$this->upsales[] = $upsale_row;

					return TRUE;

				}

			}

			return FALSE;

		}

		public function add_coupon( $coupons_id = '' ) {

			$row = $this->rpQuery->get_row("SELECT * FROM {$this->rs_coupons} WHERE coupons_id = ':coupons_id' AND is_active = '1' LIMIT 1;",

				array(

					'coupons_id' => $coupons_id

				)

			);

			if( !empty( $row ) ) {

				$this->coupon_info = $row;

				return TRUE;

			}

			return FALSE;

		}

		public function get_combo_discount_percentage() {

			$row = $this->rpQuery->get_row("SELECT amount, is_combo_discount FROM {$this->rs_planning_and_discount} WHERE is_combo_discount = '1' LIMIT 1;");

			$row = \rp_fix_combo_discount_row( $row );

			if( !empty( $row ) && $row->is_combo_discount_percentage == '1' && ((float)$row->amount) > 0 ) {

				return (float) $row->amount;

			}

			return 0;

		}

		public function get_combo_discount() {

			$row = $this->rpQuery->get_row("SELECT amount, is_combo_discount FROM {$this->rs_planning_and_discount} WHERE is_combo_discount = '1' LIMIT 1;");

			$row = \rp_fix_combo_discount_row( $row );

			if( !empty( $row ) && $row->is_combo_discount == '1' ) {

				return (float) $row->amount;

			}

			return 0;

		}

		public function calculate_combo_discount_items() {

			$items = 0;

			foreach ($this->repairs as $row) {
				
				if( ((float) $row->r_price) > 0 ) {

					$items++;

				}

			}

			foreach ($this->repair_attrs as $row) {
				
				if( ((float) $row->a_price) > 0 ) {

					$items++;

				}

			}

			return ($items - 1);

		}

		public function sort_by_price( $all_repairs = array() ) {

			// loop through all and convert to array
			// and also add x_price to the array to generalize

			$all_repairs_array = array();

			foreach ($all_repairs as $row) {

				$row = json_decode( json_encode( $row ), true );

				$row['x_price'] = array_key_exists('a_price', $row) ? $row['a_price'] : $row['r_price'];

				$all_repairs_array[] = $row;

			}

			// sort by x_price. Show max price first
			usort($all_repairs_array, function($a, $b) {

				return $b['x_price'] <=> $a['x_price'];

			});

			// convert back to object
			$all_repairs = array();

			foreach ($all_repairs_array as $row) {

				$all_repairs[] = json_decode( json_encode( $row ) );

			}

			return $all_repairs;

		}

		public function showable_repair_price( $id = 0, $type = 'repair' ) {

			if( $type == 'attr' ) {

				$key = 'a_price';

				$compare_id = 'a_id';

			} else {

				$key = 'r_price';

				$compare_id = 'r_id';

			}

			$all_repairs = array();

			foreach($this->repairs as $row) {
				
				$all_repairs[] = $row;

			}

			foreach($this->repair_attrs as $row) {
				
				$all_repairs[] = $row;

			}

			$all_repairs = $this->sort_by_price( $all_repairs );

			$combo_discount_percentage = $this->get_combo_discount_percentage();

			$combo_discount_fixed = $this->get_combo_discount();

			$modified_repairs = array();

			if( $combo_discount_percentage > 0 || $combo_discount_fixed > 0 ) {

				$done_first = FALSE;

				foreach( $all_repairs as $row ) {

					$thePrice = (float) property_exists($row, 'a_price') ? $row->a_price : $row->r_price;

					if( $thePrice > 0 && $done_first === TRUE ) {

						$row->after_discount_price = $thePrice;

						$row->is_discounted = 1;

						$modified_repairs[] = $row;

					} else {

						$done_first = TRUE;

						$row->after_discount_price = $thePrice;

						$row->is_discounted = 0;

						$modified_repairs[] = $row;

					}

				}

			} else {

				foreach( $all_repairs as $row ) {

					$thePrice = (float) property_exists($row, 'a_price') ? $row->a_price : $row->r_price;

					$row->after_discount_price = $thePrice;

					$row->is_discounted = 0;

					$modified_repairs[] = $row;

				}

			}

			// loop through all and check if property exists ($key)
			// if yes, then match the id and return the price
			// if no, then return 0

			foreach ($modified_repairs as $row) {

				if( property_exists($row, $key) ) {

					if( ((int) $row->{$compare_id}) === ((int) $id) ) {

						return array( $row->is_discounted , $row->after_discount_price );

					}

				}

			}

			return array(0, 0);
			
		}

		public function calculate() {

			$sub_total = 0;

			$only_repair_subtotal = 0;

			$subtotal_after_discount = 0;

			$combo_discount_percentage = $this->get_combo_discount_percentage();

			if( $combo_discount_percentage > 0 ) {

				$all_repairs = array();

				foreach($this->repairs as $row) {
					
					$all_repairs[] = $row;

				}

				foreach($this->repair_attrs as $row) {
					
					$all_repairs[] = $row;

				}

				$all_repairs = $this->sort_by_price( $all_repairs );

				$done_first = FALSE;

				foreach( $all_repairs as $row ) {

					$thePrice = (float) property_exists($row, 'a_price') ? $row->a_price : $row->r_price;

					$sub_total += $thePrice;

					$only_repair_subtotal += $thePrice;

					if( $thePrice > 0 && $done_first === TRUE ) {

						$rounded = round($combo_discount_percentage, 2);

						$percentage = ( $thePrice / 100 ) * $rounded;

						$after_discount = round($thePrice - $percentage, 2);

						$subtotal_after_discount += $after_discount;

					} else {

						$done_first = TRUE;

						$subtotal_after_discount += $thePrice;

					}

				}

			} else {

				foreach ($this->repairs as $row) {
				
					$sub_total += (float) $row->r_price;

					$only_repair_subtotal += (float) $row->r_price;
	
				}
	
				foreach ($this->repair_attrs as $row) {
					
					$sub_total += (float) $row->a_price;

					$only_repair_subtotal += (float) $row->a_price;
	
				}

			}

			foreach ($this->upsales as $row) {
				
				$sub_total += (float) $row->us_new_price;

			}

			$sub_total += $this->delivery_method_cost();

			$combo_discount_items = 0;

			$combo_discount_amount = 0;

			$combo_discount = 0;

			if( $combo_discount_percentage > 0 ) {

				$combo_discount = $only_repair_subtotal - $subtotal_after_discount;

			} else {

				$combo_discount_items = $this->calculate_combo_discount_items();

				$combo_discount_amount = $this->get_combo_discount();

				if( $combo_discount_items > 0 && $combo_discount_amount > 0 ) {

					$combo_discount = $combo_discount_items * $combo_discount_amount;

				} else {

					$combo_discount = 0;

				}

			}

			if( !empty( $this->coupon_info ) ) {

				if( $this->coupon_info->coupons_type == 'percent' ) {

					$sub_total_before_coupon = $sub_total - $combo_discount;

					$coupon_amount = ($sub_total_before_coupon * (float) $this->coupon_info->coupons_amount) / 100;

				} else {

					$coupon_amount = (float) $this->coupon_info->coupons_amount;

				}

			} else {

				$coupon_amount = 0;

			}

			$total = $sub_total - ($combo_discount + $coupon_amount);

			$calculations = array( 'sub_total' => $sub_total, 'combo_discount' => $combo_discount, 'coupon_amount' => $coupon_amount, 'total' => $total );

			return $calculations;

		}

		public function prepare_post_data( $r_ids = array(), $u_ids = array() ) {

			$rids = array();

			$array_names = array();

			$array_prices = array();

			$aids = array();

			$upids = array();

			foreach ($r_ids as $r_id) {

				$added_repair = $this->get_added_repair( $r_id );

				if( !empty( $added_repair ) ) {

					$rids[] = $r_id;

					$array_names[] = $added_repair->r_name;

					$array_prices[] = (float) $added_repair->r_price;

				} else {

					$added_repair_attr = $this->get_added_repair_attr_by_repair_id( $r_id );

					if( !empty( $added_repair_attr ) ) {

						$rids[] = $r_id;

						$array_names[] = $added_repair_attr->a_name;

						$array_prices[] = (float) $added_repair_attr->a_price;

						$aids[] = $added_repair_attr->a_id;

					}

				}

			}

			foreach ($u_ids as $u_id) {

				$added_upsale = $this->get_added_upsale( $u_id );

				if( !empty( $added_upsale ) ) {

					$upids[] = $u_id;

					$array_names[] = $added_upsale->us_name;

					$array_prices[] = (float) $added_upsale->us_new_price;

				}

			}

			return array( $rids, $array_names, $array_prices, $aids, $upids );

		}

	}

}
