<?php

namespace RepairPluginPro;

use RepairPluginPro\RP_Request_Handler;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if( !class_exists( 'RepairPluginPro\RP_Form_Inputs' ) ) {

	class RP_Form_Inputs {

		public static $errors = array();

		public static $is_admin_page = FALSE;

		public static $is_ajax_request = FALSE;

		public static $learning_mode = FALSE;

		public static $pending_processing = array();

		public static $validation_error = '';

		public static function need_to_do_validation() {

			self::$is_admin_page = RP_Request_Handler::is_repair_plugin_admin_page();

			self::$is_ajax_request = RP_Request_Handler::is_ajax_request_for_repair_plugin();

			if( self::$is_admin_page || self::$is_ajax_request ) {

				return TRUE;

			}

			return FALSE;

		}

		public static function test_exists( $test_name = '' ) {

			$test_key = 'test_' . $test_name; // test_is_email

			return method_exists( __CLASS__ , $test_key ) && is_callable( array( __CLASS__ , $test_key ) );

		}

		public static function call_the_test( $test_name = '', $value = null ) {

			// If Custom Rule is set, no need to apply default rules!
			$test_key = 'test_' . $test_name; // test_is_email

			if( self::test_exists( $test_name ) ) {

				// Has custom rule...
				return call_user_func( array( __CLASS__ , $test_key ) , $value );

			}

			throw new \Exception("Couldn't find the test: ".$test_key);
			

		}

		public static function test_required( $value = null ) {

			if( is_string( $value ) ) {

				return !empty( trim( $value ) );

			}

			return !empty( $value );

		}

		public static function test_is_phone( $phone = null ) {

			if( empty( $phone ) ) {

				return FALSE;

			}

			if( is_array( $phone ) || is_object( $phone ) ) {

				return FALSE;

			}

			return preg_match('/^([0-9\-\+\(\)\s]+)$/i', $phone) !== 0 && strlen( $phone ) > 6 && strlen( $phone ) <= 20;

		}

		public static function test_has_html( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return RP_Request_Handler::has_any_html_tag( $value );

		}

		public static function test_is_int( $value = null ) {

			if( is_int( $value ) || $value === 0 || $value === '0' ) {

				return TRUE;

			}

			if( !is_string( $value ) ) {

				return FALSE;

			}

			$value = trim( $value );

			$new_value = (int) $value;

			$new_value = $new_value . '';

			return ($new_value === $value);

		}

		public static function test_is_float( $value = null ) {

			if( is_float( $value ) || $value === '0.00' ) {

				return TRUE;

			}

			if( !is_string( $value ) ) {

				return FALSE;

			}

			$value = trim( $value );

			$new_value = (float) $value;

			$new_value = $new_value . '';

			return ($new_value === $value && strpos( $value, '.' ) !== FALSE);

		}

		public static function test_is_time( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return preg_match( '/^((([0-9]){2})(\:)(([0-9]){2})(((\:)(([0-9]){2}))|((\s)(am|pm)))*)$/i', trim( $value ) ) !== 0;

		}

		public static function test_is_date( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return preg_match( '/^(([0-9]{2,4}(\-|\/)[0-9]{2}(\-|\/)[0-9]{2,4}))$/i', trim( $value ) ) !== 0;

		}

		public static function test_is_datetime( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return preg_match( '/^(([0-9]{2,4}(\-|\/)[0-9]{2}(\-|\/)[0-9]{2,4})(\s)(([0-9]){2})(\:)(([0-9]){2})(((\:)(([0-9]){2}))|((\s)(am|pm)))*)$/i', trim( $value ) ) !== 0;

		}

		public static function test_is_amount( $value = null ) {

			if( $value === null || is_bool( $value ) ) {

				return FALSE;

			}

			if( self::test_is_int( $value ) || self::test_is_float( $value ) || is_numeric( $value ) ) {

				return TRUE;

			}

			if( is_string( $value ) ) {

				// Is Numeric? (Example: 100 or 99.99 or 99.999 or so on...)
				return preg_match( '/^([0-9])+((\.|\,)[0-9]+){0,1}$/i', trim( $value ) ) !== 0;

			}

			return FALSE;

		}

		public static function test_is_json( $value = '' ) {

			if( !empty( $value ) && is_string( $value ) && ((strpos( $value, '{' ) !== FALSE && strpos( $value, '}' ) !== FALSE) || (strpos( $value, '[' ) !== FALSE && strpos( $value, ']' ) !== FALSE)) && strpos( $value, '"' ) !== FALSE ) {

				try {

					$decode = json_decode( $value, TRUE );

					if( !empty( $decode ) ) {

						return TRUE;

					}

				} catch ( \Exception $e ) {

				}

			}

			return FALSE;

		}

		public static function test_is_email( $value = null ) {

			if( !empty( $value ) && is_string( $value ) ) {

				return !empty( is_email( $value ) );

			}

			return FALSE;

		}

		public static function test_valid_zip_code( $zip_code = null ) {

			if ( empty( $zip_code ) ) {

				return false;

			}

			if( is_array( $zip_code ) || is_object( $zip_code ) ) {

				return FALSE;

			}

			if ( 10 < strlen( trim( $zip_code ) ) ) {
				return false;
			}

			if ( ! preg_match( '/^\d{5}(\-?\d{4})?$/', $zip_code ) ) {
				return false;
			}

			return true;

		}

		public static function test_is_nested_array( $arr = null ) {

			if( empty( $arr ) || !is_array( $arr ) ) {

				return FALSE;

			}

			foreach ($arr as $value) {

				if( is_array( $value ) ) {

					return TRUE;

				}

			}

			return FALSE;

		}

		public static function test_is_associative_array( $arr = null ) {

			if( empty( $arr ) || !is_array( $arr ) ) {

				return FALSE;

			}

			foreach ($arr as $index => $value) {

				if( self::test_is_int( $index ) ) {

					return FALSE;

				}

			}

			return TRUE;

		}

		public static function test_is_empty_array( $arr = null ) {

			return is_array( $arr ) && empty( $arr );

		}

		public static function test_is_indexed_array( $arr = null ) {

			if( empty( $arr ) || !is_array( $arr ) ) {

				return FALSE;

			}

			foreach ($arr as $index => $value) {

				if( !self::test_is_int( $index ) ) {

					return FALSE;

				}

			}

			return TRUE;

		}

		public static function test_is_color_code( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return preg_match( '/(^#[0-9a-zA-Z]{3}$)|(^#[0-9a-zA-Z]{6}$)/i', trim( $value ) ) !== 0;

		}

		public static function test_is_url( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return filter_var( trim( $value ), FILTER_VALIDATE_URL ) !== FALSE;

		}

		public static function test_is_day( $value = null ) {

			if( empty( $value ) || !is_string( $value ) ) {

				return FALSE;

			}

			return in_array( trim( $value ) , array( 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday' ) ) !== FALSE;

		}

		public static function get_passed_tests( $value = null ) {

			$tests = array('required', 'is_string', 'is_array', 'is_float', 'is_object', 'is_int', 'is_email', 'valid_zip_code', 'is_phone', 'is_amount', 'is_bool', 'is_json', 'has_html', 'is_nested_array', 'is_associative_array', 'is_indexed_array', 'is_empty_array', 'is_date', 'is_datetime', 'is_time', 'is_color_code', 'is_url', 'is_day');

			$tests_result = array();

			foreach ($tests as $key => $test_name) {
			
				if( self::test_exists( $test_name ) ) {

					$tests_result[ $test_name ] = self::call_the_test( $test_name, $value ) ?? FALSE;

				} else {

					if( function_exists( $test_name ) && is_callable( $test_name ) ) {

						$tests_result[ $test_name ] = call_user_func( $test_name, $value ) ?? FALSE;

					} else {

						throw new \Exception("Test $test_name is not defined.");

					}

				}

			}

			return $tests_result;

		}

		public static function get_request_key() {

			if( self::$is_admin_page ) {

				return $_GET['page'];

			}

			if( self::$is_ajax_request ) {

				return $_REQUEST['action'];

			}

			return uniqid();

		}

		public static function is_ajax_request() {

			return trim( strtolower( $_SERVER['HTTP_X_REQUESTED_WITH'] ?? '' ) ) === 'xmlhttprequest';

		}

		public static function get_test_results_recursive( $name = '_request_', $input_arr = array() ) {

			$test_results = array();

			foreach ($input_arr as $key => $value) {

				$_key = $name.$key;

				$test_results[ $_key ] = self::get_passed_tests( $value );

				if( is_array( $value ) ) {

					if( self::test_is_indexed_array( $value ) ) {

						$test_results[ $_key.'__0' ] = self::get_passed_tests( $value[0] );

					} else {

						$items = self::get_test_results_recursive( $name, $value );

						foreach ($items as $item_key => $_value) {
							
							$test_results[ $_key.'__'.$item_key ] = $_value;

						}

					}

				}

			}

			return $test_results;

		}

		public static function str_replace_first($search, $replace, $subject) {
		    
			$pos = strpos($subject, $search);

			if ($pos !== false) {

				return substr_replace($subject, $replace, $pos, strlen($search));

			}

			return $subject;
		}

		public static function get_related_input( $key = '', $input = array() ) {

			$last_input = $input;

			if( strpos( $key, '__' ) !== FALSE ) {

				while ( strpos( $key, '__' ) !== FALSE ) {

					$current_key = explode( '__', $key )[0];

					$key = self::str_replace_first( $current_key.'__', '', $key );

					if(empty( $last_input ) || !is_array( $last_input ) || !isset( $last_input[ $current_key ] ) ) {

						$last_input = '';

					} else {

						$last_input = $last_input[$current_key];

					}

				}

			}

			$current_key = $key;

			$key = '';

			if(empty( $last_input ) || !is_array( $last_input ) || !isset( $last_input[ $current_key ] ) ) {

				$last_input = '';

			} else {

				$last_input = $last_input[$current_key];

			}

			return $last_input;

		}

		public static function compareRules( $passedTests = array(), $inputRules = array() ) {

			foreach ($inputRules as $key => $value) {
				
				if( $value == '1' ) {

					// Must Have Rule!
					if( $passedTests[ $key ] != '1' ) {

						return FALSE;

					}

				} else if( $value === 'o' ) {

					// Possible to have or not.
					// so better to skip
					continue;

				} else {

					// Must Not Have Rule!

					if( !empty( $passedTests[ $key ] ) ) {

						return FALSE;

					}

				}

			}

			return TRUE;

		}

		public static function check_inputs_against_rules( $input = array(), $allRules = array() ) {

			$items = array();

			foreach ($allRules as $key => $rules) {
				
				$items[ $key ] = self::get_related_input( $key, $input );

			}

			foreach ($items as $input_key => $input_value) {
				
				$inputRules = $allRules[ $input_key ];

				$passedTests = self::get_passed_tests( $input_value );

				if( !self::compareRules( $passedTests, $inputRules ) ) {

					self::$errors[ $input_key ] = array($input_value, $inputRules);

				}

			}

			return empty( self::$errors );

		}

		public static function rulesGenerator( $formIdToPrepare = '' ) {

			$oldRules = self::get_form_rules( $formIdToPrepare );

			if( $oldRules !== FALSE ) {

				self::save_test_results( $formIdToPrepare, $oldRules );

			}

			$requests = self::$pending_processing;

			self::$pending_processing = array();

			$preparedRules = array();

			foreach ($requests as $key => $value) {
				
				if( ($value['message'] ?? '') && strpos( $value['message'], 'form_' ) !== FALSE ) {

					$action_id = $value['message'];

					if( !isset( $preparedRules[ $action_id ] ) ) {

						$preparedRules[ $action_id ] = array();

					}

					// Is Form Learning Data...
					$rules = $value['additional'] ?? array();

					foreach ($rules as $input_key => $input_rules) {

						if( !isset( $preparedRules[ $action_id ][ $input_key ] ) ) {

							$preparedRules[ $action_id ][ $input_key ] = array();

						}
						
						foreach ($input_rules as $rule_name => $rule_value) {

							if( !isset( $preparedRules[ $action_id ][ $input_key ][ $rule_name ] ) ) {

								$preparedRules[ $action_id ][ $input_key ][ $rule_name ] = array();

							}

							if( $rule_value == '1' ) {

								$preparedRules[ $action_id ][ $input_key ][ $rule_name ][] = '1';

							} else if( $rule_value == 'o' ) {

								$preparedRules[ $action_id ][ $input_key ][ $rule_name ][] = 'o';

							} else {

								$preparedRules[ $action_id ][ $input_key ][ $rule_name ][] = '0';

							}

						}

					}

				}

			}

			$highestScoreByFormId = array();

			foreach ($preparedRules as $formId => $formItems) {

				$highestScore = 0;

				foreach ($formItems as $input_key => $inputRules) {
				
					foreach ($inputRules as $rule_key => $rule_values) {

						$is_enabled = array();

						$is_optional = FALSE;
						
						foreach ($rule_values as $value) {
							
							if( $value === '1' ) {

								$is_enabled[] = '1';

							}

							if( $value === 'o' ) {

								$is_optional = TRUE;

							}

						}

						if( $is_optional === TRUE ) {

							$preparedRules[ $formId ][ $input_key ][ $rule_key ] = 'o';

						} else {

							$final_decision = count( $is_enabled );

							if( $final_decision > $highestScore ) {

								$highestScore = $final_decision;

							}

							$preparedRules[ $formId ][ $input_key ][ $rule_key ] = $final_decision;

						}

					}

				}

				$highestScoreByFormId[ $formId ] = $highestScore;

			}

			foreach ($preparedRules as $formId => $formItems) {

				foreach ($formItems as $input_key => $inputRules) {

					foreach ($inputRules as $rule_key => $rule_value) {

						if( $rule_value !== 0 ) {

							if( $rule_value === 'o' || $rule_value !== $highestScoreByFormId[ $formId ] ) {

								$rule_value = 'o';

							} else {

								$rule_value = '1';

							}

						} else {

							$rule_value = '0';

						}

						$preparedRules[ $formId ][ $input_key ][ $rule_key ] = $rule_value;

					}

				}

			}

			foreach( $preparedRules as $key => $value ) {

				file_put_contents( WP_REPAIR_PLUGIN_PATH.'/rules/'.$key.'.json', json_encode( $value, JSON_PRETTY_PRINT ) );

			}

		}

		public static function has_form_rules( $form_id = '' ) {

			return file_exists( WP_REPAIR_PLUGIN_PATH.'/rules/'.$form_id.'.json' );

		}

		public static function get_form_rules( $form_id = '' ) {

			if( self::has_form_rules( $form_id ) ) {

				return json_decode( file_get_contents( WP_REPAIR_PLUGIN_PATH.'/rules/'.$form_id.'.json' ), TRUE );

			}

			return FALSE;

		}

		public static function save_test_results( $form_id = '', $rules = array() ) {

			$data = array();

			$data['message'] = $form_id;

			$data['additional'] = $rules;

			self::$pending_processing[] = $data;

		}

		public static function validation_handler() {

			if( self::need_to_do_validation() ) {

				$request_key = self::get_request_key();

				$method = '';

				$input = array();

				if( isset( $_GET ) && !empty( $_GET ) ) {

					foreach ($_GET as $key => $value) {
						
						$input[ $key ] = $value;

					}

					$method .= 'get';

				}

				if( isset( $_POST ) && !empty( $_POST ) ) {

					foreach ($_POST as $key => $value) {
						
						$input[ $key ] = $value;

					}

					$method .= 'post';

				}

				$form_id = 'form_'.$method.'_'.$request_key;

				if( self::$learning_mode === TRUE ) {

					$rules = self::get_test_results_recursive( '', ( $input ?? array() ) );

					self::save_test_results( $form_id, $rules );

					self::rulesGenerator( $form_id );

				} else {

					if( ($rules = self::get_form_rules( $form_id )) !== FALSE ) {

						if( !self::check_inputs_against_rules( $input, $rules ) ) {

							$errors = array_reverse( self::$errors );

							$failed_at = '';

							foreach ($errors as $key => $value) {
								
								$failed_at = $key;

							}

							$message = 'Invalid input provided for '.$failed_at. ' field.';

							if( !self::is_ajax_request() ) {

								RP_Session_Handler::start();

								$_SESSION['rp_validation_error'] =  $message;

								RP_Session_Handler::close();

								$referrer = $_SERVER['HTTP_REFERER'] ?? '';

								if( !empty( $referrer ) && strpos( $referrer, get_site_url() ) !== FALSE ) {

									wp_safe_redirect( $referrer );

								} else {

									wp_safe_redirect( get_site_url() );

								}

							} else {

								echo json_encode(array(

									'status' => false,

									'rp_validation_error' => $message

								));

							}

							exit;

						}

					}

				}

			}

		}

		public static function cache_session() {

			if( !RP_Request_Handler::is_repair_plugin_admin_page() ) {

				return;

			}

			RP_Session_Handler::start();

			$error = $_SESSION['rp_validation_error'] ?? '';

			if( !empty( $error ) ) {

			      unset( $_SESSION['rp_validation_error'] );

			}

			RP_Session_Handler::close();

			self::$validation_error = $error;

		}

		public static function get_validation_error() {

			return (self::$validation_error ?? '');

		}

		public static function init() {

			self::$learning_mode = FALSE;

			self::cache_session();

			self::validation_handler();

		}

	}

	// Run this as early as possible but after RepairPluginPro\RP_Request_Handler!
	add_action( 'plugins_loaded', array( 'RepairPluginPro\RP_Form_Inputs', 'init' ), -900 );

}
