<?php

namespace RepairPluginPro;

use RepairPluginPro\RP_Query_Builder;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class RP_Languages {

    protected $rpdb;

    public function __construct() {

        $this->rpdb = new RP_Query_Builder();

    }

    public function rp_update_translations() {

        /*
        IMPORTANT NOTE FOR BASE.REPAIR
        When language is deleted, all translations of that language should
        be deleted from rs_translations table
        */

        $keys = $_POST['keys'] ?? array();

        $values = $_POST['values'] ?? array();

        $translation_key = $_POST['translation_key'] ?? '';

        if( !empty( $translation_key ) && !empty( $keys ) && !empty( $values ) && count( $keys ) === count( $values ) ) {

            $default_language = \rp_get_default_language();

            $all_languages = \rp_get_all_languages();

            $all_languages_name = array();

            // get all languages name

            foreach($all_languages as $lang_row) {

                $all_languages_name[] = $lang_row->name;

            }

            // make sure all $keys item is in $all_languages_name

            foreach($keys as $key) {

                if( !in_array( $key, $all_languages_name ) ) {

                    echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));

                    exit();

                }

            }

            $default_translation = array();

            // Check if row exists in rs_translations table
            // where language = $key of $keys array and key = $translation_key

            foreach($keys as $index => $key) {

                if( trim($key) === trim($default_language) ) {

                    $default_translation = $values[$index];

                }

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $translation_key);
                $this->rpdb->where('language', $key);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_translations');

                if(!empty($result)) {

                    // update row

                    $this->rpdb->reset_query();
                    $this->rpdb->where('key', $translation_key);
                    $this->rpdb->where('language', $key);
                    $this->rpdb->limit(1);
                    $this->rpdb->update('rs_translations', array('value' => $values[$index]));

                } else {

                    // insert row

                    $this->rpdb->reset_query();
                    $this->rpdb->insert('rs_translations', array('key' => $translation_key, 'language' => $key, 'value' => $values[$index]));

                }

            }

            // send success message

            $this->rpdb->reset_query();
            $this->rpdb->where('key', $translation_key);
            $this->rpdb->where('language', trim($default_language));
            $this->rpdb->limit(1);
            $result = $this->rpdb->get('rs_translations');
            
            $default_translation = $result[0]->value;

            // convert &quot; to " for default_translation
            $default_translation = str_replace('&quot;', '"', $default_translation);

            echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated', 'default_translation' => $default_translation ));
            
            exit();

        } else {

            // send error message

            echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));
            
            exit();

        }

    }
    
    public function rp_get_related_translations() {

        $key = $_POST['key'] ?? '';

        $translations = array();

        $static_translations_arr = rp_get_static_translations_of_language('English (United States)');

        $static_translation = $static_translations_arr[$key] ?? '';
        
        if( !empty( $key ) ) {

            $all_languages = \rp_get_all_languages();

            foreach($all_languages as $lang_row) {

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $key);
                $this->rpdb->where('language', $lang_row->name);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_translations');

                if(!empty($result)) {
                    $lang_row->t_value = htmlspecialchars($result[0]->value);
                } else {
                    if($lang_row->name === 'Dutch (Netherlands)') {
                        $dutch_arr = rp_get_static_translations_of_language('Dutch (Netherlands)');
                        $lang_row->t_value = htmlspecialchars($dutch_arr[$key] ?? $static_translation);
                    } else {
                        $lang_row->t_value = htmlspecialchars($static_translation);
                    }
                }

                $lang_row->image = \rp_fix_language_image($lang_row->image);

                $translations[] = $lang_row;

            }

        }

        echo json_encode(Array('status'=> true, 'translations'  => $translations, 'static_translation' => htmlspecialchars($static_translation)));

        exit();

    }

    public function rp_delete_language() {

        $language_id = $_POST['rid'] ?? '';

        /* Added Only for Plugin Side -- START */

        $default_language_id = \rp_get_default_language_id();

        if( $language_id === $default_language_id ) {

            echo json_encode(Array('status'=> false, 'message'  => 'Default Language can not be deleted'));

            exit();

        }

        /* Added Only for Plugin Side -- END */

        if( !empty( $language_id ) ) {

            // check if language_id exists in rs_languages table

            $this->rpdb->reset_query();
            $this->rpdb->where('id', $language_id);
            $this->rpdb->limit(1);
            $result = $this->rpdb->get('rs_languages');

            if(!empty($result)) {

                $this->rpdb->reset_query();
                $this->rpdb->where('id', $language_id);
                $this->rpdb->delete('rs_languages');

                /*
                IMPORTANT NOTE FOR BASE.REPAIR
                When language is deleted, all translations of that language should
                be deleted from rs_translations table
                */

                $row = $result[0];

                $language_name = $row->name;

                \rp_delete_related_translations_from_all_tables($language_name);

            }

        }

        echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

        exit();

    }

    public function rp_update_language_image(){
    
        $logo = '';
        if(isset($_FILES) && !empty($_FILES) && isset($_FILES['file'])) {
            $uploadedfile = $_FILES['file']; 
            $upload_overrides = array( 'test_form' => false );
            $movefile = \wp_handle_upload( $uploadedfile, $upload_overrides );
            if(!empty($movefile["url"])){
                    $logo = $movefile["url"];
            }
            if(isset($_POST) && !empty($_POST)) {
                    $language_id = $_POST['rid'] ?? '';

                    if(!empty($language_id) && !empty($logo) && trim(strtoupper($language_id)) !== 'NEW') {

                        $this->rpdb->reset_query();
                        $this->rpdb->where('id', $language_id);
                        $this->rpdb->limit(1);
                        $result = $this->rpdb->get('rs_languages');

                        if(!empty($result)) {

                            $this->rpdb->reset_query();
                            $this->rpdb->where('id', $language_id);
                            $this->rpdb->limit(1);
                            $this->rpdb->update('rs_languages', array('image' => $logo));

                        }

                    }
            }
        }
    
        echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'logo' => $logo));
    
        exit();
    
    }

    public function rp_update_languages(){

        /* Added Only for Plugin Side -- START */

        $default_language_id = \rp_get_default_language_id();

        /* Added Only for Plugin Side -- END */
        
    
        $mid    = $_POST['mid'];     
        $rid    = $_POST['rid'] ?? array();     
        $title  = $_POST['l_name']; 
        $info   = $_POST['alt_name'];
        $status = $_POST['status'] ?? array();  
        $img_link = $_POST['img_link'];
    
        $skipped_index = array();
    
        // Update/Create Repairs of Model
        foreach($rid as $i => $r){
            if(trim(strtoupper($r)) === 'NEW') {
                    // Create
                    if(!empty(trim($title[$i]))) {
                        
                        $uq_id = rp_create_unique_language_id(trim($title[$i]));

                        $this->rpdb->reset_query();

                        $this->rpdb->insert('rs_languages', array(
                            'key' => \rp_snake_case($title[$i]),
                            'name' => trim($title[$i]),
                            'alt_name' => trim($info[$i]),
                            'image' => $img_link[$i],
                            'position' => '0',
                            'is_active' => '1',
                            'uq_id' => $uq_id
                        ));

                        $rid[$i] = $this->rpdb->insert_id();
                    } else {
                        $skipped_index[] = $i;
                    }
            } else {
                    // Update
                    $this->rpdb->reset_query();
                    $this->rpdb->where('id', $r);
                    $old_row = $this->rpdb->get('rs_languages');
        
                    if( !empty( $old_row ) ) {

                        $old_row_language_name = $old_row[0]->name;

                        \rp_update_language_name_everywhere( $old_row_language_name, $title[$i] );
    
                        $this->rpdb->reset_query();
                        $this->rpdb->where('id', $r);
                        $this->rpdb->update('rs_languages', array(
                            'key' => \rp_snake_case($title[$i]),
                            'name' => trim($title[$i]),
                            'alt_name' => trim($info[$i]),
                            'image' => $img_link[$i]
                        ));

                    }
        
            }
        }
            
        // Update Repair Position
        $r_i = 1; 
    
        foreach($rid as $i => $repairId){
    
            if(!in_array($i, $skipped_index)) {

                $this->rpdb->reset_query();
                $this->rpdb->where('id', $repairId);
                $this->rpdb->update('rs_languages', array('position' => $r_i));
    
                $r_i++;
            }
    
        }
        // Update Repair Statuses

        $this->rpdb->reset_query();
        $this->rpdb->update('rs_languages', array('is_active' => '0'));

        foreach($status as $i => $s){
            if(!in_array($i, $skipped_index)) {
                if(!empty($s)) {
                    $this->rpdb->reset_query();
                    $this->rpdb->where('id', $rid[$i]);
                    $this->rpdb->update('rs_languages', array('is_active' => '1'));
                }
            }
        }

        /* Added Only for Plugin Side -- START */

        if(!empty($default_language_id)) {
            $this->rpdb->reset_query();
            $this->rpdb->where('id', $default_language_id);
            $this->rpdb->update('rs_languages', array('is_active' => '1'));

            $default_language_row = rp_get_language_row_by_id($default_language_id);

            if( !empty($default_language_row) ) {

                rp_update_default_language($default_language_row->name);

            }

        }

        /* Added Only for Plugin Side -- END */
    
        $table_rows = array();

        $this->rpdb->reset_query();
        $this->rpdb->select('*');
        $this->rpdb->order_by('position ASC, id ASC');
        $c_repair_all = $this->rpdb->get('rs_languages');

        if(!empty($c_repair_all)) {
                foreach ($c_repair_all as $i => $c_repair) {
                    $tr = array();
                    $tr['is_active'] = $c_repair->is_active;
                    $tr['m_id'] = $mid;
                    $tr['r_id'] = $c_repair->id;
                    $tr['r_img'] = \rp_fix_language_image($c_repair->image); // Need to fix image only in plugin side for backward compatibility
                    $tr['repair_title'] = $c_repair->name;
                    $tr['repair_info'] = $c_repair->alt_name;
                    $table_rows[] = $tr;
                }
        }
    
        echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'table_rows' => $table_rows));
    
        exit();
    
    }

}
