<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class RP_Maps_API {

    protected $wpdb;

    protected $api_key = '';

    public $error = '';

	public function __construct($rpQuery = NULL) {

		$this->wpdb = $rpQuery;

        $this->api_key = \rp_get_option('rp_el_google_maps_api_key', '');

	}

    public function save_cached_response( $data = array() ) {

        if( empty( $data ) ) {

            return false;

        }

        // table: rs_maps_api_cached
        // fields: input, type, response, output

        $rs_maps_api_cached = $this->wpdb->prefix . 'rs_maps_api_cached';

        // check if already exists
        if( $this->get_cached_response( $data['type'], $data['input'] ) ) {

            $sql = "UPDATE $rs_maps_api_cached SET `response` = ':the_response', `output` = ':the_output' WHERE `type` = ':the_type' AND `input` = ':the_input'";

            $result = $this->wpdb->query( $sql, array( 'the_response' => $data['response'], 'the_output' => $data['output'], 'the_type' => $data['type'], 'the_input' => $data['input'] ) );

        } else {

            $sql = "INSERT INTO $rs_maps_api_cached (`input`, `type`, `response`, `output`) VALUES (':the_input', ':the_type', ':the_response', ':the_output')";

            $result = $this->wpdb->query( $sql, array( 'the_input' => $data['input'], 'the_type' => $data['type'], 'the_response' => $data['response'], 'the_output' => $data['output'] ) );

        }

        return $result;
        
    }

    public function get_cached_response( $type = '', $input = '' ) {
        
        if( empty( $type ) || empty( $input ) ) {

            return false;

        }

        // table: rs_maps_api_cached
        // fields: input, type, response, output

        $rs_maps_api_cached = $this->wpdb->prefix . 'rs_maps_api_cached';

        $sql = "SELECT * FROM $rs_maps_api_cached WHERE type = ':the_type' AND input = ':the_input' LIMIT 1";

        $result = $this->wpdb->get_row( $sql, array( 'the_type' => $type, 'the_input' => $input ) );

        if( !empty( $result ) ) {

            if( $type === 'formatted_address' ) {

                return $result->output;

            } else {

                return json_decode( $result->output, true );

            }

        }

        return false;
        
    }

    public function create_request_url( $params = array() ) {

        if( empty( $this->api_key ) ) {

            $this->error = 'Please enter Google Maps API Key';

            return false;

        }

        $params['key'] = $this->api_key;

        if( array_key_exists('input', $params) ) {
            $request_url = 'https://maps.googleapis.com/maps/api/place/autocomplete/json?';
        } else {
            $request_url = 'https://maps.googleapis.com/maps/api/geocode/json?';
        }

        // loop through all params and create request url

        foreach( $params as $key => $value ) {

            if( $key == 'address' || $key == 'input' ) {

                $value = urlencode( $value );

            }

            $request_url .= $key . '=' . $value . '&';

        }

        $request_url = rtrim( $request_url, '&' );

        return $request_url;

    }

    public function create_request( $params = array() ) {

        $request_url = $this->create_request_url( $params );

        if( $request_url === false ) {

            return false;

        }

        // init curl
        $curl = curl_init();
    
        curl_setopt( $curl, CURLOPT_URL, $request_url );
    
        // gzip
        curl_setopt( $curl, CURLOPT_ENCODING, "gzip" );
    
        curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
    
        curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false );
    
        curl_setopt( $curl, CURLOPT_SSL_VERIFYHOST, false );
    
        $response = curl_exec( $curl );
    
        $info = curl_getinfo( $curl );
    
        curl_close( $curl );

        if( $info['http_code'] == 200 && !empty( $response ) ) {

            return json_decode( $response, true );

        }
        
        $this->error = 'Failed to receive response from Google Maps API. HTTP Code: ' . $info['http_code'] . ' Response: ' . $response;

        return false;

    }

    public function get_formatted_address( $latlng = '' ) {

        // replace double spaces with single space
        $latlng = preg_replace( '/\s+/', ' ', $latlng );

        $latlng = trim( $latlng );

        // make sure it only contains 0-9, ., comma, space or -
        if( !preg_match( '/^[0-9\.\,\s\-\+]+$/', $latlng ) ) {

            $this->error = 'Invalid latitude and longitude';

            return false;

        }

        $cached_response = $this->get_cached_response( 'formatted_address', $latlng );

        if( !empty( $cached_response ) ) {

            return $cached_response;

        }

        $params = array(

            'latlng' => $latlng,

        );//formatted_address

        $data = $this->create_request( $params );

        // make sure $data['results'][0]['formatted_address'] is not empty

        if( !empty( $data ) && isset( $data['results'] ) ) {

            $results = $data['results'];

            if( !empty( $results ) ) {

                $the_result = $results[0];

                if( isset( $the_result['formatted_address'] ) && !empty( $the_result['formatted_address'] ) ) {

                    $this->save_cached_response( array(

                        'input' => $latlng,

                        'type' => 'formatted_address',

                        'response' => json_encode( $data ),

                        'output' => $the_result['formatted_address']

                    ) );

                    return $the_result['formatted_address'];

                }

            }

        }

        $this->error = 'Failed to get formatted address from the response. Response: ' . json_encode( $data );

        return false;

    }

    public function fix_address( $address = '' ) {

        // only allow letters, numbers, spaces, commas, points, and dashes

        $address = preg_replace( '/[^a-zA-Z0-9\s\,\.\-\(\)]/', '', $address );

        // replace double spaces with single space
        $address = preg_replace( '/\s+/', ' ', $address );

        $address = trim( strtolower( $address ) );

        return $address;

    }

    public function get_autocomplete_suggestions( $input = '' ) {

        $input = $this->fix_address( $input );

        $cached_response = $this->get_cached_response( 'autocomplete', $input );

        if( !empty( $cached_response ) ) {

            return $cached_response;

        }

        $params = array(

            'input' => $input,

        );

        $data = $this->create_request( $params );

        if( !empty( $data ) && isset($data['status']) && $data['status'] == 'OK' && isset($data['predictions']) && !empty($data['predictions']) && is_array($data['predictions']) ) {

            $gpredictions = $data['predictions'];

            $predictions = array();

            foreach( $gpredictions as $thePrediction ) {

                $predictions[] = $thePrediction['description'];

            }

            // limit to 5 results
            $predictions = array_slice( $predictions, 0, 5 );

            $this->save_cached_response( array(

                'input' => $input,

                'type' => 'autocomplete',

                'response' => json_encode( $data ),

                'output' => json_encode( $predictions )

            ) );

            return $predictions;

        }

        return array();

    }

    public function get_lat_long( $address = '' ) {

        $address = $this->fix_address( $address );

        $cached_response = $this->get_cached_response( 'lat_long', $address );

        if( !empty( $cached_response ) ) {

            return $cached_response;

        }

        $params = array(

            'address' => $address,

        );

        $data = $this->create_request( $params );

        // make sure $data['results'][0]['geometry']['location'] is not empty

        if( !empty( $data ) && isset( $data['results'] ) ) {

            $results = $data['results'];

            if( !empty( $results ) ) {

                $the_result = $results[0];

                if( isset( $the_result['geometry'] ) && !empty( $the_result['geometry'] ) ) {

                    $geometry = $the_result['geometry'];

                    if( isset( $geometry['location'] ) && !empty( $geometry['location'] ) ) {

                        $the_response = array(
                            $geometry['location']['lat'],
                            $geometry['location']['lng']
                        );

                        $this->save_cached_response( array(

                            'input' => $address,

                            'type' => 'lat_long',

                            'response' => json_encode( $data ),

                            'output' => json_encode( $the_response )

                        ) );

                        return $the_response;
                        
                    }

                }

            }

        }

        $this->error = 'Failed to get longitude and latitude from the response. Response: ' . json_encode( $data );

        return false;

    }

    public function find_radius_distance($lat1 = '', $lon1 = '', $lat2 = '', $lon2 = '') {

        $lat1 = (float) $lat1;
        $lon1 = (float) $lon1;
        $lat2 = (float) $lat2;
        $lon2 = (float) $lon2;

        $Rm = 3961; // mean radius of the earth (miles) at 39 degrees from the equator
        $Rk = 6373; // mean radius of the earth (km) at 39 degrees from the equator
    
        // get values for lat1, lon1, lat2, and lon2
        $t1 = $lat1;
        $n1 = $lon1;
        $t2 = $lat2;
        $n2 = $lon2;
    
        // convert coordinates to radians
        $lat1 = deg2rad($t1);
        $lon1 = deg2rad($n1);
        $lat2 = deg2rad($t2);
        $lon2 = deg2rad($n2);
    
        // find the differences between the coordinates
        $dlat = $lat2 - $lat1;
        $dlon = $lon2 - $lon1;
    
        // here's the heavy lifting
        $a  = pow(sin($dlat/2),2) + cos($lat1) * cos($lat2) * pow(sin($dlon/2),2);
        $c  = 2 * atan2(sqrt($a),sqrt(1-$a)); // great circle distance in radians
        $dm = $c * $Rm; // great circle distance in miles
        $dk = $c * $Rk; // great circle distance in km
    
        // round the results down to the nearest 1/1000
        $mi = round($dm,3);
        $km = round($dk,3);

        $rp_el_prefered_distance_unit = rp_get_option('rp_el_prefered_distance_unit', 'km');

        if( $rp_el_prefered_distance_unit == 'km' ) {

            return $km;

        }

        return $mi;
    
    }

}
