<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class RP_Queries_Handler {

	private $wpdb;

	public $prefix = 'wp_';

	public $posts = 'wp_posts';

	public $insert_id = NULL;

	public $preparedVars = array();

	public $generatedIds = array();

	public function __construct( $wpdb ) {

		$this->wpdb = $wpdb;

		$this->prefix = $wpdb->prefix;

		$this->posts = $wpdb->posts;
		
	}

	public function rp_escape_sql( $string = '' ) {

		if( empty( $string ) ) {

		      return $string;

		}

		if( is_int( $string ) || is_float( $string ) ) {

		      return $string;

		}

		if( !is_string( $string ) ) {

		      if( function_exists('trigger_error') ) {

		            rpDebug('Argument #1 provided to rp_escape_sql function should be a string. Wrong argument provided: '.json_encode( $string ));

		            return '';

		      }

		}

		if( empty( trim( $string ) ) ) {

		      return $string;

		}

		// Is Numeric? (Example: 100 or 99.99 or 99.999 or so on...)
		if( preg_match( '/^([0-9])+(\.[0-9]+){0,1}$/i', trim( $string ) ) ) {

		      return trim( $string );

		}

		// Pass String In Original Form to "prepareGenerate" function.
		return $this->prepareGenerate( wp_unslash( $string ) );

	}

	public function escape_query_vars( $args = array() ) {

		if( empty( $args ) ) {

			return;

		}

		foreach ($args as $key => $value) {

			$args[ $key ] = $this->rp_escape_sql( $value );

		}

		return $args;

	}

	public function generate_unique_id( $length = 32 ) {

		$tries = 0;

		while ( $tries < 32 ) {

			$id = $this->generate_random_string( $length ); // Generate Random String

			if( array_search( $id, $this->generatedIds ) === FALSE ) {

				$this->generatedIds[] = $id;

				return $id;

			}

			$tries++;

		}

		// This will never happen but let's still keep something in backup

		$id = $this->generate_random_string( $length, uniqid() );

		$this->generatedIds[] = $id;

		return $id;

	}

	public function generate_random_string( $length = 32, $additional_string = '' ) {

		$chars = 'abcdefghijklmnopqrstuvwxyz';
		$chars .= 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$chars .= '1234567890';
		$chars .= $additional_string;

		$chars = str_split( $chars );

		$total_chars = count( $chars );

		$count = 0;

		$id = '';

		while( $count < $length ) {

			$randomNumber = rand( 0, ($total_chars - 1 ));

			$id .= $chars[ $randomNumber ];

			$count++;

		}

		return $id;

	}

	public function prepareGenerate( $string = '' ) {

		$uniqueId = $this->generate_unique_id( 20 );

		$this->preparedVars[ $uniqueId ] = $string;

		return $uniqueId;

	}

	public function get_charset_collate() {

		return $this->wpdb->get_charset_collate();

	}

	public function prepare_sql( $sql = '' ) {

		$count = 1;

		$vars = array();

		foreach ($this->preparedVars as $key => $value) {

			if( strpos( $sql, $key ) !== FALSE ) {

				if( strpos( str_replace( ' ', '', $sql ), 'IN('.$key.')' ) !== FALSE ) {

					$sql = str_replace( $key, wp_slash($value), $sql );

				} else if( strpos( str_replace( ' ', '', $sql ), 'LIKE\''.$key.'\'' ) !== FALSE ) {

					$sql = str_replace( "'".$key."'", '%s', $sql );

					$vars[] = $this->wpdb->esc_like($value);

				} else if( strpos( str_replace( ' ', '', $sql ), 'LIKE\'%'.$key.'%\'' ) !== FALSE ) {

					$sql = str_replace( "'%".$key."%'", "%s", $sql );

					$vars[] = '%' . $this->wpdb->esc_like($value) . '%';

				} else {

					if( strpos( $sql, "'".$key."'" ) !== FALSE ) {

						$sql = str_replace( "'".$key."'", '%s', $sql );

						$vars[] = $value;

					} else {

						if( strpos( $sql, $key ) !== FALSE ) {

							$org_sql = $sql;

							$sql = str_replace( "'".$key."'", '%s', $sql );

							$vars[] = $value;

							rpDebug('Looks like invalid query: [ORIGINAL] '.$org_sql.' [PREPARED] '.$sql);

						}

					}

				}

				$count++;

			}

		}

		if( !empty( $vars ) ) {

			$input = array( $sql );

			foreach ($vars as $var) {
			
				$input[] = $var;

			}

			$prepared = call_user_func_array( array( $this->wpdb, 'prepare' ), $input );

			return $prepared;

		}

		return $sql;

	}

	public static function str_replace_first($search, $replace, $subject) {
	    
		$pos = strpos($subject, $search);

		if ($pos !== false) {

			return substr_replace($subject, $replace, $pos, strlen($search));

		}

		return $subject;
	}

	public function get_all_query_variable_names( $sql = '' ) {

		$all_matches = array();

		while( preg_match( '/(\'|\%)*\:[a-zA-Z\_]([a-zA-Z0-9\_])+(\%|\')*/', $sql, $matches ) ) {

			$all_matches[] = $matches[0];

			$count = 1;

			$sql = self::str_replace_first( $matches[0], '', $sql, $count );

		}

		return $all_matches;

	}

	public function prepare_easy_query( $sql = '', $args = array() ) {

		if( !empty( $args ) ) {

			$vars = $this->get_all_query_variable_names($sql);

			if( empty( $vars ) ) {

				throw new \Exception("Args provided but doesn't match with given query ".$sql);

			}

			$matched_args = array();

			foreach ($vars as $match) {

				$match = str_replace( array('\'', ':', '%'), '', $match );

				if( !isset( $args[ $match ] ) ) {

					throw new \Exception("Failed to parse your query. ".$match." argument couldn't be found in passed data.");

				}

				$matched_args[] = $match;

			}

			foreach ($matched_args as $ig => $key) {

				$value = $this->rp_escape_sql( $args[ $key ] );
		
				if( strpos( $sql, ':'.$key ) === FALSE ) {

					throw new \Exception("Something is unexpected with the query: ".$sql);

				}

				if( strpos( $sql, "':".$key."'" ) === FALSE ) {

					$matches = null;

					if( !preg_match( '/\'(\%)*\:'.$key.'(\%)*\'/', $sql, $matches ) ) {

						throw new \Exception("The ".$key." isn't wrapped with single quotes(') in the sql string: ".$sql);

					} else {

						$sql = self::str_replace_first( $matches[0], "'%".$value."%'", $sql );

					}

				} else {

					$sql = self::str_replace_first( '\':'.$key . '\'', "'".$value."'", $sql );

				}

			}

		}

		return $sql;

	}

	public function get_results( $sql = '', $args = array() ) {

		$sql = $this->prepare_easy_query( $sql, $args );

		$sql = $this->prepare_sql( $sql );

		return $this->wpdb->get_results( $sql );

	}

	public function query( $sql = '', $args = array() ) {

		$sql = $this->prepare_easy_query( $sql, $args );

		$sql = $this->prepare_sql( $sql );

		$query = $this->wpdb->query( $sql );

		$this->insert_id = ($this->wpdb->insert_id ?? NULL);

		return $query;

	}

	public function insert( $table_name = '', $sql = '' ) {

		$insert = $this->wpdb->insert( $table_name, $sql );

		$this->insert_id = ($this->wpdb->insert_id ?? NULL);

		return $insert;

	}

	public function get_row( $sql = '', $args = array() ) {

		$sql = $this->prepare_easy_query( $sql, $args );

		$sql = $this->prepare_sql( $sql );

		return $this->wpdb->get_row( $sql );

	}

	public function get_var( $sql = '', $args = array() ) {

		$sql = $this->prepare_easy_query( $sql, $args );

		$sql = $this->prepare_sql( $sql );

		return $this->wpdb->get_var( $sql );

	}


	public function example_query() {

		global $rpQuery;

		$result = $rpQuery->get_results(

		      "SELECT * FROM wp_rs_model WHERE m_name LIKE '%:m_name%' AND m_id = ':m_id'",

		      array(

		            'm_name' => 'iphone',

		            'm_id' => 1882

		      )

		);

		return $result;

	}

}

