<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class RP_Query_Builder {
    
    protected $conn;
    
    protected $where = array();

    protected $order_by = '';

    protected $select = '*';

    protected $table_name = '';

    protected $limit = array();

    protected $data = array();

    protected $wpdb;

    public $last_query = '';

    public function __construct() {

        global $rpQuery;

        global $wpdb;

        $this->conn = $rpQuery;

        $this->wpdb = $wpdb;

    }

    public function insert_id() {

        return $this->conn->insert_id;

    }

    public function reset_query() {

        $this->where = array();
        $this->order_by = '';
        $this->select = '*';
        $this->table_name = '';
        $this->limit = array();
        $this->data = array();

    }

    public function select($columns = '*') {

        if (is_array($columns)) {
            $columns = implode(', ', $columns);
        }

        $this->select = $columns;

    }

    public function order_by($column = '', $order = '') {

        if(strpos($column, 'ASC') === false && strpos($column, 'DESC') === false) {
            $order = strtoupper($order);
            if($order !== 'ASC' && $order !== 'DESC') {
                $order = 'ASC';
            }
            $column = $column . ' ' . $order;
        } else {
            $order = '';
        }

        $this->order_by = $column;

    }

    public function limit($limit = false, $offset = false) {

        $this->limit = array(
            $limit,
            $offset
        );

    }

    public function prepare_query($type = 'SELECT') {

        $this->table_name = $this->conn->prefix . $this->table_name;

        $query = '';

        switch ($type) {
            case 'SELECT':
                $query = 'SELECT ' . $this->select . ' FROM ' . $this->table_name;
                break;
            case 'INSERT':
                $query = 'INSERT INTO ' . $this->table_name;
                break;
            case 'UPDATE':
                $query = 'UPDATE ' . $this->table_name;
                break;
            case 'DELETE':
                $query = 'DELETE FROM ' . $this->table_name;
                break;
        }

        if( !empty($this->data) ) {

            if( $type === 'INSERT' ) {
                $keys = array();
                $values = array();
                foreach($this->data as $key => $value) {
                    $keys[] = $key;
                    $values[] = rp_escape_sql($value);
                }
                $keys_str = "`" . implode('`, `', $keys) . "`";
                $values_str = "'" . implode("', '", $values) . "'";
                $query .= " ({$keys_str}) VALUES ({$values_str})";
            }

            if( $type === 'UPDATE' ) {

                $query .= ' SET ';
                foreach($this->data as $key => $value) {
                    $query .= '`' . $key . '` = \'' . rp_escape_sql($value) . '\', ';
                }
                $query = rtrim($query, ', ');

            }

        }

        if (count($this->where) > 0) {
            $query .= ' WHERE ';
            foreach ($this->where as $where) {
                $query .= $where[0] . ' \'' . rp_escape_sql($where[1]) . '\' AND ';
            }
            $query = rtrim($query, ' AND ');
        }

        if (!empty($this->order_by)) {
            $query .= ' ORDER BY ' . $this->order_by;
        }

        if (!empty($this->limit)) {
            if($this->limit[0] !== false) {
                $query .= ' LIMIT ' . $this->limit[0];
            }
            if($this->limit[1] !== false) {
                $query .= ' OFFSET ' . $this->limit[1];
            }
        }

        $this->last_query = $query;

        $this->reset_query();

        return $query;

    }

    public function where($column = '', $value = '') {

        $column = explode(' ', $column);

        $valid_operators = array('=', '!=', '>', '<', '>=', '<=', 'LIKE', 'NOT LIKE', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN', 'EXISTS', 'NOT EXISTS');

        if (count($column) == 1) {
            $column = '`' . $column[0] . '` = ';
        } else {
            // check if operator is valid
            if (in_array($column[1], $valid_operators)) {
                $column = '`' . $column[0] . '` ' . $column[1] . ' ';
            } else {
                throw new \Exception('Invalid operator');
            }
        }

        $this->where[] = array(
            $column,
            $value
        );

    }

    public function update($table_name = '', $data = array()) {

        $this->table_name = $table_name;

        $this->data = $data;

        $query = $this->prepare_query('UPDATE');

        $result = $this->conn->query($query);

        return $result;

    }

    public function insert($table_name = '', $data = array()) {

        $this->table_name = $table_name;

        $this->data = $data;

        $query = $this->prepare_query('INSERT');

        $result = $this->conn->query($query);

        return $result;

    }
    
    public function delete($table_name = '') {

        $this->table_name = $table_name;

        $query = $this->prepare_query('DELETE');

        $result = $this->conn->query($query);

        return $result;

    }

    public function get($table_name = '') {
        
        $this->table_name = $table_name;

        $query = $this->prepare_query('SELECT');

        $result = $this->conn->get_results($query);

        return $result;

    }

}
