<?php

namespace RepairPluginPro;

use RepairPluginPro\RP_Query_Builder;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class RP_Repair_Translations {

    protected $rpdb;

    public function __construct() {

        $this->rpdb = new RP_Query_Builder();

    }

    public function rp_update_r_translations() {

        /*
        IMPORTANT NOTE FOR BASE.REPAIR
        When language is deleted, all translations of that language should
        be deleted from rs_translations table
        */

        $keys = $_POST['keys'] ?? array();

        $values = $_POST['values'] ?? array();

        $translation_key = $_POST['translation_key'] ?? '';

        $translation_uqid = $_POST['translation_uqid'] ?? '';

        $translation_connection_id = $_POST['translation_connection_id'] ?? '';

        if( !empty( $translation_key ) && !empty( $keys ) && !empty( $values ) && count( $keys ) === count( $values ) && !empty( $translation_uqid ) && !empty( $translation_connection_id ) ) {

            $default_language = \rp_get_default_language();

            $all_languages = \rp_get_all_languages();

            $all_languages_name = array();

            // get all languages name

            foreach($all_languages as $lang_row) {

                $all_languages_name[] = $lang_row->name;

            }

            // make sure all $keys item is in $all_languages_name

            foreach($keys as $key) {

                if( !in_array( $key, $all_languages_name ) ) {

                    echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));

                    exit();

                }

            }

            $default_translation = '';

            // Check if row exists in rs_translations table
            // where language = $key of $keys array and key = $translation_key

            foreach($keys as $index => $key) {

                if( trim($key) === trim($default_language) ) {

                    $default_translation = $values[$index];

                }

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $translation_key);
                $this->rpdb->where('uq_id', $translation_uqid);
                $this->rpdb->where('m_id_fk', $translation_connection_id);
                $this->rpdb->where('language', $key);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_r_translations');

                if(!empty($result)) {

                    // update row

                    $this->rpdb->reset_query();
                    $this->rpdb->where('key', $translation_key);
                    $this->rpdb->where('uq_id', $translation_uqid);
                    $this->rpdb->where('m_id_fk', $translation_connection_id);
                    $this->rpdb->where('language', $key);
                    $this->rpdb->limit(1);
                    $this->rpdb->update('rs_r_translations', array('value' => $values[$index]));

                } else {

                    // insert row

                    $this->rpdb->reset_query();
                    $this->rpdb->insert('rs_r_translations', array('key' => $translation_key, 'language' => $key, 'value' => $values[$index], 'uq_id' => $translation_uqid, 'm_id_fk' => $translation_connection_id));

                }

            }

            $what_info = 'r_name';

            if($translation_key === 'repair_name') {
                $what_info = 'r_name';
            }

            if($translation_key === 'repair_desc') {
                $what_info = 'r_discription';
            }

            if($translation_key === 'repair_badge') {
                $what_info = 'r_badge';
            }

            $data_to_update = array();
            $data_to_update[$what_info] = $default_translation;

            $this->rpdb->reset_query();
            $this->rpdb->where('r_uq_id', $translation_uqid);
            $this->rpdb->where('m_id_fk', $translation_connection_id);
            $this->rpdb->update('rs_repair', $data_to_update);

            // send error message

            echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated', 'default_translation' => htmlspecialchars($default_translation)));
            
            exit();

        } else {

            // send error message

            echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));
            
            exit();

        }

    }
    
    public function rp_get_r_translations() {

        $key = $_POST['key'] ?? '';
        $uqid = $_POST['uqid'] ?? '';
        $connection_id = $_POST['connection_id'] ?? '';

        $translations = array();

        $static_translation = '';
        
        if( !empty( $key ) ) {

            $static_translation = \rp_get_static_translations_of_repair($key, $uqid, $connection_id);

            $all_languages = \rp_get_all_languages();

            foreach($all_languages as $lang_row) {

                $result_text = rp_get_static_translations_of_repair_by_language($key, $uqid, $connection_id, $lang_row->name);

                if(!empty($result_text)) {
                    $lang_row->t_value = htmlspecialchars($result_text);
                } else {
                    $lang_row->t_value = htmlspecialchars($static_translation);
                }

                $lang_row->image = \rp_fix_language_image($lang_row->image);

                $translations[] = $lang_row;

            }

        }

        echo json_encode(Array('status'=> true, 'translations'  => $translations, 'static_translation' => htmlspecialchars($static_translation)));

        exit();

    }

    public function rp_update_rattr_translations() {

        /*
        IMPORTANT NOTE FOR BASE.REPAIR
        When language is deleted, all translations of that language should
        be deleted from rs_translations table
        */

        $keys = $_POST['keys'] ?? array();

        $values = $_POST['values'] ?? array();

        $translation_key = $_POST['translation_key'] ?? '';

        $translation_uqid = $_POST['translation_uqid'] ?? '';

        $translation_connection_id = $_POST['translation_connection_id'] ?? '';

        if( !empty( $translation_key ) && !empty( $keys ) && !empty( $values ) && count( $keys ) === count( $values ) && !empty( $translation_uqid ) && !empty( $translation_connection_id ) ) {

            $default_language = \rp_get_default_language();

            $all_languages = \rp_get_all_languages();

            $all_languages_name = array();

            // get all languages name

            foreach($all_languages as $lang_row) {

                $all_languages_name[] = $lang_row->name;

            }

            // make sure all $keys item is in $all_languages_name

            foreach($keys as $key) {

                if( !in_array( $key, $all_languages_name ) ) {

                    echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));

                    exit();

                }

            }

            $default_translation = '';

            // Check if row exists in rs_translations table
            // where language = $key of $keys array and key = $translation_key

            foreach($keys as $index => $key) {

                if( trim($key) === trim($default_language) ) {

                    $default_translation = $values[$index];

                }

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $translation_key);
                $this->rpdb->where('uq_id', $translation_uqid);
                $this->rpdb->where('r_id_fk', $translation_connection_id);
                $this->rpdb->where('language', $key);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_rattr_translations');

                if(!empty($result)) {

                    // update row

                    $this->rpdb->reset_query();
                    $this->rpdb->where('key', $translation_key);
                    $this->rpdb->where('uq_id', $translation_uqid);
                    $this->rpdb->where('r_id_fk', $translation_connection_id);
                    $this->rpdb->where('language', $key);
                    $this->rpdb->limit(1);
                    $this->rpdb->update('rs_rattr_translations', array('value' => $values[$index]));

                } else {

                    // insert row

                    $this->rpdb->reset_query();
                    $this->rpdb->insert('rs_rattr_translations', array('key' => $translation_key, 'language' => $key, 'value' => $values[$index], 'uq_id' => $translation_uqid, 'r_id_fk' => $translation_connection_id));

                }

            }

            $what_info = 'a_name';

            if($translation_key === 'attr_name') {
                $what_info = 'a_name';
            }

            if($translation_key === 'attr_desc') {
                $what_info = 'a_desc';
            }

            if($translation_key === 'attr_badge') {
                $what_info = 'a_badge';
            }

            $data_to_update = array();
            $data_to_update[$what_info] = $default_translation;

            $this->rpdb->reset_query();
            $this->rpdb->where('a_uq_id', $translation_uqid);
            $this->rpdb->where('r_id_fk', $translation_connection_id);
            $this->rpdb->update('rs_repair_attr', $data_to_update);

            // send error message

            echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated', 'default_translation' => htmlspecialchars($default_translation)));
            
            exit();

        } else {

            // send error message

            echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));
            
            exit();

        }

    }

    public function rp_get_rattr_translations() {

        $key = $_POST['key'] ?? '';
        $uqid = $_POST['uqid'] ?? '';
        $connection_id = $_POST['connection_id'] ?? '';

        $translations = array();

        $static_translation = '';
        
        if( !empty( $key ) ) {

            $static_translation = \rp_get_static_translations_of_attr($key, $uqid, $connection_id);

            $all_languages = \rp_get_all_languages();

            foreach($all_languages as $lang_row) {

                $result_text = rp_get_static_translations_of_attr_by_language($key, $uqid, $connection_id, $lang_row->name);

                if(!empty($result_text)) {
                    $lang_row->t_value = htmlspecialchars($result_text);
                } else {
                    $lang_row->t_value = htmlspecialchars($static_translation);
                }

                $lang_row->image = \rp_fix_language_image($lang_row->image);

                $translations[] = $lang_row;

            }

        }

        echo json_encode(Array('status'=> true, 'translations'  => $translations, 'static_translation' => htmlspecialchars($static_translation)));

        exit();

    }
    
    public function rp_update_dr_translations() {

        /*
        IMPORTANT NOTE FOR BASE.REPAIR
        When language is deleted, all translations of that language should
        be deleted from rs_translations table
        */

        $keys = $_POST['keys'] ?? array();

        $values = $_POST['values'] ?? array();

        $translation_key = $_POST['translation_key'] ?? '';

        $translation_uqid = $_POST['translation_uqid'] ?? '';

        if( !empty( $translation_key ) && !empty( $keys ) && !empty( $values ) && count( $keys ) === count( $values ) && !empty( $translation_uqid ) ) {

            $default_language = \rp_get_default_language();

            $all_languages = \rp_get_all_languages();

            $all_languages_name = array();

            // get all languages name

            foreach($all_languages as $lang_row) {

                $all_languages_name[] = $lang_row->name;

            }

            // make sure all $keys item is in $all_languages_name

            foreach($keys as $key) {

                if( !in_array( $key, $all_languages_name ) ) {

                    echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));

                    exit();

                }

            }

            global $wpdb;

            $wpdb->query("START TRANSACTION");

            $default_translation = '';

            // Check if row exists in rs_translations table
            // where language = $key of $keys array and key = $translation_key

            foreach($keys as $index => $key) {

                if( trim($key) === trim($default_language) ) {

                    $default_translation = $values[$index];

                }

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $translation_key);
                $this->rpdb->where('uq_id', $translation_uqid);
                $this->rpdb->where('language', $key);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_dr_translations');

                if(!empty($result)) {

                    // update row

                    $this->rpdb->reset_query();
                    $this->rpdb->where('key', $translation_key);
                    $this->rpdb->where('uq_id', $translation_uqid);
                    $this->rpdb->where('language', $key);
                    $this->rpdb->limit(1);
                    $this->rpdb->update('rs_dr_translations', array('value' => $values[$index]));

                    // update matching rows
                    $old_value = $result[0]->value;

                    // \rp_update_matching_repair_translations($translation_key, $translation_uqid, $key, $old_value, $values[$index]);

                } else {

                    // insert row

                    $this->rpdb->reset_query();
                    $this->rpdb->insert('rs_dr_translations', array('key' => $translation_key, 'language' => $key, 'value' => $values[$index], 'uq_id' => $translation_uqid));

                    // delete existing & create new
                    // \rp_create_matching_repair_translations($translation_key, $translation_uqid, $key, $values[$index]);

                }

            }

            $what_info = 'dr_name';

            if($translation_key === 'repair_name') {
                $what_info = 'dr_name';
            }

            if($translation_key === 'repair_desc') {
                $what_info = 'dr_discription';
            }

            if($translation_key === 'repair_badge') {
                $what_info = 'dr_badge';
            }

            $data_to_update = array();
            $data_to_update[$what_info] = $default_translation;

            $this->rpdb->reset_query();
            $this->rpdb->where('dr_uq_id', $translation_uqid);
            $this->rpdb->limit(1);
            $old_row = $this->rpdb->get('rs_default_repair');

            $this->rpdb->reset_query();
            $this->rpdb->where('dr_uq_id', $translation_uqid);
            $this->rpdb->update('rs_default_repair', $data_to_update);

            if( !empty( $old_row ) ) {

                $old_row = $old_row[0];

                \rp_update_matching_repair_name_in_row($translation_key, $translation_uqid, $old_row, $default_translation);

            }

            $wpdb->query('COMMIT');

            // send error message

            echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated', 'default_translation' => htmlspecialchars($default_translation)));
            
            exit();

        } else {

            // send error message

            echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));
            
            exit();

        }

    }

    public function rp_get_dr_translations() {

        $key = $_POST['key'] ?? '';
        $uqid = $_POST['uqid'] ?? '';

        $translations = array();

        $static_translation = '';
        
        if( !empty( $key ) ) {

            $static_translation = \rp_get_static_translations_of_drepair($key, $uqid);

            $all_languages = \rp_get_all_languages();

            foreach($all_languages as $lang_row) {

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $key);
                $this->rpdb->where('uq_id', $uqid);
                $this->rpdb->where('language', $lang_row->name);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_dr_translations');

                if(!empty($result)) {
                    $lang_row->t_value = htmlspecialchars($result[0]->value);
                } else {
                    $lang_row->t_value = htmlspecialchars($static_translation);
                }

                $lang_row->image = \rp_fix_language_image($lang_row->image);

                $translations[] = $lang_row;

            }

        }

        echo json_encode(Array('status'=> true, 'translations'  => $translations, 'static_translation' => htmlspecialchars($static_translation)));

        exit();

    }

    public function rp_get_drattr_translations() {

        $key = $_POST['key'] ?? '';
        $uqid = $_POST['uqid'] ?? '';

        $translations = array();

        $static_translation = '';
        
        if( !empty( $key ) ) {

            $static_translation = \rp_get_static_translations_of_dattr($key, $uqid);

            $all_languages = \rp_get_all_languages();

            foreach($all_languages as $lang_row) {

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $key);
                $this->rpdb->where('uq_id', $uqid);
                $this->rpdb->where('language', $lang_row->name);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_dr_translations');

                if(!empty($result)) {
                    $lang_row->t_value = htmlspecialchars($result[0]->value);
                } else {
                    $lang_row->t_value = htmlspecialchars($static_translation);
                }

                $lang_row->image = \rp_fix_language_image($lang_row->image);

                $translations[] = $lang_row;

            }

        }

        echo json_encode(Array('status'=> true, 'translations'  => $translations, 'static_translation' => htmlspecialchars($static_translation)));

        exit();

    }

    public function rp_update_drattr_translations() {

        /*
        IMPORTANT NOTE FOR BASE.REPAIR
        When language is deleted, all translations of that language should
        be deleted from rs_translations table
        */

        $keys = $_POST['keys'] ?? array();

        $values = $_POST['values'] ?? array();

        $translation_key = $_POST['translation_key'] ?? '';

        $translation_uqid = $_POST['translation_uqid'] ?? '';

        if( !empty( $translation_key ) && !empty( $keys ) && !empty( $values ) && count( $keys ) === count( $values ) && !empty( $translation_uqid ) ) {

            $default_language = \rp_get_default_language();

            $all_languages = \rp_get_all_languages();

            $all_languages_name = array();

            // get all languages name

            foreach($all_languages as $lang_row) {

                $all_languages_name[] = $lang_row->name;

            }

            // make sure all $keys item is in $all_languages_name

            foreach($keys as $key) {

                if( !in_array( $key, $all_languages_name ) ) {

                    echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));

                    exit();

                }

            }

            global $wpdb;

            $wpdb->query("START TRANSACTION");

            $default_translation = '';

            // Check if row exists in rs_translations table
            // where language = $key of $keys array and key = $translation_key

            foreach($keys as $index => $key) {

                if( trim($key) === trim($default_language) ) {

                    $default_translation = $values[$index];

                }

                $this->rpdb->reset_query();
                $this->rpdb->where('key', $translation_key);
                $this->rpdb->where('uq_id', $translation_uqid);
                $this->rpdb->where('language', $key);
                $this->rpdb->limit(1);
                $result = $this->rpdb->get('rs_dr_translations');

                if(!empty($result)) {

                    // update row

                    $this->rpdb->reset_query();
                    $this->rpdb->where('key', $translation_key);
                    $this->rpdb->where('uq_id', $translation_uqid);
                    $this->rpdb->where('language', $key);
                    $this->rpdb->limit(1);
                    $this->rpdb->update('rs_dr_translations', array('value' => $values[$index]));

                    // update matching rows
                    $old_value = $result[0]->value;

                    // \rp_update_matching_attr_translations($translation_key, $translation_uqid, $key, $old_value, $values[$index]);

                } else {

                    // insert row

                    $this->rpdb->reset_query();
                    $this->rpdb->insert('rs_dr_translations', array('key' => $translation_key, 'language' => $key, 'value' => $values[$index], 'uq_id' => $translation_uqid));

                    // delete existing & create new
                    // \rp_create_matching_attr_translations($translation_key, $translation_uqid, $key, $values[$index]);

                }

            }

            $what_info = 'da_name';

            if($translation_key === 'attr_name') {
                $what_info = 'da_name';
            }

            if($translation_key === 'attr_desc') {
                $what_info = 'da_desc';
            }

            if($translation_key === 'attr_badge') {
                $what_info = 'da_badge';
            }

            $data_to_update = array();
            $data_to_update[$what_info] = $default_translation;

            $this->rpdb->reset_query();
            $this->rpdb->where('da_uq_id', $translation_uqid);
            $this->rpdb->limit(1);
            $old_row = $this->rpdb->get('rs_default_repair_attr');

            $this->rpdb->reset_query();
            $this->rpdb->where('da_uq_id', $translation_uqid);
            $this->rpdb->update('rs_default_repair_attr', $data_to_update);

            if( !empty( $old_row ) ) {

                $old_row = $old_row[0];

                \rp_update_matching_attr_name_in_row($translation_key, $translation_uqid, $old_row, $default_translation);

            }

            $wpdb->query("COMMIT");

            // send error message

            echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated', 'default_translation' => htmlspecialchars($default_translation)));
            
            exit();

        } else {

            // send error message

            echo json_encode(Array('status'=> false, 'message'  => 'Invalid Request'));
            
            exit();

        }

    }

}
