<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if( !class_exists( 'RepairPluginPro\RP_Request_Handler' ) ) {

	class RP_Request_Handler {

		// ---------------------------
		// If custom rules for input key are not 
		// defined, all inputs in these arrays 
		// will automatically get sanitized:
		// $_POST, $_GET, $_REQUEST 
		// ------------------------

		public static $custom_rules = array(

			'location_address' => array(

				'type' => 'allowed',

				'items' => array( 'br' ) // Allow <br> tag for location address

			),

			'noti_message' => array(

				'type' => 'disallowed',

				'items' => array( 'script', 'link' ) // Disallow script & link tags

			)

		);

		public static $forms = array();

		public static $admin_pages = array(

			'repair-plugin',

			'wp_repair_appointments',

			'wp_repair_manage_categories',

			'wp_repair_manage_models',

			'wp_repair_manage_repairs',

			'wp_repair_manage_upsales',

			'wp_repair_download_libraries',

			'wp_repair_default_repairs',

			'wp_repair_settings'

		);

		// ---------------------------------
		// HTML Tags To Remove:
		// Going to escape the final output anyway so no worries even if 
		// any HTML tag isn't present below.
		// --------------------

		public static $html_tags = array(

			'p', 'h1', 'h2',

			'h3', 'h4', 'h5',

			'h6', 'span', 'b',

			'strong', 'script', 'br',

			'hr', 'footer', 'head',

			'body', 'header', 'article',

			'section', 'div', 'a',

			'button', 'input', 'textarea',

			'html', 'link', 'style',

			'table', 'thead', 'tbody',

			'tr', 'td', 'th',

			'video', 'title', 'meta',

			'ul', 'ol', 'li',

			'template', 'svg', 'sup',

			'summary', 'sub', 'strike',

			'source', 'small', 'select',

			'samp', 's', 'ruby',

			'rt', 'rp', 'q',

			'progress', 'pre', 'picture',

			'img', 'param', 'output',

			'option', 'optgroup', 'iframe',

			'object', 'noscript', 'noframes',

			'nav', 'meter', 'menu',

			'mark', 'map', 'main',

			'i', 'legend', 'label',

			'frame', 'frameset', 'form',

			'font', 'figure', 'figcaption',

			'fieldset', 'embed', 'em',

			'dt', 'dl', 'dir',

			'dialog', 'col', 'details',

			'data', 'datalist', 'code',

			'canvas', 'blockquote', 'big',

			'base', 'audio', 'aside',

			'area', 'address'

		);

		public static function has_any_html_tag( $string = '' ) {

			foreach (self::$html_tags as $tag) {

				$matches = self::get_html_tag_matches( $string, $tag );

				if( !empty( $matches ) ) {

					return TRUE;

				}

			}

			return FALSE;

		}

		public static function get_html_tag_matches( $string = '', $tag = '' ) {

			$all_matches = array();

			while( preg_match( "/(\<(".$tag.")(\s)*\>)|(\<(".$tag.")(\s)([^\<|\>])*\>)/i", $string, $matches ) ) {

				$string = str_replace( $matches[0], '', $string );

				$all_matches[] = $matches[0];

			}

			while( preg_match( "/\<(\s)*(\/)(\s)*(".$tag.")(\s)*\>/i", $string, $matches ) ) {

				$string = str_replace( $matches[0], '', $string );

				$all_matches[] = $matches[0];

			}

			return $all_matches;

		}

		public static function remove_html_tags( $string = '', $type = '', $items = array() ) {

			if( $type === 'disallowed' && empty( $items ) ) {

				// Allow Everything!
				return $string;

			}

			if( empty( $string ) || !is_string( $string ) || empty( trim( $string ) ) ) {

				return $string;

			}

			$tags = self::$html_tags;

			$strip_tags = array();

			foreach ($tags as $key => $value) {

				if( !empty( $type ) ) {

					if( $type === 'allowed' ) {

						if( in_array( $value, $items ) ) {

							continue;

						}

					} else {

						if( !in_array( $value, $items ) ) {

							continue;

						}

					}

				}

				$strip_tags[] = $value;

			}

			foreach ($strip_tags as $tag) {

				$matches = self::get_html_tag_matches( $string, $tag );

				foreach ($matches as $match) {
					
					$string = str_replace( $match, '', $string );

				}

			}

			// Strip extra whitespace that may now exists after removing HTML tags
			$string = trim( preg_replace( '/ +/', ' ', $string ) );

			return $string;

		}

		public static function escape_html_entities( $string = '', $type = '', $items = array() ) {

			if( $type === 'disallowed' && empty( $items ) ) {

				// Allow Everything!
				return $string;

			}

			if( empty( $string ) || !is_string( $string ) || empty( trim( $string ) ) ) {

				return $string;

			}

			$tags = self::$html_tags;

			$allowed_tags = array();

			if( !empty( $type ) ) {

				foreach ($tags as $key => $value) {
					
					if( $type === 'allowed' ) {

						if( in_array( $value, $items ) ) {

							$allowed_tags[] = $value;

						}

					} else {

						if( !in_array( $value, $items ) ) {

							$allowed_tags[] = $value;

						}

					}

				}

			}

			if( !empty( $allowed_tags ) ) {

				$tempChange = array();

				foreach ($allowed_tags as $key => $value) {

					$matches = self::get_html_tag_matches( $string, $value );

					foreach ($matches as $match) {

						$md5 = md5( $match );

						$string = str_replace( $match, $md5, $string );

						$tempChange[ $md5 ] = $match;

					}

				}

			}

			$to_escape = array(

				'<' => '&lt;',

				'>' => '&gt;',

				'"' => '&quot;'

			);

			foreach ($to_escape as $key => $value) {

				$string = str_replace( $key, $value, $string );

			}

			if( !empty( $allowed_tags ) ) {

				foreach ($tempChange as $key => $value) {

					$string = str_replace( $key, $value, $string );

				}

			}

			return $string;

		}

		public static function is_ajax_request_for_repair_plugin( $forms = array() ) {

			if( empty( $forms ) ) {

				$forms = self::$forms;

			}

			if( isset( $_REQUEST['action'] ) || (isset( $_POST ) && isset( $_POST['action'] ) ) ) {

				$action = isset( $_REQUEST['action'] ) ? $_REQUEST['action'] : ($_POST['action'] ?? '');

				$action = trim( $action );

				if( !empty( $action ) ) {

					foreach ($forms as $_action) {

						if( trim( $_action ) === $action ) {

							return TRUE;

						}

					}

				}

			}

			return FALSE;

		}

		public static function is_repair_plugin_admin_page( $admin_pages = array() ) {

			if( empty( $admin_pages ) ) {

				$admin_pages = self::$admin_pages;

			}

			$page = isset( $_GET['page'] ) ? $_GET['page'] : '';

			// Check if user is withint /wp-admin/
			$is_wp_admin = is_admin();

			if( !empty( $page ) && $is_wp_admin ) {

				foreach ($admin_pages as $admin_page) {

					if( trim( $page ) === trim( $admin_page ) ) {

						return TRUE;

					}

				}

			}

			return FALSE;

		}

		public static function need_to_do_request_protection( $admin_pages = array(), $forms = array() ) {

			if( self::is_repair_plugin_admin_page( $admin_pages ) || self::is_ajax_request_for_repair_plugin( $forms ) ) {

				return TRUE;

			}

			return FALSE;

		}

		public static function strip_tags_recursive( $input = null, $type = '', $items = array() ) {

			if( $input === null || $input === FALSE || $input === '' ) {

				return $input;

			}

			if( is_array( $input ) ) {

				$new_items = array();

				foreach ($input as $org_key => $o_input) {

					$custom_rule_key = 'handle_parse_' . $org_key;

					// If Custom Rule is set, no need to apply default rules!

					if(method_exists( __CLASS__ , $custom_rule_key ) && is_callable( array( __CLASS__ , $custom_rule_key ) ) ) {

						// Has custom rule...
						$new_items[$org_key] = call_user_func( array( __CLASS__ , $custom_rule_key ) , $o_input );

						continue;

					}

					if( empty( $type ) ) {

						if( !isset( self::$custom_rules[ $org_key ] ) ) {

							$new_items[$org_key] = self::strip_tags_recursive( $o_input );

						} else {

							$new_items[$org_key] = self::strip_tags_recursive( $o_input, self::$custom_rules[ $org_key ]['type'], self::$custom_rules[ $org_key ]['items'] );

						}

					} else {

						$new_items[$org_key] = self::strip_tags_recursive( $o_input, $type, $items );

					}

				}

				return $new_items;

			} else if( is_numeric( $input ) || preg_match( '/^([0-9\.\,])+$/', $input ) ) {

				return $input;

			} else if( is_string( $input ) ) {

				$input = self::remove_html_tags( $input, $type, $items );

				$input = self::escape_html_entities( $input, $type, $items );

				return $input;

			} else {

				return '';

			}

		}

		public static function handle_parse_the_dl_form( $input = '' ) {

			return self::remove_html_tags( $input );

		}

		public static function do_request_protection() {

			if( self::need_to_do_request_protection() ) {

				if( isset( $_POST ) && !empty( $_POST ) ) {

					$_POST = self::strip_tags_recursive( $_POST );

				}

				if( isset( $_GET ) && !empty( $_GET ) ) {

					$_GET = self::strip_tags_recursive( $_GET );

				}

				if( isset( $_REQUEST ) && !empty( $_REQUEST ) ) {

					$_REQUEST = self::strip_tags_recursive( $_REQUEST );

				}

			}

		}

		public static function init() {

			if( isset($_REQUEST['action']) && $_REQUEST['action'] == 'rp_update_translations' ) {

				if( isset($_REQUEST['translation_key']) && (strpos( $_REQUEST['translation_key'], '_email_template' ) !== FALSE || strpos( $_REQUEST['translation_key'], 'ship_device_instructions' ) !== FALSE) ) {

					self::$custom_rules['values'] = array(

						'type' => 'disallowed',

						'items' => array( 'script', 'link' ) // Disallow script & link tags

					);
					
				}

			}

			self::do_request_protection();

		}

	}

	// Run this as early as possible!
	add_action( 'plugins_loaded', array( 'RepairPluginPro\RP_Request_Handler', 'init' ), -999 );

}
