<?php

namespace RepairPluginPro;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class Search_Bar {
	protected $shortcode_page_link = '';

	public function update_shortcode_page_link() {

		$custom_link = rp_get_search_box_redirect_link();

		if( !empty($custom_link) ) {

			$this->shortcode_page_link = $custom_link;
			
		} else {

			$this->shortcode_page_link = rp_get_shortcode_page_link();

		}

	}

	public function get_model_link($name = '') {

		if(empty(trim($this->shortcode_page_link))) {
			return '#';
		}

		$name = str_replace('-', '_', $name);
		$name = str_replace('+', 'plus', $name);
		$name = str_replace(' ', '-', $name);

		$name = '?selected=repairs/'.$name;

		return trim($this->shortcode_page_link) . $name;

	}

	public function get_json() {
		$categories = $this->get_all_categories();

		list($prepared_brands, $brand_ids) = $this->get_all_brands($categories);

		$prepared_models = $this->get_all_models($brand_ids);

		$prepared_brands = rp_sanitize_output($prepared_brands);

		$prepared_models = rp_sanitize_output($prepared_models);

		return json_encode(array(
			'brands' => $prepared_brands,
			'models' => $prepared_models
		));
	}

	public function get_all_categories() {
		global $rpQuery;

		$rs_category = $rpQuery->prefix . "rs_category";

		$db_categories = $rpQuery->get_results("SELECT * FROM `$rs_category` WHERE is_active = '1'");

		$prepared_categories = array();

		foreach ($db_categories as $key => $value) {
			
			$prepared_categories[] = $value->c_id;

		}

		return $prepared_categories;
	}

	public function get_all_brands($categories = array()) {
		global $rpQuery;

		$brand_table = $rpQuery->prefix . "rs_brand";

		if(empty($categories)) {
			return array( array(), array() );
		}

		$categories_ids_str = join(',', $categories);

		$db_brands = $rpQuery->get_results("SELECT * FROM `$brand_table` WHERE is_active = '1' AND c_id_fk IN (".rp_escape_sql($categories_ids_str).")");

		$prepared_brands = array();

		$brand_ids = array();

		foreach ($db_brands as $key => $value) {
			
			$prepared_brands[$value->b_id] = $value->b_name;

			$brand_ids[] = $value->b_id;

		}

		return array($prepared_brands, $brand_ids);
	}

	public function get_all_models($brand_ids = array()) {
		global $rpQuery;

		$this->update_shortcode_page_link();

		if(empty($brand_ids)) {
			return array();
		}

		$brand_ids_str = join(',', $brand_ids);

		$model_table = $rpQuery->prefix . "rs_model";

		$db_models = $rpQuery->get_results("SELECT m_name, m_code, brand_id_fk FROM `$model_table` WHERE is_active = '1' AND brand_id_fk IN (".rp_escape_sql($brand_ids_str).") ORDER BY release_date DESC");

		foreach ($db_models as $key => $value) {
			$value->url = $this->get_model_link($value->m_name);
		}

		return $db_models;
	}
}
