<?php

// Exit if accessed directly

defined( 'ABSPATH' ) || exit;

if(!defined('RP_DP_PLUGIN_FILE')) {

    define( 'RP_DP_PLUGIN_FILE', __FILE__ );

    define( 'RP_DP_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );

    define( 'RP_DP_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );

    define( 'RP_DP_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

    define( 'RP_DP_ASSETS_CACHE_V', WP_REPAIR_ASSETS_CACHE_V );

    define( 'RP_DP_BASE_SERVER_URL', WP_REPAIR_BASE_SERVER_URL );

    define( 'RP_DP_REPEAT_CRON_AFTER_X_SECONDS', ( 15 * 60 ) ); // 15 minutes

}

function rp_round_up_nearest_five($price, $endWith = 95) {
    // if price is negative, return price
    if($price < 0) {
        return $price;
    }
    // if $endWith false, return ceil($price)
    if( $endWith === false || $endWith === 'ceil') {
        return ceil($price);
    }
    $endWith = (int) $endWith;
    // get the integer part of the price
    $integer = floor($price);
    // roundup integer to nearest 5
    $integer = ceil($integer / 5) * 5;
    // if decimal is 0.00, return integer
    $integer = $integer - (1 - ( $endWith / 100 ));
    if($integer < $price) {
        $integer = $integer + 5;
    }
    // format the output
    $output = number_format($integer, 2, '.', '');
    // if ends with .00, return integer
    if($output == floor($output)) {
        $output = floor($output);
    }
    return $output;

}

function rp_dynamic_pricing_require_message() {
    // dynamic pricing addon require Repairplugin to be installed and activated
    $mesesage = 'Dynamic Pricing addon require Repairplugin to be installed and activated';
    echo '<div class="notice notice-error is-dismissible"><p>' . $mesesage . '</p></div>';

    return;
}

function rp_dynamic_pricing_addon_init() {

    if( !defined('WP_REPAIR_PLUGIN_FILE') ) {
        // Add admin notice
        add_action('admin_notices', 'rp_dynamic_pricing_require_message');
        return;
    }

    do_action('rp_dynamic_pricing_initialize');

}

add_action('init', 'rp_dynamic_pricing_addon_init', 1000);


function rp_add_dynamic_pricing_tab_in_sidebar( $sidebar_items ) {
    // Example:
    // 'dynamic_pricing' => array(
    //     'title' => 'Dynamic Pricing',
    //     'icon' => 'fas fa-dollar-sign',
    //     'link' => createTheLink('dynamic_pricing'),
    //     'render_condition' => true,
    //     'position' => 65
    // )

    $sidebar_items['dynamic_pricing'] = array(
        'title' => 'Dynamic Pricing',
        'icon' => 'fas fa-dollar-sign',
        'link' => createTheLink('dynamic_pricing'),
        'render_condition' => true,
        'position' => 65
    );

    return $sidebar_items;

}

add_filter('rp_admin_settings_sidebar_items', 'rp_add_dynamic_pricing_tab_in_sidebar');


function rp_dynamic_pricing_tab_actual_html() {
    
    $fileName = RP_DP_PLUGIN_PATH . 'html/dynamic_pricing_settings_tab.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}

function rp_handle_dynamic_pricing_page_html( $return = FALSE, $section = '' ) {

    if( $section == 'dynamic_pricing' ) {
        // return the html for the dynamic pricing page
        rp_dynamic_pricing_tab_actual_html();

        return TRUE;

    }

    return $return;

}

add_filter('rp_returning_admin_setting_page_html', 'rp_handle_dynamic_pricing_page_html', 10, 2);


function rp_dp_download_suppliers() {

    if( !RP_DynamicPricingAddon::is_active() ) {

        return array();

    }

    global $rpQuery;

	require_once RP_DP_PLUGIN_PATH . 'classes/Base_API.php';

	$base_api = new RepairPluginPro\Base_API($rpQuery);
	
	$response = $base_api->fetch('Endpoints/fetch_suppliers', array(), FALSE);
	
	$suppliers = array();
	
	if( isset( $response ) && is_object( $response ) && property_exists( $response, 'suppliers' ) ) {
	
		$suppliers = $response->suppliers;
	
	}

    return $suppliers;

}

function rp_get_cached_or_download_suppliers() {

    $rp_dp_last_downloaded_suppliers = rp_get_option('rp_dp_last_downloaded_suppliers', FALSE);

    $suppliers = FALSE;
    $last_synced = FALSE;

    if( $rp_dp_last_downloaded_suppliers !== FALSE && is_array($rp_dp_last_downloaded_suppliers) && isset( $rp_dp_last_downloaded_suppliers['last_synced'] ) ) {

        if( $rp_dp_last_downloaded_suppliers['last_synced'] > ( time() - 600 ) ) {

            // it is less than 10 minutes since the last download, use the cached version
            $suppliers = $rp_dp_last_downloaded_suppliers['response'];
            $last_synced = $rp_dp_last_downloaded_suppliers['last_synced'];

        }

    }

    if( $suppliers === FALSE ) {

        // download the suppliers
        $suppliers = rp_dp_download_suppliers();

        if( !empty( $suppliers ) ) {

            $last_synced = time();

            // save the suppliers
            rp_update_option('rp_dp_last_downloaded_suppliers', array(
                'last_synced' => $last_synced,
                'response' => $suppliers
            ));

        } else {

            $suppliers = FALSE;
            $last_synced = FALSE;

        }

    }

    if( $suppliers === FALSE && $rp_dp_last_downloaded_suppliers !== FALSE && is_array($rp_dp_last_downloaded_suppliers) && isset( $rp_dp_last_downloaded_suppliers['last_synced'] ) ) {

        // download failed
        // more than 10 minutes but we have no other option, use the cached version
        $suppliers = $rp_dp_last_downloaded_suppliers['response'];
        $last_synced = $rp_dp_last_downloaded_suppliers['last_synced'];

    }

    return array($suppliers, $last_synced);

}

function rp_get_enabled_suppliers() {

    if( !RP_DynamicPricingAddon::is_active() ) {

        return array();

    }

    static $cached_suppliers = FALSE;

    if( $cached_suppliers !== FALSE ) {
        return $cached_suppliers;
    }

    list( $suppliers, $last_synced ) = rp_get_cached_or_download_suppliers();

    $rp_dp_enabled_suppliers = rp_get_option('rp_dp_enabled_suppliers', array());

    // make sure enabled suppliers are in the list of suppliers
    if( !empty( $rp_dp_enabled_suppliers ) && is_array( $rp_dp_enabled_suppliers ) && !empty( $suppliers ) ) {

        foreach($rp_dp_enabled_suppliers as $key => $supplier_id) {
            $found = false;
            foreach($suppliers as $supplier) {
                if($supplier->id == $supplier_id) {
                    $found = true;
                    break;
                }
            }
            if(!$found) {
                unset($rp_dp_enabled_suppliers[$key]);
            }
        }
        $rp_dp_enabled_suppliers = array_values($rp_dp_enabled_suppliers);

    } else {

        $rp_dp_enabled_suppliers = array();

    }

    $cached_suppliers = $rp_dp_enabled_suppliers;

    return $rp_dp_enabled_suppliers;

}

function rp_is_enabled_dynamic_pricing() {

    if( !RP_DynamicPricingAddon::is_active() ) {

        return false;

    }

    $enabled_suppliers = rp_get_enabled_suppliers();

    $rp_dynamic_pricing_is_enabled = rp_get_option('rp_dynamic_pricing_is_enabled', '1');

    $final_result =  $rp_dynamic_pricing_is_enabled == '1' && !empty( $enabled_suppliers );

    return apply_filters('rp_filter_is_enabled_dynamic_pricing', $final_result);

}

function rp_dp_x_minutes_ago( $time = 0 ) {

    $current_time = time();

    $time_difference = $current_time - $time;

    $minutes = round($time_difference / 60);

    if($minutes < 1) {
        $minutes = 'less than a minute ago';
    } else if( (int)$minutes === 1 ) {
        $minutes = $minutes . ' minute ago';
    } else {
        $minutes = $minutes . ' minutes ago';
    }

    return $minutes;

}


function rp_dp_download_datafeed_from_base( $page = 1 ) {

    global $rpQuery;

	require_once RP_DP_PLUGIN_PATH . 'classes/Base_API.php';

    $enabled_suppliers = rp_get_enabled_suppliers();

	$base_api = new RepairPluginPro\Base_API($rpQuery);
	
	$response = $base_api->fetch('Endpoints/fetch_dp_datafeed', array('page' => $page, 'supplier_ids' => json_encode($enabled_suppliers)), TRUE);
	
	$datafeed = array();
	
	if( isset( $response ) && is_object( $response ) && property_exists( $response, 'datafeed' ) ) {
	
		$datafeed = $response;
	
	}

    return $datafeed;

}

function rp_dp_process_the_datafeed( $datafeed = array(), $offset = 0, $limit = 0) {
    
    global $rpQuery;

    $rs_dp_linking = $rpQuery->prefix . 'rs_dp_linking';

    $enabled_suppliers = rp_get_enabled_suppliers();
    
    // delete * from rs_dp_linking where id > $offset and id < $limit
    $rpQuery->query("DELETE FROM `$rs_dp_linking` WHERE `id` > {$offset} AND `id` < {$limit} AND `supplier_id` IN (". implode(',', $enabled_suppliers) .")");
    
    $ids_batches = array();

    // create batch of 500 ids
    foreach($datafeed as $row) {

        $row = (array)$row;

        $ids_batches[] = $row['id'];

        if( count($ids_batches) >= 500 ) {

            // delete * from rs_dp_linking where id in (ids_batches)
            $rpQuery->query("DELETE FROM `$rs_dp_linking` WHERE `id` IN (". implode(',', $ids_batches) .")");

            $ids_batches = array();

        }

    }

    if( !empty( $ids_batches ) ) {

        // delete * from rs_dp_linking where id in (ids_batches)
        $rpQuery->query("DELETE FROM `$rs_dp_linking` WHERE `id` IN (". implode(',', $ids_batches) .")");

        $ids_batches = array();

    }

    if( !empty( $datafeed ) ) {

        $updated_at = date('Y-m-d H:i:s', time());

        foreach($datafeed as $row) {

            $row = (array)$row;

            $row['updated_at'] = $updated_at;

            $rpQuery->insert($rs_dp_linking, $row);

        }

    }

}

function rp_dp_create_rs_dp_linking_table() {

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $charset_collate = $rpQuery->get_charset_collate();

    $rs_dp_linking = $rpQuery->prefix . 'rs_dp_linking';

    $sql = array();

    $sql[] = "CREATE TABLE IF NOT EXISTS `$rs_dp_linking` (
        `id` int(11) NOT NULL AUTO_INCREMENT,
        `supplier_id` int(11) NOT NULL,
        `supplier_name` varchar(255) NOT NULL,
        `m_uq_id` varchar(255) NOT NULL,
        `repair_uqid` varchar(255) NOT NULL,
        `sku` varchar(255) NOT NULL,
        `status` varchar(255) NOT NULL,
        `on_stock` varchar(255) NOT NULL,
        `price` varchar(255) NOT NULL,
        `fetched_status` varchar(255) NOT NULL,
        `updated_at` varchar(255) NOT NULL,
        PRIMARY KEY (`id`)
    ) $charset_collate";

    dbDelta( $sql );

}


function rp_dp_do_processing() {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return;
    }

    global $rpQuery;

    require_once RP_DP_PLUGIN_PATH . 'classes/longProcess/load.php';

    $saveHandler = new RepairPluginPro\LongProcessSaveHandler( $rpQuery );

    $longProcess = new RepairPluginPro\LongProcess( $saveHandler, RP_DP_REPEAT_CRON_AFTER_X_SECONDS );

    $longProcess->register_process( 'step', function( $page, $longProcess ) { 

        $response = rp_dp_download_datafeed_from_base( $page );

        if( !empty( $response ) ) {

            if( property_exists( $response, 'last_id' ) && (int) $response->last_id > 0 ) {

                $offset = (int) $response->id_offset; // 100
                $limit = (int) $response->id_limit; // 201
                $last_id = (int) $response->last_id; // 188
                $datafeed = $response->datafeed;

                // process the datafeed...
                rp_dp_process_the_datafeed( $datafeed, $offset, $limit );

                $limit = $limit - 1; // 200

                $end_items = $limit;

                if( $end_items > $last_id ) {
                    $end_items = $last_id;
                }

                $longProcess->output = "Downloading and processing: " . ($offset + 1) . " to " . ($end_items) . " of " . $last_id . " items";

                if( $last_id > $limit ) {
                    // we have more data to process
                    return $page + 1;
                } else {
                    // we have finished processing the datafeed
                    if( rp_get_option('rp_dp_finished_cron_processing', FALSE) === FALSE ) {

                        rp_add_option('rp_dp_finished_cron_processing', time());

                    } else {

                        rp_update_option('rp_dp_finished_cron_processing', time());

                    }
                    return TRUE;
                }

            } else {

                $longProcess->output = "Looks like RepairPlugin Server doesn't have any data related to enabled suppliers.";

                return TRUE;

            }

        } else {

            $longProcess->log_errors('Invalid response received from RepairPlugin Server.');

            return TRUE;

        }

    });

    if( !$longProcess->has_lock_file() ) {

        $longProcess->create_lock_file();

        try {

            $longProcess->process();

            if( defined('RP_DP_CRON_DO_NOT_ECHO_OUTPUT') && RP_DP_CRON_DO_NOT_ECHO_OUTPUT === TRUE ) {

                // Do nothing

            } else {

                echo $longProcess->get_output();

            }

            $longProcess->delete_lock_file();

        } catch ( \Exception|\Error|\Throwable|\ErrorException $e ) {

            $longProcess->delete_lock_file();

            $longProcess->log_errors( $e->getMessage() . ' on line ' . $e->getLine() . ' in file ' . $e->getFile() );

            throw new \Exception( $e->getMessage() );

        }

    } else {

        echo "Process already running";

    }

}

function rp_dp_is_waiting_period() {

    // repeat after X seconds
    $repeatAfterXSeconds = RP_DP_REPEAT_CRON_AFTER_X_SECONDS;

    // get current time
    $current_time = time();

    // get last time
    $last_time = rp_get_option('rp_dp_long_process_last_time', FALSE);

    if( $last_time === FALSE ) {

        // not a waiting period because no last time was found
        return FALSE;

    }

    // check if X seconds have passed
    if( ($current_time - $last_time) < $repeatAfterXSeconds ) {

        // waiting period
        return TRUE;

    }

    // not a waiting period
    return FALSE;

}

function rp_dp_move_errors_to_options_table() {

    $option_name = 'rp_dp_errors';

    $fileName = 'errors.log';

    $filePath = RP_DP_PLUGIN_PATH . $fileName;

    // if file exists, get the file and unserialize it
    if( file_exists( $filePath ) ) {

        $file = file_get_contents( $filePath );

        if( !empty( $file ) ) {

            $errors = unserialize( $file );

            if( !empty( $errors ) ) {

                $previous_errors = json_decode( rp_get_option( $option_name, json_encode( array() ) ), TRUE );

                if( empty( $previous_errors ) ) {

                    $previous_errors = array();

                }

                // add new errors to previous errors at the end
                $errors = array_merge( $previous_errors, $errors );

                // limit to 100 errors
                $errors = array_slice( $errors, -100 );

                // save to options table
                rp_update_option( $option_name, json_encode( $errors ) );

            }

        }

    }

    if( file_exists( $filePath ) ) {

        unlink( $filePath );

    }

}

add_action('rp_dynamic_pricing_initialize', 'rp_dp_move_errors_to_options_table');

function rp_dp_get_custom_configured_suppliers( $m_uq_id = '' ) {

    global $rpQuery;

    $rs_model = $rpQuery->prefix . 'rs_model';

    // get brand_id_fk of the model
    $brand_id_fk = $rpQuery->get_var("SELECT `brand_id_fk` FROM `$rs_model` WHERE `m_uq_id` = '{$m_uq_id}'");

    if( empty( $brand_id_fk ) ) {

        return array();

    }

    $enabled_suppliers = rp_get_enabled_suppliers();

    if( empty( $enabled_suppliers ) ) {

        return array();

    }

    $linked_suppliers = array();

    foreach( $enabled_suppliers as $theSupplierId ) {

        $linked_brands = rp_get_option('rp_dp_supplier_linking_' . $theSupplierId, FALSE);

        if( $linked_brands === FALSE ) {

            $linked_brands = rp_get_all_brand_ids_for_dp_supplier_linking();

        }

        if( !empty( $linked_brands ) && in_array( $brand_id_fk, $linked_brands ) ) {

            $linked_suppliers[] = $theSupplierId;

        }

    }

    return $linked_suppliers;

}

function rp_dp_get_skus( $m_uq_id = '', $repair_uqid = '' ) {

    global $rpQuery;

    $rs_dp_linking = $rpQuery->prefix . 'rs_dp_linking';

    $enabled_suppliers = rp_dp_get_custom_configured_suppliers( $m_uq_id );

    if( empty( $enabled_suppliers ) ) {

        return array();

    }

    $m_uq_id = rp_escape_sql( $m_uq_id );

    $repair_uqid = rp_escape_sql( $repair_uqid );

    $sql = "SELECT `price`, `sku`, `supplier_id`, `supplier_name` FROM `$rs_dp_linking` WHERE `m_uq_id` = '{$m_uq_id}' AND `repair_uqid` = '{$repair_uqid}' AND `status` = 'published' AND `fetched_status` = 'OK' AND `on_stock` = 'Y' AND `supplier_id` IN (" . implode( ',', $enabled_suppliers ) . ")";

    $result = $rpQuery->get_results( $sql );

    if( !empty( $result ) ) {

        return $result;

    }

    return array();

}

function rp_dp_cron_job_handler() {

    if( rp_get_option('rp_which_way_for_processing_datafeed', 'wordpress') !== 'cron' ) {

        echo 'CRON Job option is disabled.';

        exit;

    }

    if( !defined('RP_DP_DOING_CRON_JOB') || !defined('WP_REPAIR_PLUGIN_FILE') ) {

        return;

    }

    if( function_exists('rp_add_combined_index') ) {

        rp_dp_create_rs_dp_linking_table();

        rp_add_combined_index('rs_dp_linking', 'linking_qs', array('m_uq_id(48)', 'repair_uqid(48)', 'status(32)', 'fetched_status(32)', 'on_stock(32)', 'supplier_id'), TRUE);

    }

    rp_dp_do_processing();

    exit;

}

function rp_dp_cron_job_using_wordpress() {

    if( !rp_is_enabled_dynamic_pricing() ) {

        echo 'Dynamic Pricing is disabled.';

        exit;

    }

    if( rp_dp_is_waiting_period() ) {

        echo 'Waiting period.';

        exit;

    }

    if( !isset( $_POST['rp_dp_run_process'] ) ) {

        echo 'Invalid request.';

        exit;

    }

    if( rp_get_option('rp_which_way_for_processing_datafeed', 'wordpress') !== 'wordpress' ) {

        echo 'Syncronization using Wordpress option is disabled.';
        
        exit;

    }

    RepairPluginPro\rp_cron_job_verifiy(array(
        'rp_token_error' => 'No token detected?'
    ));

    if( function_exists('rp_add_combined_index') ) {

        rp_dp_create_rs_dp_linking_table();

        rp_add_combined_index('rs_dp_linking', 'linking_qs', array('m_uq_id(48)', 'repair_uqid(48)', 'status(32)', 'fetched_status(32)', 'on_stock(32)', 'supplier_id'), TRUE);

    }

    define('RP_DP_CRON_DO_NOT_ECHO_OUTPUT', TRUE);

    rp_dp_do_processing();

    if( rp_dp_is_waiting_period() ) {

        echo 'Waiting period.';

    } else {

        RepairPluginPro\rp_cron_job_response(array('status' => 'success'));

    }

    exit;

}

function _run_rp_dp_cron_job_handler_through_cron() {

    if( isset( $_REQUEST['action'] ) && ( $_REQUEST['action'] ?? '' ) == 'rp_dp_cron_job_handler' ) {

        if( defined('RP_DP_DOING_CRON_JOB') && RP_DP_DOING_CRON_JOB === TRUE ) {
         
            rp_dp_cron_job_handler();

        }

    }

}

add_action('init', '_run_rp_dp_cron_job_handler_through_cron', 9999);

rp_ajax_for_public('rp_dp_cron_job_using_wordpress', 'rp_dp_cron_job_using_wordpress');


function rp_handle_dynamic_pricing_processing() {

    if( !rp_is_enabled_dynamic_pricing() || rp_get_option('rp_which_way_for_processing_datafeed', 'wordpress') !== 'wordpress' || rp_dp_is_waiting_period() ) {

        return;

    }

    ?>
    <script type="text/javascript">
        window.repeatableDynamicPricingProcessingAR = false;
        function repeatableDynamicPricingProcessing(rp_token = false) {
            let theData;
            var theTime = window._getCurrentTime();
            if(rp_token === true) {
                theData = {
                    action: 'rp_dp_cron_job_using_wordpress',
                    rp_run_token: true,
                    rp_dp_run_process: 'true'
                }
            } else {
                theData = {
                    action: 'rp_dp_cron_job_using_wordpress',
                    rp_dp_run_process: 'true'
                }
            }
            window.repeatableDynamicPricingProcessingAR = true;
            jQuery.ajax({
                type: "POST",
                url: '<?php echo get_site_url(); ?>/wp-admin/admin-ajax.php?time='+theTime,
                data: theData,
                success: function(data) {
                    try {
                        data = JSON.parse(data);
                    } catch (e) {

                    }
                    if(data && data.rp_token_error) {
                        if(data.rp_try_after && data.rp_try_after !== false && data.rp_try_after > 0) {
                            window.setTimeout(function(){
                                repeatableDynamicPricingProcessing(true);
                            }, window._getNextSafeSchedule(data.rp_try_after));
                        }
                    } else if(data && data.rp_try_after && data.rp_try_after !== false && data.rp_try_after > 0) {
                        window.setTimeout(function(){
                            repeatableDynamicPricingProcessing(true);
                        }, window._getNextSafeSchedule(data.rp_try_after));
                    } else {
                      window.repeatableDynamicPricingProcessingAR = false;
                    }
                },
                error: function() {
                    window.repeatableDynamicPricingProcessingAR = false;
                }
            });
        }
        window._repeatableDynamicPricingProcessing = function(rp_token = false) {
            repeatableDynamicPricingProcessing(rp_token);
        }
        jQuery(window).on("load", function(){
            window.setTimeout(function(){
                repeatableDynamicPricingProcessing();
            }, window._getNextSafeSchedule(1));
        });
    </script>
    <?php
}
add_action('admin_footer','rp_handle_dynamic_pricing_processing', 11);
add_action('wp_footer','rp_handle_dynamic_pricing_processing', 11);



function rp_dp_human_readable_difference( $date1 = '', $date2 = '' ) {

    $diff = new \DateTime( $date1 );
    $now = new \DateTime( $date2 );
    $interval = $now->diff( $diff );

    // year, month, day, hour, minute, second
    $y = $interval->y;
    $m = $interval->m;
    $d = $interval->d;
    $h = $interval->h;
    $i = $interval->i;
    $s = $interval->s;

    $response = '';
    
    if( $y > 0 ) {
        $response .= $y . ' year';
        if( $y > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $m > 0 ) {
        $response .= $m . ' month';
        if( $m > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $d > 0 ) {
        $response .= $d . ' day';
        if( $d > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $h > 0 ) {
        $response .= $h . ' hour';
        if( $h > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $i > 0 ) {
        $response .= $i . ' minute';
        if( $i > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $s > 0 ) {
        $response .= $s . ' second';
        if( $s > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    return $response ?? 'Just now';

}

function rp_filter_default_repair_price_th( $th = '' ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $th;
    }

    return str_replace('Price', 'Fallback', $th) . '<th style="width: 75px"><p>Margin</p></th>';

}

add_filter('rp_default_repair_price_th', 'rp_filter_default_repair_price_th', 10, 1);

function rp_filter_individual_repair_price_th( $th = '' ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $th;
    }

    return str_replace('Price', 'Fallback', $th) . '<th style="width: 75px"><p>Margin</p></th>';

}

add_filter('rp_individual_repair_price_th', 'rp_filter_individual_repair_price_th', 10, 1);

function rp_get_lowest_default_repair_attribute_margin($dr_id = 0) {

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $result = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE `dr_id_fk` = '".rp_escape_sql($dr_id)."'");

        $lowest_price = FALSE;
        $lowest_margin = FALSE;

      foreach ($result as $row) {
        
            $thePrice = (float) str_replace( ',', '.', $row->da_price );
            $theMargin = (float) str_replace( ',', '.', ($row->da_margin ?? 0) );
            if( $lowest_price === FALSE ) {
                $lowest_price = $thePrice;
                $lowest_margin = $theMargin;
            } else if( $thePrice < $lowest_price ) {
                $lowest_price = $thePrice;
                $lowest_margin = $theMargin;
            } else {

            }

      }

      return $lowest_margin;

}

function rp_get_lowest_repair_attribute_margin($r_id = 0) {

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $result = $rpQuery->get_results("SELECT * FROM `$rs_repair_attr` WHERE `r_id_fk` = '".rp_escape_sql($r_id)."'");

      $lowest_price = FALSE;
      $lowest_margin = 0;

      foreach ($result as $row) {
            
            $thePrice = (float) str_replace( ',', '.', $row->a_price );
            $theMargin = (float) str_replace( ',', '.', ($row->a_margin ?? 0) );
            if( $lowest_price === FALSE ) {
                $lowest_price = $thePrice;
                $lowest_margin = $theMargin;
            } else if( $thePrice < $lowest_price ) {
                $lowest_price = $thePrice;
                $lowest_margin = $theMargin;
            } else {

            }

      }

      return $lowest_margin;

}


function rp_filter_default_repair_price_td( $td = '', $passData = array() ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $td;
    }

    // extract the data
    extract( $passData );

    $r->dr_margin = $r->dr_margin ?? 0;

    if(!empty($r_attr)) {

        $r->dr_margin = rp_get_lowest_default_repair_attribute_margin($r->dr_id);

    }

    if($decimal == 'comma'){
        $amount = str_replace('.',',',$r->dr_margin);
    }else{
        $amount = $r->dr_margin;		
    }

    $readonly = '';

    if($r->is_free === '1' || !empty($r_attr)) {
        $readonly = 'readonly="readonly"';
    }

    $to_be_filtered_td = '<td style="width: 75px"><input '.$readonly.' class="repair-price" id="repair-margin'.$m_id.'" type="text"  placeholder="99,99" name="rmargin[]" value="'.$amount.'" required></td>';

    return $td . $to_be_filtered_td;

}

add_filter('rp_default_repair_price_td', 'rp_filter_default_repair_price_td', 10, 2);

function rp_filter_individual_repair_price_td( $td = '', $passData = array() ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $td;
    }

    // extract the data
    extract( $passData );

    $r->r_margin = $r->r_margin ?? 0;

    if(!empty($r_attr)) {

        $r->r_margin = rp_get_lowest_repair_attribute_margin($r->r_id);

    }

    if($decimal == 'comma'){
        $amount = str_replace('.',',',$r->r_margin);
    }else{
        $amount = $r->r_margin;		
    }

    $readonly = '';

    if($r->is_free === '1' || !empty($r_attr)) {
        $readonly = 'readonly="readonly"';
    }

    $to_be_filtered_td = '<td style="width: 75px"><input '.$readonly.' class="repair-price" id="repair-margin'.$m_id.'" type="text"  placeholder="99,99" name="rmargin[]" value="'.$amount.'" required></td>';

    return $td . $to_be_filtered_td;

}

add_filter('rp_individual_repair_price_td', 'rp_filter_individual_repair_price_td', 10, 2);

function rp_filter_default_repair_attr_price_td( $td = '', $passData = array() ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $td;
    }

    // extract the data
    extract( $passData );

    $ra->da_margin = $ra->da_margin ?? 0;

    if($decimal == 'comma'){
        $amount = str_replace('.',',',$ra->da_margin);
    }else{
        $amount = $ra->da_margin;			
    }

    $to_be_filtered_td = '<td style="width: 75px;"><input class="repair-price" type="text" id="ramargin'.$m_id.'" placeholder="99,99" name="ramargin[]" value="'.$amount.'" required></td>';

    return $td . $to_be_filtered_td;

}

add_filter('rp_default_repair_attr_price_td', 'rp_filter_default_repair_attr_price_td', 10, 2);

function rp_filter_individual_repair_attr_price_td( $td = '', $passData = array() ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $td;
    }

    // extract the data
    extract( $passData );

    $ra->a_margin = $ra->a_margin ?? 0;

    if($decimal == 'comma'){
        $amount = str_replace('.',',',$ra->a_margin);
    }else{
        $amount = $ra->a_margin;			
    }

    $to_be_filtered_td = '<td style="width: 75px;"><input class="repair-price" type="text" id="ramargin'.$m_id.'" placeholder="99,99" name="ramargin[]" value="'.$amount.'" required></td>';

    return $td . $to_be_filtered_td;

}

add_filter('rp_individual_repair_attr_price_td', 'rp_filter_individual_repair_attr_price_td', 10, 2);

function rp_dp_create_margin_columns() {

    rp_dp_create_rs_dp_linking_table();

    rp_add_column_to_table_if_not_exists('rs_default_repair', 'dr_margin', 'float NULL DEFAULT 0');

    rp_add_column_to_table_if_not_exists('rs_default_repair_attr', 'da_margin', 'float NULL DEFAULT 0');

    rp_add_column_to_table_if_not_exists('rs_repair', 'r_margin', 'float NULL DEFAULT 0');

    rp_add_column_to_table_if_not_exists('rs_repair_attr', 'a_margin', 'float NULL DEFAULT 0');

    rp_add_column_to_table_if_not_exists('rs_order_repairs', 'pricing_method', 'varchar(32) NULL DEFAULT NULL');

    rp_add_column_to_table_if_not_exists('rs_order_repairs', 'pricing_info', 'text NULL DEFAULT NULL');

    rp_add_column_to_table_if_not_exists('rs_order_repair_attr', 'pricing_method', 'varchar(32) NULL DEFAULT NULL');

    rp_add_column_to_table_if_not_exists('rs_order_repair_attr', 'pricing_info', 'text NULL DEFAULT NULL');

}

add_action('rp_dynamic_pricing_initialize', 'rp_dp_create_margin_columns');

function rp_before_returning_default_repair_tr( $tr = array(), $c_repair = null ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $tr;
    }

    $tr['repair_margin'] = $c_repair->dr_margin ?? 0;

    if( !empty( $tr['attrs'] ) ) {

        $tr['repair_margin'] = rp_get_lowest_default_repair_attribute_margin( $c_repair->dr_id );

    }

    return $tr;

}

add_filter('rp_before_returning_default_repair_tr', 'rp_before_returning_default_repair_tr', 10, 2);

function rp_before_returning_default_repair_attr( $attr = array(), $c_attr = null ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $attr;
    }

    $attr['a_margin'] = $c_attr->da_margin ?? 0;

    return $attr;

}

add_filter('rp_before_returning_default_repair_attr', 'rp_before_returning_default_repair_attr', 10, 2);

function rp_before_returning_individual_repair_tr( $tr = array(), $c_repair = null ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $tr;
    }

    $tr['repair_margin'] = $c_repair->r_margin ?? 0;

    if( !empty( $tr['attrs'] ) ) {

        $tr['repair_margin'] = rp_get_lowest_repair_attribute_margin( $c_repair->r_id );

    }

    return $tr;

}

add_filter('rp_before_returning_individual_repair_tr', 'rp_before_returning_individual_repair_tr', 10, 2);

function rp_before_returning_individual_repair_attr( $attr = array(), $c_attr = null ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $attr;
    }

    $attr['a_margin'] = $c_attr->a_margin ?? 0;

    return $attr;

}

add_filter('rp_before_returning_individual_repair_attr', 'rp_before_returning_individual_repair_attr', 10, 2);

function rp_get_linked_models_count() {

    global $rpQuery;

    $rs_model = $rpQuery->prefix . 'rs_model';

    $rs_repair = $rpQuery->prefix . 'rs_repair';

    $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

    $rs_dp_linking = $rpQuery->prefix . 'rs_dp_linking';

    // select * from rs_dp_linking GROUP BY m_uq_id

    $enabled_suppliers = rp_get_enabled_suppliers();

    if( empty( $enabled_suppliers ) ) {

        return 0;

    }

    $m_uq_ids = $rpQuery->get_results("SELECT `m_uq_id` FROM `$rs_dp_linking` 
    WHERE `supplier_id` IN (".implode(',', $enabled_suppliers).")
    AND `status` = 'published' AND `fetched_status` = 'OK' 
    GROUP BY `m_uq_id`");

    if( empty( $m_uq_ids ) ) {

        return 0;

    }

    $found_linking = 0;

    foreach( $m_uq_ids as $row ) {

        // select * from rs_model where m_uq_id = $row->m_uq_id

        $model = $rpQuery->get_results("SELECT m_id FROM `$rs_model` WHERE `m_uq_id` = '$row->m_uq_id'");

        if( empty( $model ) ) {
            continue;
        }

        $model = $model[0];

        // select * from rs_repair where m_id = $model->m_id

        $repairs = $rpQuery->get_results("SELECT r_id,r_uq_id FROM `$rs_repair` WHERE `m_id_fk` = '$model->m_id'");

        if( empty( $repairs ) ) {
            continue;
        }

        $repair_uq_ids = array();

        foreach( $repairs as $repairRow ) {

            $repair_uq_ids[] = $repairRow->r_uq_id;

            // now find the repair attributes

            $attrs = $rpQuery->get_results("SELECT a_uq_id FROM `$rs_repair_attr` WHERE `r_id_fk` = '$repairRow->r_id'");

            if( empty( $attrs ) ) {
                continue;
            }
            
            foreach( $attrs as $attrRow ) {

                $repair_uq_ids[] = $attrRow->a_uq_id;

            }

        }

        // now check if m_uq_id and repair_uqid matches in rs_dp_linking

        $linked_models = $rpQuery->get_results("SELECT id FROM `$rs_dp_linking` WHERE `m_uq_id` = '$row->m_uq_id' AND `repair_uqid` IN ('" . implode("','", $repair_uq_ids) . "') AND `status` = 'published' AND `fetched_status` = 'OK' LIMIT 1");

        if( empty( $linked_models ) ) {
            continue;
        }

        $found_linking++;

    }

    return $found_linking;

}

function rp_get_total_models_linked() {

    global $rpQuery;

    $rs_model = $rpQuery->prefix . 'rs_model';

    // select COUNT(*) FROM rs_model

    $result = $rpQuery->get_results("SELECT COUNT(*) AS total FROM `$rs_model`");

    if( empty( $result ) ) {
        return array(0,0);
    }

    $total_models = $result[0]->total;

    $linked_models_count = rp_get_linked_models_count();

    return array($total_models,$linked_models_count);

}

function _rp_manage_changed_margin_fields( $final_result = TRUE, $attr_rows = array(), $dr_attr_rows = array() ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $final_result;
    }

    if( $final_result === FALSE ) {
        return FALSE;
    }

    $uq_ids = array();
    $margin = array();

    foreach( $dr_attr_rows as $dr_attr_row ) {

          $uq_ids[] = $dr_attr_row->da_uq_id;
          $margin[] = $dr_attr_row->da_margin;

    }

    $results = array();

    foreach( $attr_rows as $attr_row ) {

          if( !in_array( $attr_row->a_uq_id, $uq_ids ) ) {

                return FALSE;

          } else {

                $results[] = $margin[array_search($attr_row->a_uq_id, $uq_ids)] === $attr_row->a_margin;

          }

    }

    return array_search(FALSE, $results) === FALSE;

}

add_filter('rp_attributes_exactly_matches_default_with_price', '_rp_manage_changed_margin_fields', 10, 3);

function _rp_manage_changed_margin_fields_2( $final_result , $row = null, $dr_row = null) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $final_result;
    }

    if( $final_result === FALSE ) {
        return FALSE;
    }

    return ($row->r_margin === $dr_row->dr_margin);

}

add_filter('rp_is_fully_unchanged_default_repair_individually', '_rp_manage_changed_margin_fields_2', 10, 3);

function _rp_also_reset_margin( $result = FALSE ) {

    if( !rp_is_enabled_dynamic_pricing() ) {
        return $result;
    }

    return TRUE;

}

add_filter('rp_reset_default_repair_individually_margin', '_rp_also_reset_margin', 10, 1);

function _rp_add_margin_field_while_adding_repair( $dr_newdata = array(), $dr_repair = null ) {

    $dr_newdata['r_margin'] = $dr_repair->dr_margin;

    return $dr_newdata;

}

add_filter('rp_processing_repair_for_manually_added_model', '_rp_add_margin_field_while_adding_repair', 10, 2);

function _rp_also_process_attr_margin( $result = FALSE ) {

    return TRUE;

}

add_filter('rp_process_attr_margin_for_manually_added_model', '_rp_also_process_attr_margin', 10, 1);

function rp_reset_margin_field_while_default_reset( $result = FALSE ) {

    return TRUE;

}

add_filter('rp_reset_margin_field_while_default_reset', 'rp_reset_margin_field_while_default_reset', 10, 1);

function rp_find_average_price_sku_row( $skus = array() ) {

    if( count( $skus ) == 1 ) {

        $skus[0]->processed_price = $skus[0]->price;

        return $skus[0];
    }

    $total_price = 0;

    foreach( $skus as $sku ) {

        $total_price += $sku->price;

    }

    $average_price = round( $total_price / count( $skus ), 2 );

    // now loop through skus again and find the one which is closest to average price

    $closest_sku = FALSE;

    foreach( $skus as $sku ) {

        if( $sku->price <= $average_price ) {

            if( $closest_sku === FALSE ) {

                $closest_sku = $sku;

            } else {

                if( $sku->price > $closest_sku->price ) {

                    // if this sku is closer to average price than previous one
                    // then overwrite previous one

                    $closest_sku = $sku;

                }

            }

        }

    }

    // overwrite closest sku price with average price

    $closest_sku->processed_price = $average_price;

    return $closest_sku;

}

function rp_find_highest_price_sku_row( $skus = array() ) {

    if( count( $skus ) == 1 ) {

        $skus[0]->processed_price = $skus[0]->price;

        return $skus[0];
    }

    $highest_price = FALSE;

    foreach( $skus as $sku ) {

        if( $highest_price === FALSE ) {

            $highest_price = $sku;

        } else {

            if( $sku->price > $highest_price->price ) {

                $highest_price = $sku;

            }

        }

    }

    $highest_price->processed_price = $highest_price->price;

    return $highest_price;

}

function rp_find_lowest_price_sku_row( $skus = array() ) {

    if( count( $skus ) == 1 ) {

        $skus[0]->processed_price = $skus[0]->price;

        return $skus[0];

    }

    $lowest_price = FALSE;

    foreach( $skus as $sku ) {

        if( $lowest_price === FALSE ) {

            $lowest_price = $sku;

        } else {

            if( $sku->price < $lowest_price->price ) {

                $lowest_price = $sku;

            }

        }

    }

    $lowest_price->processed_price = $lowest_price->price;

    return $lowest_price;

}

function rp_dp_get_tax_rate() {

    static $cached = FALSE;

    if( $cached !== FALSE ) {

        return $cached;

    }

    global $rpQuery;

    $rs_localization = $rpQuery->prefix . "rs_localization"; 

    $tax_row = $rpQuery->get_row("SELECT * FROM $rs_localization LIMIT 1;");

    if( !empty( $tax_row ) ) {

        $cached = $tax_row->tax;

        return $tax_row->tax;

    }

    $cached = 0;

    return 0;

}

function rp_repair_has_any_attributes( $r_id = 0 ) {

    global $rpQuery;

    $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

    $sql = "SELECT * FROM `$rs_repair_attr` WHERE r_id_fk = '".rp_escape_sql($r_id)."' LIMIT 1;";

    $repairAttr = $rpQuery->get_results( $sql );

    if( !empty( $repairAttr ) ) {

        return true;

    }

    return false;

}

function rp_get_active_repair_attributes( $r_id = 0 ) {

    global $rpQuery;

    $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

    $sql = "SELECT * FROM `$rs_repair_attr` WHERE r_id_fk = '".rp_escape_sql($r_id)."' AND is_active = '1';";

    $repairAttr = $rpQuery->get_results( $sql );

    if( !empty( $repairAttr ) ) {

        return $repairAttr;

    }

    return array();

}

function rp_get_only_visible_attributes( $repairRow = null ) {
    
    global $rpQuery;

    $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

    $sql = "SELECT * FROM `$rs_repair_attr` WHERE r_id_fk = '".rp_escape_sql($repairRow->r_id)."' AND is_active = '1';";

    $repairAttrs = $rpQuery->get_results( $sql );

    if( !empty( $repairAttrs ) ) {

        $m_uq_id = get_m_uq_id_by_m_id( $repairRow->m_id_fk );

        $repairAttrs = rp_filter_all_repair_attr_before_return( $repairAttrs, $m_uq_id );

    } else {

        $repairAttrs = array();

    }

    return $repairAttrs;

}

function rp_find_single_attribtute_of_repair( $m_uq_id = '', $repairRow = null ) {

    $visible_attributes = rp_get_only_visible_attributes( $repairRow );

    if( is_array($visible_attributes) && count( $visible_attributes ) === 1 ) {

        return $visible_attributes[0];

    }

    return NULL;

}

function rp_dp_get_dynamic_price( $m_uq_id = '', $repairRow = null ) {

    $must_use_por = FALSE;
    
    if( property_exists( $repairRow, 'r_price' ) ) {

        $default = $repairRow->r_price;
        $margin = $repairRow->r_margin;
        $uqid = $repairRow->r_uq_id;

        if( $repairRow->is_free ) {
            return array('free', 0, array());
        }

        // if( rp_repair_has_any_attributes( $repairRow->r_id ) ) {

        //     if( !rp_is_enabled_dynamic_pricing() ) {
        //         return array('price_on_request', 0, array());
        //     }

        //     $must_use_por = TRUE;

        // }

    } else if( property_exists( $repairRow, 'a_price' ) ) {

        $default = $repairRow->a_price;
        $margin = $repairRow->a_margin;
        $uqid = $repairRow->a_uq_id;

        $relatedRepairRow = rp_find_repair_row_by_id( $repairRow->r_id_fk );

        if( !empty( $relatedRepairRow ) && $relatedRepairRow->is_free ) {
            return array('free', 0, array());
        }
        
    } else {

        // unexpected
        return array('price_on_request', 0, array());

    }

    if( !rp_is_enabled_dynamic_pricing() ) {
        return array('fallback', $default, array());
    }

    // $rp_dp_default_margin_setting = (float) rp_get_option('rp_dp_default_margin_setting', 0);

    // temporary commented to remove this functionality
    $rp_dp_default_margin_setting = 0;

    if( $margin == 0 && $rp_dp_default_margin_setting > 0 ) {

        $margin = $rp_dp_default_margin_setting;

    }

    if( $margin == 0 || empty( $skus = rp_dp_get_skus( $m_uq_id, $uqid ) ) ) {

        $rp_dp_if_part_is_unavailable = rp_get_option('rp_dp_if_part_is_unavailable', 'use_fallback');

        if( $rp_dp_if_part_is_unavailable === 'use_price_on_request' || $must_use_por === TRUE ) {

            return array('price_on_request', 0, array());

        } else {

            return array('fallback', $default, array());

        }

    }

    $rp_dp_which_price_to_consider = rp_get_option('rp_dp_which_price_to_consider', 'cheapest');

    if( $rp_dp_which_price_to_consider === 'most_expensive' ) {

        $sku_row = rp_find_highest_price_sku_row( $skus );

    } else if( $rp_dp_which_price_to_consider === 'average' ) {

        $sku_row = rp_find_average_price_sku_row( $skus );

    } else {

        $sku_row = rp_find_lowest_price_sku_row( $skus );

    }

    $taxRate = (float) (rp_dp_get_tax_rate() ?? 0);
    
    $stockPrice = $sku_row->processed_price;
    $stockPrice = round($stockPrice, 2);
    if( $taxRate != 0 ) {
        $taxPercent = $taxRate / 100;
    } else {
        $taxPercent = 0;
    }
    $finalPriceBeforeRoundUp = $stockPrice + $margin; // Add margin to the price
    $taxPrice = round( ($finalPriceBeforeRoundUp * $taxPercent), 2 );
    $finalPriceBeforeRoundUp = round($finalPriceBeforeRoundUp + $taxPrice, 2); // Add tax to the price

    $rp_dp_round_up_dynamic_price = rp_get_option('rp_dp_round_up_dynamic_price', '1');
    $expected_values = array('ceil', '0', '95', '99', '100');
    $rp_dp_round_up_method = rp_get_option('rp_dp_round_up_method', 'ceil');

    if( $rp_dp_round_up_dynamic_price == '1' && in_array( $rp_dp_round_up_method, $expected_values ) ) {
        // Need to round up the price
        $finalPriceAfterRoundUp = rp_round_up_nearest_five( $finalPriceBeforeRoundUp, $rp_dp_round_up_method );
    } else {
        $finalPriceAfterRoundUp = $finalPriceBeforeRoundUp;
    }

    // final tax calculation:
    $finalTax = round( ($finalPriceAfterRoundUp / ($taxRate + 100)) * $taxRate, 2 );
    $finalExcludingTax = $finalPriceAfterRoundUp - $finalTax;

    return array('dynamic', $finalPriceAfterRoundUp, array(
        'all_skus' => $skus,
        'sku_row' => $sku_row,
        'stock_price' => $stockPrice,
        'margin' => $margin,
        'tax_rate' => $taxRate,
        'tax_price' => $taxPrice,
        'final_tax' => $finalTax,
        'final_excluding_tax' => $finalExcludingTax,
        'before_round_up' => $finalPriceBeforeRoundUp,
        'after_round_up' => $finalPriceAfterRoundUp,
        'rp_dp_round_up_dynamic_price' => $rp_dp_round_up_dynamic_price,
        'rp_dp_round_up_method' => $rp_dp_round_up_method,
        'rp_dp_which_price_to_consider' => $rp_dp_which_price_to_consider,
        'calculated_at_time' => time()
    ));

}


function rp_filter_all_repairs_before_return( $all_repairs = array(), $m_uq_id = '', $repair_attrs_from_db = array() ) {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return $all_repairs;

    }

    foreach( $all_repairs as $key => $repairRow ) {

        $last_synced_date = rp_get_option('rp_dp_finished_cron_processing', '');

        // format = 24/12/22
        if( !empty( $last_synced_date ) ) {

            $repairRow->last_synced_date = date( rp_get_option('rp_dp_date_format_front_end', 'd/m/y'), $last_synced_date);

        }

        if( !isset( $repair_attrs_from_db[ $repairRow->r_id ] ) ) {

            // Only do when no repair attr is found in db

            list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $repairRow );

            $repairRow->r_price = $price . '';

            if( $method === 'dynamic' && !empty( $last_synced_date ) ) {

                $repairRow->is_dynamic = '1';

            } else {

                $repairRow->is_dynamic = '0';

            }

            $all_repairs[$key] = $repairRow;

        } else {

            $repairRow->is_dynamic = '0';

            foreach( $repair_attrs_from_db[ $repairRow->r_id ] as $attrKey => $attrRow ) {

                list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $attrRow );

                if( $method === 'dynamic' && !empty( $last_synced_date ) ) {

                    $repairRow->is_dynamic = '1';

                }

            }

            // attr found so make the price 0
            $repairRow->r_price = '0';

            $all_repairs[$key] = $repairRow;

        }

    }

    return $all_repairs;

}

add_filter('rp_filter_all_repairs_before_return', 'rp_filter_all_repairs_before_return', 10, 3); // [R1]

function rp_filter_all_repairs_before_return_extended( $all_repairs = array(), $m_uq_id = '', $repair_attrs_from_db = array() ) {

    $rp_hide_attribute_if_only_one = rp_get_option('rp_hide_attribute_if_only_one', '1');

    if( $rp_hide_attribute_if_only_one != '1' || empty( $all_repairs ) ) {

        return $all_repairs;

    }

    foreach( $all_repairs as $key => $repairRow ) {

        $singleAttribute = rp_find_single_attribtute_of_repair( $m_uq_id, $repairRow );

        if( !empty( $singleAttribute ) ) {

            $repairRow->r_price = $singleAttribute->a_price;

            $repairRow->is_dynamic = $singleAttribute->is_dynamic ?? '0';

        }

        $all_repairs[$key] = $repairRow;

    }

    return $all_repairs;

}

add_filter('rp_filter_all_repairs_before_return', 'rp_filter_all_repairs_before_return_extended', 11, 3);


function rp_filter_all_repair_attr_before_return( $repair_attr = array(), $m_uq_id = '' ) {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return $repair_attr;

    }

    $rp_hide_attribute_if_no_part = rp_get_option('rp_hide_attribute_if_no_part', '0');

    foreach( $repair_attr as $key => $attrRow ) {

        list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $attrRow );

        if( $rp_hide_attribute_if_no_part === '1' && ($price == 0 || $price == 0.00 || empty($price)) ) {
                
            unset( $repair_attr[$key] );

            continue;

        } else {

            $attrRow->a_price = $price . '';

            $attrRow->is_dynamic = $method === 'dynamic' ? '1' : '0';

            $repair_attr[$key] = $attrRow;

        }

    }

    $reindex = array();

    foreach( $repair_attr as $key => $attrRow ) {

        $reindex[] = $attrRow;

    }

    return $reindex;

}

add_filter('rp_filter_all_repair_attr_before_return', 'rp_filter_all_repair_attr_before_return', 10, 2); // [A1]

function rp_hide_attribute_if_only_one( $repair_attr = array(), $m_uq_id = '' ) {

    // Show repair attribute information if there is only one attribute

    $rp_hide_attribute_if_only_one = rp_get_option('rp_hide_attribute_if_only_one', '1');

    if( $rp_hide_attribute_if_only_one === '1' && count( $repair_attr ) === 1 ) {

        return array();

    }

    return $repair_attr;

}

add_filter('rp_filter_all_repair_attr_before_return', 'rp_hide_attribute_if_only_one', 11, 2);

function get_m_uq_id_by_m_id( $m_id = 0 ) {

    global $rpQuery;

    static $cache = array();

    $m_id = intval( $m_id );

    if( isset( $cache[ $m_id ] ) ) {

        return $cache[ $m_id ];

    }

    $rs_model = $rpQuery->prefix . 'rs_model';

    $sql = "SELECT * FROM `$rs_model` WHERE m_id = $m_id";

    $modelRow = $rpQuery->get_row( $sql );

    if( !empty( $modelRow ) ) {

        $cache[ $m_id ] = $modelRow->m_uq_id;

        return $cache[ $m_id ];

    }

    return '';

}

function rp_filter_before_adding_repair_to_cart( $row = null ) {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return $row;

    }

    $m_uq_id = get_m_uq_id_by_m_id($row->m_id_fk);

    if(empty($m_uq_id)) {

        return $row;

    }

    list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $row );

    $row->r_price = $price . '';

    return $row;

}

add_filter('rp_filter_before_adding_repair_to_cart', 'rp_filter_before_adding_repair_to_cart', 10, 1); // [R2]

function rp_filter_before_adding_repair_to_cart_extended( $row = null ) {

    $rp_hide_attribute_if_only_one = rp_get_option('rp_hide_attribute_if_only_one', '1');

    if( $rp_hide_attribute_if_only_one != '1' || empty( $row ) ) {

        return $row;

    }

    $m_uq_id = get_m_uq_id_by_m_id($row->m_id_fk);

    if(empty($m_uq_id)) {

        return $row;

    }

    $singleAttribute = rp_find_single_attribtute_of_repair( $m_uq_id, $row );

    if( !empty( $singleAttribute ) ) {

        $row->r_price = $singleAttribute->a_price;

        $row->is_dynamic = $singleAttribute->is_dynamic ?? '0';

    }

    return $row;

}

add_filter('rp_filter_before_adding_repair_to_cart', 'rp_filter_before_adding_repair_to_cart_extended', 11, 1);

function rp_filter_before_adding_repair_attr_to_cart( $row = null, $repairRow = null ) {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return $row;

    }

    $m_uq_id = get_m_uq_id_by_m_id($repairRow->m_id_fk);

    if(empty($m_uq_id)) {

        return $row;

    }

    list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $row );

    $row->a_price = $price . '';

    $row->is_dynamic = $method === 'dynamic' ? '1' : '0';

    return $row;

}

add_filter('rp_filter_before_adding_repair_attr_to_cart', 'rp_filter_before_adding_repair_attr_to_cart', 10, 2);


function rp_inserted_row_in_rs_order_repairs( $insert_id = 0, $order_id_fk = 0, $r_id = 0 ) {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return;

    }

    global $rpQuery;

    $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

    $related_repair = rp_get_where_query('rs_repair', 'r_id', $r_id, TRUE, '=');

    if( !empty( $related_repair ) ) {

        $m_uq_id = get_m_uq_id_by_m_id($related_repair->m_id_fk);

        if(!empty($m_uq_id)) {

            list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $related_repair );

            // update rs_order_repairs where or_id = $insert_id
            // set pricing_method = $method, pricing_info = serialize($debug)

            $rpQuery->query("UPDATE `$rs_order_repairs` SET pricing_method = ':pricing_method', pricing_info = ':pricing_info' WHERE or_id = ':or_id' AND o_id_fk = ':o_id_fk'", array(

                'pricing_method' => $method,

                'pricing_info' => serialize($debug),

                'or_id' => $insert_id,

                'o_id_fk' => $order_id_fk

            ));

        }

    }

}

add_action('rp_inserted_row_in_rs_order_repairs', 'rp_inserted_row_in_rs_order_repairs', 10, 3);

function rp_inserted_row_in_rs_order_repairs_extended( $insert_id = 0, $order_id_fk = 0, $r_id = 0 ) {

    $rp_hide_attribute_if_only_one = rp_get_option('rp_hide_attribute_if_only_one', '1');

    if( $rp_hide_attribute_if_only_one != '1' ) {

        return;

    }

    global $rpQuery;

    $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

    $related_repair = rp_get_where_query('rs_repair', 'r_id', $r_id, TRUE, '=');

    if( !empty( $related_repair ) ) {

        $m_uq_id = get_m_uq_id_by_m_id($related_repair->m_id_fk);

        if(!empty($m_uq_id)) {

            $visible_attributes = rp_get_only_visible_attributes( $related_repair );

            if( is_array($visible_attributes) && count( $visible_attributes ) === 1 ) {

                $attribute = $visible_attributes[0];

                rp_insert_into_rs_order_repair_attr( $order_id_fk, $attribute->a_id );

            }

        }

    }

}

add_action('rp_inserted_row_in_rs_order_repairs', 'rp_inserted_row_in_rs_order_repairs_extended', 11, 3);


function rp_inserted_row_in_rs_order_repair_attr( $insert_id = 0, $order_id_fk = 0, $a_id = 0 ) {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return;

    }

    global $rpQuery;

    $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

    $related_repair_attr = rp_get_where_query('rs_repair_attr', 'a_id', $a_id, TRUE, '=');

    if(!empty($related_repair_attr)) {

        $repairRow = rp_find_repair_row_by_id( $related_repair_attr->r_id_fk );

        $m_uq_id = get_m_uq_id_by_m_id($repairRow->m_id_fk);

        if(!empty($m_uq_id)) {

            list( $method, $price, $debug ) = rp_dp_get_dynamic_price( $m_uq_id, $related_repair_attr );

            // update rs_order_repair_attr where ora_id = $insert_id
            // set pricing_method = $method, pricing_info = serialize($debug)

            $rpQuery->query("UPDATE `$rs_order_repair_attr` SET pricing_method = ':pricing_method', pricing_info = ':pricing_info' WHERE ora_id = ':ora_id' AND o_id_fk = ':o_id_fk'", array(

                'pricing_method' => $method,

                'pricing_info' => serialize($debug),

                'ora_id' => $insert_id,

                'o_id_fk' => $order_id_fk

            ));

        }

    }

}

add_action('rp_inserted_row_in_rs_order_repair_attr', 'rp_inserted_row_in_rs_order_repair_attr', 10, 3);

function rp_dp_show_price_was_updated_on( $bool = FALSE ) {

    return TRUE;

}

add_action('rp_dp_show_price_was_updated_on', 'rp_dp_show_price_was_updated_on', 10, 1);


function rp_dp_add_new_translation_by_release( $new_translation_groups = array() ) {

    $new_translation_groups['rp_dp_plugin_translation_v_1p0'] = array(

        'the_price_was_updated_on'

    );

    return $new_translation_groups;

}

add_filter('rp_force_fetch_new_translation_groups', 'rp_dp_add_new_translation_by_release', 11, 1);

function rp_get_uq_ids_of_all_available_repairs() {

    global $rpQuery;

    $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

    $rs_category = $rpQuery->prefix . 'rs_category';

    // first of all, get all rows from rs_default_repair group by dr_uq_id and dr_category

    $default_repairs = $rpQuery->get_results("SELECT $rs_default_repair.dr_uq_id, $rs_category.c_uq_id FROM `$rs_default_repair` 
        INNER JOIN `$rs_category` ON `$rs_default_repair`.dr_category = `$rs_category`.c_id 
        GROUP BY dr_uq_id, dr_category");

    $dr_uq_ids = array();

    foreach( $default_repairs as $default_repair ) {

        $dr_uq_ids[] = $default_repair->c_uq_id. '|' .  $default_repair->dr_uq_id;

    }

    if( empty( $dr_uq_ids ) ) {

        return array();

    }

    $dr_uq_ids = array_unique( $dr_uq_ids );

    return $dr_uq_ids;

}

function rp_download_attributes_of_installed_repairs() {

    global $rpQuery;

	require_once RP_DP_PLUGIN_PATH . 'classes/Base_API.php';

	$base_api = new RepairPluginPro\Base_API($rpQuery);
	
	$response = $base_api->fetch('Endpoints/fetch_all_attributes', array(), FALSE);
	
	$all_attributes = array();
	
	if( isset( $response ) && is_object( $response ) && property_exists( $response, 'all_attributes' ) ) {
	
		$all_attributes = $response->all_attributes;
	
	}

    return $all_attributes;

}

function rp_get_cached_or_download_all_attributes() {

    $rp_dp_last_downloaded_all_attrs = rp_get_option('rp_dp_last_downloaded_all_attrs', FALSE);

    $all_attributes = FALSE;
    $last_synced = FALSE;

    if( $rp_dp_last_downloaded_all_attrs !== FALSE && is_array($rp_dp_last_downloaded_all_attrs) && isset( $rp_dp_last_downloaded_all_attrs['last_synced'] ) ) {

        if( $rp_dp_last_downloaded_all_attrs['last_synced'] > ( time() - 600 ) ) {

            // it is less than 10 minutes since the last download, use the cached version
            $all_attributes = $rp_dp_last_downloaded_all_attrs['response'];
            $last_synced = $rp_dp_last_downloaded_all_attrs['last_synced'];

        }

    }

    if( $all_attributes === FALSE ) {

        // download the all_attributes
        $all_attributes = rp_download_attributes_of_installed_repairs();

        if( !empty( $all_attributes ) ) {

            $last_synced = time();

            // save the all_attributes
            rp_update_option('rp_dp_last_downloaded_all_attrs', array(
                'last_synced' => $last_synced,
                'response' => $all_attributes
            ));

        } else {

            $all_attributes = FALSE;
            $last_synced = FALSE;

        }

    }

    if( $all_attributes === FALSE && $rp_dp_last_downloaded_all_attrs !== FALSE && is_array($rp_dp_last_downloaded_all_attrs) && isset( $rp_dp_last_downloaded_all_attrs['last_synced'] ) ) {

        // download failed
        // more than 10 minutes but we have no other option, use the cached version
        $all_attributes = $rp_dp_last_downloaded_all_attrs['response'];
        $last_synced = $rp_dp_last_downloaded_all_attrs['last_synced'];

    }

    return array($all_attributes, $last_synced);

}

function rp_find_missing_repair_attributes() {

    if( !rp_is_enabled_dynamic_pricing() ) {

        return array();

    }

    global $rpQuery;

    $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

    $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

    $rs_category = $rpQuery->prefix . 'rs_category';

    list( $all_attributes, $last_synced ) = rp_get_cached_or_download_all_attributes();

    $missingAttributes = array();

    $availableRepairs = rp_get_uq_ids_of_all_available_repairs();

    if( !empty( $all_attributes ) ) {

        foreach( $all_attributes as $theAttribute ) {

            list( $repair_uq_id, $c_uq_id, $da_uq_id ) = explode('|', $theAttribute);

            if( !in_array( $c_uq_id . '|' . $repair_uq_id, $availableRepairs ) ) {

                continue;

            }

            $sql = "SELECT * FROM `$rs_default_repair_attr` 
                INNER JOIN `$rs_category` ON `$rs_default_repair_attr`.dr_category = `$rs_category`.c_id 
                INNER JOIN `$rs_default_repair` ON `$rs_default_repair_attr`.dr_id_fk = `$rs_default_repair`.dr_id
                WHERE `$rs_category`.c_uq_id = ':c_uq_id' AND `$rs_default_repair_attr`.da_uq_id = ':da_uq_id'";

            $related_default_repair_attr = $rpQuery->get_row($sql, array( 'c_uq_id' => $c_uq_id, 'da_uq_id' => $da_uq_id ));

            if( empty( $related_default_repair_attr ) ) {

                $missingAttributes[] = $theAttribute;

            }
            
        }

    }

    return $missingAttributes;

}

function rp_update_dp_supplier_linking() {

    rp_verify_csrf_token_for_ajax('dynamic_pricing');

    $supplier_id = $_POST['supplier_id'] ?? '0';

    $b_ids = $_POST['b_ids'] ?? array();

    if( !empty( $supplier_id ) ) {

        rp_update_option('rp_dp_supplier_linking_' . $supplier_id, $b_ids);

    }

    echo json_encode(array('status' => true));

    exit();

}

rp_ajax_for_admin('rp_update_dp_supplier_linking', 'rp_update_dp_supplier_linking');

function rp_get_all_brand_ids_for_dp_supplier_linking() {

    global $rpQuery;

    $rs_brand = $rpQuery->prefix . 'rs_brand';

    $sql = "SELECT b_id FROM `$rs_brand`";

    $all_brands = $rpQuery->get_results($sql);

    if( empty( $all_brands ) ) {

        return array();

    }

    $all_brand_ids = array();

    foreach( $all_brands as $brand_row ) {

        $all_brand_ids[] = $brand_row->b_id;

    }

    return $all_brand_ids;

}

function rp_get_manage_dynamic_pricing_linking_markup() {

    $supplier_id = $_POST['supplier_id'] ?? '0';

    $linked_brands = rp_get_option('rp_dp_supplier_linking_' . $supplier_id, FALSE);

    if( $linked_brands === FALSE ) {

        $linked_brands = rp_get_all_brand_ids_for_dp_supplier_linking();

    }

    ob_start();

    $categories = rp_get_all_models_for_upsales_page();

    $show_update_btn = false;

    if( !empty( $categories ) ) {

        $show_update_btn = true;

        ?>
        <p style="font-family: 'Montserrat'; font-size: 13px; margin-bottom: 0px;">Choose which categories and brands to link with this supplier (<b><?php echo $_POST['supplier_name'] ?? ''; ?></b>).</p>
        <input type="hidden" name="supplier_id" value="<?php echo $supplier_id; ?>">
        <div class="details-container clearfix">
            <div class="wp-repair-col-4">
                <div class="multicheckboxes">
                    <ul id="tree">
                        <?php 
                        foreach($categories as $category_row) {
                            $category_row->c_name = rp_sanitize_output($category_row->c_name);
                            $c_checked = '';
                            // check if any brand is linked with this category
                            foreach($category_row->_brands as $brand_row) {
                                if( in_array( $brand_row->b_id, $linked_brands ) ) {
                                    $c_checked = 'checked="checked"';
                                    break;
                                }
                            }
                            ?>
                            <li>
                            <button type="button" class="btn-dropdown"><span class="chevron up"><i class="fas fa-chevron-down"></i></span></button>
                            <label class="checkbox-container">
                                <input type="checkbox" <?php echo $category_row->c_name; ?> name="c_id" value="<?php echo $category_row->c_name; ?>" <?php echo $c_checked; ?>>
                                <span class="checkmark"></span>
                                <span class="text"><?php echo $category_row->c_name; ?></span>
                            </label>
                            <ul class="opened">
                            <?php
                            foreach($category_row->_brands as $brand_row) {
                                $checked = in_array( $brand_row->b_id, $linked_brands ) ? 'checked="checked"' : '';
                                $brand_row->b_name = rp_sanitize_output($brand_row->b_name);
                                ?>
                                <li>
                                <label class="checkbox-container">
                                        <input type="checkbox" <?php echo $brand_row->b_name; ?> name="b_id" value="<?php echo $brand_row->b_id; ?>" <?php echo $checked; ?>>
                                        <span class="checkmark"></span>
                                        <span class="text"><?php echo $brand_row->b_name; ?></span>
                                </label>
                                </li>
                                <?php
                            }
                            ?>
                            </ul>
                            </li>
                            <?php
                        }
                        ?>
                    </ul>
                </div>
            </div>
        </div> 
        <?php

    } else {

        ?>
        <p>Looks like you do not have any created categories and brands, You need to create a category that contains both a brand and a model before you can manage linking.</p>
        <p>If you prefer not to do this manually, you can <a href="?page=wp_repair_download_libraries&section=download_libraries">download over 1900 models</a> from our database, and our import function will automatically create a category and brand associated with each model.</p>
        <?php
        
    }

    $html = ob_get_clean();

    echo json_encode(array('status' => true, 'html_markup' => $html, 'show_update_btn' => $show_update_btn));

    exit();
    
}

rp_ajax_for_admin('rp_get_manage_dynamic_pricing_linking_markup', 'rp_get_manage_dynamic_pricing_linking_markup');
