<?php

use RepairPluginPro\RP_Maps_API;

// Exit if accessed directly

defined( 'ABSPATH' ) || exit;

if(!defined('RP_EL_PLUGIN_FILE')) {

    define( 'RP_EL_PLUGIN_FILE', __FILE__ );

    define( 'RP_EL_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );

    define( 'RP_EL_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );

    define( 'RP_EL_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

    if( defined('WP_REPAIR_ASSETS_CACHE_V') ) {

        define( 'RP_EL_ASSETS_CACHE_V', WP_REPAIR_ASSETS_CACHE_V );

        define( 'RP_EL_BASE_SERVER_URL', WP_REPAIR_BASE_SERVER_URL );

    } else {

        define( 'RP_EL_ASSETS_CACHE_V', 'v' . time() );

        define( 'RP_EL_BASE_SERVER_URL', 'https://base.repairplugin.com/' );

    }

    define( 'RP_EL_REPEAT_CRON_AFTER_X_SECONDS', ( 15 * 60 ) ); // 15 minutes

}

function rp_is_active_enhanced_locations_addon() {

    $final_result = RP_EnhancedLocationsAddon::is_active();

    return apply_filters('rp_filter_is_active_enhanced_locations_addon', $final_result);

}

function rp_enhanced_locations_require_message() {
    // Enhanced Locations addon require Repairplugin to be installed and activated
    $mesesage = 'Enhanced Locations addon require Repairplugin to be installed and activated';
    echo '<div class="notice notice-error is-dismissible"><p>' . $mesesage . '</p></div>';

    return;
}

function rp_enhanced_locations_addon_init() {

    if( !defined('WP_REPAIR_PLUGIN_FILE') ) {
        // Add admin notice
        add_action('admin_notices', 'rp_enhanced_locations_require_message');
        return;
    }

    do_action('rp_enhanced_locations_initialize');

}

add_action('init', 'rp_enhanced_locations_addon_init', 1000);

function rp_add_enhanced_locations_tab_in_sidebar( $sidebar_items = array() ) {

    $sidebar_items['enhanced_locations'] = array(
        'title' => 'Enhanced Locations',
        'icon' => 'fas fa-map-marker-alt',
        'link' => createTheLink('enhanced_locations'),
        'render_condition' => true,
        'position' => 15
    );

    return $sidebar_items;

}

add_filter('rp_admin_settings_sidebar_items', 'rp_add_enhanced_locations_tab_in_sidebar');

function rp_enhanced_locations_tab_actual_html() {
    
    $fileName = RP_EL_PLUGIN_PATH . 'html/enhanced_locations_settings_tab.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}

function rp_handle_enhanced_locations_page_html( $return = FALSE, $section = '' ) {

    if( $section == 'enhanced_locations' ) {
        // return the html for the dynamic pricing page
        rp_enhanced_locations_tab_actual_html();

        return TRUE;

    }

    return $return;

}

add_filter('rp_returning_admin_setting_page_html', 'rp_handle_enhanced_locations_page_html', 10, 2);


function rp_create_rs_schedule_groups_table() {
      
    /* Added in Enhanced Locations V1.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_schedule_groups (

    `id` int(11) NOT NULL AUTO_INCREMENT,

    `name` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    
    PRIMARY KEY (`id`))

    $charset_collate";

    dbDelta( $sql );

}

add_action('rp_enhanced_locations_initialize', 'rp_create_rs_schedule_groups_table', 999);


function rp_create_rs_schedule_group_timing_table() {
      
    /* Added in Enhanced Locations V1.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_schedule_group_timing (

    `cs_id` int(11) NOT NULL AUTO_INCREMENT,

    `cs_day` varchar(255) NOT NULL,

    `cs_open_timing` varchar(255) NOT NULL,

    `cs_close_timing` varchar(255) NOT NULL,

    `cs_breaktime_from` varchar(255) NOT NULL,

    `cs_breaktime_to` varchar(255) NOT NULL,

    `is_closed` tinyint(4) NOT NULL DEFAULT 1,

    `schedule_group_id` int(11) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`cs_id`))

    $charset_collate";

    dbDelta( $sql );

}

add_action('rp_enhanced_locations_initialize', 'rp_create_rs_schedule_group_timing_table', 999);

function rp_get_total_schedule_groups() {

    global $rpQuery;

    $table_rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $sql = "SELECT COUNT(*) FROM $table_rs_schedule_groups";

    $results = $rpQuery->get_var($sql);

    if( !empty($results) ) {

        return $results;

    }

    return 0;

}

function rp_get_all_schedule_groups( $current_page = 0, $limit = 0 ) {

    global $rpQuery;

    $table_rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $offset = 0;

    if( $current_page > 0 ) {

        $offset = ( $current_page - 1 ) * $limit;

    }

    $sql = "SELECT * FROM $table_rs_schedule_groups";

    // Add limit and offset
    if( $limit > 0 ) {

        $sql .= " LIMIT $limit";

    }

    if( $offset > 0 ) {

        $sql .= " OFFSET $offset";

    }

    $results = $rpQuery->get_results($sql);

    if( !empty($results) ) {

        return $results;

    }

    return array();

}

function rp_get_schedule_group_timing( $schedule_group_id = 0 ) {

    global $rpQuery;

    $schedule_group_id = (int) $schedule_group_id;

    $table_rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    $sql = "SELECT * FROM $table_rs_schedule_group_timing WHERE `schedule_group_id` = $schedule_group_id";

    $results = $rpQuery->get_results($sql);

    if( !empty($results) ) {

        return $results;

    }

    return array();

}

function rp_verify_enhanced_location_is_active() {

    if( rp_is_active_enhanced_locations_addon() === false ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Something went wrong. Please try again later.'));

        exit;

    }

}

function rp_delete_schedule_group(){

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $id = (int) ($_POST['id'] ?? 0);

    if( $id <= 0 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid ID'));

        exit;

    }

    $table_rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $table_rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    // make sure it is not the last schedule group
    $total_schedule_groups = rp_get_total_schedule_groups();

    if( $total_schedule_groups <= 1 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'You cannot delete the last schedule group'));

        exit;

    }

    // check if this schedule group is assigned to any location
    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $sql = "SELECT COUNT(*) FROM $rs_company_location WHERE `schedule_group_id` = $id";

    $results = $rpQuery->get_var($sql);
    
    $in_use = FALSE; 

    if( !empty($results) ) {

        $in_use = TRUE;

    }

    // if in_use, make sure we receive alt_schedule_group_id in $_POST and that id is not equal to current schedule group id and that id should be a valid schedule group id
    if( $in_use === TRUE ) {

        $alt_schedule_group_id = (int) ($_POST['alt_schedule_group_id'] ?? 0);

        if( $alt_schedule_group_id <= 0 ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Alternative Schedule Group ID'));

            exit;

        }

        if( $alt_schedule_group_id == $id ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Alternative Schedule Group ID cannot be same as current Schedule Group ID'));

            exit;

        }

        $sql = "SELECT COUNT(*) FROM $table_rs_schedule_groups WHERE `id` = $alt_schedule_group_id";

        $results = $rpQuery->get_var($sql);

        if( empty($results) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Alternative Schedule Group ID'));

            exit;

        }

        // update all locations with alt_schedule_group_id
        $sql = "UPDATE `$rs_company_location` SET `schedule_group_id` = $alt_schedule_group_id WHERE `schedule_group_id` = $id";

        $rpQuery->query($sql);

    }

    $sql = "DELETE FROM `$table_rs_schedule_groups` WHERE `id` = $id";

    $rpQuery->query($sql);

    $sql = "DELETE FROM `$table_rs_schedule_group_timing` WHERE `schedule_group_id` = $id";

    $rpQuery->query($sql);

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

    exit;

}

function rp_deleting_schedule_group(){

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $id = (int) ($_POST['id'] ?? 0);

    if( $id <= 0 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid ID'));

        exit;

    }

    $available_schedules = rp_get_all_schedule_groups();
    
    // loop through all available schedules and remove the $id from the array
    foreach( $available_schedules as $key => $schedule ) {

        if( $schedule->id == $id ) {

            unset($available_schedules[$key]);

        }

    }

    $available_schedules = array_values($available_schedules);

    // check if any rs_company_location is using this schedule group
    $table_rs_company_location = $rpQuery->prefix . "rs_company_location";

    $sql = "SELECT * FROM `$table_rs_company_location` WHERE `schedule_group_id` = $id";

    $results = $rpQuery->get_results($sql);

    $in_use = 0;

    if( !empty($results) ) {

        $in_use = 1;

    }

    echo json_encode(Array('status'=> true, 'available_schedules'  => $available_schedules, 'in_use' => $in_use));

    exit;

}

function rp_deleting_location(){

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $id = (int) ($_POST['id'] ?? 0);

    if( $id <= 0 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid ID'));

        exit;

    }

    // check if any rs_company_location is using this schedule group
    $table_rs_company_location = $rpQuery->prefix . "rs_company_location";

    // make sure it is not the last location
    $total_locations = $rpQuery->get_var("SELECT COUNT(*) FROM $table_rs_company_location");

    $safe_to_delete = false;

    if( $total_locations > 1 ) {

        $safe_to_delete = true;

    }

    echo json_encode(Array('status'=> true, 'safe_to_delete' => $safe_to_delete));

    exit;

}

function rp_el_apply_limit_to_locations( $locations = array() ) {

    if( empty( $locations ) ) {

        return $locations;

    }

    if( !RP_EnhancedLocationsAddon::is_active() ) {

        // Enhanced Locations isn't active
        // So no need to apply limit
        return $locations;

    }

    $the_limit = RP_EnhancedLocationsAddon::get_limit();

    if( $the_limit === FALSE ) {

        // unlimited
        return $locations;

    }

    if( $the_limit == 0 ) {

        return array();

    }

    $the_limit = (int) $the_limit;

    $locations = array_slice($locations, 0, $the_limit);

    return $locations;

}

function rp_verify_customer_is_not_reaching_limit() {
    
    $the_limit = RP_EnhancedLocationsAddon::get_limit();

    if( $the_limit === false ) {

        // unlimited
        return TRUE;

    }

    if( $the_limit == 0 ) {

        return FALSE;

    }

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $sql = "SELECT COUNT(*) FROM $rs_company_location";

    $results = $rpQuery->get_var($sql);

    if( ( (int) $results ) >= (int) $the_limit ) {

        return FALSE;

    }

    return TRUE;

}

function rp_customer_reaching_limit_verification() {

    $not_reaching_limit = rp_verify_customer_is_not_reaching_limit();

    if( $not_reaching_limit === FALSE ) {

        // Reaching limit
        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'You have reached the limit of locations you can create. Please upgrade your license to create more locations.'));

        exit;

    }

}

function rp_get_location_data_for_edit() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $id = (int) ($_POST['id'] ?? 0);

    $is_clone = (int) ($_POST['is_clone'] ?? FALSE);

    $_rp_company_email = rp_get_settings_company_email();

    $available_schedules = rp_get_all_schedule_groups();

    $available_schedules = rp_sanitize_output($available_schedules);

    if( empty( $id ) ) {

        rp_customer_reaching_limit_verification();

        $is_synced = !empty($_rp_company_email) ? true : false;

        // it is a create form
        echo json_encode(Array('status'=> true, 'data'  => array(
            'id' => 0,
            'name' => '',
            'telephone' => '',
            'address' => '',
            'zipcode_+_city' => '',
            'google_map_link' => '',
            'email' => $_rp_company_email, 
            'schedule' => '',
            'photo' => '',
            'available_schedules' => $available_schedules,
            'is_synced' => $is_synced
        )));

        exit;

    } else {

        // verify if the location exists
        $sql = "SELECT * FROM $rs_company_location WHERE `location_id` = $id";

        $results = $rpQuery->get_row($sql);

        if( empty($results) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Location ID'));

            exit;

        }

        // if not empty $_rp_company_email

        $is_synced = 0;

        if( !empty( $_rp_company_email ) && ( empty( $results->location_email ) || trim( $results->location_email ) == $_rp_company_email ) ) {
                
            $is_synced = 1;
    
        }

        if( empty( $results->location_email ) ) {

            $results->location_email = $_rp_company_email;

        }

        if( $is_clone === 1 ) {
            rp_customer_reaching_limit_verification();
            $results->location_id = 0;
            $results->location_name = $results->location_name . ' (Copy)';
        }

        $results = rp_sanitize_output($results);
        
        echo json_encode(Array('status'=> true, 'data'  => array(
            'id' => $results->location_id,
            'name' => $results->location_name,
            'telephone' => ($results->location_telephone ?? ''),
            'address' => $results->location_address,
            'zipcode_+_city' => $results->location_city_district,
            'google_map_link' => $results->google_map_link,
            'email' => $results->location_email,
            'schedule' => $results->schedule_group_id,
            'photo' => $results->location_image,
            'available_schedules' => $available_schedules,
            'is_synced' => $is_synced
        )));

        exit;

    }

    exit;

}

function rp_get_schedule_data_for_edit() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    $id = (int) ($_POST['id'] ?? 0);

    $is_clone = (int) ($_POST['is_clone'] ?? FALSE);

    if( empty( $id ) ) {

        $timings = array();

        $days = array('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');

        $row_data = array(
            'open_timing' => '09:00',
            'open_timing_ampm' => 'AM',
            'close_timing' => '06:00',
            'close_timing_ampm' => 'PM',
            'break_start_timing' => '01:00',
            'break_start_timing_ampm' => 'PM',
            'break_end_timing' => '02:00',
            'break_end_timing_ampm' => 'PM',
        );

        foreach($days as $theDay) {

            $row_data_copy = $row_data;

            $row_data_copy['day'] = $theDay;

            if( $theDay === 'Sunday' ) {

                $row_data_copy['open_timing'] = '';
                $row_data_copy['open_timing_ampm'] = '';
                $row_data_copy['close_timing'] = '';
                $row_data_copy['close_timing_ampm'] = '';
                $row_data_copy['break_start_timing'] = '';
                $row_data_copy['break_start_timing_ampm'] = '';
                $row_data_copy['break_end_timing'] = '';
                $row_data_copy['break_end_timing_ampm'] = '';

            }

            $timings[] = $row_data_copy;

        }

        // it is a create form
        echo json_encode(Array(
            'status'=> true, 
            'data'  => array(
                'id' => 0,
                'name' => '',
            ), 
            'timings' => $timings
        ));

        exit;

    } else {

        // verify if the location exists
        $sql = "SELECT * FROM $rs_schedule_groups WHERE `id` = $id";

        $results = $rpQuery->get_row($sql);

        if( empty($results) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Schedule Group ID'));

            exit;

        }

        $timings = array();

        $days = array('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');

        foreach( $days as $theDay ) {

            // get timing from rs_schedule_group_timing
            $sql = "SELECT * FROM `$rs_schedule_group_timing` WHERE `schedule_group_id` = $id AND `cs_day` = '$theDay'";

            $timing_results = $rpQuery->get_row($sql);

            $row_data = array(
                'day' => $theDay,
                'open_timing' => '',
                'open_timing_ampm' => '',
                'close_timing' => '',
                'close_timing_ampm' => '',
                'break_start_timing' => '',
                'break_start_timing_ampm' => '',
                'break_end_timing' => '',
                'break_end_timing_ampm' => '',
            );

            if( !empty( $timing_results ) ) {

                if( !empty( $timing_results->cs_open_timing ) ) {

                    $row_data['open_timing'] = date('h:i', strtotime($timing_results->cs_open_timing));
                    $row_data['open_timing_ampm'] = date('A', strtotime($timing_results->cs_open_timing));

                }

                if( !empty( $timing_results->cs_close_timing ) ) {

                    $row_data['close_timing'] = date('h:i', strtotime($timing_results->cs_close_timing));
                    $row_data['close_timing_ampm'] = date('A', strtotime($timing_results->cs_close_timing));

                }

                if( !empty( $timing_results->cs_breaktime_from ) ) {

                    $row_data['break_start_timing'] = date('h:i', strtotime($timing_results->cs_breaktime_from));
                    $row_data['break_start_timing_ampm'] = date('A', strtotime($timing_results->cs_breaktime_from));

                }

                if( !empty( $timing_results->cs_breaktime_to ) ) {

                    $row_data['break_end_timing'] = date('h:i', strtotime($timing_results->cs_breaktime_to));
                    $row_data['break_end_timing_ampm'] = date('A', strtotime($timing_results->cs_breaktime_to));

                }

            }

            $timings[] = $row_data;

        }

        if( $is_clone === 1 ) {
            $results->id = 0;
            $results->name = $results->name . ' (Copy)';
        }

        $results->name = rp_sanitize_output($results->name);
        
        echo json_encode(Array('status'=> true, 'data'  => array(
            'id' => $results->id,
            'name' => $results->name
        ), 'timings' => $timings));

        exit;

    }

    exit;

}

function rp_get_timings_of_schedule_group() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    $id = (int) ($_POST['id'] ?? 0);

    if( empty( $id ) ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Schedule Group ID'));

        exit;

    } else {

        // verify if the location exists
        $sql = "SELECT * FROM $rs_schedule_groups WHERE `id` = $id";

        $results = $rpQuery->get_row($sql);

        if( empty($results) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Schedule Group ID'));

            exit;

        }

        $timings = array();

        $days = array('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');

        foreach( $days as $theDay ) {

            // get timing from rs_schedule_group_timing
            $sql = "SELECT * FROM `$rs_schedule_group_timing` WHERE `schedule_group_id` = $id AND `cs_day` = '$theDay'";

            $timing_results = $rpQuery->get_row($sql);

            $row_data = array(
                'day' => $theDay,
                'open_timing' => '',
                'open_timing_ampm' => '',
                'close_timing' => '',
                'close_timing_ampm' => '',
                'break_start_timing' => '',
                'break_start_timing_ampm' => '',
                'break_end_timing' => '',
                'break_end_timing_ampm' => '',
            );

            if( !empty( $timing_results ) ) {

                if( !empty( $timing_results->cs_open_timing ) ) {

                    $row_data['open_timing'] = date('h:i', strtotime($timing_results->cs_open_timing));
                    $row_data['open_timing_ampm'] = date('A', strtotime($timing_results->cs_open_timing));

                }

                if( !empty( $timing_results->cs_close_timing ) ) {

                    $row_data['close_timing'] = date('h:i', strtotime($timing_results->cs_close_timing));
                    $row_data['close_timing_ampm'] = date('A', strtotime($timing_results->cs_close_timing));

                }

                if( !empty( $timing_results->cs_breaktime_from ) ) {

                    $row_data['break_start_timing'] = date('h:i', strtotime($timing_results->cs_breaktime_from));
                    $row_data['break_start_timing_ampm'] = date('A', strtotime($timing_results->cs_breaktime_from));

                }

                if( !empty( $timing_results->cs_breaktime_to ) ) {

                    $row_data['break_end_timing'] = date('h:i', strtotime($timing_results->cs_breaktime_to));
                    $row_data['break_end_timing_ampm'] = date('A', strtotime($timing_results->cs_breaktime_to));

                }

            }

            $timings[] = $row_data;

        }

        $results = rp_sanitize_output($results);
        
        echo json_encode(Array('status'=> true, 'data'  => array(
            'id' => $results->id,
            'name' => $results->name
        ), 'timings' => $timings));

        exit;

    }

    exit;

}

function rp_handle_location_image_upload() {

    if(isset($_FILES) && !empty($_FILES) && isset($_FILES['location_image'])) {
        $uploadedfile = $_FILES['location_image']; 
        $upload_overrides = array( 'test_form' => false );
        $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
        if(!empty($movefile["url"])){
            $logo = $movefile["url"];
            return $logo;
        }
    }

    return ($_POST['old_location_image'] ?? '');

}

function rp_create_or_edit_location() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $required_fields = array( 'name', 'telephone', 'address', 'zipcode_city', 'email', 'schedule' );

    // make sure all required fields are present
    foreach( $required_fields as $field ) {

        if( empty( $_POST[$field] ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Please fill all required fields'));

            exit;

        }

    }

    $name = sanitize_text_field( $_POST['name'] ?? '' );

    $telephone = sanitize_text_field( $_POST['telephone'] ?? '' );

    $address = sanitize_text_field( $_POST['address'] ?? '' );

    $zipcode_city = sanitize_text_field( $_POST['zipcode_city'] ?? '' );

    $google_map_link = sanitize_text_field( $_POST['google_map_link'] ?? '' );

    $loc_email = sanitize_text_field( $_POST['email'] ?? '' );

    $schedule = sanitize_text_field( $_POST['schedule'] ?? '' );

    $location_image = rp_handle_location_image_upload();

    // make sure selected schedule is valid
    $schedule_groups = rp_get_all_schedule_groups();

    $schedule_group_ids = array();

    foreach( $schedule_groups as $schedule_group ) {

        $schedule_group_ids[] = $schedule_group->id;

    }

    if( !in_array( $schedule, $schedule_group_ids ) ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid schedule'));

        exit;

    }

    if(!empty($loc_email) && filter_var($loc_email, FILTER_VALIDATE_EMAIL) !== FALSE) {

        if( trim($loc_email) !== rp_get_settings_placeholder_email() && trim($loc_email) !== rp_get_settings_company_email() ) {

            // Skip... So It will save the email.

        } else {

            $loc_email = '';

        }

    } else {

        $loc_email = '';

    }

    $location_id = (int) ($_POST['id'] ?? 0);

    if( $location_id == 0 ) {

        // create new location
        $sql = "INSERT INTO `$rs_company_location` (`location_name`, `location_address`, `location_city_district`, `google_map_link`, `location_email`, `schedule_group_id`, `location_image`, `location_telephone`) VALUES (':name', ':address', ':zipcode_city', ':google_map_link', ':loc_email', ':schedule', ':location_image', ':telephone')";

        $rpQuery->query($sql, array(
            'name' => $name,
            'address' => $address,
            'zipcode_city' => $zipcode_city,
            'google_map_link' => $google_map_link,
            'loc_email' => $loc_email,
            'schedule' => $schedule,
            'location_image' => $location_image,
            'telephone' => $telephone
        ));

        $insert_id = $rpQuery->insert_id;

        rp_apply_schedule_to_location($schedule, $insert_id);

    } else {

        // verify location id
        $verify = $rpQuery->get_row("SELECT * FROM $rs_company_location WHERE `location_id` = $location_id");

        if( empty( $verify ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid location ID'));

            exit;

        }

        $sql = "UPDATE `$rs_company_location` SET `location_name` = ':name', `location_address` = ':address', `location_city_district` = ':zipcode_city', `google_map_link` = ':google_map_link', `location_email` = ':loc_email', `schedule_group_id` = ':schedule', `location_image` = ':location_image', `location_telephone` = ':location_telephone' WHERE `location_id` = $location_id";

        $rpQuery->query($sql, array(
            'name' => $name,
            'address' => $address,
            'zipcode_city' => $zipcode_city,
            'google_map_link' => $google_map_link,
            'loc_email' => $loc_email,
            'schedule' => $schedule,
            'location_image' => $location_image,
            'location_telephone' => $telephone
        ));

        rp_apply_schedule_to_location($schedule, $location_id);

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully created / updated'));

    exit;

}

function rp_create_or_edit_schedule() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $required_fields = array( 'name' );

    // make sure all required fields are present
    foreach( $required_fields as $field ) {

        if( empty( $_POST[$field] ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Please fill all required fields'));

            exit;

        }

    }

    $name = sanitize_text_field( $_POST['name'] ?? '' );

    $schedule_group_id = (int) ($_POST['id'] ?? 0);

    $cs_day = $_POST['day'] ?? array();

    $required_days = array( 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday' );

    if( empty( $cs_day ) || !is_array( $cs_day ) || count( $cs_day ) != count( $required_days ) ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Unexpected inputs provided'));

        exit;

    }

    foreach( $required_days as $day ) {

        if( !in_array( $day, $cs_day ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Unexpected inputs provided'));

            exit;

        }

    }

    $required_as_array = array( 'open_timing', 'open_timing_ampm', 'close_timing', 'close_timing_ampm', 'break_start_timing', 'break_start_timing_ampm', 'break_end_timing', 'break_end_timing_ampm' );

    foreach( $required_as_array as $field ) {

        if( empty( $_POST[$field] ?? '' ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Unexpected inputs provided'));

            exit;

        }

        if( !is_array( $_POST[$field] ?? '' ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Unexpected inputs provided'));

            exit;

        }

        if( count( $_POST[$field] ) != count( $required_days ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Unexpected inputs provided'));

            exit;

        }

    }

    if( $schedule_group_id == 0 ) {

        // create new location
        $sql = "INSERT INTO `$rs_schedule_groups` (`name`) VALUES (':name')";

        $rpQuery->query($sql, array(
            'name' => $name,
        ));

        // insert id 
        $schedule_group_id = $rpQuery->insert_id;

    } else {

        // verify location id
        $verify = $rpQuery->get_row("SELECT * FROM $rs_schedule_groups WHERE `id` = $schedule_group_id");

        if( empty( $verify ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid location ID'));

            exit;

        }

        $sql = "UPDATE `$rs_schedule_groups` SET `name` = ':name' WHERE `id` = $schedule_group_id";

        $rpQuery->query($sql, array(
            'name' => $name,
        ));

    }

    // check if schedule_group_id exists in rs_schedule_group_timing
    $rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    $verify = $rpQuery->get_row("SELECT * FROM $rs_schedule_group_timing WHERE `schedule_group_id` = $schedule_group_id");

    if( empty( $verify ) ) {

        foreach( $cs_day as $day_index => $day ) {

            $open_timing = sanitize_text_field( $_POST['open_timing'][$day_index] ?? '' );

            $open_timing_ampm = sanitize_text_field( $_POST['open_timing_ampm'][$day_index] ?? '' );

            $close_timing = sanitize_text_field( $_POST['close_timing'][$day_index] ?? '' );

            $close_timing_ampm = sanitize_text_field( $_POST['close_timing_ampm'][$day_index] ?? '' );

            $break_start_timing = sanitize_text_field( $_POST['break_start_timing'][$day_index] ?? '' );

            $break_start_timing_ampm = sanitize_text_field( $_POST['break_start_timing_ampm'][$day_index] ?? '' );

            $break_end_timing = sanitize_text_field( $_POST['break_end_timing'][$day_index] ?? '' );

            $break_end_timing_ampm = sanitize_text_field( $_POST['break_end_timing_ampm'][$day_index] ?? '' );

            if( empty( $open_timing ) || empty( $close_timing ) ) {

                $is_closed = 1;

                $cs_open_time = '';

                $cs_close_time = '';

                $cs_breaktime_from = '';

                $cs_breaktime_to = '';

            } else {

                $is_closed = 0;

                $cs_open_time = date( 'H:i', strtotime( $open_timing . ' ' . $open_timing_ampm ) );

                $cs_close_time = date( 'H:i', strtotime( $close_timing . ' ' . $close_timing_ampm ) );

                if( !empty( $break_start_timing ) ) {

                    $cs_breaktime_from = date( 'H:i', strtotime( $break_start_timing . ' ' . $break_start_timing_ampm ) );

                } else {

                    $cs_breaktime_from = '';

                }

                if( !empty( $break_end_timing ) ) {

                    $cs_breaktime_to = date( 'H:i', strtotime( $break_end_timing . ' ' . $break_end_timing_ampm ) );

                } else {

                    $cs_breaktime_to = '';

                }

            }

            $sql = "INSERT INTO `$rs_schedule_group_timing` (`schedule_group_id`, `cs_day`, `cs_open_timing`, `cs_close_timing`, `cs_breaktime_from`, `cs_breaktime_to`, `is_closed`) VALUES (':schedule_group_id', ':cs_day', ':cs_open_time', ':cs_close_time', ':cs_breaktime_from', ':cs_breaktime_to', ':is_closed')";

            $rpQuery->query($sql, array(
                'schedule_group_id' => $schedule_group_id,
                'cs_day' => $day,
                'cs_open_time' => $cs_open_time,
                'cs_close_time' => $cs_close_time,
                'cs_breaktime_from' => $cs_breaktime_from,
                'cs_breaktime_to' => $cs_breaktime_to,
                'is_closed' => $is_closed,
            ));

        }

    } else {

        foreach( $cs_day as $day_index => $day ) {

            $open_timing = sanitize_text_field( $_POST['open_timing'][$day_index] ?? '' );

            $open_timing_ampm = sanitize_text_field( $_POST['open_timing_ampm'][$day_index] ?? '' );

            $close_timing = sanitize_text_field( $_POST['close_timing'][$day_index] ?? '' );

            $close_timing_ampm = sanitize_text_field( $_POST['close_timing_ampm'][$day_index] ?? '' );

            $break_start_timing = sanitize_text_field( $_POST['break_start_timing'][$day_index] ?? '' );

            $break_start_timing_ampm = sanitize_text_field( $_POST['break_start_timing_ampm'][$day_index] ?? '' );

            $break_end_timing = sanitize_text_field( $_POST['break_end_timing'][$day_index] ?? '' );

            $break_end_timing_ampm = sanitize_text_field( $_POST['break_end_timing_ampm'][$day_index] ?? '' );

            if( empty( $open_timing ) || empty( $close_timing ) ) {

                $is_closed = 1;

                $cs_open_time = '';

                $cs_close_time = '';

                $cs_breaktime_from = '';

                $cs_breaktime_to = '';

            } else {

                $is_closed = 0;

                $cs_open_time = date( 'H:i', strtotime( $open_timing . ' ' . $open_timing_ampm ) );

                $cs_close_time = date( 'H:i', strtotime( $close_timing . ' ' . $close_timing_ampm ) );

                if( !empty( $break_start_timing ) ) {

                    $cs_breaktime_from = date( 'H:i', strtotime( $break_start_timing . ' ' . $break_start_timing_ampm ) );

                } else {

                    $cs_breaktime_from = '';

                }

                if( !empty( $break_end_timing ) ) {

                    $cs_breaktime_to = date( 'H:i', strtotime( $break_end_timing . ' ' . $break_end_timing_ampm ) );

                } else {

                    $cs_breaktime_to = '';

                }

            }

            $sql = "UPDATE `$rs_schedule_group_timing` SET `cs_open_timing` = ':cs_open_time', `cs_close_timing` = ':cs_close_time', `cs_breaktime_from` = ':cs_breaktime_from', `cs_breaktime_to` = ':cs_breaktime_to', `is_closed` = ':is_closed' WHERE `schedule_group_id` = ':schedule_group_id' AND `cs_day` = ':cs_day'";

            $rpQuery->query($sql, array(
                'cs_open_time' => $cs_open_time,
                'cs_close_time' => $cs_close_time,
                'cs_breaktime_from' => $cs_breaktime_from,
                'cs_breaktime_to' => $cs_breaktime_to,
                'is_closed' => $is_closed,
                'cs_day' => $day,
                'schedule_group_id' => $schedule_group_id,
            ));

        }

    }

    rp_update_all_locations_using_this_schedule( $schedule_group_id );

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully created / updated'));

    exit;

}

function rp_update_all_locations_using_this_schedule( $schedule_group_id = 0 ) {

    if( empty( $schedule_group_id ) ) {

        return;

    }

    $schedule_group_id = (int) $schedule_group_id;

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    // get all locations that are using this schedule

    $locations = $rpQuery->get_results("SELECT * FROM `$rs_company_location` WHERE `schedule_group_id` = $schedule_group_id");

    // If not empty locations, loop through all location_row and rp_apply_schedule_to_location
    if( !empty( $locations ) ) {

        foreach( $locations as $location_row ) {

            rp_apply_schedule_to_location( $schedule_group_id, $location_row->location_id );
    
        }

    }

    return true;

}

function rp_delete_enhanced_location() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

    $location_id = (int) ($_POST['id'] ?? 0);

    if( $location_id == 0 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid location ID'));

        exit;

    }

    // verify location id
    $verify = $rpQuery->get_row("SELECT * FROM $rs_company_location WHERE `location_id` = $location_id");

    if( empty( $verify ) ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid location ID'));

        exit;

    }

    // make sure we will still have at least one location row after deleting this one
    $locations = $rpQuery->get_results("SELECT * FROM $rs_company_location");

    if( count( $locations ) == 1 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'You must keep at least one location'));

        exit;

    }

    $rpQuery->query("DELETE FROM `$rs_company_location` WHERE `location_id` = $location_id");

    $rpQuery->query("DELETE FROM `$rs_company_schedules` WHERE `location_id_fk` = $location_id");

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

    exit;


}

function rp_update_schedule_group_of_location() {

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('enhanced_locations');

    $id = (int) ($_POST['id'] ?? 0);

    $schedule = (int) ($_POST['value'] ?? 0);

    if( $id == 0 || $schedule == 0 ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid location ID or schedule ID'));

        exit;

    }

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $available_schedules = rp_get_all_schedule_groups();

    // make sure the schedule id is valid
    if( !empty( $available_schedules ) ) {

        $schedule_group_ids = array();

        foreach( $available_schedules as $schedule_group ) {

            $schedule_group_ids[] = $schedule_group->id;

        }

    } else {

        $schedule_group_ids = array();

    }

    if( !in_array( $schedule, $schedule_group_ids ) ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid schedule ID'));

        exit;

    }

    // make sure the location id is valid
    $verify = $rpQuery->get_row("SELECT * FROM $rs_company_location WHERE `location_id` = $id");

    if( empty( $verify ) ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid location ID'));

        exit;

    }

    $rpQuery->query("UPDATE `$rs_company_location` SET `schedule_group_id` = $schedule WHERE `location_id` = $id");

    rp_apply_schedule_to_location( $schedule, $id );

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    exit;

}

function rp_enhanced_locations_addon_initialize() {
  
    rp_ajax_for_admin('rp_delete_schedule_group','rp_delete_schedule_group');

    rp_ajax_for_admin('rp_deleting_schedule_group','rp_deleting_schedule_group');

    rp_ajax_for_admin('rp_deleting_location','rp_deleting_location');

    rp_ajax_for_admin('rp_get_location_data_for_edit', 'rp_get_location_data_for_edit');

    rp_ajax_for_admin('rp_get_schedule_data_for_edit', 'rp_get_schedule_data_for_edit');

    // Need to rp_apply_schedule_to_location
    rp_ajax_for_admin('rp_create_or_edit_location', 'rp_create_or_edit_location');

    // Need to rp_apply_schedule_to_location
    rp_ajax_for_admin('rp_create_or_edit_schedule', 'rp_create_or_edit_schedule');

    rp_ajax_for_admin('rp_delete_enhanced_location', 'rp_delete_enhanced_location');

    // Need to rp_apply_schedule_to_location
    rp_ajax_for_admin('rp_update_schedule_group_of_location', 'rp_update_schedule_group_of_location');

    rp_ajax_for_admin('rp_get_timings_of_schedule_group', 'rp_get_timings_of_schedule_group');

    rp_ajax_for_public('rp_search_nearest_locations', 'rp_search_nearest_locations');

    rp_ajax_for_public('rp_get_place_autocomplete_suggestions', 'rp_get_place_autocomplete_suggestions');

}

add_action('rp_enhanced_locations_initialize', 'rp_enhanced_locations_addon_initialize');


function rp_get_all_company_locations( $current_page = 0, $limit = 0, $where_sql = '' ) {

    global $rpQuery;

    $offset = 0;

    if( $current_page > 0 ) {

        $offset = ( $current_page - 1 ) * $limit;

    }

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $sql = "SELECT * FROM (SELECT *,
    CASE WHEN `location_email` IS NOT NULL AND `location_email` != '' THEN
    CONCAT(`location_name`, ' ', `location_email`, ' ', `location_address`, ' ', `location_city_district`, `location_telephone`)
    ELSE
    CONCAT(`location_name`, ' ', '".rp_get_settings_company_email()."', ' ', `location_address`, ' ', `location_city_district`, `location_telephone`) END AS `search_text`
    FROM $rs_company_location) AS subquery";

    $sql .= " {$where_sql}";

    // Add limit and offset

    if( $limit > 0 ) {

        $sql .= " LIMIT $limit";

    }

    if( $offset > 0 ) {

        $sql .= " OFFSET $offset";

    }

    $results = $rpQuery->get_results($sql);

    if( !empty($results) ) {

        return $results;

    }

    return array();

}

function rp_get_schedule_used_count( $schedule_id = 0 ) {

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $sql = "SELECT COUNT(*) FROM $rs_company_location WHERE `schedule_group_id` = $schedule_id";

    $results = $rpQuery->get_var($sql);

    if( !empty($results) ) {

        return $results;

    }

    return 0;


}

function rp_get_total_company_locations( $where_sql = '' ) {

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $sql = "SELECT * FROM (SELECT *,
    CASE WHEN `location_email` IS NOT NULL AND `location_email` != '' THEN
    CONCAT(`location_name`, ' ', `location_email`, ' ', `location_address`, ' ', `location_city_district`, `location_telephone`)
    ELSE
    CONCAT(`location_name`, ' ', '".rp_get_settings_company_email()."', ' ', `location_address`, ' ', `location_city_district`, `location_telephone`) END AS `search_text`
    FROM $rs_company_location) AS subquery";

    $sql .= " {$where_sql}";

    $results = $rpQuery->get_results($sql);

    if( !empty($results) ) {

        return count($results);

    }

    return 0;

}

function rp_add_schedule_group_column_to_location_table() {

    if( function_exists('rp_add_column_to_table_if_not_exists') ) {

        rp_add_column_to_table_if_not_exists( 'rs_company_location', 'schedule_group_id', 'int(11) NOT NULL' );

        rp_add_column_to_table_if_not_exists( 'rs_company_location', 'location_image', 'varchar(1000) NULL DEFAULT NULL' );

        rp_add_column_to_table_if_not_exists( 'rs_company_location', 'location_telephone', "varchar(255) NULL DEFAULT ''" );

        if( rp_is_active_enhanced_locations_addon() === TRUE ) {

            rp_autocreate_schedule_groups_first_time();

        }

    }

}

add_action('init', 'rp_add_schedule_group_column_to_location_table', 9999);


function rp_apply_schedule_to_location( $schedule_id = 0, $location_id = 0 ) {

    // if empty schedule_id or location_id, return false
    if( empty($schedule_id) || empty($location_id) ) {

        return false;

    }

    global $rpQuery;

    $rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

    // get schedule group
    $schedule_group = $rpQuery->get_row("SELECT * FROM `$rs_schedule_groups` WHERE `id` = $schedule_id");

    // get timing of schedule group
    $schedule_group_timing = $rpQuery->get_results("SELECT * FROM `$rs_schedule_group_timing` WHERE `schedule_group_id` = $schedule_id");

    // get location
    $location = $rpQuery->get_row("SELECT * FROM `$rs_company_location` WHERE `location_id` = $location_id");

    // make sure $schedule_group, $schedule_group_timing and $location are not empty
    if( empty($schedule_group) || empty($schedule_group_timing) || empty($location) ) {

        return false;

    }

    // loop through all schedule group timing
    foreach( $schedule_group_timing as $timing_row ) {

        $cs_day = $timing_row->cs_day;

        $cs_open_timing = $timing_row->cs_open_timing;

        $cs_close_timing = $timing_row->cs_close_timing;

        $cs_breaktime_from = $timing_row->cs_breaktime_from;

        $cs_breaktime_to = $timing_row->cs_breaktime_to;

        $is_closed = $timing_row->is_closed;

        $exists = $rpQuery->get_row("SELECT * FROM `$rs_company_schedules` WHERE `location_id_fk` = $location_id AND `cs_day` = '{$cs_day}'");

        if( !empty( $exists ) ) {

            // Need to update
            $update_id = $exists->cs_id;

            // update cs_day, cs_open_timing, cs_close_timing, cs_breaktime_from, cs_breaktime_to, is_closed
            $rpQuery->query("UPDATE `$rs_company_schedules` SET `cs_day` = '{$cs_day}', `cs_open_timing` = '{$cs_open_timing}', `cs_close_timing` = '{$cs_close_timing}', `cs_breaktime_from` = '{$cs_breaktime_from}', `cs_breaktime_to` = '{$cs_breaktime_to}', `is_closed` = '{$is_closed}' WHERE `cs_id` = $update_id");

        } else {

            // Need to create
            $rpQuery->query("INSERT INTO `$rs_company_schedules` (`location_id_fk`, `cs_day`, `cs_open_timing`, `cs_close_timing`, `cs_breaktime_from`, `cs_breaktime_to`, `is_closed`) VALUES ('$location_id', '{$cs_day}', '{$cs_open_timing}', '{$cs_close_timing}', '{$cs_breaktime_from}', '{$cs_breaktime_to}', '{$is_closed}')");

        }

    }

    return true;

}

function rp_getting_special_schedule_filter( $schedules = array(), $loc_id = 0, $day = '', $date_ymd = '' ) {

    if( rp_is_active_enhanced_locations_addon() === false ) {

        return $schedules;

    }

    global $rpQuery;

    $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    // select * from rs_company_location where location_id = $loc_id
    $location = $rpQuery->get_row("SELECT * FROM `$rs_company_location` WHERE `location_id` = $loc_id");

    $schedule_group_id = 0;

    $schedule_group_timing = array();

    if( !empty( $location ) ) {

        $schedule_group_id = $location->schedule_group_id ?? 0;

        if( !empty( $schedule_group_id ) ) {

            // make sure schedule_group_id is not empty and exists in rs_schedule_groups table
            $exists_group = $rpQuery->get_row("SELECT * FROM `$rs_schedule_groups` WHERE `id` = $schedule_group_id");

            if( !empty( $exists_group ) ) {

                // get timing of schedule group
                $schedule_group_timing = $rpQuery->get_results("SELECT * FROM `$rs_schedule_group_timing` WHERE `schedule_group_id` = $schedule_group_id AND `cs_day` = '".rp_escape_sql($day)."'");

            }

        }

    }

    // get only m-d of Y-m-d format from $date_ymd
    $date_md = explode('-', $date_ymd);
    $date_md = $date_md[1] . '-' . $date_md[2];

    $result = rp_get_special_timing( $date_md, $loc_id );

    if( !empty( $schedule_group_id ) ) {

        $special_schedule_timing = rp_get_special_timing( $date_md, 'sg_' . $schedule_group_id );

    } else {

        $special_schedule_timing = array();

    }

    // If have special schedule related to location return it

    // If have special schedule related to schedule group return it

    // If have schedule group timing return it

    // Else return normal schedule

    if( !empty( $result ) ) {

        $result->cs_day = $day;

        $schedules = array($result);

    } else if( !empty( $special_schedule_timing ) ) {

        $special_schedule_timing->cs_day = $day;

        $schedules = array($special_schedule_timing);

    } else if( !empty( $schedule_group_timing ) ) {

        $schedules = $schedule_group_timing;

    } else {

        $schedules = $rpQuery->get_results("SELECT * FROM `$rs_company_schedules` WHERE `cs_day` = '".rp_escape_sql($day)."' AND `location_id_fk` = '".rp_escape_sql($loc_id)."'");

    }

    return $schedules;

}

add_filter('rp_getting_special_schedule_filter', 'rp_getting_special_schedule_filter', 10, 4);

function rp_get_main_location_id_of_company() {

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    // return first location of company

    $location = $rpQuery->get_row("SELECT `location_id` FROM `$rs_company_location` ORDER BY `location_id` ASC LIMIT 1;");

    if( !empty( $location ) ) {

        return $location->location_id;

    }

    return 0;

}

function rp_autocreate_schedule_groups_first_time() {

    // check if schedule groups already exists
    $schedule_groups = rp_get_all_schedule_groups();

    if( !empty( $schedule_groups ) ) {

        // Already exists so do not do anything

        return;

    }

    global $rpQuery;
    
    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

    $rs_schedule_groups = $rpQuery->prefix . "rs_schedule_groups";

    $rs_schedule_group_timing = $rpQuery->prefix . "rs_schedule_group_timing";

    $created_schedule_ids = array();

    // get all locations from rs_company_location

    $locations = $rpQuery->get_results("SELECT * FROM `$rs_company_location`");

    if( !empty( $locations ) ) {

        // loop through all locations and get the schedules of that location in rs_company_schedules (location_id_fk)
        foreach( $locations as $the_location_row ) {

            $location_schedules = $rpQuery->get_results("SELECT * FROM `$rs_company_schedules` WHERE `location_id_fk` = {$the_location_row->location_id}");

            // loop through all location_schedules

            $unique_id = '';

            foreach( $location_schedules as $theTimingRow ) {

                // cs_day, cs_open_timing, cs_close_timing, cs_breaktime_from, cs_breaktime_to, is_closed

                $unique_id .= $theTimingRow->cs_day . $theTimingRow->cs_open_timing . $theTimingRow->cs_close_timing . $theTimingRow->cs_breaktime_from . $theTimingRow->cs_breaktime_to . $theTimingRow->is_closed;

            }

            $unique_id = md5( $unique_id );

            if( !isset( $created_schedule_ids [ $unique_id ] ) )  {
                
                // Schedule doesn't match so need to create new schedule group
                $schedule_group_name = $the_location_row->location_name . ' (Schedule)';

                // insert into rs_schedule_groups (`name`)
                $rpQuery->query("INSERT INTO `$rs_schedule_groups` (`name`) VALUES ('".rp_escape_sql($schedule_group_name)."')");

                $schedule_group_id = $rpQuery->insert_id;

                foreach( $location_schedules as $theTimingRow ) {

                    if( $theTimingRow->is_closed == 1 ) {

                        $theTimingRow->cs_open_timing = '';

                        $theTimingRow->cs_close_timing = '';

                        $theTimingRow->cs_breaktime_from = '';

                        $theTimingRow->cs_breaktime_to = '';

                    }

                    $rpQuery->query("INSERT INTO `$rs_schedule_group_timing` (`schedule_group_id`, `cs_day`, `cs_open_timing`, `cs_close_timing`, `cs_breaktime_from`, `cs_breaktime_to`, `is_closed`) VALUES ('".rp_escape_sql($schedule_group_id)."', '".rp_escape_sql($theTimingRow->cs_day)."', '".rp_escape_sql($theTimingRow->cs_open_timing)."', '".rp_escape_sql($theTimingRow->cs_close_timing)."', '".rp_escape_sql($theTimingRow->cs_breaktime_from)."', '".rp_escape_sql($theTimingRow->cs_breaktime_to)."', '".rp_escape_sql($theTimingRow->is_closed)."')");

                }

                // assign schedule to location
                $rpQuery->query("UPDATE `$rs_company_location` SET `schedule_group_id` = '".rp_escape_sql($schedule_group_id)."' WHERE `location_id` = '".rp_escape_sql($the_location_row->location_id)."'");

                $created_schedule_ids[ $unique_id ] = $schedule_group_id;

            } else {

                // Schedule matches, so just assign to the location
                $schedule_group_id = $created_schedule_ids[ $unique_id ];

                // assign schedule to location
                $rpQuery->query("UPDATE `$rs_company_location` SET `schedule_group_id` = '".rp_escape_sql($schedule_group_id)."' WHERE `location_id` = '".rp_escape_sql($the_location_row->location_id)."'");

            }

        }

    }

}

function rp_get_location_autocomplete_suggestions( $address = '' ) {

    global $rpQuery;

    require_once RP_EL_PLUGIN_PATH . 'classes/RP_Maps_API.php';

    $maps_api = new RepairPluginPro\RP_Maps_API( $rpQuery );

    $address = $maps_api->fix_address( trim( $address ) );

    // if address is too short, skip
    if( strlen( $address ) <= 6 ) {

        return false;

    }

    $response = $maps_api->get_autocomplete_suggestions( $address );

    if( $response !== false ) {

        return $response;

    }

    return false;

}

function rp_get_location_longitude_and_latitude( $address = '' ) {

    global $rpQuery;

    require_once RP_EL_PLUGIN_PATH . 'classes/RP_Maps_API.php';

    $maps_api = new RepairPluginPro\RP_Maps_API( $rpQuery );

    $address = $maps_api->fix_address( trim( $address ) );

    // if address is too short, skip
    if( strlen( $address ) <= 6 ) {

        return false;

    }

    $response = $maps_api->get_lat_long( $address );

    if( $response !== false ) {

        return $response;

    }

    return false;

}

function rp_get_location_formatted_address( $latlng = '' ) {

    global $rpQuery;

    require_once RP_EL_PLUGIN_PATH . 'classes/RP_Maps_API.php';

    $maps_api = new RepairPluginPro\RP_Maps_API( $rpQuery );

    $response = $maps_api->get_formatted_address( $latlng );

    if( $response !== false ) {

        return $response;

    }

    return false;

}

function rp_create_maps_api_cached_table() {
      
    /* Added in Enhanced Locations V1.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_maps_api_cached = $rpQuery->prefix . "rs_maps_api_cached"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_maps_api_cached (

    `id` int(11) NOT NULL AUTO_INCREMENT,

    `input` varchar(255) NOT NULL,

    `type` varchar(255) NOT NULL,

    `response` TEXT NOT NULL,

    `output` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    
    PRIMARY KEY (`id`))

    $charset_collate";

    dbDelta( $sql );

}

add_action('rp_enhanced_locations_initialize', 'rp_create_maps_api_cached_table', 999);

function rp_alter_maps_api_cached_table() {

    // alter output column and make it varchar(2000)

    global $rpQuery;

    $table_rs_maps_api_cached = $rpQuery->prefix . "rs_maps_api_cached";

    // modify 
    $rpQuery->query("ALTER TABLE `$table_rs_maps_api_cached` MODIFY `output` varchar(2000) NOT NULL");

}

add_action('rp_enhanced_locations_initialize', 'rp_alter_maps_api_cached_table', 999);


function rp_get_lat_long_of_location( $location_id = '' ) {

    global $rpQuery;

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    // make sure location exists
    $location_row = $rpQuery->get_row("SELECT * FROM `$rs_company_location` WHERE `location_id` = '".rp_escape_sql($location_id)."'");

    if( !empty( $location_row ) ) {

        $address = trim( $location_row->location_address . ' ' . $location_row->location_city_district );

        return rp_get_location_longitude_and_latitude( $address );

    }

    return false;

}

function rp_cache_locations_latlong_if_not_already() {

    $rp_el_google_maps_api_key = rp_get_option('rp_el_google_maps_api_key', '');

    $rp_enable_search_by_nearest_location = rp_get_option('rp_enable_search_by_nearest_location', '1');

    if( empty( $rp_el_google_maps_api_key ) || $rp_enable_search_by_nearest_location != '1' ) {

        return;

    }

    global $rpQuery;

    require_once RP_EL_PLUGIN_PATH . 'classes/RP_Maps_API.php';

    $maps_api = new RepairPluginPro\RP_Maps_API( $rpQuery );

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    $locations = $rpQuery->get_results("SELECT * FROM `$rs_company_location`");

    foreach( $locations as $location_row ) {

        $address = trim( $location_row->location_address . ' ' . $location_row->location_city_district );

        $address = $maps_api->fix_address( $address );

        // if address is too short, skip
        if( strlen( $address ) <= 6 ) {

            continue;

        }

        $response = $maps_api->get_cached_response( 'lat_long', $address );

        if( empty( $response ) ) {

            $response = $maps_api->get_lat_long( $address );

            return;

        }

    }

}

add_action('rp_enhanced_locations_initialize', 'rp_cache_locations_latlong_if_not_already', 9999);

function rp_sort_locations_by_nearest_radius_distance( $input_lat_lng = array() ) {

    if( !empty( $input_lat_lng ) && is_string( $input_lat_lng ) ) {

        $input_lat_lng = explode(',', $input_lat_lng);

    }

    global $rpQuery;

    require_once RP_EL_PLUGIN_PATH . 'classes/RP_Maps_API.php';

    $maps_api = new RepairPluginPro\RP_Maps_API( $rpQuery );

    $rs_company_location = $rpQuery->prefix . "rs_company_location";

    // Get all locations
    $locations = $rpQuery->get_results("SELECT * FROM `$rs_company_location`");

    $locations = rp_el_apply_limit_to_locations( $locations );

    if( empty( $input_lat_lng ) || !is_array( $input_lat_lng ) || count( $input_lat_lng ) != 2 ) {

        return $locations;

    }

    if( !empty( $locations ) ) {

        $found_lat_lng = array();

        $unfound_lat_lng = array();

        foreach( $locations as $location_row ) {

            $location_lat_lng = rp_get_lat_long_of_location( $location_row->location_id );

            if( !empty( $location_lat_lng ) ) {

                $distance = $maps_api->find_radius_distance( $input_lat_lng[0], $input_lat_lng[1], $location_lat_lng[0], $location_lat_lng[1] );

                $location_row->distance = $distance;

                $location_row->lat_lng = implode( ',', $location_lat_lng );

                $found_lat_lng[] = $location_row;

            } else {

                $location_row->distance = 'N/A';

                $location_row->lat_lng = '';

                $unfound_lat_lng[] = $location_row;

            }

        }

        if( !empty( $found_lat_lng ) ) {

            // convert found_lat_lng to array
            $found_lat_lng = json_decode( json_encode( $found_lat_lng ), true );

            // sort by smallest distance
            usort($found_lat_lng, function($a, $b) {

                return $a['distance'] <=> $b['distance'];

            });

            // convert back to object
            $found_lat_lng = json_decode( json_encode( $found_lat_lng ) );

        }
        
        $merged = array();

        // loop through all found lat lng
        foreach( $found_lat_lng as $found ) {

            $merged[] = $found;

        }

        // loop through all unfound lat lng
        foreach( $unfound_lat_lng as $unfound ) {

            $merged[] = $unfound;

        }

        return $merged;

    }

    return array();

}

function _rp_search_nearest_location( $input_address = '', $input_lat_lng = '' ) {

    $valid_lat_lng = true;

    if( !empty( $input_lat_lng ) ) {

        // replace double spaces with single space
        $input_lat_lng = preg_replace( '/\s+/', ' ', $input_lat_lng );

        $input_lat_lng = trim( $input_lat_lng );

        // make sure it only contains 0-9, ., comma, space or -
        if( !preg_match( '/^[0-9\.\,\s\-\+]+$/', $input_lat_lng ) ) {

            $valid_lat_lng = false;

        }

    }

    if( !empty( $input_lat_lng ) && $valid_lat_lng === true ) {
        
        // need to get formatted address from lat lng
        $formatted_address = rp_get_location_formatted_address( $input_lat_lng );

        $locations = rp_sort_locations_by_nearest_radius_distance( $input_lat_lng );

    } else if( !empty( $input_address ) && is_string( $input_address ) && strlen( trim( $input_address ) ) > 6 ) {

        // need to get lat lng from formatted address
        $formatted_address = $input_address;

        $input_lat_lng = rp_get_location_longitude_and_latitude( $formatted_address );

        $locations = rp_sort_locations_by_nearest_radius_distance( $input_lat_lng );

    } else {

        $formatted_address = '';

        $locations = rp_sort_locations_by_nearest_radius_distance( array() );

    }

    return array('locations' => $locations, 'formatted_address' => $formatted_address);

}

function rp_is_enabled_location_selection_pdf() {

    $allow_location_selection = rp_get_option('rp_el_enable_location_selection_pdf', '1') == '1';

    return rp_is_active_enhanced_locations_addon() && $allow_location_selection;

}

function rp_is_enabled_ship_device_location() {

    $allow_location_selection = rp_get_option('rp_el_ship_device_choose_location', '1') == '1';

    return rp_is_active_enhanced_locations_addon() && $allow_location_selection;

}

function rp_get_nearest_location_for_customer( $customer_id_fk = 0 ) {

    if( empty( $customer_id_fk ) ) {

        return FALSE;

    }

    global $rpQuery;

    $rs_customer = $rpQuery->prefix . "rs_customer";

    // select * from rs_customer where customer_id = $customer_id_fk
    $customer = $rpQuery->get_row("SELECT * FROM `$rs_customer` WHERE `customer_id` = $customer_id_fk");

    if( !empty( $customer ) ) {

        $address = trim( $customer->customer_house_no . ' ' . $customer->customer_street_address . ' ' . $customer->customer_city . ' ' . $customer->customer_zipcode );

        return rp_get_nearest_location_for_delivery_methods( $address );

    }

    return FALSE;

}

function rp_get_nearest_location_for_delivery_methods( $input_address = '' ) {

    $rp_el_google_maps_api_key = rp_get_option('rp_el_google_maps_api_key', '');

    $rp_el_auto_assign_location = rp_get_option('rp_el_auto_assign_location', '1');

    if( !empty( $rp_el_google_maps_api_key ) && $rp_el_auto_assign_location == '1' && rp_is_active_enhanced_locations_addon() === true ) {

        $search_results = _rp_search_nearest_location( $input_address );

        $locations = $search_results['locations'];

        if( !empty( $locations ) ) {

            $location = $locations[0];

            return $location->location_id;

        }

    }

    return FALSE;

}

function rp_el_location_markup( $input_address = '', $input_lat_lng = '', $is_pdf_location_layout = false ) {

    $search_results = _rp_search_nearest_location( $input_address, $input_lat_lng );

    $locations = $search_results['locations'];

    $formatted_address = $search_results['formatted_address'];

    $rp_el_google_maps_api_key = rp_get_option('rp_el_google_maps_api_key', '');

    $rp_enable_search_by_nearest_location = rp_get_option('rp_enable_search_by_nearest_location', '1');

    $rp_el_show_location_image_on_frontend = rp_get_option('rp_el_show_location_image_on_frontend', '1');

    $search_by_location = !empty( $rp_el_google_maps_api_key ) && $rp_enable_search_by_nearest_location == '1';

    $rp_el_show_location_image_on_frontend = $rp_el_show_location_image_on_frontend == '1';

    if( rp_is_active_enhanced_locations_addon() === false ) {

        $search_by_location = false;

        $rp_el_show_location_image_on_frontend = false;

    }

    require RP_EL_PLUGIN_PATH . 'html/el_location_markup.php';

}

function rp_search_nearest_locations() {

    $rp_el_google_maps_api_key = rp_get_option('rp_el_google_maps_api_key', '');

    $rp_enable_search_by_nearest_location = rp_get_option('rp_enable_search_by_nearest_location', '1');

    if( empty( $rp_el_google_maps_api_key ) || $rp_enable_search_by_nearest_location != '1' ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Something went wrong. Please try again later.'));

        exit;

    }

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('frontend');

    $input_address = trim( $_POST['address'] ?? '' );

    $input_lat_lng = trim( $_POST['lat_lng'] ?? '' );

    $pdf_offer = trim( $_POST['pdf_offer'] ?? '0' );

    ob_start();

    $pdf_offer = $pdf_offer == '1' ? true : false;

    rp_el_location_markup( $input_address, $input_lat_lng, $pdf_offer );

    $html = ob_get_contents();

    ob_end_clean();

    echo json_encode(Array('status'=> true, 'html' => $html));

    exit;

}

function rp_get_place_autocomplete_suggestions() {

    $rp_el_google_maps_api_key = rp_get_option('rp_el_google_maps_api_key', '');

    $rp_enable_search_by_nearest_location = rp_get_option('rp_enable_search_by_nearest_location', '1');

    if( empty( $rp_el_google_maps_api_key ) || $rp_enable_search_by_nearest_location != '1' ) {

        echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Something went wrong. Please try again later.'));

        exit;

    }

    rp_verify_enhanced_location_is_active();

    rp_verify_csrf_token_for_ajax('frontend');

    $input_address = trim( $_POST['address'] ?? '' );

    $suggestions = rp_get_location_autocomplete_suggestions( $input_address );

    echo json_encode(Array('status'=> true, 'suggestions' => $suggestions));

    exit;

}

function rp_el_add_new_translation_by_release( $new_translation_groups = array() ) {

    $new_translation_groups['rp_el_plugin_translation_v_1p0'] = array(

        'please_select_any_delivery_method',

        'search_for_nearest_location',

        'approx_radius_distance',

        'browser_does_not_support_geolocation',

        'error_getting_geolocation',

        'address_is_too_short'

    );

    $new_translation_groups['rp_el_plugin_translation_v_1p1'] = array(

        'live_location_btn_text'

    );

    $new_translation_groups['rp_el_plugin_translation_v_1p2'] = array(

        'nearest_location_distance_unit_km',
        'nearest_location_distance_unit_miles'

    );

    return $new_translation_groups;

}

add_filter('rp_force_fetch_new_translation_groups', 'rp_el_add_new_translation_by_release', 12, 1);
