<?php
// Exit if accessed directly
defined( 'ABSPATH' ) || exit;
global $rpQuery;
global $wp_repair_lang;
if(!function_exists('getTheImageURL')) {
	function getTheImageURL($str = '') {
		return WP_REPAIR_PLUGIN_URL.'assets/images/'.$str;
	}
}

function rp_convert_timeslot_to_24_hours($timeslot = '') {
	$exploded = explode(' — ', $timeslot);
	$start = date('H:i', strtotime($exploded[0]));
	$end = date('H:i', strtotime($exploded[1]));
	$text = $start.' — '.$end;
	return $text;
}

function rp_is_same_timeslot($slot1 = '', $slot2 = '') {
	return rp_convert_timeslot_to_24_hours($slot1) === rp_convert_timeslot_to_24_hours($slot2);	
}

function rp_get_delivery_method_timeslots($method = '') {

	global $rpQuery;

    $rs_delivery_timeslots = $rpQuery->prefix . 'rs_delivery_timeslots';

    // get all entries from $rs_delivery_timeslots where delivery_method = $method
    $all_timeslots = $rpQuery->get_results("SELECT * FROM `$rs_delivery_timeslots` WHERE `delivery_method` = '".rp_escape_sql($method)."' ORDER BY `slot_start` ASC");

	$prepared_slots = array();

	foreach($all_timeslots as $timeslot_row) {
		$start = date('H:i', strtotime($timeslot_row->slot_start));
		$end = date('H:i', strtotime($timeslot_row->slot_end));
		$text = $start.' — '.$end;
		$prepared_slots[] = $text;
	}

	return $prepared_slots;

}

$rpPrintFormsLaters = array();


$rs_planning_and_discount = $rpQuery->prefix . "rs_planning_and_discount";

$planning_discount = $rpQuery->get_results("SELECT * FROM $rs_planning_and_discount");

$time_slots = $planning_discount[0]->booking_slots_length;

$time_slots = str_replace(strtolower(' Minutes (Default)'), '', strtolower($time_slots));

$time_slots = trim(str_replace('minutes', '', strtolower($time_slots)));

$time_slots = (int) $time_slots;

_rp_set_default_tiemzone();

$rs_orders    = $rpQuery->prefix . "rs_orders";

$rs_customer    = $rpQuery->prefix . "rs_customer";

$rs_coupons    = $rpQuery->prefix . "rs_coupons";

$rs_customer_business    = $rpQuery->prefix . "rs_customer_business";

$rs_company_location    = $rpQuery->prefix . "rs_company_location";

$rs_order_upsales    = $rpQuery->prefix . "rs_order_upsales";

$rs_upsales    = $rpQuery->prefix . "rs_upsales";

$rs_order_repairs    = $rpQuery->prefix . "rs_order_repairs";

$rs_repair    = $rpQuery->prefix . "rs_repair";

$rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

$rs_order_repair_attr    = $rpQuery->prefix . "rs_order_repair_attr";

$rs_category    = $rpQuery->prefix . "rs_category";

$rs_brand    = $rpQuery->prefix . "rs_brand";

$rs_model    = $rpQuery->prefix . "rs_model";

function rs_filter_appointment_results($results = array(), $limit = 10) {

	$pagenum = isset( $_GET['pagenum'] ) ? absint( $_GET['pagenum'] ) : 1;

	$offset   = ( $pagenum - 1 ) * $limit;

	$results = apply_filters('rs_filter_appointment_results', $results);

	$result_count = count( $results );

	$num_of_pages = ceil($result_count / $limit);

	$results = array_slice($results, $offset, $limit);

	$required_get_paramters = array( 'search', 'from_date', 'to_date', 'booking-user-type', 'booking-location', 'booking-status', 'delivery-method' );

	$get_paramters = array();

	foreach($required_get_paramters as $parameter) {

		if(isset( $_REQUEST[ $parameter ] ) && !empty( trim( $_REQUEST[ $parameter ] ) ) ) {

			$get_paramters[] = $parameter .'='. urlencode( trim( $_REQUEST[ $parameter ] ) );

		}

	}

	if(!empty($get_paramters)) {

		$get_paramters_str = '&'. join('&', $get_paramters);

	} else {

		$get_paramters_str = '';

	}

	$page_links = paginate_links( array(
		'base' => add_query_arg( 'pagenum', '%#%' ,'admin.php?page=wp_repair_appointments'.$get_paramters_str),
		'mid_size'  => 5,
		'format' => '',
		'prev_text' => __( '&laquo;', 'text-domain' ),
		'next_text' => __( '&raquo;', 'text-domain' ),
		'total' => $num_of_pages,
		'current' => $pagenum
	));

	return array( $results, $page_links, $get_paramters_str );

}



function _filter_by_from_date( $results = array() ) {

	if(isset($_REQUEST['from_date']) && !empty($_REQUEST['from_date'])) {

		$filtered = array();

		foreach($results as $row) {

			if( strpos($row->delivery_date, '-') !== 4 ) {

				$row_date_arr = explode('-', $row->delivery_date);

				// Converted d-m-Y to Y-m-d
				$row_date = $row_date_arr[2] . $row_date_arr[1] . $row_date_arr[0];

			} else {

				$row_date = $row->delivery_date;

			}


			if( strtotime( $row_date ) >= strtotime( $_REQUEST['from_date'] ) ) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_from_date');


function _filter_by_to_date( $results = array() ) {

	if(isset($_REQUEST['to_date']) && !empty($_REQUEST['to_date'])) {

		$filtered = array();

		foreach($results as $row) {

			$_delivery_method = str_replace(' ', '_', strtolower(trim($row->delivery_method)));
			if( $_delivery_method === 'ship_device' ) {
				$row->delivery_date = date('d-m-Y', strtotime($row->created_time));
				$row->delivery_time = date('H:i', strtotime($row->created_time));
			}

			if( strpos($row->delivery_date, '-') !== 4 ) {

				$row_date_arr = explode('-', $row->delivery_date);

				// Converted d-m-Y to Y-m-d
				$row_date = $row_date_arr[2] . $row_date_arr[1] . $row_date_arr[0];

			} else {

				$row_date = $row->delivery_date;

			}


			if( strtotime( $row_date ) <= strtotime( $_REQUEST['to_date'] ) ) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_to_date');



function _filter_by_user_type( $results = array() ) {

	if(isset($_REQUEST['booking-user-type']) && !empty($_REQUEST['booking-user-type'])) {

		$filtered = array();

		foreach($results as $row) {

			$row_user_type = 'Personal';

			if($row->cb_name != '') {

				$row_user_type = 'Business';

			}

			if($_REQUEST['booking-user-type'] === $row_user_type) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_user_type');


function _filter_by_location_name( $results = array() ) {

	if(isset($_REQUEST['booking-location']) && !empty($_REQUEST['booking-location'])) {

		$filtered = array();

		foreach($results as $row) {

			if( strtolower( trim( $row->location_name ) ) === strtolower( trim( $_REQUEST['booking-location'] ) ) ) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_location_name');


function _filter_by_status( $results = array() ) {

	$status_ids = array(
		'pending',
		'offered',
		'approved',
		'canceled'
	);

	if(isset( $_REQUEST['booking-status'] ) && !empty( $_REQUEST['booking-status'] )) {

		$filtered = array();

		foreach($results as $row) {

			$status_id = array_search( strtolower( $_REQUEST['booking-status'] ), $status_ids ) + 1;

			if((int) $row->status_id_fk === (int) $status_id) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_status');


function _filter_by_delivery_method( $results = array() ) {

	if(isset( $_REQUEST['delivery-method'] ) && !empty( $_REQUEST['delivery-method'] )) {

		$filtered = array();

		foreach($results as $row) {

			if( trim( strtolower( $row->delivery_method ) ) === trim( strtolower( $_REQUEST['delivery-method'] ) ) ) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_delivery_method');


function rp_add_temporary_columns_for_search($results = array()) {

	global $rpQuery;

	$rs_order_upsales    = $rpQuery->prefix . "rs_order_upsales";

	$rs_upsales    = $rpQuery->prefix . "rs_upsales";

	$rs_order_repairs    = $rpQuery->prefix . "rs_order_repairs";

	$rs_repair    = $rpQuery->prefix . "rs_repair";

	$rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

	$rs_order_repair_attr    = $rpQuery->prefix . "rs_order_repair_attr";

	$rs_category    = $rpQuery->prefix . "rs_category";

	$rs_brand    = $rpQuery->prefix . "rs_brand";

	$rs_model    = $rpQuery->prefix . "rs_model";

	$localization = $rpQuery->prefix . "rs_localization";

	$localization_data = $rpQuery->get_results("SELECT * FROM $localization");

	$decimal = $localization_data[0]->set_decimals;

	foreach($results as $key => $row) {

		$_delivery_method = str_replace(' ', '_', strtolower(trim($row->delivery_method)));
		if( $_delivery_method === 'ship_device' ) {
			$row->delivery_date = date('d-m-Y', strtotime($row->created_time));
			$row->delivery_time = date('H:i', strtotime($row->created_time));
		}

		$row_delivery_date = $row->delivery_date;

		if( strpos($row->delivery_date, '-') !== 4 ) {

			$row_date_arr = explode('-', $row->delivery_date);

			// Converted d-m-Y to Y-m-d
			$row_delivery_date = $row_date_arr[2] . $row_date_arr[1] . $row_date_arr[0];

		} else {

			$row_delivery_date = $row->delivery_date;

		}

		if( $_delivery_method === 'pickup_service' || $_delivery_method === 'repair_on_location' ) {
			$DateTime = $row->delivery_time; 
			$DateTime = explode(' — ', $DateTime);
			if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
				$newDateTime1 = date('H:i', strtotime($DateTime[0]));
				$newDateTime2 = date('H:i', strtotime($DateTime[1]));
				$newDateTime = $newDateTime1 . ' — ' . $newDateTime2;
			} else {
				$newDateTime1 = date('g:i a', strtotime($DateTime[0]));
				$newDateTime2 = date('g:i a', strtotime($DateTime[1]));
				$newDateTime = $newDateTime1 . ' — ' . $newDateTime2;
			}
		} else {
			$DateTime = $row->delivery_time; 
			if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
				$newDateTime = date('H:i', strtotime($DateTime));
			} else {
				$newDateTime = date('g:i a', strtotime($DateTime));
			}
		}

		$row_delivery_time = $newDateTime;

		$results[$key]->_date_column = $row_delivery_time . ' '. date( 'F d, Y', strtotime($row_delivery_date) );

		// Add _appointment_no_column
		$results[$key]->_appointment_no_column = '#'.$row->appointment_number;

		// Add _device_column
		$m_id = $row->m_id_fk;

		$models = $rpQuery->get_results("SELECT * FROM $rs_model WHERE m_id = '".rp_escape_sql($m_id)."'");

		if(!empty($models)) {

			$b_id = $models[0]->brand_id_fk;

			$brands = $rpQuery->get_results("SELECT * FROM $rs_brand WHERE b_id = '".rp_escape_sql($b_id)."'");

			$c_id = $models[0]->device_type; 

			$cats = $rpQuery->get_results("SELECT * FROM $rs_category WHERE c_id = '".rp_escape_sql($c_id)."'");

			$results[$key]->_device_column = $cats[0]->c_name.' '.$brands[0]->b_name.' '.$models[0]->m_name;

		} else {

			$results[$key]->_device_column = '';

		}

		// Add _repair_upsale_column 
		$o_id = $row->o_id;
		$o_upsales = $rpQuery->get_results("SELECT * FROM $rs_order_upsales WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
		$order_repairs = $rpQuery->get_results("SELECT * FROM $rs_order_repairs WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
		$or_attr = $rpQuery->get_results("SELECT * FROM $rs_order_repair_attr WHERE o_id_fk = '".rp_escape_sql($o_id)."'");

		// Has Saved Price on Appointment Creation
		$hasSaved = FALSE;
		if(!empty($o_upsales)) {
			if(!empty($o_upsales[0]->upsale_name ?? '')) {
				$hasSaved = TRUE;
			}
		}
		if(!empty($order_repairs)) {
			if(!empty($order_repairs[0]->repair_name ?? '')) {
				$hasSaved = TRUE;
			}
		}
		if(!empty($or_attr)) {
			if(!empty($or_attr[0]->repair_attr_name ?? '')) {
				$hasSaved = TRUE;
			}
		}
		if($hasSaved === FALSE) {
			$o_upsales = $rpQuery->get_results("SELECT * FROM $rs_order_upsales AS o_up JOIN $rs_upsales AS up ON o_up.upsales_id_fk = up.upsales_id WHERE o_up.o_id_fk = '".rp_escape_sql($o_id)."'");
			$order_repairs = $rpQuery->get_results("SELECT * FROM $rs_order_repairs AS o_r JOIN $rs_repair AS r ON r.r_id = o_r.r_id_fk WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
			$or_attr = $rpQuery->get_results("SELECT * FROM $rs_order_repair_attr AS o_ra JOIN  $rs_repair_attr AS ra ON ra.a_id = o_ra.ra_id_fk WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
		}

		list($order_repairs, $or_attr) = rp_translate_locked_repairs_and_attributes($order_repairs, $or_attr);

		$o_upsales = rp_translate_locked_upsales($o_upsales);

		if(($or_attr) || ($order_repairs)){ 

			if($or_attr && $or_attr[0]->r_id_fk == $order_repairs[0]->r_id_fk){

				$results[$key]->_repair_upsale_column = ($order_repairs[0]->repair_name ?? $order_repairs[0]->r_name) . ' - ' . ($or_attr[0]->repair_attr_name ?? $or_attr[0]->a_name);

			} else{

				$results[$key]->_repair_upsale_column = ($order_repairs[0]->repair_name ?? $order_repairs[0]->r_name);

			} 

		} else { 

			if($o_upsales){

				$results[$key]->_repair_upsale_column = ($o_upsales[0]->upsale_name ?? $o_upsales[0]->us_name);

			} else {

				$results[$key]->_repair_upsale_column = '';

			}
		}

		// Add _price_column
		if($decimal == 'comma'){

			$results[$key]->_price_column = str_replace('.',',',$row->total_price);

		} else {

			$results[$key]->_price_column = $row->total_price;

		}

		// Add _user_column
		$results[$key]->_user_column = $row->customer_name . ' ' . $row->customer_phone;

		// Add _user_type_column
		$results[$key]->_user_type_column = 'Personal';

		if($row->cb_name != '') {

			$results[$key]->_user_type_column = 'Business';

		}

		// Add _status_column

		$status_ids = array(
			'pending',
			'offered',
			'approved',
			'canceled'
		);

		$results[$key]->_status_column = $status_ids[ $row->status_id_fk - 1 ];

	}

	return $results;

}

function _filter_by_search_input( $results = array() ) {

	if(isset( $_REQUEST['search'] ) && !empty( trim( $_REQUEST['search'] ) )) {

		$filtered = array();


		// 1. Process every row to make sure
		// it has same column text as we want
		// to appear on front-end
		$results = rp_add_temporary_columns_for_search( $results );

		$searchable_column = array( '_appointment_no_column', '_date_column', '_device_column', '_repair_upsale_column', '_price_column', '_user_column', '_user_type_column', '_status_column' );

		// 2. Then Filter...
		foreach($results as $row) {

			$match_found = false;

			foreach( $searchable_column as $col ) {

				$column_value = $row->{$col};

				if( strpos( strtolower($column_value) , strtolower( trim( $_REQUEST['search'] ) ) ) !== FALSE ) {

					$match_found = true;

				}

			}

			if($match_found === true) {

				$filtered[] = $row;

			}

		}

		return $filtered;

	}

	return $results;

}

add_filter('rs_filter_appointment_results', '_filter_by_search_input');

$orders = $rpQuery->get_results("SELECT o.*, cu.*, c.*,cb.*,loc.* FROM $rs_orders as o 
	LEFT JOIN $rs_customer as cu ON o.customer_id_fk = cu.customer_id 
	LEFT JOIN $rs_coupons as c ON c.coupons_id = o.coupon_id_fk 
	LEFT JOIN $rs_customer_business as cb ON cu.customer_id = cb.customer_id_fk 
	LEFT JOIN $rs_company_location as loc ON loc.location_id = o.location_id_fk 
	ORDER BY o.o_date_time DESC;");

if( !empty( $orders ) ) {

	$orders = rp_sanitize_output( $orders );

}

// The line with list function below is equal to these lines:
// ----------------------------------------------
// $result = rs_filter_appointment_results($orders, 2);
// $orders = $result[0];
// $page_links = $result[1];
// $get_paramters_str = $result[2];
// ------------------------------------------------
list( $orders, $page_links, $get_paramters_str ) = rs_filter_appointment_results($orders, 20);



if (isset($_POST['deleting_appointment_id'])) {

	rp_verify_csrf_token_for_simple_form('appointments');

	$del_id = (int) $_POST['deleting_appointment_id'] ?? 0;

	$order_delivery_method = rp_find_delivery_method_of_order( $del_id );

	if( !empty( $order_delivery_method ) && $order_delivery_method != 'ship device' && $order_delivery_method != 'pdf offer' ) {

		rp_trigger_outgoing_webhook( 'scheduled_appointment_deleted', $del_id );

	} else if( !empty( $order_delivery_method ) && $order_delivery_method == 'ship device' ) {

		rp_trigger_outgoing_webhook( 'ship_device_appointment_deleted', $del_id );

	} else if( !empty( $order_delivery_method ) && $order_delivery_method == 'pdf offer' ) {

		rp_trigger_outgoing_webhook( 'pdf_offer_deleted', $del_id );

	} else {

		// Do nothing!

	}

    $rpQuery->query("DELETE FROM $rs_orders WHERE o_id='".rp_escape_sql($del_id)."'");
    echo "<script>location.replace('admin.php?page=wp_repair_appointments');</script>";
    exit();
}

$localization = $rpQuery->prefix . "rs_localization";
$company_location = $rpQuery->prefix . "rs_company_location";
$localization_data = $rpQuery->get_results("SELECT * FROM $localization");
$locations = $rpQuery->get_results("SELECT * FROM $company_location");

$localization_data = rp_sanitize_output($localization_data);
$locations = rp_sanitize_output($locations);

$decimal = $localization_data[0]->set_decimals;

$allTooltips = array();
?>
<style type="text/css">
	.wp-repair-container button.trash img{
		width: 19px;
	}
	.wp-repair-container .table tbody tr{
		background: #fbfbfd;		
	}
	.wp-repair-container .table tbody .closed td{
		border-top: 2px solid #eeeeee;
		border-bottom: 2px solid #eeeeee;
	}
	.wp-repair-container tbody tr.closed td:first-child{
		border-left: 2px solid #eeeeee;
	}
	.wp-repair-container tbody tr.closed td:last-child{
		border-right: 2px solid #eeeeee;
	}
	.update_appointment_status{
		float: right;
		padding: 8px 12px;
		font-size: 14px;
		border-radius: 5px;
		color: #fff;
		background-color: #005a9f;
		border: 0px;
		letter-spacing: 1px;
		display: inline-block;
		cursor: pointer;
		margin-right: 20px;
	}
	.edit_appointment_price{
		float: right;
		    padding: 8px 12px;
		    font-size: 14px;
		    border-radius: 5px;
		    color: #1b1b1b;
		    background-color: #f7ae00;
		    font-weight: 500;
		    border: 0px;
		    letter-spacing: 1px;
		    display: inline-block;
		    cursor: pointer;
		    margin-right: 8px;
	}
	.ui-selectmenu-menu.ui-selectmenu-open {
		top: 839.996px;
		    left: 59.0134px;
		    max-height: 250px;
		    overflow-y: auto;
		    box-shadow: 0px 0px 3px rgb(0 0 0 / 30%);
	}
	#_search_appointments {
		font-weight: 600;
		    text-transform: uppercase;
		    margin: 0px;
		    cursor: pointer;
		    border-radius: 8px;
		    padding: 10px 22px;
		    font-size: 14px;
		    width: auto;
		    letter-spacing: 0.5px;
		    border: 0px;
		    background: #1275c2;
		    color: #fff;
		    height: 44px;
		    margin-top: 15px;
		    display: inline-block;
	}
	a.rp_sortable_link {
		text-decoration: none;
		outline: 0 !important;
		box-shadow: unset !important;
		border: 0px !important;
	}
	a.rp_sortable_link:hover, 
	a.rp_sortable_link:focus, 
	a.rp_sortable_link:active {
		text-decoration: underline;
		outline: 0 !important;
		box-shadow: unset !important;
		border: 0px !important;
	}
	.navigation .tablenav-pages {
		margin: 1em;
		position: relative;
		display: inline-block;
		left: 50%;
		transform: translateX(-50%);
		float: unset;
	}
	.navigation .tablenav-pages .page-numbers {
		cursor: pointer;
	}
	.navigation .tablenav-pages .page-numbers,
	.navigation .tablenav-pages .page-numbers:active,
	.navigation .tablenav-pages .page-numbers:focus {
		padding: 5px 10px;
		background: #fafafa;
		display: inline-table;
		border: 1px solid #aaa !important;
		border-radius: 5px;
		text-decoration: none;
		min-width: 32px;
		text-align: center;
		outline: 0 !important;
		box-shadow: unset !important;
		box-sizing: border-box;
	}
	.navigation .tablenav-pages .page-numbers.current,
	.navigation .tablenav-pages .page-numbers:hover {
		display: inline-table;
		padding: 5px 10px;
		border-radius: 5px;
		text-decoration: none;
		min-width: 32px;
		background: #1275c2;
		color: #fff;
		font-weight: 700;
		border: 1px solid #1275c2 !important;
		outline: 0 !important;
		box-shadow: unset !important;
	}
	.navigation .tablenav-pages .page-numbers.dots:hover {
		cursor: text;
		background: #aaa !important;
		border: 1px solid #aaa !important;
	}
	.rp-a-normalize .wp-repair-repair-card img {
		width: 100%;
		height: auto;
	}
	.rp-a-normalize .wp-repair-repair-card .text-all {
		display: inline-table;
		padding: 0px;
		padding-left: 10px;
		line-height: 1.4em;
		float: left;
	}
	.rp-a-normalize .wp-repair-repair-card .text-all p {
		line-height: 1.4em;
		margin: 0px;
		font-size: 12px;
		color: #404040;
	}
	.rp-a-normalize .wp-repair-repair-card .text-all p.repair-name {
		font-weight: 600;
	}
	.rp-a-normalize .wp-repair-repair-card .repair-img {
		width: 30px;
		display: inline-table;
		float: left;
		position: relative;
		top: 2px;
	}
	.rp-a-normalize {
		width: 145px;
    display: inline-table;
		padding: 10px 10px;
		background-color: #fff;
		border: 1.5px solid #eee;
		border-radius: 8px;
		-webkit-transition: all 0.4s ease;
		-moz-transition: all 0.4s ease;
		-o-transition: all 0.4s ease;
		-ms-transition: all 0.4s ease;
		transition: all 0.4s ease;
	}
	.rp-a-normalize:hover,
	.rp-a-normalize:focus,
	.rp-a-normalize:active {
		border: 1.5px solid #2781c7 !important;
    box-shadow: 0 0 0 0.8px #2781c7 !important;
    border-color: #2781c7 !important;
	}
	.clearfix {
		padding: 10px;
		clear: both;
	}
	.clearfix:after {
		content: "";
		display: table;
		clear: both;
	}
	.wp-repair-col-12.filters {
		margin-top: 25px;
	}
	.wp-repair-container h1.main-heading {
		padding-left: 3px;
	}
	.wp-repair-container .search-filter .search-field {
		width: 617px;
	}
	.wp-repair-container .search-filter {
		margin-right: 2px;
	}
	#_search_appointments {
		font-weight: 600;
    text-transform: uppercase;
    margin: 0px;
    cursor: pointer;
    border-radius: 5px;
    padding: 8px 22px;
    font-size: 14px;
    width: auto;
    letter-spacing: 0.5px;
    border: 0px;
    background: #1275c2;
    color: #fff;
    height: 42px;
    height: 36px;
    margin-top: 15px;
    display: inline-block;
	}
	.more-appointment-items {
		border: 1px solid #1275c2;
    background: #fff;
    color: #1275c2;
    padding-right: 6px;
    padding-left: 6px;
    margin-left: 3px;
    font-weight: 600;
    border-radius: 4px;
    font-size: 14px;
	}
	.appointment-number {
		color: #adadad;
		margin-left: 5px;
		width: 108px;
		text-align: center;
		display: inline-table;
	}
	.customer-details-popup table {
		font-size: 15px;
		width: 100%;
		text-align: left;
		padding-bottom: 5px;
	}
	.customer-details-popup table th,
	.customer-details-popup table td {
		padding: 5px;
	}
	.customer-details-popup .va-modal-title {
		padding-left: 5px;
	}
	.ui-selectmenu-disabled {
		opacity: 0.65 !important;
	}
	.wp-repair-inner-col .full-repair-name {
		display: inline-block;
	}
	.wp-repair-inner-col .percent-off-badge {
		font-weight: 700;
		font-size: 10px;
		background: #fff;
		color: #1275c2;
		border: 1px solid #1275c2;
		line-height: 1.1em;
		text-transform: uppercase;
		padding: 2px 5px 2px 5px !important;
		box-sizing: border-box;
		border-radius: 6px;
		vertical-align: middle;
		position: relative;
		top: -1px;
		white-space: nowrap;
		display: inline-block;
		height: auto;
		width: auto;
	}
	.wp-repair-inner-col .full-repair-name.has-discounted {
		margin-right: 5px;
	}
</style>
<div class="wp-repair-container">
	<div class="wp-repair-row">
		<div class="wp-repair-col-12">
			<div class="c-header">
				<form action="" method="GET">
					<input type="hidden" name="page" value="wp_repair_appointments">
					<div class="wp-repair-row">
						<div class="wp-repair-col-12">
							<h1 class="main-heading">Appointments</h1>
						</div>
						<div class="wp-repair-col-12 filters">
							<p class="filter-by-text">From Date</p>
							<input type="date" name="from_date" class="from_date" style="width:150px;" value="<?php echo trim( $_REQUEST['from_date'] ?? '' ); ?>">
							<p class="filter-by-text">To Date</p>
							<input type="date" name="to_date" class="to_date" style="width:150px;" value="<?php echo trim( $_REQUEST['to_date'] ?? '' ); ?>">
							<p class="filter-by-text">Type</p>
							<select name="booking-user-type" class="type" style="width:150px;">
								<?php
								$booking_user_type_options = array(
									'0' => 'Show All',
									'Personal' => 'Personal',
									'Business' => 'Business'
								);
								foreach ($booking_user_type_options as $_key => $user_type_option) {
									if($_key === trim($_REQUEST['booking-user-type'] ?? '')) {
										echo sprintf('<option value="%s" selected="selected">%s</option>', $_key, $user_type_option);
									} else {
										echo sprintf('<option value="%s">%s</option>', $_key, $user_type_option);
									}
								}
								?>
							</select>
							<p class="filter-by-text">Location</p>
							<select name="booking-location" class="location" style="width:150px;">
								<?php
								$location_options = array(
									'0' => 'Location'
								);
								if($locations) {
									foreach ($locations as $loc) {
										$location_options[$loc->location_name] = $loc->location_name;
									}
								}
								foreach ($location_options as $_key => $_location_name) {
									if($_key === trim($_REQUEST['booking-location'] ?? '')) {
										echo sprintf('<option value="%s" selected="selected">%s</option>', $_key, $_location_name);
									} else {
										echo sprintf('<option value="%s">%s</option>', $_key, $_location_name);
									}
								}
								?>
							</select>
							<p class="filter-by-text">Status</p>
							<select name="booking-status" class="status" style="width:150px;">
								<?php
								$status_options = array(
									'0' => 'Status',
									'Pending' => 'Pending',
									'Offered' => 'Offered',
									'Approved' => 'Approved',
									'Canceled' => 'Canceled'
								);
								foreach ($status_options as $_key => $_status_option) {
									if($_key === trim($_REQUEST['booking-status'] ?? '')) {
										echo sprintf('<option value="%s" selected="selected">%s</option>', $_key, $_status_option);
									} else {
										echo sprintf('<option value="%s">%s</option>', $_key, $_status_option);
									}
								}
								?>
							</select>
							<p class="filter-by-text">Method</p>
							<select name="delivery-method" class="dmethod" style="width:150px;">
								<?php
								$dmethod_options = array(
									'' => 'Method',
									'Come By Our Store' => 'Come By Our Store',
									'Ship Device' => 'Ship Device',
									'Pickup Service' => 'Pickup Service',
									'Repair on Location' => 'Repair on Location',
									'PDF Offer' => 'PDF Offer'
								);
								foreach ($dmethod_options as $_key => $_dmethod_option) {
									if($_key === trim($_REQUEST['delivery-method'] ?? '')) {
										echo sprintf('<option value="%s" selected="selected">%s</option>', $_key, $_dmethod_option);
									} else {
										echo sprintf('<option value="%s">%s</option>', $_key, $_dmethod_option);
									}
								}
								?>
							</select>
						</div>
						<div class="wp-repair-col-12">
							<div class="search-filter">
								<span class="search-icon"><i class="fas fa-search"></i></span>
								<input placeholder="Find appointment or customer" type="search" name="search" class="search-field" value="<?php echo trim( $_REQUEST['search'] ?? '' ); ?>">
							</div>
							<button id="_search_appointments" type="submit" name="_search_appointments">Search <i class="fas fa-caret-right"></i></button>
							<?php
							if(!empty($get_paramters_str)) {
								?>
								<a class="rp_sortable_link" style="display: inline-block; margin-left: 15px; font-size: 15px;" href="?page=wp_repair_appointments"><i class="fas fa-undo-alt"></i> Reset Filters</a>
								<?php
							}
							?>
						</div>
					</div>
				</form>
			</div>
			<div class="table-responsive">
				<form method="post" action="" id="appointment_parent_form"> 
				<?php
				rp_csrf_token_input_field('appointments');
				?>
				<table class="table">
					<thead>
						<tr>
							<th style="min-width: 170px;"></th>
							<th style="min-width: 115px;">Date</th>
							<th style="min-width: 120px;">Device</th>
							<th style="min-width: 160px;">Repair/Upsale</th>
							<th style="width: 50px;">Price</th>
							<th style="width: 130px;">Name</th>
							<th style="width: 40px;">Type</th>
							<th style="width: 160px;">Status</th>
							<th style="width: 35px;"></th>
						</tr>
					</thead>
					<tbody>
						<?php if($orders){foreach($orders as $order){
							$o_id = $order->o_id;
							$o_upsales = $rpQuery->get_results("SELECT * FROM $rs_order_upsales WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
							$order_repairs = $rpQuery->get_results("SELECT * FROM $rs_order_repairs WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
							$or_attr = $rpQuery->get_results("SELECT * FROM $rs_order_repair_attr WHERE o_id_fk = '".rp_escape_sql($o_id)."'");


							$appointment_por_repairs = rp_get_por_repairs_by_order_id( $o_id );

							$appointment_por_repair_attrs = rp_get_por_repair_attr_by_order_id( $o_id );

							// Has Saved Price on Appointment Creation
							$hasSaved = FALSE;
							if(!empty($o_upsales)) {
								if(!empty($o_upsales[0]->upsale_name ?? '')) {
									$hasSaved = TRUE;
								}
							}
							if(!empty($order_repairs)) {
								if(!empty($order_repairs[0]->repair_name ?? '')) {
									$hasSaved = TRUE;
								}
							}
							if(!empty($or_attr)) {
								if(!empty($or_attr[0]->repair_attr_name ?? '')) {
									$hasSaved = TRUE;
								}
							}
							if($hasSaved === FALSE) {
								$o_upsales = $rpQuery->get_results("SELECT * FROM $rs_order_upsales AS o_up JOIN $rs_upsales AS up ON o_up.upsales_id_fk = up.upsales_id WHERE o_up.o_id_fk = '".rp_escape_sql($o_id)."'");
								$order_repairs = $rpQuery->get_results("SELECT * FROM $rs_order_repairs AS o_r JOIN $rs_repair AS r ON r.r_id = o_r.r_id_fk WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
								$or_attr = $rpQuery->get_results("SELECT * FROM $rs_order_repair_attr AS o_ra JOIN  $rs_repair_attr AS ra ON ra.a_id = o_ra.ra_id_fk WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
							}

							list($order_repairs, $or_attr) = rp_translate_locked_repairs_and_attributes($order_repairs, $or_attr);

							$o_upsales = rp_translate_locked_upsales($o_upsales);

							$m_id = $order->m_id_fk;
							$models = $rpQuery->get_results("SELECT * FROM $rs_model WHERE m_id = '".rp_escape_sql($m_id)."'");
							$b_id = $models[0]->brand_id_fk;
							$brands = $rpQuery->get_results("SELECT * FROM $rs_brand WHERE b_id = '".rp_escape_sql($b_id)."'");
							$c_id = $models[0]->device_type; 
							$cats = $rpQuery->get_results("SELECT * FROM $rs_category WHERE c_id = '".rp_escape_sql($c_id)."'");

							$o_upsales = rp_sanitize_output($o_upsales);
							$order_repairs = rp_sanitize_output($order_repairs);
							$or_attr = rp_sanitize_output($or_attr);	
							$models = rp_sanitize_output($models);
							$brands = rp_sanitize_output($brands);
							$cats = rp_sanitize_output($cats);
							$_delivery_method = str_replace(' ', '_', strtolower(trim($order->delivery_method)));
							if( $_delivery_method === 'ship_device' ) {
								$order->delivery_date = date('d-m-Y', strtotime($order->created_time));
								$order->delivery_time = date('H:i', strtotime($order->created_time));
							}
							list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $order->o_id );
							?>
						<tr class="closed">
							<td><span class="appointment-number">#<?php echo ($order->appointment_number ?? '--') ?></span> <button type="button" class="plus-icon see-more"><i class="fas fa-caret-down"></i></button></td>
							<td><span class="date-day today"></span>
							<?php 
							if( $_delivery_method === 'pickup_service' || $_delivery_method === 'repair_on_location' ) {
								$DateTime = $order->delivery_time; 
								$DateTime = explode(' — ', $DateTime);
								if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
									$newDateTime1 = date('H:i', strtotime($DateTime[0]));
									$newDateTime2 = date('H:i', strtotime($DateTime[1]));
								} else {
									$newDateTime1 = date('g:i a', strtotime($DateTime[0]));
									$newDateTime2 = date('g:i a', strtotime($DateTime[1]));
								}
								?>
								<span class="date-time" style="font-size: 15px;">
								<?php
								if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
									echo $newDateTime1 . ' — ' . $newDateTime2;
								} else {
									echo $newDateTime1 . ' —<br>' . $newDateTime2;
								}
								?>
								</span>
								<?php
							} else {
								$DateTime = $order->delivery_time; 
								if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
									$newDateTime = date('H:i', strtotime($DateTime));
								} else {
									$newDateTime = date('g:i a', strtotime($DateTime));
								}
								?>
								<span class="date-time">
								<?php
								echo $newDateTime;
								?>
								</span>
								<?php
							}
							?>
							<br>
							<span class="date-when" data-date="<?php echo ($order->delivery_date ?? ''); ?>"><?php 

							if(!empty($order->delivery_date) && trim(strtolower($order->delivery_date)) !== 'undefined') {

								$date = date_create($order->delivery_date); 
								echo date_format($date,"F d, Y");

							} else {

								echo '--Not Selected--';

							}

							?></span>
							</td>
							<td><span class="dark-text"><?php echo ($cats[0]->c_name ?? '').' '.($brands[0]->b_name ?? '')?></span><br><span class="light-text uppercase"><?php echo $models[0]->m_name;?></span></td>
							<td><span class="dark-text">
								<?php 
								if(($or_attr) || ($order_repairs)){ 

									if($or_attr && $or_attr[0]->r_id_fk == $order_repairs[0]->r_id_fk){

										echo ($order_repairs[0]->repair_name ?? $order_repairs[0]->r_name) . ' - ' . ($or_attr[0]->repair_attr_name ?? $or_attr[0]->a_name);

									} else{

										echo ($order_repairs[0]->repair_name ?? $order_repairs[0]->r_name);

									} 

								} else { 

									if($o_upsales){

										echo ($o_upsales[0]->upsale_name ?? $o_upsales[0]->us_name);

									} 
								}

								$totalMore = 0;

								if( !empty( $order_repairs ) ) {
									$totalMore += count($order_repairs);
								}

								if( !empty( $o_upsales ) ) {
									$totalMore += count($o_upsales);
								}

								$totalMore = ($totalMore - 1);

								if( $totalMore > 0 ) {
									echo ' <span class="more-appointment-items">+'.$totalMore.'</span>';
								}

							?></span><br><span class="light-text"></span></td>
							<td>
								<span class="appointment-price">
									<?php 
									if($decimal == 'comma'){
										$total_price = str_replace('.',',',$order->total_price);
									}else{
										$total_price = $order->total_price;					
									}
									echo $total_price;
									?> 
								</span><br><span class="no-visible">--</span></td>
							<td><span class="dark-text"><?php echo $order->customer_name?></span><br><span class="light-text"><?php echo $order->customer_phone?></span></td>
							<td><?php 
							if($order->cb_name != ''){

								echo '<span class="account-type business"><i class="fas fa-briefcase"></i></span>';

							} else {

								echo '<span class="account-type business"><i class="far fa-user-circle"></i></span>';

							} ?></td>
							<td>
								<div class="custom-select booking_status">
								  	<select name="booking_status" class="booking-status">
								    	<option value="<?php if($order->status_id_fk == 1){echo "pending";}else if($order->status_id_fk == 2){echo "offered";}else if($order->status_id_fk == 3){echo "approved";}else{echo 'canceled';}?>" selected="selected"><?php if($order->status_id_fk == 1){echo "Pending";}else if($order->status_id_fk == 2){echo "Offered";}elseif($order->status_id_fk == 3){echo "Approved";}else{echo 'Canceled';}?></option>
								    	<option value="<?php if($order->status_id_fk == 1){echo "offered";}else if($order->status_id_fk == 2){echo 'approved';}else if($order->status_id_fk == 3){echo "canceled";}else if($order->status_id_fk == 4){echo "pending";}?>"><?php if($order->status_id_fk == 1){echo "Offered";}else if($order->status_id_fk == 2){echo "Approved";}else if($order->status_id_fk == 3){echo "Canceled";}else if($order->status_id_fk == 4){echo "Pending";}?></option>
								    	<option value="<?php if($order->status_id_fk == 1){echo "approved";}else if($order->status_id_fk == 2){echo "canceled";}else if($order->status_id_fk == 3){echo 'pending';}else if($order->status_id_fk == 4){echo "offered";}?>"><?php if($order->status_id_fk == 1){echo "Approved";}else if($order->status_id_fk == 2){echo "Canceled";}else if($order->status_id_fk == 3){echo "Pending";}else if($order->status_id_fk == 4){echo "Offered";}?></option>
								    	<option value="<?php if($order->status_id_fk == 1){echo "canceled";}else if($order->status_id_fk == 2){echo "pending";}else if($order->status_id_fk == 3){echo 'offered';}else if($order->status_id_fk == 4){echo "approved";}?>"><?php if($order->status_id_fk == 1){echo "Canceled";}else if($order->status_id_fk == 2){echo "Pending";}else if($order->status_id_fk == 3){echo "Offered";}else if($order->status_id_fk == 4){echo "Approved";}?></option>
									</select>
								</div>
								<input type="hidden" name="o_id" value="<?php echo $order->o_id?>">
								<input type="hidden" class="loc" value="<?php echo $order->location_name; ?>">
								<input type="hidden" class="type_name" value="<?php if($order->cb_name != ''){echo "Business";}else{echo "Personal";} ?>">
							</td>
								<td><button type="submit" name="delete_appointment" data-oid="<?php echo $order->o_id?>" class="trash"><img src="<?php echo WP_REPAIR_PLUGIN_URL.'assets/images/garbage.svg'; ?>"></button></td>
							</tr>
							<tr class="more-details">
								<td colspan="9">
									<div class="details-container">
										<div class="wp-repair-inner-col section-1">
											<p class="heading move-to">Move To</p>
											<div class="custom-select delivery_date <?php
											if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
												echo 'increase-width';
											}
											?>">
												<?php
												if( $_delivery_method == 'ship_device' ) {
													$parsedDate = date('Y-m-d', strtotime($order->delivery_date));
													?>
													<select name="delivery_date" disabled="disabled">
														<option value="<?php echo $parsedDate; ?>" selected="selected"><?php echo $parsedDate; ?></option>
													</select>
													<?php
												} else {
													?>
													<select name="delivery_date">
														<?php 
															$date = date('Y-m-d'); 
															for ($i = 0; $i < 10; $i++){
															$newdate=date('Y-m-d', strtotime($date.' +'.$i.' days'));?>
															<?php
															if(!empty($order->delivery_date) && trim(strtolower($order->delivery_date)) !== 'undefined' && $newdate === date('Y-m-d', strtotime($order->delivery_date))) {
																?>
																<option selected="selected" value="<?php echo $newdate;?>"><?php echo $newdate;?></option>
																<?php
															} else {
																?>
																<option value="<?php echo $newdate;?>"><?php echo $newdate;?></option>
																<?php
															}
															?>
														<?php }?>
													</select>
													<?php
												}
												?>
											</div>
											<div class="custom-select delivery_time <?php
											if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
												echo 'increase-width';
											}
											?>">
												<?php
												if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
													?>
													<select name="delivery_time">
														<?php 
															$timeslots = rp_get_delivery_method_timeslots($_delivery_method);
															foreach($timeslots as $time){?>
																<?php
																if(rp_is_same_timeslot($time, $order->delivery_time) === TRUE) {
																	?>
																	<option selected="selected" value="<?php echo $time;?>"><?php echo $time;?></option>
																	<?php
																} else {
																	?>
																	<option value="<?php echo $time;?>"><?php echo $time;?></option>
																	<?php
																}
																?>
															<?php }?>
													</select>
													<?php
												} else if( $_delivery_method == 'ship_device' ) {
													if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
														$parsedTime = date('H:i', strtotime($order->delivery_time));
													} else {
														$parsedTime = date('g:i a', strtotime($order->delivery_time));
													}
													?>
													<select name="delivery_time" disabled="disabled">
														<option value="<?php echo $parsedTime; ?>" selected="selected"><?php echo $parsedTime; ?></option>
													</select>
													<?php
												} else {
													?>
													<select name="delivery_time">
														<?php 
															$start = new DateTime('00:00:00');
															$end = new DateTime('23:59:59');
															$from_time = $start->format('H:i:s');
															$to_time = $end->format('H:i:s');

															$minutes   =  round(abs(strtotime($to_time) - strtotime($from_time)) / 60,2);
															for ($x = 0; $x <= $minutes; $x+=30) {
																$add_time = strtotime($from_time . " +$x minutes");
																if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
																	$slot_time[] = date('H:i',$add_time);
																} else {
																	$slot_time[] = date('g:i a',$add_time);
																}
															}
															foreach($slot_time as $time){?>
																<?php
																if(strtotime($time) === strtotime($order->delivery_time)) {
																	?>
																	<option selected="selected" value="<?php echo $time;?>"><?php echo $time;?></option>
																	<?php
																} else {
																	?>
																	<option value="<?php echo $time;?>"><?php echo $time;?></option>
																	<?php
																}
																?>
															<?php }?>
													</select>
													<?php
												}
												?>
											</div>
										</div>
										<div class="wp-repair-inner-col section-2">
											<div class="tr">
												<p class="heading">Type</p>
												<p class="text"><?php echo $cats[0]->c_name?></p>
											</div>
											<div class="tr">
												<p class="heading">Brand</p>
												<p class="text"><?php echo $brands[0]->b_name?></p>
											</div>
											<div class="tr">
												<p class="heading">Model</p>
												<p class="text"><?php echo $models[0]->m_name;?></p>
											</div>
											<div class="tr">
												<p class="heading">Color</p>
												<p class="text"><?php echo $order->color;?>
													<!-- <div class="color-box"></div> --> 
												</p>
											</div>
										</div>
										<div class="wp-repair-inner-col section-3">
											<?php 
if($order_repairs){
foreach($order_repairs as $rp){
	$rid = $rp->r_id_fk;
	$itReallyWorked = FALSE;
	if($or_attr){ 
		foreach($or_attr as $ra){
			$r_id = $ra->r_id_fk;
			if($rid == $r_id){ 
				$itReallyWorked = TRUE; 
				$repairName = ($rp->repair_name ?? $rp->r_name) . ' - ' . ($ra->repair_attr_name ?? $ra->a_name);
				$is_discounted_price = '';
				if( property_exists( $ra, 'repair_attr_price' ) ) {
					list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $ra->ra_id_fk, 'attr' );
					$repairPrice = $discounted_price;
				} else {
					$repairPrice = $ra->a_price;
				}
				if( rp_appointment_is_dynamic_price( $ra ) === TRUE ) {
					// add info icon next to it
					$tooltip_id = 'tooltip_' . uniqid();
					$allTooltips[] = array(
						'id' => $tooltip_id,
						'data' => ($ra->pricing_info),
						'row_type' => 'attr',
						'row_id' => $ra->ora_id,
						'repair_name' => $repairName
					);
					$repairName = $repairName . ' <span class="rp-tooltip" data-tooltip-content="#'.$tooltip_id.'"><i class="fas fa-info-circle" aria-hidden="true"></i></span>';
				}
				?>
				<div class="tr<?php if( rp_check_is_por_repair_attr( $ra->ra_id_fk, $appointment_por_repair_attrs ) ) { echo ' is_por_repair'; } ?>" data-rid="<?php echo $rid; ?>" data-aid="<?php echo $ra->ra_id_fk; ?>">
					<p class="text desc"><span class="full-repair-name <?php if( $is_discounted_price == '1' ) { echo 'has-discounted'; } ?>"><?php echo $repairName;?></span><?php
					if( $is_discounted_price == '1' ) {
						if( $rpCart->get_combo_discount_percentage() > 0 ) {
							echo '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
						} else {
							echo '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), '<span class="percent-off-currency">'.$localization_data[0]->set_currency.'</span>', FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
						}
					}
					?></p>
					<p class="text price">
						<?php if($decimal == 'comma'){
							$a_price = str_replace('.',',',$repairPrice);
						}else{
							$a_price = $repairPrice;					
						}
						echo $a_price;?> 
					</p>
				</div>
			<?php }
		}
		if($itReallyWorked === FALSE) {
			$itReallyWorked = TRUE;
			$repairName = $rp->repair_name ?? $rp->r_name;
			$is_discounted_price = '';
			if( property_exists( $rp, 'repair_price' ) ) {
				list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $rp->r_id_fk, 'repair' );
				$repairPrice = $discounted_price;
			} else {
				$repairPrice = $rp->r_price;
			}
			if( rp_appointment_is_dynamic_price( $rp ) === TRUE ) {
				// add info icon next to it
				$tooltip_id = 'tooltip_' . uniqid();
				$allTooltips[] = array(
					'id' => $tooltip_id,
					'data' => ($rp->pricing_info),
					'row_type' => 'repair',
					'row_id' => $rp->or_id,
					'repair_name' => $repairName
				);
				$repairName = $repairName . ' <span class="rp-tooltip" data-tooltip-content="#'.$tooltip_id.'"><i class="fas fa-info-circle" aria-hidden="true"></i></span>';
			}
			?>
			<div class="tr<?php if( rp_check_is_por_repair( $rid, $appointment_por_repairs ) ) { echo ' is_por_repair'; } ?>" data-rid="<?php echo $rid; ?>" data-aid="">
				<p class="text desc"><span class="full-repair-name <?php if( $is_discounted_price == '1' ) { echo 'has-discounted'; } ?>"><?php echo $repairName;?></span><?php
					if( $is_discounted_price == '1' ) {
						if( $rpCart->get_combo_discount_percentage() > 0 ) {
							echo '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
						} else {
							echo '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), '<span class="percent-off-currency">'.$localization_data[0]->set_currency.'</span>', FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
						}
					}
					?></p>
				<p class="text price">
					<?php if($decimal == 'comma'){
						$r_price = str_replace('.',',',$repairPrice);
					}else{
						$r_price = $repairPrice;					
					}
					echo $r_price;?>
				</p>
			</div>
			<?php 
		}
	}else{
		if($itReallyWorked === FALSE) {
			$itReallyWorked = TRUE;
			$repairName = $rp->repair_name ?? $rp->r_name;
			$is_discounted_price = '';
			if( property_exists( $rp, 'repair_price' ) ) {
				list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $rp->r_id_fk, 'repair' );
				$repairPrice = $discounted_price;
			} else {
				$repairPrice = $rp->r_price;
			}
			if( rp_appointment_is_dynamic_price( $rp ) === TRUE ) {
				// add info icon next to it
				$tooltip_id = 'tooltip_' . uniqid();
				$allTooltips[] = array(
					'id' => $tooltip_id,
					'data' => ($rp->pricing_info),
					'row_type' => 'repair',
					'row_id' => $rp->or_id,
					'repair_name' => $repairName
				);
				$repairName = $repairName . ' <span class="rp-tooltip" data-tooltip-content="#'.$tooltip_id.'"><i class="fas fa-info-circle" aria-hidden="true"></i></span>';
			}
			?>
			<div class="tr<?php if( rp_check_is_por_repair( $rid, $appointment_por_repairs ) ) { echo ' is_por_repair'; } ?>" data-rid="<?php echo $rid; ?>" data-aid="">
				<p class="text desc"><span class="full-repair-name <?php if( $is_discounted_price == '1' ) { echo 'has-discounted'; } ?>"><?php echo $repairName;?></span><?php
					if( $is_discounted_price == '1' ) {
						if( $rpCart->get_combo_discount_percentage() > 0 ) {
							echo '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
						} else {
							echo '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), '<span class="percent-off-currency">'.$localization_data[0]->set_currency.'</span>', FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
						}
					}
					?></p>
				<p class="text price">
					<?php if($decimal == 'comma'){
						$r_price = str_replace('.',',',$repairPrice);
					}else{
						$r_price = $repairPrice;					
					}
					echo $r_price;?>
				</p>
			</div>
			<?php 
		}
} }
											}?>
											<?php 
											if($o_upsales){ foreach($o_upsales as $up){
												$upsaleName = $up->upsale_name ?? $up->us_name;
												$upsalePrice = $up->upsale_price ?? $up->us_new_price;
												?>
												<div class="tr">
													<p class="text desc"><?php echo $upsaleName;?></p>
													<p class="text price">
														<?php if($decimal == 'comma'){
															$up_price = str_replace('.',',',$upsalePrice);
														}else{
															$up_price = $upsalePrice;					
														}
														echo $up_price;?> 
													</p>
												</div>
											<?php } }
											if(($order->dm_cost ?? 0) != 0 && ($order->dm_cost ?? 0) != 0.00) {
												?>
												<div class="tr">
													<p class="text desc"><?php
													$dm_tkey = rp_get_delivery_method_translation_key($order->delivery_method);
													if( !empty( $dm_tkey ) && !empty( $wp_repair_lang[$dm_tkey] ?? '' ) ) {
														echo $wp_repair_lang[$dm_tkey];
													} else {
														echo str_replace('_', ' ', $order->delivery_method);
													}
													?></p>
													<p class="text price">
														<?php 
														$dm_cost = $order->dm_cost ?? 0;
														if($decimal == 'comma'){
															$dm_cost = str_replace('.',',',$dm_cost);
														}else{
															$dm_cost = $dm_cost;					
														}
														echo $dm_cost;?> 
													</p>
												</div>
												<?php
											}
											?>
											<div class="separator"></div>
											<?php
											$needSeparator = FALSE;
											$order->combo_discount = $calculations['combo_discount'];
											if( !empty( $order->combo_discount ) && ((float) $order->combo_discount != 0 ) ) {
												$needSeparator = TRUE;
												?>
												<div class="tr">
													<p class="text desc light">Combo-discount</p>
													<p class="text price offer">
														<?php if($decimal == 'comma'){
															$combo = str_replace('.',',',$order->combo_discount);
														}else{
															$combo = $order->combo_discount;					
														}
														echo $combo;?>
													</p>
												</div>
												<?php

											}
											if( !empty( $order->coupons_amount ) && ((float) $order->coupons_amount != 0 ) ) {
												$needSeparator = TRUE;
												?>
												<div class="tr">
													<p class="text desc light"><?php if($order->coupons_code != null){echo 'Coupon';}
													if($order->coupons_type == 'percent'){
														echo ' ('.$order->coupons_amount.'%)';
													}
													?> <strong><?php echo $order->coupons_code?></strong></p>
													<p class="text price offer">
														<?php if($decimal == 'comma'){
															$coupon = str_replace('.',',',$order->coupons_amount);
														}else{
															$coupon = $order->coupons_amount;					
														}
														if($order->coupons_type == 'percent'){
															$coupon = $order->coupon_amount;  // this is saved in rs_orders
														}
														echo $coupon;?>
													</p>
												</div>
												<?php

											}
											if( $needSeparator === TRUE ) {

												?>
												<div class="separator"></div>
												<?php

											}
											?>
											<div class="tr total">
												<p class="text desc">Total</p>
												<p class="text price">
													<?php if($decimal == 'comma'){
														$total_p = str_replace('.',',',$order->total_price);
													}else{
														$total_p = $order->total_price;					
													}
													echo $total_p;?>
												</p>
											</div>
										</div>
										<div class="wp-repair-inner-col section-4">
											<div class="tr">
												<p class="heading">Name</p>
												<p class="text"><?php echo $order->customer_name?></p>
											</div>
											<div class="tr">
												<p class="heading">Email</p>
												<p class="text"><?php echo $order->customer_email?></p>
											</div>
											<div class="tr">
												<p class="heading">Phone</p>
												<p class="text"><?php echo $order->customer_phone?></p>
											</div>
											<?php
											if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
												?>
												<div class="tr">
													<p class="heading">House No.</p>
													<p class="text"><?php echo $order->customer_house_no;?></p>
												</div>
												<?php
											}
											if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
												?>
												<div class="tr">
													<p class="heading">Street</p>
													<p class="text"><?php echo $order->customer_street_address;?></p>
												</div>
												<?php
											}
											if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
												?>
												<div class="tr">
													<p class="heading">City</p>
													<p class="text"><?php echo $order->customer_city;?></p>
												</div>
												<?php
											}
											if( $_delivery_method == 'pickup_service' || $_delivery_method == 'repair_on_location' ) {
												?>
												<div class="tr">
													<p class="heading">Zipcode</p>
													<p class="text"><?php echo $order->customer_zipcode;?></p>
												</div>
												<?php
											}
											?>
											<?php if($order->cb_name != ''){?>
												<div class="tr">
													<p class="heading">Company</p>
													<p class="text"><?php echo $order->cb_name;?></p>
												</div>
											<?php }?>
											<?php
											if($order->delivery_method !== 'Pdf Offer') {
												?>
												<div class="tr">
													<p class="heading">Method</p>
													<p class="text"><?php echo $order->delivery_method;?></p>
												</div>
												<?php
											}
											if($_delivery_method === 'pickup_service' || $_delivery_method === 'repair_on_location') {
												?>
												<div class="tr">
													<p class="heading">Location</p>
													<p class="text notes">
														<?php echo $order->location_name; ?>	
													</p>
												</div>
												<?php
											} else {
												?>
												<div class="tr">
													<p class="heading">Location</p>
													<p class="text notes">
														<?php echo $order->location_name; ?>	
													</p>
												</div>
												<?php
											}
											?>
											<div class="tr">
												<p class="heading">Notes</p>
												<p class="text notes"><?php echo $order->notes?></p>
											</div>
										</div>
										<div class="clearfix">
											<?php
												$delivery_method = str_replace(' ', '_', strtolower(trim($order->delivery_method)));

												if($delivery_method !== 'ship_device' && $delivery_method !== 'pdf_offer') {
													$location = str_replace(array('<br>', "\n", "\r"), ', ', $order->location_name . ', ' . $order->location_address . ', ' . $order->location_city_district);

													if( $delivery_method === 'come_by_our_store' ) {

														$start_time = date('Y-m-d H:i', strtotime($order->delivery_date . ' '. $order->delivery_time));

														$end_time = date('Y-m-d H:i', strtotime($order->delivery_date . ' '. $order->delivery_time . ' +'.$time_slots.' minutes'));

													} else {

														$exploded_time = explode(' — ', $order->delivery_time);

														$start_time = date('Y-m-d H:i', strtotime($order->delivery_date . ' '. $exploded_time[0]));

														$end_time = date('Y-m-d H:i', strtotime($order->delivery_date . ' '. $exploded_time[1]));

													}

													$site_name = $order->customer_name;
													$appointment_details = 'You have an appointment with '.$site_name;

													if(empty($site_name)) {
														$site_name = 'Appointment';
														$appointment_details = 'You have an appointment';
													} else {
														$site_name = 'Appointment with '.$site_name;
													}

													$calendar_data = array(

														'text' => $site_name,
								
														'details' => $appointment_details,
								
														'location' => $location,
								
														'start_time' => $start_time,
								
														'end_time' => $end_time
								
													);

													$g_calendar_link = rp_create_google_calendar_link($calendar_data);
													$rpPrintFormsLaters[$order->o_id] = $calendar_data;
													?>
													<a class="rp-a-normalize" target="_blank" href="<?php echo $g_calendar_link ?>">
														<div class="wp-repair-repair-card">
															<div class="repair-img">
																<img src="<?php echo getTheImageURL('calendar-icon.gif'); ?>">
															</div>
															<div class="text-all">
																<p class="repair-mins">Add to</p>
																<p class="repair-name">Google Calendar</p>
															</div>
														</div>
													</a>
													<a class="rp-a-normalize rp-ical" href="#" data-oid="<?php echo $order->o_id; ?>">
														<div class="wp-repair-repair-card">
															<div class="repair-img">
																<img src="<?php echo getTheImageURL('calendar-icon.gif'); ?>">
															</div>
															<div class="text-all">
																<p class="repair-mins">Add to</p>
																<p class="repair-name">Apple Calendar</p>
															</div>
														</div>
													</a>
													<?php
												}
											?>
											<button type="button" name="update_appointment_status" class="update_appointment_status" style="margin-top: 13px;" data-oid="<?php echo $order->o_id?>" data-email="<?php echo $order->customer_email?>" data-name="<?php echo $order->customer_name?>">Update Data <i class="fas fa-caret-right"></i></button> 
											<button style="display: none; margin-top: 13px;" type="button" class="edit_appointment_price">Set Price <i class="fas fa-caret-right"></i></button> 
										</div>
									</div> 
								</td>
							</tr>
							<tr class="spacer"><td></td></tr>
						<?php } }?>
					</tbody>
				</table>
			</form>
			</div>
			<?php
			global $wp;
			foreach($rpPrintFormsLaters as $_o_id => $dataToPrint) {
				?>
				<form style="display: none;" id="rp_ical<?php echo $_o_id; ?>" action="<?php echo home_url($wp->request); ?>/?rp_ics=true" method="POST">
					<?php
					rp_csrf_token_input_field('appointments');
					?>
					<input type="hidden" name="text" value="<?php echo $dataToPrint['text']; ?>">
					<input type="hidden" name="details" value="<?php echo $dataToPrint['details']; ?>">
					<input type="hidden" name="location" value="<?php echo $dataToPrint['location']; ?>">
					<input type="hidden" name="start_time" value="<?php echo $dataToPrint['start_time']; ?>">
					<input type="hidden" name="end_time" value="<?php echo $dataToPrint['end_time']; ?>">
				</form>
				<?php
			}
			?>
			<div class="navigation">
				<?php 
				if ( $page_links ) {
					echo '<div class="tablenav"><div class="tablenav-pages" style="margin: 1em">' . $page_links . '</div></div>';
				}
				;?>
			</div>
			<!-- <div class="pagination">
				<p>
					<span class="total-entries">Total 6 Entries</span>
					<button class="pagination-btn"><i class="fas fa-angle-double-left"></i></button>
					<button class="pagination-btn"><i class="fas fa-chevron-left"></i></button>
					<span class="page-count">Page 1 of 2</span>
					<button class="pagination-btn"><i class="fas fa-chevron-right"></i></button>
					<button class="pagination-btn"><i class="fas fa-angle-double-right"></i></button>
				</p>
			</div> -->
		</div>
	</div>
</div>
<?php
wp_enqueue_script('wp-repair-plugin-js', WP_REPAIR_PLUGIN_URL.'assets/js/main.js?what='.uniqid());
wp_enqueue_style('wp-repair-plugin-ui-css', WP_REPAIR_PLUGIN_URL.'assets/css/ui-by-zahran.css?what='.uniqid());
wp_enqueue_script('wp-repair-plugin-ui-js', WP_REPAIR_PLUGIN_URL.'assets/js/ui-by-zahran.js?what='.uniqid());
?>
<script type="text/javascript">
	jQuery(document).on('click', ".rp-a-normalize.rp-ical", function(e){
			e.preventDefault();
			let oid = jQuery(this).data("oid");
			jQuery("#rp_ical"+oid).submit();
		});
	jQuery('button[name="delete_appointment"]').each(function(){
		jQuery(this).on("click", function(e) {
			e.preventDefault();
			var oid = jQuery(this).data("oid");
			jQuery("#appointment_parent_form").append('<input type="hidden" name="deleting_appointment_id" value="'+oid+'">');
			jQuery("#appointment_parent_form").submit();
		});
	});
  	jQuery('.update_appointment_status').each(function(){
  		jQuery(this).on("click", function() {
			var oid             = jQuery(this).data("oid");
			var booking_status  = jQuery("tr.active .booking_status .ui-selectmenu-text").html();
			if(booking_status == 'Pending'){
				var booking_status = 1;
			}else if(booking_status == 'Offered'){
				var booking_status = 2;			
			}else if(booking_status == 'Approved'){
				var booking_status = 3;						
			}else{
				var booking_status = 4;									
			}
			var delivery_time   = jQuery(this).parent().parent().find(".delivery_time .ui-selectmenu-text").html();
			var delivery_date   = jQuery(this).parent().parent().find(".delivery_date .ui-selectmenu-text").html();
			var user_name = jQuery(this).data('name');
			var email = jQuery(this).data('email');
			repairPluginUI.ajax({
			type:'POST',
	      	url: "<?php echo get_site_url().'/wp-admin/admin-ajax.php'?>",
	      	data: {action:'update_appointment_status',oid:oid,booking_status:booking_status,delivery_time:delivery_time,delivery_date:delivery_date,user_name:user_name,email:email},
	  		success: function(data) {
	      		location.reload();
	      	}
			});
		});
  	});
</script>
<script type="text/javascript">
	
	(function($){
		function showFullWidthModal(content = '', cls = '', theCloseIcon = '<i class="fas fa-times"></i>') {
			$(".va-full-width-modal").remove();
			let html = `
				<div class="va-full-width-modal ${cls}">
					<div class="va-full-width-modal-main">
						<div class="va-full-width-modal-inner">
							${content}
						</div>
						<button class="va-button va-button-default va-close-modal">
							${theCloseIcon}
						</button>
					</div>
				</div>
			`;
			$("body").append(html);
			$(".va-full-width-modal .va-full-width-modal-main").css({'top': '-50%'}).animate({'top': '50%'}, 400);
			$(".va-full-width-modal .va-close-modal").on("click", function(){
				$(this).parent().animate({'top': '-50%'}, 400, function(){
					$(this).parent().remove();
				});
			});
			$(document).on("mouseup", handOutsideClick);
			// $(".va-full-width-modal .va-submit").on("click", setAlertFormHandler);
		}
		function handOutsideClick(e) {
			var container = $('.va-full-width-modal-main');
			// if the target of the click isn't the container nor a descendant of the container
			if (!container.is(e.target) && container.has(e.target).length === 0) 
			{
			    container.find(".va-close-modal").click();
			    $(document).off("mouseup", handOutsideClick);
			}
		}
		window._showFullWidthModal = function(content = '', cls = '', theCloseIcon = '<i class="fas fa-times"></i>') {
			showFullWidthModal(content, cls, theCloseIcon);
		}
		jQuery(document).on("click", "button.edit_appointment_price", function(e){

			let parent = jQuery(this).parents('tr.more-details');

			let appointment_id = jQuery(parent).find('button.update_appointment_status').data('oid');

			let rows_markup = '';

			jQuery(parent).find('.tr.is_por_repair').each(function(){

				let r_id = jQuery(this).data('rid');

				let a_id = jQuery(this).data('aid');

				let r_name = jQuery(this).find('.text.desc').text().trim();

				let r_price = jQuery(this).find('.text.price').text().trim();

				rows_markup += `
					<tr class="rp-por-row">
						<td>${r_name}</td>
						<td>
							<input type="hidden" name="r_id[]" value="${r_id}">
							<input type="hidden" name="a_id[]" value="${a_id}">
							<input type="text" name="r_price[]" value="${r_price}">
						</td>
					</tr>
				`;

			});

			showFullWidthModal(`
					<p class="va-modal-title gold-color">Set Repair Price(s)</p>
					<table class="rp-por-table">
						<tbody>
							${rows_markup}
						</tbody>
					</table>
					<button data-oid="${appointment_id}" type="submit" class="va-button va-submit rp-set-price-form">
						Set Price <i class="fas fa-caret-right"></i>
					</button>
				`);
		});
		jQuery(document).on('click', 'button.rp-set-price-form', function(){

			if( jQuery(this).hasClass('processing') ) {

				return;

			}

			let appointment_id = jQuery(this).data('oid');

			let parent = jQuery(this).parents('.va-full-width-modal-inner');

			let rid = [];

			let aid = [];

			let valid_form = true;

			jQuery(parent).find('input[name^="r_id["]').each(function(){

				rid.push( jQuery(this).val() );

			});
			
			jQuery(parent).find('input[name^="a_id["]').each(function(){

				aid.push( jQuery(this).val() );

			});

			let r_price = [];

			jQuery(parent).find('input[name^="r_price["]').each(function(){

				r_price.push( jQuery(this).val() );

				if( jQuery(this).val() == '0' ) {

					valid_form = false;

				}

			});

			if( valid_form ) {

				var originalHTML = jQuery(this).html();	

				jQuery(this).html('<i style="position:relative; top: -2px;" class="fas fa-spinner fa-spin"></i>');

				let currentbtn = this;

				jQuery(this).addClass('processing');

				repairPluginUI.ajax({
					type: 'POST',
					url: '<?php echo get_site_url().'/wp-admin/admin-ajax.php'?>',
					data: {
						action: 'rp_update_por_repairs_price',
						appointment_id: appointment_id,
						rid: rid,
						aid: aid,
						r_price: r_price
					},
					success: function() {
						// jQuery(this).removeClass('processing');
						// jQuery(currentbtn).html('Reloading...');
						// jQuery(currentbtn).parents('.va-full-width-modal-main').find('.va-close-modal').click();

						let appointment_parent = jQuery("button.update_appointment_status[data-oid='"+appointment_id+"']").parents('.more-details');

						rid.forEach(function(_rid, i){

							jQuery(".tr.is_por_repair[data-rid='"+_rid+"']").find('.text.price').text( r_price[i] );

							jQuery(".tr.is_por_repair[data-rid='"+_rid+"']").removeClass('is_por_repair');

						});

						// showOrHideSetPriceButton();

						// repairPluginUI.alert_success('Price for selected repair(s) successfully updated.');

						window.location.reload(true);
					},
					error: function( response ) {
						jQuery(currentbtn).removeClass('processing')
						jQuery(currentbtn).html(originalHTML);
						if( response && response.rp_validation_error != '' ) {

						} else {
							repairPluginUI.alert_danger('Something went wrong, please try again later.');
						}
					}
				});

			} else {

				alert('Please enter prices for all repairs.');

			}

		});

		function showOrHideSetPriceButton() {

			jQuery("table tr.more-details").each(function(){

				let full_row = this;

				if( jQuery( this ).find('.tr.is_por_repair').length > 0 ) {

					jQuery( this ).find( '.edit_appointment_price' ).css('display', 'inline-block');

				} else {

					jQuery( this ).find( '.edit_appointment_price' ).css('display', 'none');

				}

			});

		}

		showOrHideSetPriceButton();

	}(jQuery));
	<?php
	if( !empty( $orders ) ) {
		$_orders = array();
		foreach( $orders as $order_row ) {
			$_delivery_method = str_replace(' ', '_', strtolower(trim($order_row->delivery_method)));
			if( $_delivery_method === 'pickup_service' || $_delivery_method === 'repair_on_location' ) {
				$_orders[] = $order_row;
			}
		}
		$encoded_appointment = json_encode( $_orders );
	} else {
		$encoded_appointment = json_encode( array() );
	}
	?>
	window.rp_appointment_data = <?php echo $encoded_appointment; ?>;
	(function($){
		function showFullWidthModal(content = '', additional_cls = '') {
			$(".va-full-width-modal").remove();
			let html = `
				<div class="va-full-width-modal">
					<div class="va-full-width-modal-main ${additional_cls}">
						<div class="va-full-width-modal-inner clearfix">
							${content}
						</div>
						<button class="va-button va-button-default va-close-modal">
							<i class="fas fa-times"></i>
						</button>
					</div>
				</div>
			`;
			$("body").append(html);
			$(".va-full-width-modal .va-full-width-modal-main").css({'top': '-50%'}).animate({'top': '50%'}, 400);
			$(".va-full-width-modal .va-close-modal").on("click", function(){
				$(this).parent().animate({'top': '-50%'}, 400, function(){
					$(this).parent().remove();
				});
			});
			$(document).on("mouseup", handOutsideClick);
			// $(".va-full-width-modal .va-submit").on("click", setAlertFormHandler);
		}
		function handOutsideClick(e) {
			var container = $('.va-full-width-modal-main');
			// if the target of the click isn't the container nor a descendant of the container
			if (!container.is(e.target) && container.has(e.target).length === 0) 
			{
				container.find(".va-close-modal").click();
				$(document).off("mouseup", handOutsideClick);
			}
		}
		jQuery("a.view-customer-address").on('click', function(e){
			e.preventDefault();
			var oid = jQuery(this).data('oid');
			var found_order = null;
			window.rp_appointment_data.forEach(function(row){
				if( row.o_id && row.o_id == oid ) {
					found_order = row;
				}
			});
			if( found_order !== null ) {
				let col_1 = 'width: 30%';
				let col_2 = 'width: 70%';
				showFullWidthModal(`
					<p class="va-modal-title">Customer Location Details</p>
					<table>
						<tbody>
							<tr>
								<th style="${col_1}">House No.</th>
								<td style="${col_2}">${found_order.customer_house_no}</td>
							</tr>
							<tr>
								<th style="${col_1}">Street Address:</th>
								<td style="${col_2}">${found_order.customer_street_address}</td>
							</tr>
							<tr>
								<th style="${col_1}">City</th>
								<td style="${col_2}">${found_order.customer_city}</td>
							</tr>
							<tr>
								<th style="${col_1}">Zipcode</th>
								<td style="${col_2}">${found_order.customer_zipcode}</td>
							</tr>
						</tbody>
					</table>
				`, 'customer-details-popup');
			} else {
				alert('Something went wrong, please try again.');
			}
		});
	}(jQuery));
</script>

<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/tooltipster/4.2.8/css/tooltipster.bundle.min.css" integrity="sha512-RwhQzi0EhwZ9hHs18SqM7ICqW/MH/B17PouhdCuzaUQYEdx2S4WPgkhLeZPvGr2Kb1FIm4gaIbMJGwZtxDCLQA==" crossorigin="anonymous" referrerpolicy="no-referrer" />
<script src="https://cdnjs.cloudflare.com/ajax/libs/tooltipster/4.2.8/js/tooltipster.bundle.min.js" integrity="sha512-ZKNW/Nk1v5trnyKMNuZ6kjL5aCM0kUATbpnWJLPSHFk/5FxnvF9XmpmjGbag6BEgmXiz7rL6o6uJF6InthyTSg==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>

<style type="text/css">

	.rp-tooltip-content table {
		width: 290px;
	}

	.rp-tooltip-content table th {
		width: 120px;
		font-size: 10px;
		line-height: 1.2em;
		letter-spacing: 1px;
		font-weight: 600;
		text-transform: uppercase;
		text-align: left;
	}

	.rp-tooltip-content table td {
		padding-left: 35px !important;
	}

	.rp-tooltip-content .view-more-details,
	.rp-tooltip-content .view-more-details:hover,
	.rp-tooltip-content .view-more-details:focus,
	.rp-tooltip-content .view-more-details:active {
		width: 100%;
		margin-top: 10px;
		background: #1275c2;
		color: #fff;
		padding: 9px;
		cursor: pointer;
		outline: 0px;
		box-shadow: 0px;
		border: 0px;
		border-radius: 6px;
	}

	.rp-tooltip {
		cursor: pointer;
	}

	.tooltipster-sidetip .tooltipster-content {
		background: #404040;
	}

	.tooltipster-sidetip.tooltipster-top .tooltipster-arrow-background {
		border-top-color: #404040;
	}

	.dynamic-pricing-more-details *:not(i),
	.rp-tooltip-content *:not(i) {
		font-family: 'Montserrat', sans-serif !important;
	}

	.dynamic-pricing-more-details table {
		width: 100%;
		text-align: left;
	}

	.dynamic-pricing-more-details table tr td:nth-child(1) {
		width: 39%;
	}

	.dynamic-pricing-more-details table tr td:nth-child(2) {
		width: 39%;
		letter-spacing: 2px;
	}

	.dynamic-pricing-more-details table tr td:nth-child(3) {
		width: 22%;
	}
	.rp-tooltip-content table td {
		font-weight: 400;
	}
	.rp-tooltip-content table th,
	.rp-tooltip-content table td {
		vertical-align: middle;
		display: table-cell;

	}
	.rp-tooltip-content table th {
		padding-top: 3px;
		padding-bottom: 0px;
	}
	.rp-tooltip-content table .rp-the-total th,
	.rp-tooltip-content table .rp-the-total td {
		border-top: 1px solid #5a5a5a;
		padding-top: 10px;
	}

	.rp-tooltip-content table .after-round-up th,
	.rp-tooltip-content table .after-round-up td {
		padding-bottom: 8px;
	}

	.rp-tooltip-content table .after-round-up td {
		font-weight: 600;
	}

	table.dp-pricing-info-table {
		border-collapse: collapse;
	}

	.tooltipster-sidetip .tooltipster-content {
		color: #fff;
		line-height: 20px;
		padding: 14px;
		border-radius: 8px;
		overflow: hidden;
		padding-top: 13px;
	}

	.tooltipster-sidetip .tooltipster-box {
		background: unset;
		border: 0px !important;
		border-radius: 0px;
	}

	.tooltipster-sidetip .tooltipster-content,
	.tooltipster-sidetip .tooltipster-box {
		box-shadow: unset;
		outline: 0;
		border: 0px;
		margin: 0px;
	}

	.tooltipster-sidetip.tooltipster-top .tooltipster-arrow-border {
		border-top-color: #404040;
	}
	.tooltipster-sidetip.tooltipster-right .tooltipster-arrow-background,
	.tooltipster-sidetip.tooltipster-right .tooltipster-arrow-border {
		border-right-color: #404040;
	}

	.tooltipster-sidetip.tooltipster-left .tooltipster-arrow-background,
	.tooltipster-sidetip.tooltipster-left .tooltipster-arrow-border {
		border-left-color: #404040;
	}

	.tooltipster-sidetip.tooltipster-bottom .tooltipster-arrow-background,
	.tooltipster-sidetip.tooltipster-bottom .tooltipster-arrow-border {
		border-bottom-color: #404040;
	}

	.dynamic-pricing-more-details table th.rp-popup-heading-th {
		color: #1275c2;
    	font-weight: 600;
		padding-bottom: 10px;
	}

	.dynamic-pricing-more-details table tr.dp-lgray-text th {
		font-size: 15px;
		font-weight: 500;
		color: #a1a1a1;
		padding: 4px 0px;
	}

	.dynamic-pricing-more-details table tr td {
		font-size: 15px;
		font-weight: 600;
		color: #1d2327;
		padding: 4px 0px;
	}
	.dynamic-pricing-more-details .va-full-width-modal-main {
		max-width: 550px;
	}
	.dynamic-pricing-more-details .va-full-width-modal-inner {
		padding: 25px;
	}
	.dynamic-pricing-more-details .va-full-width-modal-inner hr.rp-dp-custom-hr {
		border: 0;
		border-top: 1px solid #dcdcde;
		border-bottom: 1px solid #f6f7f7;
		margin-top: 15px;
		margin-bottom: 15px;
	}

	.dynamic-pricing-more-details.va-full-width-modal .va-modal-title {
		font-weight: 600;
		max-width: 93%;
		font-size: 13px;
		margin-top: 0px;
		margin-bottom: 10px;
		color: #1275c2;
		text-transform: uppercase;
		letter-spacing: 1.5px;
	}

	.dynamic-pricing-more-details .va-full-width-modal-inner h3 {
		font-weight: 700;
		font-size: 19px;
	}

	.dynamic-pricing-more-details .va-full-width-modal-inner .model-name-separator {
		display: inline-table;
		padding: 0px 10px;
		font-weight: 500;
	}

	.dynamic-pricing-more-details .va-full-width-modal-inner p.rp-dp-popup-model-name {
		font-size: 14px;
		font-weight: 600;
		color: #a1a1a1;
	}
	.rp-dp-rounded-time-box {
		color: #a1a1a1;
		background: #f1f1f1;
		padding: 4px 5px;
		padding-right: 8px;
		border-radius: 15px;
		font-size: 10px;
		margin-left: 6px;
	}

	.rp-dp-rounded-time-box svg {
		width: 12px;
		height: auto;
		position: relative;
		top: 2px;
		padding-right: 3px;
	}
	.rp-dp-rounded-time-box svg .rp-dp-history0 {
		fill: #a1a1a1;
	}
	.dynamic-pricing-more-details .va-button.va-close-modal {
		padding: 0px;
		background: unset;
		border: unset;
		top: 25px;
    	right: 25px;
	}

	.dynamic-pricing-more-details .va-button.va-close-modal svg {
		width: 16px;
		height: auto;
	}

	#rp-dp-cancel-icon {
		display: none !important;
	}

</style>

<div id="rp-dp-cancel-icon">
<?php echo file_get_contents(WP_REPAIR_PLUGIN_PATH.'assets/images/cancel-icon.svg'); ?>
</div>

<div class="all-tooltips" style="display: none;">
	<?php
	if( isset( $allTooltips ) && !empty( $allTooltips ) ) {
		foreach( $allTooltips as $theTooltip ) {

			$tooltip_id = $theTooltip['id'];

			$row_type = $theTooltip['row_type'];

			$row_id = $theTooltip['row_id'];

			$repair_name = $theTooltip['repair_name'];

			$decoded = unserialize( str_replace( '&quot;', '"', $theTooltip['data'] ) );

			$sku_row = $decoded['sku_row'];

			$decoded['tax_price'] = $decoded['tax_price'] ?? 0;

			$decoded['tax_rate'] = $decoded['tax_rate'] ?? 0;

			$decoded['final_tax'] = $decoded['final_tax'] ?? 0;

			$decoded['final_excluding_tax'] = $decoded['final_excluding_tax'] ?? 0;

			if( $decimal === 'comma' ) {

				$sku_row->price = number_format($sku_row->price, 2, ',', '.');
				$sku_row->processed_price = number_format($sku_row->processed_price, 2, ',', '.');
				$decoded['margin'] = number_format($decoded['margin'], 2, ',', '.');
				$decoded['tax_price'] = number_format($decoded['tax_price'], 2, ',', '.');
				$decoded['before_round_up'] = number_format($decoded['before_round_up'], 2, ',', '.');
				$decoded['after_round_up'] = number_format($decoded['after_round_up'], 2, ',', '.');
				$decoded['final_tax'] = number_format($decoded['final_tax'], 2, ',', '.');
				$decoded['final_excluding_tax'] = number_format($decoded['final_excluding_tax'], 2, ',', '.');

			} else {

				$sku_row->price = number_format($sku_row->price, 2);
				$sku_row->processed_price = number_format($sku_row->processed_price, 2);
				$decoded['margin'] = number_format($decoded['margin'], 2);
				$decoded['tax_price'] = number_format($decoded['tax_price'], 2);
				$decoded['before_round_up'] = number_format($decoded['before_round_up'], 2);
				$decoded['after_round_up'] = number_format($decoded['after_round_up'], 2);
				$decoded['final_tax'] = number_format($decoded['final_tax'], 2);				
				$decoded['final_excluding_tax'] = number_format($decoded['final_excluding_tax'], 2);

			}

			?>
			<span id="<?php echo $tooltip_id; ?>" class="rp-tooltip-content">
				<table class="dp-pricing-info-table">
					<tbody>
						<tr>
							<th>Supplier</th>
							<td><?php echo $sku_row->supplier_name ?? 'undefined'; ?></td>
						</tr>
						<tr>
							<th>SKU</th>
							<td><?php echo $sku_row->sku; ?></td>
						</tr>
						<tr>
							<th>Price</th>
							<td><?php echo $sku_row->price; ?></td>
						</tr>
						<?php
						if( $decoded['rp_dp_which_price_to_consider'] === 'average' ) {
							?>
							<tr>
								<th>Average</th>
								<td><?php echo $sku_row->processed_price; ?></td>
							</tr>
							<?php
						}
						?>
						<tr>
							<th>Margin</th>
							<td><?php echo $decoded['margin']; ?></td>
						</tr>
						<tr>
							<th>Tax (<?php echo $decoded['tax_rate']; ?>%)</th>
							<td><?php echo $decoded['tax_price']; ?></td>
						</tr>
						<tr>
							<th>Before Round Up</th>
							<td><?php echo $decoded['before_round_up']; ?></td>
						</tr>
						<tr class="after-round-up">
							<th>After Round Up</th>
							<td><?php echo $decoded['after_round_up']; ?></td>
						</tr>
						<tr class="rp-the-total">
							<th>Total Excl. Tax</th>
							<td><?php echo $decoded['final_excluding_tax']; ?></td>
						</tr>
						<tr>
							<th>Tax (<?php echo $decoded['tax_rate']; ?>%)</th>
							<td><?php echo $decoded['final_tax']; ?></td>
						</tr>
					</tbody>
				</table>
				<button class="view-more-details" type="button" data-id="<?php echo $row_id; ?>" data-type="<?php echo $row_type; ?>" data-repair-name="<?php echo $repair_name; ?>">View More Details</button>
			</span>
			<?php

		}
		?>
		<script type="text/javascript">
			jQuery(document).ready(function() {
				jQuery('.rp-tooltip').each(function(){
					jQuery(this).tooltipster({
						interactive: true,
						trigger: 'click'
					});
				});
        	});
			jQuery(document).on('click', '.rp-tooltip-content .view-more-details', function(e){
				e.preventDefault();

				if( jQuery(this).hasClass('processing') ) {

					return;

				}

				let btn = jQuery(this);
				let row_id = btn.data('id');
				let row_type = btn.data('type');
				let repair_name = btn.data('repair-name');

				let originalHTML = btn.html();

				// show loader
				btn.addClass('processing');
				btn.html('<i class="fa fa-spinner fa-spin"></i>');

				repairPluginUI.ajax({
					type: 'POST',
					url: '<?php echo get_site_url().'/wp-admin/admin-ajax.php'?>',
					data: {
						action: 'rp_get_more_details_of_sku',
						row_id: row_id,
						row_type: row_type
					},
					success: function( response ) {

						jQuery(btn).removeClass('processing')
						jQuery(btn).html(originalHTML);

						try {
							response = JSON.parse( response );
						} catch(e) {
							repairPluginUI.alert_danger('Received invalid response from server.');
							return;
						}

						if( response && response.pricing_info && response.pricing_info !== null ) {

							let skus_rows_table = '';

							if( response.pricing_info.all_skus && response.pricing_info.all_skus !== null ) {
								skus_rows_table = '<table>';
								skus_rows_table += '<thead><tr><th colspan="3" class="rp-popup-heading-th">At the time of calculation '+response.pricing_info.calculated_at_time+'</th></tr><tr class="dp-lgray-text"><th>Supplier</th><th>SKU</th><th>Price</th></tr></thead>';
								skus_rows_table += '<tbody>';
								for( let i = 0; i < response.pricing_info.all_skus.length; i++ ) {
									skus_rows_table += '<tr>';
									skus_rows_table += '<td>'+response.pricing_info.all_skus[i].supplier_name+'</td>';
									skus_rows_table += '<td>'+response.pricing_info.all_skus[i].sku+'</td>';
									skus_rows_table += '<td>'+response.pricing_info.all_skus[i].price+'</td>';
									skus_rows_table += '</tr>';
								}
								skus_rows_table += '</tbody>';
								skus_rows_table += '</table>';
							} else {
								skus_rows_table = '<table>';
								skus_rows_table += '<thead><tr><th colspan="3" class="rp-popup-heading-th">At the time of calculation '+response.pricing_info.calculated_at_time+'</th></tr><tr class="dp-lgray-text"><th>Supplier</th><th>SKU</th><th>Price</th></tr></thead>';
								skus_rows_table += '<tbody>';
								live_rows_table += '<tr><td colspan="3">No SKU Found</td></tr>';
								skus_rows_table += '</tbody>';
								skus_rows_table += '</table>';
							}

							let live_rows_table = '';

							if(  response.live_skus && response.live_skus.length > 0 ) {
								live_rows_table = '<table>';
								live_rows_table += '<thead><tr><th colspan="3" class="rp-popup-heading-th">Last synced '+response.last_processing+'</th></tr><tr class="dp-lgray-text"><th>Supplier</th><th>SKU</th><th>Price</th></tr></thead>';
								live_rows_table += '<tbody>';
								for( let i = 0; i < response.live_skus.length; i++ ) {
									live_rows_table += '<tr>';
									live_rows_table += '<td>'+response.live_skus[i].supplier_name+'</td>';
									live_rows_table += '<td>'+response.live_skus[i].sku+'</td>';
									live_rows_table += '<td>'+response.live_skus[i].price+'</td>';
									live_rows_table += '</tr>';
								}
								live_rows_table += '</tbody>';
								live_rows_table += '</table>';
							} else {
								live_rows_table = '<table>';
								live_rows_table += '<thead><tr><th colspan="3" class="rp-popup-heading-th">Last synced '+response.last_processing+'</th></tr><tr class="dp-lgray-text"><th>Supplier</th><th>SKU</th><th>Price</th></tr></thead>';
								live_rows_table += '<tbody>';
								live_rows_table += '<tr><td colspan="3">No SKU Found</td></tr>';
								live_rows_table += '</tbody>';
								live_rows_table += '</table>';
							}

							// hide tooltip
							jQuery(btn).closest('.tooltipster-base').remove();

							window._showFullWidthModal(`
							<p class="va-modal-title">More details</p>
							<p class="rp-dp-popup-model-name">${response.model_name}</p>
							<h3>${repair_name}</h3>
							<hr class="rp-dp-custom-hr">
							${skus_rows_table}
							<hr class="rp-dp-custom-hr">
							${live_rows_table}
						`, 'dynamic-pricing-more-details', jQuery("#rp-dp-cancel-icon").html());

						} else {

							repairPluginUI.alert_danger('Something went wrong, please try again later.');
							return;


						}

					},
					error: function( response ) {
						jQuery(btn).removeClass('processing')
						jQuery(btn).html(originalHTML);
						if( response && response.rp_validation_error != '' ) {

						} else {
							repairPluginUI.alert_danger('Something went wrong, please try again later.');
						}
					}
				});

			});
		</script>
		<?php
	}
	?>
</div>
