<?php
// Exit if accessed directly
defined( 'ABSPATH' ) || exit;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

function wp_repair_load_php_mailer($path = '') {
    $fileName = ABSPATH.'/wp-includes/'.$path;
    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);
    return $fileName;
}

function wp_repair_send_mail($to = '', $subject = '', $message = '') {
	if(defined('WP_REPAIR_REQUIRE_LICENSE')) {
		return;
	}
	$headers = array('Content-Type: text/html; charset=UTF-8');
	wp_mail($to, $subject, $message, $headers);
}

function prettifyWPRepairHTML($message = '') {
	ob_start();
	?>
	<!DOCTYPE html>
	<html>
	<head>
		<style type="text/css">
			p {
				font-size: 14px;
				line-height: 1.5em;
				margin: 10px 0px;
			}
			.order-summary {
				display: block;
				width: 100%;
			}
			.wp-repair-inner-col {
				max-width: 300px;
				width: 100%;
			}
			.wp-repair-inner-col .separator {
				display: table;
				height: 1.2px;
				width: 100%;
				background-color: #efefef;
			}
			.wp-repair-inner-col .text.desc {
				width: 70%;
				display: table;
				box-sizing: border-box;
				float: left;
				padding: 5px;
				margin: 5px 0px;
				font-size: 14px;
			}
			.wp-repair-inner-col .text.price {
				width: 30%;
				text-align: center;
				display: table;
				box-sizing: border-box;
				float: left;
				padding: 5px;
				margin: 5px 0px;
				font-size: 15px;
			}
			.wp-repair-inner-col .text.desc.order-summary {
				text-transform: uppercase;
				font-size: 13px;
				font-weight: 700;
				width: 100%;
			}
			.wp-repair-inner-col .wp-repair-inner-col:after,
			.order-summary {
				display: table;
				clear: both;
				content: "";
			}
			.wp-repair-inner-col .total .text.desc,
			.wp-repair-inner-col .total .text.price {
				font-size: 16px;
				font-weight: 700;
			}
			.wp-repair-inner-col .full-repair-name {
				display: inline-block;
			}
			.wp-repair-inner-col .percent-off-badge {
				font-weight: 700;
				font-size: 11px;
				background: #fff;
				color: #1275c2;
				border: 1px solid #1275c2;
				line-height: 1.1em;
				text-transform: uppercase;
				padding: 2px 5px 2px 5px !important;
				box-sizing: border-box;
				border-radius: 6px;
				vertical-align: middle;
				position: relative;
				top: -1px;
				white-space: nowrap;
				display: inline-block;
				height: auto;
				width: auto;
			}
			.wp-repair-inner-col .full-repair-name.has-discounted {
				margin-right: 5px;
			}
		</style>
	</head>
	<body>
		<?php echo $message; ?>
	</body>
	</html>
	<?php
	$content = ob_get_contents();
	ob_end_clean();
	return $content;
}

function wp_repair_prepare_order_email_subject($order_id = null, $message = '') {
	$GLOBALS['rp_is_subject_preparing'] = TRUE;

	$match = rp_find_link_pattern('RescheduleLink', $message);

	if( !empty($match) && strpos($message, 'RescheduleLink') !== FALSE ) {

		$message = str_replace($match, '', $message);

	}

	$match = rp_find_link_pattern('cancel', $message);

	if( !empty($match) && strpos($message, 'cancel') !== FALSE ) {

		$message = str_replace($match, '', $message);

	}

	$match = rp_find_link_pattern('finalize', $message);

	if( !empty($match) && strpos($message, 'finalize') !== FALSE ) {

		$message = str_replace($match, '', $message);

	}

	$message = str_replace('$FinalizeButton$', '', $message);
	$message = str_replace('$cancel$', '', $message);
	$message = str_replace('$OrderSummary$', '', $message);
	$message = wp_repair_prepare_order_email($order_id, $message, $message);
	unset($GLOBALS['rp_is_subject_preparing']);
	return $message;
}

function wp_repair_prepare_order_email($order_id = null, $subject = '', $message = '') {
	global $rpQuery;
	global $wp_repair_lang;

	$rs_orders    = $rpQuery->prefix . "rs_orders";

	$rs_customer    = $rpQuery->prefix . "rs_customer";

	$rs_coupons    = $rpQuery->prefix . "rs_coupons";

	$rs_customer_business    = $rpQuery->prefix . "rs_customer_business";

	$rs_company_location    = $rpQuery->prefix . "rs_company_location";

	$rs_order_upsales    = $rpQuery->prefix . "rs_order_upsales";

	$rs_upsales    = $rpQuery->prefix . "rs_upsales";

	$rs_order_repairs    = $rpQuery->prefix . "rs_order_repairs";

	$rs_repair    = $rpQuery->prefix . "rs_repair";

	$rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

	$rs_order_repair_attr    = $rpQuery->prefix . "rs_order_repair_attr";

	$rs_category    = $rpQuery->prefix . "rs_category";

	$rs_brand    = $rpQuery->prefix . "rs_brand";

	$rs_model    = $rpQuery->prefix . "rs_model";

	$o_id = $order_id;
	$order = $rpQuery->get_results("SELECT o.*, cl.*, cu.* FROM $rs_orders AS o LEFT JOIN $rs_company_location AS cl ON o.location_id_fk = cl.location_id LEFT JOIN $rs_customer as cu ON o.customer_id_fk = cu.customer_id WHERE o.o_id = '".rp_escape_sql($order_id)."'");
	if(empty($order)) {
		return '';
	}
	$order = $order[0];

	$_delivery_method = str_replace(' ', '_', strtolower(trim($order->delivery_method)));
	if( $_delivery_method === 'ship_device' ) {
		$order->delivery_date = date('d-m-Y', strtotime($order->created_time));
		$order->delivery_time = date('H:i', strtotime($order->created_time));
	}

	$cancel_link = '';

	$shortcode_page_link = rp_get_cancel_link_by_order_id( $order->o_id );

	if( empty( $shortcode_page_link ) ) {

		$shortcode_page_link = rp_get_shortcode_page_link();

	}

	if( empty( $shortcode_page_link ) ) {

		$m_id = $order->m_id_fk;

		$_model = $rpQuery->get_row("SELECT m_uq_id FROM $rs_model WHERE m_id = '".rp_escape_sql($m_id)." LIMIT 1;'");

		if( !empty( $_model ) ) {

			$shortcode_page_link = rp_get_specific_shortcode_page_link( $_model->m_uq_id );

		}

	}

	$order_summary = order_summary_for_email($order->o_id);
	$message = str_replace('$FinalizeButton$', '', $message);
	$message = str_replace('$OrderSummary$', $order_summary, $message);

	$match = rp_find_link_pattern('RescheduleLink', $message);

	if( !empty($match) && strpos($message, 'RescheduleLink') !== FALSE ) {

		$allowed_dmethods = array('come by our store', 'pickup service', 'repair on location');

		if( !empty( ($order->reschedule_link ?? '') ) && in_array( strtolower($order->delivery_method), $allowed_dmethods ) !== FALSE && !empty($order->reschedule_link ?? '') ) {

			$matchText = str_replace('[RescheduleLink="', '', $match);

			$matchText = str_replace('[RescheduleLink=&quot;', '', $matchText);

			$matchText = str_replace('"]', '', $matchText);

			$matchText = str_replace('&quot;]', '', $matchText);

			$message = str_replace($match, '<a target="_blank" href="'.$order->reschedule_link.'">'.$matchText.'</a>', $message);

		} else {

			$message = str_replace($match, '', $message);

		}

	}

	$match = rp_find_link_pattern('cancel', $message);

	if( !empty($match) && strpos($message, '[cancel=') !== FALSE ) {

		if(!empty($shortcode_page_link)) {

			$matchText = str_replace('[cancel="', '', $match);
			$matchText = str_replace('[cancel=&quot;', '', $matchText);
			$matchText = str_replace('"]', '', $matchText);
			$matchText = str_replace('&quot;]', '', $matchText);

			$cancel_link = $shortcode_page_link;
			$cancel_link .= '?selected=cancel&id='.$order_id.'&secret_key='.md5($order->o_id.$order->customer_id_fk);
			$cancel_link = sprintf('<a target="_blank" href="%s">'.$matchText.'</a>', $cancel_link);

			$message = str_replace($match, $cancel_link, $message);

		} else {

			$message = str_replace($match, '', $message);

		}

	}

	$match = rp_find_link_pattern('finalize', $message);

	if( !empty($match) && strpos($message, '[finalize=') !== FALSE ) {

		$message = str_replace($match, '', $message);

	}

	$search_and_replace = array(
		'$Color$' => 'color',
		'$CustomerName$' => 'customer_name',
		'$CustomerPhone$' => 'customer_phone',
		'$CustomerEmail$' => 'customer_email',
		'$CustomerHouseNo$' => 'customer_house_no',
		'$CustomerStreetAddress$' => 'customer_street_address',
		'$CustomerCity$' => 'customer_city',
		'$CustomerZipcode$' => 'customer_zipcode',
		'$CustomerNotes$' => 'notes',
		'$DeliveryMethod$' => 'delivery_method',
		'$LocationName$' => 'location_name',
		'$LocationAddress$' => 'location_address',
		'$LocationZipcodeCity$' => 'location_city_district',
		'$LocationTelephone$' => 'location_telephone',
	);

	if( rp_is_active_enhanced_locations_addon() === false ) {

		$order->location_telephone = '';

	}

	foreach ($search_and_replace as $key => $value) {
		if( property_exists($order, $value) ) {
			if( $order->{$value} === NULL ) {
				$order->{$value} = '';
			}
			$message = str_replace($key, $order->{$value}, $message);
		}
	}

	$order_delivery_date = date('d-m-Y', strtotime( ($order->delivery_date ?? '') ) );

	$delivery_time = $order->delivery_time ?? '';

	if( !empty( $delivery_time ) ) {

		$localization = $rpQuery->prefix . "rs_localization";
		$localization_data = $rpQuery->get_results("SELECT * FROM $localization");

		if( strpos( $delivery_time, ' — ' ) === FALSE ) {

            if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {

                  $delivery_time = date('H:i', strtotime( $delivery_time ) );

            } else {

                  $delivery_time = date('g:i a', strtotime( $delivery_time ) );

            }

		} else {

			$DateTime = explode(' — ', $delivery_time);

			if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
				$newDateTime1 = date('H:i', strtotime($DateTime[0]));
				$newDateTime2 = date('H:i', strtotime($DateTime[1]));
				$newDateTime = $newDateTime1 . ' — ' . $newDateTime2;
			} else {
				$newDateTime1 = date('g:i a', strtotime($DateTime[0]));
				$newDateTime2 = date('g:i a', strtotime($DateTime[1]));
				$newDateTime = $newDateTime1 . ' — ' . $newDateTime2;
			}

			$delivery_time = $newDateTime;

		}

		$message = str_replace('$AppointmentTime$', $delivery_time, $message);

	} else {

		$message = str_replace('$AppointmentTime$', '--', $message);

	}

	$message = str_replace('$AppointmentDate$', ($order_delivery_date ?? '--'), $message);

	$o_date_time_exploded = explode(' ', $order->o_date_time);

	if(!empty($o_date_time_exploded) && count($o_date_time_exploded) >= 1) {
		$o_date = $o_date_time_exploded[0];
	}
	if(!empty($o_date_time_exploded) && count($o_date_time_exploded) >= 2) {
		$o_time = $o_date_time_exploded[1];
	}

	$m_id = $order->m_id_fk;
	$models = $rpQuery->get_results("SELECT * FROM $rs_model WHERE m_id = '".rp_escape_sql($m_id)."'");
	$b_id = $models[0]->brand_id_fk;
	$brands = $rpQuery->get_results("SELECT * FROM $rs_brand WHERE b_id = '".rp_escape_sql($b_id)."'");
	$c_id = $models[0]->device_type; 
	$cats = $rpQuery->get_results("SELECT * FROM $rs_category WHERE c_id = '".rp_escape_sql($c_id)."'");


	$model = '';
	if(!empty($models)) {
		$model = $models[0]->m_name;
	}
	$brand = '';
	if(!empty($brands)) {
		$brand = $brands[0]->b_name;
	}
	$category = '';
	if(!empty($cats)) {
		$category = $cats[0]->c_name;
	}

	$o_date = date('d-m-Y', strtotime($o_date));

	$message = str_replace('$CreatedDate$', $o_date, $message);
	$message = str_replace('$CreatedTime$', $o_time, $message);
	$message = str_replace('$Category$', $category, $message);
	$message = str_replace('$Brand$', $brand, $message);
	$message = str_replace('$Model$', $model, $message);

	if(!empty($shortcode_page_link)) {
		$cancel_link = $shortcode_page_link;
		$cancel_link .= '?selected=cancel&id='.$order_id.'&secret_key='.md5($order->o_id.$order->customer_id_fk);
		$cancel_link = sprintf('<a target="_blank" href="%s">'.$wp_repair_lang['cancel_booking'].'</a>', $cancel_link);
	}

	$message = str_replace('$cancel$', $cancel_link, $message);

	if(!class_exists('PHPMailer\PHPMailer\PHPMailer')) {
		require_once wp_repair_load_php_mailer('PHPMailer/PHPMailer.php');
	}

	if(!class_exists('PHPMailer\PHPMailer\SMTP')) {
		require_once wp_repair_load_php_mailer('PHPMailer/SMTP.php');
	}

	if(!class_exists('PHPMailer\PHPMailer\Exception')) {
		require_once wp_repair_load_php_mailer('PHPMailer/Exception.php');
	}

	if(!isset($GLOBALS['rp_is_subject_preparing'])) {

		$message = prettifyWPRepairHTML($message);
		
	}


	$company_information = $rpQuery->prefix . "rs_company_information";
	$c_results = $rpQuery->get_results("SELECT * FROM $company_information");

	$message = str_replace('$ShopName$', $c_results[0]->company_name, $message);

	return $message;
}

function order_summary_for_email($order_id = null) {
	global $rpQuery;
	global $wp_repair_lang;

	ob_start();

	$rs_orders    = $rpQuery->prefix . "rs_orders";

	$rs_customer    = $rpQuery->prefix . "rs_customer";

	$rs_coupons    = $rpQuery->prefix . "rs_coupons";

	$rs_customer_business    = $rpQuery->prefix . "rs_customer_business";

	$rs_company_location    = $rpQuery->prefix . "rs_company_location";

	$rs_order_upsales    = $rpQuery->prefix . "rs_order_upsales";

	$rs_upsales    = $rpQuery->prefix . "rs_upsales";

	$rs_order_repairs    = $rpQuery->prefix . "rs_order_repairs";

	$rs_repair    = $rpQuery->prefix . "rs_repair";

	$rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

	$rs_order_repair_attr    = $rpQuery->prefix . "rs_order_repair_attr";

	$rs_category    = $rpQuery->prefix . "rs_category";

	$rs_brand    = $rpQuery->prefix . "rs_brand";

	$rs_model    = $rpQuery->prefix . "rs_model";

	$o_id = $order_id;

	$orders = $rpQuery->get_results("SELECT o.*, cu.*, c.*,cb.*,loc.* FROM $rs_orders as o LEFT JOIN $rs_customer as cu ON o.customer_id_fk = cu.customer_id LEFT JOIN $rs_coupons as c ON c.coupons_id = o.coupon_id_fk LEFT JOIN $rs_customer_business as cb ON cu.customer_id = cb.customer_id_fk LEFT JOIN $rs_company_location as loc ON loc.location_id = o.location_id_fk WHERE o_id = '".rp_escape_sql($o_id)."' ORDER BY o.o_date_time DESC");

	$order = null;

	if(!empty($orders)) {
		$order = $orders[0];
	} else {
		return '';
	}

	$localization = $rpQuery->prefix . "rs_localization";
	$localization_data = $rpQuery->get_results("SELECT * FROM $localization");
	$decimal = $localization_data[0]->set_decimals;
	$currency_symbol = $localization_data[0]->set_currency;

	$o_upsales = $rpQuery->get_results("SELECT * FROM $rs_order_upsales WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
	$order_repairs = $rpQuery->get_results("SELECT * FROM $rs_order_repairs WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
	$or_attr = $rpQuery->get_results("SELECT * FROM $rs_order_repair_attr WHERE o_id_fk = '".rp_escape_sql($o_id)."'");

	// Has Saved Price on Appointment Creation
	$hasSaved = FALSE;
	if(!empty($o_upsales)) {
		if(!empty($o_upsales[0]->upsale_name ?? '')) {
			$hasSaved = TRUE;
		}
	}
	if(!empty($order_repairs)) {
		if(!empty($order_repairs[0]->repair_name ?? '')) {
			$hasSaved = TRUE;
		}
	}
	if(!empty($or_attr)) {
		if(!empty($or_attr[0]->repair_attr_name ?? '')) {
			$hasSaved = TRUE;
		}
	}
	if($hasSaved === FALSE) {
		$o_upsales = $rpQuery->get_results("SELECT * FROM $rs_order_upsales AS o_up JOIN $rs_upsales AS up ON o_up.upsales_id_fk = up.upsales_id WHERE o_up.o_id_fk = '".rp_escape_sql($o_id)."'");
		$order_repairs = $rpQuery->get_results("SELECT * FROM $rs_order_repairs AS o_r JOIN $rs_repair AS r ON r.r_id = o_r.r_id_fk WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
		$or_attr = $rpQuery->get_results("SELECT * FROM $rs_order_repair_attr AS o_ra JOIN  $rs_repair_attr AS ra ON ra.a_id = o_ra.ra_id_fk WHERE o_id_fk = '".rp_escape_sql($o_id)."'");
	}

	list($order_repairs, $or_attr) = rp_translate_locked_repairs_and_attributes($order_repairs, $or_attr);

	$o_upsales = rp_translate_locked_upsales($o_upsales);

	$m_id = $order->m_id_fk;
	$models = $rpQuery->get_results("SELECT * FROM $rs_model WHERE m_id = '".rp_escape_sql($m_id)."'");
	$b_id = $models[0]->brand_id_fk;
	$brands = $rpQuery->get_results("SELECT * FROM $rs_brand WHERE b_id = '".rp_escape_sql($b_id)."'");
	$c_id = $models[0]->device_type; 
	$cats = $rpQuery->get_results("SELECT * FROM $rs_category WHERE c_id = '".rp_escape_sql($c_id)."'");

	$por_repairs = rp_get_por_repairs_by_order_id( $o_id );

	$por_repair_attrs = rp_get_por_repair_attr_by_order_id( $o_id );

	list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $order->o_id );
	?>
	<div class="order-summary">
		<div class="wp-repair-inner-col section-3">
			<div class="separator"></div>
			<div class="tr">
				<p class="text desc order-summary"><?php echo $wp_repair_lang['order_summary']; ?></p>
			</div>
			<div class="separator"></div>
			<?php if($order_repairs){
				foreach($order_repairs as $rp){
					$rid = $rp->r_id_fk;
					$is_por = FALSE;
					if( rp_check_is_por_repair( $rid, $por_repairs ) === TRUE ) {
						$is_por = TRUE;
					}
					$itReallyWorked = FALSE;
					if($or_attr){ 
						foreach($or_attr as $ra){
							$r_id = $ra->r_id_fk;
							if($rid == $r_id){ 
								$itReallyWorked = TRUE; 
								$repairName = ($rp->repair_name ?? $rp->r_name) .' - '. ($ra->repair_attr_name ?? $ra->a_name);
								$is_discounted_price = '';
								if( property_exists( $ra, 'repair_attr_price' ) ) {
									list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $ra->ra_id_fk, 'attr' );
									$repairPrice = $discounted_price;
								} else {
									$repairPrice = $ra->a_price;
								}
								if(rp_check_is_por_repair_attr( $ra->ra_id_fk, $por_repair_attrs )) {
									$is_por = TRUE;
								}
								?>
								<div class="tr">
									<p class="text desc"><span class="full-repair-name <?php if( $is_discounted_price == '1' ) { echo 'has-discounted'; } ?>"><?php echo $repairName;?></span><?php
									if( $is_discounted_price == '1' ) {
										if( $rpCart->get_combo_discount_percentage() > 0 ) {
											echo '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
										} else {
											echo '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), '<span class="percent-off-currency">'.$localization_data[0]->set_currency.'</span>', FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
										}
									}
									?></p>
									<p class="text price">
										<?php if($decimal == 'comma'){
											$a_price = str_replace('.',',',$repairPrice);
										}else{
											$a_price = $repairPrice;					
										}
										if( $is_por === TRUE ) {
											echo $wp_repair_lang['price_on_request_text'];
										} else {
											echo rp_handle_currency_position($a_price, $currency_symbol, TRUE);
										}
										?> 
									</p>
								</div>
							<?php }
						}
						if($itReallyWorked === FALSE) {
							$itReallyWorked = TRUE;
							$repairName = $rp->repair_name ?? $rp->r_name;
							$is_discounted_price = '';
							if( property_exists( $rp, 'repair_price' ) ) {
								list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $rp->r_id_fk, 'repair' );
								$repairPrice = $discounted_price;
							} else {
								$repairPrice = $rp->r_price;
							}
							?>
							<div class="tr">
								<p class="text desc"><span class="full-repair-name <?php if( $is_discounted_price == '1' ) { echo 'has-discounted'; } ?>"><?php echo $repairName;?></span><?php
								if( $is_discounted_price == '1' ) {
									if( $rpCart->get_combo_discount_percentage() > 0 ) {
										echo '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
									} else {
										echo '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), '<span class="percent-off-currency">'.$localization_data[0]->set_currency.'</span>', FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
									}
								}
								?></p>
								<p class="text price">
									<?php if($decimal == 'comma'){
										$r_price = str_replace('.',',',$repairPrice);
									}else{
										$r_price = $repairPrice;					
									}
									if( $is_por === TRUE ) {
										echo $wp_repair_lang['price_on_request_text'];
									} else {
										echo rp_handle_currency_position($r_price, $currency_symbol, TRUE);
									}
									?>
								</p>
							</div>
							<?php 
						}
					}else{
						if($itReallyWorked === FALSE) {
							$itReallyWorked = TRUE;
							$repairName = $rp->repair_name ?? $rp->r_name;
							$is_discounted_price = '';
							if( property_exists( $rp, 'repair_price' ) ) {
								list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $rp->r_id_fk, 'repair' );
								$repairPrice = $discounted_price;
							} else {
								$repairPrice = $rp->r_price;
							}
							?>
							<div class="tr">
								<p class="text desc"><span class="full-repair-name <?php if( $is_discounted_price == '1' ) { echo 'has-discounted'; } ?>"><?php echo $repairName;?></span><?php
								if( $is_discounted_price == '1' ) {
									if( $rpCart->get_combo_discount_percentage() > 0 ) {
										echo '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
									} else {
										echo '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), '<span class="percent-off-currency">'.$localization_data[0]->set_currency.'</span>', FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
									}
								}
								?></p>
								<p class="text price">
									<?php if($decimal == 'comma'){
										$r_price = str_replace('.',',',$repairPrice);
									}else{
										$r_price = $repairPrice;					
									}
									if( $is_por === TRUE ) {
										echo $wp_repair_lang['price_on_request_text'];
									} else {
										echo rp_handle_currency_position($r_price, $currency_symbol, TRUE);
									}
									?>
								</p>
							</div>
							<?php 
						}
					} 
				}
			}?>
			<?php if($o_upsales){ foreach($o_upsales as $up){
				$upsaleName = $up->upsale_name ?? $up->us_name;
				$upsalePrice = $up->upsale_price ?? $up->us_new_price;
				?>
				<div class="tr">
					<p class="text desc"><?php echo $upsaleName;?></p>
					<p class="text price">
						<?php if($decimal == 'comma'){
							$up_price = str_replace('.',',',$upsalePrice);
						}else{
							$up_price = $upsalePrice;					
						}
						echo rp_handle_currency_position($up_price, $currency_symbol, TRUE);
						?> 
					</p>
				</div>
			<?php } }
			if(($order->dm_cost ?? 0) != 0 && ($order->dm_cost ?? 0) != 0.00) {
				?>
				<div class="tr">
					<p class="text desc"><?php
					$dm_tkey = rp_get_delivery_method_translation_key($order->delivery_method);
					if( !empty( $dm_tkey ) && !empty( $wp_repair_lang[$dm_tkey] ?? '' ) ) {
						echo $wp_repair_lang[$dm_tkey];
					} else {
						echo str_replace('_', ' ', $order->delivery_method);
					}
					?></p>
					<p class="text price">
						<?php 
						$dm_cost = $order->dm_cost ?? 0;
						if($decimal == 'comma'){
							$dm_cost = str_replace('.',',',$dm_cost);
						}else{
							$dm_cost = $dm_cost;					
						}
						echo rp_handle_currency_position($dm_cost, $currency_symbol, TRUE);
						?> 
					</p>
				</div>
				<?php
			}
			?>
			<div class="separator"></div>
			<?php
			$needSeparator = FALSE;
			$order->combo_discount = $calculations['combo_discount'];
			if( !empty( $order->combo_discount ) && ((float) $order->combo_discount != 0 ) ) {
				$needSeparator = TRUE;
				?>
				<div class="tr">
					<p class="text desc light"><?php echo $wp_repair_lang['combo_dicount']; ?></p>
					<p class="text price offer">
						<?php if($decimal == 'comma'){
							$combo = str_replace('.',',',$order->combo_discount);
						}else{
							$combo = $order->combo_discount;					
						}
						echo rp_handle_currency_position($combo, $currency_symbol, TRUE);
						?>
					</p>
				</div>
				<?php

			}
			if( !empty( $order->coupons_amount ) && ((float) $order->coupons_amount != 0 ) ) {
				$needSeparator = TRUE;
				?>
				<div class="tr">
					<p class="text desc light"><?php if($order->coupons_code != null){echo 'Coupon';}
					if($order->coupons_type == 'percent'){
						echo ' ('.$order->coupons_amount.'%)';
					}
					?> <strong><?php echo $order->coupons_code?></strong></p>
					<p class="text price offer">
						<?php if($decimal == 'comma'){
							$coupon = str_replace('.',',',$order->coupons_amount);
						}else{
							$coupon = $order->coupons_amount;					
						}
						if($order->coupons_type == 'percent'){
							$coupon = $order->coupon_amount; // this is saved in rs_orders
						}
						if(!empty(trim($coupon))) {
							echo rp_handle_currency_position($coupon, $currency_symbol, TRUE);
						}
						?>
					</p>
				</div>
				<?php

			}
			if( $needSeparator === TRUE ) {

				?>
				<div class="separator"></div>
				<?php

			}
			?>
			<div class="tr total">
				<p class="text desc"><?php echo $wp_repair_lang['total']; ?></p>
				<p class="text price">
					<?php if($decimal == 'comma'){
						$total_p = str_replace('.',',',$order->total_price);
					}else{
						$total_p = $order->total_price;					
					}
					echo rp_handle_currency_position($total_p, $currency_symbol, TRUE);
					?>
				</p>
			</div>
			<div class="separator"></div>
		</div>
	</div>
	<?php
	$content = ob_get_contents();
	ob_end_clean();
	return $content;
}
