<?php

/**

* @wordpress-plugin

* Plugin Name:       RepairPlugin Pro

* Description:       Fast and high converting appointment plugin for repair shop owners

* Version:           1.5.8

* Requires at least: 5.2

* Requires PHP:      7.2

* Author:            RepairPlugin

* Author URI:        https://www.repairplugin.com

* Text Domain:       wp-repair-plugin

*/

use function RepairPluginPro\get_path;

// Exit if accessed directly

defined( 'ABSPATH' ) || exit;

if(!defined('WP_REPAIR_PLUGIN_FILE')) {

    define( 'WP_REPAIR_PLUGIN_FILE', __FILE__ );

    define( 'WP_REPAIR_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );

    define( 'WP_REPAIR_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );

    define( 'WP_REPAIR_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

    define( 'WP_REPAIR_ASSETS_CACHE_V', '1p5p8d13r1' );

    define( 'WP_REPAIR_SETTINGS_PLACEHOLDER_EMAIL', 'abc@gmail.com' );

    define( 'WP_REPAIR_BASE_SERVER_URL', 'https://base.repairplugin.com/' );

    define( 'WP_REPAIR_MAX_BACKGROUND_JOB_TIME', 3 );

}


$rp_cached_options = array();

function rp_delete_option( $option_name = '' ) {

      global $rp_cached_options;

      if( isset( $rp_cached_options[ $option_name ] ) ) {

            unset( $rp_cached_options[ $option_name ] );

      }

      global $wpdb;

      $options_table = $wpdb->prefix . 'options';

      $sql = "DELETE FROM `$options_table` WHERE option_name = %s";

      $sql = $wpdb->prepare( $sql, $option_name );

      $result = $wpdb->query( $sql );

      return $result;

}

function rp_update_option( $option_name = '', $option_value = '' ) {

      global $rp_cached_options;

      if( !is_numeric($option_value) && !is_integer($option_value) && is_bool( $option_value ) ) {

            $option_value = ( $option_value ) ? 'true' : 'false';

      }

      if( is_object( $option_value ) || is_array( $option_value ) ) {

            $option_value = serialize( $option_value );

      }

      $rp_cached_options[ $option_name ] = $option_value;

      global $wpdb;

      $options_table = $wpdb->prefix . 'options';

      // check if have option_name in options table
      $sql = "SELECT option_name FROM `$options_table` WHERE option_name = %s";

      $sql = $wpdb->prepare( $sql, $option_name );

      $result = $wpdb->get_row( $sql );

      if( empty( $result ) ) {

            // insert option_name and option_value into options table
            $sql = "INSERT INTO `$options_table` (option_name, option_value) VALUES (%s, %s)";

            $sql = $wpdb->prepare( $sql, $option_name, $option_value );

            $result = $wpdb->query( $sql );

            return $result;

      }

      $sql = "UPDATE `$options_table` SET option_value = %s WHERE option_name = %s";

      $sql = $wpdb->prepare( $sql, $option_value, $option_name );

      $result = $wpdb->query( $sql );

      return $result;

}

function rp_add_option( $option_name = '', $option_value = '' ) {

      global $rp_cached_options;

      if( !is_numeric($option_value) && !is_integer($option_value) && is_bool( $option_value ) ) {

            $option_value = ( $option_value ) ? 'true' : 'false';

      }

      if( is_object( $option_value ) || is_array( $option_value ) ) {

            $option_value = serialize( $option_value );

      }

      $rp_cached_options[ $option_name ] = $option_value;

      global $wpdb;

      $options_table = $wpdb->prefix . 'options';

      $sql = "INSERT INTO `$options_table` (option_name, option_value) VALUES (%s, %s)";

      $sql = $wpdb->prepare( $sql, $option_name, $option_value );

      $result = $wpdb->query( $sql );

      return $result;

}

function rp_filter_get_option_before_return( $result = '' ) {

      if( empty( $result ) || !is_string($result) ) {

            return $result;

      }

      if( is_serialized( $result ) ) {

            $result = unserialize( $result );

      }

      if( $result === 'true' ) {

            $result = true;

      }

      if( $result === 'false' ) {

            $result = false;

      }

      return $result;

}

function rp_get_option( $option_name = '', $default = '' ) {

      global $rp_cached_options;

      global $wpdb;

      $options_table = $wpdb->prefix . 'options';

      if( empty($rp_cached_options) ) {

            // select * from options table where option_name start with rp_ or wp_repair_
            $sql = "SELECT option_name, option_value FROM `$options_table` WHERE option_name LIKE 'rp_%' OR option_name LIKE 'wp_repair_%'";

            $results = $wpdb->get_results( $sql );

            if( !empty( $results ) ) {

                  foreach( $results as $result ) {

                        $rp_cached_options[ $result->option_name ] = rp_filter_get_option_before_return($result->option_value);

                  }

            }

      }

      if( !empty( $rp_cached_options ) && array_key_exists($option_name, $rp_cached_options) ) {

            return $rp_cached_options[ $option_name ];

      }

      // prepare sql using $wpdb->prepare() method

      $sql = "SELECT option_value FROM `$options_table` WHERE option_name = %s";

      $sql = $wpdb->prepare( $sql, $option_name );

      $result = $wpdb->get_row( $sql );

      if( empty( $result ) ) {

            $rp_cached_options[ $option_name ] = $default;

            return $default;

      }

      $result = $result->option_value;

      if( empty( $result ) ) {

            $rp_cached_options[ $option_name ] = $result;

            return $result;

      }

      $result = rp_filter_get_option_before_return( $result );

      $rp_cached_options[ $option_name ] = $result;

      return $result;

}


// Load an Autoloader which will load
// PHP Classes files present in the Repairplugin-pro/classes folder
function wp_repair_classes_autoloader() {

      global $rpQuery, $wpdb;

      $fileName = WP_REPAIR_PLUGIN_PATH.'classes/Autoloader.php';

      $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

      require_once $fileName;

      $rpQuery = new RepairPluginPro\RP_Queries_Handler( $wpdb );

}

wp_repair_classes_autoloader();



function rp_show_validation_error_message() {

      $error = RepairPluginPro\RP_Form_Inputs::get_validation_error();

      if( !empty( $error ) ) {

            ?>
            <style type="text/css">
                  .rp_validation_error {
                        margin-top: 15px;
                        margin-bottom: 0px;
                        background: #f8d7da;
                        color: #721c24;
                        border-radius: 8px;
                        border: 2px solid #f5c6cb;
                        padding: 12px;
                        font-size: 13px;
                        width: calc(100% - 60px);
                        line-height: 1.4em;
                  }
            </style>
            <div class="rp_validation_error"><?php echo $error; ?></div>
            <?php
            
      }

}

function rp_escape_sql( $string = '' ) {

      global $rpQuery;

      return $rpQuery->rp_escape_sql( $string );

}


function rpDebugPrint() {

      RepairPluginPro\RP_Log_Information::init( WP_REPAIR_PLUGIN_PATH . 'rpdebug.log' );

      RepairPluginPro\RP_Log_Information::debug();

}


function rpDebug( $message = '', $additional = array() ) {

      RepairPluginPro\RP_Log_Information::init( WP_REPAIR_PLUGIN_PATH . 'rpdebug.log' );

      RepairPluginPro\RP_Log_Information::log( $message, $additional );

}


function rp_ajax_for_public( $ajax_action = '', $callback = null ) {

      RepairPluginPro\RP_Ajax_Handler::ajax_for_public( $ajax_action, $callback );

}

function rp_ajax_for_admin( $ajax_action = '', $callback = null ) {

      RepairPluginPro\RP_Ajax_Handler::ajax_for_admin( $ajax_action, $callback );

}

// sanitize the input before printing it on the page
function rp_sanitize_output( $string = '' ) {

      if( empty( $string ) ) {

            return $string;

      }

      $org = $string;

      if( is_array( $string ) && !empty( $string ) && isset($string[0]) && is_object($string[0]) ) {

            foreach ($string as $key => $value) {

                  foreach ($value as $k => $v) {

                        $string[$key]->$k = RepairPluginPro\RP_Request_Handler::strip_tags_recursive( $v );

                  }

            }

            return $string;

      } else if(is_object($string)) {

            foreach ($string as $key => $value) {

                  $string->$key = RepairPluginPro\RP_Request_Handler::strip_tags_recursive( $value );

            }

            return $string;

      } else {
            
      }

      $string = RepairPluginPro\RP_Request_Handler::strip_tags_recursive( $string );

      $exclude_keys = array('ship_device_instructions', '_email_template');

      if( is_array( $org ) ) {

            foreach ($org as $key => $value) {

                  foreach ($exclude_keys as $exclude_key) {

                        if( strpos($key, $exclude_key) !== false ) {

                              $string[$key] = RepairPluginPro\RP_Request_Handler::strip_tags_recursive( $value, 'disallowed', array('script', 'link') );

                        }

                  }

            }

      }

      return $string;

}

// function to allow only snake case

function rp_snake_case($string = '') {
      
      /* Added in V1.4.4 */

      $string = preg_replace('/\s+/', '_', trim($string));

      $string = preg_replace('/[^A-Za-z0-9\_]/', '', $string);

      $string = preg_replace('/\_+/', '_', $string);

      return trim(strtolower($string));

}


function wp_repair_polylang_compatibility( $repair_language_setting = 'English (United States)' ) {

      if(function_exists('pll_current_language') && function_exists('pll_default_language') && defined('POLYLANG_BASENAME')) {

            $locale = pll_current_language('locale');
            if(empty($locale)) {
                  $locale = pll_default_language('locale');
            }

            if(isset($locale) && !empty($locale)) {
                  if(strpos(strtolower($locale), 'nl_') !== FALSE) {
                        return 'Dutch (Netherlands)';
                  } else if(strpos(strtolower($locale), 'en_') !== FALSE) {
                        return 'English (United States)';
                  } else {
                        return $repair_language_setting;
                  }
            }

      } 

      return $repair_language_setting;
      
}

function rp_get_selected_language() {

      global $rpQuery;

      $localization = $rpQuery->prefix . "rs_localization";

      $result       = $rpQuery->get_row("SELECT * FROM $localization LIMIT 1;");

      $selected = wp_repair_polylang_compatibility($result->set_language);

      if( rp_get_option('rp_enable_language_selector', '1') == '1' ) {

            $need_to_read = false;

            if( function_exists('is_admin') && !is_admin() && !defined('REST_REQUEST') ) {

                  $need_to_read = true;

            } else if( !empty($_POST['action'] ?? '') && (strpos($_POST['action'], 'rp_') === 0 || strpos($_POST['action'], 'me_post_pdf') === 0 || strpos($_POST['action'], 'get_schedules') === 0  || strpos($_POST['action'], 'get_schedules_new_dm') === 0) ) {
                  
                  $need_to_read = true;

            } else {

                  $need_to_read = false;

            }

            if( $need_to_read === true ) {

                  RepairPluginPro\RP_Session_Handler::read();

                  if( isset( $_SESSION ) && isset( $_SESSION['rp_selected_language'] ) && !empty( $_SESSION['rp_selected_language'] ) ) {

                        $selected = $_SESSION['rp_selected_language'];

                  }

            }

      }

      return trim($selected);

}

function rp_get_static_translations_based_on_category() {

      $categories = get_all_static_translations_category();

      $key_value_pair = rp_get_where_translation_is_used();

      $translations = array();

      foreach($categories as $category) {

            $translations[$category] = array();

            foreach($key_value_pair as $key => $value) {

                  if( $value === $category ) {

                        $translations[$category][] = $key;

                  }

            }


      }

      return $translations;

}

function get_all_static_translations_category() {

      $key_value_pair = rp_get_where_translation_is_used();

      $unique_categories = array(
            'Step 1',
            'Step 2',
            'Step 3',
            'Step 4',
            'Offer Popup',
            'Step 5',
            'Step 6',
            'Come By Our Store',
            'Ship Device',
            // NEWLY ADDED TRANSLATIONS - START
            'Pickup Service',
            'Repair on your location',
            // NEWLY ADDED TRANSLATIONS - END
            'Step wizard',
            'Offer PDF',
            'Offer PDF Filename',
            'Emails',
            'Searchbar',
            'Canceled Page',
            'License issue'
      );

      foreach ($key_value_pair as $key => $value) {

            // if value is not in array then add it

            if( $value === 'Remove' || $value === 'Not used' ) {
                  continue;
            }

            if( !in_array($value, $unique_categories) ) {

                  $unique_categories[] = $value;

            }

      }

      return $unique_categories;
}

function rp_get_where_translation_is_used() {

      $pair = array(
            'please_wait' => 'Offer Popup',

            'pdf_email_sent' => 'Offer Popup',

            'please_fill_all_pdf_fields' => 'Offer Popup',

            'service' => 'Offer PDF',

            'excluding_short_form' => 'Offer PDF',

            'amount' => 'Not used',

            'pdf_repair_amount' => 'Offer PDF',

            'total' => 'Offer PDF',

            'tax' => 'Offer PDF',

            'discount' => 'Offer PDF',

            'subtotal' => 'Offer PDF',

            'do_not_hesitate_to_contact_us' => 'Offer PDF',

            'kind_regards' => 'Offer PDF',

            'dear' => 'Offer PDF',

            'device_details' => 'Offer PDF',

            'imei' => 'Offer PDF',

            'appointment_offer_thank_you_message' => 'Offer PDF',
            
            'appointment_offer_footer' => 'Offer PDF',

            'finalize_appointment' => 'Emails',

            'number' => 'Offer PDF',

            'quotation' => 'Offer PDF',

            'date' => 'Not used',

            'select' => 'Step wizard',

            'review_all_page' => 'Not used',

            'type' => 'Step 1',

            'types' => 'Not used',

            'brands' => 'Not used',

            'brand' => 'Step 2',

            'no_brands_available' => 'Step 2',

            'no_models_available' => 'Step 3',

            'models' => 'Not used',

            'model' => 'Step 3',

            'repairs' => 'Step wizard',

            'repair'  => 'Step wizard',

            'order'  => 'Step wizard',

            'device'  => 'Step wizard',

            'device_mobile' => 'Step wizard',

            'thanks'  => 'Not used',

            'come_by_our_store' =>'Come By Our Store',

            'i_accept_terms' => 'Step 5',

            'please_accept_terms' => 'Step 5',

            'ship_device' =>'Ship Device',

            'Repair_within_24_hours' => 'Ship Device',

            'ship_device_instructions' =>'Ship Device',

            // NEWLY ADDED TRANSLATIONS -- START

            'user_location_zipcode' => 'Step 5',

            'user_location_house_number' => 'Step 5',

            'user_location_street_address' => 'Step 5',

            'user_location_city' => 'Step 5',

            'let_us_pickup_your_device' => 'Pickup Service',

            'pickup_service_description' => 'Pickup Service',

            'repair_on_your_location' => 'Repair on your location',
            
            'repair_on_your_location_description' => 'Repair on your location',

            // NEWLY ADDED TRANSLATIONS -- END

            'finalize_order' =>'Not used',

            'thank_you_page' => 'Not used',

            'select_device' => 'Not used',

            'select_repair' => 'Step 4',

            'select_model' => 'Not used',

            'type_model_name' => 'Step 3',

            'type_brand_name' => 'Step 2',

            'select_type' => 'Not used',

            'select_color' => 'Step 4',

            'minutes' => 'Step 4',

            'minutes_singular' => 'Step 4',
            'minutes_plural' => 'Step 4',

            'days_singular' => 'Step 4',

            'days_plural' => 'Step 4',

            'hours_singular' => 'Step 4',

            'hours_plural' => 'Step 4',

            'show_all_x_repairs' => 'Step 4',

            'no_repairs_available' => 'Step 4',

            'special_deals_for_you' =>'Step 4',

            'this_is_what_we_will_repair_for_you' => 'Step 4',

            'sub_total' => 'Step 4',

            'combo_dicount' => 'Step 4',

            'coupon_code' => 'Step 4',

            'invalid_coupon' => 'Step 4',

            'add_to' => 'Step 6',

            'calendar' => 'Step 6',

            'open_in_google_maps' => 'Step 6',

            'coupon' => 'Step 5',

            'coupon_discount' => 'Offer PDF',

            'total' => 'Offer PDF',

            'incl' => 'Step 4',

            'excluding' => 'Step 5',

            'send_offer_pdf' => 'Step 4',

            'directly_in_your_mailbox' => 'Step 4',

            'finalize' => 'Step wizard',

            'booking' => 'Step 5',

            'you_only_pay_after_your_repair' => 'Step 4',

            'receive_your_offer' => 'Offer Popup',

            'by_email' => 'Offer Popup',

            'please_fill_in_all_necessary_information' => 'Offer Popup',

            'name' => 'Offer Popup',

            'streetname_number' =>'Offer Popup',

            'zipcode_city' => 'Offer Popup',

            'your_email_address' => 'Offer Popup',

            'imei_number' => 'Offer Popup',

            'send_offer' => 'Offer Popup',

            'select_delivery_method' => "Step 5",

            'done_while_you_are_waiting' => 'Come By Our Store',

            'choose_location' => 'Step 5',

            'select_date_and_time' => "Step 5",

            'private' => 'Step 5',

            'business' => 'Step 5',

            'your_name' => 'Step 5',

            'business_name' => 'Step 5',

            'your_phone_number' => 'Step 5',  
            
            'notes' => 'Step 5',

            'confirm_booking' => 'Step 5',

            'you_can_cancel_anytime' => 'Step 5',

            'thanks_for_purchasing_from_us!' => 'Step 6',

            'you_will_receive_a_confirmation_by_email' => 'Step 6',

            'please_choose_color_and_any_repair/upsale_first_before_proceeding' => 'Step 4',
            'appointment' => 'Not used',
            'appointment_on' => 'Step 5',

            'appointment_only' => 'Step 5',
            'on/op_only' => 'Step 5',

            'invalid_license_code' => 'Remove',
            'enter_your_license_key_here' => 'Remove',
            'do_you_need_help' => 'Remove',
            'if_you_dont_have_license' => 'Remove',
            'license_code' => 'Remove',
            'license_email' => 'Remove',
            'we_will_send_updated_news_of_this_product' => 'Remove',

            'shop_is_closed_for_today' => 'Step 5',
            'shop_has_closed_for_today' => 'Step 5',
            'appointment_booking_has_closed_for_today' => 'Step 5',
            'we_are_on_a_break' => 'Step 5',
            'please_check_other_times' => 'Step 5',
            'this_timeslot_is_fully_booked' => 'Step 5',

            'translation_of_day_mon' => 'Step 5',
            'translation_of_day_tue' => 'Step 5',
            'translation_of_day_wed' => 'Step 5',
            'translation_of_day_thu' => 'Step 5',
            'translation_of_day_fri' => 'Step 5',
            'translation_of_day_sat' => 'Step 5',
            'translation_of_day_sun' => 'Step 5',

            'breaktime' => 'Step 5',

            'weekday_monday' => 'Step 5',
            'weekday_tuesday' => 'Step 5',
            'weekday_wednesday' => 'Step 5',
            'weekday_thursday' => 'Step 5',
            'weekday_friday' => 'Step 5',
            'weekday_saturday' => 'Step 5',
            'weekday_sunday' => 'Step 5',

            'registeration_id' => 'Offer PDF',
            'customer_details' => 'Offer PDF',
            'expiration_date' => 'Offer PDF',
            'quotation_date' => 'Offer PDF',

            'pdf_offer_filename' => 'Offer PDF Filename',

            'order_summary' => 'Emails',
            'cancel_booking' => 'Emails',

            'fe_no_license_with_contact' => 'License issue',
            'fe_no_license_no_contact' => 'License issue',

            'please_select_appointment_time' => 'Step 5',

            'repair_sidebar_next_step' => 'Step 4',

            'thank_you_at' => 'Step 5',

            'thank_you_address_details' => 'Step 6',

            'finalize_view_order' => 'Step 5',

            'finalize_hide_order' => 'Step 5',

            'please_select_any_delivery_method' => 'Step 5', // Please select any delivery method

            'search_for_nearest_location' => 'Step 5', // Enter your address to find nearest location

            'approx_radius_distance' => 'Step 5', // Approx. {distance} Radius Distance

            'browser_does_not_support_geolocation' => 'Step 5', // Your browser doesn't support geolocation! Please try search instead.

            'error_getting_geolocation' => 'Step 5', // Error getting location: {error}

            'address_is_too_short' => 'Step 5', // Address is too short!

            'live_location_btn_text' => 'Step 5', // Live Location

            'nearest_location_distance_unit_km' => 'Step 5', // KM

            'nearest_location_distance_unit_miles' => 'Step 5', // Miles

            'send_device_to' => 'Step 6',

            'free_repair_text' => 'Step 4', //vrije 
            'price_on_request_text' => 'Step 4', // Prijs op aanvraag
            'repair_starting_at' => 'Step 4', // vanaf

            'no_results_found' => 'Searchbar',

            'search_bar_placeholder_text' => 'Searchbar',

            'booking_cancelled' => 'Canceled Page',

            'your_appointment_has_been_cancelled' => 'Canceled Page',

            'the_price_was_updated_on' => 'Step 4',

            'combo_discount_percent_off' => 'Step 4'

      );

      return apply_filters('rp_get_where_translation_is_used', $pair);

}

function rp_get_static_translations_of_language($language = 'English (United States)') {

      if($language == 'Dutch (Netherlands)'){

            $pair = array(
                'please_wait' => 'Even wachten a.u.b.',
                'pdf_email_sent' => 'Email is succesvol verzonden naar je emailbox.',
                'please_fill_all_pdf_fields' => 'Vul a.u.b. alle velden in.',
                'service' => 'Diensten',
                'excluding_short_form' => 'excl.',
                'amount' => 'Aantal',
                'pdf_repair_amount' => 'Bedrag',
                'total' => 'Totaal',
                'tax' => 'BTW',
                'discount' => 'Combinatiekorting',
                'subtotal' => 'Subtotaal',
                'do_not_hesitate_to_contact_us' => 'We hopen u hiermee voldoende geïnformeerd te hebben. Indien u vragen heeft, neem dan alstublieft contact met ons op.',
                'kind_regards' => 'Met vriendelijke groet,',
                'dear' => 'Beste',
                'device_details' => 'Toestelgegevens',
                'imei' => 'IMEI',
                'appointment_offer_thank_you_message' => 'Hierbij ontvangt u van ons de prijsopgave voor de onderstaande diensten.',
                'appointment_offer_footer' => 'Deze offerte is afkomstig van onze website en is bedoeld als richtlijn. Aan de informatie in deze offerte kunnen geen rechten worden ontleend. Wij zijn niet aansprakelijk voor eventuele fouten of verkeerde informatie.',
                'finalize_appointment' => 'Voltooi de afspraak',
                'number' => 'Nummer',
                'quotation' => 'Offerte',
                'date' => 'Datum',
                'select' => 'Selecteer',
                'review_all_page' => 'bekijk alle pagina`s',
                'type' => 'type',
                'types' => 'types',
                'brands' => 'merken',
                'brand' => 'merk',
                'no_brands_available' => 'Geen merken beschikbaar',
                'no_models_available' => 'Geen modellen beschikbaar',
                'models' => 'modellen',
                'model' => 'model',
                'repairs' => 'reparaties',
                'repair' => 'reparatie',
                'order'  => 'afspraak',
                'device'  => 'apparaat',
                'device_mobile'  => 'apparaten',
                'thanks'  => 'Met dank',
                'come_by_our_store' =>'Kom langs in onze winkel',
                'i_accept_terms' => 'Ik accepteer de [algemene voorwaarden]',
                'please_accept_terms' => 'Accepteer de algemene voorwaarden',
                'ship_device' =>'Verstuur apparaat',
                'ship_device_instructions' =>'$ShipDeviceAddress$',
                // NEWLY ADDED TRANSLATIONS -- START
                  'user_location_zipcode' => 'Postcode',
                  'user_location_house_number' => 'Huisnummer',
                  'user_location_street_address' => 'Straat adres',
                  'user_location_city' => 'STAD',
                  'let_us_pickup_your_device' => 'Laat ons uw toestel ophalen',
                  'pickup_service_description' => 'Reparatie binnen 24 uur',
                  'repair_on_your_location' => 'Reparatie bij u op locatie',
                  'repair_on_your_location_description' => 'Klaar terwijl u wacht',
                  // NEWLY ADDED TRANSLATIONS -- END
                'finalize_order' =>'Bestelling afronden',
                'thank_you_page' => 'bedankt pagina',
                'select_device' => 'selecteer apparaat',
                'select_repair' => 'SELECTEER REPARATIE',
                'select_model' => 'Selecteer model',
                'type_model_name' => 'Typ modelnaam',
                'type_brand_name' => 'Typ merknaam',
                'select_type' => 'Selecteer type',
                'select_color' => 'SELECTEER KLEUR',
                'minutes' => 'minuten',
                'minutes_singular' => 'minuut',
                'minutes_plural' => 'minuten',
                'days_singular' => 'dag',
                'days_plural' => 'dagen',
                'hours_singular' => 'uur',
                'hours_plural' => 'uren',
                'show_all_x_repairs' => 'Toon alle % reparaties',
                'no_repairs_available' => 'Geen reparaties beschikbaar',
                'special_deals_for_you' =>'Aanbevolen accessoires',
                'this_is_what_we_will_repair_for_you' => 'Reparatie lijst',
                'sub_total' => 'Subtotaal',
                'combo_dicount' => 'Combinatiekorting',
                'coupon_code' => 'Coupon Code',
                'invalid_coupon' => 'Invalid Coupon',
                'add_to' => 'Add to',
                'calendar' => 'Calendar',
                'open_in_google_maps' => 'Open in',
                'coupon' => 'Coupon',
                'coupon_discount' => 'Coupon Korting',
                'total' => 'Totaal',
                'incl' => 'incl',
                'excluding' => 'Exclusief',
                'send_offer_pdf' => 'Stuur Offerte in PDF',
                'directly_in_your_mailbox' => 'Direct in je mailbox',
                'finalize' => 'Maak',
                'booking' => 'een afspraak',
                'you_only_pay_after_your_repair' => 'Je betaalt pas na de reparatie',
                'receive_your_offer' => 'Ontvang een offerte',
                'by_email' => 'per e-mail',
                'please_fill_in_all_necessary_information' => 'Vul a.u.b. alle benodigde informatie in',
                'name' => 'NAAM',
                'streetname_number' =>'STRAATNAAM + NUMMER',
                'zipcode_city' => 'POSTCODE + STAD',
                'your_email_address' => 'JOUW E-MAILADRES',
                'imei_number' => 'IMEI NUMMER',
                'notes' => 'NOTITIES',
                'send_offer' => 'STUUR OFFERTE',
                'select_delivery_method' => 'SELECTEER BEZORGMETHODE',
                'done_while_you_are_waiting' => 'Klaar terwijl je wacht',
                'choose_location' => 'KIES LOCATIE',
                'select_date_and_time' => 'SELECTEER DATUM EN TIJD',
                'Repair_within_24_hours' => 'Reparatie binnen 24 uur',
                'private' => 'Particulier',
                'business' => 'Zakelijk',
                'your_name' => 'NAAM',
                'business_name' => 'Bedrijfsnaam',
                'your_phone_number' => 'TELEFOON NUMMER',            
                'confirm_booking' => 'Bevestig afspraak',
                'you_can_cancel_anytime' => 'Je kunt op elk moment annuleren',
                'thanks_for_purchasing_from_us!' => 'Bedankt voor het maken van een afspraak!',
                'you_will_receive_a_confirmation_by_email' => 'Je ontvangt een bevestiging per e-mail',
                'please_choose_color_and_any_repair/upsale_first_before_proceeding' => 'Kies a.u.b. een kleur voordat je verder kunt',
                'appointment' => 'Afspraak op',
                'appointment_on' => 'Afspraak op',
    
                'appointment_only' => 'Afspraak',
                'on/op_only' => 'op',
    
                'invalid_license_code' => 'Licentie code is niet juist',
                'enter_your_license_key_here' => 'Voer de licentie code in om het product te activeren en krijg updates en premium support',
                'do_you_need_help' => 'Heb je hulp nodig? <a target="_blank" href="https://help.repairplugin.com/">Ons helpcentrum</a> bevat antwoorden op de meeste van uw vragen.',
                'if_you_dont_have_license' => 'Als je geen licentie hebt of de licentie is verlopen, dan kun je er één bestellen op <a target="_blank" href="https://www.repairplugin.com/nl/prijzen/">www.repairplugin.com/nl/prijzen</a>. Als er iets mis is gegaan en de licentie code werkt niet, neem dan contact met ons op via <a href="mailto:support@repairplugin.com">support@repairplugin.com</a>',
                'license_code' => 'Licentie code',
                'license_email' => 'Emailadres',
                'we_will_send_updated_news_of_this_product' => 'Wij sturen nieuws over dit product via dit e-mailadres, maak je geen zorgen, wij haten spam.',
    
                'shop_is_closed_for_today' => 'Vandaag gesloten!',
                'shop_has_closed_for_today' => 'De winkel is voor vandaag gesloten!',
                'appointment_booking_has_closed_for_today' => 'Het boeken van een afspraak voor vandaag is gesloten!',
                'we_are_on_a_break' => 'Unfortunately, we\'re on a break at this time slot.',
                'please_check_other_times' => 'Sorry, u kunt op zeer korte termijn geen afspraak maken. Controleer de andere beschikbare tijden.',
                'this_timeslot_is_fully_booked' => 'Dit tijdslot is volgeboekt.',
    
                'translation_of_day_mon' => 'Ma',
                'translation_of_day_tue' => 'Di',
                'translation_of_day_wed' => 'Wo',
                'translation_of_day_thu' => 'Do',
                'translation_of_day_fri' => 'Vr',
                'translation_of_day_sat' => 'Za',
                'translation_of_day_sun' => 'Zo',
    
                'breaktime' => 'Lunchpauze',
    
                'weekday_monday' => 'Maandag',
                'weekday_tuesday' => 'Dinsdag',
                'weekday_wednesday' => 'Woensdag',
                'weekday_thursday' => 'Donderdag',
                'weekday_friday' => 'Vrijdag',
                'weekday_saturday' => 'Zaterdag',
                'weekday_sunday' => 'Zondag',
    
                'registeration_id' => 'kvk',
                'customer_details' => 'KLANTGEGEVENS',
    
                'expiration_date' => 'Vervaldatum',
                'quotation_date' => 'Offertedatum',

                'pdf_offer_filename' => 'Offerte',
    
                'order_summary' => 'Order overzicht',
                'cancel_booking' => 'Annuleer afspraak',
    
                'fe_no_license_with_contact' => 'Ons afsprakensysteem is tijdelijk niet beschikbaar. Neem contact met ons op: $contact$',
                'fe_no_license_no_contact' => 'Ons afsprakensysteem is tijdelijk niet beschikbaar.',
    
                'please_select_appointment_time' => 'Selecteer Afspraaktijd',
    
                'repair_sidebar_next_step' => 'Volgende stap',
    
                'thank_you_at' => 'om',
    
                'thank_you_address_details' => 'Adresgegevens',
    
                'finalize_view_order' => 'bestelling bekijken',
    
                'finalize_hide_order' => 'bestelling verbergen',

                'please_select_any_delivery_method' => 'Please select any delivery method', // Please select any delivery method

                'search_for_nearest_location' => 'Enter your address to find nearest location', // Enter your address to find nearest location

                  'approx_radius_distance' => 'Approx. {distance} Radius Distance', // Approx. {distance} Radius Distance

                  'browser_does_not_support_geolocation' => 'Your browser doesn\'t support geolocation! Please try search instead.', // Your browser doesn't support geolocation! Please try search instead.

                  'error_getting_geolocation' => 'Error getting location: {error}', // Error getting location: {error}

                  'address_is_too_short' => 'Address is too short!', // Address is too short!

                  'live_location_btn_text' => 'Live Location', // Live Location

                  'nearest_location_distance_unit_km' => 'km', // KM

                  'nearest_location_distance_unit_miles' => 'miles', // Miles
    
                'send_device_to' => 'Verstuur apparaat naar:',
    
                'free_repair_text' => 'GRATIS', //vrije 
                'price_on_request_text' => 'Prijs op aanvraag', // Prijs op aanvraag
                'repair_starting_at' => 'vanaf', // vanaf
    
                'no_results_found' => 'Geen resultaten gevonden.',
    
                'search_bar_placeholder_text' => 'Zoek je merk, model of modelcode',

                'booking_cancelled' => 'Boeking geannuleerd', // Boeking geannuleerd

                'your_appointment_has_been_cancelled' => 'Uw afspraak is geannuleerd.', // Uw afspraak is geannuleerd

                'the_price_was_updated_on' => 'De prijs is bijgewerkt op [date]',

                'combo_discount_percent_off' => 'OFF'
            );

      } else {

            $pair = array(
                  'please_wait' => 'Please wait',
      
                  'pdf_email_sent' => 'Email sent successfully, please check your mailbox.',
      
                  'please_fill_all_pdf_fields' => 'Please fill all fields',
      
                  'service' => 'Service',

                  'excluding_short_form' => 'excl.',
      
                  'amount' => 'Amount',
      
                  'pdf_repair_amount' => 'Amount',
      
                  'total' => 'Total',
      
                  'tax' => 'Tax',
      
                  'discount' => 'Discount',
      
                  'subtotal' => 'Subtotal',
      
                  'do_not_hesitate_to_contact_us' => 'If you have any questions, please do not hesitate to contact us.',
      
                  'kind_regards' => 'Kind regards,',
      
                  'dear' => 'Dear',

                  'device_details' => 'Device details',

                  'imei' => 'IMEI',
      
                  'appointment_offer_thank_you_message' => 'Thank you for your interest in our services. We hereby make you the following offer:',

                  'appointment_offer_footer' => 'This quotation was generated from our website and is intended as a guideline. No rights can be derived from the information in this quotation. We are not liable for any errors or misinformation.',
      
                  'finalize_appointment' => 'Finalize Appointment',
      
                  'number' => 'Number',
      
                  'quotation' => 'Quotation',
      
                  'date' => 'Date',
      
                  'select' => 'Select',
      
                  'review_all_page' => 'review all pages',
      
                  'type' => 'type',
      
                  'types' => 'types',
      
                  'brands' => 'brands',
      
                  'brand' => 'Brand',

                  'no_brands_available' => 'No Brands Available',

                  'no_models_available' => 'No Models Available',
      
                  'models' => 'models',
      
                  'model' => 'model',
      
                  'repairs' => 'repairs',
      
                  'repair'  => 'repair',
      
                  'order'  => 'order',
      
                  'device'  => 'device',

                  'device_mobile'  => 'devices',
      
                  'thanks'  => 'thanks',
      
                  'come_by_our_store' =>'Come by our store',

                  'i_accept_terms' => 'I accept the [terms & conditions]',

                  'please_accept_terms' => 'Please accept terms & conditions',
      
                  'ship_device' =>'Ship device',

                  'ship_device_instructions' =>'$ShipDeviceAddress$',

                  // NEWLY ADDED TRANSLATIONS -- START
                  'user_location_zipcode' => 'Zipcode',
                  
                  'user_location_house_number' => 'House number',
                  
                  'user_location_street_address' => 'Street address',

                  'user_location_city' => 'City',

                  'let_us_pickup_your_device' => 'Let us pickup your device',

                  'pickup_service_description' => 'Repair within 24 hours',

                  'repair_on_your_location' => 'Repair on your location',
                  
                  'repair_on_your_location_description' => 'Done while you are waiting',
                  // NEWLY ADDED TRANSLATIONS -- END
      
                  'finalize_order' =>'Finalize order',
      
                  'thank_you_page' => 'Thank you page',
      
                  'select_device' => 'select device',
      
                  'select_repair' => 'SELECT REPAIR',
      
                  'select_model' => 'Select model',
      
                  'type_model_name' => 'Type model name',
      
                  'type_brand_name' => 'Type brand name',
      
                  'select_type' => 'select type',
      
                  'select_color' => 'SELECT COLOR',
      
                  'minutes' => 'minutes',
      
                  'minutes_singular' => 'minute',
                  'minutes_plural' => 'minutes',
      
                  'days_singular' => 'day',
      
                  'days_plural' => 'days',
      
                  'hours_singular' => 'hour',
      
                  'hours_plural' => 'hours',
      
                  'show_all_x_repairs' => 'Show all % repairs',

                  'no_repairs_available' => 'No repairs available',
      
                  'special_deals_for_you' =>'Recommended accessories',
      
                  'this_is_what_we_will_repair_for_you' => 'Repair List',
      
                  'sub_total' => 'sub-total',
      
                  'combo_dicount' => 'Combo-discount',
      
                  'coupon_code' => 'Coupon Code',

                  'invalid_coupon' => 'Invalid Coupon',

                  'add_to' => 'Add to',
                  'calendar' => 'Calendar',
                  'open_in_google_maps' => 'Open in',

                  'coupon' => 'Coupon',
      
                  'coupon_discount' => 'Coupon Discount',
      
                  'total' => 'Total',
      
                  'incl' => 'incl',
      
                  'excluding' => 'Excluding',
      
                  'send_offer_pdf' => 'Send Offer PDF',
      
                  'directly_in_your_mailbox' => 'Directly in your mailbox',
      
                  'finalize' => 'Finalize',
      
                  'booking' => 'booking',
      
                  'you_only_pay_after_your_repair' => 'You only pay after your repair',
      
                  'receive_your_offer' => 'Receive your offer',
      
                  'by_email' => 'by email',
      
                  'please_fill_in_all_necessary_information' => 'Please fill in all necessary information',
      
                  'name' => 'NAME',
      
                  'streetname_number' =>'STREETNAME + NUMBER',
      
                  'zipcode_city' => 'ZIPCODE + CITY',
      
                  'your_email_address' => 'EMAIL',

                  'imei_number' => 'IMEI NUMBER',
      
                  'notes' => 'NOTES',
      
                  'send_offer' => 'SEND OFFER',
      
                  'select_delivery_method' => "SELECT DELIVERY METHOD",
      
                  'done_while_you_are_waiting' => 'Done while you are waiting',
      
                  'choose_location' => 'CHOOSE LOCATION',
      
                  'select_date_and_time' => "SELECT DATE AND TIME",
      
                  'Repair_within_24_hours' => 'Repair within 24 hours',
      
                  'private' => 'Private',
      
                  'business' => 'Business',
      
                  'your_name' => 'NAME',
      
                  'business_name' => 'Bussiness Name',
      
                  'your_phone_number' => 'PHONE',            
      
                  'confirm_booking' => 'Confirm booking',
      
                  'you_can_cancel_anytime' => 'You can cancel anytime',
      
                  'thanks_for_purchasing_from_us!' => 'Thank you for purchasing from us!',
      
                  'you_will_receive_a_confirmation_by_email' => 'You will receive a confirmation by e-mail',
      
                  'please_choose_color_and_any_repair/upsale_first_before_proceeding' => 'Please choose color and any repair/upsale first before proceeding.',
                  'appointment' => 'Appointment',
                  'appointment_on' => 'Appointment on',
      
                  'appointment_only' => 'Appointment',
                  'on/op_only' => 'on',
      
                  'invalid_license_code' => 'Invalid license code',
                  'enter_your_license_key_here' => 'Enter your license key here to activate the product, and get complete feature updates and premium support.',
                  'do_you_need_help' => 'Do you need any help? <a target="_blank" href="https://help.repairplugin.com/">Our help center</a> contains answers to most of your questions.',
                  'if_you_dont_have_license' => 'If you don\'t have a license or your license is expired, you can get one on <a target="_blank" href="https://www.repairplugin.com/">repairplugin.com</a>. If there is something wrong, feel free to contact us at <a href="mailto:support@repairplugin.com">support@repairplugin.com</a>',
                  'license_code' => 'License code',
                  'license_email' => 'Email Address',
                  'we_will_send_updated_news_of_this_product' => 'We will send updated news of this product by this email address, don\'t worry, we hate spam.',
      
                  'shop_is_closed_for_today' => 'Shop is closed for today!',
                  'shop_has_closed_for_today' => 'Shop has closed for today!',
                  'appointment_booking_has_closed_for_today' => 'Appointment booking has closed for today!',
                  'we_are_on_a_break' => 'Unfortunately, we\'re on a break at this time slot.',
                  'please_check_other_times' => 'Sorry, you can\'t book an appointment on very short notice. Please check other available times.',
                  'this_timeslot_is_fully_booked' => 'This timeslot is fully booked.',
      
                  'translation_of_day_mon' => 'Mon',
                  'translation_of_day_tue' => 'Tue',
                  'translation_of_day_wed' => 'Wed',
                  'translation_of_day_thu' => 'Thu',
                  'translation_of_day_fri' => 'Fri',
                  'translation_of_day_sat' => 'Sat',
                  'translation_of_day_sun' => 'Sun',
      
                  'breaktime' => 'Breaktime',
      
                  'weekday_monday' => 'Monday',
                  'weekday_tuesday' => 'Tuesday',
                  'weekday_wednesday' => 'Wednesday',
                  'weekday_thursday' => 'Thursday',
                  'weekday_friday' => 'Friday',
                  'weekday_saturday' => 'Saturday',
                  'weekday_sunday' => 'Sunday',
      
                  'registeration_id' => 'ID',
                  'customer_details' => 'CUSTOMER DETAILS',
                  'expiration_date' => 'Expiration Date',
                  'quotation_date' => 'Quotation Date',

                  'pdf_offer_filename' => 'Offer',
      
                  'order_summary' => 'Order Summary',
                  'cancel_booking' => 'Cancel Booking',
      
                  'fe_no_license_with_contact' => 'Our appointment system is temporarily unavailable. Please contact us on: $contact$',
                  'fe_no_license_no_contact' => 'Our appointment system is temporarily unavailable.',
      
                  'please_select_appointment_time' => 'Please select appointment time',
      
                  'repair_sidebar_next_step' => 'Next step',
      
                  'thank_you_at' => 'at',
      
                  'thank_you_address_details' => 'Address Details',
      
                  'finalize_view_order' => 'view order',
      
                  'finalize_hide_order' => 'hide order',

                  'please_select_any_delivery_method' => 'Please select any delivery method', // Please select any delivery method

                  'search_for_nearest_location' => 'Enter your address to find nearest location', // Enter your address to find nearest location

                  'approx_radius_distance' => 'Approx. {distance} Radius Distance', // Approx. {distance} Radius Distance

                  'browser_does_not_support_geolocation' => 'Your browser doesn\'t support geolocation! Please try search instead.',

                  'error_getting_geolocation' => 'Error getting location: {error}', // Error getting location: {error}

                  'address_is_too_short' => 'Address is too short!', // Address is too short!

                  'live_location_btn_text' => 'Live Location', // Live Location

                  'nearest_location_distance_unit_km' => 'km', // KM

                  'nearest_location_distance_unit_miles' => 'miles', // Miles
      
                  'send_device_to' => 'Send device to:',
      
                  'free_repair_text' => 'FREE', //vrije 
                  'price_on_request_text' => 'Price on request', // Prijs op aanvraag
                  'repair_starting_at' => 'starting at', // vanaf
      
                  'no_results_found' => 'No results found.',
      
                  'search_bar_placeholder_text' => 'Please enter your brand, model or modelcode',

                  'booking_cancelled' => 'Booking cancelled', // Boeking geannuleerd

                  'your_appointment_has_been_cancelled' => 'Your appointment has been canceled.', // Uw afspraak is geannuleerd

                  'the_price_was_updated_on' => 'The price was updated on [date]',

                  'combo_discount_percent_off' => 'OFF'
      
            );

      }

      return apply_filters('rp_returning_static_translations_pair', $pair, $language);

}

function rp_get_dynamic_translations_of_language( $language = '' ) {

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . 'rs_languages';

      $table_rs_translations = $rpQuery->prefix . 'rs_translations';

      $static_translations = rp_get_static_translations_of_language( $language );

      // check if language exists in database

      $language_exists = $rpQuery->get_row( "SELECT * FROM `$table_rs_languages` WHERE `name` = '".rp_escape_sql($language)."';" );

      if ( !empty($language_exists) ) {

            $dynamic_translations = $rpQuery->get_results( "SELECT * FROM `$table_rs_translations` WHERE `language` = '".rp_escape_sql($language)."';" );

            if( !empty( $dynamic_translations ) ) {

                  foreach($dynamic_translations as $row) {

                        if( isset( $static_translations[$row->key] ) ) {

                              $static_translations[$row->key] = $row->value;

                        }

                  }

            }

      }

      return $static_translations;

}

function rp_is_valid_language_name( $set_language = '' ) {

      $all_languages = rp_get_all_languages();

      if( !empty( $all_languages ) ) {

            $all_languages_names = array();

            foreach($all_languages as $language) {

                  $all_languages_names[] = trim($language->name);

            }

            if( !in_array($set_language, $all_languages_names) ) {

                  return FALSE;
            
            }

            return TRUE;

      }

      return FALSE;

}

function rp_reset_plugin_language() {

      global $wp_repair_selected_language;

      global $wp_repair_lang;

      global $wp_repair_selected_language_org;

      global $wp_repair_lang_org;

      if( isset( $wp_repair_selected_language_org ) && !empty( $wp_repair_selected_language_org ) ) {

            $wp_repair_selected_language = $wp_repair_selected_language_org;

            $wp_repair_selected_language_org = null;

      }

      if( isset( $wp_repair_lang_org ) && !empty( $wp_repair_lang_org ) ) {

            $wp_repair_lang = $wp_repair_lang_org;

            $wp_repair_lang_org = null;

      }

}

function rp_set_new_plugin_language( $language_name = '' ) {

      global $wp_repair_selected_language;

      global $wp_repair_lang;

      global $wp_repair_selected_language_org;

      global $wp_repair_lang_org;

      $wp_repair_selected_language_org = $wp_repair_selected_language;

      $wp_repair_lang_org = $wp_repair_lang;

      if( rp_is_valid_language_name( $language_name ) === TRUE ) {

            $wp_repair_selected_language = $language_name;

            $wp_repair_lang = rp_get_dynamic_translations_of_language($language_name);

      }

}

function set_plugin_language(){

      global $wp_repair_lang;

      global $wp_repair_selected_language;

      $all_languages = rp_get_all_languages();

      $set_language = rp_get_selected_language();

      if( !empty( $all_languages ) ) {

            $all_languages_names = array();

            foreach($all_languages as $language) {

                  $all_languages_names[] = trim($language->name);

            }

            if( !in_array($set_language, $all_languages_names) ) {

                  $set_language = rp_get_default_language();
            
            }

      } else {

            $set_language = 'English (United States)';

      }

      $wp_repair_selected_language = $set_language;

      $wp_repair_lang = rp_get_dynamic_translations_of_language($set_language);
    
}

add_action('init', 'set_plugin_language', 999);

function loadTheRequiredWPRepairScript() {

    wp_enqueue_style('jquery-ui-css', 'https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.css');

    wp_enqueue_script('jquery-ui-js', 'https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.js', array('jquery'));

    if(isset($_GET['page'])) {

      wp_enqueue_style('font-awesome-css', 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css');

        switch ($_GET['page']) {

            case 'wp_repair_appointments':

                wp_enqueue_style('wp-repair-plugin-appointments-css', WP_REPAIR_PLUGIN_URL.'assets/css/appointments.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                break;

            case 'wp_repair_manage_categories':

                wp_enqueue_style('wp-repair-plugin-categories-css', WP_REPAIR_PLUGIN_URL.'assets/css/categories.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                break;

            case 'wp_repair_manage_models':

                wp_enqueue_style('wp-repair-plugin-repairs-css', WP_REPAIR_PLUGIN_URL.'assets/css/repairs.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                wp_enqueue_style('wp-repair-plugin-models-css', WP_REPAIR_PLUGIN_URL.'assets/css/models.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                break;

            case 'wp_repair_default_repairs':

                wp_enqueue_style('wp-repair-plugin-drepairs-css', WP_REPAIR_PLUGIN_URL.'assets/css/drepairs.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                break;

            case 'wp_repair_download_libraries':

                wp_enqueue_style('wp-repair-plugin-libraries-css', WP_REPAIR_PLUGIN_URL.'assets/css/download_libraries.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                break;

            case 'wp_repair_manage_upsales':

                wp_enqueue_style('wp-repair-plugin-upsales-css', WP_REPAIR_PLUGIN_URL.'assets/css/upsales.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                break;

            case 'wp_repair_settings':

                wp_enqueue_style('wp-repair-plugin-settings-css', WP_REPAIR_PLUGIN_URL.'assets/css/settings.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                if(isset($_GET['section'])) {

                    switch ($_GET['section']) {

                        case 'localization':

                            wp_enqueue_style('wp-repair-plugin-settings-localization-css', WP_REPAIR_PLUGIN_URL.'assets/css/localization.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                            break;

                        case 'planning_discount':

                            wp_enqueue_style('wp-repair-plugin-settings-planning_discount-css', WP_REPAIR_PLUGIN_URL.'assets/css/planning_discount.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                            break;

                        case 'coupons':

                            wp_enqueue_style('wp-repair-plugin-settings-coupons-css', WP_REPAIR_PLUGIN_URL.'assets/css/coupons.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                            break;

                        case 'special_opening_times':

                              wp_enqueue_style('wp-repair-plugin-settings-coupons-css', WP_REPAIR_PLUGIN_URL.'assets/css/coupons.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                              break;

                        case 'notification_offer':

                            wp_enqueue_style('wp-repair-plugin-settings-notification_offer-css', WP_REPAIR_PLUGIN_URL.'assets/css/notification_offer.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                            break;

                        case 'manage_permission':

                            wp_enqueue_style('wp-repair-plugin-settings-manage_permission-css', WP_REPAIR_PLUGIN_URL.'assets/css/manage_permission.css?what='.WP_REPAIR_ASSETS_CACHE_V);

                            break;

                        

                        default:

                            break;

                    }

                }

                break;

            

            default:

                break;

        }

    }

}

add_action('wp_enqueue_scripts', 'loadTheRequiredWPRepairScript');
add_action('admin_enqueue_scripts', 'loadTheRequiredWPRepairScript');


function wp_repair_admin_setup_tour_nav_html() {

      $requirements = array(
            'rp_setup_guide_meet_download_libraries_requirement',
            'rp_setup_guide_meet_default_repairs_requirement',
            'rp_setup_guide_meet_manage_upsales_requirement',
            'rp_setup_guide_meet_configure_settings_requirement'
      );

      if(rp_get_option('rp_skip_quick_start_wizard', FALSE) === FALSE) {

            $needToShow = FALSE;

            foreach ($requirements as $callback) {
                  
                  if(call_user_func($callback) === FALSE) {

                        $needToShow = TRUE;

                  }

            }

            if($needToShow === TRUE) {

                  $fileName = WP_REPAIR_PLUGIN_PATH.'html/setup_tour_nav.php';

                  $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

                  require_once $fileName;

            }

      }

}


function wp_repair_admin_appointments_page_html() {

      rp_show_validation_error_message();

      wp_repair_admin_setup_tour_nav_html();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/appointments.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}


function wp_repair_admin_manage_categories_page_html() {

      rp_show_validation_error_message();

      wp_repair_admin_setup_tour_nav_html();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/manage_categories.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_manage_models_page_html() {

      rp_show_validation_error_message();

      wp_repair_admin_setup_tour_nav_html();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/manage_models.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}


function wp_repair_get_rs_category_by_name( $name = '' ) {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';

      $category_name = $name;

      return $rpQuery->get_results("SELECT * FROM $rs_category WHERE `c_name` LIKE '".rp_escape_sql($category_name)."'");

}

function wp_repair_verify_selected_default_repairs_category() {

      if( !empty( $_GET['repairs_category'] ?? '' ) ) {

            $repairs_category = trim(str_replace('-', ' ', $_GET['repairs_category']));

            return !empty( wp_repair_get_rs_category_by_name( $repairs_category ) );

      }

      return FALSE;

}


function wp_repair_admin_default_repairs_page_html() {

      rp_show_validation_error_message();

    if( empty( $_GET['repairs_category'] ?? '' ) || !wp_repair_verify_selected_default_repairs_category() ) {

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/default_repairs_select_category.php';

    } else {

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/default_repairs.php';

    }

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_download_libraries_page_html() {

      rp_show_validation_error_message();

      $page = 'download_languages';

      if( isset($_REQUEST['section']) && $_REQUEST['section'] === 'download_libraries' ) {

            $page = 'download_libraries';

      }

      if( isset($_REQUEST['section']) && $_REQUEST['section'] === 'repairs' ) {

            $page = 'download_repairs';

      }

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/'.$page.'.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_general_info_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/general_info.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}


function wp_repair_admin_styling_page_html() {

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/styling.php';
  
      $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);
  
      require_once $fileName;
  
}

function wp_repair_admin_special_opening_times_page_html() {

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/special_opening_times.php';

      $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

      require_once $fileName;

}


function wp_repair_admin_localization_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/localization.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_planning_discount_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/planning_discount.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_notification_offer_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/notification_offer.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}


function wp_repair_admin_manage_permission_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/manage_permission.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_coupons_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/coupons.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}

function wp_repair_admin_configure_mail_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/configure_mail.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}

function wp_repair_admin_plugin_shortcode_page_html() {

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/plugin_shortcode.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}

function wp_repair_admin_webhooks_page_html() {

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/webhooks.php';
  
      $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);
  
      require_once $fileName;
  
  }

function wp_repair_admin_settings_page_html() {

    $section = 'general_info';

    if(isset($_GET['section']) && !empty($_GET['section'])) {

        $section = $_GET['section'];

    }

    rp_show_validation_error_message();

    wp_repair_admin_setup_tour_nav_html();

    $loaded_through_addon = apply_filters('rp_returning_admin_setting_page_html', FALSE, $section);

    if($loaded_through_addon) {

        return;

    }

    if($section === 'general_info') {

        wp_repair_admin_general_info_page_html();

    } else if($section === 'localization') {

        wp_repair_admin_localization_page_html();

    } else if($section === 'styling') {

      wp_repair_admin_styling_page_html();

  } else if($section === 'special_opening_times') {

      wp_repair_admin_special_opening_times_page_html();

  } else if($section === 'planning_discount') {

        wp_repair_admin_planning_discount_page_html();

    } else if($section === 'notification_offer') {

        wp_repair_admin_notification_offer_page_html();

    } else if($section === 'coupons') {

        wp_repair_admin_coupons_page_html();

    }else if($section === 'configure_mail') {

        wp_repair_admin_configure_mail_page_html();

    }else if($section === 'plugin_shortcode') {

        wp_repair_admin_plugin_shortcode_page_html();

    }else if($section === 'webhooks') {

        wp_repair_admin_webhooks_page_html();

    }else if($section === 'manage_permission') {

        if(!current_user_can('activate_plugins')) {
            echo 'Sorry, you are not allowed to see this page.';
        } else {
            wp_repair_admin_manage_permission_page_html();
        }

    }else {

        wp_repair_admin_general_info_page_html();

    }

}



function wp_repair_admin_manage_upsales_page_html() {

      rp_show_validation_error_message();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/manage_upsales.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

}



function wp_repair_admin_main_page_html() {

    echo 'Hi from WP Repair Plugin!';

}

function wp_repair_show_alert_if_no_license() {
      ob_start();
      if(defined('WP_REPAIR_REQUIRE_LICENSE')) {
            global $rpQuery;
            global $wp_repair_lang;

            $rs_company_information = $rpQuery->prefix . 'rs_company_information';

            $info = $rpQuery->get_results("SELECT * FROM $rs_company_information LIMIT 1;");
            $contact = '';
            if(!empty($info)) {
                  if(!empty($info[0]->company_phone)) {
                        $contact = $info[0]->company_phone;
                  } else if(!empty($info[0]->company_email)) {
                        $contact = $info[0]->company_email;
                  } else {
                        $contact = '';
                  }
            }
            ?>
            <style type="text/css">
                  .wp-repair-FE-alert-license {
                        all: initial;
                        animation : none;
                           animation-delay : 0;
                           animation-direction : normal;
                           animation-duration : 0;
                           animation-fill-mode : none;
                           animation-iteration-count : 1;
                           animation-name : none;
                           animation-play-state : running;
                           animation-timing-function : ease;
                           backface-visibility : visible;
                           background : 0;
                           background-attachment : scroll;
                           background-clip : border-box;
                           background-color : transparent;
                           background-image : none;
                           background-origin : padding-box;
                           background-position : 0 0;
                           background-position-x : 0;
                           background-position-y : 0;
                           background-repeat : repeat;
                           background-size : auto auto;
                           border : 0;
                           border-style : none;
                           border-width : medium;
                           border-color : inherit;
                           border-bottom : 0;
                           border-bottom-color : inherit;
                           border-bottom-left-radius : 0;
                           border-bottom-right-radius : 0;
                           border-bottom-style : none;
                           border-bottom-width : medium;
                           border-collapse : separate;
                           border-image : none;
                           border-left : 0;
                           border-left-color : inherit;
                           border-left-style : none;
                           border-left-width : medium;
                           border-radius : 0;
                           border-right : 0;
                           border-right-color : inherit;
                           border-right-style : none;
                           border-right-width : medium;
                           border-spacing : 0;
                           border-top : 0;
                           border-top-color : inherit;
                           border-top-left-radius : 0;
                           border-top-right-radius : 0;
                           border-top-style : none;
                           border-top-width : medium;
                           bottom : auto;
                           box-shadow : none;
                           box-sizing : content-box;
                           caption-side : top;
                           clear : none;
                           clip : auto;
                           color : inherit;
                           columns : auto;
                           column-count : auto;
                           column-fill : balance;
                           column-gap : normal;
                           column-rule : medium none currentColor;
                           column-rule-color : currentColor;
                           column-rule-style : none;
                           column-rule-width : none;
                           column-span : 1;
                           column-width : auto;
                           content : normal;
                           counter-increment : none;
                           counter-reset : none;
                           cursor : auto;
                           direction : ltr;
                           display : inline;
                           empty-cells : show;
                           float : none;
                           font : normal;
                           font-family : inherit;
                           font-size : medium;
                           font-style : normal;
                           font-variant : normal;
                           font-weight : normal;
                           height : auto;
                           hyphens : none;
                           left : auto;
                           letter-spacing : normal;
                           line-height : normal;
                           list-style : none;
                           list-style-image : none;
                           list-style-position : outside;
                           list-style-type : disc;
                           margin : 0;
                           margin-bottom : 0;
                           margin-left : 0;
                           margin-right : 0;
                           margin-top : 0;
                           max-height : none;
                           max-width : none;
                           min-height : 0;
                           min-width : 0;
                           opacity : 1;
                           orphans : 0;
                           outline : 0;
                           outline-color : invert;
                           outline-style : none;
                           outline-width : medium;
                           overflow : visible;
                           overflow-x : visible;
                           overflow-y : visible;
                           padding : 0;
                           padding-bottom : 0;
                           padding-left : 0;
                           padding-right : 0;
                           padding-top : 0;
                           page-break-after : auto;
                           page-break-before : auto;
                           page-break-inside : auto;
                           perspective : none;
                           perspective-origin : 50% 50%;
                           position : static;
                           /* May need to alter quotes for different locales (e.g fr) */
                           quotes : '\201C' '\201D' '\2018' '\2019';
                           right : auto;
                           tab-size : 8;
                           table-layout : auto;
                           text-align : inherit;
                           text-align-last : auto;
                           text-decoration : none;
                           text-decoration-color : inherit;
                           text-decoration-line : none;
                           text-decoration-style : solid;
                           text-indent : 0;
                           text-shadow : none;
                           text-transform : none;
                           top : auto;
                           transform : none;
                           transform-style : flat;
                           transition : none;
                           transition-delay : 0s;
                           transition-duration : 0s;
                           transition-property : none;
                           transition-timing-function : ease;
                           unicode-bidi : normal;
                           vertical-align : baseline;
                           visibility : visible;
                           white-space : normal;
                           widows : 0;
                           width : auto;
                           word-spacing : normal;
                           z-index : auto;
                           /* basic modern patch */
                           all: initial;
                           all: unset;
                           max-height: unset;
                           max-width: unset;
                  }
                  .wp-repair-FE-alert-license {
                        padding: 8px 12px !important;
                        font-size: 15px !important;
                        color: #004085;
                        background-color: #cce5ff !important;
                        border: 1px solid #b8daff !important;
                        border-radius: 6px !important;
                        margin: auto !important;
                        text-align: left !important;
                        display: table !important;
                        margin: 15px 0px !important;
                        margin: auto !important;
                        margin-top: 15px !important;
                        margin-bottom: 15px !important;
                        width: 100% !important;
                        max-width: 800px !important;
                        text-align: center !important;
                  }
            </style>
            <br>
            <div class="wp-repair-FE-alert-license">
                  <?php
                  if(!empty($contact)) {
                        echo str_replace('$contact$', $contact, $wp_repair_lang['fe_no_license_with_contact']);
                  } else {
                        echo $wp_repair_lang['fe_no_license_no_contact'];
                  }
                  ?>
            </div>
            <br>
            <?php
      }
      $content = ob_get_contents();
      ob_get_clean();
      return trim($content);
}

function rp_create_column_sort_link_manage_models($column_name = '', $sort_key = '') {

      $brand = $_REQUEST['brand'] ?? '';

      $category = $_REQUEST['category'] ?? '';

      $search_model = $_REQUEST['search_model'] ?? '';

      $sort_by = $_REQUEST['sort_by'] ?? '';

      $sort_order = $_REQUEST['sort_order'] ?? 'ASC';

      $brand = trim($brand);

      $category = trim($category);

      $search_model = trim($search_model, '%');

      $sort_by = trim($sort_by);

      $sort_order = trim($sort_order);

      $get_paramters = array();

      if(!empty($brand) && trim(strtolower($brand)) !== 'all') {
            $get_paramters[] = "brand=$brand";
      }

      if(!empty($category)) {
            $get_paramters[] = "category=$category";
      }

      if(!empty($search_model)) {
            $get_paramters[] = "search_model=".urlencode($search_model);
      }

      if(!empty($get_paramters)) {
            $get_paramters_str = '&'.join('&', $get_paramters);
      } else {
            $get_paramters_str = '';
      }

      if($sort_key === $sort_by) {
            // Currently Sorted...
            if($sort_order === 'ASC') {
                  return "<a class=\"rp_sortable_link\" href=\"?page=wp_repair_manage_models$get_paramters_str&sort_by=$sort_key&sort_order=DESC\">$column_name <i style=\"font-size: 14px; position: relative; top: 1px; line-height: 1.5em;\" class=\"fas fa-caret-up\"></i></a>";
            } else {
                  return "<a class=\"rp_sortable_link\" href=\"?page=wp_repair_manage_models$get_paramters_str&sort_by=$sort_key&sort_order=ASC\">$column_name <i style=\"font-size: 14px; position: relative; top: 1px; line-height: 1.5em;\" class=\"fas fa-caret-down\"></i></a>";
            }
      } else {
            return "<a class=\"rp_sortable_link\" href=\"?page=wp_repair_manage_models$get_paramters_str&sort_by=$sort_key&sort_order=ASC\">$column_name</a>";
      }

}

function wp_repair_add_colors_to_models_if_needed() {

      global $rpQuery;

      $model_table = $rpQuery->prefix . "rs_model";

      $default_color_table = $rpQuery->prefix . "rs_default_color";

      $modelcolor_table = $rpQuery->prefix . "rs_model_color";

      $m_result = $rpQuery->get_results("SELECT * FROM $model_table");

      $rpQuery->query('START TRANSACTION');

      foreach($m_result as $m){

            $color_ids = $m->color_ids;
            
            $m_id  = $m->m_id;
            
            if($color_ids != null){

                  $all_model_colors = rp_get_all_model_color_ids($m_id);

                  $color_idz = explode(',', $color_ids);

                  foreach($color_idz as $c){

                        $m_color    = $rpQuery->get_results("SELECT * FROM $default_color_table WHERE dc_id = '".rp_escape_sql($c)."'");  

                        $mc_id      = $m_color[0]->dc_id;

                        $color_name = $m_color[0]->dc_name;

                        $color_code = $m_color[0]->dc_code;

                        $mc_uq_id   = $m_color[0]->dc_uq_id;

                        if(array_search(trim($mc_uq_id), $all_model_colors) === FALSE) {

                              $all_model_colors[] = trim($mc_uq_id);

                              $rpQuery->query("INSERT INTO $modelcolor_table(color_name,color_code,mc_uq_id,m_id_fk) VALUES('".rp_escape_sql($color_name)."','".rp_escape_sql($color_code)."','".rp_escape_sql($mc_uq_id)."','".rp_escape_sql($m_id)."')");

                        }

                  }

                  $rpQuery->query("UPDATE $model_table SET color_ids='' WHERE m_id='".rp_escape_sql($m_id)."'"); 

            }

      }

      $rpQuery->query('COMMIT');

}

function wp_repair_get_brands_of_category() {
      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';
      
      $rs_brand = $rpQuery->prefix . 'rs_brand';

      $categories = $rpQuery->get_results("SELECT * FROM `$rs_category`");
      if(!empty($categories)) {

            $key_value_pair = array();

            foreach ($categories as $key => $category) {
                  if(!isset($key_value_pair[$category->c_id])) {
                        $key_value_pair[$category->c_id] = array();
                  }
                  $brands = $rpQuery->get_results("SELECT * FROM `$rs_brand` WHERE `c_id_fk` = '".rp_escape_sql($category->c_id)."'");
                  if(!empty($brands)) {
                        foreach ($brands as $_key => $brand) {
                              $key_value_pair[$category->c_id][] = array($brand->b_id, $brand->b_name);
                        }
                  }
            }

            return $key_value_pair;

      }
      return array();
}


function wp_repair_show_FE_templates() {

      $content = wp_repair_show_alert_if_no_license();

      if(!empty($content)) {

            return $content;
            
      }

      return wp_repair_show_FE_bundle_page();

}

add_shortcode('Repair_All', 'wp_repair_show_FE_templates');


function wp_repair_show_FE_bundle_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/bundle.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}


function rp_start_session_for_RP_front_end() {

      if( isset( $_GET['selected'] ) ) {

            if( strpos( $_GET['selected'], 'finalize' ) !== FALSE || strpos( $_GET['selected'], 'shortcode_finalize_order' ) !== FALSE ) {

                  RepairPluginPro\RP_Session_Handler::read();

            }

      }

}

add_action('init', 'rp_start_session_for_RP_front_end', 999);


function wp_repair_show_FE_model_shortcode_bundle_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/bundle_model_shortcode.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}




function wp_repair_show_FE_type_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/types.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}



function wp_repair_show_FE_brand_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/brands.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}



function wp_repair_show_FE_model_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/models.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}



function wp_repair_show_FE_repair_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/repairs.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}



function wp_repair_show_FE_finalize_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/repair_finalize.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}



function wp_repair_show_FE_thanks_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/thanks.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}


function wp_repair_show_FE_cancel_page() {

    ob_start();

    $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/cancel.php';

    $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

    require_once $fileName;

    $content = ob_get_clean();

    return $content;

}





function cssLineStartsWith($string, $startString) {

    $len = strlen($startString);

    return (substr($string, 0, $len) === $startString);

}

function rp_fetch_image_from_base_using_curl($link = '') {

	$curl = curl_init( $link );

	curl_setopt($curl, CURLOPT_ENCODING, 'gzip');

	curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );

	curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false );

	curl_setopt( $curl, CURLOPT_SSL_VERIFYHOST, false );

	$response = curl_exec( $curl );

	$info = curl_getinfo( $curl );

	curl_close( $curl );

	if($info['http_code'] != '200') {
		return false;
	}

	return $response;
}

function rp_fetch_image_from_base_using_fgc($link = '') {

	return file_get_contents( $link, false, stream_context_create( array( 'ssl' => array( 'verify_peer' => false, 'verify_peer_name' => false ) ) ) );

}

function rp_fetch_image_from_base($link = '') {

      // check if link contains ABSPATH to check if it is a local file

      if( strpos( $link, ABSPATH ) !== FALSE ) {

            return file_get_contents( $link );

      }

	if( filter_var( ini_get('allow_url_fopen'), FILTER_VALIDATE_BOOLEAN ) === TRUE ) {

		return rp_fetch_image_from_base_using_fgc($link);

	} else {

		return rp_fetch_image_from_base_using_curl($link);

	}

}

function editCSSLineByLine($css_file = '', $func = NULL, $save = FALSE) {

    $file = WP_REPAIR_PLUGIN_PATH.'assets/css/front_end/'.$css_file.'.css';

    if(@file_exists($file)) {

        $content = rp_fetch_image_from_base($file);

        $content = explode("\n", $content);

        foreach ($content as $lineNum => $value) {

            $value = trim($value);

            $content[$lineNum] = $func($value, $lineNum);

        }

        if($save === TRUE) {

            return file_put_contents($file, $content) !== FALSE;

        }

        return implode("\n", $content);

    }

    return '';

}

function model_shortcode( $atts ) {

    $atts = array_change_key_case( ( array )$atts, CASE_LOWER );

    $shortcode_attributes = shortcode_atts(

        array(

            'code' => '', // default url

        ), $atts );

    $code = esc_attr( $shortcode_attributes['code'] );

    if ( $code ) { 

            $GLOBALS['model_code'] = $code;
            $content = wp_repair_show_alert_if_no_license();
            if(!empty($content)) {
                  return $content;
            }

            return wp_repair_show_FE_model_shortcode_bundle_page();

    }

}

function category_shortcode( $atts ) {

      $atts = array_change_key_case( ( array )$atts, CASE_LOWER );

      $shortcode_attributes = shortcode_atts(

            array(

                  'code' => '', // default url

            ), $atts );

      $code = esc_attr( $shortcode_attributes['code'] );

      if ( $code ) { 

            $GLOBALS['category_shortcode'] = $code;
            $content = wp_repair_show_alert_if_no_license();
            if(!empty($content)) {
                  return $content;
            }

            return wp_repair_show_FE_bundle_page();

      }

}

add_shortcode('categories', 'category_shortcode');

function brand_shortcode( $atts ) {

      $atts = array_change_key_case( ( array )$atts, CASE_LOWER );

      $shortcode_attributes = shortcode_atts(

            array(

                  'code' => '', // default url

            ), $atts );

      $code = esc_attr( $shortcode_attributes['code'] );

      if ( $code ) { 

            $GLOBALS['brand_shortcode'] = $code;
            $content = wp_repair_show_alert_if_no_license();
            if(!empty($content)) {
                  return $content;
            }

            return wp_repair_show_FE_bundle_page();

      }

}

add_shortcode('brands', 'brand_shortcode');

function rp_get_settings_placeholder_email() {

      $placeholder_email = '';

      if(defined('WP_REPAIR_SETTINGS_PLACEHOLDER_EMAIL') && !empty(WP_REPAIR_SETTINGS_PLACEHOLDER_EMAIL)) {

            $placeholder_email = WP_REPAIR_SETTINGS_PLACEHOLDER_EMAIL;

      }

      return trim($placeholder_email);

}

add_shortcode( 'models', 'model_shortcode' );

register_activation_hook(__FILE__,'repair_plugin_table_create');

register_deactivation_hook(__FILE__,'repair_plugin_dactivate');

function repair_plugin_table_create(){

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );



    $table_rs_category = $rpQuery->prefix . "rs_category"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_category (

    `c_id` int(11) NOT NULL AUTO_INCREMENT,

    `c_name` varchar(255) NOT NULL,

    `c_img` varchar(255) NOT NULL,

    `upated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `c_uq_id` varchar(255) NOT NULL,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `c_position` int(11) NOT NULL,

    PRIMARY KEY (`c_id`))

    $charset_collate";

    dbDelta( $sql );





    $table_rs_brand = $rpQuery->prefix . "rs_brand"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_brand (

    `b_id` int(11) NOT NULL AUTO_INCREMENT,

    `b_name` varchar(255) NOT NULL,

    `b_img` varchar(255) NOT NULL,

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `b_uq_id` varchar(255) NOT NULL,

    `c_id_fk` int(11) NOT NULL,

    `b_date_time` datetime NULL DEFAULT NULL,

    `b_position` int(11) NOT NULL,

    PRIMARY KEY (`b_id`))

    $charset_collate";

    dbDelta( $sql );



    $table_rs_model = $rpQuery->prefix . "rs_model"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_model (

    `m_id` int(11) NOT NULL AUTO_INCREMENT,

    `m_name` varchar(255) NOT NULL,

    `m_img` varchar(255) NOT NULL,

    `device_type` int(11) NOT NULL,

    `brand_id_fk` int(11) NOT NULL,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `m_code` varchar(255) NOT NULL,

    `release_date` date NOT NULL,

    `m_uq_id` varchar(255) NOT NULL,

    `repair_ids` text NULL,

    `color_ids` text NULL,

    `m_date_time` datetime NULL DEFAULT NULL,

    `m_position` int(11) NOT NULL,

    PRIMARY KEY (`m_id`))

    $charset_collate";

    dbDelta( $sql );



    $table_rs_model_color = $rpQuery->prefix . "rs_model_color"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_model_color (

    `mc_id` int(11) NOT NULL AUTO_INCREMENT,

    `color_name` varchar(255) NOT NULL,

    `color_code` varchar(255) NOT NULL,

    `mc_uq_id` varchar(255) NOT NULL,

    `m_id_fk` int(11) NOT NULL,

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `mc_position` int(11) NOT NULL,

    PRIMARY KEY (`mc_id`))  

    $charset_collate";

    dbDelta( $sql );

    $table_rs_repair = $rpQuery->prefix . "rs_repair"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_repair (

    `r_id` int(11) NOT NULL AUTO_INCREMENT,

    `r_name` varchar(255) NOT NULL,

    `r_price` float NOT NULL,

    `r_duration` int(11) NOT NULL,

    `r_discription` varchar(255) NOT NULL,

    `r_img` varchar(255) NOT NULL,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `r_uq_id` varchar(255) NOT NULL,

    `m_id_fk` int(11) NOT NULL,

    `r_date_time` datetime NULL DEFAULT NULL,

    `r_position` int(11) NOT NULL,

    PRIMARY KEY (`r_id`))   

    $charset_collate";

    dbDelta( $sql );



    $table_rs_repair_attr = $rpQuery->prefix . "rs_repair_attr"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_repair_attr (

    `a_id` int(11) NOT NULL AUTO_INCREMENT,

    `a_name` varchar(255) NOT NULL,

    `a_price` float NOT NULL,

    `r_id_fk` int(11) NOT NULL,

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `a_uq_id` varchar(255) NOT NULL,

    `ra_position` int(11) NOT NULL,

    PRIMARY KEY (`a_id`))   

    $charset_collate";

    dbDelta( $sql );


    $table_rs_default_color = $rpQuery->prefix . "rs_default_color"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_default_color (

    `dc_id` int(11) NOT NULL AUTO_INCREMENT,

    `dc_name` varchar(255) NOT NULL,

    `dc_code` varchar(255) NOT NULL,

    `dc_uq_id` varchar(255) NOT NULL,

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `dc_position` int(11) NOT NULL,

    PRIMARY KEY (`dc_id`))  

    $charset_collate";

    dbDelta( $sql );


    $table_rs_default_repair = $rpQuery->prefix . "rs_default_repair"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_default_repair (

    `dr_id` int(11) NOT NULL AUTO_INCREMENT,

    `dr_name` varchar(255) NOT NULL,

    `dr_price` float NOT NULL,

    `dr_duration` int(11) NOT NULL,

    `dr_discription` varchar(255) NOT NULL,

    `dr_img` varchar(255) NOT NULL,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `dr_uq_id` varchar(255) NOT NULL,

    `dr_date_time` datetime NULL DEFAULT NULL,

    `dr_position` int(11) NOT NULL,

    PRIMARY KEY (`dr_id`))   

    $charset_collate";

    dbDelta( $sql );



    $table_rs_default_repair_attr = $rpQuery->prefix . "rs_default_repair_attr"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_default_repair_attr (

    `da_id` int(11) NOT NULL AUTO_INCREMENT,

    `da_name` varchar(255) NOT NULL,

    `da_price` float NOT NULL,

    `dr_id_fk` int(11) NOT NULL,

    `updated_on` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

    `is_active` tinyint(1) NOT NULL DEFAULT '1',

    `da_uq_id` varchar(255) NOT NULL,

    `da_position` int(11) NOT NULL,

    PRIMARY KEY (`da_id`))   

    $charset_collate";

    dbDelta( $sql );



    $company_information = $rpQuery->prefix . "rs_company_information"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $company_information (

    `company_id` int(11) NOT NULL AUTO_INCREMENT,

    `company_name` varchar(255) NOT NULL,

    `company_email` varchar(255) NOT NULL,

    `company_phone` varchar(255) NOT NULL,

    `company_logo` varchar(255) NOT NULL,

    `company_reg_no` varchar(255) NOT NULL,

    `tax_reg_no` varchar(255) NOT NULL,

    `company_theme_color` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`company_id`))     

    $charset_collate";

    dbDelta( $sql );

    $count_company_information = $rpQuery->get_var("SELECT COUNT(*) FROM $company_information");

    if($count_company_information == 0){

      $placeholder_email = rp_get_settings_placeholder_email();

        $rpQuery->query("INSERT INTO $company_information(company_name,company_email,company_phone,company_logo,company_reg_no,tax_reg_no,company_theme_color) VALUES('abc','".rp_escape_sql($placeholder_email)."','12345','http://site.com/picture','','','2781c7')");

    }

    $company_location = $rpQuery->prefix . "rs_company_location"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $company_location (

    `location_id` int(11) NOT NULL AUTO_INCREMENT,

    `location_name` varchar(255) NOT NULL,

    `location_address` varchar(255) NOT NULL,

    `location_city_district` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`location_id`))    

    $charset_collate";

    dbDelta( $sql );

    $count_company_location = $rpQuery->get_var("SELECT COUNT(*) FROM $company_location");

    if($count_company_location == 0){

        $rpQuery->query("INSERT INTO $company_location(location_name,location_address,location_city_district) VALUES('Repairshop','','')");

    }

    $company_schedules = $rpQuery->prefix . "rs_company_schedules"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $company_schedules (

    `cs_id` int(11) NOT NULL AUTO_INCREMENT,

    `cs_day` varchar(255) NOT NULL,

    `cs_open_timing` varchar(255) NOT NULL,

    `cs_close_timing` varchar(255) NOT NULL,

    `cs_breaktime_from` varchar(255) NOT NULL,

    `cs_breaktime_to` varchar(255) NOT NULL,

    `is_closed` tinyint(4) NOT NULL DEFAULT '1',

    `location_id_fk` int(11) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`cs_id`))  

    $charset_collate";

    dbDelta( $sql );

    $count_company_schedules = $rpQuery->get_var("SELECT COUNT(*) FROM $company_schedules");

    if($count_company_schedules == 0){

        for($i = 0; $i < 7; $i++ ){

            $days = ['Monday','Tuesday','Wednesday','Thursday','Friday','Saturday','Sunday'];

            $rpQuery->query("INSERT INTO $company_schedules(cs_day,cs_open_timing,cs_close_timing,cs_breaktime_from,cs_breaktime_to,is_closed,location_id_fk) VALUES('".rp_escape_sql($days[$i])."','09:00','18:00','12:00','13:00','0','1')");

        }

    }

    $table_coupons = $rpQuery->prefix . "rs_coupons"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_coupons (

    `coupons_id` int(11) NOT NULL AUTO_INCREMENT,

    `coupons_own_reference` varchar(255) NOT NULL,

    `coupons_code` varchar(255) NOT NULL,

    `coupons_amount` varchar(255) NOT NULL,

    -- `coupons_type` varchar(255) NOT NULL,

    `coupons_set_limit` varchar(255) NOT NULL,

    `is_active` tinyint(4) NOT NULL DEFAULT '1',

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`coupons_id`))     

    $charset_collate";

    dbDelta( $sql );

    $count_table_coupons = $rpQuery->get_var("SELECT COUNT(*) FROM $table_coupons");

    if($count_table_coupons == 0){

        $rpQuery->query("INSERT INTO $table_coupons(coupons_own_reference,coupons_code,coupons_amount,coupons_set_limit,is_active) VALUES('Customer 1','20OFFNOW','20','unlimited','0')");

    }

    $table_rs_localization = $rpQuery->prefix . "rs_localization"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_localization (

    `localization_id` int(11) NOT NULL AUTO_INCREMENT,

    `set_language` varchar(255) NOT NULL,

    `set_currency` varchar(255) NOT NULL,

    `set_price_symbols` varchar(255) NOT NULL,

    `set_decimals` varchar(255) NOT NULL,

    `show_even_price` tinyint(4) NOT NULL DEFAULT '0',

    `show_decimals` tinyint(4) NOT NULL DEFAULT '1',

    `set_time_format` varchar(255) NOT NULL,

    `tax` varchar(255) NOT NULL,

    `tax_indicator` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`localization_id`))    

    $charset_collate";

    dbDelta( $sql );

    $count_table_rs_localization = $rpQuery->get_var("SELECT COUNT(*) FROM $table_rs_localization");

    if($count_table_rs_localization == 0){

        $rpQuery->query("INSERT INTO $table_rs_localization(set_language,set_currency,set_price_symbols,set_decimals,show_even_price,show_decimals,set_time_format,tax,tax_indicator) VALUES('English (United States)','€','Left (\$10)','comma','0','1','24 hours (e.g. 15:00)','21','BTW')");

    }

    $table_rs_notification_types = $rpQuery->prefix . "rs_notification_types"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_notification_types (

    `noti_type_id` int(11) NOT NULL AUTO_INCREMENT,

    `noti_type` varchar(255) NOT NULL,

    `is_active` tinyint(4) NOT NULL DEFAULT '1',

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`noti_type_id`))    

    $charset_collate";

    dbDelta( $sql );

    $count_table_rs_notification_types = $rpQuery->get_var("SELECT COUNT(*) FROM $table_rs_notification_types");

    if($count_table_rs_notification_types == 0){

        $types = ['Booking Pending','PDF Offer','Booking Approved','Booking Canceled'];

        for($i = 0;$i < 4 ;$i++){

            $rpQuery->query("INSERT INTO $table_rs_notification_types(noti_type) VALUES('".rp_escape_sql($types[$i])."')");

        }

    }

    $table_rs_notifications_and_offers = $rpQuery->prefix . "rs_notifications_and_offers"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_notifications_and_offers (

    `noti_id` int(11) NOT NULL AUTO_INCREMENT,

    `noti_subject` varchar(255) NOT NULL,

    `noti_message` text NOT NULL,

    `noti_type_id_fk` int(11) NOT NULL,

    `shortcode_name` text NOT NULL,

    `shortcode_symbol`text NOT NULL,

    `is_active` tinyint(4) NOT NULL DEFAULT '1',

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`noti_id`))    

    $charset_collate";

    dbDelta( $sql );

    $count_table_rs_notifications_and_offers = $rpQuery->get_var("SELECT COUNT(*) FROM $table_rs_notifications_and_offers");

    if($count_table_rs_notifications_and_offers == 0){
        $subject = [get_email_template_of_wp_repair('pending')[0],get_email_template_of_wp_repair('offer')[0],get_email_template_of_wp_repair('approved')[0],get_email_template_of_wp_repair('cancelled')[0]];
        $message = [get_email_template_of_wp_repair('pending')[1],get_email_template_of_wp_repair('offer')[1],get_email_template_of_wp_repair('approved')[1],get_email_template_of_wp_repair('cancelled')[1]];
        $x = 0;
        for($i=1;$i<5;$i++){
            $rpQuery->query("INSERT INTO $table_rs_notifications_and_offers(noti_subject,noti_message,noti_type_id_fk,shortcode_name,shortcode_symbol,is_active) VALUES('".rp_escape_sql($subject[$x])."','".rp_escape_sql($message[$x])."','".rp_escape_sql($i)."','','','1')");
            $x++;
        }

    }

    $table_rs_planning_and_discount = $rpQuery->prefix . "rs_planning_and_discount"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_planning_and_discount (

    `pd_id` int(11) NOT NULL AUTO_INCREMENT,

    `booking_slots_length` varchar(255) NOT NULL,

    `booking_amount_per_slot` varchar(255) NOT NULL,

    `is_combo_discount` tinyint(4) NOT NULL DEFAULT '1',

    `amount` varchar(255) NOT NULL,

    `max_amount` varchar(255) NOT NULL,

    `come_our_store` varchar(255) NOT NULL,

    `ship_device` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`pd_id`))  

    $charset_collate";

    dbDelta( $sql );

    $count_table_rs_planning_and_discount = $rpQuery->get_var("SELECT COUNT(*) FROM $table_rs_planning_and_discount");

    if($count_table_rs_planning_and_discount == 0){

        $rpQuery->query("INSERT INTO $table_rs_planning_and_discount(booking_slots_length,booking_amount_per_slot,is_combo_discount,amount,max_amount,come_our_store,ship_device) VALUES('30 Minutes (Default)','Unlimited (Default)','1','10','None (Default)','1','1')");

    }

    $table_rs_upsales = $rpQuery->prefix . "rs_upsales"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_upsales (

    `upsales_id` int(11) NOT NULL AUTO_INCREMENT,

    `us_name` varchar(255) NOT NULL,

    `us_info` text NOT NULL,

    `us_text` varchar(255) NOT NULL,

    `us_old_price` float NOT NULL,

    `us_new_price` float NOT NULL,

    `is_active` tinyint(4) NOT NULL DEFAULT '1',

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    `upsales_position` int(11) NOT NULL,

    PRIMARY KEY (`upsales_id`))     

    $charset_collate";

    dbDelta( $sql );



    $table_rs_upsales_models = $rpQuery->prefix . "rs_upsales_models"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_upsales_models (

    `up_model_id` int(11) NOT NULL AUTO_INCREMENT,

    `m_id_fk` int(11) NOT NULL,

    `upsales_id_fk` int(11) NOT NULL,

    PRIMARY KEY (`up_model_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_users_tokens = $rpQuery->prefix . "rs_user_token"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_users_tokens (

    `user_token_id` int(11) NOT NULL AUTO_INCREMENT,

    `hash_token` varchar(255) NOT NULL,

    `license_key` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`user_token_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_customer = $rpQuery->prefix . "rs_customer"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_customer (

    `customer_id` int(11) NOT NULL AUTO_INCREMENT,

    `customer_name` varchar(255) NOT NULL,

    `customer_phone` varchar(255) NOT NULL,

    `customer_email` varchar(255) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`customer_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_customer_business = $rpQuery->prefix . "rs_customer_business"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_customer_business (

    `cb_id` int(11) NOT NULL AUTO_INCREMENT,

    `cb_name` varchar(255) NOT NULL,

    `customer_id_fk` int(11) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`cb_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_orders = $rpQuery->prefix . "rs_orders"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_orders (

    `o_id` int(11) NOT NULL AUTO_INCREMENT,

    `customer_id_fk` int(11) NOT NULL,

    `notes` text NOT NULL,

    `color` varchar(255) NOT NULL,

    `excluded_tax_price` float NOT NULL,

    `combo_discount` varchar(255) NOT NULL,

    `coupon_id_fk` int(11) DEFAULT NULL,

    `total_price` float NOT NULL,

    `delivery_method` varchar(255) NOT NULL,

    `delivery_time` varchar(255) NOT NULL,

    `delivery_date` varchar(255) NOT NULL,

    `pdf_offer` tinyint(4) NOT NULL DEFAULT '0',

    `m_id_fk` int(11) NOT NULL,

    `status_id_fk` int(11) NOT NULL,

    `location_id_fk` int(11) NOT NULL,

    `o_date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`o_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_order_status = $rpQuery->prefix . "rs_order_status"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_order_status (

    `status_id` int(11) NOT NULL AUTO_INCREMENT,

    `s_name` varchar(255) NOT NULL,

    PRIMARY KEY (`status_id`))    

    $charset_collate";

    dbDelta( $sql );



    $count_table_rs_order_status = $rpQuery->get_var("SELECT COUNT(*) FROM $table_rs_order_status");

    if($count_table_rs_order_status == 0){

        for($i=0;$i<4;$i++){

            $status = ['Pending','Offered','Approved','Canceled'];

            $rpQuery->query("INSERT INTO $table_rs_order_status(s_name) VALUES('".rp_escape_sql($status[$i])."')");

        }

    }

    $table_rs_order_repairs = $rpQuery->prefix . "rs_order_repairs"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_order_repairs (

    `or_id` int(11) NOT NULL AUTO_INCREMENT,

    `r_id_fk` int(11) NOT NULL,

    `o_id_fk` int(11) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`or_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_order_repair_attr = $rpQuery->prefix . "rs_order_repair_attr"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_order_repair_attr (

    `ora_id` int(11) NOT NULL AUTO_INCREMENT,

    `ra_id_fk` int(11) NOT NULL,

    `o_id_fk` int(11) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`ora_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_order_upsales = $rpQuery->prefix . "rs_order_upsales"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_order_upsales (

    `ou_id` int(11) NOT NULL AUTO_INCREMENT,

    `upsales_id_fk` int(11) NOT NULL,

    `o_id_fk` int(11) NOT NULL,

    `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`ou_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_configure_mail = $rpQuery->prefix . "rs_configure_mail"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_configure_mail (

    `ce_id` int(11) NOT NULL AUTO_INCREMENT,

    `my_email` varchar(255) NOT NULL,

    `my_email_password` varchar(255) NOT NULL,

    `smtp_host` varchar(255) NOT NULL,

    `smtp_port` varchar(50) NOT NULL,

    `smtp_encryption` varchar(50) NOT NULL,

    `ce_date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`ce_id`))    

    $charset_collate";

    dbDelta( $sql );



    $table_rs_user_pdfs = $rpQuery->prefix . "rs_user_pdfs"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_user_pdfs (

    `user_pdfs_id` int(11) NOT NULL AUTO_INCREMENT,

    `user_name` varchar(255) NOT NULL,

    `streetname_phone` varchar(255) NOT NULL,

    `zipcode_city` varchar(255) NOT NULL,

    `user_email` varchar(255) NOT NULL,

    `notes` text NOT NULL,

    `pdf_date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

    PRIMARY KEY (`user_pdfs_id`))    

    $charset_collate";

    dbDelta( $sql );


      if(rp_get_option('rp_use_customer_info_for_duplicate_email', FALSE) === FALSE) {
            
            rp_add_option('rp_use_customer_info_for_duplicate_email', '1');
            
      }

      $status_change_emails = array(
            1, // Pending
            2, // PDF Offer
            3, // Approved
            4 // Cancelled
      );

      foreach ($status_change_emails as $email_noti_id) {

            if(rp_get_option('wp_repair_admin_duplicate_email_status_'.$email_noti_id, FALSE) === FALSE) {

                  rp_add_option('wp_repair_admin_duplicate_email_status_'.$email_noti_id, '1');

            }
            
      }

    //------------------------------------------------------------------------------- create file / create directory
    // create file

    // $content = 
    // "; BEGIN CUSTOM PHP SETTINGS
    // memory_limit=9999M
    // upload_max_filesize=9999M
    // max_execution_time=9999
    // max_input_vars=9999
    // max_input_time=9999
    // ; END CUSTOM PHP SETTINGS";
    // $fp = fopen(ABSPATH . "/.user.ini","wb");
    // if( $fp == true ){
    //     fwrite($fp,$content);
    //     fclose($fp);
    // }
    
    // create directory
    // if ( ! is_dir( ABSPATH . 'php.ini' ) ) {
    //     wp_mkdir_p( ABSPATH . 'php.ini' );
    // }
}

function repair_plugin_dactivate(){

    global $rpQuery;

    $tableArray = [   

        // $rpQuery->prefix . "rs_category",

        // $rpQuery->prefix . "rs_brand",

        // $rpQuery->prefix . "rs_model",

        // $rpQuery->prefix . "rs_model_color",

        // $rpQuery->prefix . "rs_default_color",

        // $rpQuery->prefix . "rs_repair",

        // $rpQuery->prefix . "rs_default_repair",

        // $rpQuery->prefix . "rs_repair_attr",

        // $rpQuery->prefix . "rs_default_repair_attr",

        // $rpQuery->prefix . "rs_appointments",

        // $rpQuery->prefix . "rs_company_information",

        // $rpQuery->prefix . "rs_company_location",

        // $rpQuery->prefix . "rs_company_schedules",

        // $rpQuery->prefix . "rs_coupons",

        // $rpQuery->prefix . "rs_localization",

        // $rpQuery->prefix . "rs_notifications_and_offers",

        // $rpQuery->prefix . "rs_notification_types",

        // $rpQuery->prefix . "rs_planning_and_discount",

        // $rpQuery->prefix . "rs_upsales",

        // $rpQuery->prefix . "rs_upsales_models",

        // $rpQuery->prefix . "rs_customer",

        // $rpQuery->prefix . "rs_customer_business",

        // $rpQuery->prefix . "rs_orders",

        // $rpQuery->prefix . "rs_order_repairs",

        // $rpQuery->prefix . "rs_order_repair_attr",

        // $rpQuery->prefix . "rs_order_status",

        // $rpQuery->prefix . "rs_order_upsales",

        // $rpQuery->prefix . "rs_user_pdfs",

        // $rpQuery->prefix . "rs_configure_mail",

        // $rpQuery->prefix . "rs_user_token",

    ];
    foreach ($tableArray as $tablename) {
        $rpQuery->query("DROP TABLE IF EXISTS $tablename");
    }
}

// ----------------------------------------------- get company email

function rp_get_settings_company_email() {

      global $rpQuery;

      $company_information = $rpQuery->prefix ."rs_company_information";

      $company_results = $rpQuery->get_results("SELECT * FROM $company_information");

      $company_email = '';

      if(!empty($company_results)) {

            $company_email = $company_results[0]->company_email;

      }

      return trim($company_email);

}

//--------------------------------------------------- get coupon

function coupon_code(){

      rp_verify_csrf_token_for_ajax('frontend');

    global $rpQuery;

    global $wp_repair_lang;

    $rs_coupons = $rpQuery->prefix . "rs_coupons";

    $coupon = $_POST['coupon'];     

    $result = $rpQuery->get_results("SELECT * FROM $rs_coupons WHERE coupons_code = '".rp_escape_sql($coupon)."'");

    if($result){

        $coupons_amount = $result[0]->coupons_amount;

        $coupons_id = $result[0]->coupons_id;

        echo json_encode(Array('status'=> true, 'message'  => $coupons_amount, 'coupons_id' => $coupons_id, 'type' => $result[0]->coupons_type));

        exit();

    }else{

        echo json_encode(Array('status'=> false, 'message' => $wp_repair_lang['invalid_coupon']));

        exit();

    }

}


rp_ajax_for_public('coupon_code','coupon_code');

//-------------------------------------------------------------------delete repairs

function delete_repair(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery;

    $rs_repair = $rpQuery->prefix . "rs_repair";

    $rs_r_translations = $rpQuery->prefix . "rs_r_translations";

    $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

    $rid = $_POST['rid'];     

    $result = $rpQuery->get_row("SELECT * FROM $rs_repair WHERE r_id = '".rp_escape_sql($rid)."'");

    if( !empty( $result ) ) {

      $rpQuery->query("DELETE FROM `$rs_r_translations` WHERE uq_id = '".rp_escape_sql($result->r_uq_id)."' AND m_id_fk = '".rp_escape_sql($result->m_id_fk)."'");

      $rpQuery->query("DELETE FROM `$rs_rattr_translations` WHERE r_id_fk = '".rp_escape_sql($rid)."';");

      $rpQuery->query("DELETE FROM $rs_repair WHERE r_id='".rp_escape_sql($rid)."'");

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

    exit();

}

rp_ajax_for_admin('delete_repair','delete_repair');


//-------------------------------------------------------------------delete default repairs

function rp_add_this_default_repair_to_all_models( $dr_id = 0 ) {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      $dr_id = (int) $dr_id;

      if( empty( $dr_id ) ) {

            return;

      }

      global $rpQuery;
      
      $rs_default_repair = $rpQuery->prefix . "rs_default_repair";

      $rs_default_repair_attr = $rpQuery->prefix . "rs_default_repair_attr";

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      $rpQuery->query("START TRANSACTION");

      $dr_row = $rpQuery->get_row("SELECT * FROM `$rs_default_repair` WHERE dr_id = '".rp_escape_sql($dr_id)."' LIMIT 1;");

      $dr_attr_rows = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE dr_id_fk = '".rp_escape_sql($dr_id)."'");

      if( !empty( $dr_row ) ) {

            $dr_category = $dr_row->dr_category;

            $dr_uq_id = $dr_row->dr_uq_id;

            $m_ids = rp_get_all_m_ids_by_category( $dr_category );

            if( !empty( $m_ids ) ) {

                  $all_m_ids_str = implode(',', $m_ids);

                  $existing = $rpQuery->get_results("SELECT * FROM `$rs_repair` WHERE m_id_fk IN ({$all_m_ids_str}) AND r_uq_id = '".rp_escape_sql($dr_uq_id)."'");

                  foreach($existing as $existing_row) {

                        // remove $existing_row->m_id_fk from $m_ids using array_diff

                        $m_ids = array_diff($m_ids, array($existing_row->m_id_fk));

                  }

                  $m_ids = array_unique($m_ids);

                  foreach($m_ids as $m_id_fk) {

                        $rpQuery->query("INSERT INTO `$rs_repair` (`r_name`, `r_discription`, `r_badge`, `r_duration`, `is_active`, `is_free`, `r_price`, `r_img`, `updated_on`, `r_date_time`, `r_uq_id`, `m_id_fk`, `r_duration_measure`) VALUES ('".rp_escape_sql($dr_row->dr_name)."', '".rp_escape_sql($dr_row->dr_discription)."', '".rp_escape_sql($dr_row->dr_badge)."', '".rp_escape_sql($dr_row->dr_duration)."', '".rp_escape_sql($dr_row->is_active)."', '".rp_escape_sql($dr_row->is_free)."', '".rp_escape_sql($dr_row->dr_price)."', '".rp_escape_sql($dr_row->dr_img)."', '".rp_escape_sql($dr_row->updated_on)."', '".rp_escape_sql($dr_row->dr_date_time)."', '".rp_escape_sql($dr_row->dr_uq_id)."', '".rp_escape_sql($m_id_fk)."', '".rp_escape_sql($dr_row->dr_duration_measure)."');");

                        // get last insert id

                        $last_r_id = $rpQuery->insert_id;

                        if( !empty( $dr_attr_rows ) ) {

                              foreach( $dr_attr_rows as $dr_attr_row ) {

                                    $rpQuery->query("INSERT INTO `$rs_repair_attr` (`a_name`, `a_desc`, `a_badge`, `a_duration`, `a_duration_measure`, `a_price`, `r_id_fk`, `updated_on`, `is_active`, `a_uq_id`) VALUES ('".rp_escape_sql($dr_attr_row->da_name)."', '".rp_escape_sql($dr_attr_row->da_desc)."', '".rp_escape_sql($dr_attr_row->da_badge)."', '".rp_escape_sql($dr_attr_row->da_duration)."', '".rp_escape_sql($dr_attr_row->da_duration_measure)."', '".rp_escape_sql($dr_attr_row->da_price)."', '".rp_escape_sql($last_r_id)."', '".rp_escape_sql($dr_attr_row->updated_on)."', '".rp_escape_sql($dr_attr_row->is_active)."', '".rp_escape_sql($dr_attr_row->da_uq_id)."');");

                              }

                        }

                  }

            }

      }

      $rpQuery->query("COMMIT");

}


function rp_add_this_new_default_repair_attr_to_all_models( $da_id = 0 ) {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      $da_id = (int) $da_id;

      if( empty( $da_id ) ) {

            return;

      }

      global $rpQuery;
      
      $rs_default_repair = $rpQuery->prefix . "rs_default_repair";

      $rs_default_repair_attr = $rpQuery->prefix . "rs_default_repair_attr";

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      $rpQuery->query("START TRANSACTION");

      $da_row = $rpQuery->get_row("SELECT * FROM `$rs_default_repair_attr` WHERE da_id = '".rp_escape_sql($da_id)."' LIMIT 1;");

      if( !empty( $da_row ) ) {

            $dr_row = $rpQuery->get_row("SELECT * FROM `$rs_default_repair` WHERE dr_id = '".rp_escape_sql($da_row->dr_id_fk)."' LIMIT 1;");

            if( !empty( $dr_row ) ) {

                  $dr_category = $dr_row->dr_category;

                  $dr_uq_id = $dr_row->dr_uq_id;

                  $m_ids = rp_get_all_m_ids_by_category( $dr_category );

                  if( !empty( $m_ids ) ) {

                        $all_m_ids_str = implode(',', $m_ids);
      
                        $existing = $rpQuery->get_results("SELECT * FROM `$rs_repair` WHERE m_id_fk IN ({$all_m_ids_str}) AND r_uq_id = '".rp_escape_sql($dr_uq_id)."'");
      
                        foreach($existing as $existing_row) {

                              // add to existing repairs
                              
                              // make sure it doesnt already exist
                              $existsAttr = $rpQuery->get_row("SELECT * FROM `$rs_repair_attr` WHERE r_id_fk = '".rp_escape_sql($existing_row->r_id)."' AND a_uq_id = '".rp_escape_sql($da_row->da_uq_id)."' LIMIT 1;");

                              if( empty( $existsAttr ) ) {

                                    $rpQuery->query("INSERT INTO `$rs_repair_attr` (`a_name`, `a_desc`, `a_badge`, `a_duration`, `a_duration_measure`, `a_price`, `r_id_fk`, `updated_on`, `is_active`, `a_uq_id`) VALUES ('".rp_escape_sql($da_row->da_name)."', '".rp_escape_sql($da_row->da_desc)."', '".rp_escape_sql($da_row->da_badge)."', '".rp_escape_sql($da_row->da_duration)."', '".rp_escape_sql($da_row->da_duration_measure)."', '".rp_escape_sql($da_row->da_price)."', '".rp_escape_sql($existing_row->r_id)."', '".rp_escape_sql($da_row->updated_on)."', '".rp_escape_sql($da_row->is_active)."', '".rp_escape_sql($da_row->da_uq_id)."');");

                              }
      
                        }

                  }

            }

      }

      $rpQuery->query("COMMIT");

}

function rp_get_all_m_ids_by_category( $category_id ) {

      $cache_key = 'rp_get_all_m_ids_by_category|'.$category_id;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      global $rpQuery;

      $m_ids = array();

      $rs_model = $rpQuery->prefix . 'rs_model';

      $models = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE `device_type` = '".rp_escape_sql($category_id)."'");

      if(!empty($models)) {

            foreach ($models as $model) {
                  
                  $m_ids[] = $model->m_id;

            }

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $m_ids);

      return $m_ids;

}

function delete_default_repair(){

      rp_verify_csrf_token_for_ajax('default_repairs');

    global $rpQuery;

    $rs_repair = $rpQuery->prefix . "rs_repair";

    $default_repair_table = $rpQuery->prefix . "rs_default_repair";

    $rs_default_repair_attr = $rpQuery->prefix . "rs_default_repair_attr";

    $rs_dr_translations = $rpQuery->prefix . "rs_dr_translations";

    $rs_r_translations = $rpQuery->prefix . "rs_r_translations";

    $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

    $rid = $_POST['rid'];

    $dr_category = $_POST['dr_category'] ?? 0;

    $result = $rpQuery->get_results("SELECT dr_uq_id FROM $default_repair_table WHERE dr_id='".rp_escape_sql($rid)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

    if(!empty($result)) {

      $dr_uq_id = $result[0]->dr_uq_id;

      // delete from rs_dr_translations where uq_id = $dr_uq_id

      // select * from rs_default_repair_attr where dr_id_fk = $rid
      $attrs = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE dr_id_fk='".rp_escape_sql($rid)."'");

      if( !empty($attrs) ) {

            foreach($attrs as $attr_row) {

                  $rpQuery->query("DELETE FROM `$rs_dr_translations` WHERE uq_id = '".rp_escape_sql($attr_row->da_uq_id)."'");

                  $rpQuery->query("DELETE FROM `$rs_rattr_translations` WHERE uq_id = '".rp_escape_sql($attr_row->da_uq_id)."'");

            }

      }


      $rpQuery->query("DELETE FROM `$rs_dr_translations` WHERE uq_id = '".rp_escape_sql($dr_uq_id)."'");

      $rpQuery->query("DELETE FROM `$rs_r_translations` WHERE uq_id = '".rp_escape_sql($dr_uq_id)."'");

      $m_ids = rp_get_all_m_ids_by_category( $dr_category );

      if(!empty($m_ids)) {

            $m_ids = join(',', $m_ids);

            $result1 = $rpQuery->query("DELETE FROM $rs_repair WHERE r_uq_id='".rp_escape_sql($dr_uq_id)."' AND m_id_fk IN (".rp_escape_sql($m_ids).")");

      }

      $result2 = $rpQuery->query("DELETE FROM $default_repair_table WHERE dr_id='".rp_escape_sql($rid)."' AND dr_category = '".rp_escape_sql($dr_category)."'");

      // delete from rs_default_repair_attr where dr_id_fk = $rid
      $result3 = $rpQuery->query("DELETE FROM `$rs_default_repair_attr` WHERE dr_id_fk='".rp_escape_sql($rid)."'");

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

    exit();

}

rp_ajax_for_admin('delete_default_repair','delete_default_repair');


//-------------------------------------------------------------------delete repairs

function delete_model_related_repairs(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery;

    $rs_repair = $rpQuery->prefix . "rs_repair";

    $mrid = $_POST['mrid'];     

    $result = $rpQuery->query("DELETE FROM $rs_repair WHERE m_id_fk='".rp_escape_sql($mrid)."'");

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

}

rp_ajax_for_admin('delete_model_related_repairs','delete_model_related_repairs');


//-------------------------------------------------------------------delete repairs attr

function delete_r_attr(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery;

    $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

    $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

    $aid = $_POST['aid'];

    if( !empty($aid) ) {

      $result = $rpQuery->get_row("SELECT * FROM $rs_repair_attr WHERE a_id = '".rp_escape_sql($aid)."'");

      if( !empty( $result ) ) {

            $rpQuery->query("DELETE FROM `$rs_rattr_translations` WHERE uq_id = '".rp_escape_sql($result->a_uq_id)."' AND r_id_fk = '".rp_escape_sql($result->r_id_fk)."'");

            $rpQuery->query("DELETE FROM $rs_repair_attr WHERE a_id='".rp_escape_sql($aid)."'");

      } 

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

}

rp_ajax_for_admin('delete_r_attr','delete_r_attr');


//-------------------------------------------------------------------delete default repairs attr

function delete_dr_attr(){

      rp_verify_csrf_token_for_ajax('default_repairs');

    global $rpQuery;

    $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

    $rs_default_repair_attr = $rpQuery->prefix . "rs_default_repair_attr";

    $rs_dr_translations = $rpQuery->prefix . "rs_dr_translations";

    $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

    $aid = $_POST['aid'];

    $dr_category = $_POST['dr_category'] ?? 0;     

    $result = $rpQuery->get_results("SELECT da_uq_id FROM $rs_default_repair_attr WHERE da_id='".rp_escape_sql($aid)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

    if(!empty($result)) {

      $da_uq_id = $result[0]->da_uq_id;

      // delete from rs_dr_translations where uq_id = $da_uq_id

      $rpQuery->query("DELETE FROM `$rs_dr_translations` WHERE uq_id = '".rp_escape_sql($da_uq_id)."'");

      $rpQuery->query("DELETE FROM `$rs_rattr_translations` WHERE uq_id = '".rp_escape_sql($da_uq_id)."'");

      $r_ids = rp_get_all_repair_ids_related_to_category($dr_category);

      if(!empty($r_ids)) {

            $r_ids = join(',', $r_ids);

            $rpQuery->query("DELETE FROM $rs_repair_attr WHERE a_uq_id='".rp_escape_sql($da_uq_id)."' AND r_id_fk IN (".rp_escape_sql($r_ids).")");

      }

      $rpQuery->query("DELETE FROM $rs_default_repair_attr WHERE da_id='".rp_escape_sql($aid)."' AND dr_category = '".rp_escape_sql($dr_category)."'");

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

}

rp_ajax_for_admin('delete_dr_attr','delete_dr_attr');


//-------------------------------------------------------------------update repairs attr

function update_r_attr(){

    global $rpQuery;

    $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

    $aid   = $_POST['aid'];     

    $name  = $_POST['name'];     

    $price = $_POST['price'];      

    $rpQuery->query("UPDATE $rs_repair_attr SET a_name='".rp_escape_sql($name)."',a_price='".rp_escape_sql($price)."' WHERE a_id='".rp_escape_sql($aid)."'");

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

}

rp_ajax_for_admin('update_r_attr','update_r_attr');


//-------------------------------------------------------------------delete models

function delete_model(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery;

    $rs_repair = $rpQuery->prefix . "rs_repair";

    $rs_model = $rpQuery->prefix . "rs_model";

    $modelcolor_table = $rpQuery->prefix . "rs_model_color";

    $mid = $_POST['mid'];     

    $result = $rpQuery->query("DELETE FROM $rs_model WHERE m_id='".rp_escape_sql($mid)."'");

    $rpQuery->query("DELETE FROM $modelcolor_table WHERE m_id_fk='".rp_escape_sql($mid)."'");

    $rpQuery->query("DELETE FROM $rs_repair WHERE m_id_fk='".rp_escape_sql($mid)."'");

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

}

rp_ajax_for_admin('delete_model','delete_model');


//-------------------------------------------------------------------delete models color

function delete_model_color(){
      
      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery;

    $modelcolor_table = $rpQuery->prefix . "rs_model_color";

    $mc_id = $_POST['mc_id'] ?? null;

    if(!empty($mc_id)) {
      $rpQuery->query("DELETE FROM $modelcolor_table WHERE mc_id='".rp_escape_sql($mc_id)."'");
    }  

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

    exit();

}

rp_ajax_for_admin('delete_model_color','delete_model_color');


//------------------------------------------------------------------- get related brands

function rp_get_special_schedule($loc_id = '', $day = '', $date_ymd = '') {

      $schedules = apply_filters('rp_getting_special_schedule_filter', array(), $loc_id, $day, $date_ymd);

      if( !empty( $schedules ) ) {

            return $schedules;

      }

      global $rpQuery;

      $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

      // get only m-d of Y-m-d format from $date_ymd
      $date_md = explode('-', $date_ymd);
      $date_md = $date_md[1] . '-' . $date_md[2];

      $result = rp_get_special_timing( $date_md, $loc_id );

      if( !empty( $result ) ) {

            $result->cs_day = $day;

            $schedules = array($result);

      } else {

            $schedules = $rpQuery->get_results("SELECT * FROM $rs_company_schedules WHERE cs_day = '".rp_escape_sql($day)."' AND location_id_fk = '".rp_escape_sql($loc_id)."'");

      }

      return $schedules;

}

function get_schedules(){

    global $rpQuery;
    global $wp_repair_lang;

    _rp_set_default_tiemzone();

    $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

    $rs_planning_and_discount = $rpQuery->prefix . "rs_planning_and_discount";  

    $rs_localization = $rpQuery->prefix . "rs_localization";   

    $day    = $_POST['day'];

    $date    = $_POST['date'];

    $date_ymd = explode('-', $date);

    $date_ymd = $date_ymd[2] . '-' . $date_ymd[1] . '-' . $date_ymd[0];

    $loc_id = $_POST['loc_id']; 

    $debug_info = array();

    $debug_info[] = array(
      'rs_company_schedules' => $rs_company_schedules,
      'cs_day' => $day,
      'location_id_fk' => $loc_id
    );

    $schedules = rp_get_special_schedule($loc_id, $day, $date_ymd);

    $schedules = rp_sanitize_output($schedules);

    if( !empty( $schedules ) ) {

            if( $schedules[0]->is_closed == '1' ) {

                  if( !empty($schedules[0]->reminder_name ?? '') ) {

                        echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => trim($schedules[0]->reminder_name)));

                        exit(); 

                  }

                  $schedules = array();

            }

      }

    if(!empty($schedules)){

            $debug_info[] = 'Found related schedules';

            $debug_info[] = $schedules;

            $has_valid_breaktime = TRUE;

            $valid_break_time_start = preg_match('/(?:\d){1,2}(?:\:){1}(?:\d){1,2}(?:(\sam|\spm)|($))+/i', $schedules[0]->cs_breaktime_from);
            $valid_break_time_end = preg_match('/(?:\d){1,2}(?:\:){1}(?:\d){1,2}(?:(\sam|\spm)|($))+/i', $schedules[0]->cs_breaktime_to);
            if(empty($schedules[0]->cs_breaktime_from) || empty($schedules[0]->cs_breaktime_to)) {
                  $has_valid_breaktime = FALSE;
            }
            if($valid_break_time_start === 0 || $valid_break_time_end === 0) {
                  $has_valid_breaktime = FALSE;
            }

            if($has_valid_breaktime === TRUE) {

                  $debug_info[] = 'Breaktime is valid';

            } else {

                  $debug_info[] = 'Invalid Breaktime';

            }


        $planning_discount = $rpQuery->get_results("SELECT * FROM $rs_planning_and_discount");

        if(!empty($planning_discount)) {

            $debug_info[] = 'Got Planning & Discount Row';

            $debug_info[] = array($planning_discount[0]);

        } else {

            $debug_info[] = 'Planning & Discount Row Could\'t found!';

        }

        $time_slots = $planning_discount[0]->booking_slots_length;

        $time_slots = str_replace(' Minutes (Default)', '', $time_slots);

        $time_slots = (int) $time_slots;

        $localization = $rpQuery->get_results("SELECT * FROM $rs_localization");

        if(!empty($localization)) {

            $debug_info[] = 'Got localization row';

            $debug_info[] = array($localization[0]);

        } else {

            $debug_info[] = 'localization Row Could\'t found!';

        }

        if($localization[0]->set_time_format == '24 hours (e.g. 15:00)'){

            $from_time = date("H:i",strtotime($schedules[0]->cs_open_timing));

            $to_time   = date("H:i",strtotime($schedules[0]->cs_close_timing));

            $minutes =  round(abs(strtotime($to_time) - strtotime($from_time)) / 60,2);

            if($has_valid_breaktime === TRUE) {
                  $break_from_time = date("H:i",strtotime($schedules[0]->cs_breaktime_from));

                  $break_to_time   = date("H:i",strtotime($schedules[0]->cs_breaktime_to));

                  $break_minutes =  round(abs(strtotime($break_to_time) - strtotime($break_from_time)) / 60,2);

                  $break_start_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_from) - strtotime($schedules[0]->cs_open_timing)) / 60;
                  $break_ends_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_to) - strtotime($schedules[0]->cs_open_timing)) / 60;
            }

        }else{

            $from_time = date("g:i a",strtotime($schedules[0]->cs_open_timing));

            $to_time   = date("g:i a",strtotime($schedules[0]->cs_close_timing));

            $minutes =  round(abs(strtotime($to_time) - strtotime($from_time)) / 60,2);

            if($has_valid_breaktime === TRUE) {
                  $break_from_time = date("g:i a",strtotime($schedules[0]->cs_breaktime_from));

                  $break_to_time   = date("g:i a",strtotime($schedules[0]->cs_breaktime_to));

                  $break_minutes =  round(abs(strtotime($break_to_time) - strtotime($break_from_time)) / 60,2);

                  $break_start_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_from) - strtotime($schedules[0]->cs_open_timing)) / 60;
                  $break_ends_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_to) - strtotime($schedules[0]->cs_open_timing)) / 60;
            }

        }

        $debug_arr = array();

        $debug_arr['from_time'] = $from_time;

        $debug_arr['to_time'] = $to_time;

        $debug_arr['minutes'] = $minutes;

        if($has_valid_breaktime === TRUE) {
            $debug_arr['break_from_time'] = $break_from_time;
            $debug_arr['break_to_time'] = $break_to_time;
            $debug_arr['break_minutes'] = $break_minutes;
            $debug_arr['break_start_after_x_minutes'] = $break_start_after_x_minutes;
            $debug_arr['break_ends_after_x_minutes'] = $break_ends_after_x_minutes;
        }

        $debug_info[] = $debug_arr;

        $break_times = array();

        if($has_valid_breaktime === TRUE) {
            if($break_start_after_x_minutes >= 0 && $break_start_after_x_minutes <= $minutes && $break_ends_after_x_minutes >= 0 && $break_ends_after_x_minutes <= $minutes) {

                  $debug_info[] = 'Breaktime verified & valid!';

            } else {
                $has_valid_breaktime = FALSE;

                $debug_info[] = 'Breaktime has got invalid!';
            }
        }

        for ($x = 0; $x <= $minutes; $x+=$time_slots) {

            $add_time = strtotime($from_time)+($x*60);

            if($localization[0]->set_time_format == '24 hours (e.g. 15:00)'){

                  $adding = date('H:i',$add_time);
                  $slot_time[] = $adding;

            }else{
                  $adding = date('g:i a',$add_time);
                  $slot_time[] = $adding;

            }   
            if($has_valid_breaktime === TRUE) {
                  if($x >= $break_start_after_x_minutes && $x < $break_ends_after_x_minutes) {
                        $break_times[] = $adding;
                  }
            }

        }

        $debug_info[] = 'Find Breaktimes below:';
        $debug_info[] = $break_times;

        $last_time = strtotime(date('Y-m-d H:i:s', strtotime($date_ymd.' '.$schedules[0]->cs_close_timing . ' -'.$time_slots.' minutes')));

        $shop_close_time = strtotime(date('Y-m-d H:i:s', strtotime($date_ymd.' '.$schedules[0]->cs_close_timing)));

        $current_time = date('Y-m-d H:i:s');

        $current_time_str = strtotime($current_time);

        $booking_must_wait_time = (int) rp_get_option('wp_repair_booking_must_wait_time', 60);

        $booking_must_wait_time_until = strtotime(date('Y-m-d H:i:s', strtotime($current_time .' +'.$booking_must_wait_time.' minutes')));

        $debug_info[] = array(
            'last_time' => $last_time,
            'shop_close_time' => $shop_close_time,
            'current_time' => $current_time,
            'current_time_str' => $current_time_str,
            'booking_must_wait_time' => $booking_must_wait_time,
            'booking_must_wait_time_until' => $booking_must_wait_time_until
        );

        $time_slots_printed = FALSE;

        $not_allowed = array();

        $fully_booked = array();

        foreach($slot_time as $time_key => $time_val) {

            $loop_time = strtotime(date('Y-m-d H:i:s', strtotime($date_ymd.' '.$time_val)));

            if($loop_time < $current_time_str) {
                  
                  unset($slot_time[$time_key]);
                  continue;
                  
            }

            if($loop_time > $last_time) {

                  unset($slot_time[$time_key]);
                  continue;

            }

            if(rp_is_available_slot($loc_id, date('Y-m-d', strtotime($date_ymd.' '.$time_val)), date('H:i:s', strtotime($date_ymd.' '.$time_val))) === FALSE) {

                  $fully_booked[] = $time_val;

            } else if($loop_time < $booking_must_wait_time_until) {

                  $not_allowed[] = $time_val;

            } else {

                  $debug_info[] = 'Timeslot Printed!';

                  $time_slots_printed = TRUE;

            }

        }

        $debug_info[] = array(
            'not_allowed' => $not_allowed,
            'fully_booked' => $fully_booked
        );

        if($time_slots_printed === FALSE) {

            if($current_time_str >= $shop_close_time) {

                  echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => $wp_repair_lang['shop_has_closed_for_today']));

                  exit(); 

            } else {

                  echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => $wp_repair_lang['appointment_booking_has_closed_for_today']));

                  exit();

            }

        } else {

            $slot_time = array_values($slot_time);

        }


        echo json_encode(Array('status'=> true, 'debug_info' => $debug_info, 'message' => $slot_time, 'break_times' => $break_times, 'not_allowed' => $not_allowed, 'fully_booked' => $fully_booked));

        exit();                   

    }else{

        echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => $wp_repair_lang['shop_is_closed_for_today']));

        exit();       

    } 

}


rp_ajax_for_public('get_schedules','get_schedules');



function get_schedules_new_dm(){

      global $rpQuery;
      global $wp_repair_lang;
  
      _rp_set_default_tiemzone();
  
      $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";
  
      $rs_planning_and_discount = $rpQuery->prefix . "rs_planning_and_discount";  
  
      $rs_localization = $rpQuery->prefix . "rs_localization";   
  
      $day    = $_POST['day'] ?? '';
  
      $date    = $_POST['date'] ?? '';
  
      $date_ymd = explode('-', $date);
  
      $date_ymd = $date_ymd[2] . '-' . $date_ymd[1] . '-' . $date_ymd[0];
  
      $loc_id = $_POST['method'] ?? ''; 

      if( $loc_id === 'pickup_service' ) {

            $_method = 'ps';

      }

      if( $loc_id === 'repair_on_location' ) {

            $_method = 'rol';

      }
 
      $debug_info = array();
  
      $debug_info[] = array(
        'rs_company_schedules' => $rs_company_schedules,
        'cs_day' => $day,
        'location_id_fk' => $loc_id
      );
  
      $schedules = rp_get_special_schedule_new_methods($loc_id, $day, $date_ymd);
  
      $schedules = rp_sanitize_output($schedules);
  
      if( !empty( $schedules ) ) {
  
              if( ($schedules[0]->is_closed ?? '0') == '1' ) {
  
                    if( !empty($schedules[0]->reminder_name ?? '') ) {
  
                          echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => trim($schedules[0]->reminder_name)));
  
                          exit(); 
  
                    }
  
                    $schedules = array();
  
              }
  
      }

      $delivery_timeslots = rp_get_delivery_timeslots($loc_id, $day);

      if( !empty( $schedules ) && !empty( $delivery_timeslots ) ) {

            // Has special timing!
            if( $schedules[0]->is_closed == 0 ) {

                $from_time = strtotime($schedules[0]->cs_open_timing);
                $to_time   = strtotime($schedules[0]->cs_close_timing);

                $filtered_delivery_timeslots = array();

                foreach( $delivery_timeslots as $dt_row ) {

                    $slot_start = strtotime($dt_row->slot_start);

                    $slot_end = strtotime($dt_row->slot_end);

                    if( $slot_start >= $from_time && $slot_end <= $to_time ) {
                        $filtered_delivery_timeslots[] = $dt_row;
                    }

                }

                $delivery_timeslots = $filtered_delivery_timeslots;

            } else {
                $delivery_timeslots = array();
            }

      }
  
      if(!empty($delivery_timeslots)){
  
              $debug_info[] = 'Found related schedules';
  
              $debug_info[] = $delivery_timeslots;
  
          $localization = $rpQuery->get_results("SELECT * FROM $rs_localization");
  
          if(!empty($localization)) {
  
              $debug_info[] = 'Got localization row';
  
              $debug_info[] = array($localization[0]);
  
          } else {
  
              $debug_info[] = 'localization Row Could\'t found!';
  
          }
  
            $slot_time = array();

            $last_time = '';

            $shop_close_time = '';

            foreach($delivery_timeslots as $dt_row) {

                  $slot_start = strtotime($date_ymd . ' ' . $dt_row->slot_start);

                  $slot_end = strtotime($date_ymd . ' ' . $dt_row->slot_end);

                  if( $localization[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
                      $slot_time[] = date('H:i', $slot_start).' — '.date('H:i', $slot_end);
                  } else {
                      $slot_time[] = date('g:i a', $slot_start).' — '.date('g:i a', $slot_end);
                  }

                  if( empty( $last_time ) ) {

                      $last_time = $slot_start;

                  } else {

                      if( $last_time < $slot_start ) {

                          $last_time = $slot_start;

                      }

                  }

                  if( empty( $shop_close_time ) ) {
                      
                      $shop_close_time = $slot_end;

                  } else {

                      if( $shop_close_time < $slot_end ) {

                          $shop_close_time = $slot_end;

                      }

                  }

            }

            $current_time = date('Y-m-d H:i:s');

            $current_time_str = strtotime($current_time);

            $booking_must_wait_time = (int) rp_get_option('wp_repair_booking_must_wait_time_'.$_method, 60);

            $booking_must_wait_time_until = strtotime(date('Y-m-d H:i:s', strtotime($current_time .' +'.$booking_must_wait_time.' minutes')));
  
          $debug_info[] = array(
              'last_time' => $last_time,
              'shop_close_time' => $shop_close_time,
              'current_time' => $current_time,
              'current_time_str' => $current_time_str,
              'booking_must_wait_time' => $booking_must_wait_time,
              'booking_must_wait_time_until' => $booking_must_wait_time_until
          );
  
          $time_slots_printed = FALSE;
  
          $not_allowed = array();
  
          $fully_booked = array();
  
          foreach($slot_time as $time_key => $time_val) {

              $the_time = explode(' — ', $time_val)[0];
  
              $loop_time = strtotime(date('Y-m-d H:i:s', strtotime($date_ymd.' '.$the_time)));
  
              if($loop_time < $current_time_str) {
                    
                    unset($slot_time[$time_key]);
                    continue;
                    
              }
  
              if($loop_time > $last_time) {
  
                    unset($slot_time[$time_key]);
                    continue;
  
              }
  
              if(rp_is_available_slot_new_delivery_methods($loc_id, date('Y-m-d', strtotime($date_ymd)), $time_val) === FALSE) {
  
                    $fully_booked[] = $time_val;
  
              } else if($loop_time < $booking_must_wait_time_until) {
  
                    $not_allowed[] = $time_val;
  
              } else {
  
                    $debug_info[] = 'Timeslot Printed!';
  
                    $time_slots_printed = TRUE;
  
              }
  
          }
  
          $debug_info[] = array(
              'not_allowed' => $not_allowed,
              'fully_booked' => $fully_booked
          );
  
          if($time_slots_printed === FALSE) {
  
              if($current_time_str >= $shop_close_time) {
  
                    echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => $wp_repair_lang['shop_has_closed_for_today']));
  
                    exit(); 
  
              } else {
  
                    echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => $wp_repair_lang['appointment_booking_has_closed_for_today']));
  
                    exit();
  
              }
  
          } else {
  
              $slot_time = array_values($slot_time);
  
          }
  
  
          echo json_encode(Array('status'=> true, 'debug_info' => $debug_info, 'message' => $slot_time, 'not_allowed' => $not_allowed, 'fully_booked' => $fully_booked));
  
          exit();   
  
      }else{
  
          echo json_encode(Array('status'=> false, 'debug_info' => $debug_info, 'message' => $wp_repair_lang['shop_is_closed_for_today']));
  
          exit();       
  
      } 
  
}

rp_ajax_for_public('get_schedules_new_dm','get_schedules_new_dm');


function rp_get_special_schedule_new_methods($method = '', $day = '', $date_ymd = '') {

      if( $method === 'pickup_service' ) {

            $method = 'ps';

      }

      if( $method === 'repair_on_location' ) {

            $method = 'rol';

      }

      // get only m-d of Y-m-d format from $date_ymd
      $date_md = explode('-', $date_ymd);
      $date_md = $date_md[1] . '-' . $date_md[2];

      $result = rp_get_special_timing( $date_md, $method );

      $schedules = array();

      if( !empty( $result ) ) {

            $result->cs_day = $day;

            $schedules = array($result);

      }

      return $schedules;

}


//-------------------------------------------------------------------Check Valid Breaktime
function get_invalid_breaktimes_errors() {
    global $rpQuery;
    $company_location    = $rpQuery->prefix . "rs_company_location";
    $company_schedules   = $rpQuery->prefix . "rs_company_schedules";

    $get_location = $rpQuery->get_results("SELECT * FROM $company_location");

    $errors = array();
    foreach($get_location as $loc){
        $locid = $loc->location_id;
        $schedules = $rpQuery->get_results("SELECT * FROM $company_schedules WHERE location_id_fk = '".rp_escape_sql($locid)."'");
        if(!empty($schedules)) {
            foreach ($schedules as $sch) {
                $result = is_valid_breaktime($sch->cs_day, $locid);
                if($result !== TRUE) {
                    if(array_search($result, $errors) === FALSE) {
                        $errors[] = $result;
                    }
                }
            }
        }
    }
    return $errors;
}
function is_valid_breaktime($day, $loc_id){

    global $rpQuery;
    global $wp_repair_lang;

    $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

    $rs_planning_and_discount = $rpQuery->prefix . "rs_planning_and_discount";  

    $rs_localization = $rpQuery->prefix . "rs_localization";   

    if(!isset($GLOBALS['schedules_for_breaktime'])) {
      $_schedulesList = $rpQuery->get_results("SELECT * FROM $rs_company_schedules WHERE is_closed = '0'");
      $schedulesList = array();
      foreach ($_schedulesList as $key => $value) {
            $schedulesList[$value->location_id_fk.$value->cs_day] = $value;
      }
      $GLOBALS['schedules_for_breaktime'] = $schedulesList;
    }

    if(isset($GLOBALS['schedules_for_breaktime'][$loc_id.$day])) {
      $schedules = array($GLOBALS['schedules_for_breaktime'][$loc_id.$day]);
    } else {
      $schedules = null;
    }

    if($schedules){

            $valid_break_time_start = preg_match('/(?:\d){1,2}(?:\:){1}(?:\d){1,2}(?:(\sam|\spm)|($))+/i', $schedules[0]->cs_breaktime_from);
            $valid_break_time_end = preg_match('/(?:\d){1,2}(?:\:){1}(?:\d){1,2}(?:(\sam|\spm)|($))+/i', $schedules[0]->cs_breaktime_to);
            if(empty($schedules[0]->cs_breaktime_from) || empty($schedules[0]->cs_breaktime_to)) {
                  return TRUE;
            }
            if($valid_break_time_start === 0 || $valid_break_time_end === 0) {
                  return 'invalid_break_time';
            }

        $planning_discount = $rpQuery->get_results("SELECT * FROM $rs_planning_and_discount");

        $time_slots = $planning_discount[0]->booking_slots_length;

        $time_slots = (int) trim(explode('Minutes', $time_slots)[0]);

        $localization = $rpQuery->get_results("SELECT * FROM $rs_localization");

        if($localization[0]->set_time_format == '24 hours (e.g. 15:00)'){

            $from_time = date("H:i",strtotime($schedules[0]->cs_open_timing));

            $to_time   = date("H:i",strtotime($schedules[0]->cs_close_timing));

            $minutes =  round(abs(strtotime($to_time) - strtotime($from_time)) / 60,2);


            $break_from_time = date("H:i",strtotime($schedules[0]->cs_breaktime_from));

            $break_to_time   = date("H:i",strtotime($schedules[0]->cs_breaktime_to));

            $break_minutes =  round(abs(strtotime($break_to_time) - strtotime($break_from_time)) / 60,2);

            $break_start_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_from) - strtotime($schedules[0]->cs_open_timing)) / 60;
            $break_ends_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_to) - strtotime($schedules[0]->cs_open_timing)) / 60;

        }else{

            $from_time = date("g:i a",strtotime($schedules[0]->cs_open_timing));

            $to_time   = date("g:i a",strtotime($schedules[0]->cs_close_timing));

            $minutes =  round(abs(strtotime($to_time) - strtotime($from_time)) / 60,2);


            $break_from_time = date("g:i a",strtotime($schedules[0]->cs_breaktime_from));

            $break_to_time   = date("g:i a",strtotime($schedules[0]->cs_breaktime_to));

            $break_minutes =  round(abs(strtotime($break_to_time) - strtotime($break_from_time)) / 60,2);

            $break_start_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_from) - strtotime($schedules[0]->cs_open_timing)) / 60;
            $break_ends_after_x_minutes = (strtotime($schedules[0]->cs_breaktime_to) - strtotime($schedules[0]->cs_open_timing)) / 60;

        }

        $break_times = array();

        if($break_start_after_x_minutes >= 0 && $break_start_after_x_minutes <= $minutes && $break_ends_after_x_minutes >= 0 && $break_ends_after_x_minutes <= $minutes) {

        } else {
            return 'mistake_in_am_pm';
        }

        if($break_start_after_x_minutes > $break_ends_after_x_minutes) {
            return 'break_start_is_greater';
        }

        for ($x = 0; $x <= $minutes; $x+=$time_slots) {

            $add_time = strtotime($from_time)+($x*60);

            if($localization[0]->set_time_format == '24 hours (e.g. 15:00)'){

                  $adding = date('H:i',$add_time);
                  $slot_time[] = $adding;

            }else{
                  $adding = date('g:i a',$add_time);
                  $slot_time[] = $adding;

            }   

            if($x >= $break_start_after_x_minutes && $x < $break_ends_after_x_minutes) {
                  $break_times[] = $adding;
            }

        }

        if(empty($break_times)) {
            return 'mistake_in_am_pm';
        } else {
            return TRUE;
        }

    }else{

        return TRUE;     

    } 

}

// ---------------------------------------- Get Duplicate Email by Status

function wp_repair_admin_duplicate_email($status = '', $location_id = '') {

      return wp_repair_get_location_email( $status, $location_id );

}

// ---------------------------------------- Get Location Email by Location Id

function wp_repair_get_location_email($status = '', $location_id = '') {

      $admin_enabled = rp_get_option('wp_repair_admin_duplicate_email_status_'.$status, '0');

      if($admin_enabled === '1') {
            // Need to Send Email!

            // Get related special email
            $admin_email = wp_repair_get_location_email_org( $location_id );

            $recipient = FALSE;

            if(!empty($admin_email)) {

                  // Special Email is Set
                  // Special Email Must NOT Equal to Placeholder email & must be valid
                  if(trim($admin_email) !== rp_get_settings_placeholder_email() && filter_var(trim($admin_email), FILTER_VALIDATE_EMAIL) !== FALSE) {

                        // Good to Set As Recipient
                        $recipient = trim($admin_email);

                  }

            } else {

                  // Send Email to Company Email
                  $company_email = rp_get_settings_company_email();

                  // Company Email Must NOT Equal to Placeholder email & must be valid
                  if(trim($company_email) !== rp_get_settings_placeholder_email() && filter_var(trim($company_email), FILTER_VALIDATE_EMAIL) !== FALSE) {

                        // Good to Set As Recipient
                        $recipient = trim($company_email);

                  }

            }

            return $recipient;

      }

      // No Need to send email..
      return FALSE;

}

// ---------------------------------------- Check Whether Duplicate Email by Status is Synced with Company Email

function wp_repair_admin_duplicate_email_is_synced($status = '') {

      // Get related special email
      $admin_email = rp_get_option('wp_repair_admin_duplicate_email_status_'.$status.'_email', NULL);

      // Means it is synced with Company Email
      return empty($admin_email);

}

// ---------------------------------------- Check Whether Location Email by Status is Synced with Company Email

function wp_repair_get_location_email_org($id = '') {

      global $rpQuery;

      $company_location    = $rpQuery->prefix . "rs_company_location";

      $location = $rpQuery->get_row("SELECT * FROM $company_location WHERE location_id = $id LIMIT 1;");

      if( !empty( $location ) ) {

            return (!empty( $location->location_email ) ? $location->location_email : NULL);

      }

      return NULL;

}

function wp_repair_location_email_is_synced($id = '') {

      return empty( wp_repair_get_location_email_org( $id ) );

}


//-------------------------------------------------------------------update appointment status

function rp_get_location_id_of_order( $o_id = '' ) {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . "rs_orders";

      $order = $rpQuery->get_row("SELECT * FROM $rs_orders WHERE o_id = '".rp_escape_sql($o_id)."' LIMIT 1;");

      if( !empty( $order ) ) {

            return ($order->location_id_fk ?? 0);

      }

      return 0;

}

function rp_find_link_pattern($what = '', $message = '') {

      if( !empty( $what ) && !empty( $message ) ) {

            preg_match_all('/(?:\['.$what.'=(?:"|&quot;))[\p{L}\s!@#\$%\^&\*\(\)_\+-=\{\}\\\|;:\'\/\?.,<>]+(?:(?:"|&quot;)\])/u', $message, $matches);

            if( !empty( $matches ) && isset( $matches[0] ) && !empty( $matches[0] ) ) {

                  if( isset( $matches[0][0] ) && !empty( $matches[0][0] ) ) {

                        return $matches[0][0];

                  }

            }

      }

      return FALSE;

}

function rp_handle_currency_position($amount = 0, $currency = '', $add_space = FALSE) {

      $currency_position = rp_get_option('rp_set_price_symbol', 'left');

      $space = ($add_space === TRUE ? ' ' : '');

      if($currency_position == 'left') {

            return $currency.$space.$amount;

      } else {

            return $amount.$space.$currency;

      }

}

function rp_repair_name_fix_filter( $name = '', $r_id = '' ) {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $repair = $rpQuery->get_row("SELECT * FROM `$rs_repair` WHERE r_id = '".rp_escape_sql($r_id)."' LIMIT 1;");

      if( !empty( $repair ) ) {

            list($rows, $ig) = rp_translate_locked_repairs_and_attributes( array($repair), array() );

            $repair = $rows[0];

            if( !empty( $repair ) ) {

                  $name = $repair->r_name . ' - '. $name;

            }

      }

      return $name;

}

function rp_add_repair_name_fix_filter() {

      add_filter('rp_preparing_attributes_name', 'rp_repair_name_fix_filter', 10, 2);

}

function rp_remove_repair_name_fix_filter() {
      
      remove_filter('rp_preparing_attributes_name', 'rp_repair_name_fix_filter', 10, 2);

}

function rp_get_wordpress_upload_path() {

      $upload_dir = wp_upload_dir();

      return $upload_dir['basedir'];

}

function rp_get_wordpress_upload_dir_url() {

      $upload_dir = wp_upload_dir();

      return $upload_dir['baseurl'];

}

function rp_fix_www_image_pdf_issue( $the_image = '' ) {

      $site_url = rp_get_wordpress_upload_dir_url();

      // replace www. from site url
      $site_url = str_replace('//www.', '//', $site_url);

      // replace www. from company logo url
      $the_image = str_replace('//www.', '//', $the_image);

      // replace site url from company logo url
      $the_image = str_replace($site_url, '', $the_image);

      // readd original site url
      $the_image = rp_get_wordpress_upload_dir_url().$the_image;

      return $the_image;

}

function rp_get_company_theme_color() {

      global $rpQuery;

      $company_information = $rpQuery->prefix . "rs_company_information";

      $company_results = $rpQuery->get_results("SELECT * FROM `$company_information`");

      $theme_color = $company_results[0]->company_theme_color;

      if( !empty( $theme_color ) ) {

            return $theme_color;

      }

      return '2781c7';

}

function rp_handle_sending_pdf_offer( $received_o_id = '' ) {

      global $rpQuery;

      global $wp_repair_lang;

      $rs_localization = $rpQuery->prefix . "rs_localization"; 

      $rs_model = $rpQuery->prefix. "rs_model";

      $rs_brand = $rpQuery->prefix. "rs_brand";

      $rs_user_pdfs = $rpQuery->prefix. "rs_user_pdfs";

      $rs_customer = $rpQuery->prefix. "rs_customer";

      $rs_orders = $rpQuery->prefix. "rs_orders";

      $rs_company_location = $rpQuery->prefix . "rs_company_location";

      $rs_order_repairs = $rpQuery->prefix. "rs_order_repairs";

      $rs_order_repair_attr = $rpQuery->prefix. "rs_order_repair_attr";

      $rs_order_upsales = $rpQuery->prefix. "rs_order_upsales";

      $tax_row = $rpQuery->get_row("SELECT * FROM $rs_localization LIMIT 1;");

      if( !empty( $tax_row ) ) {

        $tax = $tax_row->tax;

      } else {

        $tax = 0;

      }

      $order_row = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE o_id = '$received_o_id' LIMIT 1;");

      if( empty( $order_row ) ) {

            return FALSE;

      }

      $mid          = $order_row->m_id_fk;

      $rp_location_id_fk = $order_row->location_id_fk;

      $selected_location_row = $rpQuery->get_row("SELECT * FROM `$rs_company_location` WHERE location_id = '$rp_location_id_fk' LIMIT 1;");

      $model_name   = '';

      if( !empty( $mid ) ) {

        $model_row = $rpQuery->get_row("SELECT m_id, m_name FROM $rs_model WHERE m_id = ':m_id' LIMIT 1;", array(

              'm_id' => $mid

        ));

        if( empty( $model_row ) ) {

              $mid = 0;

        } else {

              $model_name = $model_row->m_name;

        }

      }

      list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $received_o_id );

      $combo        = $calculations['combo_discount'];

      $coupon       = $calculations['coupon_amount'];

      $final_price  = $calculations['total'];

      $coupon_info = $rpCart->get_coupon_info();

      $rid = array();

      $aid = array();

      $upid = array();

      foreach( $rpCart->repairs as $cartRow ) {

            $rid[] = $cartRow->r_id;

      }

      foreach( $rpCart->repair_attrs as $cartRow ) {

            $rid[] = $cartRow->r_id_fk;

            $aid[] = $cartRow->a_id;

      }

      foreach( $rpCart->upsales as $cartRow ) {

            $upid[] = $cartRow->upsales_id;

      }

      rp_add_repair_name_fix_filter();

      $prepared_post_data = $rpCart->prepare_post_data( $rid, $upid );

      rp_remove_repair_name_fix_filter();

      list( $rid, $array_name, $array_price, $aid, $upid ) = $prepared_post_data;

      $order_id_fk = $received_o_id;

      $color_name   = $order_row->color;

      $customer_id_fk  = $order_row->customer_id_fk;

      $user_pdf_row = $rpQuery->get_row("SELECT * FROM `$rs_user_pdfs` WHERE `o_id_fk` = '$received_o_id' LIMIT 1;");

      $pdf_date_time = date('Y-m-d H:i:s');

      $customer_row = $rpQuery->get_row("SELECT * FROM `$rs_customer` WHERE `customer_id` = '$customer_id_fk' LIMIT 1;");

      if( !empty( $customer_row ) ) {

            $user_name    = $customer_row->customer_name;

            $user_phone = $customer_row->customer_phone;

            $email = $customer_row->customer_email;

      } else {

            $user_name = '--';

            $user_phone = '--';

            $email = '';

      }

      if( !empty( $user_pdf_row ) ) {

            $street_phone = $user_pdf_row->streetname_phone;

            $zipcode_city = $user_pdf_row->zipcode_city . ', '. $user_pdf_row->zipcode_only;

            $notes        = $user_pdf_row->notes;

            $finalize_btn_link = $user_pdf_row->finalize_link;

            $offer_id = $user_pdf_row->user_pdfs_id;

            $rpQuery->query("UPDATE `$rs_user_pdfs` SET `pdf_date_time` = '{$pdf_date_time}' WHERE `o_id_fk` = '$received_o_id' LIMIT 1;");

      } else {

            $street_phone = '';

            $zipcode_city = '';

            $notes        = '';

            $finalize_btn_link = '';

            $offer_id = '0';

      }

      if(!empty($mid)) {

        $cmodel = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE m_id = '".rp_escape_sql($mid)."' LIMIT 1;");

      } else {

        $cmodel = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE m_name = '".rp_escape_sql($model_name)."' LIMIT 1;");

      }

      $brand_name = '';
      if(!empty($cmodel)) {
        $bid = $cmodel[0]->brand_id_fk;
        $cbrand = $rpQuery->get_results("SELECT * FROM `$rs_brand` WHERE b_id = '".rp_escape_sql($bid)."'");
        if(!empty($cbrand)) {
              $brand_name = $cbrand[0]->b_name . ' ';
        }
      }

      $etp = 1 + (($tax * 1) / 100);

      $ex_tax_price = $final_price/$etp;

      $total_tax = (float)$final_price - (float)$ex_tax_price; 

      $total_tax = number_format((float)$total_tax, 2, '.', '');

      $ex_tax_price = number_format((float)$ex_tax_price, 2, '.', '');


      $company_information = $rpQuery->prefix . "rs_company_information";

      $c_results = $rpQuery->get_results("SELECT * FROM $company_information"); 

      $company_location = $rpQuery->prefix . "rs_company_location";

      $company_address = $rpQuery->get_results("SELECT * FROM $company_location"); 

      if( rp_is_active_enhanced_locations_addon() === true ) {

            if( !empty( $selected_location_row ) && !empty( $selected_location_row->location_email ?? '' ) ) {

                  $c_results[0]->company_email = $selected_location_row->location_email;

            }

            if( !empty( $selected_location_row ) && !empty( $selected_location_row->location_telephone ?? '' ) ) {

                  $c_results[0]->company_phone = $selected_location_row->location_telephone;

            }

            if( !empty( $selected_location_row ) ) {

                  $company_address = array( $selected_location_row );

            }

      }

      $rs_localization = $rpQuery->prefix . "rs_localization";

      $localization = $rpQuery->get_results("SELECT * FROM $rs_localization");

      $tax_indicator = $localization[0]->tax_indicator;

      $tax_indicator = $wp_repair_lang['tax'];

      $p_symbol = $localization[0]->set_currency;

      $decimal = $localization[0]->set_decimals;

      if(strpos($total_tax, '.') === FALSE) {
        $total_tax = number_format((float)$total_tax, 2, '.', '');
      }

      if(!empty($combo)) {
        $combo = number_format((float)$combo, 2, '.', '');
      }

      if(!empty($coupon)) {
        $coupon = number_format((float)$coupon, 2, '.', '');
      }

      $final_price = number_format((float)$final_price, 2, '.', '');

      if($decimal == 'comma'){

          $ex_tax_price = str_replace('.',',',$ex_tax_price);

          $total_tax = str_replace('.',',',$total_tax);

          $final_price = str_replace('.',',',$final_price);

          if(!empty($combo)) {
              $combo = str_replace('.', ',', $combo);
          }

          if(!empty($coupon)) {
              $coupon = str_replace('.', ',', $coupon);
          }

      }

      require_once plugin_dir_path( __FILE__).'html/front_end/dompdf/autoload.inc.php';

      $file_name = $wp_repair_lang['pdf_offer_filename'].pretty_offer_id($offer_id).'.pdf';


      $pdf = new RPDompdf\Dompdf();

      $font_family = 'font-family: Arial, sans-serif;';

      $html = ''; 

      $rp_theme_color = rp_get_company_theme_color();

      $html .= '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"> 
      <html> 
      <head> 
      <meta http-equiv="Content-Type" content="charset=utf-8" />
          <style type="text/css">
              * {
                '.$font_family.'
              }
              body, html {
                    width: 100%;
                    padding: 0px;
                    margin: 0px;
              }
              table td {
                    font-family: \'Montserrat-Medium\';
                    font-size: 13px;
              }
              body {
                    padding: 50px 55px;
                    position: relative;
              }
              .company-details {
                    border-spacing: 0px;
                    margin-top: 8px;
              }
              .company-details td {
                    font-size: 14px;
              }
              .quotation_text_dates {
                    border-spacing: 0px;
              }
              .medium {
                    font-family: \'Montserrat-Medium\';
                    line-height: 1.4em;
              }
              .medium.addr {
                    line-height: 1em;
              }
              td, th, p {
                  font-size: 14px;
                  line-height: 1em;
              }
              h1, h2, h3, h4, h5, h6, p {
                  display: table;
              }
              h2 {
                  display: inline-table;
                  float: left;
                  margin:0px;
                  padding:0px;
                  line-height: 1em;
              }
              .quotation_text td {
                  border-spacing: 0px;
                  margin:0px;
                  padding:25px 0px;
                  vertical-align: middle;
              }
              .p-table {
                    border-spacing: 0px;
              }
              .quotation_text  {
                  padding: 0px;
                  padding-top: 50px;
              }
              .company-logo {
                    display: block;
                    margin: 0px;
                    padding: 0px;
                    position: absolute;
                    top: 15px;
              }
              .font-envelope {
                    font-size: 12px;
                    font-family: \'Montserrat-Medium\';
              }
              .customer-details {
                    display: table;
                    margin: 0px;
                    padding: 0px;
                    position: absolute;
                    top: 185px;
                    left: 100px;
              }
              .font-envelope-customer-details {
                    font-family: \'Montserrat-Regular\';
                    font-size: 9px;
                    letter-spacing: 1px;
                    padding-bottom: 7px;
                    color: #5a5a5a;
              }
              .quotation_text_dates td {
                    padding: 0px;
              }
              .p-text {
                    line-height: 1.3em;
                    font-family: \'Montserrat-Medium\';
                    font-size: 13px;
              }
              h2 span,b {
                    font-family: \'Montserrat-Bold\';
              }
              .no-border-spacing,
              .no-border-spacing tr,
              .no-border-spacing tr td,
              .no-border-spacing img,
              .no-border-spacing span,
              .no-border-spacing span.medium,
              .no-border-spacing h4,
              .no-border-spacing h2,
              .no-border-spacing b {
                    border-spacing: 0px;
                    padding: 0px;
                    margin: 0px;
                    line-height: 1em;
              }
              .no-border-spacing tr td {
                    padding: 5px;
                    vertical-align:top;
              }
              .no-border-spacing.location-table tr td {
                    padding: 0px 5px;
              }
              .t-align-center {
                    text-align:center;
              }
              .t-align-right {
                  text-align:right;
                  }
              .icon,
              .location-icon,
              .phone-icon {
                    padding-top: 3px;
              }
              .icon-text {
                    font-size: 10px;
                    padding-top: 4px;
              }
              .location-table.city-district {
                    padding-bottom: 22px;
              }
              .long_email {
                    font-size: 12px;
              }
              .percent-off-badge {
                  font-family: \'Montserrat-Bold\';
                  font-weight: 700;
                  font-size: 10px;
                  background: #fff;
                  color: #'.$rp_theme_color.';
                  border: 1px solid #'.$rp_theme_color.';
                  line-height: 9px;
                  text-transform: uppercase;
                  padding: 0px 5px 0px 5px !important;
                  box-sizing: border-box;
                  border-radius: 6px;
                  vertical-align: middle;
                  white-space: nowrap;
                  display: inline-block;
                  height: 14px;
                  width: auto;
                  margin-left: 5px;
                  margin-top:2px;
            }
            </style>
      <title>Offer PDF</title> 
      </head> 
       
      <body>';
       
       if(strtolower($tax_indicator) === 'vat') {
        $tax_indicator_cd = 'VAT';
       } else if(strtolower($tax_indicator) === 'btw') {
        $tax_indicator_cd = 'btw';
       } else {
        $tax_indicator_cd = $tax_indicator;
       }

       $tax_indicator_id = $wp_repair_lang['registeration_id'];
       if(!empty($c_results[0]->company_logo) && $c_results[0]->company_logo !== 'http://site.com/picture') {

            $c_results[0]->company_logo = rp_fix_www_image_pdf_issue( $c_results[0]->company_logo );

            $company_logo_img_path = str_replace(rp_get_wordpress_upload_dir_url(), rp_get_wordpress_upload_path(), $c_results[0]->company_logo);

            list($company_logo_img_size_w, $company_logo_img_size_h) = getimagesize($company_logo_img_path);

            $company_logo_img = '<img class="company-logo" src="'.rp_encode_img_base64($c_results[0]->company_logo).'" '.rp_get_company_logo_width_height_attrs($company_logo_img_size_w, $company_logo_img_size_h).'>';

       } else {

            $company_logo_img = '';

       }
       if(strlen($c_results[0]->company_email) > 24) {
        $long_email = 'long_email';
       } else {
        $long_email = '';
       }
       if(strtolower($tax_indicator_cd) === 'btw') {
        $tax_indicator_cd = 'Btw';
        $tax_indicator_cd_small = 'btw';
        $tax_indicator = 'Btw';
       } else {
        $tax_indicator_cd = $tax_indicator_cd;
        $tax_indicator_cd_small = $tax_indicator_cd;
       }
       if(!empty($cmodel[0]->m_img) && $cmodel[0]->m_img !== 'http://site.com/picture') {

            $cmodel[0]->m_img = rp_fix_www_image_pdf_issue( $cmodel[0]->m_img );

            $m_img_path = str_replace(rp_get_wordpress_upload_dir_url(), rp_get_wordpress_upload_path(), $cmodel[0]->m_img);

            list($m_img_size_w, $m_img_size_h) = getimagesize($m_img_path);

            $m_img = '<img class="company-logo" src="'.rp_encode_img_base64($cmodel[0]->m_img).'" '.rp_get_m_img_width_height_attrs($m_img_size_w, $m_img_size_h).'>';

           } else {
            $m_img = '';
           }
      $html .= '<table width="100%">

          <tr>

              <td width="51%">'.$company_logo_img.'</td>

              <td width="49%">

                  <table class="company-details">

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing location-table">

                                      <tr>
                                            <td width="100px" class="t-align-right">'.show_dompdf_svg('location').'</td>
                                            <td width="205px"><h4><b>'.$c_results[0]->company_name.'</b></h4></td>
                                      </tr>

                                </table>

                          </td>

                      </tr>

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing location-table">

                                      <tr>
                                            <td width="100px" class="t-align-right"> </td>
                                            <td width="205px"><span class="medium addr">'.$company_address[0]->location_address.'</span></td>
                                      </tr>

                                </table>

                          </td>

                      </tr>

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing location-table city-district">

                                      <tr>
                                            <td width="100px" class="t-align-right"> </td>
                                            <td width="205px"><span class="medium addr">'.$company_address[0]->location_city_district.'</span></td>
                                      </tr>

                                </table>

                          </td>

                      </tr>

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing">

                                      <tr>

                                            <td width="100px" class="t-align-right">'.show_dompdf_svg('email').'</td>

                                            <td width="205px"><span class="medium '.$long_email.'">'. $c_results[0]->company_email.'</span></td>

                                      </tr>

                                </table>

                          </td>

                      </tr>

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing">

                                      <tr>

                                            <td width="100px" class="t-align-right">'.show_dompdf_svg('phone').'</td>

                                            <td width="205px"><span class="medium">'. $c_results[0]->company_phone.'</span></td>

                                      </tr>

                                </table>

                          </td>

                      </tr>

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing icon-text-table">

                                      <tr>

                                            <td width="100px" class="t-align-right"><div class="icon-text '.strtolower(str_replace(' ', '-', $tax_indicator_id)).'"><b>'.$tax_indicator_id.'</b></div></td>

                                            <td width="205px"><span class="medium">'. $c_results[0]->company_reg_no.'</span></td>

                                      </tr>

                                </table>

                          </td>

                      </tr>

                      <tr>

                          <td>

                                <table width="100%" class="no-border-spacing icon-text-table">

                                      <tr>

                                            <td width="100px" class="t-align-right"><div class="icon-text"><b>'.$tax_indicator_cd_small.'</b></div></td>

                                            <td width="205px"><span class="medium">'.$c_results[0]->tax_reg_no.'</span></td>

                                      </tr>

                                </table>

                          </td>

                      </tr>

                  </table>

              </td>

          </tr>

      </table>

      <table width="100%" class="customer-details">

        <tr>

            <td class="font-envelope-customer-details">'.$wp_repair_lang['customer_details'].'</td>

        </tr>

          <tr>

              <td class="font-envelope">'.$user_name.'</td>

          </tr>

          <tr>

              <td class="font-envelope">'.$street_phone.'</td>

          </tr>

          <tr>

              <td class="font-envelope">'.$zipcode_city.'</td>

          </tr>

      </table>

      <table width="100%" style="margin: 10px 0px;" class="quotation_text">

          <tr>

              <td width="70%"><h2><span>'.$wp_repair_lang['quotation'].' '.pretty_offer_id($offer_id).'</span></h2></td>

              <td width="30%">

                    <table width="100%" style="margin: 10px 0px;" class="quotation_text_dates">

                        <tr>

                            <td width="60%" style="text-align:right" class="font-envelope">'.$wp_repair_lang['quotation_date'].': &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
                            <td width="40%" style="text-align: right" class="font-envelope">'.date('d-m-Y',strtotime($pdf_date_time)).'</td>

                        </tr>

                        <tr>

                            <td width="60%" style="text-align:right" class="font-envelope">'.$wp_repair_lang['expiration_date'].': &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
                            <td width="40%" style="text-align: right" class="font-envelope">'.date('d-m-Y',strtotime($pdf_date_time .' +'.rp_get_option('rp_expire_offer_after_x_days', 14).' days')).'</td>

                        </tr>

                    </table>

              </td>

          </tr>

          <tr>
                  <td style="margin-top:0px;">
                        <div style="display:inline-block; margin-top:-60px; ">'.$m_img.'</div>
                        
                        <table width="100%" style="margin-top:-50px; margin-left:55px;" class="quotation_text_dates">
                              <tr>
                                  <td width="65%" style="line-hright:1.3em; text-transform:capitalize !important ; font-family:\'Montserrat-Regular\'; font-size:9px; letter-spacing:1px ; padding-bottom:3px; color:#5a5a5a;  ">
                                    '.strtoupper($wp_repair_lang['device_details']).'
                                  </td>  
                              </tr>
                              <tr>
                                    <td width="65%" style="line-height:1.1em; font-size:12px; text-transform:caplitalize;">
                                          '.$brand_name.$model_name.'
                                    </td>
                              </tr>
                              <tr>
                                    <td width="65%" style="line-height:1.1em; font-size:12px; text-transform:caplitalize;">
                                          '.$color_name.'
                                    </td>
                              </tr>
                              <tr>
                                    <td width="65%" style="line-height:1.1em; font-size:12px; text-transform:caplitalize;">
                                          '.$wp_repair_lang['imei'].' '.$notes.'
                                    </td>
                              </tr>
                        </table>
                  </td>
    
            </tr>

      </table>

      <table class="p-table" width="100%" style="margin: -10px 0px 10px 0px;">

          <tr>

              <td style="margin:0px 0px 5px 0px" class="p-text">'.$wp_repair_lang['dear'].' '.$user_name.',</td>

          </tr>

      </table>

      <table class="p-table" width="100%" style="margin: 10px 0px;">

          <tr>

              <td style="margin:5px 0px" class="p-text">'.$wp_repair_lang['appointment_offer_thank_you_message'].'</td>

          </tr>

      </table>

      <table width="100%" style="margin: 25px 0px 0px 0px;border-bottom: 1px solid; padding: 5px 0px;">

          <tr>

              <td width="10%" style="text-transform: capitalize"></td>

              <td width="65%" style="text-transform: capitalize;"><b>'.$wp_repair_lang['repairs'].'</b></td>

              <td width="15%" style="text-transform: capitalize;text-align: right"><b>'.$wp_repair_lang['pdf_repair_amount'].'</b></td>

              <td width="10%" style="text-transform: capitalize;text-align: right"><b>'.$tax_indicator.'</b></td>

          </tr>

      </table>';

      if($array_name){
        $array_name_count = count($array_name);
        $array_name_worked = 0;
        foreach($array_name as $i => $a_name){if($a_name != ''){
              $array_name_worked++;
              if($array_name_worked === $array_name_count) {
                    $border_bottom = '1px solid #000';
              } else {
                    $border_bottom = '0.8px solid #929292';
                    
              }

            //correct_repair_ids_order
            $repair_details = $rpCart->correct_repair_ids_order[ $i ] ?? array();
            $is_discounted_price = '';
            if(!empty( $repair_details ) && $repair_details['type'] != 'upsale') {
                  list($is_discounted_price, $discounted_price) = $rpCart->showable_repair_price( $repair_details['id'], $repair_details['type'] );
                  $array_price[$i] = $discounted_price;
            }

        $repair_ex_tax_price = $array_price[$i];

        $repair_ex_tax_price = number_format((float)$repair_ex_tax_price, 2, '.', '');

        if($decimal == 'comma'){

            $repair_ex_tax_price = str_replace('.',',',$repair_ex_tax_price);

        }

        $percent_off_badge_text = '';

        if( $is_discounted_price == '1' ) {

            if( $rpCart->get_combo_discount_percentage() > 0 ) {
                  $percent_off_badge_text = '<span class="percent-off-badge">'.$rpCart->get_combo_discount_percentage().'% '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
            } else {
                  $percent_off_badge_text = '<span class="percent-off-badge">'.rp_handle_currency_position($rpCart->get_combo_discount(), $localization[0]->set_currency, FALSE).' '.$wp_repair_lang['combo_discount_percent_off'].'</span>';
            }

        }

      $html .=

          '<table width="100%" style="margin: 0px;padding: 8px 0px 7px 0px;border-bottom: '.$border_bottom.';">

              <tr>

                  <td width="10%">1 x</td>

                  <td width="65%" style="line-height: 1em; text-transform: capitalize;">'. $brand_name.$model_name.' - '.$a_name.$percent_off_badge_text.'</td>

                  <td width="15%" style="line-height: 1em; text-transform: capitalize;text-align: right">'.rp_handle_currency_position($repair_ex_tax_price, $p_symbol, TRUE).'</td>

                  <td width="10%" style="line-height: 1em; text-transform: capitalize;text-align: right">'.$tax.'%</td>

              </tr>

          </table>';

      } } } 

      if(!empty($combo)) {
        $html .= '<table width="90%" style="text-align:right; margin: 0px;padding: 8px 0px 0px 0px;">

          <tr>

              <td width="85%" style="line-height: 1em; text-transform: capitalize;text-align: right;">'.$wp_repair_lang['discount'].'</td>

              <td style="line-height: 1em; text-transform: capitalize;text-align: right">'.rp_handle_currency_position($combo, $p_symbol, TRUE).'</td>

          </tr>

      </table>

      ';
      }

      if(!empty($coupon_info) && ($coupon_info->coupons_type ?? '') == 'percent') {
            $coupon_td_first = $wp_repair_lang['coupon_discount'] . ' (' . $coupon_info->coupons_amount . '%)';
      } else {
            $coupon_td_first = $wp_repair_lang['coupon_discount'];
      }

      if(!empty($coupon)) {
        $html .= '<table width="90%" style="text-align:right; margin: 0px;padding: 8px 0px 0px 0px;">

          <tr>

              <td width="85%" style="line-height: 1em; text-transform: capitalize;text-align: right;">'.$coupon_td_first.'</td>

              <td style="line-height: 1em; text-transform: capitalize;text-align: right">'.rp_handle_currency_position($coupon, $p_symbol, TRUE).'</td>

          </tr>

      </table>

      ';
      }

      $html .=

      '<table width="90%" style="text-align:right; margin: 0px;padding: 6px 0px 0px 0px;">

          <tr>

              <td width="85%" style="line-height: 1em;text-transform: capitalize;text-align: right;"><b>'.$wp_repair_lang['subtotal'].' '.$wp_repair_lang['excluding_short_form'].' '.$tax_indicator.'</b></td>

              <td style="line-height: 1em; text-transform: capitalize;text-align: right">'.rp_handle_currency_position($ex_tax_price, $p_symbol, TRUE).'</td>

          </tr>  

      </table>

      <table width="90%" style="text-align:right; margin: 0px;padding: 6px 0px 0px 0px;">

          <tr>

              <td width="85%" style="line-height: 1em; text-transform: capitalize;text-align:right;margin-left:auto;float:right">'.$tax.'% '.$tax_indicator.'</td>

              <td style="text-transform: capitalize;text-align: right">'.rp_handle_currency_position($total_tax, $p_symbol, TRUE).'</td>

          </tr>       

      </table>

      <table width="90%" style="text-align:right;">

          <tr>

              <td width="50%"></td><td width="50%" style="line-height: 1em; border-bottom: 1px solid;text-align:right;margin: 0px;padding: 4px 0px 0px 0px;"></td>

          </tr>

      </table>

      <table width="90%" style="text-align:right; margin-bottom: 25px;">

          <tr>

              <td width="85%" style="text-transform: capitalize;text-align: right;margin: 0px;padding: 2px 0px 0px 0px; line-height: 1em;"><b>'.$wp_repair_lang['total'].'</b></td>

              <td style="text-transform: capitalize;text-align: right;margin: 0px;padding: 2px 0px 0px 0px; line-height: 1em;"><b>'.rp_handle_currency_position($final_price, $p_symbol, TRUE).'</b></td>

          </tr>

      </table>

      <table class="p-table" width="100%" style="margin: 10px 0px 10px 0px">

          <tr>

              <td class="p-text">'.$wp_repair_lang['do_not_hesitate_to_contact_us'].'</td>

          </tr>

      </table>

      <table class="p-table" width="100%" style="margin: 10px 0px">

          <tr>

              <td class="p-text">'.$wp_repair_lang['kind_regards'].'</td>

          </tr>

      </table>

      <table class="p-table" width="100%" style="margin: 10px 0px">

          <tr>

              <td class="p-text">'.$c_results[0]->company_name.'</td>

          </tr>

      </table>';

      $html .= '</body> 
      </html> ';

      $options = $pdf->getOptions(); 
          $options->set(array('isRemoteEnabled' => true));
          $pdf->setOptions($options);

      $pdf->loadHtml($html);

      $pdf->setPaper('A4', 'portrait');

      $pdf->render();


      // PDF Footer
      $canvas = $pdf->get_canvas();

      $the_text = $wp_repair_lang['appointment_offer_footer'];

      $font = "Arial";

      $font_size = 7;

      $page_width = $canvas->get_width();

      $page_height = $canvas->get_height();

      $max_width = 0.86 * $page_width;

      $lines = array();

      $line = "";

      $words = explode(" ", $the_text);

      foreach ($words as $word) {

            $test_line = $line . $word . " ";

            $test_width = $canvas->get_text_width($test_line, $font, $font_size);

            if ($test_width > $max_width) {

                  $lines[] = $line;

                  $line = $word . " ";

            } else {

                  $line = $test_line;

            }

      }

      array_push($lines, $line);

      $line_count = count($lines);

      $line_height = $font_size * 1.5;

      $total_height = $line_count * $line_height;

      $start_y = $page_height - 30 - ( $total_height / 2 ) ;


      for($i = 0; $i < $line_count; $i++) {

            $y = $start_y + ($i * $line_height);

            $text_width = $canvas->get_text_width($lines[$i], $font, $font_size);

            $center_position = ($page_width / 2) - ($text_width / 2);

            $canvas->text($center_position, $y, $lines[$i], $font, $font_size, array(0.5,0.5,0.5));
      }
      

      $file = $pdf->output();

      if( defined( 'WP_REPAIR_DEVELOPMENT_MODE' ) ) {

        file_put_contents(WP_REPAIR_PLUGIN_PATH.$file_name, $file);
        echo json_encode(Array('status'=> true, 'message'  => 'Successfully Sent'));
        exit();

      }

      file_put_contents($file_name, $file);

      //$file_name = 'report_'.time();

      // $pdf->load_html($html,'UTF-8');

      // $pdf->render();

      // $pdf->stream($file_name.".pdf");

      $rs_notifications_and_offers = $rpQuery->prefix . "rs_notifications_and_offers";

      $offers = $rpQuery->get_results("SELECT * FROM $rs_notifications_and_offers WHERE noti_type_id_fk ='2'");

      $pdf_offer  = $offers[0]->is_active;

      if($pdf_offer == '1'){

          $to      = $email; 

            $email_translation_key = get_translation_key_by_noti_type_id($offers[0]->noti_type_id_fk);

            $subject = $wp_repair_lang[$email_translation_key.'_email_subject'];

            $message = $wp_repair_lang[$email_translation_key.'_email_template'];

          $message = '<div class="display: block; width: 100%; margin-bottom: 20px;">'.$message.'</div>';

          $link = $finalize_btn_link;

          $headers = array('Content-Type: text/html; charset=UTF-8');

          $mail_attachment = array($file_name);   
          
          include_once WP_REPAIR_PLUGIN_PATH.'html/order_emails.php';

          $rp_theme_color = rp_get_company_theme_color();

          $message = str_replace('$FinalizeButton$', '<br><p><a href="'.$link.'" style="background-color:#'.$rp_theme_color.'; color:white; padding:12px 20px; text-decoration:none; font-size: 1em;border-radius: 5px;margin: 20px 0px;">'.ucfirst(strtolower($wp_repair_lang['finalize_appointment'])).'</a></p><br>', $message);


      $match = rp_find_link_pattern('finalize', $message);

	if( !empty($match) && strpos($message, '[finalize=') !== FALSE ) {

		$matchText = str_replace('[finalize="', '', $match);
            $matchText = str_replace('[finalize=&quot;', '', $matchText);
            $matchText = str_replace('"]', '', $matchText);
            $matchText = str_replace('&quot;]', '', $matchText);

            $finalizeMarkup = '<br><p><a href="'.$link.'" style="background-color:#'.$rp_theme_color.'; color:white; padding:12px 20px; text-decoration:none; font-size: 1em;border-radius: 5px;margin: 20px 0px;">'.$matchText.'</a></p><br>';

            $message = str_replace($match, $finalizeMarkup, $message);

	}



          $message = wp_repair_prepare_order_email($order_id_fk, $subject, $message);

          $subject = wp_repair_prepare_order_email_subject($order_id_fk, $subject);

          if(!defined('WP_REPAIR_REQUIRE_LICENSE')) {

              wp_mail($to, $subject, $message, $headers, $mail_attachment);

              $duplicate_email = wp_repair_admin_duplicate_email('2', rp_get_location_id_of_order($order_id_fk));
              if(!empty($duplicate_email)) {
                    rp_wp_mail_change_sender_info($email, $user_name);
                  wp_mail($duplicate_email, $subject, $message, $headers, $mail_attachment);
              }

          }

          unlink($file_name); 

          return TRUE;

      }

      return FALSE;

}

// where noti_type_id_fk = 5, check if is_active = 1 in rs_notifications_and_offers table

function rp_check_if_notifaction_is_active($noti_type_id_fk = 0){

      if( $noti_type_id_fk === 0 ) {

            return FALSE;

      }

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . "rs_notifications_and_offers";

      $offer = $rpQuery->get_row("SELECT is_active FROM $rs_notifications_and_offers WHERE noti_type_id_fk = '$noti_type_id_fk' LIMIT 1;");

      if($offer->is_active == '1'){

            return TRUE;

      }

      return FALSE;

}

function rp_quick_debug_email( $oid = '', $is_newly_created = FALSE ) {

      add_action('init', function() use ($oid, $is_newly_created) {

            rp_send_appointment_status_update_email( $oid, $is_newly_created, TRUE );

            exit;

      });

}

function rp_send_appointment_status_update_email( $oid = '', $is_newly_created = FALSE, $debugging = FALSE ){

      global $rpQuery;

      global $wp_repair_lang;
  
      include_once WP_REPAIR_PLUGIN_PATH.'html/order_emails.php';

      if( $debugging === TRUE ) {

            if( !defined('WP_REPAIR_DEVELOPMENT_MODE') ) {

                  define('WP_REPAIR_DEVELOPMENT_MODE', TRUE);

            }

      }

      rp_set_order_related_language($oid);
  
      $rs_orders = $rpQuery->prefix . "rs_orders";
  
      $rs_customer = $rpQuery->prefix . "rs_customer";
  
      $rs_user_pdfs = $rpQuery->prefix . "rs_user_pdfs";
  
      $rs_notifications_and_offers = $rpQuery->prefix . "rs_notifications_and_offers";
  
      $order_row = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE `o_id` = '$oid' LIMIT 1;");

      if($is_newly_created === TRUE && (!empty(rp_get_por_repairs_by_order_id( $oid )) || !empty(rp_get_por_repair_attr_by_order_id( $oid ))) && rp_check_if_notifaction_is_active( 5 ) ) {

            $order_row->status_id_fk = 5;

      }

      $_delivery_method = str_replace(' ', '_', strtolower(trim($order_row->delivery_method)));

      if( $order_row->status_id_fk == 3 && $_delivery_method == 'ship_device' ) {
                  
            $order_row->status_id_fk = 6;

      }

      if( $order_row->status_id_fk == 3 && $_delivery_method == 'pickup_service' ) {
                  
            $order_row->status_id_fk = 9;

      }

      if( $order_row->status_id_fk == 3 && $_delivery_method == 'repair_on_location' ) {
            
            $order_row->status_id_fk = 10;

      }

      $order_row->status_id_fk = apply_filters('rp_override_status_while_sending_email', $order_row->status_id_fk, $order_row);

      $status = $order_row->status_id_fk;
  
      $user_pdf_row = $rpQuery->get_row("SELECT * FROM `$rs_user_pdfs` WHERE `o_id_fk` = '$oid' LIMIT 1;");
  
      if( $status == '2' && !empty( $user_pdf_row ) ) {
        
        // Need to send an offer!
        $pdf_result = rp_handle_sending_pdf_offer( $oid );

        rp_unset_order_related_language();

        return $pdf_result;
  
      }
  
      $customer_row = $rpQuery->get_row("SELECT * FROM `$rs_customer` WHERE `customer_id` = '{$order_row->customer_id_fk}' LIMIT 1;");
  
      $user_name     = $customer_row->customer_name;
  
      $email         = $customer_row->customer_email;
  
      $rp_location_id = rp_get_location_id_of_order( $oid );
  
      $offer = $rpQuery->get_row("SELECT * FROM $rs_notifications_and_offers WHERE `noti_type_id_fk` = '{$status}' LIMIT 1;");
  
      $is_active = $offer->is_active;

      $to = $email;

      if( $is_active == '1' ) {

            $email_translation_key = get_translation_key_by_noti_type_id($offer->noti_type_id_fk);

            $subject = $wp_repair_lang[$email_translation_key.'_email_subject'];

            $message = $wp_repair_lang[$email_translation_key.'_email_template'];
  
            $message = wp_repair_prepare_order_email($oid, $subject, $message);

            if( $debugging === TRUE ) {

                  print_r( $message );

                  exit;

            }
  
            $subject = wp_repair_prepare_order_email_subject($oid, $subject);
            
            wp_repair_send_mail($to, $subject, $message);
  
            $duplicate_email = wp_repair_admin_duplicate_email($status, $rp_location_id);
              
            if(!empty($duplicate_email)) {
                  
                  rp_wp_mail_change_sender_info($email, $user_name);
                  
                  wp_repair_send_mail($duplicate_email, $subject, $message);
            
            }

            rp_unset_order_related_language();

            return TRUE;

      }
      
      rp_unset_order_related_language();

      return FALSE;
  
}

function update_appointment_status(){

      rp_verify_csrf_token_for_ajax('appointments');

    global $rpQuery;

    $rs_orders = $rpQuery->prefix . "rs_orders";

    $localization = $rpQuery->prefix . "rs_localization";
      
    $localization_data = $rpQuery->get_results("SELECT * FROM `$localization`");

    $oid           = $_POST['oid'];

    $status        = $_POST['booking_status'];

    $delivery_time = trim( $_POST['delivery_time'] ?? '' );

    $delivery_date = trim( $_POST['delivery_date'] ?? '' );

    $user_name     = $_POST['user_name'];

    $email         = $_POST['email'];


    if( !empty( $delivery_date ) ) {

      $delivery_date = date('d-m-Y', strtotime( $delivery_date ) );

    }

    if( !empty( $delivery_time ) ) {

      if( strpos( $delivery_time, ' — ' ) === FALSE ) {

            if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {

                  $delivery_time = date('H:i', strtotime( $delivery_time ) );

            } else {

                  $delivery_time = date('g:i a', strtotime( $delivery_time ) );

            }

      } else {

            $DateTime = explode(' — ', $delivery_time);

            if( $localization_data[0]->set_time_format == '24 hours (e.g. 15:00)' ) {
                  $newDateTime1 = date('H:i', strtotime($DateTime[0]));
                  $newDateTime2 = date('H:i', strtotime($DateTime[1]));
                  $newDateTime = $newDateTime1 . ' — ' . $newDateTime2;
            } else {
                  $newDateTime1 = date('g:i a', strtotime($DateTime[0]));
                  $newDateTime2 = date('g:i a', strtotime($DateTime[1]));
                  $newDateTime = $newDateTime1 . ' — ' . $newDateTime2;
            }

            $delivery_time = $newDateTime;

      }

    }

    $sql = "UPDATE `$rs_orders` SET status_id_fk = '".rp_escape_sql($status)."'";
    if( !empty( $delivery_date ) ) {
      $sql .= ", delivery_date = '".rp_escape_sql($delivery_date)."'";
    }
    
    if( !empty( $delivery_time ) ) {
      $sql .= ", delivery_time = '".rp_escape_sql($delivery_time)."'";
    }
    $sql .= " WHERE o_id = '".rp_escape_sql($oid)."'";
    
    $rpQuery->query( $sql );

      // $rpQuery->query("UPDATE $rs_orders SET status_id_fk = '".rp_escape_sql($status)."', delivery_time = '".rp_escape_sql($delivery_time)."' , delivery_date = '".rp_escape_sql($delivery_date)."' WHERE o_id = '".rp_escape_sql($oid)."'");

      if( !empty( $oid ?? 0 ) ) {

            $order_delivery_method = rp_find_delivery_method_of_order( $oid );

            if( !empty( $order_delivery_method ) && $order_delivery_method != 'ship device' && $order_delivery_method != 'pdf offer' ) {

                  if( (int) $status !== 4 ) {
                        rp_trigger_outgoing_webhook( 'scheduled_appointment_updated', $oid );
                  } else {
                        rp_trigger_outgoing_webhook( 'scheduled_appointment_cancelled', $oid );
                  }

            } else if( !empty( $order_delivery_method ) && $order_delivery_method == 'ship device' ) {

                  if( (int) $status !== 4 ) {
                        rp_trigger_outgoing_webhook( 'ship_device_appointment_updated', $oid );
                  } else {
                        rp_trigger_outgoing_webhook( 'ship_device_appointment_cancelled', $oid );
                  }

            } else if( !empty( $order_delivery_method ) && $order_delivery_method == 'pdf offer' ) {

                  if( (int) $status !== 4 ) {
                        rp_trigger_outgoing_webhook( 'pdf_offer_updated', $oid );
                  } else {
                        rp_trigger_outgoing_webhook( 'pdf_offer_cancelled', $oid );
                  }

            } else {

                  // Do nothing!

            }

      }

    if( rp_send_appointment_status_update_email( $oid ) === TRUE ) {

      echo json_encode(Array('status'=> true, 'message'  => 'Successfully Sent'));

    } else {

      echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    }

    exit();

}

rp_ajax_for_admin('update_appointment_status','update_appointment_status');


//-------------------------------------------------------------------add location

function add_location(){

      rp_verify_csrf_token_for_ajax('general_info');

    global $rpQuery;

    $company_location    = $rpQuery->prefix . "rs_company_location";

    $company_schedules   = $rpQuery->prefix . "rs_company_schedules";

    $loc_name    = $_POST['loc_name'];

    $loc_address = $_POST['loc_address'];

    $loc_email = $_POST['loc_email'] ?? '';

    if(!empty($loc_email) && filter_var($loc_email, FILTER_VALIDATE_EMAIL) !== FALSE) {

      if( trim($loc_email) !== rp_get_settings_placeholder_email() && trim($loc_email) !== rp_get_settings_company_email() ) {

            // Skip... So It will save the email.

      } else {

            $loc_email = '';

      }

    } else {

      $loc_email = '';

    }

    $loc_city    = $_POST['loc_city'];

    $loc_google_map    = $_POST['loc_google_map'] ?? '';

    $schedule_group_id = 0;

    $rpQuery->query("INSERT INTO $company_location(location_name,location_address,location_city_district,google_map_link,location_email,schedule_group_id) VALUES('".rp_escape_sql($loc_name)."','".rp_escape_sql($loc_address)."','".rp_escape_sql($loc_city)."','".rp_escape_sql($loc_google_map)."','".rp_escape_sql($loc_email)."', '".rp_escape_sql($schedule_group_id)."')");



    $location_id       = $rpQuery->insert_id;

    $cs_day            = $_POST['cs_day'];

    $cs_open_timing    = $_POST['cs_open_timing'];

    $cs_close_timing   = $_POST['cs_close_timing'];

    $cs_breaktime_from = $_POST["cs_breaktime_from"];

    $cs_breaktime_to   = $_POST["cs_breaktime_to"];

    $is_closed         = $_POST["is_closed"];



    foreach($cs_day as $i => $day){

        if($is_closed[$i] == 'true'){

            $closed = '1';

        }else{

            $closed = '0';                

        }



        $rpQuery->query("INSERT INTO $company_schedules(cs_day,cs_open_timing,cs_close_timing,cs_breaktime_from,cs_breaktime_to,is_closed,location_id_fk) VALUES('".rp_escape_sql($day)."','".rp_escape_sql($cs_open_timing[$i])."','".rp_escape_sql($cs_close_timing[$i])."','".rp_escape_sql($cs_breaktime_from[$i])."','".rp_escape_sql($cs_breaktime_to[$i])."','".rp_escape_sql($closed)."','".rp_escape_sql($location_id)."')"); 

    }

}

rp_ajax_for_admin('add_location','add_location');


//-------------------------------------------------------------------update location

function update_locations(){

      rp_verify_csrf_token_for_ajax('general_info');

    global $rpQuery;

    $company_location    = $rpQuery->prefix . "rs_company_location";

    $company_schedules   = $rpQuery->prefix . "rs_company_schedules";

    $loc_id           = $_POST['loc_id'];

    $location_name    = $_POST['location_name'];

    $location_address = $_POST['location_address'];

    $location_city    = $_POST['location_city'];

    $google_maps_link    = $_POST['google_maps_link'];

    $loc_email = $_POST['location_email'] ?? '';

    if(!empty($loc_email) && filter_var($loc_email, FILTER_VALIDATE_EMAIL) !== FALSE) {

      if( trim($loc_email) !== rp_get_settings_placeholder_email() && trim($loc_email) !== rp_get_settings_company_email() ) {

            // Skip... So It will save the email.

      } else {

            $loc_email = '';

      }

    } else {

      $loc_email = '';

    }

    $schedule_group_id = 0;

    $rpQuery->query("UPDATE $company_location SET location_name='".rp_escape_sql($location_name)."',location_address='".rp_escape_sql($location_address)."',location_city_district='".rp_escape_sql($location_city)."',google_map_link='".rp_escape_sql($google_maps_link)."',location_email='".rp_escape_sql($loc_email)."',schedule_group_id='".rp_escape_sql($schedule_group_id)."' WHERE location_id='".rp_escape_sql($loc_id)."'");

    $cs_id             = $_POST['cs_id'];

    $cs_day            = $_POST['cs_day'];

    $cs_open_timing    = $_POST['cs_open_timing'];

    $cs_close_timing   = $_POST['cs_close_timing'];

    $cs_breaktime_from = $_POST["cs_breaktime_from"];

    $cs_breaktime_to   = $_POST["cs_breaktime_to"];

    $is_closed         = $_POST["is_closed"];

    foreach($cs_id as $i => $id){

        if($is_closed[$i] == 'true'){

            $closed = '1';

        }else{

            $closed = '0';                

        }

        $rpQuery->query("UPDATE $company_schedules SET cs_day='".rp_escape_sql($cs_day[$i])."',cs_open_timing='".rp_escape_sql($cs_open_timing[$i])."',cs_close_timing='".rp_escape_sql($cs_close_timing[$i])."',cs_breaktime_from='".rp_escape_sql($cs_breaktime_from[$i])."',cs_breaktime_to='".rp_escape_sql($cs_breaktime_to[$i])."',is_closed='".rp_escape_sql($closed)."',location_id_fk='".rp_escape_sql($loc_id)."' WHERE cs_id='".rp_escape_sql($id)."'");

        //$i++;

        //$abc[] = $closed;

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated'));

    exit();

}

rp_ajax_for_admin('update_locations','update_locations');


//-------------------------------------------------------------------delete location

function delete_location(){

      rp_verify_csrf_token_for_ajax('general_info');

    global $rpQuery;

    $company_location    = $rpQuery->prefix . "rs_company_location";

    $loc_id = $_POST['loc_id'];

    $rpQuery->query("DELETE FROM $company_location WHERE location_id='".rp_escape_sql($loc_id)."'");

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));

}

rp_ajax_for_admin('delete_location','delete_location');


// ---------------------------------------- Update Category & Brand Position
function rp_update_category_and_brand_position() {

      rp_verify_csrf_token_for_ajax('manage_categories');

      global $rpQuery;

      $rs_category    = $rpQuery->prefix . "rs_category";

      $c_position = $_POST['c_position'] ?? array();

      $i = 1; 

      foreach($c_position as $cp){

          $rpQuery->query("UPDATE $rs_category SET c_position='".rp_escape_sql($i)."' WHERE c_id='".rp_escape_sql($cp)."'");

          $i++;

      }

      $rs_brand    = $rpQuery->prefix . "rs_brand";

      $b_position = $_POST['b_position'] ?? array();

      $i = 1; 

      foreach($b_position as $bp){

          $rpQuery->query("UPDATE $rs_brand SET b_position='".rp_escape_sql($i)."' WHERE b_id='".rp_escape_sql($bp)."'");

          $i++;

      }

      echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

      exit();
}

rp_ajax_for_admin('update_category_and_brand_position','rp_update_category_and_brand_position');

//-------------------------------------------------------------------update category position

function update_category_position(){

    global $rpQuery;

    $rs_category    = $rpQuery->prefix . "rs_category";

    $c_position = $_POST['c_position'];

    $i = 1; 

    foreach($c_position as $cp){

        $rpQuery->query("UPDATE $rs_category SET c_position='".rp_escape_sql($i)."' WHERE c_id='".rp_escape_sql($cp)."'");

        $i++;

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    exit();

}

rp_ajax_for_admin('update_category_position','update_category_position');


//-------------------------------------------------------------------update brand position

function update_brand_position(){

    global $rpQuery;

    $rs_brand    = $rpQuery->prefix . "rs_brand";

    $b_position = $_POST['b_position'];

    $i = 1; 

    foreach($b_position as $bp){

        $rpQuery->query("UPDATE $rs_brand SET b_position='".rp_escape_sql($i)."' WHERE b_id='".rp_escape_sql($bp)."'");

        $i++;

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    exit();

}

rp_ajax_for_admin('update_brand_position','update_brand_position');


//-------------------------------------------------------------------update model position

function update_model_position(){

    global $rpQuery;

    $rs_model    = $rpQuery->prefix . "rs_model";

    $rs_model_color    = $rpQuery->prefix . "rs_model_color";

    $m_position = $_POST['m_position'];

    $mc_position = $_POST['mc_position'];

    $i = 1; 

    foreach($m_position as $mp){

        $rpQuery->query("UPDATE $rs_model SET m_position='".rp_escape_sql($i)."' WHERE m_id='".rp_escape_sql($mp)."'");

        $i++;

    }

    $i = 1; 

    foreach($mc_position as $mcp){

        $rpQuery->query("UPDATE $rs_model_color SET mc_position='".rp_escape_sql($i)."' WHERE mc_id='".rp_escape_sql($mcp)."'");

        $i++;

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    exit();

}

rp_ajax_for_admin('update_model_position','update_model_position');


//-------------------------------------------------------------------update new model position

function get_model_position_first_found_index($brand_id = '', $positionByBrand = array()) {
      foreach ($positionByBrand[$brand_id] as $index => $value) {
            if(in_array($value->m_id, $_POST['m_position'])) {
                  return $index;
            }
      }
      return 0;
}

function update_new_model_position(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery;

    $front_end_position = array();

    $rs_model    = $rpQuery->prefix . "rs_model";

    if(isset($_POST['m_position']) && !empty($_POST['m_position'])) {
      $m_position = implode(',', $_POST['m_position']);

      $all_model_info = $rpQuery->get_results("SELECT m_id, m_position, brand_id_fk FROM $rs_model  WHERE m_id IN(".rp_escape_sql($m_position).");");

      if(!empty($all_model_info)) {
            $brandIDs = array();
            $getBrandByModel = array();
            foreach ($all_model_info as $key => $value) {
                  if(array_search($value->brand_id_fk, $brandIDs) === FALSE) {
                        $brandIDs[] = $value->brand_id_fk;
                  }
                  $getBrandByModel[$value->m_id] = $value->brand_id_fk;
            }
            $brandIDs = implode(',', $brandIDs);

            $all_model_info = $rpQuery->get_results("SELECT m_id, m_position, brand_id_fk FROM $rs_model  WHERE brand_id_fk IN(".rp_escape_sql($brandIDs).")  ORDER BY brand_id_fk ASC, m_position ASC, release_date DESC;");

            $usableMPosition = array();

            foreach ($_POST['m_position'] as $mposid) {
                  if(!isset($usableMPosition[$getBrandByModel[$mposid]])) {
                        $usableMPosition[$getBrandByModel[$mposid]] = array();
                  }
                  $usableMPosition[$getBrandByModel[$mposid]][] = $mposid;
            }

            if(!empty($all_model_info)) {
                  $positionByBrand = array();
                  foreach ($all_model_info as $key => $value) {
                        if(!isset($positionByBrand[$value->brand_id_fk])) {
                              $positionByBrand[$value->brand_id_fk] = array();
                        }
                        $positionByBrand[$value->brand_id_fk][] = $value;
                  }
                  foreach($positionByBrand as $brand => $models) {
                        $all_zero = TRUE;
                        foreach( $models as $modelRow ) {
                              if( $modelRow->m_position != 0 && $modelRow->m_position >= 1 ) {
                                    $all_zero = FALSE;
                                    break;
                              }
                        }
                        if( $all_zero === TRUE ) {
                              $xyz = 1;
                              foreach( $models as $modelKey => $modelRow ) {
                                    $modelRow->m_position = $xyz;
                                    // update model position
                                    $rpQuery->query("UPDATE $rs_model SET m_position='".rp_escape_sql($xyz)."' WHERE m_id='".rp_escape_sql($modelRow->m_id)."'");
                                    $xyz++;
                                    $models[$modelKey] = $modelRow;
                              }
                        }
                        $positionByBrand[$brand] = $models;
                  }
                  foreach ($positionByBrand as $brand => $models) {
                        $prepare_for_db = array();
                        $isStart = FALSE;
                        if(in_array($positionByBrand[$brand][0]->m_id, $_POST['m_position'])) {
                              $isStart = TRUE;
                        }
                        if($isStart === FALSE) {
                              $firstIndex = get_model_position_first_found_index($brand, $positionByBrand);

                              $firstIndex = $firstIndex + 1;
                              
                              foreach ($usableMPosition[$brand] as $_m_id) {
                                    $prepare_for_db[$_m_id] = $firstIndex;
                                    $firstIndex++;
                              }
                        } else {
                              $i = 1;
                              foreach ($usableMPosition[$brand] as $_m_id) {
                                    $prepare_for_db[$_m_id] = $i;
                                    $i++;
                              }
                        }
                        foreach ($prepare_for_db as $update_m_id => $position_to_set) {
                              $rpQuery->query("UPDATE $rs_model SET m_position='".rp_escape_sql($position_to_set)."' WHERE m_id='".rp_escape_sql($update_m_id)."'");
                              $front_end_position[$update_m_id] = $position_to_set;
                        }
                  }

            }
      }
    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated', 'positions' => $front_end_position));

    exit();

}

rp_ajax_for_admin('update_new_model_position','update_new_model_position');


//-------------------------------------------------------------------update repair

function update_repair(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery; $repairs = array(); $r_attrs = array();$s= array();

    $rs_repair    = $rpQuery->prefix . "rs_repair";

    $rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

    $rs_model    = $rpQuery->prefix . "rs_model";


    $planning = $rpQuery->prefix . "rs_planning_and_discount";
    $planning_result = $rpQuery->get_results("SELECT * FROM $planning");

    $default_repair_length = 30;

    if(!empty($planning_result)) {
      $lengthD = $planning_result[0]->booking_slots_length;
      if(!empty(trim($lengthD))) {
            $lengthD = trim(explode(' ', trim($lengthD))[0]);
            if(!empty($lengthD)) {
                  $default_repair_length = (float) $lengthD;
            }
      }
    }


    $mid    = $_POST['mid'];     
    $rid    = $_POST['rid'] ?? array();     
    $title  = $_POST['title'];     
    $mins   = $_POST['mins'];
    $duration_measure = $_POST['duration_measure'];     
    $info   = $_POST['info']; 
    $badge = $_POST['badge'];
    $rprice = $_POST['rprice'];  
    $rmargin = $_POST['rmargin'] ?? array();
    $rfree = $_POST['rfree'] ?? array();  
    $status = $_POST['status'] ?? array(); 
    $img_link = $_POST['img_link'];
    $skipped_index = array();

    // Update/Create Repairs of Model
    foreach($rid as $i => $r){
      if(preg_match('/,/',$rprice[$i])) {
          $price = str_replace(',','.',$rprice[$i]);
      }else{
          $price = $rprice[$i];                   
      }

      if(empty(trim($price))) {
            $price = 0;
      }

      $marginPrice = FALSE;

      if( !empty( $rmargin ) ) {
            if(preg_match('/,/',$rmargin[$i])) {
                  $marginPrice = str_replace(',','.',$rmargin[$i]);
            }else{
                  $marginPrice = $rmargin[$i];                   
            }
            if(empty(trim($marginPrice))) {
                  $marginPrice = 0;
            }
      }

      if( array_search( ($duration_measure[$i] ?? '' ) , array('minutes', 'hours', 'days') ) === FALSE ) {
            $duration_measure[$i] = 'minutes';
      }

      $is_free = $rfree[$i] ?? 0;

      if(empty(trim($mins[$i]))) {
            $mins[$i] = $default_repair_length;
      }
      if(trim(strtoupper($r)) === 'NEW') {
            // Create
            if(!empty(trim($title[$i]))) {
                  $r_uq_id      = uniqid('repair_');
                  if( $marginPrice !== FALSE ) {
                        $r_margin = ', r_margin';
                        $r_margin_value = ", '".rp_escape_sql($marginPrice)."'";
                  } else {
                        $r_margin = '';
                        $r_margin_value = '';
                  }
                  $rpQuery->query("INSERT INTO $rs_repair (r_name, r_duration, r_discription, r_badge, r_price, m_id_fk, r_uq_id, r_img, is_free, r_duration_measure $r_margin) VALUES ('".rp_escape_sql($title[$i])."', '".rp_escape_sql($mins[$i])."', '".rp_escape_sql($info[$i])."', '".rp_escape_sql($badge[$i])."', '".rp_escape_sql($price)."', '".rp_escape_sql($mid)."', '".rp_escape_sql($r_uq_id)."', '".rp_escape_sql($img_link[$i])."', '".rp_escape_sql($is_free)."', '".rp_escape_sql($duration_measure[$i])."' $r_margin_value);");
                  $rid[$i] = $rpQuery->insert_id;
            } else {
                  $skipped_index[] = $i;
            }
      } else {
            // Update
            if( $marginPrice !== FALSE ) {
                  $r_margin = ', r_margin = "'.rp_escape_sql($marginPrice).'"';
            } else {
                  $r_margin = '';
            }
            $rpQuery->query("UPDATE $rs_repair SET r_duration='".rp_escape_sql($mins[$i])."',r_price='".rp_escape_sql($price)."',r_img='".rp_escape_sql($img_link[$i])."',is_free='".rp_escape_sql($is_free)."',r_duration_measure='".rp_escape_sql($duration_measure[$i])."' $r_margin WHERE r_id='".rp_escape_sql($r)."' AND m_id_fk ='".rp_escape_sql($mid)."'");
      }
    }
    // Update/Create Repair Attrs of Model   
    $raid = $_POST['raid'] ?? array();
      foreach ($raid as $i => $arr) {
            if(!in_array($i, $skipped_index)) {
                  $a_id   = $_POST['raid'][$i]; 
                  $a_name   = $_POST['raname'][$i]; 
                  $a_desc   = $_POST['radesc'][$i]; 
                  $a_badge   = $_POST['rabadge'][$i]; 
                  $a_duration   = $_POST['raduration'][$i]; 
                  $a_duration_measure   = $_POST['radurationmeasure'][$i]; 
                  $a_price   = $_POST['raprice'][$i]; 
                  $a_margin = $_POST['ramargin'] ?? array();
                  if( !empty( $a_margin ) ) {
                        $a_margin = $_POST['ramargin'][$i] ?? array();
                  }
                  $a_is_active = $_POST['raactive'][$i] ?? array();
                  $skipped_a_index = array();
                  foreach($a_id as $x => $a){
                        if( empty( trim( $a_duration[$x] ?? '' ) ) ) {
                              $a_duration[$x] = $default_repair_length;
                        }
                        if(preg_match('/,/',$a_price[$x])) {
                              $a_price[$x] = str_replace(',','.',$a_price[$x]);
                        }
                        if(empty(trim($a_price[$x]))) {
                              $a_price[$x] = 0;
                        }
                        $attrMarginPrice = FALSE;
                        if( !empty( $a_margin ) ) {
                              if(preg_match('/,/',$a_margin[$x])) {
                                    $attrMarginPrice = str_replace(',','.',$a_margin[$x]);
                              } else {
                                    $attrMarginPrice = $a_margin[$x];
                              }
                              if(empty(trim($a_margin[$x]))) {
                                   $attrMarginPrice = 0;
                              }
                        }
                        if(strtoupper($a_id[$x]) === 'NEW') {
                              if(!empty(trim($a_name[$x]))) {
                                    if( $attrMarginPrice !== FALSE ) {
                                          $_a_margin = ', `a_margin`';
                                          $_a_margin_value = ", '".rp_escape_sql($attrMarginPrice)."'";
                                    } else {
                                          $_a_margin = '';
                                          $_a_margin_value = '';
                                    }
                                    $a_uq_id  = uniqid('rattr_');
                                    $rpQuery->query("INSERT INTO $rs_repair_attr (`a_name`, `a_desc`, `a_badge`, `a_duration`, `a_duration_measure`, `a_price`, `a_uq_id`, `r_id_fk` $_a_margin) VALUES('".rp_escape_sql($a_name[$x])."', '".rp_escape_sql($a_desc[$x])."', '".rp_escape_sql($a_badge[$x])."', '".rp_escape_sql($a_duration[$x])."', '".rp_escape_sql($a_duration_measure[$x])."', '".rp_escape_sql($a_price[$x])."', '".rp_escape_sql($a_uq_id)."', '".rp_escape_sql($rid[$i])."' $_a_margin_value)");
                                    $a_id[$x] = $rpQuery->insert_id;
                              } else {
                                    $skipped_a_index[] = $x;
                              }
                        } else {
                              if( $attrMarginPrice !== FALSE ) {
                                    $_a_margin = ', a_margin = "'.rp_escape_sql($attrMarginPrice).'"';
                              } else {
                                    $_a_margin = '';
                              }
                              $rpQuery->query("UPDATE $rs_repair_attr SET a_price='".rp_escape_sql($a_price[$x])."', a_duration='".rp_escape_sql($a_duration[$x])."', a_duration_measure='".rp_escape_sql($a_duration_measure[$x])."' $_a_margin WHERE a_id='".rp_escape_sql($a_id[$x])."'");
                        }
                  }

                  foreach($a_id as $x => $a){

                        if(!in_array($x, $skipped_a_index)) {

                              if( !empty( $a_is_active[ $x ] ) ) {

                                    $_is_active = '1';

                              } else {

                                    $_is_active = '0';

                              }

                              $rpQuery->query("UPDATE $rs_repair_attr SET is_active='".rp_escape_sql($_is_active)."' WHERE a_id='".rp_escape_sql($a_id[$x])."'");

                        }

                  }

                  if(isset($_POST['sub_sorting_reset']) && !empty($_POST['sub_sorting_reset']) && isset($_POST['sub_sorting_reset'][$i])) {
                        $sub_sorting_has_changed = $_POST['sub_sorting_reset'][$i];
                        if(!empty($sub_sorting_has_changed) && $sub_sorting_has_changed !== 'false' && $sub_sorting_has_changed !== false && $sub_sorting_has_changed !== '0') {

                              $a_i = 1; 

                              foreach($a_id as $x => $a){

                                    if(!in_array($x, $skipped_a_index)) {

                                          $rpQuery->query("UPDATE $rs_repair_attr SET ra_position='".rp_escape_sql($a_i)."' WHERE a_id='".rp_escape_sql($a_id[$x])."'");

                                          $a_i++;

                                    }

                              }
                        }
                  }
            }
      }     
    // Update Repair Position
      if(isset($_POST['main_sorting_has_changed']) && !empty($_POST['main_sorting_has_changed']) && $_POST['main_sorting_has_changed'] !== 'false' && $_POST['main_sorting_has_changed'] !== '0' && $_POST['main_sorting_has_changed'] !== false) {
            $r_i = 1; 

            foreach($rid as $i => $repairId){

                if(!in_array($i, $skipped_index)) {
                  $rpQuery->query("UPDATE $rs_repair SET r_position='".rp_escape_sql($r_i)."' WHERE r_id='".rp_escape_sql($repairId)."'");

                  $r_i++;
                }

            }
      }
    // Update Repair Attr Position
    // Update Repair Statuses
      $rpQuery->query("UPDATE $rs_repair SET is_active= '0' WHERE m_id_fk ='".rp_escape_sql($mid)."'");
      foreach($status as $i => $s){
          if(!in_array($i, $skipped_index)) {
            if(!empty($s)) {
              $rpQuery->query("UPDATE $rs_repair SET is_active='1' WHERE r_id='".rp_escape_sql($rid[$i])."' AND m_id_fk ='".rp_escape_sql($mid)."'");
            }
          }
      }
      $default_listing_range = (int) ($_POST['default_listing_range'] ?? 6);

      $default_listing_range = rp_round_to_even_number( $default_listing_range );
      
      rp_update_option( 'rp_default_listing_index_model_' . $mid, $default_listing_range );
    // Return Result Array

      $table_rows = rp_get_model_related_repairs_for_admin_page( $mid );

      echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'table_rows' => $table_rows));

      exit();

}

rp_ajax_for_admin('update_repair','update_repair');

function rp_get_model_related_repairs_for_admin_page( $mid = '' ) {

      if( empty( $mid ) ) {

            return array();

      }

      global $rpQuery;

      $rs_repair    = $rpQuery->prefix . "rs_repair";

      $rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";
      $default_listing_range = $_POST['default_listing_range'] ?? NULL;
      $device_type = $_POST['device_type'] ?? 0;
      $default_listing_range = $default_listing_range ? $default_listing_range : rp_get_option( 'rp_default_listing_index_model_' . $mid, FALSE);
      if( $default_listing_range === FALSE ){
		$default_listing_range = rp_get_option( 'rp_default_listing_index_'. $device_type, 6 );
	}
      $default_listing_range = (int) $default_listing_range;
      $default_listing_range = rp_round_to_even_number( $default_listing_range );
      $table_rows = array();
      $c_repair_all = $rpQuery->get_results("SELECT * FROM $rs_repair WHERE `m_id_fk` = '".rp_escape_sql($mid)."' ORDER BY `r_position` ASC;");
      $c_repair_all = rp_fix_default_repair_sorting($c_repair_all, $mid);
      if(!empty($c_repair_all)) {
            $last_index = count($c_repair_all) - 1;
            if( $default_listing_range > ($last_index + 1) ) {
                  $default_listing_range = $last_index + 1;
            }
            if( 0 == $default_listing_range && $last_index !== -1 ){
                  $seprator_tr = array();
                  $seprator_tr['listing_range'] = $default_listing_range;
                  $table_rows[] = $seprator_tr;
            }
            foreach ($c_repair_all as $i => $c_repair) {
                  $tr = array();
                  $tr['is_active'] = $c_repair->is_active;
                  $tr['is_free'] = $c_repair->is_free;
                  $tr['r_duration_measure'] = ( !empty( $c_repair->r_duration_measure ) ? $c_repair->r_duration_measure : '' );
                  $tr['m_id'] = $mid;
                  $tr['r_id'] = $c_repair->r_id;
                  $tr['r_img'] = $c_repair->r_img;
                  $tr['r_uq_id'] = $c_repair->r_uq_id;
                  $tr['repair_title'] = rp_get_static_translations_of_repair('repair_name', $c_repair->r_uq_id, $c_repair->m_id_fk);
                  $tr['repair_min'] = $c_repair->r_duration;
                  $tr['repair_info'] = rp_get_static_translations_of_repair('repair_desc', $c_repair->r_uq_id, $c_repair->m_id_fk);
                  $tr['repair_badge'] = rp_get_static_translations_of_repair('repair_badge', $c_repair->r_uq_id, $c_repair->m_id_fk);
                  $tr['repair_price'] = $c_repair->r_price;
                  $tr['is_fully_unchanged'] = (rp_is_fully_unchanged_default_repair_individually($c_repair->r_id, $mid) === TRUE ? '1' : '0');
                  $tr = rp_sanitize_output($tr);
                  $tr['attrs'] = array();
                  $repair_attrs = $rpQuery->get_results("SELECT * FROM $rs_repair_attr WHERE `r_id_fk` = '".rp_escape_sql($c_repair->r_id)."' ORDER BY `ra_position` ASC");
                  $repair_attrs = rp_fix_default_repair_attr_sorting($repair_attrs, $mid);
                  if(!empty($repair_attrs)) {
                        foreach ($repair_attrs as $x => $c_attr) {
                              $attr = array();
                              $attr['a_id'] = $c_attr->a_id;
                              $attr['a_name'] = rp_get_static_translations_of_attr('attr_name', $c_attr->a_uq_id, $c_attr->r_id_fk);
                              $attr['attr_min'] = $c_attr->a_duration;
                              $attr['a_duration_measure'] = ( !empty( $c_attr->a_duration_measure ) ? $c_attr->a_duration_measure : '' );
                              $attr['a_desc'] = rp_get_static_translations_of_attr('attr_desc', $c_attr->a_uq_id, $c_attr->r_id_fk);
                              $attr['a_badge'] = rp_get_static_translations_of_attr('attr_badge', $c_attr->a_uq_id, $c_attr->r_id_fk);
                              $attr['a_price'] = $c_attr->a_price;
                              $attr['is_active'] = $c_attr->is_active;
                              $attr['a_uq_id'] = $c_attr->a_uq_id;
                              $attr = rp_sanitize_output($attr);
                              $attr = apply_filters('rp_before_returning_individual_repair_attr', $attr, $c_attr);
                              $tr['attrs'][] = $attr;
                        }
                        $tr['repair_price'] = rp_get_lowest_repair_attribute_price( $c_repair->r_id );
                  }

                  $tr = apply_filters('rp_before_returning_individual_repair_tr', $tr, $c_repair);
                  
                  $table_rows[] = $tr;

                  if( $i+1 == $default_listing_range ){
                        $seprator_tr = array();
                        $seprator_tr['listing_range'] = $default_listing_range;
                        $table_rows[] = $seprator_tr;
                  }
            }
      }

      return $table_rows;

}


//-------------------------------------------------------------------update default repair

function update_unchanged_repair_img($dr_row_old = NULL) {

      global $rpQuery;

      $rs_repair    = $rpQuery->prefix . "rs_repair";

      $rs_default_repair    = $rpQuery->prefix . "rs_default_repair";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE dr_id = '".rp_escape_sql($dr_row_old->dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $is_changed = FALSE;

      if(!empty($new_row)) {

            $columns_check = array('img');

            foreach ($columns_check as $col_check) {
                  
                  $the_col = 'dr_'.$col_check;

                  if(trim($new_row[0]->{$the_col}) !== trim($dr_row_old->{$the_col})) {

                        $is_changed = TRUE;

                  }

            }

      }

      if($is_changed === TRUE) {

            $m_ids = rp_get_all_m_ids_by_category( $dr_category );

            if(!empty($m_ids)) {

                  $m_ids = join(',', $m_ids);

                  $unchanged = $rpQuery->get_results("SELECT r_id FROM $rs_repair WHERE r_uq_id = '".rp_escape_sql($dr_row_old->dr_uq_id)."' AND r_img LIKE '".rp_escape_sql($dr_row_old->dr_img)."' AND m_id_fk IN (".rp_escape_sql($m_ids).")");

                  if(!empty($unchanged)) {

                        $new_row = $new_row[0];

                        $allIds = array();

                        foreach ($unchanged as $key => $row) {
                        
                              $allIds[] = $row->r_id;

                        }

                        $all_ids_str = '(' . join(', ', $allIds) . ')';

                        $updated_on  = date('Y-m-d H:i:s');

                        $rpQuery->query("UPDATE $rs_repair SET r_img = '".rp_escape_sql($new_row->dr_img)."', updated_on = '".rp_escape_sql($updated_on)."' WHERE r_id IN $all_ids_str");

                  }

            }

      }

}

function update_unchanged_repair_attr_name($da_row_old = NULL) {

      global $rpQuery;

      $rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

      $rs_default_repair_attr    = $rpQuery->prefix . "rs_default_repair_attr";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr WHERE da_id = '".rp_escape_sql($da_row_old->da_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $is_changed = FALSE;

      if(!empty($new_row)) {

            $columns_check = array('name');

            foreach ($columns_check as $col_check) {
                  
                  $the_col = 'da_'.$col_check;

                  if(trim($new_row[0]->{$the_col}) !== trim($da_row_old->{$the_col})) {

                        $is_changed = TRUE;

                  }

            }

      }

      if($is_changed === TRUE) {

            $r_ids = rp_get_all_repair_ids_related_to_category( $dr_category );

            if(!empty($r_ids)) {

                  $r_ids = join(',', $r_ids);

                  $unchanged = $rpQuery->get_results("SELECT a_id FROM $rs_repair_attr WHERE a_uq_id = '".rp_escape_sql($da_row_old->da_uq_id)."' AND a_name LIKE '".rp_escape_sql($da_row_old->da_name)."' AND r_id_fk IN (".rp_escape_sql($r_ids).")");

                  if(!empty($unchanged)) {

                        $new_row = $new_row[0];

                        $allIds = array();

                        foreach ($unchanged as $key => $row) {
                        
                              $allIds[] = $row->a_id;

                        }

                        $all_ids_str = '(' . join(', ', $allIds) . ')';

                        $updated_on  = date('Y-m-d H:i:s');

                        $rpQuery->query("UPDATE $rs_repair_attr SET a_name = '".rp_escape_sql($new_row->da_name)."', updated_on = '".rp_escape_sql($updated_on)."' WHERE a_id IN $all_ids_str");

                  }

            }

      }

}

function update_unchanged_repair_attr_COLUMNS($da_row_old = NULL) {

      global $rpQuery;

      $rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

      $rs_default_repair_attr    = $rpQuery->prefix . "rs_default_repair_attr";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr WHERE da_id = '".rp_escape_sql($da_row_old->da_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $is_changed = FALSE;

      if(!empty($new_row)) {

            $columns_check = array('is_active', 'duration', 'duration_measure');

            foreach ($columns_check as $col_check) {

                  if( $col_check === 'is_active' ) {

                        $the_col = $col_check;

                  } else {

                        $the_col = 'da_'.$col_check;

                  }

                  if(trim($new_row[0]->{$the_col}) !== trim($da_row_old->{$the_col})) {

                        $is_changed = TRUE;

                  }

            }

      }

      if($is_changed === TRUE) {

            $r_ids = rp_get_all_repair_ids_related_to_category( $dr_category );

            if(!empty($r_ids)) {

                  $r_ids = join(',', $r_ids);

                  $unchanged = $rpQuery->get_results("SELECT a_id FROM $rs_repair_attr WHERE a_uq_id = '".rp_escape_sql($da_row_old->da_uq_id)."' AND a_duration = '".rp_escape_sql($da_row_old->da_duration)."' AND a_duration_measure = '".rp_escape_sql($da_row_old->da_duration_measure)."' AND a_name LIKE '".rp_escape_sql($da_row_old->da_name)."' AND is_active = '".rp_escape_sql($da_row_old->is_active)."' AND r_id_fk IN (".rp_escape_sql($r_ids).")");

                  if(!empty($unchanged)) {

                        $new_row = $new_row[0];

                        $allIds = array();

                        foreach ($unchanged as $key => $row) {
                        
                              $allIds[] = $row->a_id;

                        }

                        $all_ids_str = '(' . join(', ', $allIds) . ')';

                        $updated_on  = date('Y-m-d H:i:s');

                        $rpQuery->query("UPDATE $rs_repair_attr SET is_active = '".rp_escape_sql($new_row->is_active)."', a_duration = '".rp_escape_sql($new_row->da_duration)."', a_duration_measure = '".rp_escape_sql($new_row->da_duration_measure)."', updated_on = '".rp_escape_sql($updated_on)."' WHERE a_id IN $all_ids_str");

                  }

            }

      }

}

function update_unchanged_repair_attr_price($da_row_old = NULL, $old_rows = null, $create_attr_later = array()) {

      global $rpQuery;

      $rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

      $rs_default_repair_attr    = $rpQuery->prefix . "rs_default_repair_attr";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr WHERE da_id = '".rp_escape_sql($da_row_old->da_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $dr_row_old = null;
      foreach( $old_rows as $oldRow ) {

            if( $new_row[0]->dr_id_fk === $oldRow->dr_id ) {

                  $dr_row_old = $oldRow;

            }

      }

      $is_changed = FALSE;

      if(!empty($new_row)) {

            $columns_check = array('price');

            if( isset( $_POST['rmargin'] ) && !empty( ($_POST['rmargin'] ?? array()) ) ) {

                  $columns_check[] = 'margin';
                  
            }

            foreach ($columns_check as $col_check) {
                  
                  $the_col = 'da_'.$col_check;

                  if(trim($new_row[0]->{$the_col}) !== trim($da_row_old->{$the_col})) {

                        $is_changed = TRUE;

                  }

            }

      }

      if($is_changed === TRUE) {

            $r_ids = rp_get_all_repair_ids_related_to_category($dr_category);

            if(!empty($r_ids)) {

                  $r_ids = join(',', $r_ids);

                  if( in_array( 'margin', $columns_check ) ) {

                        $a_margin_like = " AND a_margin LIKE '".rp_escape_sql($da_row_old->da_margin)."' ";
                        $a_margin_set = " a_margin = '".rp_escape_sql($new_row[0]->da_margin)."', ";

                  } else {

                        $a_margin_like = '';
                        $a_margin_set = '';

                  }

                  $unchanged = $rpQuery->get_results("SELECT a_id, r_id_fk FROM $rs_repair_attr WHERE a_uq_id = '".rp_escape_sql($da_row_old->da_uq_id)."' $a_margin_like AND a_price LIKE '".rp_escape_sql($da_row_old->da_price)."' AND r_id_fk IN (".rp_escape_sql($r_ids).")");

                  if(!empty($unchanged)) {

                        $new_row = $new_row[0];

                        $allIds = array();

                        foreach ($unchanged as $key => $row) {

                              if( rp_is_unchanged_default_repair_individually( $row->r_id_fk, $dr_category, $dr_row_old, $create_attr_later ) ) {

                                    $allIds[] = $row->a_id;

                              }

                        }

                        if( !empty( $allIds ) ) {

                              $all_ids_str = '(' . join(', ', $allIds) . ')';

                              $updated_on  = date('Y-m-d H:i:s');

                              $rpQuery->query("UPDATE $rs_repair_attr SET $a_margin_set a_price = '".rp_escape_sql($new_row->da_price)."', updated_on = '".rp_escape_sql($updated_on)."' WHERE a_id IN $all_ids_str");

                        }

                  }

            }

      }

}

function update_unchanged_repair_price($dr_row_old = NULL, $create_attr_later = array()) {

      global $rpQuery;

      $rs_repair    = $rpQuery->prefix . "rs_repair";

      $rs_default_repair    = $rpQuery->prefix . "rs_default_repair";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE dr_id = '".rp_escape_sql($dr_row_old->dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $is_changed = FALSE;

      if(!empty($new_row)) {

            $columns_check = array('price');

            if( isset( $_POST['rmargin'] ) && !empty( $_POST['rmargin'] ) ) {

                  $columns_check[] = 'margin';

            }

            foreach ($columns_check as $col_check) {
                  
                  $the_col = 'dr_'.$col_check;

                  if(trim($new_row[0]->{$the_col}) !== trim($dr_row_old->{$the_col})) {

                        $is_changed = TRUE;

                  }

            }

      }

      if($is_changed === TRUE) {

            $m_ids = rp_get_all_m_ids_by_category( $dr_category );

            if(!empty($m_ids)) {

                  $m_ids = join(',', $m_ids);

                  if( in_array( 'margin', $columns_check ) ) {

                        $margin_like = " AND r_margin LIKE '".rp_escape_sql($dr_row_old->dr_margin)."' ";
                        $margin_set = " r_margin = '".rp_escape_sql($new_row[0]->dr_margin)."', ";

                  } else {

                        $margin_like = '';
                        $margin_set = '';

                  }

                  $unchanged = $rpQuery->get_results("SELECT r_id FROM $rs_repair WHERE r_uq_id = '".rp_escape_sql($dr_row_old->dr_uq_id)."' AND r_price LIKE '".rp_escape_sql($dr_row_old->dr_price)."' $margin_like AND m_id_fk IN (".rp_escape_sql($m_ids).")");

                  if(!empty($unchanged)) {

                        $new_row = $new_row[0];

                        $allIds = array();

                        foreach ($unchanged as $key => $row) {

                              if( rp_is_unchanged_default_repair_individually($row->r_id, $dr_category, $dr_row_old, $create_attr_later) ) {

                                    $allIds[] = $row->r_id;

                              }

                        }

                        if( !empty( $allIds ) ) {

                              $all_ids_str = '(' . join(', ', $allIds) . ')';

                              $updated_on  = date('Y-m-d H:i:s');

                              $rpQuery->query("UPDATE $rs_repair SET $margin_set r_price = '".rp_escape_sql($new_row->dr_price)."', updated_on = '".rp_escape_sql($updated_on)."' WHERE r_id IN $all_ids_str");

                        }

                  }

            }

      }

}

function rp_update_matching_repair_name_in_row($key = '', $uqid = '', $old_dr_row = null, $new_value = '') {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $what_info = 'r_name';
      $what_info_2 = 'dr_name';

      if($key === 'repair_name') {
            $what_info = 'r_name';
            $what_info_2 = 'dr_name';
      }

      if($key === 'repair_desc') {
            $what_info = 'r_discription';
            $what_info_2 = 'dr_discription';
      }

      if($key === 'repair_badge') {
            $what_info = 'r_badge';
            $what_info_2 = 'dr_badge';
      }

      $rpQuery->query("UPDATE $rs_repair SET `$what_info` = '".rp_escape_sql($new_value)."' WHERE r_uq_id = '".rp_escape_sql($uqid)."' AND `$what_info` = '".rp_escape_sql($old_dr_row->{$what_info_2})."'");

}


function rp_update_matching_attr_name_in_row($key = '', $uqid = '', $old_dr_row = null, $new_value = '') {

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      $what_info = 'a_name';
      $what_info_2 = 'da_name';

      if($key === 'attr_name') {
            $what_info = 'a_name';
            $what_info_2 = 'da_name';
      }

      if($key === 'attr_desc') {
            $what_info = 'a_desc';
            $what_info_2 = 'da_desc';
      }

      if($key === 'attr_badge') {
            $what_info = 'a_badge';
            $what_info_2 = 'da_badge';
      }

      $rpQuery->query("UPDATE `$rs_repair_attr` SET `$what_info` = '".rp_escape_sql($new_value)."' WHERE a_uq_id = '".rp_escape_sql($uqid)."' AND `$what_info` = '".rp_escape_sql($old_dr_row->{$what_info_2})."'");

}

function rp_create_matching_repair_translations($key = '', $uqid = '', $language = '', $new_value = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $rs_r_translations = $rpQuery->prefix . "rs_r_translations";

      $rpQuery->query("DELETE FROM `$rs_r_translations` WHERE `key` = '".rp_escape_sql($key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($language)."';");

      // select * from rs_repair where m_id_fk IN $m_ids and r_uq_id = $uqid

      $m_id_fks = $rpQuery->get_results("SELECT m_id_fk FROM `$rs_repair` WHERE `r_uq_id` = '".rp_escape_sql($uqid)."'");

      if( !empty( $m_id_fks ) ) {

            $rows_to_insert = array();

            foreach ($m_id_fks as $_row) {

                  $m_id = $_row->m_id_fk;

                  $rows_to_insert[] = array(
                              
                        'key' => $key,

                        'uq_id' => $uqid,

                        'language' => $language,

                        'value' => $new_value,

                        'm_id_fk' => $m_id

                  );

            }

            rp_insert_multiple_rows('rs_r_translations', $rows_to_insert, FALSE);

      }

}

function rp_create_matching_attr_translations($key = '', $uqid = '', $language = '', $new_value = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

      $rpQuery->query("DELETE FROM `$rs_rattr_translations` WHERE `key` = '".rp_escape_sql($key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($language)."';");

      // select * from rs_repair_attr where r_id_fk in $r_ids and a_uq_id = $uqid

      $r_id_fks = $rpQuery->get_results("SELECT r_id_fk FROM `$rs_repair_attr` WHERE `a_uq_id` = '".rp_escape_sql($uqid)."'");

      if( !empty( $r_id_fks ) ) {

            $rows_to_insert = array();

            foreach($r_id_fks as $_row) {

                  $r_id_fk = $_row->r_id_fk;

                  $rows_to_insert[] = array(

                        'key' => $key,

                        'uq_id' => $uqid,

                        'language' => $language,

                        'value' => $new_value,

                        'r_id_fk' => $r_id_fk

                  );

            }

            rp_insert_multiple_rows('rs_rattr_translations', $rows_to_insert, FALSE);

      }

}


function rp_update_matching_repair_translations($key = '', $uqid = '', $language = '', $old_value = '', $new_value = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_r_translations = $rpQuery->prefix . "rs_r_translations";

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $rpQuery->query("UPDATE `$rs_r_translations` SET `value` = '".rp_escape_sql($new_value)."' WHERE `key` = '".rp_escape_sql($key)."' AND `language` = '".rp_escape_sql($language)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `value` = '".rp_escape_sql($old_value)."';");


      // select * from rs_repair where m_id_fk IN $m_ids and r_uq_id = $uqid

      $m_id_fks = $rpQuery->get_results("SELECT m_id_fk FROM `$rs_repair` WHERE `r_uq_id` = '".rp_escape_sql($uqid)."'");

      if( !empty( $m_id_fks ) ) {

            $rows_to_insert = array();

            $all_m_ids = array();

            foreach ($m_id_fks as $_row) {

                  $all_m_ids[] = $_row->m_id_fk;

            }

            $all_m_ids_str = join(',', $all_m_ids);

            // find existing m_ids 
            $existing_mids = $rpQuery->get_results("SELECT m_id_fk FROM `$rs_r_translations` WHERE `key` = '".rp_escape_sql($key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($language)."' AND `m_id_fk` IN ({$all_m_ids_str});");

            if( !empty( $existing_mids ) ) {

                  foreach($existing_mids as $e_mid_row) {

                        $e_mid = $e_mid_row->m_id_fk;

                        // remove $e_mid from $all_m_ids

                        $all_m_ids = array_diff($all_m_ids, array($e_mid));

                  }

            }

            $all_m_ids = array_values($all_m_ids);

            foreach ($all_m_ids as $m_id_fk) {

                  $rows_to_insert[] = array(

                        'key' => $key,

                        'uq_id' => $uqid,

                        'language' => $language,

                        'value' => $new_value,

                        'm_id_fk' => $m_id_fk

                  );

            }

            rp_insert_multiple_rows('rs_r_translations', $rows_to_insert, FALSE);

      }

}

function rp_create_matching_repair_translations_if_not_already($key = '', $uqid = '', $language = '', $new_value = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_r_translations = $rpQuery->prefix . "rs_r_translations";

      $rs_repair = $rpQuery->prefix . "rs_repair";

      // select * from rs_repair where m_id_fk IN $m_ids and r_uq_id = $uqid

      $m_id_fks = $rpQuery->get_results("SELECT m_id_fk FROM `$rs_repair` WHERE `r_uq_id` = '".rp_escape_sql($uqid)."'");

      if( !empty( $m_id_fks ) ) {

            $rows_to_insert = array();

            $all_m_ids = array();

            foreach ($m_id_fks as $_row) {

                  $all_m_ids[] = $_row->m_id_fk;

            }

            $all_m_ids_str = join(',', $all_m_ids);

            // find existing m_ids 
            $existing_mids = $rpQuery->get_results("SELECT m_id_fk FROM `$rs_r_translations` WHERE `key` = '".rp_escape_sql($key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($language)."' AND `m_id_fk` IN ({$all_m_ids_str});");

            if( !empty( $existing_mids ) ) {

                  foreach($existing_mids as $e_mid_row) {

                        $e_mid = $e_mid_row->m_id_fk;

                        // remove $e_mid from $all_m_ids

                        $all_m_ids = array_diff($all_m_ids, array($e_mid));

                  }

            }

            $all_m_ids = array_values($all_m_ids);

            foreach ($all_m_ids as $m_id_fk) {

                  $rows_to_insert[] = array(

                        'key' => $key,

                        'uq_id' => $uqid,

                        'language' => $language,

                        'value' => $new_value,

                        'm_id_fk' => $m_id_fk

                  );

            }

            rp_insert_multiple_rows('rs_r_translations', $rows_to_insert, FALSE);

      }

}

function rp_update_matching_attr_translations($key = '', $uqid = '', $language = '', $old_value = '', $new_value = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      $rpQuery->query("UPDATE `$rs_rattr_translations` SET `value` = '".rp_escape_sql($new_value)."' WHERE `key` = '".rp_escape_sql($key)."' AND `language` = '".rp_escape_sql($language)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `value` = '".rp_escape_sql($old_value)."';");

      // select * from rs_repair_attr where r_id_fk in $r_ids and a_uq_id = $uqid

      $r_id_fks = $rpQuery->get_results("SELECT r_id_fk FROM `$rs_repair_attr` WHERE `a_uq_id` = '".rp_escape_sql($uqid)."'");

      if( !empty( $r_id_fks ) ) {

            $rows_to_insert = array();

            $all_r_ids = array();

            foreach ($r_id_fks as $_row) {

                  $all_r_ids[] = $_row->r_id_fk;

            }

            $all_r_ids_str = join(',', $all_r_ids);

            $existing_rids = $rpQuery->get_results("SELECT r_id_fk FROM `$rs_rattr_translations` WHERE `key` = '".rp_escape_sql($key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($language)."' AND `r_id_fk` IN ({$all_r_ids_str});");

            if( !empty( $existing_rids ) ) {

                  foreach($existing_rids as $e_rid_row) {

                        $e_rid = $e_rid_row->r_id_fk;

                        // remove $e_rid from $all_r_ids

                        $all_r_ids = array_diff($all_r_ids, array($e_rid));

                  }

            }

            $all_r_ids = array_values($all_r_ids);

            foreach($all_r_ids as $r_id_fk) {

                  $rows_to_insert[] = array(

                        'key' => $key,

                        'uq_id' => $uqid,

                        'language' => $language,

                        'value' => $new_value,

                        'r_id_fk' => $r_id_fk

                  );

            }

            rp_insert_multiple_rows('rs_rattr_translations', $rows_to_insert, FALSE);

      }

}

function rp_create_matching_attr_translations_if_not_already($key = '', $uqid = '', $language = '', $new_value = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      // select * from rs_repair_attr where r_id_fk in $r_ids and a_uq_id = $uqid

      $r_id_fks = $rpQuery->get_results("SELECT r_id_fk FROM `$rs_repair_attr` WHERE `a_uq_id` = '".rp_escape_sql($uqid)."'");

      if( !empty( $r_id_fks ) ) {

            $rows_to_insert = array();

            $all_r_ids = array();

            foreach ($r_id_fks as $_row) {

                  $all_r_ids[] = $_row->r_id_fk;

            }

            $all_r_ids_str = join(',', $all_r_ids);

            $existing_rids = $rpQuery->get_results("SELECT r_id_fk FROM `$rs_rattr_translations` WHERE `key` = '".rp_escape_sql($key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($language)."' AND `r_id_fk` IN ({$all_r_ids_str});");

            if( !empty( $existing_rids ) ) {

                  foreach($existing_rids as $e_rid_row) {

                        $e_rid = $e_rid_row->r_id_fk;

                        // remove $e_rid from $all_r_ids

                        $all_r_ids = array_diff($all_r_ids, array($e_rid));

                  }

            }

            $all_r_ids = array_values($all_r_ids);

            foreach($all_r_ids as $r_id_fk) {

                  $rows_to_insert[] = array(

                        'key' => $key,

                        'uq_id' => $uqid,

                        'language' => $language,

                        'value' => $new_value,

                        'r_id_fk' => $r_id_fk

                  );

            }

            rp_insert_multiple_rows('rs_rattr_translations', $rows_to_insert, FALSE);

      }

}

function update_unchanged_repair_COLUMNS($dr_row_old = NULL) {

      global $rpQuery;

      $rs_repair    = $rpQuery->prefix . "rs_repair";

      $rs_default_repair    = $rpQuery->prefix . "rs_default_repair";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE dr_id = '".rp_escape_sql($dr_row_old->dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $is_changed = FALSE;

      if(!empty($new_row)) {

            $columns_check = array('duration', 'duration_measure', 'is_active', 'is_free');

            foreach ($columns_check as $col_check) {
                  
                  if($col_check === 'is_active' || $col_check === 'is_free') {
                        $the_col = $col_check;
                  } else {
                        $the_col = 'dr_'.$col_check;
                  }

                  if(trim($new_row[0]->{$the_col}) !== trim($dr_row_old->{$the_col})) {

                        $is_changed = TRUE;

                  }

            }

      }

      if($is_changed === TRUE) {

            $m_ids = rp_get_all_m_ids_by_category( $dr_category );

            if(!empty($m_ids)) {

                  $m_ids = join(',', $m_ids);

                  $unchanged = $rpQuery->get_results("SELECT r_id FROM $rs_repair WHERE r_uq_id = '".rp_escape_sql($dr_row_old->dr_uq_id)."' AND r_duration = '".rp_escape_sql($dr_row_old->dr_duration)."' AND is_active = '".rp_escape_sql($dr_row_old->is_active)."' AND is_free = '".rp_escape_sql($dr_row_old->is_free)."' AND m_id_fk IN (".rp_escape_sql($m_ids).")");

                  if(!empty($unchanged)) {

                        $new_row = $new_row[0];

                        $allIds = array();

                        foreach ($unchanged as $key => $row) {
                        
                              $allIds[] = $row->r_id;

                        }

                        $all_ids_str = '(' . join(', ', $allIds) . ')';

                        $updated_on  = date('Y-m-d H:i:s');

                        $rpQuery->query("UPDATE $rs_repair SET r_duration = '".rp_escape_sql($new_row->dr_duration)."', r_duration_measure = '".rp_escape_sql($new_row->dr_duration_measure)."', is_active = '".rp_escape_sql($new_row->is_active)."', is_free = '".rp_escape_sql($new_row->is_free)."', updated_on='".rp_escape_sql($updated_on)."' WHERE r_id IN $all_ids_str");

                  }

            }

      }

}

function create_default_repair_attr_for_all_models($da_id = NULL, $create_attr_later = array(), $old_rows = array()) {

      global $rpQuery;

      $rs_repair    = $rpQuery->prefix . "rs_repair";

      $rs_default_repair    = $rpQuery->prefix . "rs_default_repair";

      $rs_repair_attr    = $rpQuery->prefix . "rs_repair_attr";

      $rs_default_repair_attr    = $rpQuery->prefix . "rs_default_repair_attr";

      $dr_category = $_POST['dr_category'] ?? '';

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr WHERE da_id = '".rp_escape_sql($da_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      if(empty($new_row)) {
            return;
      }

      $dr_id = $new_row[0]->dr_id_fk;

      $parent_repair = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE dr_id = '".rp_escape_sql($dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      if(empty($parent_repair)) {
            return;
      }

      $dr_row_old = $parent_repair[0];

      $dr_row_old_2 = null;

      foreach( $old_rows as $oldRow ) {

            if( $new_row[0]->dr_id_fk === $oldRow->dr_id ) {

                  $dr_row_old_2 = $oldRow;

            }

      }

      $m_ids = rp_get_all_m_ids_by_category( $dr_category );

      if(!empty($m_ids)) {

            $m_ids = join(',', $m_ids);

            $unchanged = $rpQuery->get_results("SELECT r_id FROM $rs_repair WHERE r_name = '".rp_escape_sql($dr_row_old->dr_name)."' AND r_discription = '".rp_escape_sql($dr_row_old->dr_discription)."' AND r_badge = '".rp_escape_sql($dr_row_old->dr_badge)."' AND r_duration = '".rp_escape_sql($dr_row_old->dr_duration)."' AND is_active = '".rp_escape_sql($dr_row_old->is_active)."' AND m_id_fk IN (".rp_escape_sql($m_ids).")");

            if(!empty($unchanged)) {

                  $new_row = $new_row[0];

                  foreach ($unchanged as $key => $row) {

                        if( rp_is_unchanged_default_repair_individually( $row->r_id, $dr_category, $dr_row_old, $create_attr_later ) || rp_is_unchanged_default_repair_individually( $row->r_id, $dr_category, $dr_row_old_2, $create_attr_later ) ) {

                              $r_id_fk = $row->r_id;

                              if( isset( $_POST['rmargin'] ) && !empty( $_POST['rmargin'] ) ) {
                                    $r_margin = ', `a_margin`';
                                    $r_margin_value = ", '".rp_escape_sql($new_row->da_margin)."'";
                              } else {
                                    $r_margin = '';
                                    $r_margin_value = '';
                              }

                              $rpQuery->query("INSERT INTO $rs_repair_attr (`a_name`, `a_duration`, `a_duration_measure`, `a_desc`, `a_badge`, `a_price`, `r_id_fk`, `updated_on`, `is_active`, `a_uq_id` $r_margin) VALUES ('".rp_escape_sql($new_row->da_name)."', '".rp_escape_sql($new_row->da_duration)."', '".rp_escape_sql($new_row->da_duration_measure)."', '".rp_escape_sql($new_row->da_desc)."', '".rp_escape_sql($new_row->da_badge)."', '".rp_escape_sql($new_row->da_price)."', '".rp_escape_sql($r_id_fk)."', '".rp_escape_sql($new_row->updated_on)."', '".rp_escape_sql($new_row->is_active)."', '".rp_escape_sql($new_row->da_uq_id)."' $r_margin_value);");

                        }

                  }

            }

      }

}

function create_default_repair_for_all_models($dr_id = NULL) {

      global $rpQuery;

      $rs_model = $rpQuery->prefix . "rs_model";

      $rs_repair    = $rpQuery->prefix . "rs_repair";

      $rs_default_repair    = $rpQuery->prefix . "rs_default_repair";

      $dr_category = $_POST['dr_category'] ?? 0;

      $new_row = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE dr_id = '".rp_escape_sql($dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

      $m_ids = rp_get_all_m_ids_by_category( $dr_category );

      if(!empty($m_ids)) {

            $m_ids = join(',', $m_ids);

            $models = $rpQuery->get_results("SELECT m_id FROM $rs_model WHERE m_id IN (".rp_escape_sql($m_ids).");");

            if(!empty($models)) {

                  $new_row = $new_row[0];

                  foreach ($models as $key => $row) {

                        $m_id_fk = $row->m_id;

                        if( isset( $_POST['rmargin'] ) && !empty( $_POST['rmargin'] ) ) {

                              $r_margin = ', `r_margin`';
                              $r_margin_value = ", '".rp_escape_sql($new_row->dr_margin)."'";

                        } else {
                              
                              $r_margin = '';
                              $r_margin_value = '';

                        }

                        $rpQuery->query("INSERT INTO $rs_repair (`r_name`, `r_discription`, `r_badge`, `r_duration`, `is_active`, `is_free`, `r_price`, `r_img`, `updated_on`, `r_date_time`, `r_uq_id`, `m_id_fk`, `r_duration_measure` $r_margin) VALUES ('".rp_escape_sql($new_row->dr_name)."', '".rp_escape_sql($new_row->dr_discription)."', '".rp_escape_sql($new_row->dr_badge)."', '".rp_escape_sql($new_row->dr_duration)."', '".rp_escape_sql($new_row->is_active)."', '".rp_escape_sql($new_row->is_free)."', '".rp_escape_sql($new_row->dr_price)."', '".rp_escape_sql($new_row->dr_img)."', '".rp_escape_sql($new_row->updated_on)."', '".rp_escape_sql($new_row->dr_date_time)."', '".rp_escape_sql($new_row->dr_uq_id)."', '".rp_escape_sql($m_id_fk)."', '".rp_escape_sql($new_row->dr_duration_measure)."' $r_margin_value);");

                  }

            }

      }

}

if(!function_exists('rp_get_total_counts_of_used_repairs')) {

      function rp_get_total_counts_of_used_repairs($dr_uq_id = null, $dr_category = 0) {
            if(empty($dr_uq_id)) {
                  return 0;
            }

            global $rpQuery;

            $rs_repair = $rpQuery->prefix . "rs_repair";

            $rs_model = $rpQuery->prefix . "rs_model";

            $total = $rpQuery->get_var("SELECT COUNT(m_id) FROM `$rs_model` WHERE m_id IN (SELECT m_id_fk FROM `$rs_repair` WHERE `r_uq_id` LIKE '".rp_escape_sql($dr_uq_id)."') AND device_type = '".rp_escape_sql($dr_category)."'");

            if(empty($total)) {
                  return 0;
            } else {
                  return $total;
            }
      }

}

function update_default_repair(){

      rp_verify_csrf_token_for_ajax('default_repairs');

    global $rpQuery; $repairs = array(); $r_attrs = array();$s= array();

    $rs_repair    = $rpQuery->prefix . "rs_default_repair";

    $rs_repair_attr    = $rpQuery->prefix . "rs_default_repair_attr";

    $rs_model    = $rpQuery->prefix . "rs_model";


    $planning = $rpQuery->prefix . "rs_planning_and_discount";
    $planning_result = $rpQuery->get_results("SELECT * FROM $planning");

    $default_repair_length = 30;

    if(!empty($planning_result)) {
      $lengthD = $planning_result[0]->booking_slots_length;
      if(!empty(trim($lengthD))) {
            $lengthD = trim(explode(' ', trim($lengthD))[0]);
            if(!empty($lengthD)) {
                  $default_repair_length = (float) $lengthD;
            }
      }
    }


    $mid    = $_POST['mid'];     
    $rid    = $_POST['rid'] ?? array();     
    $title  = $_POST['title'];     
    $mins   = $_POST['mins'];     
    $duration_measure   = $_POST['duration_measure'];     
    $info   = $_POST['info'];
    $badge   = $_POST['badge'];     
    $rprice = $_POST['rprice'];
    $rmargin = $_POST['rmargin'] ?? array();
    $rfree = $_POST['rfree'] ?? array();
    $status = $_POST['status'] ?? array();  
    $attr_status = $_POST['attr_status'] ?? array();
    $img_link = $_POST['img_link'];

    $dr_category = $_POST['dr_category'] ?? 0;

    $create_attr_later = array();

    $skipped_index = array();

    $old_rows = array();

    $old_rows_attr = array();

    $newly_created_repair = array();

    $rpQuery->query('START TRANSACTION');

    // Update/Create Repairs of Model
    foreach($rid as $i => $r){
      if(preg_match('/,/',$rprice[$i])) {
          $price = str_replace(',','.',$rprice[$i]);
      }else{
          $price = $rprice[$i];                   
      }

      if(empty(trim($price))) {
            $price = 0;
      }

      $marginPrice = FALSE;

      if( !empty( $rmargin ) ) {
            if(preg_match('/,/',$rmargin[$i])) {
                  $marginPrice = str_replace(',','.',$rmargin[$i]);
            }else{
                  $marginPrice = $rmargin[$i];                   
            }
            if(empty(trim($marginPrice))) {
                  $marginPrice = 0;
            }
      }

      if( array_search( ($duration_measure[$i] ?? '' ) , array('minutes', 'hours', 'days') ) === FALSE ) {
            $duration_measure[$i] = 'minutes';
      }

      $is_free = $rfree[$i] ?? 0;

      if(empty(trim($mins[$i]))) {
            $mins[$i] = $default_repair_length;
      }
      if(trim(strtoupper($r)) === 'NEW') {
            // Create
            if(!empty(trim($title[$i]))) {
                  $r_uq_id      = uniqid('repair_');
                  $r_date_time  = date('Y-m-d H:i:s');
                  $updated_on  = date('Y-m-d H:i:s');
                  if( $marginPrice !== FALSE ) {
                        $dr_margin = ', dr_margin';
                        $dr_margin_value = ", '".rp_escape_sql($marginPrice)."'";
                  } else {
                        $dr_margin = '';
                        $dr_margin_value = '';
                  }
                  $rpQuery->query("INSERT INTO $rs_repair (dr_name, dr_duration, dr_discription, dr_badge, dr_price, dr_uq_id, dr_img, dr_date_time, updated_on, is_free, dr_category, dr_duration_measure $dr_margin) VALUES ('".rp_escape_sql($title[$i])."', '".rp_escape_sql($mins[$i])."', '".rp_escape_sql($info[$i])."', '".rp_escape_sql($badge[$i])."', '".rp_escape_sql($price)."', '".rp_escape_sql($r_uq_id)."', '".rp_escape_sql($img_link[$i])."', '".rp_escape_sql($r_date_time)."', '".rp_escape_sql($updated_on)."', '".rp_escape_sql($is_free)."', '".rp_escape_sql($dr_category)."', '".rp_escape_sql($duration_measure[$i])."' $dr_margin_value);");
                  $rid[$i] = $rpQuery->insert_id;
                  $newly_created_repair[] = $rpQuery->insert_id;
            } else {
                  $skipped_index[] = $i;
            }
      } else {
            // Update

            $old_row = $rpQuery->get_results("SELECT * FROM $rs_repair WHERE dr_id='".rp_escape_sql($r)."' AND dr_category='".rp_escape_sql($dr_category)."';");

            $old_rows[$r] = $old_row[0];

            if( $marginPrice !== FALSE ) {
                  $dr_margin = ', dr_margin = "'.rp_escape_sql($marginPrice).'"';
            } else {
                  $dr_margin = '';
            }

            $updated_on  = date('Y-m-d H:i:s');

            $rpQuery->query("UPDATE $rs_repair SET dr_duration='".rp_escape_sql($mins[$i])."',dr_price='".rp_escape_sql($price)."',dr_img='".rp_escape_sql($img_link[$i])."',updated_on='".rp_escape_sql($updated_on)."',is_free='".rp_escape_sql($is_free)."',dr_duration_measure='".rp_escape_sql($duration_measure[$i])."' $dr_margin WHERE dr_id='".rp_escape_sql($r)."' AND dr_category='".rp_escape_sql($dr_category)."';");

      }
    }
    // Update/Create Repair Attrs of Model   
    $raid = $_POST['raid'] ?? array();
      foreach ($raid as $i => $arr) {
            if(!in_array($i, $skipped_index)) {
                  $a_id   = $_POST['raid'][$i]; 
                  $a_name   = $_POST['raname'][$i];
                  $a_duration   = $_POST['raduration'][$i]; 
                  $a_duration_measure   = $_POST['radurationmeasure'][$i]; 
                  $a_desc = $_POST['radesc'][$i];
                  $a_badge = $_POST['rabadge'][$i];
                  $a_price   = $_POST['raprice'][$i]; 
                  $a_margin = $_POST['ramargin'] ?? array();
                  if( !empty( $a_margin ) ) {
                        $a_margin = $_POST['ramargin'][$i] ?? array();
                  }
                  $a_is_active = $_POST['raactive'][$i] ?? array();
                  $skipped_a_index = array();
                  foreach($a_id as $x => $a){
                        if( empty( trim( $a_duration[$x] ?? '' ) ) ) {
                              $a_duration[$x] = $default_repair_length;
                        }
                        if(preg_match('/,/',$a_price[$x])) {
                              $a_price[$x] = str_replace(',','.',$a_price[$x]);
                        }
                        if(empty(trim($a_price[$x]))) {
                              $a_price[$x] = 0;
                        }
                        $attrMarginPrice = FALSE;
                        if( !empty( $a_margin ) ) {
                              if(preg_match('/,/',$a_margin[$x])) {
                                    $attrMarginPrice = str_replace(',','.',$a_margin[$x]);
                              } else {
                                    $attrMarginPrice = $a_margin[$x];
                              }
                              if(empty(trim($a_margin[$x]))) {
                                   $attrMarginPrice = 0;
                              }
                        }
                        if(strtoupper($a_id[$x]) === 'NEW') {
                              if(!empty(trim($a_name[$x]))) {
                                    if( $attrMarginPrice !== FALSE ) {
                                          $da_margin = ', `da_margin`';
                                          $da_margin_value = ", '".rp_escape_sql($attrMarginPrice)."'";
                                    } else {
                                          $da_margin = '';
                                          $da_margin_value = '';
                                    }
                                    $a_uq_id  = uniqid('rattr_');
                                    $rpQuery->query("INSERT INTO $rs_repair_attr (`da_name`, `da_duration`, `da_duration_measure`, `da_desc`, `da_badge`, `da_price`, `da_uq_id`, `dr_id_fk`, `dr_category` $da_margin) VALUES('".rp_escape_sql($a_name[$x])."', '".rp_escape_sql($a_duration[$x])."', '".rp_escape_sql($a_duration_measure[$x])."', '".rp_escape_sql($a_desc[$x])."', '".rp_escape_sql($a_badge[$x])."', '".rp_escape_sql($a_price[$x])."', '".rp_escape_sql($a_uq_id)."', '".rp_escape_sql($rid[$i])."', '".rp_escape_sql($dr_category)."' $da_margin_value)");
                                    $a_id[$x] = $rpQuery->insert_id;
                                    $create_attr_later[] = $a_id[$x];
                              } else {
                                    $skipped_a_index[] = $x;
                              }
                        } else {

                              $old_row_attr = $rpQuery->get_results("SELECT * FROM $rs_repair_attr WHERE da_id='".rp_escape_sql($a_id[$x])."' AND dr_category='".rp_escape_sql($dr_category)."';");

                              $old_rows_attr[$a_id[$x]] = $old_row_attr[0];

                              if( $attrMarginPrice !== FALSE ) {
                                    $da_margin = ', da_margin = "'.rp_escape_sql($attrMarginPrice).'"';
                              } else {
                                    $da_margin = '';
                              }

                              $rpQuery->query("UPDATE $rs_repair_attr SET da_price='".rp_escape_sql($a_price[$x])."', da_duration='".rp_escape_sql($a_duration[$x])."', da_duration_measure='".rp_escape_sql($a_duration_measure[$x])."' $da_margin WHERE da_id='".rp_escape_sql($a_id[$x])."' AND dr_category='".rp_escape_sql($dr_category)."'");
                        }
                  }
                  $a_i = 1; 

                  foreach($a_id as $x => $a){

                        if(!in_array($x, $skipped_a_index)) {

                              $rpQuery->query("UPDATE $rs_repair_attr SET da_position='".rp_escape_sql($a_i)."' WHERE da_id='".rp_escape_sql($a_id[$x])."' AND dr_category='".rp_escape_sql($dr_category)."'");

                              $a_i++;

                        }

                  }

                  foreach($a_id as $x => $a){

                        if(!in_array($x, $skipped_a_index)) {

                              if( !empty($a_is_active[$x]) ) {
                                    $_is_active = '1';
                              } else {
                                    $_is_active = '0';
                              }

                              $rpQuery->query("UPDATE $rs_repair_attr SET is_active='".rp_escape_sql($_is_active)."' WHERE da_id='".rp_escape_sql($a_id[$x])."' AND dr_category='".rp_escape_sql($dr_category)."'");

                              $a_i++;

                        }

                  }
            }
      }     
    // Update Repair Position
      $r_i = 1; 

      foreach($rid as $i => $repairId){

          if(!in_array($i, $skipped_index)) {
            $rpQuery->query("UPDATE $rs_repair SET dr_position='".rp_escape_sql($r_i)."' WHERE dr_id='".rp_escape_sql($repairId)."' AND dr_category='".rp_escape_sql($dr_category)."'");

            $r_i++;
          }

      }
    // Update Repair Attr Position
    // Update Repair Statuses
      $rpQuery->query("UPDATE $rs_repair SET is_active= '0' WHERE dr_category='".rp_escape_sql($dr_category)."'");
      foreach($status as $i => $s){
          if(!in_array($i, $skipped_index)) {
            if(!empty($s)) {
              $rpQuery->query("UPDATE $rs_repair SET is_active='1' WHERE dr_id='".rp_escape_sql($rid[$i])."' AND dr_category='".rp_escape_sql($dr_category)."'");
            }
          }
      }

      // Update/Create Repairs of Model
      foreach($rid as $i => $r){
            if(!in_array($r, $skipped_index)) {

                  if(!in_array($r, $newly_created_repair)) {

                        // Update

                        $old_row = $old_rows[$r];

                        update_unchanged_repair_COLUMNS($old_row);

                        update_unchanged_repair_price($old_row, $create_attr_later);

                        update_unchanged_repair_img($old_row);

                  } else {

                        create_default_repair_for_all_models($r);

                  }
            }
      }

      foreach ($create_attr_later as $attr_id) {
            create_default_repair_attr_for_all_models($attr_id, $create_attr_later, $old_rows);
      }

      $raid = $_POST['raid'] ?? array();
      foreach ($raid as $i => $arr) {
            if(!in_array($i, $skipped_index)) {
                  $a_id   = $_POST['raid'][$i]; 
                  $a_name   = $_POST['raname'][$i];
                  $skipped_a_index = array();
                  foreach($a_id as $x => $a){
                        if(strtoupper($a_id[$x]) === 'NEW') {
                              if(!empty(trim($a_name[$x]))) {
                                    // New Create Here...

                                    // Already Created in Above Similar Loop..

                              } else {

                                    // Skip This...

                              }
                        } else {

                              // Old Update Here

                              $old_row = $old_rows_attr[$a_id[$x]];

                              update_unchanged_repair_attr_price($old_row, $old_rows, $create_attr_later);

                              update_unchanged_repair_attr_COLUMNS($old_row);

                        }
                  }
            }
      }     


    // Return Result Array

      $rpQuery->query('COMMIT');
      $default_listing_range = (int) $_POST['default_listing_range'] ?? 6;
      $default_listing_range = rp_round_to_even_number($default_listing_range);
      $table_rows = array();
      $c_repair_all = $rpQuery->get_results("SELECT * FROM $rs_repair WHERE dr_category = '".rp_escape_sql($dr_category)."' ORDER BY `dr_position` ASC;");
      if(!empty($c_repair_all)) {
            $last_index = count($c_repair_all) - 1;
            if( $default_listing_range > ($last_index + 1) ) {
                  $default_listing_range = $last_index + 1;
            }
            if( 0 == $default_listing_range && $last_index !== -1 ){
                  $seprator_tr = array();
                  $seprator_tr['listing_range'] = 0;
                  $table_rows[] = $seprator_tr;
            }
            foreach ($c_repair_all as $i => $c_repair) {
                  $tr = array();
                  $tr['is_free'] = $c_repair->is_free;
                  $tr['is_active'] = $c_repair->is_active;
                  $tr['dr_duration_measure'] = ( !empty( $c_repair->dr_duration_measure ) ? $c_repair->dr_duration_measure : '' );
                  $tr['m_id'] = $mid;
                  $tr['total_used'] = rp_get_total_counts_of_used_repairs($c_repair->dr_uq_id, $dr_category);
                  $tr['r_id'] = $c_repair->dr_id;
                  $tr['r_img'] = $c_repair->dr_img;
                  $tr['dr_uq_id'] = $c_repair->dr_uq_id;
                  $tr['repair_title'] = rp_get_static_translations_of_drepair('repair_name', $c_repair->dr_uq_id);
                  $tr['repair_min'] = $c_repair->dr_duration;
                  $tr['repair_info'] = rp_get_static_translations_of_drepair('repair_desc', $c_repair->dr_uq_id);
                  $tr['repair_badge'] = rp_get_static_translations_of_drepair('repair_badge', $c_repair->dr_uq_id);
                  $tr['repair_price'] = $c_repair->dr_price;
                  $tr = rp_sanitize_output($tr);
                  $tr['attrs'] = array();
                  $repair_attrs = $rpQuery->get_results("SELECT * FROM $rs_repair_attr WHERE `dr_id_fk` = '".rp_escape_sql($c_repair->dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' ORDER BY `da_position` ASC");
                  if(!empty($repair_attrs)) {
                        foreach ($repair_attrs as $x => $c_attr) {
                              $attr = array();
                              $attr['a_id'] = $c_attr->da_id;
                              $attr['a_name'] = rp_get_static_translations_of_dattr('attr_name', $c_attr->da_uq_id);
                              $attr['attr_min'] = $c_attr->da_duration;
                              $attr['da_duration_measure'] = ( !empty( $c_attr->da_duration_measure ) ? $c_attr->da_duration_measure : '' );
                              $attr['a_desc'] = rp_get_static_translations_of_dattr('attr_desc', $c_attr->da_uq_id);
                              $attr['a_badge'] = rp_get_static_translations_of_dattr('attr_badge', $c_attr->da_uq_id);
                              $attr['a_price'] = $c_attr->da_price;
                              $attr['is_active'] = $c_attr->is_active;
                              $attr['da_uq_id'] = $c_attr->da_uq_id;
                              $attr = rp_sanitize_output($attr);
                              $attr = apply_filters('rp_before_returning_default_repair_attr', $attr, $c_attr);
                              $tr['attrs'][] = $attr;
                        }
                        $tr['repair_price'] = rp_get_lowest_default_repair_attribute_price( $c_repair->dr_id );
                  }
                  $tr = apply_filters('rp_before_returning_default_repair_tr', $tr, $c_repair);
                  $table_rows[] = $tr;
                  if( $i+1 == $default_listing_range ){
                        $seprator_tr = array();
                        $seprator_tr['listing_range'] = $default_listing_range;
                        $table_rows[] = $seprator_tr;
                  }
            }
      }

      // get previous position
      $previous_position = rp_get_option( 'rp_default_listing_index_' . ((int)$dr_category), FALSE );

      if( $previous_position !== FALSE && $previous_position !== $default_listing_range ) {

            rp_update_default_listing_index_individually( (int) $previous_position, (int) $default_listing_range, (int) $dr_category );

      }

      rp_update_option( 'rp_default_listing_index_' . ((int)$dr_category), $default_listing_range );

      echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'table_rows' => $table_rows));

      exit();

}

rp_ajax_for_admin('update_default_repair','update_default_repair');

function rp_update_default_listing_index_individually($previous_position = 0, $new_position = 0, $dr_category = 0) {

      if( $dr_category === 0 || $previous_position === $new_position ) {
            // dr category is not set or previous position and new position is same
            return;
      }

      // steps to update default listing index individually
      // 1. get all model ids(m_id) from rs_model table where device_type = $dr_category
      // 2. get all options from wp_options table where option_name like 'rp_default_listing_index_model_%' AND option_value = $previous_position
      // 3. loop through all options and replace rp_default_listing_index_model_ from option_name to get remaining part of option_name (m_id)
      // 4. check if m_id is in array of model ids
      // 5. if yes then update that option with new position

      global $rpQuery;

      $rs_model = $rpQuery->prefix . "rs_model";

      $options_table = $rpQuery->prefix . "options";

      $model_ids = $rpQuery->get_results("SELECT m_id FROM $rs_model WHERE device_type = '".rp_escape_sql($dr_category)."'");

      $model_ids = wp_list_pluck($model_ids, 'm_id');
      
      $options = $rpQuery->get_results("SELECT * FROM $options_table WHERE option_name LIKE 'rp_default_listing_index_model_%' AND option_value = '".rp_escape_sql($previous_position)."'");

      if( !empty($options) ) {

            foreach ($options as $key => $option) {

                  $option_name = $option->option_name;

                  $option_name = str_replace('rp_default_listing_index_model_', '', $option_name);

                  if( !empty($option_name) && is_numeric($option_name) ) {

                        $option_name = (int) $option_name;

                        if( in_array($option_name, $model_ids) ) {

                              // delete $option_name
                              rp_delete_option($option->option_name);
      
                        }

                  }

            }

      }

}

//-------------------------------------------------------------------update model

function update_model(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery; $abc = array();

    $rs_model    = $rpQuery->prefix . "rs_model";

    $rs_model_color    = $rpQuery->prefix . "rs_model_color";

    $mid     = $_POST['mid'];     
    $m_name  = $_POST['m_name'];     
    $m_code  = $_POST['m_code'];     
    $r_date  = $_POST['r_date'];     
    $status  = $_POST['status'];  

    // if (isset($_POST['img'])){
    //     $file=$_POST['img'];
    //     list($type, $file) = explode(';', $file);
    //     list(, $file)      = explode(',', $file);
    //     $file = base64_decode($file);
    //     $upload_overrides = array('test_form' => false);
    //     $movefile = wp_handle_upload($file, $upload_overrides);
    //     if($movefile["url"] != "" || $movefile["url"] != null ){
    //         $m_logo   = $movefile['url'];       
    //         $rpQuery->query("UPDATE $rs_model SET m_img='".rp_escape_sql($m_logo)."' WHERE m_id='".rp_escape_sql($mid)."'");          
    //     }
    // }  
    $rpQuery->query("UPDATE $rs_model SET m_name='".rp_escape_sql($m_name)."',is_active='".rp_escape_sql($status)."',m_code='".rp_escape_sql($m_code)."',release_date='".rp_escape_sql($r_date)."' WHERE m_id='".rp_escape_sql($mid)."'");

    $mc_name   = $_POST['mc_name'] ?? array();
    $mc_code   = $_POST['mc_code'] ?? array();
    $mc_id     = $_POST['mcid'] ?? array();

      foreach($mc_id as $x => $mcid){
            if(!empty(trim($mc_name[$x]))) {
                  if(strtoupper(trim($mcid)) === 'NEW') {
                        $mc_uq_id  = uniqid('modelcolor_');

                        $rpQuery->query("INSERT INTO $rs_model_color(color_name,color_code,mc_uq_id,m_id_fk) VALUES('".rp_escape_sql($mc_name[$x])."','".rp_escape_sql($mc_code[$x])."','".rp_escape_sql($mc_uq_id)."','".rp_escape_sql($mid)."')");

                        $mc_id[$x] = $rpQuery->insert_id;
                  } else {

                        $rpQuery->query("UPDATE $rs_model_color SET color_name='".rp_escape_sql($mc_name[$x])."',color_code='".rp_escape_sql($mc_code[$x])."' WHERE mc_id='".rp_escape_sql($mcid)."'");

                  }
            } else {
                  unset($mc_id[$x]);
            }
      }

    $i = 1;

    foreach($mc_id as $x => $mcid){
      $rpQuery->query("UPDATE $rs_model_color SET mc_position='".rp_escape_sql($i)."' WHERE mc_id='".rp_escape_sql($mcid)."'");
      $i++;
    }

    if(isset($_POST['m_sort'])) {

      $m_sort = $_POST['m_sort'] ?? 0;
      $m_sort = (int) $m_sort;

      // get brand_id_fk of rs_model
      $brand_id_fk = $rpQuery->get_row("SELECT brand_id_fk FROM $rs_model WHERE m_id='".rp_escape_sql($mid)."' LIMIT 1;");
      if( !empty( $brand_id_fk ) ) {
            $brand_id_fk = $brand_id_fk->brand_id_fk;
            $all_model_info = $rpQuery->get_results("SELECT m_id, m_position, brand_id_fk FROM $rs_model  WHERE brand_id_fk = {$brand_id_fk}  ORDER BY m_position ASC, release_date DESC;");
            $sortedModels = array();
            if( !empty( $all_model_info ) ) {
                  foreach( $all_model_info as $theKey => $theValue ) {
                        $sortedModels[] = $theValue->m_id;
                  }
            }
            // remove $mid from $sortedModels
            $sortedModels = array_diff( $sortedModels, array( $mid ) );

            // array values
            $sortedModels = array_values( $sortedModels );

            $sort_pos = 1;

            if( $m_sort > count( $sortedModels ) ) {
                  $m_sort = count( $sortedModels ) + 1;
            }
            
            foreach( $sortedModels as $theKey => $theValue ) {

                  if( $sort_pos === $m_sort ) {
                        // update m_position
                        $rpQuery->query("UPDATE $rs_model SET m_position='".rp_escape_sql($m_sort)."' WHERE m_id='".rp_escape_sql($mid)."'");

                        $sort_pos++;
                  }

                  // update m_position
                  $rpQuery->query("UPDATE $rs_model SET m_position='".rp_escape_sql($sort_pos)."' WHERE m_id='".rp_escape_sql($theValue)."'");
                  $sort_pos++;

                  if( $sort_pos === $m_sort ) {
                        // update m_position
                        $rpQuery->query("UPDATE $rs_model SET m_position='".rp_escape_sql($m_sort)."' WHERE m_id='".rp_escape_sql($mid)."'");

                        $sort_pos++;
                  }

            }
      }

    }

    if(isset($_POST['m_link'])) {
      $m_link = $_POST['m_link'];
      $rpQuery->query("UPDATE $rs_model SET m_img='".rp_escape_sql($m_link)."' WHERE m_id='".rp_escape_sql($mid)."'");

    }

      echo json_encode(
            array(
                  'status' => true, 
                  'message' => 'Successfully Updated', 
                  'color_info' => array(
                  'mc_name' => $mc_name,
                  'mc_id' => $mc_id,
                  'mc_code' => $mc_code
                )
            )
      );

    exit();

}

rp_ajax_for_admin('update_model','update_model');



function update_model_img(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery; $abc = array();

    $rs_model    = $rpQuery->prefix . "rs_model";

    $rs_model_color    = $rpQuery->prefix . "rs_model_color";

    $logo = '';

      if(isset($_FILES) && !empty($_FILES) && isset($_FILES['file']) && isset($_POST) && isset($_POST['mid']) && !empty($_POST['mid'])) {
            $mid = $_POST['mid'];
            $uploadedfile = $_FILES['file']; 
            $upload_overrides = array( 'test_form' => false );
            $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
            if(!empty($movefile["url"])){
                  $logo = $movefile["url"];
                  $rpQuery->query("UPDATE $rs_model SET m_img = '".rp_escape_sql($logo)."' WHERE m_id = '".rp_escape_sql($mid)."' LIMIT 1;");
            }
      }

    echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'logo' => $logo));

    exit();

}

rp_ajax_for_admin('update_model_img','update_model_img');



// --------------------------------------------------------- add model
function wp_repair_get_brand_name_by_id($b_id = '') {
      if(!empty($b_id)) {
            global $rpQuery;

            $rs_brand    = $rpQuery->prefix . "rs_brand";

            $rows = $rpQuery->get_results("SELECT `b_name` FROM $rs_brand WHERE `b_id` = '".rp_escape_sql($b_id)."' LIMIT 1;");

            if(!empty($rows)) {
                  return $rows[0]->b_name;
            } else {
                  return '';
            }
      }
      return '';
}

function rp_get_brand_by_uqid( $uqid = '' ) {

      if(!empty($uqid)) {
            global $rpQuery;

            $rs_brand    = $rpQuery->prefix . "rs_brand";

            $row = $rpQuery->get_row("SELECT * FROM `$rs_brand` WHERE `b_uq_id` = '".rp_escape_sql($uqid)."' LIMIT 1;");

            if(!empty($row)) {
                  return $row;
            } else {
                  return '';
            }
      }
      return '';

}

function rp_get_category_id_by_c_id_fk($c_id_fk = '') {

      if(!empty($c_id_fk)) {
            global $rpQuery;

            $rs_category    = $rpQuery->prefix . "rs_category";

            $row = $rpQuery->get_row("SELECT * FROM `$rs_category` WHERE `c_id` = '".rp_escape_sql($c_id_fk)."' LIMIT 1;");

            if(!empty($row)) {
                  return $row;
            } else {
                  return '';
            }
      }
      return '';


}

function wp_repair_get_category_name_by_id($c_id = '') {
      if(!empty($c_id)) {
            global $rpQuery;

            $rs_category    = $rpQuery->prefix . "rs_category";

            $rows = $rpQuery->get_results("SELECT `c_name` FROM $rs_category WHERE `c_id` = '".rp_escape_sql($c_id)."' LIMIT 1;");

            if(!empty($rows)) {
                  return $rows[0]->c_name;
            } else {
                  return '';
            }
      }
      return '';
}
function wp_repair_add_model() {

      rp_verify_csrf_token_for_ajax('manage_models');

      global $rpQuery;

      $rs_model    = $rpQuery->prefix . "rs_model";

      $uploadedfile = $_FILES['m_img'] ?? FALSE;
      if($uploadedfile !== FALSE) {
            $upload_overrides = array( 'test_form' => false ); 
            $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
            $m_img = $movefile["url"];
      } else {
            $m_img = '';
      }
      $m_name       = $_POST['mname'] ?? '';
      $m_name       = trim($m_name);
      $m_code       = $_POST['mcode'] ?? '';
      $release_date = date('Y-m-d', strtotime($_POST['releasedate']));
      $device_type  = $_POST['device_type'];
      $brand_id_fk  = $_POST['brand_id_fk'];
      $m_uq_id      = uniqid('model_');

      $exists = $rpQuery->get_results("SELECT * FROM $rs_model WHERE `device_type` = '".rp_escape_sql($device_type)."' AND `brand_id_fk` = '".rp_escape_sql($brand_id_fk)."' AND `m_name` = '".rp_escape_sql($m_name)."' LIMIT 1;");
      if(!empty($exists)) {
            echo json_encode(array('status' => false, 'message' => 'already_exists'));
            exit();
      }

      $r_ids = rp_get_all_default_repair_ids_of_category($device_type, TRUE);

      $rpQuery->query("INSERT INTO $rs_model(m_name,m_img,device_type,brand_id_fk,m_code,release_date,m_uq_id,repair_ids) VALUES('".rp_escape_sql($m_name)."','".rp_escape_sql($m_img)."','".rp_escape_sql($device_type)."','".rp_escape_sql($brand_id_fk)."','".rp_escape_sql($m_code)."','".rp_escape_sql($release_date)."','".rp_escape_sql($m_uq_id)."', '".rp_escape_sql($r_ids)."')");

      $m_id = $rpQuery->insert_id;

      rp_process_colors_and_repair_for_model( $m_id );

      // rp_trigger_translations_update_for_all_dr_repairs();

      $related_repairs = rp_get_model_related_repairs_for_admin_page( $m_id );

      echo json_encode(Array('status' => true, 'message' => 'Successfully Created', 'added_model' => array(
            'm_id' => $m_id,
            'device_type' => wp_repair_get_category_name_by_id($device_type),
            'brand' => wp_repair_get_brand_name_by_id($brand_id_fk),
            'model_name' => $m_name,
            'model_code' => $m_code,
            'release_date' => $release_date,
            'm_link' => $m_img,
            'model_uq_id' => $m_uq_id,
            'related_repairs' => $related_repairs
      )));

      exit();
}
rp_ajax_for_admin('wp_repair_add_model','wp_repair_add_model');



// --------------------------------------------------------- update repair image
function update_repair_img(){

      rp_verify_csrf_token_for_ajax('manage_models');

    global $rpQuery; $abc = array();

    $rs_repair    = $rpQuery->prefix . "rs_repair";

    $logo = '';
      if(isset($_FILES) && !empty($_FILES) && isset($_FILES['file'])) {
            $uploadedfile = $_FILES['file']; 
            $upload_overrides = array( 'test_form' => false );
            $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
            if(!empty($movefile["url"])){
                  $logo = $movefile["url"];
            }
            if(isset($_POST) && !empty($_POST)) {
                  $rid = $_POST['rid'] ?? '';
                  if(!empty($rid) && !empty($logo) && trim(strtoupper($rid)) !== 'NEW') {
                        $rpQuery->query("UPDATE $rs_repair SET r_img = '".rp_escape_sql($logo)."' WHERE r_id = '".rp_escape_sql($rid)."' LIMIT 1;");
                  }
            }
      }

    echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'logo' => $logo));

    exit();

}

rp_ajax_for_admin('update_repair_img','update_repair_img');



// --------------------------------------------------------- update default repair image
function update_default_repair_img(){

      rp_verify_csrf_token_for_ajax('default_repairs');

    global $rpQuery; $abc = array();

    $rs_repair    = $rpQuery->prefix . "rs_repair";

    $rs_default_repair    = $rpQuery->prefix . "rs_default_repair";

    $logo = '';
      if(isset($_FILES) && !empty($_FILES) && isset($_FILES['file'])) {
            $uploadedfile = $_FILES['file']; 
            $upload_overrides = array( 'test_form' => false );
            $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
            if(!empty($movefile["url"])){
                  $logo = $movefile["url"];
            }
            if(isset($_POST) && !empty($_POST)) {
                  $rid = $_POST['rid'] ?? '';
                  $dr_category = $_POST['dr_category'] ?? 0;
                  if(!empty($rid) && !empty($logo) && trim(strtoupper($rid)) !== 'NEW') {
                        $result = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE dr_id = '".rp_escape_sql($rid)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");
                        if(!empty($result)) {

                              $dr_uq_id = $result[0]->dr_uq_id;

                              $dr_img = $result[0]->dr_img;

                              $rpQuery->query("UPDATE $rs_default_repair SET dr_img = '".rp_escape_sql($logo)."' WHERE dr_id = '".rp_escape_sql($rid)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

                              $m_ids = rp_get_all_m_ids_by_category( $dr_category );

                              if(!empty($m_ids)) {

                                    $m_ids = join(',',$m_ids);

                                    $rpQuery->query("UPDATE $rs_repair SET r_img = '".rp_escape_sql($logo)."' WHERE r_uq_id = '".rp_escape_sql($dr_uq_id)."' AND r_img = '".rp_escape_sql($dr_img)."' AND m_id_fk IN (".rp_escape_sql($m_ids).");");

                              }

                        }

                  }
            }
      }

    echo json_encode(Array('status' => true, 'message' => 'Successfully Updated', 'logo' => $logo));

    exit();

}

rp_ajax_for_admin('update_default_repair_img','update_default_repair_img');


// --------------------------------------------------------- update upsales image
function update_upsales_img(){

      rp_verify_csrf_token_for_ajax('manage_upsales');

      global $rpQuery; 

      $abc = array();

      $rs_upsales    = $rpQuery->prefix . "rs_upsales";

      $logo = '';
      if(isset($_FILES) && !empty($_FILES) && isset($_FILES['file'])) {
            $uploadedfile = $_FILES['file']; 
            $upload_overrides = array( 'test_form' => false );
            $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
            if(!empty($movefile["url"])){
                  $logo = $movefile["url"];
            }
            if(isset($_POST) && !empty($_POST)) {
                  $uid = $_POST['uid'] ?? '';
                  if(!empty($uid) && !empty($logo) && trim(strtoupper($uid)) !== 'NEW') {
                        $rpQuery->query("UPDATE $rs_upsales SET upsales_img = '".rp_escape_sql($logo)."' WHERE upsales_id = '".rp_escape_sql($uid)."' LIMIT 1;");
                  }
            }
      }

      echo json_encode(Array('status' => true, 'success' => 'Successfully Updated', 'logo' => $logo));

      exit();

}

rp_ajax_for_admin('update_upsales_img','update_upsales_img');


//-------------------------------------------------------------------update repair position

function update_repair_position(){

    global $rpQuery;

    $rs_repair    = $rpQuery->prefix . "rs_repair";

    $r_position = $_POST['r_position'];

    $i = 1; 

    foreach($r_position as $rp){

        $rpQuery->query("UPDATE $rs_repair SET r_position='".rp_escape_sql($i)."' WHERE r_id='".rp_escape_sql($rp)."'");

        $i++;

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    exit();

}

rp_ajax_for_admin('update_repair_position','update_repair_position');


//-------------------------------------------------------------------update upsales position

function update_upsales_position(){

      rp_verify_csrf_token_for_ajax('manage_upsales');

    global $rpQuery;

    $rs_upsales    = $rpQuery->prefix . "rs_upsales";

    $up_position = $_POST['up_position'];

    $i = 1; 

    foreach($up_position as $up){

        $rpQuery->query("UPDATE $rs_upsales SET upsales_position='".rp_escape_sql($i)."' WHERE upsales_id='".rp_escape_sql($up)."'");

        $i++;

    }

    echo json_encode(Array('status'=> true, 'message'  => 'Successfully updated'));

    exit();

}

rp_ajax_for_admin('update_upsales_position','update_upsales_position');


//-------------------------------------------------------------------pdf

function show_dompdf_svg($which = '') {
      if($which === 'location') {
            $which = 'locationicon';
            $svg = rp_fetch_image_from_base(WP_REPAIR_PLUGIN_PATH.'assets/images/'.$which.'.svg');
            return '<div class="location-icon"><img src="data:image/svg+xml;base64,'.base64_encode($svg).'"  width="12.8" height="16" /></div>';
      }
      if($which === 'phone') {
            $svg = rp_fetch_image_from_base(WP_REPAIR_PLUGIN_PATH.'assets/images/'.$which.'.svg');
            return '<div class="phone-icon"><img src="data:image/svg+xml;base64,'.base64_encode($svg).'"  width="18" height="18" /></div>';
      }
      $svg = rp_fetch_image_from_base(WP_REPAIR_PLUGIN_PATH.'assets/images/'.$which.'.svg');
      return '<div class="icon"><img src="data:image/svg+xml;base64,'.base64_encode($svg).'"  width="16" height="16" /></div>';
}

function pretty_offer_id($offer_id = 0) {
      $offer_id_length = strlen((string)$offer_id);
      if($offer_id_length >= 4) {
            return date('Y'). '-' . $offer_id;
      } else if($offer_id_length === 3) {
            return date('Y'). '-0' . $offer_id;
      } else if($offer_id_length === 2) {
            return date('Y'). '-00' . $offer_id;
      } else if($offer_id_length === 1) {
            return date('Y'). '-000' . $offer_id;
      } else {
            return date('Y'). '-000' . $offer_id;
      }
}

function rp_get_m_img_width_height_attrs($width = 150, $height = 150) {
      if((($height * 100) / $width) > 50) {
            // Is More Squarish
            $ratio = $height / $width;
            $width = 70/$ratio;
            return ' height="70" width="'.($width).'"';
      } else {
            // Is Perfect!
            $ratio = $height / $width;
            return ' width="70" height="'.(70*$ratio).'"';
      }
}

function rp_get_company_logo_width_height_attrs($width = 200, $height = 200) {
      if((($height * 100) / $width) > 50) {
            // Is More Squarish
            $ratio = $height / $width;
            $width = 100/$ratio;
            $padding = (200 - $width) / 2;
            return ' height="100" width="'.($width).'"';
      } else {
            // Is Perfect!
            $ratio = $height / $width;
            return ' width="200" height="'.(200*$ratio).'"';
      }
}


function rp_encode_img_base64($img_path = false) {

    if($img_path && !empty($img_path)){

        $path = str_replace((get_site_url(). '/'), ABSPATH, $img_path);

        $type = pathinfo($path, PATHINFO_EXTENSION);

        $data = rp_fetch_image_from_base($path);

        return 'data:image/' . $type . ';base64,' . base64_encode($data);
    }

    return '';

}

function rp_get_where_query($tableName = '', $key = '', $value = '', $single = FALSE, $compare = '=', $prefix = TRUE) {

      global $rpQuery;

      if($prefix === TRUE) {

            $tableName = $rpQuery->prefix . $tableName;

      }

      if($single === TRUE) {

            return $rpQuery->get_row("SELECT * FROM `$tableName` WHERE `$key` $compare '$value' LIMIT 1;");

      } else {

            return $rpQuery->get_results("SELECT * FROM `$tableName` WHERE `$key` $compare '$value'");

      }

}

function rp_get_repair_lock_price( $r_id = '' ) {

      /* Added in V1.4.2 */
      $related_repair = rp_get_where_query('rs_repair', 'r_id', $r_id, TRUE, '=');

      if(!empty($related_repair)) {

            $related_repair = apply_filters('rp_filter_before_adding_repair_to_cart', $related_repair);

            return array($related_repair->r_name, $related_repair->r_price);

      }

      return array('', '');

}

function rp_insert_into_rs_order_repairs( $order_id_fk = '' , $r_id = '' ) {
      /* Added in V1.4.2 */
      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix. "rs_order_repairs";

      list( $relatedRepairName, $relatedRepairPrice ) = rp_get_repair_lock_price($r_id);

      if( isset( $_POST['get_o_id_fk'] ) && !empty( $_POST['get_o_id_fk'] )) {

            $locked_price = rp_get_locked_repair_price( $_POST['get_o_id_fk'], $r_id );

            if( $locked_price !== NULL ) {

                  $relatedRepairPrice = $locked_price;

            }

      }

      // make sure it doesn't exist already

      $check = $rpQuery->get_row("SELECT * FROM $rs_order_repairs WHERE r_id_fk = '".rp_escape_sql($r_id)."' AND o_id_fk = '".rp_escape_sql($order_id_fk)."' LIMIT 1");

      if( empty( $check ) ) {

            $rpQuery->query("INSERT INTO $rs_order_repairs(r_id_fk,o_id_fk,repair_name,repair_price) VALUES('".rp_escape_sql($r_id)."','".rp_escape_sql($order_id_fk)."','".rp_escape_sql($relatedRepairName)."', '".rp_escape_sql($relatedRepairPrice)."')"); 

            do_action('rp_inserted_row_in_rs_order_repairs', $rpQuery->insert_id, $order_id_fk, $r_id);

      }

}

function rp_find_repair_attr_row_by_id( $a_id = '' ) {

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix. "rs_repair_attr";

      $result = $rpQuery->get_row("SELECT * FROM $rs_repair_attr WHERE a_id = '".rp_escape_sql($a_id)."' LIMIT 1;");

      return $result ?? array();

}

function rp_find_repair_row_by_id( $r_id = '' ) {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix. "rs_repair";

      $result = $rpQuery->get_row("SELECT * FROM $rs_repair WHERE r_id = '".rp_escape_sql($r_id)."' LIMIT 1;");

      return $result ?? array();

}

function rp_get_repair_attr_lock_price( $a_id = '' ) {

      /* Added in V1.4.2 */
      global $rpQuery;

      $related_repair_attr = rp_get_where_query('rs_repair_attr', 'a_id', $a_id, TRUE, '=');

      if(!empty($related_repair_attr)) {

            $repairRow = rp_find_repair_row_by_id( $related_repair_attr->r_id_fk );

            $related_repair_attr = apply_filters('rp_filter_before_adding_repair_attr_to_cart', $related_repair_attr, $repairRow);

            return array($related_repair_attr->a_name, $related_repair_attr->a_price, $related_repair_attr->r_id_fk);

      }

      return array('', '', '');

}

function rp_insert_into_rs_order_repair_attr( $order_id_fk = '' , $a_id = '' ) {
      /* Added in V1.4.2 */
      global $rpQuery;

      $rs_order_repair_attr = $rpQuery->prefix. "rs_order_repair_attr";

      list($relatedRepairAttrName, $relatedRepairAttrPrice, $relatedRepairId) = rp_get_repair_attr_lock_price($a_id);

      if( isset( $_POST['get_o_id_fk'] ) && !empty( $_POST['get_o_id_fk'] )) {

            $locked_price = rp_get_locked_repair_attr_price( $_POST['get_o_id_fk'], $relatedRepairId, $a_id );

            if( $locked_price !== NULL ) {

                  $relatedRepairAttrPrice = $locked_price;

            }

      }

      // make sure this doesn't already exist in the cart
      $alreadyExists = $rpQuery->get_row("SELECT * FROM $rs_order_repair_attr WHERE ra_id_fk = '".rp_escape_sql($a_id)."' AND o_id_fk = '".rp_escape_sql($order_id_fk)."' LIMIT 1;");

      if( empty( $alreadyExists ) ) {

            $rpQuery->query("INSERT INTO $rs_order_repair_attr(ra_id_fk,o_id_fk,repair_attr_name,repair_attr_price,r_id_fk) VALUES('".rp_escape_sql($a_id)."','".rp_escape_sql($order_id_fk)."', '".rp_escape_sql($relatedRepairAttrName)."', '".rp_escape_sql($relatedRepairAttrPrice)."', '".rp_escape_sql($relatedRepairId)."')");

            do_action('rp_inserted_row_in_rs_order_repair_attr', $rpQuery->insert_id, $order_id_fk, $a_id);

      }

}     

function rp_get_upsale_lock_price( $up_id = '' ) {

      /* Added in V1.4.2 */
      global $rpQuery;

      $related_upsale = rp_get_where_query('rs_upsales', 'upsales_id', $up_id, TRUE, '=');

      if(!empty($related_upsale)) {

            return array($related_upsale->us_name, $related_upsale->us_new_price);

      }

      return array('', '');

}

function rp_insert_into_rs_order_upsales( $order_id_fk = '' , $up_id = '' ) {
      /* Added in V1.4.2 */
      global $rpQuery;

      $rs_order_upsales = $rpQuery->prefix. "rs_order_upsales";

      list($relatedUpsaleName, $relatedUpsalePrice) = rp_get_upsale_lock_price($up_id);

      if( isset( $_POST['get_o_id_fk'] ) && !empty( $_POST['get_o_id_fk'] )) {

            $locked_price = rp_get_locked_upsale_price( $_POST['get_o_id_fk'], $up_id );

            if( $locked_price !== NULL ) {

                  $relatedUpsalePrice = $locked_price;

            }

      }

      $rpQuery->query("INSERT INTO $rs_order_upsales(upsales_id_fk,o_id_fk,upsale_name,upsale_price) VALUES('".rp_escape_sql($up_id)."','".rp_escape_sql($order_id_fk)."', '".rp_escape_sql($relatedUpsaleName)."', '".rp_escape_sql($relatedUpsalePrice)."')");

}     


function me_post_pdf(){

      rp_verify_csrf_token_for_ajax('frontend');

      if( rp_verify_google_recaptcha() === FALSE ) {

            echo json_encode(
                  array(
                        'status'=> false, 
                        'rp_validation_error' => 'Form submission blocked by Google ReCaptcha. Please refresh the page and try again after few seconds.'
                  )
            );
      
            exit();

      }

      _rp_set_default_tiemzone();
      
      global $wp_repair_selected_language;

    global $rpQuery;

    global $wp_repair_lang;

    $rs_localization = $rpQuery->prefix . "rs_localization"; 

    $rs_model = $rpQuery->prefix. "rs_model";

    $rs_brand = $rpQuery->prefix. "rs_brand";

    $tax_row = $rpQuery->get_row("SELECT * FROM $rs_localization LIMIT 1;");

    if( !empty( $tax_row ) ) {

      $tax = $tax_row->tax;

    } else {

      $tax = 0;

    }

    $mid          = $_POST['mid'] ?? 0;

    $model_name   = '';

    if( !empty( $mid ) ) {

      $model_row = $rpQuery->get_row("SELECT m_id, m_name FROM $rs_model WHERE m_id = ':m_id' LIMIT 1;", array(

            'm_id' => $mid

      ));

      if( empty( $model_row ) ) {

            $mid = 0;

      } else {

            $model_name = $model_row->m_name;

      }

    }

    if( empty( $mid ) ) {

      $model_name   = $_POST['model_name'] ?? '';

      if( !empty( $model_name ) ) {

            $model_name   = str_replace('-', ' ', $model_name);

            $model_name   = str_replace('_', '-', $model_name);

            $model_name   = str_replace('plus', '+', $model_name);

            $model_row = $rpQuery->get_row("SELECT m_id FROM `$rs_model` WHERE m_name = '".rp_escape_sql($model_name)."' LIMIT 1;");

            if( !empty( $model_row ) ) {

              $mid = $model_row->m_id;

            }

      }

    }

    if( empty( $mid ) ) {

      echo json_encode(array(

            'status' => false,

            'rp_validation_error' => 'Invalid input provided.'

      ));

      exit;

    }

    $pdf_location_id = 0;

    if( rp_is_enabled_location_selection_pdf() === true && isset( $_POST ) && !empty( $_POST ) && isset( $_POST['pdf_location_id'] ) && !empty( $_POST['pdf_location_id'] ) ) {

      $pdf_location_id = (int) trim( $_POST['pdf_location_id'] );

      $rs_company_location = $rpQuery->prefix . "rs_company_location";

      $all_locations = $rpQuery->get_results("SELECT * FROM $rs_company_location;");

      // loop through all locations and make sure the location id is valid
      $found_location = FALSE;

      foreach( $all_locations as $location ) {

            if( $location->location_id == $pdf_location_id ) {

                  $found_location = TRUE;

                  break;

            }

      }

      if( $found_location === FALSE ) {

            echo json_encode(array(

                  'status' => false,
      
                  'rp_validation_error' => 'Invalid input provided.'
      
            ));
      
            exit;

      }

    }

    $rpCart       = new RepairPluginPro\RP_Cart( $mid );

    $rid          = $_POST['rid'] ?? array();

    foreach ($rid as $repair_id) {

      $rpCart->add_repair( ((int) $repair_id) );

    }

    $aid          = $_POST['aid'] ?? array();

    foreach ($aid as $attr_id) {

      $rpCart->add_repair_attr( ((int) $attr_id) );

    }

    $upid         = $_POST['upid'] ?? array();

    foreach ($upid as $upsale_id) {

      $rpCart->add_upsale( ((int) $upsale_id) );

    }

    $coupon_id       = $_POST['coupon_id'] ?? '';

    $rpCart->add_coupon( ((int) $coupon_id) );

    $calculations = $rpCart->calculate();

    $combo        = $calculations['combo_discount'];

    $coupon       = $calculations['coupon_amount'];

    $final_price  = $calculations['total'];

    $prepared_post_data = $rpCart->prepare_post_data( $rid, $upid );

    list( $rid, $array_name, $array_price, $aid, $upid ) = $prepared_post_data;

    $color_name   = $_POST['color_name'];

    $user_name    = $_POST['user_name'];

    $street_phone = $_POST['street_phone'];

    $user_phone = $_POST['user_phone'];

    $zipcode_city = $_POST['zipcode_city'];

    $zipcode_only = $_POST['zipcode_only'];

    $email        = $_POST['user_email'];

    $notes        = $_POST['notes'];


    if(!empty($mid)) {

      $cmodel = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE m_id = '".rp_escape_sql($mid)."' LIMIT 1;");

    } else {

      $cmodel = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE m_name = '".rp_escape_sql($model_name)."' LIMIT 1;");

    }

    $rs_user_pdfs = $rpQuery->prefix. "rs_user_pdfs";

    $rpQuery->query("INSERT INTO $rs_user_pdfs(user_name,streetname_phone,zipcode_city,zipcode_only,user_email,notes) VALUES('".rp_escape_sql($user_name)."','".rp_escape_sql($street_phone)."','".rp_escape_sql($zipcode_city)."','".rp_escape_sql($zipcode_only)."','".rp_escape_sql($email)."','".rp_escape_sql($notes)."')");

    $offer_id = $rpQuery->insert_id;

    $rs_customer = $rpQuery->prefix. "rs_customer";

    $rpQuery->query("INSERT INTO $rs_customer(customer_name,customer_phone,customer_email) VALUES('".rp_escape_sql($user_name)."','".rp_escape_sql($user_phone)."','".rp_escape_sql($email)."')");

    $customer_id_fk  = $rpQuery->insert_id;

    $etp = 1 + (($tax * 1) / 100);

    $ex_tax_price = $final_price/$etp;

    $ex_tax_price = number_format((float)$ex_tax_price, 2, '.', '');

    $d_method  = 'Pdf Offer';

    $time = date("H:i");

    $date = date("Y-m-d");

    $rs_orders = $rpQuery->prefix. "rs_orders";

    $rs_order_repairs = $rpQuery->prefix. "rs_order_repairs";

    $rs_order_repair_attr = $rpQuery->prefix. "rs_order_repair_attr";

    $rs_order_upsales = $rpQuery->prefix. "rs_order_upsales";

    $rs_company_location = $rpQuery->prefix . "rs_company_location";
      
    $location_row = $rpQuery->get_row("SELECT * FROM `$rs_company_location` LIMIT 1;");

    $loc_id = '1';

    if( !empty($location_row) ) {

      $loc_id = $location_row->location_id;

    }

    if( !empty( $pdf_location_id ) ) {

      $loc_id = $pdf_location_id;

    }

    $rpQuery->query("INSERT INTO $rs_orders(customer_id_fk,notes,color,excluded_tax_price,combo_discount,coupon_id_fk,total_price,delivery_method,delivery_time,delivery_date,pdf_offer,m_id_fk,status_id_fk,location_id_fk, coupon_amount  ) VALUES('".rp_escape_sql($customer_id_fk)."','".rp_escape_sql($notes)."','".rp_escape_sql($color_name)."','".rp_escape_sql($ex_tax_price)."','".rp_escape_sql($combo)."','".rp_escape_sql($coupon_id)."','".rp_escape_sql($final_price)."','".rp_escape_sql($d_method)."','".rp_escape_sql($time)."','".rp_escape_sql($date)."','1','".rp_escape_sql($mid)."','2','".rp_escape_sql($loc_id)."', '".rp_escape_sql($coupon)."')");

    $order_id_fk = $rpQuery->insert_id;

    rp_set_new_appointment_number($order_id_fk, date('Y-m-d H:i:s'));

    rp_set_coupon_settings($order_id_fk);

    rp_save_order_related_language( $order_id_fk, $wp_repair_selected_language );

    rp_save_cancel_link_by_order_id( $order_id_fk, ($_POST['created_through_url'] ?? '') );

    $link    = $_POST['link'];

    $link    = str_replace(' ','-',$link);

    $link    = str_replace('%20','-',$link);

    $link    = str_replace('?selected=finalize', '?o_id_fk='.$order_id_fk.'&c_id='.$customer_id_fk.'&uid='.substr(md5($customer_id_fk), 0, 8).'&selected=finalize', $link);

    $link    = str_replace('?selected=shortcode_finalize_order', '?o_id_fk='.$order_id_fk.'&c_id='.$customer_id_fk.'&uid='.substr(md5($customer_id_fk), 0, 8).'&selected=shortcode_finalize_order', $link);

    $rpQuery->query("UPDATE `$rs_user_pdfs` SET `o_id_fk` = '$order_id_fk', `finalize_link` = '".rp_escape_sql($link)."' WHERE `user_pdfs_id` = '$offer_id' LIMIT 1;");

    if($rid != null){

        foreach($rid as $r_id){ 

            rp_insert_into_rs_order_repairs($order_id_fk, $r_id);

        }   

    }   

    if($aid != null){

        foreach($aid as $a_id){

            rp_insert_into_rs_order_repair_attr($order_id_fk, $a_id);

        }

    }

    if($upid != null){

        foreach($upid as $up_id){

            rp_insert_into_rs_order_upsales($order_id_fk, $up_id);

        }

    }

    if( !empty( $order_id_fk ?? 0 ) ) {

      rp_trigger_outgoing_webhook( 'pdf_offer_created', $order_id_fk );

    }

    if( rp_send_appointment_status_update_email( $order_id_fk, TRUE ) === TRUE ) {

      echo json_encode(Array('status'=> true, 'message'  => 'Successfully Sent'));

    } else {

      echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated'));

    }

    exit();

}


rp_ajax_for_public('me_post_pdf','me_post_pdf');

function get_email_template_of_wp_repair($which = 'pending') {
      $templates = array(
            'pending' => array(
                  'Wachten op goedkeuring',
                  '<p>Beste $CustomerName$,</p> <p>Bedankt voor het maken van&nbsp;een afspraak bij $LocationName$ om jouw&nbsp;$Model$ te laten repareren. Deze afspraak dient eerst goedgekeurd te worden door ons team.&nbsp;</p> <h3 style="font-family: sans-serif; font-size: 24px; border-radius: 8px; padding: 6px 12px; margin-bottom: 10px; background-color: #d5e4f4; color: #000000; border: 1px solid #b5c6b6;">Jouw afspraak op $AppointmentDate$ wordt zo snel mogelijk verwerkt.</h3> <p>&nbsp;</p> <p>$OrderSummary$</p> <p>&nbsp;</p> <p><strong>Uw gegevens<br></strong>Naam:&nbsp;$CustomerName$<br>Email:&nbsp;$CustomerEmail$<br>Tel:&nbsp;$CustomerPhone$</p> <p>&nbsp;</p> <p>Tijdens de reparatie kun je wachten bij ons in de winkel. We hebben supersnelle Wi-fi en heerlijke koffie. Komt het toch niet uit en wil je de afspraak annuleren?<br /><br />Klik dan op de volgende link: $cancel$.</p> <p>&nbsp;</p> <p>Met vriendelijke groet,</p> <p><strong>$LocationName$<br /></strong>$LocationAddress$<br />$LocationZipcodeCity$</p> <p>&nbsp;</p> <p>&nbsp;</p> <footer style="margin-top: 20px; font-size: 80%; color: #888; clear: both;"> <p>Je krijgt deze e-mail omdat er een afspraak is gemaakt met dit emailadres op de website van $LocationName$. Deze afspraak dient eerst goedgekeurd te worden door ons team. Heb je deze afspraak niet zelf gemaakt, neem dan contact met ons op.</p> </footer>'
            ),
            'approved' => array(
                  'Uw afspraak is goedgekeurd',
                  '<p>Beste $CustomerName$,</p> <p>Bedankt voor het maken van&nbsp;een afspraak bij $LocationName$ om jouw&nbsp;$Model$ te laten repareren.</p> <h3 style="font-family: sans-serif; font-size: 24px; border-radius: 8px; padding: 6px 12px; margin-bottom: 10px; background-color: #d5e4f4; color: #000000; border: 1px solid #b5c6b6;">De afspraak is op $AppointmentDate$, om $AppointmentTime$ uur.</h3> <p>&nbsp;</p> <p>$OrderSummary$</p> <p>&nbsp;</p> <p><strong>Uw gegevens<br></strong>Naam:&nbsp;$CustomerName$<br>Email:&nbsp;$CustomerEmail$<br>Tel:&nbsp;$CustomerPhone$</p> <p>&nbsp;</p> <p>Tijdens de reparatie kun je wachten bij ons in de winkel. We hebben supersnelle Wi-fi en heerlijke koffie. Komt het toch niet uit en wil je de afspraak annuleren?<br /><br />Klik dan op de volgende link: $cancel$.</p> <p>&nbsp;</p> <p>Met vriendelijke groet,</p> <p><strong>$LocationName$<br /></strong>$LocationAddress$<br />$LocationZipcodeCity$</p> <p>&nbsp;</p> <p>&nbsp;</p> <footer style="margin-top: 20px; font-size: 80%; color: #888; clear: both;"> <p>Je krijgt deze e-mail omdat er een afspraak is gemaakt met dit emailadres op de website van $LocationName$. Deze e-mail is een bevestiging van de gemaakte afspraak. Heb je deze afspraak niet zelf gemaakt, neem dan contact met ons op.</p> </footer>'
            ),
            'offer' => array(
                  'Uw Prijsopgaaf',
                  '<p>Beste $CustomerName$,</p> <p>Bedankt voor aanvragen van een offerte voor de reparatie van jouw $Model$.</p> <h3 style="font-family: sans-serif; font-size: 24px; border-radius: 8px; padding: 6px 12px; margin-bottom: 10px; background-color: #d5e4f4; color: #000000; border: 1px solid #b5c6b6;">De offerte voor de reparatie vindt je in de bijlage.</h3> <p>&nbsp;</p> <p>$OrderSummary$</p> <p>&nbsp;</p> <p><strong>Uw gegevens<br></strong>Naam:&nbsp;$CustomerName$<br>Email:&nbsp;$CustomerEmail$<br>Tel:&nbsp;$CustomerPhone$</p> <p>&nbsp;</p> <p>Meestal veranderen de prijzen niet, maar in deze tijden zou het kunnen dat wij onderdelen duurder inkopen dan oorspronkelijk verwerkt in de genoemde prijzen. Vandaar dat de offerte maximaal 14 dagen geldig is vanaf de offerte datum.<br /><br />Wil je de reparatie laten uitvoeren? Druk dan op onderstaande knop om de afspraak te voltooien.<br />&nbsp;</p> <p>Met vriendelijke groet,</p> <p><strong>$LocationName$</strong><br />$LocationAddress$<br />$LocationZipcodeCity$<br /><br />$FinalizeButton$<br /></p> <p>&nbsp;&nbsp;&nbsp;</p> <p>&nbsp;</p> <footer style="margin-top: 20px; font-size: 80%; color: #888; clear: both;"> <p>Je krijgt deze e-mail omdat er een offerte is aangevraagd met dit emailadres op de website van $LocationName$. Heb je deze offerte niet zelf aangevraagd, neem dan contact met ons op.</p> </footer>'
            ),
            'cancelled' => array(
                  'Uw afspraak is geannuleerd',
                  '<p>Beste $CustomerName$,</p> <p>Hierbij bevestigen we dat de afspraak voor het repareren van de&nbsp;$Model$ is geannuleerd.</p> <h3 style="font-family: sans-serif; font-size: 24px; border-radius: 8px; padding: 6px 12px; margin-bottom: 10px; background-color: #dd1935; color: #ffffff; border: 1px solid #b5c6b6;">De afspraak op $AppointmentDate$ om $AppointmentTime$ is geannuleerd.</h3> <p>&nbsp;</p> <p>$OrderSummary$</p> <p>&nbsp;</p> <p><strong>Uw gegevens<br></strong>Naam:&nbsp;$CustomerName$<br>Email:&nbsp;$CustomerEmail$<br>Tel:&nbsp;$CustomerPhone$</p> <p>&nbsp;</p> <p>Wil je opnieuw een afspraak inplannen? Bezoek dan onze website.</p> <p>Met vriendelijke groet,</p> <p><strong>$LocationName$<br /></strong>$LocationAddress$<br />$LocationZipcodeCity$<br /><br /><strong><br /></strong></p> <p>&nbsp;</p> <footer style="margin-top: 20px; font-size: 80%; color: #888; clear: both;"> <p>Je krijgt deze e-mail omdat er een afspraak is geannuleerd met dit emailadres op de website van $LocationName$. Deze e-mail is een bevestiging van de geannuleerde afspraak. Heb je deze afspraak niet zelf gemaakt geannuleerd, neem dan contact met ons op.</p> </footer>'
            )
      );
      return $templates[$which];
}

function set_default_wp_repair_email_templates() {
      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $noti = $rpQuery->get_results("SELECT * FROM `$rs_notifications_and_offers` LIMIT 1;");

      if(!empty($noti)) {

            if(rp_get_option('set_default_wp_repair_email_templates', FALSE) === FALSE) {

                  $list = array('pending', 'offer', 'approved', 'cancelled');

                  foreach ($list as $key => $value) {
                        $id = $key + 1;
                        $template = get_email_template_of_wp_repair($value);
                        $subject = $template[0];
                        $message = $template[1];
                        $noti = $rpQuery->query("UPDATE `$rs_notifications_and_offers` SET `noti_subject` = '".rp_escape_sql($subject)."', `noti_message` = '".rp_escape_sql($message)."' WHERE `noti_id` = '".rp_escape_sql($id)."';");
                  }

                  rp_add_option('set_default_wp_repair_email_templates', TRUE);

            }

      }
}
add_action('init', 'set_default_wp_repair_email_templates');

//--------------------------------------------------------------------------------------

require_once plugin_dir_path( __FILE__)."RepairpluginBase.php";
require_once plugin_dir_path( __FILE__)."RepairpluginDynamic.php";
require_once plugin_dir_path( __FILE__)."RepairpluginEnhanced.php";
require_once plugin_dir_path( __FILE__)."RP_DynamicPricingAddon.php";
require_once plugin_dir_path( __FILE__)."RP_EnhancedLocationsAddon.php";

class Repairplugin {

    public $plugin_file=__FILE__;

    public $responseObj;

    public $licenseMessage;

    public $showMessage=false;

    public $slug="repair-plugin";

    function __construct() {

        add_action( 'admin_print_styles', [ $this, 'SetAdminStyle' ] );

        $licenseKey=rp_get_option("Repairplugin_lic_Key","");

        $liceEmail=rp_get_option( "Repairplugin_lic_email","");

        RepairpluginBase::addOnDelete(function(){

           rp_delete_option("Repairplugin_lic_Key");

        });

        if(RepairpluginBase::CheckWPPlugin($licenseKey,$liceEmail,$this->licenseMessage,$this->responseObj,__FILE__)){

            add_action( 'admin_menu', [$this,'ActiveAdminMenu'],99999);

            add_action( 'admin_post_Repairplugin_el_deactivate_license', [ $this, 'action_deactivate_license' ] );

                //$this->licenselMessage=$this->mess;

                //***Write you plugin's code here***

        }else{

            define('WP_REPAIR_REQUIRE_LICENSE', TRUE);

            if(!empty($licenseKey) && !empty($this->licenseMessage)){

               $this->showMessage=true;

            }

            rp_update_option("Repairplugin_lic_Key","") || rp_add_option("Repairplugin_lic_Key","");

            add_action( 'admin_post_Repairplugin_el_activate_license', [ $this, 'action_activate_license' ] );

            add_action( 'admin_menu', [$this,'InactiveMenu']);

        }

    }

    function SetAdminStyle() {

        wp_register_style( "RepairpluginLic", plugin_dir_url( __FILE__)."_lic_style.css");

        wp_enqueue_style( "RepairpluginLic" );

    }

    function get_menu_name( $value = '' ) {

      $all_menu_items = array(

            'appointments' => 'Appointments',

            'manage categories' => 'Categories',

            'manage models' => 'Models & Repairs',

            'manage upsales' => 'Upsales',

            'download libraries' => 'Download',

            'default repairs' => 'Default Repairs',

            'settings' => 'Settings'

      );

      return $all_menu_items[ $value ];

    }

    function ActiveAdminMenu(){

        add_menu_page ("RepairPlugin", "WP Repair", "activate_plugins", $this->slug, [$this,"Activated"], " dashicons-star-filled " , 25);

        $allPages = array('appointments', 'manage categories', 'manage models', 'manage upsales', 'download libraries', 'default repairs', 'settings');

        foreach ($allPages as $key => $value) {

            $page = str_replace(' ', '_', strtolower($value));

            $callback = 'wp_repair_admin_main_page_html';

            if(function_exists('wp_repair_admin_'.$page.'_page_html')) {

                $callback = 'wp_repair_admin_'.$page.'_page_html';

            }

            $menu_name = $this->get_menu_name( $value );

            add_submenu_page(

                $this->slug,

                ucwords(strtolower($value)),

                ucwords(strtolower($menu_name)),

                'manage_wp_repair_plugin',

                'wp_repair_'.$page,

                $callback,

                                

            );

        }

        _rp_change_admin_submenu_first_time_name();

    }

    function InactiveMenu() {

        add_menu_page( 'Repairplugin','WP Repair',"activate_plugins", $this->slug,  [$this,"LicenseForm"], " dashicons-star-filled " ,25);

        _rp_change_admin_submenu_first_time_name();

        // $allPages = array('appointments', 'manage categories', 'manage models', 'manage repairs', 'manage upsales', 'download libraries', 'settings');

        // foreach ($allPages as $key => $value) {

        //     $page = str_replace(' ', '_', strtolower($value));

        //     $callback = 'wp_repair_admin_main_page_html';

        //     if(function_exists('wp_repair_admin_'.$page.'_page_html')) {

        //         $callback = 'wp_repair_admin_'.$page.'_page_html';

        //     }

        //     add_submenu_page(

        //         $this->slug,

        //         ucwords(strtolower($value)),

        //         ucwords(strtolower($value)),

        //         'manage_wp_repair_plugin',

        //         'wp_repair_'.$page,

        //         $callback,

                                

        //     );

        // }

    }

    function action_activate_license(){

        check_admin_referer( 'el-license' );

        $licenseKey=!empty($_POST['el_license_key'])?$_POST['el_license_key']:"";

        $licenseEmail=!empty($_POST['el_license_email'])?$_POST['el_license_email']:"";

        rp_update_option("Repairplugin_lic_Key",$licenseKey) || rp_add_option("Repairplugin_lic_Key",$licenseKey);

        rp_update_option("Repairplugin_lic_email",$licenseEmail) || rp_add_option("Repairplugin_lic_email",$licenseEmail);

        rp_update_option('_site_transient_update_plugins','');

        wp_safe_redirect(admin_url( 'admin.php?page='.$this->slug));

        exit;

    }

    function action_deactivate_license() {

        check_admin_referer( 'el-license' );

        $message="";

        if(RepairpluginBase::RemoveLicenseKey(__FILE__,$message)){

        }

        rp_update_option("Repairplugin_lic_Key","") || rp_add_option("Repairplugin_lic_Key","");

        rp_update_option('_site_transient_update_plugins','');

        wp_safe_redirect(admin_url( 'admin.php?page='.$this->slug));

        exit;

    }

    function Activated(){

      rp_show_validation_error_message();

      wp_repair_admin_setup_tour_nav_html();

        ?>

        <form method="post" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>">

            <input type="hidden" name="action" value="Repairplugin_el_deactivate_license"/>

            <div class="el-license-container">

                <h3 class="el-license-title"><i class="dashicons-before dashicons-star-filled"></i> <?php _e("RepairPlugin License Info",$this->slug);?> </h3>

                <hr>

                <ul class="el-license-info">

                <li>

                    <div>

                        <span class="el-license-info-title"><?php _e("Status",$this->slug);?></span>



                        <?php if ( isset($this->responseObj->is_valid) && $this->responseObj->is_valid ) : ?>

                            <span class="el-license-valid"><?php _e("Valid",$this->slug);?></span>

                        <?php else : ?>

                            <span class="el-license-valid"><?php _e("Invalid",$this->slug);?></span>

                        <?php endif; ?>

                    </div>

                </li>



                <li>

                    <div>

                        <span class="el-license-info-title"><?php _e("License Type",$this->slug);?></span>

                        <?php echo $this->responseObj->license_title ?? ''; ?>

                    </div>

                </li>



               <li>

                   <div>

                       <span class="el-license-info-title"><?php _e("License Expired on",$this->slug);?></span>

                       <?php echo $this->responseObj->expire_date ?? '';

                       if(isset($this->responseObj->expire_renew_link) && !empty($this->responseObj->expire_renew_link)){

                           ?>

                           <a target="_blank" class="el-blue-btn" href="<?php echo $this->responseObj->expire_renew_link; ?>">Renew</a>

                           <?php

                       }

                       ?>

                   </div>

               </li>



               <li>

                   <div>

                       <span class="el-license-info-title"><?php _e("Support Expired on",$this->slug);?></span>

                       <?php

                           echo $this->responseObj->support_end ?? '';

                        if(isset($this->responseObj->support_renew_link) && !empty($this->responseObj->support_renew_link)){

                            ?>

                               <a target="_blank" class="el-blue-btn" href="<?php echo $this->responseObj->support_renew_link; ?>">Renew</a>

                            <?php

                        }

                       ?>

                   </div>

               </li>

                <li>

                    <div>

                        <span class="el-license-info-title"><?php _e("Your License Key",$this->slug);?></span>

                        <span class="el-license-key"><?php
                        $theLicense = $this->responseObj->license_key ?? '';
                        echo esc_attr( substr($theLicense,0,9)."XXXXXXXX-XXXXXXXX".substr($theLicense,-9) ); ?></span>

                    </div>

                </li>

                </ul>

                <p style="margin-top: 25px; margin-bottom:0px;">
                  <?php 
                  global $wp_repair_lang;
                  _e($wp_repair_lang['do_you_need_help'],$this->slug); ?>
                </p>

                <div class="el-license-active-btn">

                    <?php wp_nonce_field( 'el-license' ); ?>

                    <?php submit_button('Deactivate'); ?>

                </div>

            </div>

        </form>

    <?php

    }



    function LicenseForm() {

        global $wp_repair_lang;

        ?>

    <form method="post" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>">

        <input type="hidden" name="action" value="Repairplugin_el_activate_license"/>

        <div class="el-license-container">

            <h3 class="el-license-title"><i class="dashicons-before dashicons-star-filled"></i> <?php _e("RepairPlugin Pro",$this->slug);?></h3>

            <hr>

            <?php

            if(!empty($this->showMessage) && !empty($this->licenseMessage)){
                $licenseError = $this->licenseMessage;
                $licenseErrorKey = strtolower(str_replace(' ', '_', $licenseError));
                if(isset($wp_repair_lang[$licenseErrorKey])) {
                    $licenseError = $wp_repair_lang[$licenseErrorKey];
                }

                ?>

                <div class="notice notice-error is-dismissible">

                    <p><?php echo _e($licenseError,$this->slug); ?></p>

                </div>

                <?php

            }

            ?>

            <p><?php _e($wp_repair_lang['enter_your_license_key_here'],$this->slug);?></p>

            <p><?php _e($wp_repair_lang['if_you_dont_have_license'],$this->slug);?></p>

            <div class="el-license-field">

                <label for="el_license_key"><?php _e($wp_repair_lang['license_code'],$this->slug);?></label>

                <input type="text" class="regular-text code" name="el_license_key" size="50" placeholder="xxxxxxxx-xxxxxxxx-xxxxxxxx-xxxxxxxx" required="required">

            </div>

            <div class="el-license-field">

                <label for="el_license_key"><?php _e($wp_repair_lang['license_email'],$this->slug);?></label>

                <?php

                    $purchaseEmail   = rp_get_option( "Repairplugin_lic_email", get_bloginfo( 'admin_email' ));

                ?>

                <input type="text" class="regular-text code" name="el_license_email" size="50" value="<?php echo $purchaseEmail; ?>" placeholder="" required="required">

                <div><small><?php _e($wp_repair_lang['we_will_send_updated_news_of_this_product'],$this->slug);?></small></div>

            </div>

            <p style="margin-top: 25px; margin-bottom:0px;">
                  <?php _e($wp_repair_lang['do_you_need_help'],$this->slug); ?>
            </p>

            <div class="el-license-active-btn">

                <?php wp_nonce_field( 'el-license' ); ?>

                <?php submit_button('Activate'); ?>

            </div>

        </div>

    </form>

        <?php

    }

}

function _rp_change_admin_submenu_first_time_name($file = '') {
      /*Added in 1.4.2*/
      global $menu, $submenu;

      if(!empty($menu) && is_array($menu)) {

            foreach ($menu as $key => $menu_item) {
            
                  if(is_array($menu_item) && isset($menu_item[0]) && $menu_item[0] === 'WP Repair') {

                        $menu_item[0] = "<span class='repairplugin-admin-menu-item'>RepairPlugin</span> Pro";

                        if(isset($menu_item[6])) {

                              $menu_item[6] = WP_REPAIR_PLUGIN_URL.'assets/images/rp-dasbhoard-icon-white.svg';

                        }

                        $menu[$key] = $menu_item;

                  }

            }

      }
      
      if(isset($submenu['repair-plugin']) && isset($submenu['repair-plugin'][0]) && isset($submenu['repair-plugin'][0][0]) && trim($submenu['repair-plugin'][0][0]) === 'WP Repair') {

            $submenu['repair-plugin'][0][0] = 'License Information';

      }

      return $file;

}

add_filter('submenu_file', '_rp_change_admin_submenu_first_time_name', 10, 1);

function _rp_style_RepairPlugin_menu_item() {
      /*Added in 1.4.2*/
      ?>
      <style type="text/css">
            #adminmenu li.menu-top:hover a.toplevel_page_repair-plugin .wp-menu-image img, 
            #adminmenu li.wp-has-current-submenu a.toplevel_page_repair-plugin .wp-menu-image img,
            #adminmenu li a.toplevel_page_repair-plugin .wp-menu-image img {
                  padding: 7px 6px 2px 6px !important;
                  opacity: 1 !important;
                  width: 20px;
            }
            #adminmenu li a.toplevel_page_repair-plugin .repairplugin-admin-menu-item {
                  font-weight: 700 !important;
            }
            #adminmenu li a.toplevel_page_repair-plugin,
            #adminmenu li.wp-has-current-submenu a.wp-has-current-submenu.toplevel_page_repair-plugin {
                  background: #0556a5 !important;
                  color: #fff !important;
            }
            #adminmenu li a.toplevel_page_repair-plugin.wp-not-current-submenu,
            #adminmenu li.wp-has-current-submenu a.toplevel_page_repair-plugin.wp-not-current-submenu {
                  color: #f0f0f1 !important;
            }
            #adminmenu li a.toplevel_page_repair-plugin:hover,
            #adminmenu li.wp-has-current-submenu a.wp-has-current-submenu.toplevel_page_repair-plugin:hover {
                  background: #2271b1 !important;
                  color: #fff !important;
            }
      </style>
      <?php

}

add_action('admin_head', '_rp_style_RepairPlugin_menu_item');


function wp_repair_download_libraries_images_get_progress_data() {
      global $rpQuery;
  
      $rs_languages = $rpQuery->prefix . "rs_languages";
      $rs_category = $rpQuery->prefix . "rs_category";
      $rs_brand = $rpQuery->prefix . "rs_brand";
      $rs_model = $rpQuery->prefix . "rs_model";
      $rs_repair = $rpQuery->prefix . "rs_repair";
      $rs_default_repair = $rpQuery->prefix . "rs_default_repair";
  
      $allImages = array();
      $alreadyDoneAllImages = array();

      $languages = $rpQuery->get_results("SELECT DISTINCT `image` FROM $rs_languages;");

      foreach ($languages as $key => $language) {
            $language->dl_image = $language->image;
      
            if( array_search($language->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $language->dl_image;
                  $allImages[] = $language;
            }
      }
  
      $categories = $rpQuery->get_results("SELECT DISTINCT `c_img` FROM $rs_category;");
  
      foreach ($categories as $key => $category) {
            $category->dl_image = $category->c_img;
      
            if( array_search($category->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $category->dl_image;
                  $allImages[] = $category;
            }
      }
  
      $brands = $rpQuery->get_results("SELECT DISTINCT `b_img` FROM $rs_brand;");
  
      foreach ($brands as $key => $brand) {
            $brand->dl_image = $brand->b_img;
      
            if( array_search($brand->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $brand->dl_image;
                  $allImages[] = $brand;
            }
      }
  
      $models = $rpQuery->get_results("SELECT DISTINCT `m_img` FROM $rs_model;");
  
      foreach ($models as $key => $model) {
            $model->dl_image = $model->m_img;
      
            if( array_search($model->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $model->dl_image;
                  $allImages[] = $model;
            }
      }
  
      $repairs = $rpQuery->get_results("SELECT DISTINCT `r_img` FROM $rs_repair;");
  
      foreach ($repairs as $key => $repair) {
            $repair->dl_image = $repair->r_img;
      
            if( array_search($repair->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $repair->dl_image;
                  $allImages[] = $repair;
            }
      }

      $drepairs = $rpQuery->get_results("SELECT DISTINCT `dr_img` FROM $rs_default_repair;");

      foreach ($drepairs as $key => $drepair) {
            $drepair->dl_image = $drepair->dr_img;

                  if( array_search($drepair->dl_image, $alreadyDoneAllImages) === FALSE ) {
                        $alreadyDoneAllImages[] = $drepair->dl_image;
                        $allImages[] = $drepair;
                  }
      }
  
      $fullUniqueImages = array();
      $fullUniqueImages_count = 0;
  
      foreach ($allImages as $key => $value) {
  
            if(!isset($fullUniqueImages[$value->dl_image])) {
                  $fullUniqueImages[$value->dl_image] = 1;
                  $fullUniqueImages_count++;
            } else {
                  $fullUniqueImages[$value->dl_image]++;
            }
      
            if(strpos($value->dl_image, 'https://base.repairplugin.com/') === FALSE) {
                  unset($allImages[$key]);
            }
      }
  
      $onlyUnique = array();
      $onlyUniqueCount = 0;
  
      foreach ($allImages as $key => $value) {
            if(!isset($onlyUnique[$value->dl_image])) {
                  $onlyUnique[$value->dl_image] = 1;
                  $onlyUniqueCount++;
            } else {
                  $onlyUnique[$value->dl_image]++;
            }
      }
  
      $onlyUniqueCount = $onlyUniqueCount - 1;
  
      $response = array(
          'total' => $fullUniqueImages_count,
          'remaining' => $onlyUniqueCount
      );
      return $response;
}
  

function download_libraries_images_to_local() {
    global $rpQuery;

    RepairPluginPro\rp_cron_job_verifiy(array(
        'rp_token_error' => 'No token detected?'
    ));

    $rs_category = $rpQuery->prefix . "rs_category";
    $rs_brand = $rpQuery->prefix . "rs_brand";
    $rs_model = $rpQuery->prefix . "rs_model";
    $rs_repair = $rpQuery->prefix . "rs_repair";
    $rs_default_repair = $rpQuery->prefix . "rs_default_repair";
    $rs_languages = $rpQuery->prefix . "rs_languages";

    $allImages = array();
    $alreadyDoneAllImages = array();

    $_languages = $rpQuery->get_results("SELECT DISTINCT `image` FROM $rs_languages;");

    foreach ($_languages as $key => $lang) {
            $lang->dl_image = $lang->image;

            if( array_search($lang->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $lang->dl_image;
                  $allImages[] = $lang;
            }
    }

    $categories = $rpQuery->get_results("SELECT DISTINCT `c_img` FROM $rs_category;");

    foreach ($categories as $key => $category) {
            $category->dl_image = $category->c_img;

            if( array_search($category->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $category->dl_image;
                  $allImages[] = $category;
            }
    }

    $brands = $rpQuery->get_results("SELECT DISTINCT `b_img` FROM $rs_brand;");

    foreach ($brands as $key => $brand) {
        $brand->dl_image = $brand->b_img;

            if( array_search($brand->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $brand->dl_image;
                  $allImages[] = $brand;
            }
    }

    $models = $rpQuery->get_results("SELECT DISTINCT `m_img` FROM $rs_model;");

    foreach ($models as $key => $model) {
        $model->dl_image = $model->m_img;

            if( array_search($model->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $model->dl_image;
                  $allImages[] = $model;
            }
    }

    $repairs = $rpQuery->get_results("SELECT DISTINCT `r_img` FROM $rs_repair;");

    foreach ($repairs as $key => $repair) {
        $repair->dl_image = $repair->r_img;

            if( array_search($repair->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $repair->dl_image;
                  $allImages[] = $repair;
            }
    }

    $drepairs = $rpQuery->get_results("SELECT DISTINCT `dr_img` FROM $rs_default_repair;");

    foreach ($drepairs as $key => $drepair) {
        $drepair->dl_image = $drepair->dr_img;

            if( array_search($drepair->dl_image, $alreadyDoneAllImages) === FALSE ) {
                  $alreadyDoneAllImages[] = $drepair->dl_image;
                  $allImages[] = $drepair;
            }
    }

    $totalImagesInTheServer = count($allImages);

    $fullUniqueImages = array();
    $fullUniqueImages_count = 0;
    $removeFromMedia = array();
    $wp_upload_dir = wp_upload_dir();
    $wp_upload_dir_url = $wp_upload_dir['url'];

    foreach ($allImages as $key => $value) {

        if(!isset($fullUniqueImages[$value->dl_image])) {
            $fullUniqueImages[$value->dl_image] = 1;
            $fullUniqueImages_count++;
        } else {
            $fullUniqueImages[$value->dl_image]++;
        }

        if(strpos($value->dl_image, $wp_upload_dir_url) !== FALSE) {
            if(array_search($value->dl_image, $removeFromMedia) === FALSE) {
                $removeFromMedia[] = $value->dl_image;
            }
        }

        if(strpos($value->dl_image, 'https://base.repairplugin.com/') === FALSE) {
            unset($allImages[$key]);
        }
    }

    $already_deleted = FALSE;
    if(!empty($removeFromMedia)) {
        if(!rp_get_option('wp_repair_images_removed_from_media')) {
            rp_add_option('wp_repair_images_removed_from_media', TRUE);
            foreach ($removeFromMedia as $key => $dl_image) {
                $rpQuery->query("DELETE FROM $rpQuery->posts WHERE `post_type` = 'attachment' AND `post_status` = 'inherit' AND `guid` = '".rp_escape_sql($dl_image)."' LIMIT 1;");
            }
        } else {
            $already_deleted = TRUE;
        }
    }

    $onlyUnique = array();
    $onlyUniqueCount = 0;

    foreach ($allImages as $key => $value) {
        if(!isset($onlyUnique[$value->dl_image])) {
            $onlyUnique[$value->dl_image] = 1;
            $onlyUniqueCount++;
        } else {
            $onlyUnique[$value->dl_image]++;
        }
    }

    $onlyUniqueCount = $onlyUniqueCount - 1;

    arsort($onlyUnique);

    $remainingDownload = count($allImages);

    if(!empty($onlyUnique)) {
        $itWorked = FALSE;
        foreach ($onlyUnique as $link => $ig) {
            if($itWorked === FALSE) {
                $itWorked = TRUE;
                $imageData = rp_fetch_image_from_base( str_replace( ' ', '%20', trim( $link ) ) );

                if(empty($imageData)) {
                  continue;
                }

                $only_file_name = explode('/', $link);
                $only_file_name = $only_file_name[(count($only_file_name) - 1)];

                $wp_upload_dir = wp_upload_dir();
                $upload_path = $wp_upload_dir['path'];

                $uniqueFileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $upload_path.'/'.str_replace(array(':', '_', ' ', '!', '@', '#', '$','%','^','&','*','(',')', ',', '=', '+'), '-', $only_file_name));

                $fileExtension = '';

                $mayBeFileExtension = substr($uniqueFileName, (strlen($uniqueFileName) - 8));

                $mayBeFileExtension = explode('.', $mayBeFileExtension);
                $mayBeFileExtension = $mayBeFileExtension[(count($mayBeFileExtension) - 1)];

                if(file_exists($uniqueFileName)) {
                    $uniqueFileName = str_replace('.'.$mayBeFileExtension, '', $uniqueFileName);
                    $uniqueFileName .= '-'.uniqid() . '.' . $mayBeFileExtension;
                }

                file_put_contents($uniqueFileName, $imageData);

                $old_link_for_db = $link;
                $new_link_for_db = $wp_upload_dir['url'] . '/' . basename( $uniqueFileName );

                $results = array();

                $results[] = $rpQuery->query("UPDATE $rs_category SET `c_img` = '".rp_escape_sql($new_link_for_db)."' WHERE `c_img` = '".rp_escape_sql($old_link_for_db)."';");

                $results[] = $rpQuery->query("UPDATE $rs_brand SET `b_img` = '".rp_escape_sql($new_link_for_db)."' WHERE `b_img` = '".rp_escape_sql($old_link_for_db)."';");

                $results[] = $rpQuery->query("UPDATE $rs_model SET `m_img` = '".rp_escape_sql($new_link_for_db)."' WHERE `m_img` = '".rp_escape_sql($old_link_for_db)."';");

                $results[] = $rpQuery->query("UPDATE $rs_repair SET `r_img` = '".rp_escape_sql($new_link_for_db)."' WHERE `r_img` = '".rp_escape_sql($old_link_for_db)."';");

                $results[] = $rpQuery->query("UPDATE $rs_default_repair SET `dr_img` = '".rp_escape_sql($new_link_for_db)."' WHERE `dr_img` = '".rp_escape_sql($old_link_for_db)."';");

                $results[] = $rpQuery->query("UPDATE $rs_languages SET `image` = '".rp_escape_sql($new_link_for_db)."' WHERE `image` = '".rp_escape_sql($old_link_for_db)."';");

                $remainingDownload--;
            }
        }
    }
    $response = array(
        'total' => $fullUniqueImages_count,
        'remaining' => $onlyUniqueCount,
        'already_deleted' => $already_deleted
    );
    RepairPluginPro\rp_cron_job_response($response);
}

rp_ajax_for_public('download_libraries_images','download_libraries_images_to_local');



function rp_process_pending_background_jobs() {
      global $rpQuery;
  
      RepairPluginPro\rp_cron_job_verifiy(array(
          'rp_token_error' => 'No token detected?'
      ));

      // use microtime to track seconds
      $start_time = microtime(true);

      $rs_background_jobs = $rpQuery->prefix . 'rs_background_jobs';

      $pending_jobs = 1;

      do {
            rp_process_rs_background_jobs_table();
            // check time in seconds
            $end_time = microtime(true);
            $time = $end_time - $start_time;

            // count all pending jobs where processed = 0
            $pending_jobs = $rpQuery->get_var("SELECT COUNT(id) FROM `$rs_background_jobs` WHERE `processed` = 0");
      } while( $time < WP_REPAIR_MAX_BACKGROUND_JOB_TIME && ((int) $pending_jobs > 0) );

      $total_rows = $rpQuery->get_var("SELECT COUNT(id) FROM `$rs_background_jobs`");

      // count all pending jobs where processed = 0
      $pending_jobs = $rpQuery->get_var("SELECT COUNT(id) FROM `$rs_background_jobs` WHERE `processed` = 0");
      
      $response = array(
          'total' => (int) $total_rows,
          'remaining' => (int) $pending_jobs
      );
      RepairPluginPro\rp_cron_job_response($response);
}

rp_ajax_for_public('rp_process_pending_background_jobs','rp_process_pending_background_jobs');

function rp_javascript_get_next_run_schedule() {

      ?>
      <script type="text/javascript">
            window.lastWpRepairCRONRan = null;
          function getCurrentTime() {
              let theTime = new Date().getTime() / 1000;
              return parseFloat(theTime.toFixed(0));
          }
          window._getCurrentTime = function() {
              return getCurrentTime();
          }
          function getNextSafeSchedule(rp_try_after) {
              let newRun = getNextRun();
              if((getCurrentTime() + rp_try_after) >= newRun) {
                  window.lastWpRepairCRONRan = getCurrentTime() + rp_try_after;
                  console.log(window.lastWpRepairCRONRan - getCurrentTime());
                  return (window.lastWpRepairCRONRan - getCurrentTime()) * 1000;
              } else {
                  console.log(window.lastWpRepairCRONRan - getCurrentTime());
                  return (window.lastWpRepairCRONRan - getCurrentTime()) * 1000;
              }
          }
          window._getNextSafeSchedule = function(rp_try_after) {
              return getNextSafeSchedule(rp_try_after);
          }
          function getNextRun() {
              if(window.lastWpRepairCRONRan === null || window.lastWpRepairCRONRan < getCurrentTime()) {
                  window.lastWpRepairCRONRan = getCurrentTime() + 3;
                  return window.lastWpRepairCRONRan;
              }
              window.lastWpRepairCRONRan = window.lastWpRepairCRONRan + 3;
              return window.lastWpRepairCRONRan;
          }
      </script>
      <?php

}

function download_libraries_ajax_handler() {
    ?>
    <script type="text/javascript">
        window.repeatableImagesDownloadAlreadyRunning = false;
        function repeatableImagesDownload(rp_token = false) {
            if(window.repairsAndColorsUpdate && window.repairsAndColorsUpdate === true) {
                  jQuery("#rp_downloading_libraries_images").hide();
                  return;
            }
            let theData;
            var theTime = window._getCurrentTime();
            if(rp_token === true) {
                theData = {
                    action: 'download_libraries_images',
                    rp_run_token: true
                }
            } else {
                theData = {
                    action: 'download_libraries_images'
                }
            }
            window.repeatableImagesDownloadAlreadyRunning = true;
            jQuery.ajax({
                type: "POST",
                url: '<?php echo get_site_url(); ?>/wp-admin/admin-ajax.php?time='+theTime,
                data: theData,
                success: function(data) {
                    try {
                        data = JSON.parse(data);
                    } catch (e) {

                    }
                    if(data && data.rp_token_error) {
                        if(data.rp_try_after && data.rp_try_after !== false && data.rp_try_after > 0) {
                            window.setTimeout(function(){
                                repeatableImagesDownload(true);
                            }, window._getNextSafeSchedule(data.rp_try_after));
                        }
                    } else if(data && data.total) {
                        let total = data.total;
                        let remaining = data.remaining;
                        let done_images = total - remaining;

                        let percentRemaining = (remaining * 100) / total;
                        let percentDone = 100 - percentRemaining;

                        jQuery("#rp_downloading_libraries_images .done").text(done_images);
                        jQuery("#rp_downloading_libraries_images .total").text(total);

                        if(percentDone < 100) {
                            jQuery("#rp_downloading_libraries_images").show();
                            if(data.rp_try_after && data.rp_try_after !== false && data.rp_try_after > 0) {
                                window.setTimeout(function(){
                                    repeatableImagesDownload(true);
                                }, window._getNextSafeSchedule(data.rp_try_after));
                            }
                        } else {
                            jQuery("#rp_downloading_libraries_images").hide();
                            window.repeatableImagesDownloadAlreadyRunning = false;
                        }

                        jQuery("#rp_downloading_libraries_images .desc").show();

                        jQuery("#rp_downloading_libraries_images .myBar").css({'width':percentDone+'%'});
                    } else {

                    }
                },
                error: function() {

                }
            });
        }

        function updateRepairsAndColorsForModels(rp_token = false) {
            return false;
            let theData;
            var theTime = window._getCurrentTime();
            if(rp_token === true) {
                theData = {
                    action: 'wp_repair_download_libraries_update_repairs',
                    rp_run_token: true
                }
            } else {
                theData = {
                    action: 'wp_repair_download_libraries_update_repairs'
                }
            }
            jQuery.ajax({
                'url': '<?php echo get_site_url(); ?>/wp-admin/admin-ajax.php?time='+theTime,
                'type': 'POST',
                'data': theData,
                success: function(response) {
                    try {
                        response = JSON.parse(response);
                    } catch(e) {

                    }
                    if(response && response.rp_token_error) {
                        if(response.rp_try_after && response.rp_try_after !== false && response.rp_try_after > 0) {
                            window.setTimeout(function(){
                                updateRepairsAndColorsForModels(true);
                            }, window._getNextSafeSchedule(response.rp_try_after));
                        }
                    } else if(response && response.total_models) {
                        let total = response.total_models;
                        let total_done = response.models_done_count;
                        let remaining = total - total_done;

                        let percentRemaining = (remaining * 100) / total;
                        let percentDone = 100 - percentRemaining;

                        jQuery("#rp_creating_repairs_and_colors_for_models .done").text(total_done);
                        jQuery("#rp_creating_repairs_and_colors_for_models .total").text(total);

                        if(percentDone < 100) {
                            jQuery("#rp_creating_repairs_and_colors_for_models").show();
                        } else {
                            jQuery("#rp_creating_repairs_and_colors_for_models").hide();
                        }

                        jQuery("#rp_creating_repairs_and_colors_for_models .desc").show();

                        jQuery("#rp_creating_repairs_and_colors_for_models .myBar").css({'width':percentDone+'%'});
                    }
                    if(response && response.next_request && response.next_request === true) {
                        if(response.rp_try_after && response.rp_try_after !== false && response.rp_try_after > 0) {
                            window.setTimeout(function(){
                                updateRepairsAndColorsForModels(true);
                            }, window._getNextSafeSchedule(response.rp_try_after));
                        }
                    }
                    if(response && response.next_request === false) {
                        window.repairsAndColorsUpdate = false;
                        jQuery("#rp_downloading_libraries_images").show();
                        jQuery("#rp_creating_repairs_and_colors_for_models").hide();
                        repeatableImagesDownload();
                    }
                },
                error: function() {

                }
            });
        }
        jQuery(window).on("load", function(){
            window.setTimeout(function(){
                repeatableImagesDownload();
            }, window._getNextSafeSchedule(1));
        });
    </script>
    <?php
}

add_action('admin_footer', 'rp_javascript_get_next_run_schedule');
add_action('wp_footer', 'rp_javascript_get_next_run_schedule');

if(!defined('WP_REPAIR_DEVELOPMENT_MODE')) {
    add_action('admin_footer','download_libraries_ajax_handler', 12);
    add_action('wp_footer','download_libraries_ajax_handler', 12);
}



function rp_handle_pending_jobs_processing() {
      ?>
      <script type="text/javascript">
          window.repeatablePendingJobsProcessingAR = false;
          function repeatablePendingJobsProcessing(rp_token = false) {
              let theData;
              var theTime = window._getCurrentTime();
              if(rp_token === true) {
                  theData = {
                      action: 'rp_process_pending_background_jobs',
                      rp_run_token: true,
                      rp_process_background_job: 'true'
                  }
              } else {
                  theData = {
                      action: 'rp_process_pending_background_jobs',
                      rp_process_background_job: 'true'
                  }
              }
              window.repeatablePendingJobsProcessingAR = true;
              jQuery.ajax({
                  type: "POST",
                  url: '<?php echo get_site_url(); ?>/wp-admin/admin-ajax.php?time='+theTime,
                  data: theData,
                  success: function(data) {
                      try {
                          data = JSON.parse(data);
                      } catch (e) {
  
                      }
                      if(data && data.rp_token_error) {
                          if(data.rp_try_after && data.rp_try_after !== false && data.rp_try_after > 0) {
                              window.setTimeout(function(){
                                  repeatablePendingJobsProcessing(true);
                              }, window._getNextSafeSchedule(data.rp_try_after));
                          }
                      } else if(data && data.total) {
                          let total = data.total;
                          let remaining = data.remaining;
                          let done_images = total - remaining;
  
                          let percentRemaining = (remaining * 100) / total;
                          let percentDone = 100 - percentRemaining;

                              jQuery("#rp_processing_pending_jobs .done").text(done_images);
                              jQuery("#rp_processing_pending_jobs .total").text(total);

                              if(percentDone < 100) {
                              jQuery("#rp_processing_pending_jobs").show();
                              } else {
                              jQuery("#rp_processing_pending_jobs").hide();
                              }

                              jQuery("#rp_processing_pending_jobs .desc").show();

                              jQuery("#rp_processing_pending_jobs .myBar").css({'width':percentDone+'%'});
  
                          if(percentDone < 100) {
                              if(data.rp_try_after && data.rp_try_after !== false && data.rp_try_after > 0) {
                                  window.setTimeout(function(){
                                      repeatablePendingJobsProcessing(true);
                                  }, window._getNextSafeSchedule(data.rp_try_after));
                              }
                          } else {
                              jQuery("#rp_processing_pending_jobs").hide();
                              window.repeatablePendingJobsProcessingAR = false;
                          }
  
                      } else {
                        jQuery("#rp_processing_pending_jobs").hide();
                        window.repeatablePendingJobsProcessingAR = false;
                      }
                  },
                  error: function() {
                        window.repeatablePendingJobsProcessingAR = false;
                  }
              });
          }
          window._repeatablePendingJobsProcessing = function(rp_token = false) {
              repeatablePendingJobsProcessing(rp_token);
          }
          jQuery(window).on("load", function(){
              window.setTimeout(function(){
                  repeatablePendingJobsProcessing();
              }, window._getNextSafeSchedule(1));
          });
      </script>
      <?php
}
add_action('admin_footer','rp_handle_pending_jobs_processing');
add_action('wp_footer','rp_handle_pending_jobs_processing');


function download_libraries_from_server_ajax_handler() {

      rp_verify_csrf_token_for_ajax('download_libraries');

    global $rpQuery;
    if(!current_user_can( 'manage_wp_repair_plugin' )) {
        echo json_encode(array('error'=>'Something went wrong, please try again later.'));
        exit();
    }
    if(isset($_POST['the_dl_form'])) {
        $dl_class = new RepairPluginPro\Download_Libraries($rpQuery);
        $c_id = isset($_POST['c_id']) ? $_POST['c_id'] : '';
        $b_id = isset($_POST['b_id']) ? $_POST['b_id'] : '';
        $m_id = isset($_POST['m_id']) ? $_POST['m_id'] : '';
        if(empty($c_id) && empty($b_id) && empty($m_id)) {
            echo json_encode(array('error' => 'You have not selected any items to download'));
            exit();
        }
        $data = $dl_class->fetch_repair_base(TRUE);
        $dl_class->save_handler($data);
        exit();
    } else {
        echo json_encode(array('error'=>'Something went wrong, please try again later.'));
        exit();
    }
}

rp_ajax_for_admin('wp_repair_download_libraries','download_libraries_from_server_ajax_handler');

function download_libraries_update_repairs_ajax_handler() {
      echo json_encode(array('success'=>'Repairs & Colors Successfully Added!'));
    exit();

    global $rpQuery;
    $dl_class = new RepairPluginPro\Download_Libraries($rpQuery);
    $dl_class->update_repairs_and_colors_for_models();
    echo json_encode(array('success'=>'Repairs & Colors Successfully Added!'));
    exit();
}

rp_ajax_for_admin('wp_repair_download_libraries_update_repairs','download_libraries_update_repairs_ajax_handler');


function download_languages_from_server_ajax_handler() {

      rp_verify_csrf_token_for_ajax('download_languages');

      global $rpQuery;
      if(!current_user_can( 'manage_wp_repair_plugin' )) {
            echo json_encode(array('error'=>'Something went wrong, please try again later.'));
            exit();
      }
      if(isset($_POST['lang_ids'])) {
            $lang_ids = $_POST['lang_ids'] ?? array();
            if(empty($lang_ids)) {
                  echo json_encode(array('error' => 'You have not selected any items to download'));
                  exit();
            }
            $baselang_class = new RepairPluginPro\Download_Languages($rpQuery);

            $alreadyDownloaded = \rp_get_uq_id_of_all_rs_languages();

            foreach($lang_ids as $key => $lang_id) {
                  if(in_array($lang_id, $alreadyDownloaded)) {
                        unset($lang_ids[$key]);
                  }
            }

            $lang_ids = array_values($lang_ids);

            if( empty( $lang_ids ) ) {

                  echo json_encode(array('success' => 'up_to_date_already'));
                  exit();

            }

            $_POST['lang_uq_ids'] = $lang_ids;

            $response = $baselang_class->fetch_repair_base(TRUE, FALSE);

            $baselang_class->save_handler($response);
            exit();
      } else {
            echo json_encode(array('error'=>'Something went wrong, please try again later.'));
            exit();
      }
}

rp_ajax_for_admin('wp_repair_download_languages','download_languages_from_server_ajax_handler');

function rp_get_fixed_downloading_attributes( $a_uq_ids = array() ) {

      if(empty($a_uq_ids)) {
            return array( array() , array() );
      }

      $main_repairs = array();

      $filtered_a_uq_ids = array();

      foreach( $a_uq_ids as $key => $a_uq_id ) {

            list( $cat_uqid, $repair_uqid, $attr_uqid ) = explode('|', $a_uq_id);

            $filtered_a_uq_ids[] = $cat_uqid . '|' . $attr_uqid;

            $main_repairs[] = $cat_uqid . '|' . $repair_uqid;

      }

      return array( $main_repairs , $filtered_a_uq_ids );

}

function download_repairs_from_server_ajax_handler() {

      rp_verify_csrf_token_for_ajax('download_repairs');

      global $rpQuery;
      if(!current_user_can( 'manage_wp_repair_plugin' )) {
            echo json_encode(array('error'=>'Something went wrong, please try again later.'));
            exit();
      }
      if(isset($_POST['the_dl_form'])) {
            $baserepair_class = new RepairPluginPro\Download_Repairs($rpQuery);

            $r_uq_ids = $_POST['r_uq_ids'] ?? array();

            $a_uq_ids = $_POST['a_uq_ids'] ?? array();

            if(empty($r_uq_ids)) {
                  echo json_encode(array('error' => 'You have not selected any items to download'));
                  exit();
            }

            //rp_get_fixed_downloading_attributes
            list( $main_repairs_of_attrs , $a_uq_ids ) = rp_get_fixed_downloading_attributes( $a_uq_ids );

            $alreadyDownloaded = rp_get_all_dr_uq_id_and_c_uq_id();

            $alreadyDownloadedAttrs = rp_get_all_da_uq_id_and_c_uq_id();

            foreach($r_uq_ids as $key => $r_uqid) {
                  if(in_array($r_uqid, $alreadyDownloaded)) {
                        unset($r_uq_ids[$key]);
                  }
            }

            $r_uq_ids = array_values($r_uq_ids);

            foreach($a_uq_ids as $akey => $a_uqid) {
                  if(in_array($a_uqid, $alreadyDownloadedAttrs)) {
                        unset($a_uq_ids[$akey]);
                  }
            }

            $a_uq_ids = array_values($a_uq_ids);

            if( !empty( $a_uq_ids ) ) {

                  // re add to r_uq_ids
                  foreach( $main_repairs_of_attrs as $adding_attr_repair ) {

                        if( !in_array( $adding_attr_repair, $r_uq_ids ) ) {

                              $r_uq_ids[] = $adding_attr_repair;

                        }

                  }

            }

            if( empty( $r_uq_ids ) && empty( $a_uq_ids ) ) {

                  echo json_encode(array('success' => 'up_to_date_already'));
                  exit();

            }

            $_POST['r_uq_ids'] = $r_uq_ids;

            $_POST['a_uq_ids'] = $a_uq_ids;

            $response = $baserepair_class->fetch_repair_base(TRUE, FALSE);
            
            $baserepair_class->save_handler($response);
            exit();
      } else {
            echo json_encode(array('error'=>'Something went wrong, please try again later.'));
            exit();
      }
}

rp_ajax_for_admin('wp_repair_download_repairs','download_repairs_from_server_ajax_handler');


function rp_add_index_to_column_if_not_exists( $table_name = '', $column_name = '', $prefix = TRUE ){
      global $rpQuery;

      $table_name_org = $table_name;

      if($prefix === TRUE) {

            $table_name = $rpQuery->prefix . $table_name;

      }

      $option_key = 'rp_index_added_'.$table_name_org.'_'.$column_name;

      $has_created_index = rp_get_option($option_key, FALSE);

      if(empty($has_created_index)) {

            $found_index = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.STATISTICS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($table_name)."' AND index_name='".rp_escape_sql($column_name)."';");

            if(empty($found_index)) {

                  rp_add_option($option_key, TRUE);

                  $sql = "ALTER TABLE `$table_name` ADD INDEX(`$column_name`);";

                  $rpQuery->query($sql);

                  return TRUE;
            }

      }

      return FALSE;

}

function wp_repair_permission_init() {
    new RepairPluginPro\Permission();
}

add_action('init', 'wp_repair_permission_init', 999);

function wp_repair_search_box_shortcode_handler() {

      $content = wp_repair_show_alert_if_no_license();
      if(!empty($content)) {
            return $content;
      }

      ob_start();

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/front_end/search_box_shortcode.php';

      $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

      require_once $fileName;

      $content = ob_get_clean();

      return $content;
}

add_shortcode('Repair_Search_Box', 'wp_repair_search_box_shortcode_handler');

function trigger_wp_repair_js_doc_ready() {
      ?>
      <script>
            if(typeof jQuery == 'function') {
                  jQuery(document).trigger('wpRepairReady');
            } else {
                  window.addEventListener("load", function(){
                        if(typeof jQuery == 'function') {
                              jQuery(document).trigger('wpRepairReady');
                        } else {
                              window.setTimeout(function(){
                                    jQuery(document).trigger('wpRepairReady');
                              },300);
                        }
                  });
            }
      </script>
      <?php
}
add_action('wp_footer', 'trigger_wp_repair_js_doc_ready', 9999999999);
add_action('admin_footer', 'trigger_wp_repair_js_doc_ready', 9999999999);

function wp_repair_list_sorting_button($styling = '', $more_class = '') {
      if(!empty($styling)) {
            $style_tag = 'style="'.$styling.'"';
      } else {
            $style_tag = '';
      }
      ?>
      <span <?php echo $style_tag; ?> class="repair-list-sort <?php echo $more_class; ?>">
            <i class="fas fa-caret-up" data-which="up"></i>
            <i class="fas fa-caret-down" data-which="down"></i>
      </span>
      <?php
}






// --------------------------------------------------------- get categories for front-end
function _rp_fe_get_categories() {
      global $rpQuery;

      $category_table = $rpQuery->prefix . "rs_category";

      $c_result = $rpQuery->get_results("SELECT c_name, c_id, c_img FROM $category_table WHERE is_active = '1' ORDER BY c_position ASC");

      if(empty($c_result)) {

            $c_result = array();

      }

      $c_result = rp_sanitize_output($c_result);

      return json_encode(array(

            'status' => TRUE,

            'results' => $c_result,

            'which_page' => 'types'

      ));
}
function rp_fe_get_categories(){

      if(!isset($_POST['via_back_button'])) {

            $load_step = rp_get_front_end_firsttime_load_step();

            if(!empty($load_step)) {

                  $selected = $load_step[0];

                  if($selected === 'repairs') {

                        $_POST['c_name'] = $load_step[1][0];
                        $_POST['b_name'] = $load_step[1][1];
                        $_POST['m_name'] = $load_step[1][2];

                        echo _rp_fe_get_repairs();

                        exit();

                  } else if($selected === 'models') {

                        $_POST['c_name'] = $load_step[1][0];
                        $_POST['b_name'] = $load_step[1][1];

                        echo _rp_fe_get_models();

                        exit();

                  } else if($selected === 'brands') {

                        $_POST['c_name'] = $load_step[1][0];

                        echo _rp_fe_get_brands();

                        exit();

                  } else {

                        echo _rp_fe_get_categories();

                        exit();

                  }

            } else {

                  echo _rp_fe_get_categories();

                  exit();

            }

      } else {

            echo _rp_fe_get_categories();

            exit();

      }

}


rp_ajax_for_public('rp_fe_get_categories','rp_fe_get_categories');


// --------------------------------------------------------- get brands for front-end
function _rp_fe_get_brands() {
      global $rpQuery;

      $brand_table = $rpQuery->prefix . "rs_brand";

      $category_table = $rpQuery->prefix . "rs_category";

      $cname = $_POST['c_name'] ?? '';

      $brands = array();

      if(!empty($cname)) {
            $c_result = $rpQuery->get_results("SELECT * FROM $category_table WHERE c_name = '".rp_escape_sql($cname)."' ORDER BY c_position ASC");

            $c_id = $c_result[0]->c_id;
            
            $brands = $rpQuery->get_results("SELECT b_id, b_name, b_img FROM $brand_table WHERE c_id_fk = '".rp_escape_sql($c_id)."' AND is_active = '1' ORDER BY b_position ASC, b_name ASC");
      }

      if(empty($brands)) {

            $brands = array();

      }

      $brands = rp_sanitize_output($brands);

      $cname = rp_sanitize_output($cname);

      return json_encode(array(

            'status' => TRUE,

            'results' => $brands,

            'which_page' => 'brands',

            'c_name' => $cname
            
      ));
}

function rp_fe_get_brands(){

      if(!isset($_POST['via_back_button'])) {

            $cname = $_POST['c_name'] ?? '';

            $load_step = rp_get_front_end_firsttime_load_step($cname);

            if(!empty($load_step)) {

                  $selected = $load_step[0];

                  if($selected === 'repairs') {

                        $_POST['c_name'] = $load_step[1][0];
                        $_POST['b_name'] = $load_step[1][1];
                        $_POST['m_name'] = $load_step[1][2];

                        echo _rp_fe_get_repairs();

                        exit();

                  } else if($selected === 'models') {

                        $_POST['c_name'] = $load_step[1][0];
                        $_POST['b_name'] = $load_step[1][1];

                        echo _rp_fe_get_models();

                        exit();

                  } else {

                        echo _rp_fe_get_brands();

                        exit();

                  }

            } else {

                  echo _rp_fe_get_brands();

                  exit();

            }

      } else {

            echo _rp_fe_get_brands();

            exit();

      }

}


rp_ajax_for_public('rp_fe_get_brands','rp_fe_get_brands');




// --------------------------------------------------------- get models for front-end
function _rp_fe_get_models() {
      global $rpQuery;

      $brand_table = $rpQuery->prefix . "rs_brand";

      $category_table = $rpQuery->prefix . "rs_category";

      $model_table = $rpQuery->prefix . "rs_model";

      $cname = $_POST['c_name'] ?? '';

      $bname = $_POST['b_name'] ?? '';

      $models = array();

      if(!empty($cname) && !empty($bname)) {
            $c_result = $rpQuery->get_results("SELECT * FROM $category_table WHERE c_name = '".rp_escape_sql($cname)."'");

            $c_id = $c_result[0]->c_id;

            $brands = $rpQuery->get_results("SELECT * FROM $brand_table WHERE b_name = '".rp_escape_sql($bname)."' AND c_id_fk = '".rp_escape_sql($c_id)."'");

            $b_id = $brands[0]->b_id;
            
            $models = $rpQuery->get_results("SELECT m_id, m_name, m_img, m_code, brand_id_fk FROM $model_table WHERE brand_id_fk = '".rp_escape_sql($b_id)."' AND device_type = '".rp_escape_sql($c_id)."' AND is_active = '1' ORDER BY m_position ASC, release_date DESC");
      }

      if(empty($models)) {

            $models = array();

      }

      $models = rp_sanitize_output($models);

      $cname = rp_sanitize_output($cname);

      $bname = rp_sanitize_output($bname);

      return json_encode(array(

            'status' => TRUE,

            'results' => $models,

            'which_page' => 'models',

            'c_name' => $cname,

            'b_name' => $bname
            
      ));
}
function rp_fe_get_models(){

      if(!isset($_POST['via_back_button'])) {

            $cname = $_POST['c_name'] ?? '';

            $bname = $_POST['b_name'] ?? '';

            $load_step = rp_get_front_end_firsttime_load_step($cname, $bname);

            if(!empty($load_step)) {

                  $selected = $load_step[0];

                  if($selected === 'repairs') {

                        $_POST['c_name'] = $load_step[1][0];
                        $_POST['b_name'] = $load_step[1][1];
                        $_POST['m_name'] = $load_step[1][2];

                        echo _rp_fe_get_repairs();

                        exit();

                  } else {

                        echo _rp_fe_get_models();

                        exit();

                  }

            } else {

                  echo _rp_fe_get_models();

                  exit();

            }

      } else {

            echo _rp_fe_get_models();

            exit();

      }

}


rp_ajax_for_public('rp_fe_get_models','rp_fe_get_models');

// ------------------------------------------
function rp_set_coupon_settings( $order_id_fk = 0, $parent_order_id = 0 ) {

      global $rpQuery;

      if( !empty( $parent_order_id ) ) {

            // get combo_dstatus, combo_damount, combo_dtype from rs_orders where o_id = $parent_order_id
            // and then assign to rs_orders where o_id = $order_id_fk

            $rs_orders = $rpQuery->prefix . 'rs_orders';

            $parent_order = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE `o_id` = '".rp_escape_sql($parent_order_id)."'");

            if( !empty( $parent_order ) ) {

                  $rpQuery->query("UPDATE `$rs_orders` SET `combo_dstatus` = '".rp_escape_sql($parent_order->combo_dstatus)."', `combo_damount` = '".rp_escape_sql($parent_order->combo_damount)."', `combo_dtype` = '".rp_escape_sql($parent_order->combo_dtype)."' WHERE `o_id` = '".rp_escape_sql($order_id_fk)."'");

            }

      } else {

            $planing_discount_table = $rpQuery->prefix . "rs_planning_and_discount";

            $planning_discount = $rpQuery->get_results("SELECT * FROM `$planing_discount_table`");

            $planning_discount = rp_fix_combo_discount_row( $planning_discount );

            if( !empty( $planning_discount ) ) {

                  $planning_discount = $planning_discount[0];

                  $combo_dstatus = 0;

                  if( $planning_discount->is_combo_discount == '1' ) {

                        // Combo discount is enabled
                        $combo_dstatus = 1;
                        $combo_damount = $planning_discount->amount;
                        $combo_dtype = 'Fixed';

                  } else if( $planning_discount->is_combo_discount_percentage == '1' ) {

                        // Combo discount is enabled
                        $combo_dstatus = 1;
                        $combo_damount = $planning_discount->amount;
                        $combo_dtype = 'Percentage';

                  } else {

                        // Combo discount is disabled

                  }

                  if( $combo_dstatus == 1 ) {

                        // Update rs_order where o_id = $order_id_fk (combo_dstatus, combo_damount, combo_dtype)

                        $rs_orders = $rpQuery->prefix . 'rs_orders';

                        $rpQuery->query("UPDATE `$rs_orders` SET `combo_dstatus` = '".rp_escape_sql($combo_dstatus)."', `combo_damount` = '".rp_escape_sql($combo_damount)."', `combo_dtype` = '".rp_escape_sql($combo_dtype)."' WHERE `o_id` = '".rp_escape_sql($order_id_fk)."'");

                  }

            }

      }

}

// -------------------------------------------
function _rp_fix_combo_discount_row_order_related( $row = NULL, $o_id_fk = 0 ) {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $order_row = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE `o_id` = '".rp_escape_sql($o_id_fk)."'");

      if( empty( $order_row ) ) {

            return $row;

      }

      if( $order_row->combo_dstatus == '1' ) {

            if( strtolower( trim( $order_row->combo_dtype ) ) == 'fixed' ) {

                  // Fixed Discount

                  $row->is_combo_discount = '1';

                  $row->is_combo_discount_percentage = '0';

            } else {

                  // Percentage Discount

                  $row->is_combo_discount = '0';

                  $row->is_combo_discount_percentage = '1';

            }

            $row->amount = $order_row->combo_damount;

      } else {

            $row->is_combo_discount = '0';

            $row->is_combo_discount_percentage = '0';

      }

      return $row;

}

function _rp_fix_combo_discount_row( $row = NULL, $o_id_fk = 0 ) {

      if( !empty( $o_id_fk ) ) {

            return _rp_fix_combo_discount_row_order_related( $row, $o_id_fk );

      }

      if( $row->is_combo_discount == '1' ) {
            
            $selected_opt = trim(rp_get_option('rp_combo_discount_type', 'Fixed'));

            if( strtolower( $selected_opt ) == 'percentage' ) {

                  // Percentage Discount

                  $row->is_combo_discount = '0';

                  $row->is_combo_discount_percentage = '1';
                  
            } else {

                  // Fixed Discount

                  $row->is_combo_discount = '1';

                  $row->is_combo_discount_percentage = '0';

            }

      } else {

            $row->is_combo_discount = '0';

            $row->is_combo_discount_percentage = '0';
            
      }

      return $row;

}

function rp_fix_combo_discount_row( $planning_discount = NULL, $o_id_fk = 0 ) {

      if( empty( $planning_discount ) ) {

            return $planning_discount;

      }

      if( is_object( $planning_discount ) ) {

            $row = $planning_discount;

            $row = _rp_fix_combo_discount_row( $row, $o_id_fk );

            $planning_discount = $row;

            return $planning_discount;

      } else if( is_array( $planning_discount ) ) {

            $row = $planning_discount[0];

            $row = _rp_fix_combo_discount_row( $row, $o_id_fk );

            $planning_discount[0] = $row;

            return $planning_discount;

      } else {

            return $planning_discount;

      }

}

// --------------------------------------------------------- get repairs for front-end
function _rp_fe_get_repairs() {
      global $wp;
      global $rpQuery;
      global $wp_repair_lang;

      _rp_set_default_tiemzone();

      $m_name = $_POST['m_name'] ?? '';
      $b_name = $_POST['b_name'] ?? '';
      $c_name = '';

      $m_name = trim($m_name);
      $b_name = trim($b_name);

      $brand_table = $rpQuery->prefix. "rs_brand";
      $model_table = $rpQuery->prefix. "rs_model";
      $default_repair_table = $rpQuery->prefix. "rs_default_repair";
      $dr_attr_table = $rpQuery->prefix. "rs_default_repair_attr";
      $r_attr_table  = $rpQuery->prefix. "rs_repair_attr";
      $default_color_table = $rpQuery->prefix. "rs_default_color";
      $modelcolor_table = $rpQuery->prefix. "rs_model_color";
      $rs_repair = $rpQuery->prefix . "rs_repair";
      $rs_localization = $rpQuery->prefix . "rs_localization";
      $rs_coupons = $rpQuery->prefix . "rs_coupons";

      $show_coupon_field = !empty($rpQuery->get_results("SELECT * FROM $rs_coupons WHERE is_active = '1' LIMIT 1;"));

      $all_brands = null;

      if(!empty($b_name)) {
            $all_brands = $rpQuery->get_results("SELECT * FROM $brand_table WHERE b_name = '".rp_escape_sql($b_name)."'");
      }

      if(!empty($all_brands) && !empty($b_name)) {

            $search_brands = array();

            foreach ($all_brands as $key => $value) {
                  $search_brands[] = $value->b_id;
            }

            $search_brands = '('.implode(',', $search_brands).')';

            $models = $rpQuery->get_results("SELECT * FROM $model_table WHERE m_name = '".rp_escape_sql($m_name)."' AND brand_id_fk IN $search_brands");

      } else {

            $models = $rpQuery->get_results("SELECT * FROM $model_table WHERE m_name = '".rp_escape_sql($m_name)."'");

      }

      $m_id = $models[0]->m_id;

      if(!empty($models)) {
            $brand_table = $rpQuery->prefix . "rs_brand";
            $brands = $rpQuery->get_results("SELECT * FROM $brand_table WHERE b_id = '".rp_escape_sql($models[0]->brand_id_fk)."' LIMIT 1;");
            if(!empty($brands) && !empty($models)) {
                  $models[0]->b_name = $brands[0]->b_name;
                  $b_name = $brands[0]->b_name;
            } 
      }

      $c_name = '';

      if(!empty($models)) {
            $category_table = $rpQuery->prefix . "rs_category";
            $category = $rpQuery->get_results("SELECT * FROM $category_table WHERE c_id = '".rp_escape_sql($models[0]->device_type)."' LIMIT 1;");
            if(!empty($category) && !empty($models)) {
                  $models[0]->c_name = $category[0]->c_name;
                  $c_name = $category[0]->c_name;
            }
      }

      $rpQuery->query('START TRANSACTION');
      $repair_ids = $models[0]->repair_ids;
      if($repair_ids != null){
            $all_model_repairs = rp_get_all_model_repair_ids($m_id);
            $repair_idz = explode(',', $repair_ids);
            foreach($repair_idz as $r){
                  $m_repair    = $rpQuery->get_results("SELECT * FROM $default_repair_table WHERE dr_id = '".rp_escape_sql($r)."'");    
                  $r_attr      = $rpQuery->get_results("SELECT * FROM $dr_attr_table WHERE dr_id_fk = '".rp_escape_sql($r)."'");  
                                    
                  $r_name        = $m_repair[0]->dr_name;
                  $r_price       = $m_repair[0]->dr_price;
                  $r_duration    = $m_repair[0]->dr_duration;
                  $r_discription = $m_repair[0]->dr_discription;
                  $r_img         = $m_repair[0]->dr_img;
                  $is_active     = $m_repair[0]->is_active;
                  $r_uq_id       = $m_repair[0]->dr_uq_id;
                                     
                  if(array_search(trim($r_uq_id), $all_model_repairs) === FALSE){
                        $dr_newdata = array(
                            'r_name'=>$r_name,
                            'r_price'=>$r_price,
                            'r_duration'=>$r_duration,
                            'r_discription'=>$r_discription,
                            'r_img'=>$r_img,
                            'is_active'=>$is_active,
                            'r_uq_id'=>$r_uq_id,
                            'm_id_fk'=>$m_id,
                        );
                        $rpQuery->insert($rs_repair, $dr_newdata);
                        $r_id_fk = $rpQuery->insert_id;
                        foreach($r_attr as $a){

                              $ra_name   = $a->da_name;
                              $ra_price  = $a->da_price;
                              $r_id_fk   = $dr_id;
                              $is_active = $a->is_active;
                              $a_uq_id   = $a->da_uq_id;

                              $da_check = $rpQuery->get_results("SELECT * FROM $r_attr_table WHERE a_uq_id = '".rp_escape_sql($a_uq_id)."' AND r_id_fk = '".rp_escape_sql($r_id_fk)."'");
                              if($da_check == null){
                                    $rpQuery->query("INSERT INTO $r_attr_table(a_name,a_price,r_id_fk,a_uq_id) VALUES('".rp_escape_sql($ra_name)."','".rp_escape_sql($ra_price)."','".rp_escape_sql($r_id_fk)."','".rp_escape_sql($a_uq_id)."')");
                              }
                        }
                  }
            }
            $rpQuery->query("UPDATE $model_table SET repair_ids='' WHERE m_id='".rp_escape_sql($m_id)."'"); 
      }
      $color_ids = $models[0]->color_ids;
      if($color_ids != null){
            $all_model_colors = rp_get_all_model_color_ids($m_id);
            $color_idz = explode(',', $color_ids);
            foreach($color_idz as $c){
                  $m_color    = $rpQuery->get_results("SELECT * FROM $default_color_table WHERE dc_id = '".rp_escape_sql($c)."'");      

                  $mc_id      = $m_color[0]->dc_id;
                  $color_name = $m_color[0]->dc_name;
                  $color_code = $m_color[0]->dc_code;
                  $mc_uq_id   = $m_color[0]->dc_uq_id;

                  if(array_search(trim($mc_uq_id), $all_model_colors) === FALSE) {
                        $all_model_colors[] = trim($mc_uq_id);
                        $rpQuery->query("INSERT INTO $modelcolor_table(color_name,color_code,mc_uq_id,m_id_fk) VALUES('".rp_escape_sql($color_name)."','".rp_escape_sql($color_code)."','".rp_escape_sql($mc_uq_id)."','".rp_escape_sql($m_id)."')");
                  }

            }

            $rpQuery->query("UPDATE $model_table SET color_ids='' WHERE m_id='".rp_escape_sql($m_id)."'");
      }     

      $rpQuery->query('COMMIT');

      $model_colors = $rpQuery->get_results("SELECT * FROM $modelcolor_table WHERE m_id_fk = '".rp_escape_sql($m_id)."' ORDER BY mc_position ASC, color_name ASC");
      $count_colors = $rpQuery->get_var("SELECT COUNT(*) FROM $modelcolor_table WHERE m_id_fk = '".rp_escape_sql($m_id)."'");
      $count_repairs = $rpQuery->get_var("SELECT COUNT(*) FROM $rs_repair WHERE m_id_fk = '".rp_escape_sql($m_id)."' AND is_active = '1'");

      $all_repairs = $rpQuery->get_results("SELECT * FROM $rs_repair WHERE m_id_fk = '".rp_escape_sql($m_id)."' AND is_active = '1' ORDER BY r_position ASC, r_id ASC;");

      $limited_repairs = array();
      $default_listing_range = rp_get_option( 'rp_default_listing_index_model_'. $m_id, FALSE );
	if( $default_listing_range === FALSE ){
		$default_listing_range = rp_get_option( 'rp_default_listing_index_'. strval( $models[0]->device_type ), 6 );
	}
      $default_listing_range = (int) $default_listing_range;
      $default_listing_range = rp_round_to_even_number($default_listing_range);
      $repairs = array();

      if(!empty($all_repairs)) {

            $all_repairs = rp_fix_default_repair_sorting($all_repairs, $m_id);

            foreach($all_repairs as $ar_index => $ar_row) {

                  $ar_row->r_name = rp_get_static_translations_of_repair('repair_name', $ar_row->r_uq_id, $ar_row->m_id_fk, 'selected');

                  $ar_row->r_discription = rp_get_static_translations_of_repair('repair_desc', $ar_row->r_uq_id, $ar_row->m_id_fk, 'selected');

                  $ar_row->r_badge = rp_get_static_translations_of_repair('repair_badge', $ar_row->r_uq_id, $ar_row->m_id_fk, 'selected');

                  $all_repairs[$ar_index] = $ar_row;

            }

            $limited_repairs = array_slice($all_repairs, 0, $default_listing_range);
            $limit = $count_repairs - $default_listing_range;
            if(count($all_repairs) > $default_listing_range) {
                  $repairs = array_slice($all_repairs, $default_listing_range);
            }

      }

      // $limited_repairs = $rpQuery->get_results("SELECT * FROM $rs_repair WHERE m_id_fk = '".rp_escape_sql($m_id)."' AND is_active = '1' ORDER BY r_position ASC, r_id ASC LIMIT 6");
      // $limit = $count_repairs - 6;
      // $repairs = $rpQuery->get_results("SELECT * FROM $rs_repair WHERE m_id_fk = '".rp_escape_sql($m_id)."' AND is_active = '1' ORDER BY r_position ASC, r_id ASC limit $limit offset 6");

      $planing_discount_table = $rpQuery->prefix . "rs_planning_and_discount";
      $planning_discount = $rpQuery->get_results("SELECT * FROM $planing_discount_table");
      $planning_discount = rp_fix_combo_discount_row( $planning_discount );
      $tax = $rpQuery->get_results("SELECT * FROM $rs_localization");

      $upsales_models_table = $rpQuery->prefix . "rs_upsales_models";
      $rs_upsales = $rpQuery->prefix . "rs_upsales";
      $upsales = $rpQuery->get_results("SELECT upm.*,up.* FROM $upsales_models_table AS upm JOIN $rs_upsales AS up ON upm.upsales_id_fk = up.upsales_id WHERE upm.m_id_fk = '".rp_escape_sql($m_id)."' AND up.is_active = '1' ORDER BY upsales_position");

      foreach($upsales as $up_index => $up_row) {

            $up_row->us_name = rp_get_static_translations_of_upsale_selected('us_name', $up_row->upsales_id, 'selected');

            $up_row->us_text = rp_get_static_translations_of_upsale_selected('us_text', $up_row->upsales_id, 'selected');

            $up_row->us_info = rp_get_static_translations_of_upsale_selected('us_info', $up_row->upsales_id, 'selected');

            $upsales[$up_index] = $up_row;

      }

      $company_information = $rpQuery->prefix . "rs_company_information";
      $company_results = $rpQuery->get_results("SELECT * FROM $company_information");
      $theme_color = $company_results[0]->company_theme_color;

      $localization = $rpQuery->get_results("SELECT * FROM $rs_localization");
      $decimal = $localization[0]->set_decimals;

      $offers_table = $rpQuery->prefix . "rs_notifications_and_offers";
      $offers = $rpQuery->get_results("SELECT * FROM $offers_table");

      $type_table = $rpQuery->prefix . "rs_notification_types";
      $types = $rpQuery->get_results("SELECT * FROM $type_table");

      $repair_attrs_from_db = array();

      if(!empty($limited_repairs)){ 
            foreach($limited_repairs as $c_limited_key => $r){
                  $n = $r->r_price;
                  $repair_attr = $rpQuery->get_results("SELECT * FROM $r_attr_table WHERE r_id_fk = '".rp_escape_sql($r->r_id)."' AND is_active = '1' ORDER BY ra_position ASC");
                  $repair_attr = rp_fix_default_repair_attr_sorting($repair_attr, $m_id);

                  if(!empty($repair_attr)) {

                        foreach($repair_attr as $ra_index => $ra_row) {
                              
                              $ra_row->a_name = rp_get_static_translations_of_attr('attr_name', $ra_row->a_uq_id, $ra_row->r_id_fk, 'selected');

                              $ra_row->a_desc = rp_get_static_translations_of_attr('attr_desc', $ra_row->a_uq_id, $ra_row->r_id_fk, 'selected');

                              $ra_row->a_badge = rp_get_static_translations_of_attr('attr_badge', $ra_row->a_uq_id, $ra_row->r_id_fk, 'selected');
      
                              $repair_attr[$ra_index] = $ra_row;
      
                        }

                        $repair_attr = apply_filters('rp_filter_all_repair_attr_before_return', $repair_attr, $models[0]->m_uq_id);
                        
                        // only append if not empty
                        if(!empty($repair_attr)) {

                              $repair_attrs_from_db[$r->r_id] = $repair_attr;

                        }
                  }
            }

            $limited_repairs = apply_filters('rp_filter_all_repairs_before_return', $limited_repairs, $models[0]->m_uq_id, $repair_attrs_from_db);
      }

      if(!empty($repairs)){ 
            foreach($repairs as $r_key => $r){
                  $n = $r->r_price;
                  $repair_attr = $rpQuery->get_results("SELECT * FROM $r_attr_table WHERE r_id_fk = '".rp_escape_sql($r->r_id)."' AND is_active = '1' ORDER BY ra_position ASC");
                  $repair_attr = rp_fix_default_repair_attr_sorting($repair_attr, $m_id);
                  if(!empty($repair_attr)) {

                        foreach($repair_attr as $ra_index => $ra_row) {
                              
                              $ra_row->a_name = rp_get_static_translations_of_attr('attr_name', $ra_row->a_uq_id, $ra_row->r_id_fk, 'selected');

                              $ra_row->a_desc = rp_get_static_translations_of_attr('attr_desc', $ra_row->a_uq_id, $ra_row->r_id_fk, 'selected');

                              $ra_row->a_badge = rp_get_static_translations_of_attr('attr_badge', $ra_row->a_uq_id, $ra_row->r_id_fk, 'selected');
      
                              $repair_attr[$ra_index] = $ra_row;
      
                        }

                        $repair_attr = apply_filters('rp_filter_all_repair_attr_before_return', $repair_attr, $models[0]->m_uq_id);

                        // only append if not empty
                        if(!empty($repair_attr)) {

                              $repair_attrs_from_db[$r->r_id] = $repair_attr;

                        }
                  }
            }
            $repairs = apply_filters('rp_filter_all_repairs_before_return', $repairs, $models[0]->m_uq_id, $repair_attrs_from_db);
      }

      $m_name = rp_sanitize_output($m_name);

      $c_name = rp_sanitize_output($c_name);

      $b_name = rp_sanitize_output($b_name);

      $models = rp_sanitize_output($models);

      $model_colors = rp_sanitize_output($model_colors);

      $count_repairs = rp_sanitize_output($count_repairs);

      $limited_repairs = rp_sanitize_output($limited_repairs);

      $repairs = rp_sanitize_output($repairs);

      $localization = rp_sanitize_output($localization);

      $upsales = rp_sanitize_output($upsales);

      $decimal = rp_sanitize_output($decimal);

      $tax = rp_sanitize_output($tax);

      $types = rp_sanitize_output($types);

      if( !empty($repair_attrs_from_db) && is_array($repair_attrs_from_db) ) {
            foreach($repair_attrs_from_db as $rakey => $ravalue) {
                  $repair_attrs_from_db[$rakey] = rp_sanitize_output($ravalue);
            }
      }

      $planning_discount = rp_sanitize_output($planning_discount);

      $show_coupon_field = rp_sanitize_output($show_coupon_field);

      return json_encode(array(

            'status' => TRUE,

            'm_name' => $m_name,

            'c_name' => $c_name,

            'b_name' => $b_name,

            'models' => $models,

            'model_colors' => $model_colors,

            'count_repairs' => $count_repairs,

            'limited_repairs' => $limited_repairs,

            'remaining_repairs' => $repairs,

            'localization' => $localization,

            'upsales' => $upsales,

            'decimal' => $decimal,

            'tax' => $tax,

            'types' => $types,

            'repair_attrs_from_db' => $repair_attrs_from_db,

            'planning_discount' => $planning_discount,

            'show_coupon_field' => $show_coupon_field,

            'which_page' => 'repairs'

      ));
}
function rp_fe_get_repairs(){

      echo _rp_fe_get_repairs();

      exit();

}


rp_ajax_for_public('rp_fe_get_repairs','rp_fe_get_repairs');

// --------------------------------------------------------- get order summary
function rp_render_order_summary( $models = array(), $color_name = '', $_r_id = NULL, $o_id_fk = NULL, $a_id = NULL, $up_id = NULL, $coupon = NULL, $coupon_type = NULL, $coupon_info = NULL, $total_price = 0, $sub_total = 0, $opened = '0', $selected_dm = '', $dm_key = '', $dm_cost = 0, $rpCart = NULL, $combo = 0 ) {

      global $rpQuery;

      global $wp_repair_lang;

      $planning_discount_table = $rpQuery->prefix . "rs_planning_and_discount";

      $planning_discount = $rpQuery->get_results("SELECT * FROM `$planning_discount_table`");

      if( !empty( $o_id_fk ) ) {

            $planning_discount = rp_fix_combo_discount_row( $planning_discount, $o_id_fk );

      } else {

            $planning_discount = rp_fix_combo_discount_row( $planning_discount );

      }

      $localization_table = $rpQuery->prefix . "rs_localization";

      $localization = $rpQuery->get_results("SELECT * FROM $localization_table");

      $tax = $localization[0]->tax;

      $decimal = $localization[0]->set_decimals;

      $repair_table = $rpQuery->prefix . "rs_repair";

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      $rs_upsales = $rpQuery->prefix . "rs_upsales";

      ob_start();

      require_once WP_REPAIR_PLUGIN_PATH . 'html/front_end/finalize_order_summary.php';

      $content = ob_get_contents();

      ob_end_clean();

      return $content;

}

function rp_get_order_summary_markup() {

      rp_verify_csrf_token_for_ajax('frontend');

      global $rpQuery;

      $opened = $_POST['opened'] ?? '0';

      $selected_dm = $_POST['id'] ?? '';

      $model_table = $rpQuery->prefix . "rs_model";

      $category_table = $rpQuery->prefix . "rs_category";

      $brand_table = $rpQuery->prefix . "rs_brand";

      $rs_coupons = $rpQuery->prefix . "rs_coupons";

      $rs_customer = $rpQuery->prefix . "rs_customer";

      $rs_customer_business = $rpQuery->prefix . "rs_customer_business";

      $rs_orders = $rpQuery->prefix . "rs_orders";
      
      if( isset($_SESSION['finalize_booking_arr']) && !empty( $_SESSION['finalize_booking_arr'] ) ) {

            $arr = $_SESSION['finalize_booking_arr'];

            $m_name = $arr['m_name'];

            $color_name = $arr['color_name'];

            $m_id_fk = $arr['m_id_fk'];

            $coupon_id = $arr['coupon_id'];

            $b_id = $arr['b_id'];

            $r_id = $arr['r_id'];

            $a_id = $arr['a_id'];

            $up_id = $arr['upid'];

            $coupon_id = $arr['coupon_id'];

            $get_c_id = $arr['get_c_id'] ?? '';

            $get_uid = $arr['get_uid'] ?? '';

            $get_o_id_fk = $arr['get_o_id_fk'] ?? '';

            $reschedule = $arr['reschedule'] ?? '';

            $models = array();

            if(isset($m_id_fk)) {

                  if(!empty($b_id)) {
      
                        $models = $rpQuery->get_results("SELECT m.* ,b.b_name,c.c_name FROM $model_table AS m JOIN $brand_table AS b ON m.brand_id_fk = b.b_id JOIN $category_table AS c ON m.device_type = c.c_id WHERE m.m_name='".rp_escape_sql($m_name)."' AND m.brand_id_fk = '".rp_escape_sql($b_id)."'");
      
                  } else {
      
                        $models = $rpQuery->get_results("SELECT m.* ,b.b_name,c.c_name FROM $model_table AS m JOIN $brand_table AS b ON m.brand_id_fk = b.b_id JOIN $category_table AS c ON m.device_type = c.c_id WHERE m.m_name='".rp_escape_sql($m_name)."'");
      
                  }
      
                  if( !empty( $models ) ) {
      
                        $m_id_fk = $models[0]->m_id;
      
                        $brandQuery = $rpQuery->get_results("SELECT * FROM $brand_table WHERE b_id = '".rp_escape_sql($models[0]->brand_id_fk)."'");
      
                        if(!empty($brandQuery)) {
      
                              $b_name = $brandQuery[0]->b_name;
      
                        }
      
                  } else {
      
                        $m_id_fk = NULL;
      
                  }
      
            } else {
                  $m_id_fk = NULL;
            }
      
            if( empty( $m_id_fk ) ) {
      
                  echo json_encode(array(
      
                        'status' => false,
      
                        'rp_validation_error' => 'Invalid input provided.'
      
                  ));

                  exit;
      
            }

            $coupon = 0;

            if( isset( $coupon_id ) && !empty( $coupon_id ) ) {

                  $coupon_detail = $rpQuery->get_row("SELECT * FROM `$rs_coupons` WHERE coupons_id = '".rp_escape_sql($coupon_id)."' LIMIT 1;");

                  if( empty( $coupon_detail ) ){

                        $coupon_id = '';

                  }

            } else {

                  $coupon_id = '';

            }

            if(isset($get_c_id) && !empty($get_c_id)) {
                  $_GET['c_id'] = $get_c_id;
            }
      
            if(isset($get_uid) && !empty($get_uid)) {
                  $_GET['uid'] = $get_uid;
            }
      
            if(isset($reschedule) && !empty($reschedule)) {
                  $_GET['reschedule'] = $reschedule;
            }
      
            $o_id_fk = 0;
      
            if(isset($_GET['c_id']) && isset($_GET['uid'])) {
                  if(!empty($_GET['c_id']) && !empty($_GET['uid'])) {
                        if(trim(strtolower($_GET['uid'])) === substr(md5($_GET['c_id']), 0, 8)) {
                              $_customer_details = $rpQuery->get_results("SELECT * FROM `$rs_customer` WHERE customer_id = '".rp_escape_sql($_GET['c_id'])."' LIMIT 1;");
      
                              if(!empty($_customer_details)) {

                                    $o_id_fk = $get_o_id_fk ?? 0;

                                    if(isset($_GET['reschedule']) && !empty($_GET['reschedule']) && !empty($o_id_fk)) {
                                          // select notes from rs_orders table
                                          $rp_order_details = $rpQuery->get_results("SELECT * FROM `$rs_orders` WHERE o_id = '".rp_escape_sql($o_id_fk)."' LIMIT 1;");
      
                                    }
                              }
                        }
                  }
            }

            $delivery_method_key = '';

            $delivery_method_cost = 0;

            if( !empty( $o_id_fk ) ) {

                  rp_set_order_related_language($o_id_fk);
      
                  $_GET['o_id_fk'] = $o_id_fk;

                  $_POST['selected_dm_id'] = $selected_dm;
      
                  list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $o_id_fk );
      
                  $combo        = $calculations['combo_discount'];
      
                  $coupon       = $calculations['coupon_amount'];
      
                  $total_price  = $calculations['total'];
      
                  $sub_total  = $calculations['sub_total'];

                  $delivery_method_key = $rpCart->delivery_method_translation_key();

                  $delivery_method_cost = $rpCart->delivery_method_cost();
      
                  $coupon_info = $rpCart->get_coupon_info();
      
                  if( !empty($coupon_info) ) {
      
                        $coupon_type = $coupon_info->coupons_type;
                        
                  }
      
                  $rid = array();
      
                  $aid = array();
      
                  $upid = array();
      
                  foreach( $rpCart->repairs as $cartRow ) {
      
                        $rid[] = $cartRow->r_id;
      
                  }
      
                  foreach( $rpCart->repair_attrs as $cartRow ) {
      
                        $rid[] = $cartRow->r_id_fk;
      
                        $aid[] = $cartRow->a_id;
      
                  }
      
                  foreach( $rpCart->upsales as $cartRow ) {
      
                        $upid[] = $cartRow->upsales_id;
      
                  }
      
                  $prepared_post_data = $rpCart->prepare_post_data( $rid, $upid );
      
                  list( $rid, $array_name, $array_price, $aid, $upid ) = $prepared_post_data;
      
            } else if( $m_id_fk !== NULL ) {
      
                  $rpCart       = new RepairPluginPro\RP_Cart( $m_id_fk );
      
                  $rid          = $r_id ?? array();
      
                  foreach ($rid as $repair_id) {
      
                        $rpCart->add_repair( ((int) $repair_id) );
      
                  }
      
                  $aid          = $a_id ?? array();
      
                  foreach ($aid as $attr_id) {
      
                        $rpCart->add_repair_attr( ((int) $attr_id) );
      
                  }
      
                  $upid         = $up_id ?? array();
      
                  foreach ($upid as $upsale_id) {
      
                        $rpCart->add_upsale( ((int) $upsale_id) );
      
                  }
                  
                  if( !empty( $selected_dm ) ) {

                        $rpCart->add_delivery_method( $selected_dm );

                  }
      
                  if( !empty( $coupon_id ) ) {
      
                        $rpCart->add_coupon( ((int) $coupon_id) );
      
                  }
      
                  $calculations = $rpCart->calculate();
      
                  $combo        = $calculations['combo_discount'];
      
                  $coupon       = $calculations['coupon_amount'];
      
                  $total_price  = $calculations['total'];
      
                  $sub_total  = $calculations['sub_total'];
     
                  $delivery_method_key = $rpCart->delivery_method_translation_key();

                  $delivery_method_cost = $rpCart->delivery_method_cost();

                  $coupon_info = $rpCart->get_coupon_info();
      
                  if( !empty($coupon_info) ) {
      
                        $coupon_type = $coupon_info->coupons_type;
      
                  }
      
                  $prepared_post_data = $rpCart->prepare_post_data( $rid, $upid );
      
                  list( $r_id, $array_name, $array_price, $a_id, $up_id ) = $prepared_post_data;
      
            } else {
      
            }

            $order_summary = rp_render_order_summary(
                  $models, 
                  $color_name ?? '',
                  $r_id ?? NULL,
                  $o_id_fk ?? NULL,
                  $a_id ?? NULL,
                  $up_id ?? NULL,
                  $coupon ?? NULL,
                  $coupon_type ?? NULL,
                  $coupon_info ?? NULL,
                  $total_price ?? 0,
                  $sub_total ?? 0,
                  $opened ?? '0',
                  $selected_dm ?? '',
                  $delivery_method_key ?? '',
                  $delivery_method_cost ?? 0,
                  $rpCart,
                  $combo
            );

            echo json_encode(array(
                  
                  'status' => true,
                  
                  'order_summary' => $order_summary,

                  'total_price' => $total_price

            ));

            exit;

      } else {

            echo json_encode(array(
      
                  'status' => false,

                  'rp_validation_error' => 'Something went wrong, please refresh the page and try agian.'

            ));

            exit;

      }

}

rp_ajax_for_public('rp_get_order_summary_markup','rp_get_order_summary_markup');

// --------------------------------------------------------- get finalize repairs for front-end
function _rp_fe_finalize_repairs(){

      global $rpQuery;
      global $wp_repair_lang;

      $m_name = $_POST['m_name'] ?? '';
      $b_name = '';
      $b_id = NULL;

      $company_information = $rpQuery->prefix . "rs_company_information";
      $company_results = $rpQuery->get_results("SELECT * FROM $company_information");
      $theme_color = $company_results[0]->company_theme_color;

      $planning_discount_table = $rpQuery->prefix . "rs_planning_and_discount";
      $planning_discount = $rpQuery->get_results("SELECT * FROM $planning_discount_table");
      $planning_discount = rp_fix_combo_discount_row( $planning_discount );

      $company_location = $rpQuery->prefix . "rs_company_location";
      $location = $rpQuery->get_results("SELECT * FROM $company_location");

      $rp_ship_device_location_id = rp_get_option('wp_repair_ship_device_location_id', 1);
      $ship_device_location = $rpQuery->get_results("SELECT * FROM $company_location WHERE `location_id` = '".rp_escape_sql($rp_ship_device_location_id)."'");
      if(empty($ship_device_location)) {
            $ship_device_location = $location;
      }

      $localization_table = $rpQuery->prefix . "rs_localization";
      $localization = $rpQuery->get_results("SELECT * FROM $localization_table");
      $tax = $localization[0]->tax;
      $decimal = $localization[0]->set_decimals;

      $offer_table = $rpQuery->prefix . "rs_notifications_and_offers";
      $offers = $rpQuery->get_results("SELECT * FROM $offer_table");

      $type_table = $rpQuery->prefix . "rs_notification_types";
      $type = $rpQuery->get_results("SELECT * FROM $type_table");

      $uri_path = $_SERVER['REQUEST_URI'];
      $uri_segments = explode('/', $uri_path);
      $segment = count($uri_segments);

      $pdf_offer = '';

      if (isset($_POST['finalize_booking'])) {
            $color_name   = $_POST['color_name'] ?? '';
            $total_price  = $_POST['total_price'] ?? '';
            $sub_total    = $_POST['sub_total'] ?? '';
            $pdf_offer    = $_POST['pdf_offer'] ?? '';
            $m_id_fk      = $_POST['m_id_fk'] ?? '';
            $c_id         = $_POST['c_id'] ?? '';
            $b_id         = $_POST['b_id'] ?? '';
            $r_id         = $_POST['rid'] ?? array();
            $a_id         = $_POST['aid'] ?? array();
            $coupon_id       = $_POST['coupon_id'] ?? '';
            $up_id        = $_POST['upid'] ?? array();
      }else if($segment > 6){
            global $rpQuery;
            $pdf_offer = '1';
            $segment = end($uri_segments);

            $secondKey = count($uri_segments) -2;
            $thirdKey = count($uri_segments) -3;
            $forthKey = count($uri_segments) -4;
            $fifthKey = count($uri_segments) -5;
            $sixKey = count($uri_segments) -6;
            $sevenKey = count($uri_segments) -7;
            $eightKey = count($uri_segments) -8;
            $nineKey = count($uri_segments) -9;

            $sub_total = explode('=', $segment)[1];

            $second_segment = $uri_segments[$secondKey];
            $coupon_id = explode('=', $second_segment)[1];

            $third_segment = $uri_segments[$thirdKey];
            $total_price = explode('=', $third_segment)[1];

            $forth_segment = $uri_segments[$forthKey];
            $r_id = explode('=', $forth_segment)[1];
            $r_id = explode(',', $r_id);

            $fifth_segment = $uri_segments[$fifthKey];
            $a_id = explode('=', $fifth_segment)[1];
            $a_id = explode(',', $a_id);

            $six_segment = $uri_segments[$sixKey];
            $up_id = explode('=', $six_segment)[1];
            $up_id = explode(',', $up_id);

            $seven_segment = $uri_segments[$sevenKey];
            $color = explode('=', $seven_segment)[1];
            $color = str_replace('-slash-', '/', $color);
            $color_name = str_replace('-',' ',$color);

            $eight_segment = $uri_segments[$eightKey];
            $m_name = explode('=', $eight_segment)[1];
            $m_name = str_replace('-',' ',$m_name);

            $nine_segment = $uri_segments[$nineKey] ?? '';
            if(!empty($nine_segment)) {
                  $b_id = explode('=', $nine_segment)[1];
            } else {
                  $b_id = '';
            }

            $m_id_fk = TRUE;
      }

      if($b_id === 'shortcode_finalize_order' || $b_id === 'finalize') {
            $b_id = '';
      }

      $model_table = $rpQuery->prefix . "rs_model";
      $category_table = $rpQuery->prefix . "rs_category";
      $brand_table = $rpQuery->prefix . "rs_brand";
      $repair_table = $rpQuery->prefix . "rs_repair";
      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";
      $rs_customer = $rpQuery->prefix . "rs_customer";
      $rs_customer_business = $rpQuery->prefix . "rs_customer_business";
      $rs_orders = $rpQuery->prefix . "rs_orders";
      $rs_order_repairs = $rpQuery->prefix . "rs_order_repairs";
      $rs_order_repair_attr = $rpQuery->prefix . "rs_order_repair_attr";
      $rs_order_upsales = $rpQuery->prefix . "rs_order_upsales";
      $rs_upsales = $rpQuery->prefix . "rs_upsales";
      $rs_coupons = $rpQuery->prefix . "rs_coupons";
      $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

      if(isset($m_id_fk)) {

            if(!empty($b_id)) {

                  $models = $rpQuery->get_results("SELECT m.* ,b.b_name,c.c_name FROM $model_table AS m JOIN $brand_table AS b ON m.brand_id_fk = b.b_id JOIN $category_table AS c ON m.device_type = c.c_id WHERE m.m_name='".rp_escape_sql($m_name)."' AND m.brand_id_fk = '".rp_escape_sql($b_id)."'");

            } else {

                  $models = $rpQuery->get_results("SELECT m.* ,b.b_name,c.c_name FROM $model_table AS m JOIN $brand_table AS b ON m.brand_id_fk = b.b_id JOIN $category_table AS c ON m.device_type = c.c_id WHERE m.m_name='".rp_escape_sql($m_name)."'");

            }

            if( !empty( $models ) ) {

                  $m_id_fk = $models[0]->m_id;

                  $brandQuery = $rpQuery->get_results("SELECT * FROM $brand_table WHERE b_id = '".rp_escape_sql($models[0]->brand_id_fk)."'");

                  if(!empty($brandQuery)) {

                        $b_name = $brandQuery[0]->b_name;

                  }

            } else {

                  $m_id_fk = NULL;

            }

      } else {
            $m_id_fk = NULL;
      }

      if( empty( $m_id_fk ) ) {

            return json_encode(array(

                  'status' => false,

                  'rp_validation_error' => 'Invalid input provided.'

            ));

      }

      $coupon = 0;

      if( isset( $coupon_id ) && !empty( $coupon_id ) ) {

            $coupon_detail = $rpQuery->get_row("SELECT * FROM $rs_coupons WHERE coupons_id = '".rp_escape_sql($coupon_id)."' LIMIT 1;");

            if( empty( $coupon_detail ) ){

                  $coupon_id = '';

            }

      } else {

            $coupon_id = '';

      }

      $_customer_name = '';
      $_customer_phone = '';
      $_customer_email = '';
      $_customer_notes = '';
      $_customer_business_name = '';

      if(isset($_POST['get_c_id'])) {
            $_GET['c_id'] = $_POST['get_c_id'];
      }

      if(isset($_POST['get_uid'])) {
            $_GET['uid'] = $_POST['get_uid'];
      }

      if(isset($_POST['rp_reschedule'])) {
            $_GET['reschedule'] = $_POST['rp_reschedule'];
      }

      $o_id_fk = 0;

      if(isset($_GET['c_id']) && isset($_GET['uid'])) {
            if(!empty($_GET['c_id']) && !empty($_GET['uid'])) {
                  if(trim(strtolower($_GET['uid'])) === substr(md5($_GET['c_id']), 0, 8)) {
                        $_customer_details = $rpQuery->get_results("SELECT * FROM `$rs_customer` WHERE customer_id = '".rp_escape_sql($_GET['c_id'])."' LIMIT 1;");

                        if(!empty($_customer_details)) {
                              $_customer_name = $_customer_details[0]->customer_name;
                              $_customer_phone = $_customer_details[0]->customer_phone;
                              $_customer_email = $_customer_details[0]->customer_email;
                              $o_id_fk = $_POST['get_o_id_fk'] ?? 0;
                              if(isset($_GET['reschedule']) && !empty($_GET['reschedule']) && !empty($o_id_fk)) {
                                    // select notes from rs_orders table
                                    $rp_order_details = $rpQuery->get_results("SELECT * FROM `$rs_orders` WHERE o_id = '".rp_escape_sql($o_id_fk)."' LIMIT 1;");

                                    if( !empty( $rp_order_details ) ) {      
                                          $_customer_notes = $rp_order_details[0]->notes;
                                    }

                              }

                              $rs_customer_business = $rpQuery->prefix . "rs_customer_business";

                              // select * from rs_customer_business table where customer_id_fk = $_GET['c_id']

                              $rp_customer_business_details = $rpQuery->get_results("SELECT * FROM `$rs_customer_business` WHERE customer_id_fk = '".rp_escape_sql($_GET['c_id'])."' LIMIT 1;");

                              if( !empty( $rp_customer_business_details ) ) { 
                                    $_customer_business_name = $rp_customer_business_details[0]->cb_name;
                              }
                        }
                  }
            }
      }

      $delivery_method_key = '';

      $delivery_method_cost = 0;

      if( !empty( $o_id_fk ) ) {

            $planning_discount = rp_fix_combo_discount_row( $planning_discount, $o_id_fk );

            rp_set_order_related_language($o_id_fk);

            $_GET['o_id_fk'] = $o_id_fk;

            list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $o_id_fk );

            $combo        = $calculations['combo_discount'];

            $coupon       = $calculations['coupon_amount'];

            $total_price  = $calculations['total'];

            $sub_total  = $calculations['sub_total'];

            $delivery_method_key = $rpCart->delivery_method_translation_key();

            $delivery_method_cost = $rpCart->delivery_method_cost();

            $coupon_info = $rpCart->get_coupon_info();

            if( !empty($coupon_info) ) {

                  $coupon_type = $coupon_info->coupons_type;
                  
            }

            $rid = array();

            $aid = array();

            $upid = array();

            foreach( $rpCart->repairs as $cartRow ) {

                  $rid[] = $cartRow->r_id;

            }

            foreach( $rpCart->repair_attrs as $cartRow ) {

                  $rid[] = $cartRow->r_id_fk;

                  $aid[] = $cartRow->a_id;

            }

            foreach( $rpCart->upsales as $cartRow ) {

                  $upid[] = $cartRow->upsales_id;

            }

            $prepared_post_data = $rpCart->prepare_post_data( $rid, $upid );

            list( $rid, $array_name, $array_price, $aid, $upid ) = $prepared_post_data;

      } else if( $m_id_fk !== NULL ) {

            $rpCart       = new RepairPluginPro\RP_Cart( $m_id_fk );

            $rid          = $r_id ?? array();

            foreach ($rid as $repair_id) {

                  $rpCart->add_repair( ((int) $repair_id) );

            }

            $aid          = $a_id ?? array();

            foreach ($aid as $attr_id) {

                  $rpCart->add_repair_attr( ((int) $attr_id) );

            }

            $upid         = $up_id ?? array();

            foreach ($upid as $upsale_id) {

                  $rpCart->add_upsale( ((int) $upsale_id) );

            }

            if( !empty( $coupon_id ) ) {

                  $rpCart->add_coupon( ((int) $coupon_id) );

            }

            $calculations = $rpCart->calculate();

            $combo        = $calculations['combo_discount'];

            $coupon       = $calculations['coupon_amount'];

            $total_price  = $calculations['total'];

            $sub_total  = $calculations['sub_total'];

            $coupon_info = $rpCart->get_coupon_info();

            if( !empty($coupon_info) ) {

                  $coupon_type = $coupon_info->coupons_type;

            }

            $prepared_post_data = $rpCart->prepare_post_data( $rid, $upid );

            list( $r_id, $array_name, $array_price, $a_id, $up_id ) = $prepared_post_data;

      } else {

      }

      RepairPluginPro\RP_Session_Handler::start();

      $_SESSION['finalize_booking_arr'] = array();

      $_SESSION['finalize_booking_arr']['m_name'] = $m_name;
      $_SESSION['finalize_booking_arr']['b_id'] = $b_id;
      $_SESSION['finalize_booking_arr']['color_name'] = $color_name;
      $_SESSION['finalize_booking_arr']['total_price'] = $total_price;
      $_SESSION['finalize_booking_arr']['sub_total'] = $sub_total;
      $_SESSION['finalize_booking_arr']['pdf_offer'] = $pdf_offer;
      $_SESSION['finalize_booking_arr']['m_id_fk'] = $m_id_fk;
      $_SESSION['finalize_booking_arr']['r_id'] = $r_id;
      $_SESSION['finalize_booking_arr']['a_id'] = $a_id;
      $_SESSION['finalize_booking_arr']['upid'] = $up_id;
      $_SESSION['finalize_booking_arr']['coupon_id'] = $coupon_id;

      if(isset($_GET['c_id']) && isset($_GET['uid'])) {
            $_SESSION['finalize_booking_arr']['get_c_id'] = $_GET['c_id'];
            $_SESSION['finalize_booking_arr']['get_uid'] = $_GET['uid'];
      }

      if( isset( $_GET['o_id_fk'] ) ) {

            $_SESSION['finalize_booking_arr']['get_o_id_fk'] = $_GET['o_id_fk'];

      }

      if( isset( $_GET['reschedule'] ) ) {

            $_SESSION['finalize_booking_arr']['reschedule'] = $_GET['reschedule'];

      }

      RepairPluginPro\RP_Session_Handler::close();

      ob_start();
      require_once WP_REPAIR_PLUGIN_PATH.'html/front_end/finalize_repairs_html.php';
      $content = ob_get_contents();
      ob_end_clean();

      return json_encode(array(
            'localization' => $localization,
            'planning_discount' => $planning_discount,
            'rp_total_price' => $total_price,
            'template' => $content,
            'status' => true,
            'm_name' => $m_name,
            'b_name' => $b_name
      ));

}


function rp_fe_finalize_repairs(){

      echo _rp_fe_finalize_repairs();
      
      exit();

}


rp_ajax_for_public('rp_fe_finalize_repairs','rp_fe_finalize_repairs');

function rp_get_delivery_method_translation_key( $delivery_method_name = '' ) {

      $translation_key = '';

      // remove one or more space with single space
      $delivery_method_name = preg_replace('/\s+/', ' ', $delivery_method_name);

      // trim
      $delivery_method_name = trim( $delivery_method_name );

      // strtolower
      $delivery_method_name = strtolower( $delivery_method_name );

      // replace space or - with _
      $delivery_method_name = preg_replace('/\s+/', '_', $delivery_method_name);

      $delivery_method_name = preg_replace('/-+/', '_', $delivery_method_name);

      if( $delivery_method_name == 'come_our_store' || $delivery_method_name == 'come_by_our_store' ) {

            $translation_key = 'come_by_our_store';

      }

      if( $delivery_method_name == 'ship_device' || $delivery_method_name == 'ship_your_device' ) {

            $translation_key = 'ship_device';

      }

      if( $delivery_method_name == 'pickup_service' ) {

            $translation_key = 'let_us_pickup_your_device';

      }

      if( $delivery_method_name == 'repair_on_location' || $delivery_method_name == 'repair_on_your_location' ) {

            $translation_key = 'repair_on_your_location';

      }

      return $translation_key;

}


// --------------------------------------------------------- get finalize repairs for front-end

function rp_fe_finalize_booking_reschedule() {

      global $wp_repair_selected_language;

      global $rpQuery;

      $company_location = $rpQuery->prefix . "rs_company_location";
      $location = $rpQuery->get_results("SELECT * FROM $company_location");

      $rp_ship_device_location_id = rp_get_option('wp_repair_ship_device_location_id', 1);
      $ship_device_location = $rpQuery->get_results("SELECT * FROM $company_location WHERE `location_id` = '".rp_escape_sql($rp_ship_device_location_id)."'");
      if(empty($ship_device_location)) {
            $ship_device_location = $location;
      }

      $type_table = $rpQuery->prefix . "rs_notification_types";
      $type = $rpQuery->get_results("SELECT * FROM $type_table");

      $model_table = $rpQuery->prefix . "rs_model";
      $category_table = $rpQuery->prefix . "rs_category";
      $brand_table = $rpQuery->prefix . "rs_brand";
      $repair_table = $rpQuery->prefix . "rs_repair";
      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";
      $rs_customer = $rpQuery->prefix . "rs_customer";
      $rs_customer_business = $rpQuery->prefix . "rs_customer_business";
      $rs_orders = $rpQuery->prefix . "rs_orders";
      $rs_order_repairs = $rpQuery->prefix . "rs_order_repairs";
      $rs_order_repair_attr = $rpQuery->prefix . "rs_order_repair_attr";
      $rs_order_upsales = $rpQuery->prefix . "rs_order_upsales";
      $rs_upsales = $rpQuery->prefix . "rs_upsales";
      $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

      $m_id_fk = $_POST['m_id_fk'] ?? 0;

      $model_info = $rpQuery->get_row("SELECT * FROM $model_table WHERE m_id = ':m_id_fk' LIMIT 1;", array( 'm_id_fk' => $m_id_fk ));

      if( empty( $model_info ) ) {

            echo json_encode(array(

                  'status' => false,

                  'rp_validation_error' => 'Invalid input provided.'

            ));

            exit;

      }


      if (isset($_POST['confirm_booking'])) {
            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'ship device') {
                  if( !rp_is_enabled_ship_device_location() ) {
                        $_POST['location_id'] = $ship_device_location[0]->location_id;
                  }
                  $_POST['date'] = date('d-m-Y');
                  $_POST['time'] = '';
            }
            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'pickup service') {
                  $_POST['location_id'] = $location[0]->location_id;
            }
            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'repair on location') {
                  $_POST['location_id'] = $location[0]->location_id;
            }
            $user_name     = $_POST['user_name'];
            $business_name = $_POST['business_name'];
            $user_email    = $_POST['user_email'];
            $user_phone    = $_POST['user_phone'];
            $color         = $_POST['color_name'];
            $notes        = $_POST['notes'] ?? '';
            $time         = $_POST['time'] ?? '';
            $date         = $_POST['date'] ?? '';
            $d_method     = $_POST['d_method'] ?? '';
            $location_id  = $_POST['location_id'] ?? '';
            $pdf_offer    = $_POST['pdf_offer'] ?? '';
            
            $_POST['get_o_id_fk'] = $_POST['get_o_id_fk'] ?? 0;

            $c_order_info = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE o_id = ':get_o_id_fk' LIMIT 1;", array( 'get_o_id_fk' => $_POST['get_o_id_fk'] ));

            if( !empty($c_order_info) ) {

                  $customer_id_fk = $c_order_info->customer_id_fk;

            } else {

                  $customer_id_fk = 0;

            }

            $nearest_location_id = rp_get_nearest_location_for_customer( $customer_id_fk );

            if( !empty( $nearest_location_id ) ) {

                  $_POST['location_id'] = $nearest_location_id;

                  $location_id = $nearest_location_id;

            }

            $c_business_info = $rpQuery->get_row("SELECT * FROM `$rs_customer_business` WHERE customer_id_fk = ':customer_id_fk' LIMIT 1;", array( 'customer_id_fk' => $customer_id_fk ));

            if( !empty($c_business_info) ) {

                  $business_name = $c_business_info->cb_name;

            } else {

                  $business_name = '';

            }

            $m_id_fk      = $_POST['m_id_fk'] ?? 0;
            $sub_total    = $_POST['sub_total'] ?? '';
            $total_price  = $_POST['total_price'] ?? '';
            $coupon_id_fk = $_POST['coupons_id'] ?? 0;
            $combo        = $_POST['combo'] ?? '';
            $r_id         = $_POST['r_id'] ?? array();
            $a_id         = $_POST['a_id'] ?? array();
            $up_id        = $_POST['up_id'] ?? array();

            $dm_cost = 0;

            if( isset( $_POST['get_o_id_fk'] ) && !empty( $_POST['get_o_id_fk'] ) ) {

                  list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $_POST['get_o_id_fk'] );

                  $combo        = $calculations['combo_discount'];

                  $coupon       = $calculations['coupon_amount'];

                  $total_price  = $calculations['total'];

                  $sub_total  = $calculations['sub_total'];

                  $dm_cost = $rpCart->delivery_method_cost();

                  $r_id = array();

                  $a_id = array();

                  $up_id = array();

                  foreach( $rpCart->repairs as $cartRow ) {

                        $r_id[] = $cartRow->r_id;

                  }

                  foreach( $rpCart->repair_attrs as $cartRow ) {

                        $r_id[] = $cartRow->r_id_fk;

                        $a_id[] = $cartRow->a_id;

                  }

                  foreach( $rpCart->upsales as $cartRow ) {

                        $up_id[] = $cartRow->upsales_id;

                  }

                  $prepared_post_data = $rpCart->prepare_post_data( $r_id, $up_id );

                  list( $r_id, $array_name, $array_price, $a_id, $up_id ) = $prepared_post_data;

            } else {

                  $rpCart       = new RepairPluginPro\RP_Cart( $m_id_fk );

                  $r_id          = $_POST['r_id'] ?? array();

                  foreach ($r_id as $repair_id) {

                    $rpCart->add_repair( ((int) $repair_id) );

                  }

                  $a_id          = $_POST['a_id'] ?? array();

                  foreach ($a_id as $attr_id) {

                    $rpCart->add_repair_attr( ((int) $attr_id) );

                  }

                  $up_id         = $_POST['up_id'] ?? array();

                  foreach ($up_id as $upsale_id) {

                    $rpCart->add_upsale( ((int) $upsale_id) );

                  }

                  $rpCart->add_delivery_method( $d_method );

                  $rpCart->add_coupon( ((int) $coupon_id_fk) );

                  $calculations = $rpCart->calculate();

                  $combo        = $calculations['combo_discount'];

                  $coupon       = $calculations['coupon_amount'];

                  $total_price  = $calculations['total'];

                  $sub_total  = $calculations['sub_total'];

                  $dm_cost = $rpCart->delivery_method_cost();

                  $prepared_post_data = $rpCart->prepare_post_data( $r_id, $up_id );

                  list( $r_id, $array_name, $array_price, $a_id, $up_id ) = $prepared_post_data;

            }
            
            $status = $type[2]->is_active;
            if($status == '1'){
                  $status = '3';
            }else{
                  $status = '1';
            }

            $order_id_fk = $_POST['get_o_id_fk'] ?? 0;

            $created_time = date('Y-m-d H:i:s');

            // update location_id_fk, date, time column in orders table
            $rpQuery->query("UPDATE $rs_orders SET `location_id_fk` = '".rp_escape_sql($location_id)."', `delivery_date` = '".rp_escape_sql($date)."', `delivery_time` = '".rp_escape_sql($time)."',  `reminder` = '0', `aftersales` = '0' WHERE o_id = '".rp_escape_sql($order_id_fk)."'");

            if($r_id){
                  $r_id_str = join(',', $r_id);
            } else {
                  $r_id_str = '';
            }
            if($a_id){
                  $a_id_str = join(',', $a_id);
            } else {
                  $a_id_str = '';
            }
            $up_id = $_POST['up_id'] ?? '';
            if($up_id){
                  $up_id_str = join(',', $up_id);
            } else {
                  $up_id_str = '';
            }

            $rs_timeslot = $rpQuery->prefix . 'rs_timeslot';

            $rpQuery->query("DELETE FROM `$rs_timeslot` WHERE o_id = '".rp_escape_sql($order_id_fk)."' LIMIT 1;");

            $rs_timeslot_dm = $rpQuery->prefix . 'rs_timeslot_dm';

            $rpQuery->query("DELETE FROM `$rs_timeslot_dm` WHERE o_id = '".rp_escape_sql($order_id_fk)."' LIMIT 1;");

            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'come by our store') {

                  rp_book_available_slot(array(
                        'loc_id' => $location_id,
                        'a_date' => $date,
                        'a_time' => $time,
                        'o_id' => $order_id_fk
                  ));

            }

            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'pickup service') {

                  rp_book_available_slot_new_dm(array(
                        'method_id' => 'ps',
                        'a_date' => $date,
                        'a_time' => $time,
                        'o_id' => $order_id_fk
                  ));

            }

            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'repair on location') {

                  rp_book_available_slot_new_dm(array(
                        'method_id' => 'rol',
                        'a_date' => $date,
                        'a_time' => $time,
                        'o_id' => $order_id_fk
                  ));

            }

            RepairPluginPro\RP_Session_Handler::start();

            $_SESSION['o_id'] = $order_id_fk;

            unset($_SESSION['finalize_booking_arr']);

            RepairPluginPro\RP_Session_Handler::close();

      }

      if( !empty( $order_id_fk ?? 0 ) ) {

            rp_trigger_outgoing_webhook( 'scheduled_appointment_updated', $order_id_fk );

      }

      rp_fe_thank_you_page();

      exit();

}

function rp_fe_finalize_booking(){

      rp_verify_csrf_token_for_ajax('frontend');

      if( rp_verify_google_recaptcha() === FALSE ) {

            echo json_encode(
                  array(
                        'status'=> false, 
                        'rp_validation_error' => 'Form submission blocked by Google ReCaptcha. Please refresh the page and try again after few seconds.'
                  )
            );
      
            exit();

      }

      _rp_set_default_tiemzone();

      global $wp_repair_selected_language;

      global $rpQuery;

      if( ($_POST['rp_reschedule'] ?? '') == '1' && !empty(($_POST['get_o_id_fk'] ?? '')) ) {

            rp_fe_finalize_booking_reschedule();

            exit;

      }

      $company_location = $rpQuery->prefix . "rs_company_location";
      $location = $rpQuery->get_results("SELECT * FROM $company_location");

      $rp_ship_device_location_id = rp_get_option('wp_repair_ship_device_location_id', 1);
      $ship_device_location = $rpQuery->get_results("SELECT * FROM $company_location WHERE `location_id` = '".rp_escape_sql($rp_ship_device_location_id)."'");
      if(empty($ship_device_location)) {
            $ship_device_location = $location;
      }

      $type_table = $rpQuery->prefix . "rs_notification_types";
      $type = $rpQuery->get_results("SELECT * FROM $type_table");

      $model_table = $rpQuery->prefix . "rs_model";
      $category_table = $rpQuery->prefix . "rs_category";
      $brand_table = $rpQuery->prefix . "rs_brand";
      $repair_table = $rpQuery->prefix . "rs_repair";
      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";
      $rs_customer = $rpQuery->prefix . "rs_customer";
      $rs_customer_business = $rpQuery->prefix . "rs_customer_business";
      $rs_orders = $rpQuery->prefix . "rs_orders";
      $rs_order_repairs = $rpQuery->prefix . "rs_order_repairs";
      $rs_order_repair_attr = $rpQuery->prefix . "rs_order_repair_attr";
      $rs_order_upsales = $rpQuery->prefix . "rs_order_upsales";
      $rs_upsales = $rpQuery->prefix . "rs_upsales";
      $rs_company_schedules = $rpQuery->prefix . "rs_company_schedules";

      $m_id_fk = $_POST['m_id_fk'] ?? 0;

      $model_info = $rpQuery->get_row("SELECT * FROM $model_table WHERE m_id = ':m_id_fk' LIMIT 1;", array( 'm_id_fk' => $m_id_fk ));

      if( empty( $model_info ) ) {

            echo json_encode(array(

                  'status' => false,

                  'rp_validation_error' => 'Invalid input provided.'

            ));

            exit;

      }


      if (isset($_POST['confirm_booking'])) {
            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'ship device') {
                  if( !rp_is_enabled_ship_device_location() ) {
                        $_POST['location_id'] = $ship_device_location[0]->location_id;
                  }
                  $_POST['date'] = date('d-m-Y');
                  $_POST['time'] = '';
            }
            $_delivery_method_short = '';
            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'pickup service') {
                  $_POST['location_id'] = $location[0]->location_id;
                  $_delivery_method_short = 'ps';
            }
            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'repair on location') {
                  $_POST['location_id'] = $location[0]->location_id;
                  $_delivery_method_short = 'rol';
            }
            $user_name     = $_POST['user_name'];
            $business_name = $_POST['business_name'];
            $user_email    = $_POST['user_email'];
            $user_phone    = $_POST['user_phone'];
            $color         = $_POST['color_name'];
            $notes        = $_POST['notes'] ?? '';
            $time         = $_POST['time'] ?? '';
            $date         = $_POST['date'] ?? '';
            $d_method     = $_POST['d_method'] ?? '';
            $location_id  = $_POST['location_id'] ?? '';
            $pdf_offer    = $_POST['pdf_offer'] ?? '';

            if( !empty( $_delivery_method_short ) ) {

                  $house_no = $_POST[$_delivery_method_short.'_house_no'] ?? '';
                  $street_address = $_POST[$_delivery_method_short.'_street_address'] ?? '';
                  $city = $_POST[$_delivery_method_short.'_user_city'] ?? '';
                  $zipcode = $_POST[$_delivery_method_short.'_user_zipcode'] ?? '';

                  $rpQuery->query("INSERT INTO $rs_customer(customer_name,customer_phone,customer_email,customer_house_no,customer_street_address,customer_city,customer_zipcode) VALUES('".rp_escape_sql($user_name)."','".rp_escape_sql($user_phone)."','".rp_escape_sql($user_email)."','".rp_escape_sql($house_no)."','".rp_escape_sql($street_address)."','".rp_escape_sql($city)."','".rp_escape_sql($zipcode)."')");
                  $customer_id_fk  = $rpQuery->insert_id;

                  $nearest_location_id = rp_get_nearest_location_for_customer( $customer_id_fk );

                  if( !empty( $nearest_location_id ) ) {

                        $_POST['location_id'] = $nearest_location_id;

                        $location_id = $nearest_location_id;

                  }

            } else {

                  $rpQuery->query("INSERT INTO $rs_customer(customer_name,customer_phone,customer_email) VALUES('".rp_escape_sql($user_name)."','".rp_escape_sql($user_phone)."','".rp_escape_sql($user_email)."')");
                  $customer_id_fk  = $rpQuery->insert_id;

            }

            if($business_name != null){
                  $rpQuery->query("INSERT INTO $rs_customer_business(cb_name,customer_id_fk) VALUES('".rp_escape_sql($business_name)."','".rp_escape_sql($customer_id_fk)."')");         
            }
            $m_id_fk      = $_POST['m_id_fk'] ?? 0;
            $sub_total    = $_POST['sub_total'] ?? '';
            $total_price  = $_POST['total_price'] ?? '';
            $coupon_id_fk = $_POST['coupons_id'] ?? 0;
            $combo        = $_POST['combo'] ?? '';
            $r_id         = $_POST['r_id'] ?? array();
            $a_id         = $_POST['a_id'] ?? array();
            $up_id        = $_POST['up_id'] ?? array();

            $dm_cost = 0;

            $parent_order_id = 0;

            if( isset( $_POST['get_o_id_fk'] ) && !empty( $_POST['get_o_id_fk'] ) ) {

                  $parent_order_id = $_POST['get_o_id_fk'];

                  $_POST['selected_dm_id'] = $d_method;

                  list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $_POST['get_o_id_fk'] );

                  $combo        = $calculations['combo_discount'];

                  $coupon       = $calculations['coupon_amount'];

                  $total_price  = $calculations['total'];

                  $sub_total  = $calculations['sub_total'];

                  $dm_cost = $rpCart->delivery_method_cost();

                  $r_id = array();

                  $a_id = array();

                  $up_id = array();

                  foreach( $rpCart->repairs as $cartRow ) {

                        $r_id[] = $cartRow->r_id;

                  }

                  foreach( $rpCart->repair_attrs as $cartRow ) {

                        $r_id[] = $cartRow->r_id_fk;

                        $a_id[] = $cartRow->a_id;

                  }

                  foreach( $rpCart->upsales as $cartRow ) {

                        $up_id[] = $cartRow->upsales_id;

                  }

                  $prepared_post_data = $rpCart->prepare_post_data( $r_id, $up_id );

                  list( $r_id, $array_name, $array_price, $a_id, $up_id ) = $prepared_post_data;

            } else {

                  $rpCart       = new RepairPluginPro\RP_Cart( $m_id_fk );

                  $r_id          = $_POST['r_id'] ?? array();

                  foreach ($r_id as $repair_id) {

                    $rpCart->add_repair( ((int) $repair_id) );

                  }

                  $a_id          = $_POST['a_id'] ?? array();

                  foreach ($a_id as $attr_id) {

                    $rpCart->add_repair_attr( ((int) $attr_id) );

                  }

                  $up_id         = $_POST['up_id'] ?? array();

                  foreach ($up_id as $upsale_id) {

                    $rpCart->add_upsale( ((int) $upsale_id) );

                  }

                  $rpCart->add_delivery_method( $d_method );

                  $rpCart->add_coupon( ((int) $coupon_id_fk) );

                  $calculations = $rpCart->calculate();

                  $combo        = $calculations['combo_discount'];

                  $coupon       = $calculations['coupon_amount'];

                  $total_price  = $calculations['total'];

                  $sub_total  = $calculations['sub_total'];

                  $dm_cost = $rpCart->delivery_method_cost();

                  $prepared_post_data = $rpCart->prepare_post_data( $r_id, $up_id );

                  list( $r_id, $array_name, $array_price, $a_id, $up_id ) = $prepared_post_data;

            }
            
            $status = $type[2]->is_active;
            if($status == '1'){
                  $status = '3';
            }else{
                  $status = '1';
            }

            $created_time = date('Y-m-d H:i:s');

            $rpQuery->query("INSERT INTO $rs_orders(customer_id_fk,notes,color,excluded_tax_price,combo_discount,coupon_id_fk,total_price,delivery_method,delivery_time,delivery_date,pdf_offer,m_id_fk,status_id_fk,location_id_fk,coupon_amount,created_time,dm_cost) VALUES('".rp_escape_sql($customer_id_fk)."','".rp_escape_sql($notes)."','".rp_escape_sql($color)."','".rp_escape_sql($sub_total)."','".rp_escape_sql($combo)."','".rp_escape_sql($coupon_id_fk)."','".rp_escape_sql($total_price)."','".rp_escape_sql($d_method)."','".rp_escape_sql($time)."','".rp_escape_sql($date)."','".rp_escape_sql($pdf_offer)."','".rp_escape_sql($m_id_fk)."','".rp_escape_sql($status)."','".rp_escape_sql($location_id)."','".rp_escape_sql($coupon)."','".rp_escape_sql($created_time)."','".rp_escape_sql($dm_cost)."')");

            $order_id_fk = $rpQuery->insert_id;

            rp_set_new_appointment_number($order_id_fk, $created_time);

            rp_set_coupon_settings($order_id_fk, $parent_order_id);

            if($r_id){
                  $r_id_str = join(',', $r_id);
            } else {
                  $r_id_str = '';
            }
            if($a_id){
                  $a_id_str = join(',', $a_id);
            } else {
                  $a_id_str = '';
            }
            $up_id = $_POST['up_id'] ?? '';
            if($up_id){
                  $up_id_str = join(',', $up_id);
            } else {
                  $up_id_str = '';
            }

            $created_through_url = $_REQUEST['created_through_url'] ?? '';

            if( ($_REQUEST['is_model_shortcode'] ?? '0') === '1' ) {

                  $action = 'shortcode_finalize_order';

            } else {

                  $action = 'finalize';

            }

            $reschedule_link = $created_through_url . '/?reschedule=1&o_id_fk=' . $order_id_fk . '&c_id=' . $customer_id_fk . '&uid=' . substr(md5($customer_id_fk), 0, 8) . '&selected='.$action.'/b_id='. $model_info->brand_id_fk .'/model_name=' . $model_info->m_name . '/color=' . $color . '/u=' . $up_id_str . '/a=' . $a_id_str . '/r=' . $r_id_str . '/finalprice=' . $total_price . '/coupon=' . $coupon . '/subtotal=' . $sub_total;

            // update reschedule link in rs_orders table 

            $rpQuery->query("UPDATE `$rs_orders` SET `reschedule_link` = '".rp_escape_sql($reschedule_link)."' WHERE o_id = '".rp_escape_sql($order_id_fk)."' LIMIT 1;");

            rp_save_order_related_language($order_id_fk, $wp_repair_selected_language);

            rp_save_cancel_link_by_order_id( $order_id_fk, ($_POST['created_through_url'] ?? '') );

            if($r_id){
                  foreach($r_id as $rid){ 
                        rp_insert_into_rs_order_repairs($order_id_fk, $rid);
                  }
            }     
            if($a_id){
                  foreach($a_id as $aid){
                        rp_insert_into_rs_order_repair_attr($order_id_fk, $aid);
                  }
            }
            $up_id = $_POST['up_id'] ?? '';
            if($up_id){
                  foreach($up_id as $upid){
                        rp_insert_into_rs_order_upsales($order_id_fk, $upid);
                  }
            }

            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'come by our store') {

                  rp_book_available_slot(array(
                        'loc_id' => $location_id,
                        'a_date' => $date,
                        'a_time' => $time,
                        'o_id' => $order_id_fk
                  ));

            }

            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'pickup service') {

                  rp_book_available_slot_new_dm(array(
                        'method_id' => 'ps',
                        'a_date' => $date,
                        'a_time' => $time,
                        'o_id' => $order_id_fk
                  ));

            }

            if(isset($_POST['d_method']) && strtolower($_POST['d_method']) === 'repair on location') {

                  rp_book_available_slot_new_dm(array(
                        'method_id' => 'rol',
                        'a_date' => $date,
                        'a_time' => $time,
                        'o_id' => $order_id_fk
                  ));

            }

            RepairPluginPro\RP_Session_Handler::start();

            $_SESSION['o_id'] = $order_id_fk;

            unset($_SESSION['finalize_booking_arr']);

            RepairPluginPro\RP_Session_Handler::close();

      }

      if( !empty( $order_id_fk ?? 0 ) ) {

            $order_delivery_method = rp_find_delivery_method_of_order( $order_id_fk );

            if( !empty( $order_delivery_method ) && $order_delivery_method != 'ship device' ) {

                  rp_trigger_outgoing_webhook( 'scheduled_appointment_created', $order_id_fk );

            } else {

                  rp_trigger_outgoing_webhook( 'ship_device_appointment_created', $order_id_fk );

            }

      }

      rp_fe_thank_you_page();

      exit();
}

rp_ajax_for_public('rp_fe_finalize_booking','rp_fe_finalize_booking');

function rp_find_delivery_method_of_order( $order_id_fk = 0 ) {

      if( empty( $order_id_fk ) ) {

            return FALSE;

      }

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $sql = "SELECT * FROM `$rs_orders` WHERE `o_id` = '{$order_id_fk}'";

      $row = $rpQuery->get_row( $sql );

      if( !empty( $row ) ) {

            return strtolower(trim($row->delivery_method ?? ''));

      }

      return FALSE;

}


// --------------------------------------------------------- get thank you page for front-end
function rp_fe_thank_you_page(){
      global $rpQuery;
      global $wp_repair_lang;
      $rs_orders = $rpQuery->prefix . "rs_orders";
      $rs_company_location = $rpQuery->prefix . "rs_company_location";
      $rs_customer = $rpQuery->prefix . "rs_customer";
      $rs_notifications_and_offers = $rpQuery->prefix . "rs_notifications_and_offers";
      $company_information = $rpQuery->prefix ."rs_company_information";
      $company_results = $rpQuery->get_results("SELECT * FROM $company_information");
      $theme_color = $company_results[0]->company_theme_color;

      RepairPluginPro\RP_Session_Handler::read();
      $order_id = $_SESSION['o_id'] ?? '';

      if(isset($order_id) && !empty($order_id)){
            $order = $rpQuery->get_results("SELECT o.*, cl.*, cu.* FROM $rs_orders AS o LEFT JOIN $rs_company_location AS cl ON o.location_id_fk = cl.location_id LEFT JOIN $rs_customer as cu ON o.customer_id_fk = cu.customer_id WHERE o.o_id = '".rp_escape_sql($order_id)."'");
            rp_send_appointment_status_update_email($order_id, TRUE);
      }
      RepairPluginPro\RP_Session_Handler::start();

      $_SESSION['c_name'] = '';
      $_SESSION['b_name'] = '';
      $_SESSION['m_name'] = '';
      $_SESSION['o_id']   = '';

      RepairPluginPro\RP_Session_Handler::close();

      ob_start();
      require_once WP_REPAIR_PLUGIN_PATH.'html/front_end/thank_you_html.php';
      $content = ob_get_contents();
      ob_end_clean();

      echo json_encode(array(
            'template' => $content,
            'status' => true
      ));

      exit();
}

rp_ajax_for_public('rp_fe_thank_you_page','rp_fe_thank_you_page');



// --------------------------------------------------------- get cancel page for front-end
function _rp_fe_cancel_page(){
      global $rpQuery;
      global $wp_repair_lang;
      $rs_orders = $rpQuery->prefix . "rs_orders";
      $rs_company_location = $rpQuery->prefix . "rs_company_location";
      $rs_customer = $rpQuery->prefix . "rs_customer";
      $rs_notifications_and_offers = $rpQuery->prefix . "rs_notifications_and_offers";
      $company_information = $rpQuery->prefix ."rs_company_information";
      $company_results = $rpQuery->get_results("SELECT * FROM $company_information");
      $theme_color = $company_results[0]->company_theme_color;
      $order_id = $_POST['id'] ?? '';
      $secret_key = $_POST['secret_key'] ?? '';
      if(isset($order_id) && !empty($order_id)){
            $order = $rpQuery->get_results("SELECT o.*, cl.*, cu.* FROM $rs_orders AS o LEFT JOIN $rs_company_location AS cl ON o.location_id_fk = cl.location_id LEFT JOIN $rs_customer as cu ON o.customer_id_fk = cu.customer_id WHERE o.o_id = '".rp_escape_sql($order_id)."'");

            if(!empty($order)) {

                        include_once WP_REPAIR_PLUGIN_PATH.'html/order_emails.php';

                        if(md5($order[0]->o_id . $order[0]->customer_id_fk) === $secret_key) {
                              rp_set_order_related_language($order_id);
                              $status = $order[0]->status_id_fk;
                              $offers = $rpQuery->get_results("SELECT * FROM $rs_notifications_and_offers");
                              if($status != '4') {
                                    $rpQuery->query("UPDATE $rs_orders SET `status_id_fk`='4' WHERE `o_id` = '".rp_escape_sql($order[0]->o_id)."'");

                                    $order_delivery_method = rp_find_delivery_method_of_order( $order[0]->o_id );

                                    if( !empty( $order_delivery_method ) && $order_delivery_method != 'ship device' && $order_delivery_method != 'pdf offer' ) {

                                          rp_trigger_outgoing_webhook( 'scheduled_appointment_cancelled', $order[0]->o_id );

                                    } else if( !empty( $order_delivery_method ) && $order_delivery_method == 'ship device' ) {

                                          rp_trigger_outgoing_webhook( 'ship_device_appointment_cancelled', $order[0]->o_id );

                                    } else if( !empty( $order_delivery_method ) && $order_delivery_method == 'pdf offer' ) {

                                          rp_trigger_outgoing_webhook( 'pdf_offer_cancelled', $order[0]->o_id );

                                    } else {

                                          // Do nothing!

                                    }
                                    $status = '4';
                                    $pending  = $offers[0]->is_active;
                                    $offer    = $offers[1]->is_active;
                                    $approved = $offers[2]->is_active;
                                    $cancel   = $offers[3]->is_active;
                                    $user_name       = $order[0]->customer_name;
                                    $user_email      = $order[0]->customer_email;
                                    $configure_email = $rpQuery->prefix . "rs_configure_mail";
                                    $settings_detail = $rpQuery->get_results("SELECT * FROM $configure_email");
                                    $to = $user_email;
                                    if(($status == '1') && ($pending == '1')){
                                      $email_translation_key = get_translation_key_by_noti_type_id($offers[0]->noti_type_id_fk);
                                      $subject = $wp_repair_lang[$email_translation_key.'_email_subject'];
                                      $message = $wp_repair_lang[$email_translation_key.'_email_template'];
                                      $message = wp_repair_prepare_order_email($order_id, $subject, $message);
                                      $subject = wp_repair_prepare_order_email_subject($order_id, $subject);
                                      wp_repair_send_mail($to, $subject, $message);
                                      $duplicate_email = wp_repair_admin_duplicate_email('1', rp_get_location_id_of_order( $order_id ));
                                      if(!empty($duplicate_email)) {
                                          rp_wp_mail_change_sender_info($user_email, $user_name);
                                          wp_repair_send_mail($duplicate_email, $subject, $message);
                                      }
                                  }else if(($status == '2') && ($offer == '1')){ 
                                    $email_translation_key = get_translation_key_by_noti_type_id($offers[1]->noti_type_id_fk);
                                    $subject = $wp_repair_lang[$email_translation_key.'_email_subject'];
                                    $message = $wp_repair_lang[$email_translation_key.'_email_template'];
                                      $message = wp_repair_prepare_order_email($order_id, $subject, $message);
                                      $subject = wp_repair_prepare_order_email_subject($order_id, $subject);
                                      wp_repair_send_mail($to, $subject, $message);
                                      $duplicate_email = wp_repair_admin_duplicate_email('2', rp_get_location_id_of_order( $order_id ));
                                      if(!empty($duplicate_email)) {
                                          rp_wp_mail_change_sender_info($user_email, $user_name);
                                          wp_repair_send_mail($duplicate_email, $subject, $message);
                                      }
                                  }else if(($status == '3') && ($approved == '1')){
                                    $email_translation_key = get_translation_key_by_noti_type_id($offers[2]->noti_type_id_fk);
                                    $subject = $wp_repair_lang[$email_translation_key.'_email_subject'];
                                    $message = $wp_repair_lang[$email_translation_key.'_email_template'];
                                      $message = wp_repair_prepare_order_email($order_id, $subject, $message);
                                      $subject = wp_repair_prepare_order_email_subject($order_id, $subject);
                                      wp_repair_send_mail($to, $subject, $message);
                                      $duplicate_email = wp_repair_admin_duplicate_email('3', rp_get_location_id_of_order( $order_id ));
                                      if(!empty($duplicate_email)) {
                                          rp_wp_mail_change_sender_info($user_email, $user_name);
                                          wp_repair_send_mail($duplicate_email, $subject, $message);
                                      }
                                  }else if(($status == '4') && ($cancel == '1')){
                                    $email_translation_key = get_translation_key_by_noti_type_id($offers[3]->noti_type_id_fk);
                                    $subject = $wp_repair_lang[$email_translation_key.'_email_subject'];
                                    $message = $wp_repair_lang[$email_translation_key.'_email_template'];
                                      $message = wp_repair_prepare_order_email($order_id, $subject, $message);
                                      $subject = wp_repair_prepare_order_email_subject($order_id, $subject);
                                      wp_repair_send_mail($to, $subject, $message);
                                      $duplicate_email = wp_repair_admin_duplicate_email('4', rp_get_location_id_of_order( $order_id ));
                                      if(!empty($duplicate_email)) {
                                          rp_wp_mail_change_sender_info($user_email, $user_name);
                                          wp_repair_send_mail($duplicate_email, $subject, $message);
                                      }
                                  }
                              }
                        } else {
                              $order = NULL;
                        }

            }
      }

      ob_start();
      require_once WP_REPAIR_PLUGIN_PATH.'html/front_end/cancel_html.php';
      $content = ob_get_contents();
      ob_end_clean();

      return json_encode(array(
            'template' => $content,
            'status' => true
      ));
}
function rp_fe_cancel_page(){

      echo _rp_fe_cancel_page();

      exit();
}

rp_ajax_for_public('rp_fe_cancel_page','rp_fe_cancel_page');

/*
UPSALES RESET / SINGLE DELETE
*/
function handle_upsale_single_delete() {

      rp_verify_csrf_token_for_ajax('manage_upsales');

      global $rpQuery;

      $upsales_models_tables = $rpQuery->prefix . "rs_upsales_models";

      $model_id = $_POST['up_model_id'] ?? '' ;

      if(!empty($model_id)){

            $rpQuery->query("DELETE FROM $upsales_models_tables  WHERE `up_model_id` = '".rp_escape_sql($model_id)."' ");

            echo json_encode(array(
                  'success' => 'Successfully deleted.'
            ));
            exit();

      }

      echo json_encode(array(
            'error' => 'Something went wrong, please try again later.'
      ));

      exit();
}

rp_ajax_for_admin('delete_upsales_model', 'handle_upsale_single_delete');


// RESET UPSALES QUERY

function rp_upsale_reset() {

      rp_verify_csrf_token_for_ajax('manage_upsales');

      global $rpQuery;

      $upsales_reset_table = $rpQuery->prefix . "rs_upsales_models";

      $upsales_reset_id = $_POST['upsales_reset_id'] ?? '' ;

      if(!empty($upsales_reset_id)) {

            $rpQuery->query("DELETE FROM $upsales_reset_table WHERE `upsales_id_fk` = '".rp_escape_sql($upsales_reset_id)."'; ");

            echo json_encode(array(
                  'success' => 'Selected uspale successfully reset.'
            ));
            exit();

      }
    
      echo json_encode(array(
            'error' => 'Something went wrong, please try again later.'
      ));

      exit();
}


rp_ajax_for_admin('reset_upsales', 'rp_upsale_reset');


/*
UPSALES IMAGE UPLOAD
*/

function add_upsales_img_column_to_existing_db() {
      global $rpQuery;

      $rs_upsales = $rpQuery->prefix . 'rs_upsales';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_upsales)."' AND COLUMN_NAME = 'upsales_img';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_upsales` ADD `upsales_img` varchar(255) NOT NULL;");
            
      }
}

add_action('init', 'add_upsales_img_column_to_existing_db', 999);


/*
Location Google Map Link
*/

function add_google_map_column_to_location_table() {
      global $rpQuery;

      $rs_company_location = $rpQuery->prefix . 'rs_company_location';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_company_location)."' AND COLUMN_NAME = 'google_map_link';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_company_location` ADD `google_map_link` varchar(1000) NOT NULL;");
            
      }
}

add_action('init', 'add_google_map_column_to_location_table', 999);


/*
Add is_free column to rs_repair table
*/

function add_is_free_column_to_rs_repair_table() {
      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair)."' AND COLUMN_NAME = 'is_free';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair` ADD `is_free` int(1) NOT NULL;");
            
      }
}

add_action('init', 'add_is_free_column_to_rs_repair_table', 999);

/*
Add is_free column to rs_default_repair table
*/

function add_is_free_column_to_rs_default_repair_table() {
      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair)."' AND COLUMN_NAME = 'is_free';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair` ADD `is_free` int(1) NOT NULL;");
            
      }
}

add_action('init', 'add_is_free_column_to_rs_default_repair_table', 999);


function rp_wp_mail_change_sender_info($sender_email = '', $sender_name = '') {

      if(rp_get_option('rp_use_customer_info_for_duplicate_email', '0') === '1') {

            if(!empty($sender_email)) {

                  $GLOBALS['rp_sender_email'] = $sender_email;

                  add_filter('wp_mail', function($atts = NULL){

                        if(!empty($atts)) {

                              if(!isset($atts['headers'])) {

                                    $atts['headers'] = array();

                              }

                              if(is_array($atts['headers']) !== FALSE) {

                                     $atts['headers'][] = 'Reply-To: '. $GLOBALS['rp_sender_email'];

                              } else if(is_string($atts['headers'])) {

                                    $atts['headers'] .= "\nReply-To: ". $GLOBALS['rp_sender_email'];

                              } else {

                              }
                        }
                        
                        return $atts;

                  }, 999);

            }

            if(!empty($sender_name)) {

                  $GLOBALS['rp_sender_name'] = $sender_name;

                  add_filter('wp_mail_from_name', function(){

                        return $GLOBALS['rp_sender_name'];

                  }, 999);

            }

      }
      
}

function rp_fix_default_repair_attr_sorting($current_rows = array(), $m_id = '') {

      global $rpQuery;

      $rs_model = $rpQuery->prefix . 'rs_model';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $m_id = (int) $m_id;

      $model_info = $rpQuery->get_results("SELECT device_type FROM `$rs_model` WHERE m_id = '".rp_escape_sql($m_id)."'");

      if(!empty($model_info)) {

            $result = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE dr_category = '".rp_escape_sql($model_info[0]->device_type)."'");

      } else {

            $result = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr`");

      }

      if(!empty($result)) {

            $key_value_pair = array();
            $default_repairs = array();

            foreach ($result as $key => $value) {
                  
                  $key_value_pair[$value->da_uq_id] = $value->da_position;
                  $default_repairs[] = $value->da_uq_id;

            }

            $not_default = 1;

            foreach ($current_rows as $key => $value) {
            
                  if(!in_array($value->a_uq_id, $default_repairs)) {

                        $value->ra_position = $not_default;

                        $current_rows[$key] = $value;

                        $not_default++;

                  } else {

                        if(($value->ra_position == '0' || $value->ra_position == '') && isset($key_value_pair[$value->a_uq_id])) {

                              // Skip..

                        } else {

                              $value->ra_position = $not_default;

                              $current_rows[$key] = $value;

                              $not_default++;

                        }

                  }

            }

            foreach ($current_rows as $key => $value) {
            
                  if(!in_array($value->a_uq_id, $default_repairs)) {

                        // Skip..

                  } else {

                        if(($value->ra_position == '0' || $value->ra_position == '') && isset($key_value_pair[$value->a_uq_id])) {

                              $value->ra_position = $not_default+$key_value_pair[$value->a_uq_id];

                              $current_rows[$key] = $value;

                        }

                  }

            }

      }

      $sort_position = array();

      foreach ($current_rows as $key => $value) {
            
            $sort_position[] = array('key'=>$key, 'position'=>$value->ra_position);

      }

      usort($sort_position, function ($item1, $item2) {
          return $item1['position'] <=> $item2['position'];
      });

      $new_arr = array();

      foreach ($sort_position as $row) {
            $new_arr[] = $current_rows[$row['key']];
      }

      return $new_arr;

}

function rp_fix_default_repair_sorting($current_rows = array(), $m_id = '') {

      global $rpQuery;

      $rs_model = $rpQuery->prefix . 'rs_model';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $m_id = (int) $m_id;

      $model_info = $rpQuery->get_results("SELECT device_type FROM `$rs_model` WHERE m_id = '".rp_escape_sql($m_id)."'");

      if(!empty($model_info)) {

            $result = $rpQuery->get_results("SELECT * FROM `$rs_default_repair` WHERE dr_category = '".rp_escape_sql($model_info[0]->device_type)."'");

      } else {

            $result = $rpQuery->get_results("SELECT * FROM `$rs_default_repair`");

      }

      if(!empty($result)) {

            $key_value_pair = array();
            $default_repairs = array();

            foreach ($result as $key => $value) {
                  
                  $key_value_pair[$value->dr_uq_id] = $value->dr_position;
                  $default_repairs[] = $value->dr_uq_id;

            }

            $not_default = 1;

            foreach ($current_rows as $key => $value) {
            
                  if(!in_array($value->r_uq_id, $default_repairs)) {

                        $value->r_position = $not_default;

                        $current_rows[$key] = $value;

                        $not_default++;

                  } else {

                        if(($value->r_position == '0' || $value->r_position == '') && isset($key_value_pair[$value->r_uq_id])) {

                              // Skip...

                        } else {

                              $value->r_position = $not_default;

                              $current_rows[$key] = $value;

                              $not_default++;

                        }

                  }

            }

            foreach ($current_rows as $key => $value) {
            
                  if(!in_array($value->r_uq_id, $default_repairs)) {

                        // Skip..

                  } else {

                        if(($value->r_position == '0' || $value->r_position == '') && isset($key_value_pair[$value->r_uq_id])) {

                              $value->r_position = $not_default+$key_value_pair[$value->r_uq_id];

                              $current_rows[$key] = $value;

                        }

                  }

            }

      }

      $sort_position = array();

      foreach ($current_rows as $key => $value) {
            
            $sort_position[] = array('key'=>$key, 'position'=>$value->r_position);

      }

      usort($sort_position, function ($item1, $item2) {
          return $item1['position'] <=> $item2['position'];
      });

      $new_arr = array();

      foreach ($sort_position as $row) {
            $new_arr[] = $current_rows[$row['key']];
      }

      return $new_arr;

}

function rp_get_all_repair_ids_related_to_category( $dr_category = 0 ) {

      $cache_key = 'rp_get_all_repair_ids_related_to_category|'.$dr_category;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $m_ids = rp_get_all_m_ids_by_category( $dr_category );

      $r_ids = array();

      if( !empty( $m_ids ) ) {

            $m_ids = join(',', $m_ids);

            $related_repairs = $rpQuery->get_results("SELECT r_id FROM `$rs_repair` WHERE m_id_fk IN (".rp_escape_sql($m_ids).")");

            if(!empty($related_repairs)) {

                  foreach ($related_repairs as $row) {
                        
                        $r_ids[] = $row->r_id;

                  }

            }

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $r_ids);

      return $r_ids;
}

function rp_reset_default_repairs_order_everywhere() {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $dr_category = $_POST['dr_category'] ?? 0;

      $rows = $rpQuery->get_results("SELECT dr_uq_id FROM $rs_default_repair WHERE dr_category = '".rp_escape_sql($dr_category)."'");

      $all_uq_ids = array();

      if(!empty($rows)) {

            $m_ids = rp_get_all_m_ids_by_category( $dr_category );

            if(!empty($m_ids)) {

                  $m_ids = join(',', $m_ids);

                  foreach ($rows as $key => $row) {

                        $all_uq_ids[] = $row->dr_uq_id;

                  }

                  $all_uq_ids_str = '(\'' . join('\', \'', $all_uq_ids) . '\')';

                  $rpQuery->query("UPDATE $rs_repair SET r_position = '0' WHERE r_uq_id IN ".($all_uq_ids_str)." AND m_id_fk IN (".rp_escape_sql($m_ids).")");

            }

      }

      $rows = $rpQuery->get_results("SELECT da_uq_id FROM $rs_default_repair_attr WHERE dr_category = '".rp_escape_sql($dr_category)."'");

      $all_uq_ids = array();

      if(!empty($rows)) {

            $r_ids = rp_get_all_repair_ids_related_to_category($dr_category);

            if(!empty($r_ids)) {

                  $r_ids = join(',', $r_ids);

                  foreach ($rows as $key => $row) {

                        $all_uq_ids[] = $row->da_uq_id;

                  }

                  $all_uq_ids_str = '(\'' . join('\', \'', $all_uq_ids) . '\')';

                  $rpQuery->query("UPDATE $rs_repair_attr SET ra_position = '0' WHERE a_uq_id IN ".($all_uq_ids_str)." AND r_id_fk IN (".rp_escape_sql($r_ids).")");

            }

      }

}

/* NOT NEEDED AFTER 1.4 UPDATE */
// function rp_reset_default_repair_sorting_first_time() {
//       if(rp_get_option('rp_reset_default_repair_sorting_first_time', FALSE) === FALSE) {
//             rp_reset_default_repairs_order_everywhere();
//             rp_add_option('rp_reset_default_repair_sorting_first_time', TRUE);
//       }
// }
// add_action('init', 'rp_reset_default_repair_sorting_first_time', 99999);


function rp_reset_default_sorting() {

      rp_verify_csrf_token_for_ajax('default_repairs');

      rp_reset_default_repairs_order_everywhere();

      echo json_encode(array('success' => 'Default sorting has been reset.'));

      exit();

}

rp_ajax_for_admin('reset_default_sorting', 'rp_reset_default_sorting');

function rp_reset_default_repair_ajax() {

      rp_verify_csrf_token_for_ajax('default_repairs');

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $dr_id = $_POST['dr_id'] ?? '';

      $dr_category = $_POST['dr_category'] ?? 0;

      $reset_margin_field = apply_filters('rp_reset_margin_field_while_default_reset', FALSE);

      if(!empty($dr_id)) {

            $default_repair = $rpQuery->get_results("SELECT * FROM $rs_default_repair WHERE `dr_id` = '".rp_escape_sql($dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."' LIMIT 1;");

            if(!empty($default_repair)) {

                  $default_repair = $default_repair[0];

                  $dr_img = $default_repair->dr_img;

                  $dr_img = $dr_img;

                  $m_ids = rp_get_all_m_ids_by_category( $dr_category );

                  if(!empty($m_ids)) {

                        $m_ids = join(',', $m_ids);

                        if( $reset_margin_field === TRUE ) {
                              $margin_field_sql = "`r_margin` = '".rp_escape_sql($default_repair->dr_margin)."',";
                        } else {
                              $margin_field_sql = '';
                        }

                        $rpQuery->query("UPDATE $rs_repair SET $margin_field_sql `r_name` = '".rp_escape_sql($default_repair->dr_name)."', `r_price` = '".rp_escape_sql($default_repair->dr_price)."', `r_duration` = '".rp_escape_sql($default_repair->dr_duration)."', `r_duration_measure` = '".rp_escape_sql($default_repair->dr_duration_measure)."', `r_discription` = '".rp_escape_sql($default_repair->dr_discription)."', `r_badge` = '".rp_escape_sql($default_repair->dr_badge)."', `r_img` = '".rp_escape_sql($dr_img)."', is_active = '".rp_escape_sql($default_repair->is_active)."' WHERE `r_uq_id` LIKE '".rp_escape_sql($default_repair->dr_uq_id)."' AND m_id_fk IN (".rp_escape_sql($m_ids).")");

                        $default_repair_attr = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr WHERE `dr_id_fk` = '".rp_escape_sql($dr_id)."' AND dr_category = '".rp_escape_sql($dr_category)."';");

                        if(!empty($default_repair_attr)) {

                              $r_ids = rp_get_all_repair_ids_related_to_category($dr_category);

                              if(!empty($r_ids)) {

                                    $r_ids = join(',', $r_ids);

                                    foreach ($default_repair_attr as $dra_row) {

                                          if( $reset_margin_field === TRUE ) {
                                                $a_margin_sql = "`a_margin` = '".rp_escape_sql($dra_row->da_margin)."',";
                                          } else {
                                                $a_margin_sql = '';
                                          }
                                          
                                          $rpQuery->query("UPDATE $rs_repair_attr SET $a_margin_sql `a_name` = '".rp_escape_sql($dra_row->da_name)."', `a_duration` = '".rp_escape_sql($dra_row->da_duration)."', `a_duration_measure` = '".rp_escape_sql($dra_row->da_duration_measure)."', `a_desc` = '".rp_escape_sql($dra_row->da_desc)."', `a_badge` = '".rp_escape_sql($dra_row->da_badge)."', `a_price` = '".rp_escape_sql($dra_row->da_price)."', `is_active` = '".rp_escape_sql($dra_row->is_active)."' WHERE `a_uq_id` = '".rp_escape_sql($dra_row->da_uq_id)."' AND `r_id_fk` IN (".rp_escape_sql($r_ids).")");

                                    }

                              }

                        }

                  }

            }

      }

      echo json_encode(array('success' => 'Selected default repair has been reset.'));

      exit();

}

rp_ajax_for_admin('reset_default_repair_ajax', 'rp_reset_default_repair_ajax');



function rp_create_apple_calendar_link($options = array()) {

      header('Content-Type: text/calendar; charset=utf-8');
      header('Content-Disposition: attachment; filename=appointment.ics');

      $start_time = new DateTime("{$options['start_time']}:00", new DateTimeZone(wp_timezone_string()));
      $start_time->setTimezone(new DateTimeZone("UTC"));
      $start_time = $start_time->format("Ymd-His");

      $end_time = new DateTime("{$options['end_time']}:00", new DateTimeZone(wp_timezone_string()));
      $end_time->setTimezone(new DateTimeZone("UTC"));
      $end_time = $end_time->format("Ymd-His");

      $ics = new RepairPluginPro\RP_ICS(array(
            'location' => $options['location'],

            'description' => $options['details'],

            'dtstart' => str_replace('-', 'T', $start_time),

            'dtend' => str_replace('-', 'T', $end_time),

            'summary' => $options['text'],

            'url' => get_site_url()
      ));

      echo $ics->to_string();

      exit();

}


function rp_create_google_calendar_link($options = array()) {

      $start_time = new DateTime("{$options['start_time']}:00", new DateTimeZone(wp_timezone_string()));

      $timezone = wp_timezone_string();

      if(!empty($timezone) && strpos($timezone, '/') !== false) {
            // Skip...
      } else {
            $timezone = "Europe/Amsterdam";
      }

      $start_time->setTimezone(new DateTimeZone($timezone));
      $start_time = $start_time->format("Ymd-His");

      $end_time = new DateTime("{$options['end_time']}:00", new DateTimeZone(wp_timezone_string()));
      $end_time->setTimezone(new DateTimeZone($timezone));
      $end_time = $end_time->format("Ymd-His");

      $url = 'https://www.google.com/calendar/event';

      $params = array(

            'action' => 'TEMPLATE',
            'dates' => str_replace('-', 'T', "$start_time/$end_time"),
            'text' => $options['text'],
            'details' => $options['details'],
            'location' => $options['location'],
            'trp' => 'false',
            'ctz' => $timezone,
            'sprop' => 'website:'.get_site_url()

      );

      $get_paramaters = array();

      foreach ($params as $key => $value) {

            $get_paramaters[] = $key.'='.urlencode($value);

      }

      $get_paramaters_str = join('&', $get_paramaters);

      return $url.'?'.$get_paramaters_str;

}



function rp_handle_ics_file_download() {
      if(isset($_REQUEST['rp_ics']) && $_REQUEST['rp_ics'] === 'true' && isset($_POST) && !empty($_POST)) {

            rp_verify_csrf_token_for_apple_calendar();

            rp_create_apple_calendar_link(array(

                  'text' => $_POST['text'],

                  'details' => $_POST['details'],

                  'location' => $_POST['location'],

                  'start_time' => $_POST['start_time'],

                  'end_time' => $_POST['end_time']

            ));
      }
}

add_action('init', 'rp_handle_ics_file_download');


function _rp_set_default_tiemzone() {
      date_default_timezone_set(wp_timezone_string());
}

function repair_plugin_time_slot_tracking_table(){

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_timeslot = $rpQuery->prefix . "rs_timeslot"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_timeslot (

    `ts_id` int(11) NOT NULL AUTO_INCREMENT,

    `loc_id` varchar(255) NOT NULL,

    `a_date` varchar(255) NOT NULL,

    `a_time` varchar(255) NOT NULL,

    `o_id` varchar(255) NOT NULL,

    PRIMARY KEY (`ts_id`))

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_time_slot_tracking_table');


function repair_plugin_time_slot_dm_tracking_table(){

      global $rpQuery;
  
      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );
  
      $table_rs_timeslot = $rpQuery->prefix . "rs_timeslot_dm"; 
  
      $charset_collate = $rpQuery->get_charset_collate();
  
      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_timeslot (
  
      `ts_id` int(11) NOT NULL AUTO_INCREMENT,
  
      `method_id` varchar(255) NOT NULL,
  
      `a_date` varchar(255) NOT NULL,
  
      `a_time` varchar(255) NOT NULL,
  
      `o_id` varchar(255) NOT NULL,
  
      PRIMARY KEY (`ts_id`))
  
      $charset_collate";
  
      dbDelta( $sql );
  
}
  
add_action('init', 'repair_plugin_time_slot_dm_tracking_table');

function rp_book_available_slot($data = array()) {

      global $rpQuery;

      $rs_timeslot = $rpQuery->prefix . 'rs_timeslot';

      $insert_keys = array();

      $insert_values = array();

      foreach ($data as $key => $value) {
            
            $insert_keys[] = $key;

            if($key === 'a_time') {

                  $insert_values[] = rp_escape_sql(date('H:i:s', strtotime($value)));

            } else if($key === 'a_date') {

                  $the_date = explode('-', $value);

                  $the_date = $the_date[2] . '-' . $the_date[1] . '-' . $the_date[0];

                  $insert_values[] = rp_escape_sql($the_date);

            } else {

                  $insert_values[] = rp_escape_sql($value);

            }

      }

      $insert_keys_str = '`'.join('`,`', $insert_keys).'`';

      $insert_values_str = "'" . join("','", $insert_values) . "'";

      $rpQuery->query("INSERT INTO `$rs_timeslot` ($insert_keys_str) VALUES ($insert_values_str)");

}

function rp_book_available_slot_new_dm($data = array()) {

      global $rpQuery;

      $rs_timeslot_dm = $rpQuery->prefix . 'rs_timeslot_dm';

      $insert_keys = array();

      $insert_values = array();

      foreach ($data as $key => $value) {
            
            $insert_keys[] = $key;

            if($key === 'a_time') {

                  $time = explode(' — ', $value);

                  $time = date('H:i', strtotime($time[0])) . ' — ' . date('H:i', strtotime($time[1]));

                  $insert_values[] = rp_escape_sql($time);

            } else if($key === 'a_date') {

                  $the_date = explode('-', $value);

                  $the_date = $the_date[2] . '-' . $the_date[1] . '-' . $the_date[0];

                  $insert_values[] = rp_escape_sql($the_date);

            } else {

                  $insert_values[] = rp_escape_sql($value);

            }

      }

      $insert_keys_str = '`'.join('`,`', $insert_keys).'`';

      $insert_values_str = "'" . join("','", $insert_values) . "'";

      $rpQuery->query("INSERT INTO `$rs_timeslot_dm` ($insert_keys_str) VALUES ($insert_values_str)");

}

function rp_is_available_slot($loc_id = NULL, $date = '', $time = '') {

      if(empty($loc_id) || empty($date) || empty($time)) {

            return FALSE;

      }

      $time = date('H:i:s', strtotime($time));

      global $rpQuery;

      $planning = $rpQuery->prefix . "rs_planning_and_discount";

      $rs_timeslot = $rpQuery->prefix . 'rs_timeslot';

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $planning_result = $rpQuery->get_results("SELECT * FROM $planning");

      $booking_amount_per_slot = trim($planning_result[0]->booking_amount_per_slot);

      if(empty($booking_amount_per_slot)) {
            $booking_amount_per_slot = 'unlimited';
      }

      if(strpos(strtolower($booking_amount_per_slot), 'unlimited') !== FALSE) {
            return TRUE;
      }

      $booking_amount_per_slot = (int) $booking_amount_per_slot;

      $rows = $rpQuery->get_results("SELECT * FROM `$rs_timeslot` WHERE `loc_id` = '".rp_escape_sql($loc_id)."' AND `a_date` LIKE '".rp_escape_sql($date)."' AND `a_time` LIKE '".rp_escape_sql($time)."'");

      if(!empty($rows)) {

            $o_ids = array();

            foreach ($rows as $row) {
                  
                  $o_ids[] = $row->o_id;

            }

            $o_ids_str = join(',', $o_ids);

            $active_orders_count = $rpQuery->get_var("SELECT COUNT(o_id) FROM `$rs_orders` WHERE `o_id` IN (".rp_escape_sql($o_ids_str).") AND `status_id_fk` IN (1,3)");

            return $booking_amount_per_slot > $active_orders_count;

      }

      return TRUE;

}


function rp_is_available_slot_new_delivery_methods($method = NULL, $date = '', $time = '') {

      if(empty($method) || empty($date) || empty($time)) {

            return FALSE;

      }

      if( $method === 'pickup_service' ) {

            $_method = 'ps';

      }

      if( $method === 'repair_on_location' ) {

            $_method = 'rol';

      }

      $time = explode(' — ', $time);

      $time = date('H:i', strtotime($time[0])) . ' — ' . date('H:i', strtotime($time[1]));

      global $rpQuery;

      $rs_timeslot_dm = $rpQuery->prefix . 'rs_timeslot_dm';

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $booking_amount_per_slot = trim(rp_get_option('wp_repair_booking_amount_per_slot_'.$_method, 'Unlimited'));

      if(empty($booking_amount_per_slot)) {
            $booking_amount_per_slot = 'unlimited';
      }

      if(strpos(strtolower($booking_amount_per_slot), 'unlimited') !== FALSE) {
            return TRUE;
      }

      $booking_amount_per_slot = (int) $booking_amount_per_slot;

      $rows = $rpQuery->get_results("SELECT * FROM `$rs_timeslot_dm` WHERE `method_id` = '".rp_escape_sql($_method)."' AND `a_date` LIKE '".rp_escape_sql($date)."' AND `a_time` LIKE '".rp_escape_sql($time)."'");

      if(!empty($rows)) {

            $o_ids = array();

            foreach ($rows as $row) {
                  
                  $o_ids[] = $row->o_id;

            }

            $o_ids_str = join(',', $o_ids);

            $active_orders_count = $rpQuery->get_var("SELECT COUNT(o_id) FROM `$rs_orders` WHERE `o_id` IN (".rp_escape_sql($o_ids_str).") AND `status_id_fk` IN (1,3)");

            return $booking_amount_per_slot > $active_orders_count;

      }

      return TRUE;

}


function rp_get_front_end_results_count() {

      global $rpQuery;

      $categories_count = array();

      $brands_count = array();

      $models_count = array();

      $rs_category = $rpQuery->prefix . "rs_category";

      $rs_brand = $rpQuery->prefix . "rs_brand";

      $rs_model = $rpQuery->prefix . "rs_model";

      $c_results = $rpQuery->get_results("SELECT c_id FROM $rs_category WHERE is_active = '1';");

      if(!empty($c_results)) {

            foreach ($c_results as $c_row) {

                  $categories_count[] = $c_row->c_id;
                  
                  $b_results = $rpQuery->get_results("SELECT b_id FROM $rs_brand WHERE c_id_fk = '".rp_escape_sql($c_row->c_id)."' AND is_active = '1';");

                  $brands_count[$c_row->c_id] = array();

                  if(!empty($b_results)) {

                        foreach ($b_results as $b_row) {

                              $brands_count[$c_row->c_id][] = $b_row->b_id;

                              $m_results = $rpQuery->get_results("SELECT m_id FROM $rs_model WHERE brand_id_fk = '".rp_escape_sql($b_row->b_id)."' AND is_active = '1';");

                              $models_count[$b_row->b_id] = array();

                              if(!empty($m_results)) {

                                    foreach ($m_results as $m_row) {

                                          $models_count[$b_row->b_id][] = $m_row->m_id;

                                    }

                              }

                        }

                  }

            }

      }

      return array($categories_count, $brands_count, $models_count);

}


function rp_get_b_id_by_c_name_and_b_name($c_name = '', $b_name = '') {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . "rs_category";

      $rs_brand = $rpQuery->prefix . "rs_brand";

      $rows = $rpQuery->get_results("SELECT b.b_id FROM $rs_brand AS b 
            LEFT JOIN $rs_category AS c ON b.c_id_fk = c.c_id
            WHERE b.b_name LIKE '".rp_escape_sql($b_name)."' AND c.c_name LIKE '".rp_escape_sql($c_name)."' LIMIT 1");

      if(!empty($rows)) {

            return $rows[0]->b_id;

      }

      return 0;

}

function rp_get_c_id_by_c_name($c_name = '') {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . "rs_category";

      $rows = $rpQuery->get_results("SELECT c_id FROM $rs_category WHERE c_name LIKE '".rp_escape_sql($c_name)."' LIMIT 1");

      if(!empty($rows)) {

            return $rows[0]->c_id;

      }

      return 0;

}

function rp_get_c_name_by_c_id($c_id = '') {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . "rs_category";

      $rows = $rpQuery->get_results("SELECT c_name FROM $rs_category WHERE c_id = '".rp_escape_sql($c_id)."' LIMIT 1");

      if(!empty($rows)) {

            return $rows[0]->c_name;

      }

      return '';

}

function rp_get_b_name_by_b_id($b_id = '') {
      
      global $rpQuery;

      $rs_brand = $rpQuery->prefix . "rs_brand";

      $rows = $rpQuery->get_results("SELECT b_name FROM $rs_brand WHERE b_id = '".rp_escape_sql($b_id)."' LIMIT 1");

      if(!empty($rows)) {

            return $rows[0]->b_name;

      }

      return '';

}

function rp_get_m_name_by_m_id($m_id = '') {
      
      global $rpQuery;

      $rs_model = $rpQuery->prefix . "rs_model";

      $rows = $rpQuery->get_results("SELECT m_name FROM $rs_model WHERE m_id = '".rp_escape_sql($m_id)."' LIMIT 1");

      if(!empty($rows)) {

            return $rows[0]->m_name;

      }

      return '';

}

function rp_get_front_end_firsttime_load_step($c_name = '', $b_name = '') {

      if(rp_get_option('wp_repair_skip_steps', '1') != '1') {
            return NULL;
      }

      global $rpQuery;

      list($categories, $brands, $models) = rp_get_front_end_results_count();

      if(!empty($b_name)) {

            $c_id = rp_get_c_id_by_c_name($c_name);

            $categories = array( $c_id );

            $b_id = rp_get_b_id_by_c_name_and_b_name($c_name, $b_name);

            $brands[ $categories[0] ] = array( $b_id );


      } else if(!empty($c_name)) {

            $c_id = rp_get_c_id_by_c_name($c_name);

            $categories = array($c_id);

      } else {

            // No Need To Do Anything!

      }

      if(count($categories) === 1) {

            if(isset($brands[ $categories[0] ]) && count( $brands[ $categories[0] ] ) === 1) {

                  if(isset($models[ $brands[ $categories[0] ][0] ]) && count($models[ $brands[ $categories[0] ][0] ]) === 1) {

                        return array(
                              'repairs', 
                              array( 
                                    rp_get_c_name_by_c_id( $categories[0] ), 
                                    rp_get_b_name_by_b_id( $brands[ $categories[0] ][0] ),
                                    rp_get_m_name_by_m_id( $models[ $brands[ $categories[0] ][0] ][0] )
                              )
                        );

                  } else {

                        return array(
                              'models', 
                              array( 
                                    rp_get_c_name_by_c_id( $categories[0] ), 
                                    rp_get_b_name_by_b_id( $brands[ $categories[0] ][0] )
                              )
                        );

                  }

            } else {

                  return array(
                        'brands', 
                        array( 
                              rp_get_c_name_by_c_id( $categories[0] ) 
                        )
                  );

            }

      } else {

            return array('types', array());

      }

}

function rp_get_all_model_repair_ids($m_id = NULL) {

      if(empty($m_id)) {
            return array();
      }

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $repairs = $rpQuery->get_results("SELECT r_uq_id FROM `$rs_repair` WHERE m_id_fk = '".rp_escape_sql($m_id)."'");

      $ids = array();

      if(!empty($repairs)) {

            foreach ($repairs as $repair) {
                  
                  $ids[] = trim($repair->r_uq_id);

            }

      }

      return $ids;

}


function rp_get_all_model_color_ids($m_id = NULL) {

      if(empty($m_id)) {
            return array();
      }

      global $rpQuery;

      $rs_model_color = $rpQuery->prefix . 'rs_model_color';

      $colors = $rpQuery->get_results("SELECT mc_uq_id FROM `$rs_model_color` WHERE m_id_fk = '".rp_escape_sql($m_id)."'");

      $ids = array();

      if(!empty($colors)) {

            foreach ($colors as $color) {
                  
                  $ids[] = trim($color->mc_uq_id);

            }

      }

      return $ids;

}


/* Function to Easily Add Column To Table if not exists*/
function rp_add_column_to_table_if_not_exists( $table_name = '', $column_name = '', $dataType = '', $prefix = TRUE ) {

      global $rpQuery;

      if($prefix === TRUE) {
            $table_name = $rpQuery->prefix . $table_name;
      }

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($table_name)."' AND COLUMN_NAME = '".rp_escape_sql($column_name)."';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$table_name` ADD `$column_name` $dataType;");

            return TRUE;
      }

      return FALSE;
}

/*
1. Add dr_category column to rs_default_repair table
2. Split Default Repairs Based on Category
*/

function rp_fix_default_repairs_based_on_category() {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $categories = $rpQuery->get_results("SELECT * FROM $rs_category");

      $categories_ids = array();

      if(!empty($categories)) {

            foreach ($categories as $category) {
                  
                  $categories_ids[] = $category->c_id;

            }

            // Start Work From Here!!!! Need TO Migrate Default Repairs!!!

            $org_default_repairs = $rpQuery->get_results("SELECT * FROM $rs_default_repair");

            $org_default_repairs_attrs = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr");

            if(!empty( $org_default_repairs )) {

                  foreach ($categories_ids as $cat_id) {
                        
                        foreach ($org_default_repairs as $default_repair) {

                              $dr_id = $default_repair->dr_id;
                              
                              $for_insert = array();

                              foreach ($default_repair as $key => $value) {
                                    
                                    $for_insert[$key] = $value;

                              }

                              unset($for_insert['dr_id']);

                              $for_insert['dr_category'] = $cat_id;

                              $rpQuery->insert($rs_default_repair, $for_insert);

                              $new_dr_id = $rpQuery->insert_id;

                              $related_default_repairs_attrs = $rpQuery->get_results("SELECT * FROM $rs_default_repair_attr WHERE `dr_id_fk` = '".rp_escape_sql($dr_id)."'");

                              if( !empty( $related_default_repairs_attrs ) ) {

                                    foreach ($related_default_repairs_attrs as $related_attr) {
                                          
                                          $da_id = $related_attr->da_id;
                                          
                                          $for_insert = array();

                                          foreach ($related_attr as $key => $value) {
                                                
                                                $for_insert[$key] = $value;

                                          }

                                          unset($for_insert['da_id']);

                                          $for_insert['dr_id_fk'] = $new_dr_id;

                                          $for_insert['dr_category'] = $cat_id;

                                          $rpQuery->insert($rs_default_repair_attr, $for_insert);

                                    }

                              }

                        }

                  }

            }

            if(!empty($org_default_repairs)) {

                  $delete_ids = array();

                  foreach ($org_default_repairs as $row) {
                        
                        $delete_ids[] = $row->dr_id;

                  }

                  $delete_ids_str = join(',', $delete_ids);

                  $rpQuery->query("DELETE FROM $rs_default_repair WHERE dr_id IN (".rp_escape_sql($delete_ids_str).")");

            }

            if(!empty($org_default_repairs_attrs)) {

                  $delete_ids = array();

                  foreach ($org_default_repairs_attrs as $row) {
                        
                        $delete_ids[] = $row->da_id;

                  }

                  $delete_ids_str = join(',', $delete_ids);

                  $rpQuery->query("DELETE FROM $rs_default_repair_attr WHERE da_id IN (".rp_escape_sql($delete_ids_str).")");

            }

      }

}

function rp_fix_images_in_default_repairs_table() {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $entries = $rpQuery->get_results("SELECT dr_uq_id FROM `$rs_default_repair` WHERE dr_img LIKE '%base.repairplugin.com%'");

      if(!empty($entries)) {

            foreach ($entries as $entry) {
                  
                  $allReapirs = $rpQuery->get_results("SELECT r_img FROM `$rs_repair` WHERE r_uq_id = '".rp_escape_sql($entry->dr_uq_id)."'");

                  if(!empty($allReapirs)) {

                        $repeatitiveImage = array();

                        foreach ($allReapirs as $repair) {
                              
                              if(!isset( $repeatitiveImage[ $repair->r_img ] )) {
                                    $repeatitiveImage[ $repair->r_img ] = 0; 
                              }

                              $repeatitiveImage[ $repair->r_img ]++;

                        }

                        arsort($repeatitiveImage);

                        $best_image = array_keys($repeatitiveImage)[0];

                        $best_image = $best_image;

                        $rpQuery->query("UPDATE `$rs_default_repair` SET `dr_img` = '".rp_escape_sql($best_image)."' WHERE dr_uq_id = '".rp_escape_sql($entry->dr_uq_id)."'");

                        $rpQuery->query("UPDATE `$rs_repair` SET `r_img` = '".rp_escape_sql($best_image)."' WHERE r_uq_id = '".rp_escape_sql($entry->dr_uq_id)."'");

                  }

            }

      }

}

function add_dr_category_column_to_rs_default_repair_table() {

      global $rpQuery;

      rp_add_column_to_table_if_not_exists( 'rs_default_repair_attr', 'dr_category', 'int(11) NOT NULL' );

      $created = rp_add_column_to_table_if_not_exists( 'rs_default_repair', 'dr_category', 'int(11) NOT NULL' );

      if($created === TRUE && rp_get_option('rp_fix_default_repairs_based_on_category', FALSE) === FALSE) {

            rp_add_option('rp_fix_default_repairs_based_on_category', 'done');

            $rpQuery->query("START TRANSACTION");

            rp_fix_images_in_default_repairs_table();

            rp_fix_default_repairs_based_on_category();

            $rpQuery->query("COMMIT");

      }

}

add_action('init', 'add_dr_category_column_to_rs_default_repair_table', 999);



function rp_get_lowest_repair_attribute_price($r_id = 0) {

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $result = $rpQuery->get_results("SELECT * FROM `$rs_repair_attr` WHERE `r_id_fk` = '".rp_escape_sql($r_id)."'");

      $prices = array();

      foreach ($result as $row) {
            
            $prices[] = (float) str_replace( ',', '.', $row->a_price );

      }

      if(empty($prices)) {

            return 0;

      }

      return min($prices);

}


function rp_get_lowest_default_repair_attribute_price($dr_id = 0) {

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $result = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE `dr_id_fk` = '".rp_escape_sql($dr_id)."'");

      $prices = array();

      foreach ($result as $row) {
            
            $prices[] = (float) str_replace( ',', '.', $row->da_price );

      }

      if(empty($prices)) {

            return 0;

      }

      return min($prices);

}


function _rp_get_all_default_repair_ids_of_category( $category = '' ) {

      $cache_key = 'rp_get_all_default_repair_ids_of_category|'.$category;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $result = $rpQuery->get_results("SELECT dr_id FROM `$rs_default_repair` WHERE dr_category = '".rp_escape_sql($category)."'");

      $dr_ids = array();

      if(!empty($result)) {

            foreach ($result as $drepair) {
                  
                  $dr_ids[] = $drepair->dr_id;

            }

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $dr_ids);

      return $dr_ids;

}

function rp_get_all_default_repair_ids_of_category( $category = '', $join = FALSE ) {

      $dr_ids = _rp_get_all_default_repair_ids_of_category( $category );

      if($join === TRUE) {

            if(empty($dr_ids)) {

                  return '';

            }

            $dr_ids = join(',',$dr_ids);

      }

      return $dr_ids;

}

function rp_setup_guide_meet_download_libraries_requirement() {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';

      $rs_brand = $rpQuery->prefix . 'rs_brand';

      $rs_model = $rpQuery->prefix . 'rs_model';

      $required_tables = array( $rs_category, $rs_brand, $rs_model );

      $meet_requirement = TRUE;

      foreach ($required_tables as $table) {
            
            if(empty($rpQuery->get_results("SELECT * FROM $table"))) {

                  $meet_requirement = FALSE;

            }
      }

      return $meet_requirement;

}


function rp_setup_guide_meet_default_repairs_requirement() {

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $required_tables = array( $rs_default_repair );

      $meet_requirement = TRUE;

      foreach ($required_tables as $table) {
            
            if(empty($rpQuery->get_results("SELECT * FROM $table"))) {

                  $meet_requirement = FALSE;

            }
      }

      return $meet_requirement;

}


function rp_setup_guide_meet_manage_upsales_requirement() {

      global $rpQuery;

      $rs_upsales = $rpQuery->prefix . 'rs_upsales';

      $required_tables = array( $rs_upsales );

      $meet_requirement = TRUE;

      foreach ($required_tables as $table) {
            
            if(empty($rpQuery->get_results("SELECT * FROM $table WHERE is_active = '1'"))) {

                  $meet_requirement = FALSE;

            }
      }

      return $meet_requirement;

}


function rp_setup_guide_meet_configure_settings_requirement() {

      global $rpQuery;

      $rs_company_information = $rpQuery->prefix . 'rs_company_information';

      $result = $rpQuery->get_results("SELECT * FROM $rs_company_information");

      if(!empty($result)) {

            if($result[0]->company_name === 'abc' || $result[0]->company_email === rp_get_settings_placeholder_email()) {

                  return FALSE;

            }

            return TRUE;

      }

      return FALSE;
}

function rp_keep_setup_tour_nav_uptodate_part() {

      $fileName = WP_REPAIR_PLUGIN_PATH.'html/setup_tour_nav_uptodate.php';

      $fileName = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $fileName);

      require_once $fileName;

}


function rp_keep_setup_tour_nav_uptodate() {

      rp_keep_setup_tour_nav_uptodate_part();

      exit;
      
}

rp_ajax_for_admin('rp_keep_setup_tour_nav_uptodate', 'rp_keep_setup_tour_nav_uptodate');


function rp_skip_quick_start_wizard() {

      if( rp_get_option('rp_skip_quick_start_wizard', FALSE) === FALSE ) {

            rp_add_option('rp_skip_quick_start_wizard', TRUE);

      }

}

rp_ajax_for_admin('rp_skip_quick_start_wizard', 'rp_skip_quick_start_wizard');

function rp_quick_search_database($selectWhat = '', $tableName = '', $columnName = '', $searchWith = '=', $searchFor = '', $prefix = TRUE) {

      global $rpQuery;

      if($prefix === TRUE) {

            $tableName = $rpQuery->prefix . $tableName;

      }

      return $rpQuery->get_results("SELECT $selectWhat FROM `$tableName` WHERE `$columnName` $searchWith '$searchFor'");

} 

function rp_add_default_upsales_rows() {

      /* Added in V1.4.2 */

      global $rpQuery;

      // If NOT Already Added!
      if(rp_get_option('rp_added_default_upsales_rows', FALSE) === FALSE) {

            $default_upsales_img_directory = WP_REPAIR_PLUGIN_URL . 'assets/images/upsales/';

            // 100% Confirm If NOT Already Added!
            if( empty( rp_quick_search_database('upsales_id', 'rs_upsales', 'upsales_img', 'LIKE', '%'.rp_escape_sql( $default_upsales_img_directory ).'%') ) ) {

                  $upsales = array();

                  $upsale_row = array();
                  $upsale_row['us_name'] = 'Mobile Safety Glass';
                  $upsale_row['us_info'] = 'Bescherm het scherm met deze keiharde screen protector.';
                  $upsale_row['us_text'] = 'Bescherm het scherm met deze keiharde screen protector.';
                  $upsale_row['us_old_price'] = '0';
                  $upsale_row['us_new_price'] = '19.95';
                  $upsale_row['is_active'] = '0';
                  $upsale_row['date_time'] = date('Y-m-d H:i:s');
                  $upsale_row['upsales_position'] = '1';
                  $upsale_row['upsales_img'] = $default_upsales_img_directory . 'glassprotector.jpg';
                  $upsales[] = $upsale_row;

                  $upsale_row = array();
                  $upsale_row['us_name'] = 'Apple oplaad kabel';
                  $upsale_row['us_info'] = 'De nieuwe standaard aansluiting, speciaal voor deze iPhone.';
                  $upsale_row['us_text'] = 'De nieuwe standaard aansluiting, speciaal voor deze iPhone.';
                  $upsale_row['us_old_price'] = '0';
                  $upsale_row['us_new_price'] = '25';
                  $upsale_row['is_active'] = '0';
                  $upsale_row['date_time'] = date('Y-m-d H:i:s');
                  $upsale_row['upsales_position'] = '2';
                  $upsale_row['upsales_img'] = $default_upsales_img_directory . 'usb-c-1.jpg';
                  $upsales[] = $upsale_row;

                  $upsale_row = array();
                  $upsale_row['us_name'] = 'Apple 20w oplaad adapter';
                  $upsale_row['us_info'] = 'Apple 20w oplaad adapter';
                  $upsale_row['us_text'] = 'Laadt je toestel razendsnel op met deze 20W adapter.';
                  $upsale_row['us_old_price'] = '0';
                  $upsale_row['us_new_price'] = '25';
                  $upsale_row['is_active'] = '0';
                  $upsale_row['date_time'] = date('Y-m-d H:i:s');
                  $upsale_row['upsales_position'] = '3';
                  $upsale_row['upsales_img'] = $default_upsales_img_directory . 'Apple_USB_C_lichtnetadapter_20w.jpg';
                  $upsales[] = $upsale_row;

                  foreach ($upsales as $single_upsale_row) {
                        
                        $rpQuery->insert($rpQuery->prefix . 'rs_upsales', $single_upsale_row);

                  }

            }

            rp_add_option('rp_added_default_upsales_rows', '1');

      }

}

add_action('init', 'rp_add_default_upsales_rows', 999);


function rp_add_appointment_price_tracking_column() {

      /* Added in V1.4.2 */

      if(rp_get_option('rp_added_appointment_price_tracking_column', FALSE) === FALSE) {

            rp_add_column_to_table_if_not_exists('rs_order_upsales', 'upsale_name', 'varchar(255) NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_order_upsales', 'upsale_price', 'float NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_order_repairs', 'repair_name', 'varchar(255) NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_order_repairs', 'repair_price', 'float NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_order_repair_attr', 'repair_attr_name', 'varchar(255) NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_order_repair_attr', 'repair_attr_price', 'float NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_order_repair_attr', 'r_id_fk', 'int(11) NULL DEFAULT NULL');

            rp_add_option('rp_added_appointment_price_tracking_column', '1');

      }

}

add_action('init', 'rp_add_appointment_price_tracking_column', 999);

function rp_first_time_update_rs_order_repairs($order_id = 0) {

      /* Added in V1.4.2 */
      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $related_repairs = rp_get_where_query('rs_order_repairs', 'o_id_fk', $order_id);
      
      foreach ($related_repairs as $order_repair) {

            if(!empty($order_repair->repair_name)) {
                  //Already Set
                  continue;

            }
            
            list( $relatedRepairName, $relatedRepairPrice ) = rp_get_repair_lock_price($order_repair->r_id_fk);

            $rpQuery->query("UPDATE `$rs_order_repairs` SET `repair_name` = '".rp_escape_sql($relatedRepairName)."', `repair_price` = '".rp_escape_sql($relatedRepairPrice)."' WHERE `or_id` = '".rp_escape_sql($order_repair->or_id)."'");

      }

}

function rp_first_time_update_rs_order_repair_attr($order_id = 0) {

      /* Added in V1.4.2 */
      global $rpQuery;

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $related_repair_attrs = rp_get_where_query('rs_order_repair_attr', 'o_id_fk', $order_id);
      
      foreach ($related_repair_attrs as $order_repair_attr) {

            if(!empty($order_repair_attr->repair_attr_name)) {
                  //Already Set
                  continue;

            }
            
            list($relatedRepairAttrName, $relatedRepairAttrPrice, $relatedRepairId) = rp_get_repair_attr_lock_price($order_repair_attr->ra_id_fk);

            $rpQuery->query("UPDATE `$rs_order_repair_attr` SET `repair_attr_name` = '".rp_escape_sql($relatedRepairAttrName)."', `repair_attr_price` = '".rp_escape_sql($relatedRepairAttrPrice)."', `r_id_fk` = '".rp_escape_sql($relatedRepairId)."' WHERE `ora_id` = '".rp_escape_sql($order_repair_attr->ora_id)."'");

      }

}


function rp_first_time_update_rs_order_upsales($order_id = 0) {

      /* Added in V1.4.2 */
      global $rpQuery;

      $rs_order_upsales = $rpQuery->prefix . 'rs_order_upsales';

      $related_upsales = rp_get_where_query('rs_order_upsales', 'o_id_fk', $order_id);
      
      foreach ($related_upsales as $order_upsale) {

            if(!empty($order_upsale->upsale_name)) {
                  //Already Set
                  continue;

            }
            
            list( $relatedUpsaleName, $relatedUpsalePrice ) = rp_get_upsale_lock_price($order_upsale->upsales_id_fk);

            $rpQuery->query("UPDATE `$rs_order_upsales` SET `upsale_name` = '".rp_escape_sql($relatedUpsaleName)."', `upsale_price` = '".rp_escape_sql($relatedUpsalePrice)."' WHERE `ou_id` = '".rp_escape_sql($order_upsale->ou_id)."'");

      }

}



function rp_lazy_lock_first_time_repair_price() {

      /* Added in V1.4.2 */
      global $rpQuery;

      $lock_previous_appointments = array();

      $lock = rp_get_option('rp_lock_previous_appointments_prices', FALSE);

      $added_just_now = FALSE;

      if($lock === FALSE) {

            // Need To Lock Previous Appointments...
            $rs_orders = $rpQuery->prefix . 'rs_orders';

            $orders = $rpQuery->get_results("SELECT o_id FROM `$rs_orders`");

            $order_ids = array();

            foreach ($orders as $order) {
                  
                  $order_ids['order_id_'.$order->o_id] = '0';

            }

            $lock_previous_appointments = $order_ids;

            $added_just_now = TRUE;

            rp_add_option('rp_lock_previous_appointments_prices', $order_ids);

      } else {

            if(!empty($lock)) {

                  $lock_previous_appointments = $lock;

            }

      }

      if($added_just_now === FALSE) {

            $unprocessed = array();

            foreach ($lock_previous_appointments as $key => $value) {

                  if($value == '0') {

                        $unprocessed[] = $key;

                  }

            }

            if(!empty($unprocessed)) {

                  $unprocessed = $unprocessed[ (count($unprocessed) - 1) ]; // Last Item

                  $order_id = (int) str_replace('order_id_', '', $unprocessed);

                  // Process the row here!
                  // Lock Repairs Price
                  rp_first_time_update_rs_order_repairs($order_id);

                  // Lock Repair Attributes Price
                  rp_first_time_update_rs_order_repair_attr($order_id);

                  // Lock Upsales Price
                  rp_first_time_update_rs_order_upsales($order_id);

                  // Mark Item as Proccessed!
                  $lock_previous_appointments[$unprocessed] = '1';

                  // Update The Option!
                  rp_update_option('rp_lock_previous_appointments_prices', $lock_previous_appointments);

            }

      }

}

add_action('init', 'rp_lazy_lock_first_time_repair_price', 999);


function rp_find_and_fix_duplicated_model_colors() {

      /* Added in V1.4.2 */

      global $rpQuery;

      $rs_model_color = $rpQuery->prefix . 'rs_model_color';

      $row = $rpQuery->get_row( "SELECT mc_uq_id, color_name, m_id_fk, COUNT(*) AS count FROM $rs_model_color GROUP BY mc_uq_id, color_name, m_id_fk HAVING COUNT(*) > 1 LIMIT 1;" );

      if( !empty( $row ) ) {

            $delete_limit = ((int) $row->count) - 1; // 2 - 1 = 1

            $rpQuery->query( "DELETE FROM $rs_model_color WHERE mc_uq_id = '{$row->mc_uq_id}' AND m_id_fk = {$row->m_id_fk} AND color_name = '{$row->color_name}' ORDER BY mc_id DESC LIMIT $delete_limit" );

      }

}

add_action('init', 'rp_find_and_fix_duplicated_model_colors');



/*
Add r_duration_measure column to rs_repair table
*/

function add_r_duration_measure_column_to_rs_repair() {
      /* Added in V1.4.3 */

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair)."' AND COLUMN_NAME = 'r_duration_measure';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair` ADD `r_duration_measure` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_r_duration_measure_column_to_rs_repair', 999);

/*
Add dr_duration_measure column to rs_default_repair table
*/

function add_dr_duration_measure_column_to_rs_default_repair() {
      /* Added in V1.4.3 */

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair)."' AND COLUMN_NAME = 'dr_duration_measure';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair` ADD `dr_duration_measure` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_dr_duration_measure_column_to_rs_default_repair', 999);

/*
Add Location Email Column
*/

function add_location_email_column_to_location_table() {
      /* Added in V1.4.3 */

      global $rpQuery;

      $rs_company_location = $rpQuery->prefix . 'rs_company_location';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_company_location)."' AND COLUMN_NAME = 'location_email';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_company_location` ADD `location_email` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_location_email_column_to_location_table', 999);

/*
Get uq_id of all rs_languages
*/

function rp_get_uq_id_of_all_rs_languages() {

      global $rpQuery;

      $rs_languages = $rpQuery->prefix . 'rs_languages';

      $results = $rpQuery->get_results("SELECT `uq_id` FROM `$rs_languages`");

      $all_uq_ids = array();

      if( !empty( $results ) ) {

            foreach( $results as $row ) {

                  $all_uq_ids[] = $row->uq_id;

            }

      }

      return $all_uq_ids;

}


/*
Create RepairPlugin Languages Table
*/
function repair_plugin_languages_table(){
      /* Added in V1.4.4 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_languages = $rpQuery->prefix . "rs_languages"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_languages (

    `id` int(11) NOT NULL AUTO_INCREMENT,

    `key` varchar(255) NOT NULL,

    `name` varchar(255) NOT NULL,

    `alt_name` varchar(255) NOT NULL,

    `image` varchar(255) NOT NULL,

    `position` int(11) NULL DEFAULT NULL,

    PRIMARY KEY (`id`))

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_languages_table');


/*
Add Default Languages in Languages Table
*/
function repair_plugin_add_languages_in_table(){
      /* Added in V1.4.4 */
      /* Revised in V1.5.0 */

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . "rs_languages"; 

      if( rp_get_option('rp_added_first_time_languages_rows', FALSE) === FALSE ) {

            rp_add_option('rp_added_first_time_languages_rows', '1');

            $row = $rpQuery->get_row("SELECT id FROM `$table_rs_languages` WHERE `key` = 'english_united_states' LIMIT 1;");

            if( empty( $row ) ) {

                  $rpQuery->query("INSERT INTO `$table_rs_languages` (`key`, `name`, `alt_name`, `image`, `position`) VALUES('english_united_states', 'English (United States)', 'English (US)', 'assets/images/english_united_states.png', '2')");

            }

            $row = $rpQuery->get_row("SELECT id FROM `$table_rs_languages` WHERE `key` = 'dutch_netherlands' LIMIT 1;");

            if( empty( $row ) ) {

                  $rpQuery->query("INSERT INTO `$table_rs_languages` (`key`, `name`, `alt_name`, `image`, `position`) VALUES('dutch_netherlands', 'Dutch (Netherlands)', 'Netherlands', 'assets/images/dutch_netherlands.png', '1')");

            }

            $rpQuery->query("UPDATE `$table_rs_languages` SET `alt_name` = 'English (US)' WHERE `alt_name` LIKE 'English (United States)';");

            $rpQuery->query("UPDATE `$table_rs_languages` SET `alt_name` = 'Netherlands' WHERE `alt_name` LIKE 'Dutch (Netherlands)';");

      }

}

add_action('init', 'repair_plugin_add_languages_in_table', 999);


function rp_get_all_languages(){
      /* Added in V1.4.4 */

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . "rs_languages"; 

      return $rpQuery->get_results("SELECT * FROM `$table_rs_languages` ORDER BY `position`;");

}

function rp_get_enabled_languages_old() {

      /* Added in V1.4.4 */

      $all_languages = rp_get_all_languages();

      $all_languages_name = array();

      foreach ($all_languages as $lang_row) {

            $all_languages_name[] = $lang_row->name;

      }

      return rp_get_option('rp_get_enabled_languages', $all_languages_name);

}

function rp_get_enabled_languages() {

      /* REVISED in V1.5.0 */

      $all_languages = rp_get_all_languages();

      $enabled_languages_names = array();

      $all_languages_name = array();

      foreach ($all_languages as $lang_row) {

            if( $lang_row->is_active == '1' ) {

                  $enabled_languages_names[] = $lang_row->name;

            }

            $all_languages_name[] = $lang_row->name;

      }

      if( !empty($enabled_languages_names) ) {

            return $enabled_languages_names;

      }

      return $all_languages_name;

}

function rp_get_rows_of_enabled_languages() {

      /* Added in V1.4.4 */

      $enabled_languages = rp_get_enabled_languages();

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . "rs_languages"; 

      $where_in = array();

      foreach ($enabled_languages as $lang_name) {
            
            $where_in[] = rp_escape_sql( $lang_name );

      }

      $where_in_str = "'". join("', '", $where_in) . "'";

      return $rpQuery->get_results("SELECT * FROM `$table_rs_languages` WHERE `name` IN ($where_in_str) ORDER BY position ASC;");

}

function rp_change_frontend_language() {

      $selected_language = $_POST['selected_language'] ?? '';

      if( !empty( $selected_language ) ) {

            RepairPluginPro\RP_Session_Handler::start();

            $_SESSION['rp_selected_language'] = $selected_language;

            RepairPluginPro\RP_Session_Handler::close();

      }

      echo json_encode( array( 'status' => true, 'message' => 'Language successfully selected!' ) );

      exit;

}

rp_ajax_for_public('rp_change_frontend_language', 'rp_change_frontend_language');

function rp_fix_addon_switch_settings_on_backend() {

    RepairPluginPro\RP_Session_Handler::read();

    RepairPluginPro\RP_Session_Handler::start();

    $_SESSION['rp_filter_is_active_enhanced_locations_addon'] = '1';

    $_SESSION['rp_filter_is_enabled_dynamic_pricing'] = '1';

    RepairPluginPro\RP_Session_Handler::close();

}

function rp_get_shortcode_page_link() {

      global $rpQuery;

      $shortcode_page = $rpQuery->get_results("SELECT * FROM {$rpQuery->posts} WHERE `post_type` = 'page' AND `post_content` LIKE '%[Repair_All]%' AND `post_status` = 'publish'");

      if(!empty($shortcode_page)) {

            return get_permalink($shortcode_page[0]->ID);

      } else {

            $shortcode_page_childs = $rpQuery->get_results("SELECT * FROM {$rpQuery->posts} WHERE `post_type` = 'revision' AND `post_content` LIKE '%[Repair_All]%' AND `post_status` = 'inherit'");

            if(!empty($shortcode_page_childs)) {

                  $post_ids = array();

                  foreach ($shortcode_page_childs as $child) {

                        if(array_search($child->post_parent, $post_ids) === FALSE) {

                              $post_ids[] = $child->post_parent;

                        }

                  }

                  $post_ids_str = join(',', $post_ids);

                  $shortcode_page = $rpQuery->get_results("SELECT * FROM {$rpQuery->posts} WHERE `ID` IN (".rp_escape_sql($post_ids_str).") AND `post_type` = 'page' AND `post_status` = 'publish'");

                  if(!empty($shortcode_page)) {

                        return get_permalink($shortcode_page[0]->ID);

                  }

            }

      }

      return '';

}

function rp_get_specific_shortcode_page_link( $m_uq_id = '' ) {

      global $rpQuery;

      $shortcode_page = $rpQuery->get_results("SELECT * FROM {$rpQuery->posts} WHERE `post_type` = 'page' AND `post_content` LIKE '%[models code=$m_uq_id]%' AND `post_status` = 'publish'");

      if(!empty($shortcode_page)) {

            return get_permalink($shortcode_page[0]->ID);

      } else {

            $shortcode_page_childs = $rpQuery->get_results("SELECT * FROM {$rpQuery->posts} WHERE `post_type` = 'revision' AND `post_content` LIKE '%[models code=$m_uq_id]%' AND `post_status` = 'inherit'");

            if(!empty($shortcode_page_childs)) {

                  $post_ids = array();

                  foreach ($shortcode_page_childs as $child) {

                        if(array_search($child->post_parent, $post_ids) === FALSE) {

                              $post_ids[] = $child->post_parent;

                        }

                  }

                  $post_ids_str = join(',', $post_ids);

                  $shortcode_page = $rpQuery->get_results("SELECT * FROM {$rpQuery->posts} WHERE `ID` IN (".rp_escape_sql($post_ids_str).") AND `post_type` = 'page' AND `post_status` = 'publish'");

                  if(!empty($shortcode_page)) {

                        return get_permalink($shortcode_page[0]->ID);

                  }

            }

      }

      return '';

}

function rp_get_por_repairs_by_order_id( $order_id = '' ) {

      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $order_repairs = $rpQuery->get_results("SELECT * FROM `$rs_order_repairs` WHERE `o_id_fk` = '{$order_id}';");

      if( !empty( $order_repairs ) ) {

            $r_id_fk = array();

            foreach( $order_repairs as $row ) {

                  $repair_attrs = $rpQuery->get_results("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '{$order_id}' AND `r_id_fk` = '".rp_escape_sql($row->r_id_fk)."';");

                  if( empty( $repair_attrs ) && ((float) $row->repair_price) <= 0 ) {

                        $r_id_fk[] = $row->r_id_fk;

                  }

            }

            if( !empty( $r_id_fk ) ) {

                  $r_id_fk = join(',', $r_id_fk);

                  $price_on_request = $rpQuery->get_results("SELECT * FROM `$rs_repair` WHERE `r_id` IN ({$r_id_fk}) AND `r_price` = '0' AND `is_free` != '1';");

                  return apply_filters('returning_por_repairs', $price_on_request, $order_id);

            }

      }

      return apply_filters('returning_por_repairs', array(), $order_id);

}

function rp_check_is_por_repair( $needle = '', $haystack = array() ) {

      if(empty($haystack)) {
            return FALSE;
      }

      /* Added in V1.4.4 */

      foreach( $haystack as $row ) {

            if( ((int) $row->r_id) === ((int) $needle) ) {

                  return TRUE;

            }

      }

      return FALSE;

}


function rp_get_por_repair_attr_by_order_id( $order_id = '' ) {

      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $order_repairs = $rpQuery->get_results("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '{$order_id}';");

      if( !empty( $order_repairs ) ) {

            $ra_id_fk = array();

            foreach( $order_repairs as $row ) {

                  if( ((float) $row->repair_attr_price) <= 0 ) {

                        $ra_id_fk[] = $row->ra_id_fk;

                  }

            }

            if( !empty( $ra_id_fk ) ) {

                  $ra_id_fk = join(',', $ra_id_fk);

                  $price_on_request = $rpQuery->get_results("SELECT * FROM `$rs_repair_attr` WHERE `a_id` IN ({$ra_id_fk}) AND `a_price` = '0';");

                  return apply_filters('returning_por_repair_attrs', $price_on_request, $order_id);

            }

      }

      return apply_filters('returning_por_repair_attrs', array(), $order_id);

}

function rp_check_is_por_repair_attr( $needle = '', $haystack = array() ) {

      if(empty($haystack)) {
            return FALSE;
      }

      /* Added in V1.4.4 */

      foreach( $haystack as $row ) {

            if( ((int) $row->a_id) === ((int) $needle) ) {

                  return TRUE;

            }

      }

      return FALSE;

}

function rp_get_dynamic_pricing_por_repair_attr( $o_id = '' ) {

      global $rpQuery;

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      // select * where o_id_fk = $o_id

      $repair_attrs = $rpQuery->get_results("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '{$o_id}';");

      $a_ids = array();

      if( !empty( $repair_attrs ) ) {

            foreach( $repair_attrs as $attrRow ) {

                  if( property_exists( $attrRow, 'pricing_method' ) && $attrRow->pricing_method === 'price_on_request' ) {

                        $a_ids[] = $attrRow->ra_id_fk;
      
                  }

            }

      }

      return $a_ids;

}

function rp_get_dynamic_pricing_por_repairs( $o_id = '' ) {

      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      // select * where o_id_fk = $o_id

      $order_repairs = $rpQuery->get_results("SELECT * FROM `$rs_order_repairs` WHERE `o_id_fk` = '{$o_id}';");

      $r_ids = array();

      if( !empty( $order_repairs ) ) {

            // loop through all order repairs and check if $order_repairs->r_id_fk AND $o_id matches in rs_order_repair_attr
            foreach( $order_repairs as $oRepairRow ) {

                  // select * where o_id_fk = $o_id AND r_id_fk = $order_repairs->r_id_fk

                  $repair_attrs = $rpQuery->get_results("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '{$o_id}' AND `r_id_fk` = '".rp_escape_sql($oRepairRow->r_id_fk)."';");

                  if( empty( $repair_attrs ) ) {

                        if( property_exists( $oRepairRow, 'pricing_method' ) && $oRepairRow->pricing_method === 'price_on_request' ) {

                              $r_ids[] = $oRepairRow->r_id_fk;

                        }

                  }

            }

      }

      return $r_ids;

}

function rp_update_por_repairs_price() {

      rp_verify_csrf_token_for_ajax('appointments');

      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $oid = $_POST['appointment_id'] ?? '';

      $all_rids = $_POST['rid'] ?? array();

      $all_aids = $_POST['aid'] ?? array();

      $r_price = $_POST['r_price'] ?? array();

      if( !empty( $oid ) && !empty( $all_rids ) && !empty( $all_aids ) && !empty( $r_price ) && is_array( $all_rids ) && is_array( $r_price ) && count( $all_rids ) > 0 && count($all_rids) === count( $r_price ) ) {

            $appointment_por_repairs = rp_get_por_repairs_by_order_id( $oid );

            $appointment_por_repair_attrs = rp_get_por_repair_attr_by_order_id( $oid );

            $dynamic_pricing_por_repairs = rp_get_dynamic_pricing_por_repairs( $oid );

            $dynamic_pricing_por_repair_attr = rp_get_dynamic_pricing_por_repair_attr( $oid );

            $affected_orders = array();

            foreach( $all_rids as $index => $rid ) {

                  if( !empty( $all_aids[ $index ] ) ) {

                        // Is repair attribute
                        continue;

                  }

                  $the_price = $r_price[ $index ];

                  if( preg_match('/,/', $the_price) ) {

                      $the_price = str_replace(',', '.', $the_price);

                  }

                  $order_ids = array();

                  if( rp_check_is_por_repair( $rid, $appointment_por_repairs ) ) {

                        $order_ids[] = $oid;

                        if( !in_array( $rid, $dynamic_pricing_por_repairs ) ) {

                              $current_less_7_days = date('Y-m-d H:i:s', strtotime( date('Y-m-d H:i:s') . ' -7 days' ));

                              $related_rows = $rpQuery->get_results("SELECT rs_o.o_id FROM `$rs_order_repairs` AS rs_or JOIN `$rs_orders` AS rs_o ON rs_or.o_id_fk = rs_o.o_id WHERE rs_or.r_id_fk = '$rid' AND rs_o.o_date_time >= '$current_less_7_days' AND rs_or.repair_price = '0';");

                              if( !empty( $related_rows ) ) {

                                    foreach( $related_rows as $r_row ) {

                                          if( array_search( $r_row->o_id, $order_ids ) === FALSE ) {

                                                $order_ids[] = $r_row->o_id;

                                          }

                                    }

                              }

                        }

                  }

                  if( !empty( $order_ids ) ) {

                        $order_ids_str = join(',', $order_ids);

                        if( !in_array( $rid, $dynamic_pricing_por_repairs ) ) {

                              // Update Model Related Price
                              $rpQuery->query("UPDATE `$rs_repair` SET `r_price` = '{$the_price}' WHERE `r_id` = '$rid';");

                        }

                        // Update Locked Prices
                        $rpQuery->query("UPDATE `$rs_order_repairs` SET `repair_price` = '{$the_price}' WHERE `r_id_fk` = '$rid' AND `o_id_fk` IN ({$order_ids_str})");

                        foreach( $order_ids as $_o_id ) {

                              if( array_search( $_o_id, $affected_orders ) === FALSE ) {

                                    $affected_orders[] = $_o_id;

                              }

                        }

                  }

            }

            foreach( $all_aids as $index => $a_id ) {

                  if( empty( $a_id ) ) {

                        continue;

                  }

                  $the_price = $r_price[ $index ];

                  if( preg_match('/,/', $the_price) ) {

                      $the_price = str_replace(',', '.', $the_price);

                  }

                  $order_ids = array();

                  if( rp_check_is_por_repair_attr( $a_id, $appointment_por_repair_attrs ) ) {

                        $order_ids[] = $oid;

                        if( !in_array( $a_id, $dynamic_pricing_por_repair_attr ) ) {

                              $current_less_7_days = date('Y-m-d H:i:s', strtotime( date('Y-m-d H:i:s') . ' -7 days' ));

                              $related_rows = $rpQuery->get_results("SELECT rs_o.o_id FROM `$rs_order_repair_attr` AS rs_or JOIN `$rs_orders` AS rs_o ON rs_or.o_id_fk = rs_o.o_id WHERE rs_or.ra_id_fk = '$a_id' AND rs_o.o_date_time >= '$current_less_7_days' AND rs_or.repair_attr_price = '0';");

                              if( !empty( $related_rows ) ) {

                                    foreach( $related_rows as $r_row ) {

                                          if( array_search( $r_row->o_id, $order_ids ) === FALSE ) {

                                                $order_ids[] = $r_row->o_id;

                                          }

                                    }

                              }

                        }

                  }

                  if( !empty( $order_ids ) ) {

                        $order_ids_str = join(',', $order_ids);

                        if( !in_array( $a_id, $dynamic_pricing_por_repair_attr ) ) {

                              // Update Model Related Price
                              $rpQuery->query("UPDATE `$rs_repair_attr` SET `a_price` = '{$the_price}' WHERE `a_id` = '$a_id';");

                        }

                        // Update Locked Prices
                        $rpQuery->query("UPDATE `$rs_order_repair_attr` SET `repair_attr_price` = '{$the_price}' WHERE `ra_id_fk` = '$a_id' AND `o_id_fk` IN ({$order_ids_str})");

                        foreach( $order_ids as $_o_id ) {

                              if( array_search( $_o_id, $affected_orders ) === FALSE ) {

                                    $affected_orders[] = $_o_id;

                              }

                        }

                  }

            }

            foreach( $affected_orders as $order_id ) {

                  rp_calculate_and_set_appointment_total( $order_id );

                  // Only send email if there is no price on request
                  // repair left!
                  if( empty( rp_get_por_repairs_by_order_id( $order_id ) ) && empty( rp_get_por_repair_attr_by_order_id( $order_id ) ) ) {

                        rp_send_appointment_status_update_email( $order_id );

                  }

            }

            echo json_encode( array( 'success' => true ) );

            exit;

      }

      echo json_encode( array( 'error' => true ) );

      exit;

}

function rp_get_latest_order_calculations( $order_id = '' ) {

      /* Added in V1.4.4 */

      global $rpQuery;

      $rpCart = new RepairPluginPro\RP_Cart_Recalc( $order_id );

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_order_upsales = $rpQuery->prefix . 'rs_order_upsales';

      $rows = $rpQuery->get_results("SELECT r_id_fk FROM `$rs_order_repairs` WHERE `o_id_fk` = '$order_id';");

      if( !empty( $rows ) ) {

            foreach( $rows as $row ) {

                  $rpCart->add_repair( $row->r_id_fk );

            }

      }

      $rows = $rpQuery->get_results("SELECT ra_id_fk FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '$order_id';");

      if( !empty( $rows ) ) {

            foreach( $rows as $row ) {

                  $rpCart->add_repair_attr( $row->ra_id_fk );
                  
            }

      }

      $rows = $rpQuery->get_results("SELECT upsales_id_fk FROM `$rs_order_upsales` WHERE `o_id_fk` = '$order_id';");

      if( !empty( $rows ) ) {

            foreach( $rows as $row ) {

                  $rpCart->add_upsale( $row->upsales_id_fk );
                  
            }

      }

      $row = $rpQuery->get_row("SELECT `delivery_method`, `dm_cost` FROM `$rs_orders` WHERE o_id = '$order_id' LIMIT 1;");

      if( !empty( $row ) ) {

            if( empty( $row->delivery_method ) || strtolower(trim(str_replace(' ', '_', $row->delivery_method))) == 'pdf_offer' ) {

                  $dm_cost = 0;

                  if( !empty( $_POST['selected_dm_id'] ?? '' ) ) {

                        $rpCart->add_delivery_method( $_POST['selected_dm_id'], 0, TRUE );

                        $dm_cost = (float) $rpCart->delivery_method_cost();
            
                  }

            } else {
                  
                  $dm_cost = (float) ($row->dm_cost ?? 0);

                  $rpCart->add_delivery_method( $row->delivery_method, $dm_cost );

            }

      }

      $row = $rpQuery->get_row("SELECT coupon_id_fk FROM `$rs_orders` WHERE o_id = '$order_id' LIMIT 1;");

      if( !empty( $row ) ) {

            $rpCart->add_coupon( $row->coupon_id_fk );

      }

      return array( $rpCart, $rpCart->calculate() );

}

function rp_calculate_and_set_appointment_total( $order_id = '' ) {

      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $order_id );

      $combo        = $calculations['combo_discount'];

      $final_price  = $calculations['total'];

      $sub_total  = $calculations['sub_total'];

      $rpQuery->query("UPDATE `$rs_orders` SET `excluded_tax_price` = '$sub_total', `combo_discount` = '$combo', `total_price` = '$final_price' WHERE `o_id` = '$order_id';");

}

rp_ajax_for_admin('rp_update_por_repairs_price', 'rp_update_por_repairs_price');


/*
Add finalize_link column to rs_user_pdfs table
*/

function add_finalize_link_column_to_rs_user_pdfs() {
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_user_pdfs = $rpQuery->prefix . 'rs_user_pdfs';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_user_pdfs)."' AND COLUMN_NAME = 'finalize_link';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_user_pdfs` ADD `finalize_link` varchar(1000) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_finalize_link_column_to_rs_user_pdfs', 999);


/*
Add zipcode_only column to rs_user_pdfs table
*/

function add_zipcode_only_column_to_rs_user_pdfs() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_user_pdfs = $rpQuery->prefix . 'rs_user_pdfs';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_user_pdfs)."' AND COLUMN_NAME = 'zipcode_only';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_user_pdfs` ADD `zipcode_only` varchar(1000) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_zipcode_only_column_to_rs_user_pdfs', 999);

/*
Add o_id_fk column to rs_user_pdfs table
*/

function add_o_id_fk_column_to_rs_user_pdfs() {
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_user_pdfs = $rpQuery->prefix . 'rs_user_pdfs';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_user_pdfs)."' AND COLUMN_NAME = 'o_id_fk';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_user_pdfs` ADD `o_id_fk` int(11) NOT NULL;");
            
      }
}

add_action('init', 'add_o_id_fk_column_to_rs_user_pdfs', 999);

function rp_get_locked_repair_price( $order_id = '', $r_id = '' ) {

      if( empty( $order_id ) || empty( $r_id ) ) {

            return NULL;

      }

      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $row = $rpQuery->get_row("SELECT * FROM `$rs_order_repairs` WHERE `o_id_fk` = '$order_id' AND `r_id_fk` = '$r_id' LIMIT 1;");

      if( !empty( $row ) ) {

            return $row->repair_price;

      }

      return NULL;

}

function rp_get_locked_repair_attr_price( $order_id = '', $r_id = '', $a_id = '' ) {

      if( empty( $order_id ) || empty( $r_id ) || empty( $a_id ) ) {

            return NULL;

      }

      global $rpQuery;

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $row = $rpQuery->get_row("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '$order_id' AND `r_id_fk` = '$r_id' AND `ra_id_fk` = '$a_id' LIMIT 1;");

      if( !empty( $row ) ) {

            return $row->repair_attr_price;

      }

      return NULL;

}


function rp_get_locked_upsale_price( $order_id = '', $u_id = '' ) {

      if( empty( $order_id ) || empty( $u_id ) ) {

            return NULL;

      }

      global $rpQuery;

      $rs_order_upsales = $rpQuery->prefix . 'rs_order_upsales';

      $row = $rpQuery->get_row("SELECT * FROM `$rs_order_upsales` WHERE `o_id_fk` = '$order_id' AND `upsales_id_fk` = '$u_id' LIMIT 1;");

      if( !empty( $row ) ) {

            return $row->upsale_price;

      }

      return NULL;

}

// add 'Price On Request' in noti_type column of rs_notification_types table

function add_price_on_request_in_rs_notification_types() {
      
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_notification_types = $rpQuery->prefix . 'rs_notification_types';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notification_types` WHERE `noti_type` = 'Price On Request' LIMIT 1;");

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notification_types` (`noti_type`, `is_active`) VALUES ('Price On Request', '1');");
            
      }
}

add_action('init', 'add_price_on_request_in_rs_notification_types', 999);


// insert into all columns from rs_notifications_and_offers table if noti_type_id_fk 5 doesn't exist

function insert_por_markup_in_rs_notifications_and_offers() {
      
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notifications_and_offers` WHERE `noti_type_id_fk` = '5' LIMIT 1;");

      $subject = 'Bedankt voor het aanvragen van een prijs voor uw $Model$';

      $message = '<p>Geachte $CustomerName$,</p> <p>Bedankt voor het aanvragen van een prijs voor uw $Model$. Wij waarderen uw belangstelling voor ons bedrijf.</p> <p>Wij zullen u binnen 2 uur op dezelfde werkdag antwoord geven met een offerte. Mocht u intussen nog vragen hebben of meer informatie nodig hebben, aarzel dan niet om contact met ons op te nemen.</p> <p>Nogmaals bedankt voor uw aanvraag. Wij kijken ernaar uit om u te helpen de problemen op te lossen.</p> <p>Met vriendelijke groet, $ShopName$</p>';

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notifications_and_offers` (`noti_subject`, `noti_message`, `noti_type_id_fk`, `is_active`, `date_time`) VALUES ('$subject', '$message', '5', '1', '".date('Y-m-d H:i:s')."');");
            
      }
}

add_action('init', 'insert_por_markup_in_rs_notifications_and_offers', 999);


/*
Add cancel_link column to rs_user_pdfs table
*/

function add_cancel_link_column_to_rs_orders() {
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'cancel_link';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `cancel_link` varchar(1000) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_cancel_link_column_to_rs_orders', 999);


/*
Get cancel_link by order id from rs_orders table
*/

function rp_get_cancel_link_by_order_id( $order_id = '' ) {

      if( empty( $order_id ) ) {

            return NULL;

      }

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $row = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE `o_id` = '$order_id' LIMIT 1;");

      if( !empty( $row ) ) {

            return $row->cancel_link . '/';

      }

      return NULL;

}

/*
Save cancel_link by order id in rs_orders table
*/

function rp_save_cancel_link_by_order_id( $order_id = '', $cancel_link = '' ) {

      if( empty( $order_id ) || empty( $cancel_link ) ) {

            return NULL;

      }

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $rpQuery->query("UPDATE `$rs_orders` SET `cancel_link` = '".rp_escape_sql($cancel_link)."' WHERE `o_id` = '$order_id';");

}

/* RepairPlugin Get selected currency with backward compatibility */

function rp_get_selected_currency() {

      /* Added in v1.5.0 */

      $currency = rp_get_option('rp_localization_selected_currency', NULL);

      if( empty( $currency ) ) {

            global $rpQuery;
            
            $localization = $rpQuery->prefix . "rs_localization";

            $row = $rpQuery->get_row("SELECT * FROM $localization");

            if( !empty( $row ) ) {

                  $symbol = $row->set_currency;

                  if( $symbol === '$' ) {

                        $currency = '$ USD (United States Dollar)';

                  } else if( $symbol === '£' ) {

                        $currency = '£ GBP (United Kingdom Pound)';

                  } else {

                        $currency = '€ EUR (Euro Member Countries)';

                  }

            } else {

                  $currency = '€ EUR (Euro Member Countries)';

            }

      }

      return $currency;

}

/* RepairPlugin Available Currencies */

function rp_get_all_currencies() {

      /* Added in v1.5.0 */

      $currencies = array(
            'Lek ALL (Albania Lek)' => 'Lek',
            '؋ AFN (Afghanistan Afghani)' => '؋',
            '$ ARS (Argentina Peso)' => '$',
            'ƒ AWG (Aruba Guilder)' => 'ƒ',
            '$ AUD (Australia Dollar)' => '$',
            '₼ AZN (Azerbaijan Manat)' => '₼',
            '$ BSD (Bahamas Dollar)' => '$',
            '$ BBD (Barbados Dollar)' => '$',
            'Br BYN (Belarus Ruble)' => 'Br',
            'BZ$ BZD (Belize Dollar)' => 'BZ$',
            '$ BMD (Bermuda Dollar)' => '$',
            '$b BOB (Bolivia Bolíviano)' => '$b',
            'KM BAM (Bosnia and Herzegovina Convertible Mark)' => 'KM',
            'P BWP (Botswana Pula)' => 'P',
            'лв BGN (Bulgaria Lev)' => 'лв',
            'R$ BRL (Brazil Real)' => 'R$',
            '$ BND (Brunei Darussalam Dollar)' => '$',
            '៛ KHR (Cambodia Riel)' => '៛',
            '$ CAD (Canada Dollar)' => '$',
            '$ KYD (Cayman Islands Dollar)' => '$',
            '$ CLP (Chile Peso)' => '$',
            '¥ CNY (China Yuan Renminbi)' => '¥',
            '$ COP (Colombia Peso)' => '$',
            '₡ CRC (Costa Rica Colon)' => '₡',
            'kn HRK (Croatia Kuna)' => 'kn',
            '₱ CUP (Cuba Peso)' => '₱',
            'Kč CZK (Czech Republic Koruna)' => 'Kč',
            'kr DKK (Denmark Krone)' => 'kr',
            'RD$ DOP (Dominican Republic Peso)' => 'RD$',
            '$ XCD (East Caribbean Dollar)' => '$',
            '£ EGP (Egypt Pound)' => '£',
            '$ SVC (El Salvador Colon)' => '$',
            '€ EUR (Euro Member Countries)' => '€',
            '£ FKP (Falkland Islands (Malvinas) Pound)' => '£',
            '$ FJD (Fiji Dollar)' => '$',
            '¢ GHS (Ghana Cedi)' => '¢',
            '£ GIP (Gibraltar Pound)' => '£',
            'Q GTQ (Guatemala Quetzal)' => 'Q',
            '£ GGP (Guernsey Pound)' => '£',
            '$ GYD (Guyana Dollar)' => '$',
            'L HNL (Honduras Lempira)' => 'L',
            '$ HKD (Hong Kong Dollar)' => '$',
            'Ft HUF (Hungary Forint)' => 'Ft',
            'kr ISK (Iceland Krona)' => 'kr',
            '₹ INR (India Rupee)' => '₹',
            'Rp IDR (Indonesia Rupiah)' => 'Rp',
            'ریال IRR (Iran Rial)' => 'ریال',
            '£ IMP (Isle of Man Pound)' => '£',
            '₪ ILS (Israel Shekel)' => '₪',
            'J$ JMD (Jamaica Dollar)' => 'J$',
            '¥ JPY (Japan Yen)' => '¥',
            '£ JEP (Jersey Pound)' => '£',
            'лв KZT (Kazakhstan Tenge)' => 'лв',
            '₩ KPW (Korea (North) Won)' => '₩',
            '₩ KRW (Korea (South) Won)' => '₩',
            'лв KGS (Kyrgyzstan Som)' => 'лв',
            '₭ LAK (Laos Kip)' => '₭',
            '£ LBP (Lebanon Pound)' => '£',
            '$ LRD (Liberia Dollar)' => '$',
            'ден MKD (Macedonia Denar)' => 'ден',
            'RM MYR (Malaysia Ringgit)' => 'RM',
            '₨ MUR (Mauritius Rupee)' => '₨',
            '$ MXN (Mexico Peso)' => '$',
            '₮ MNT (Mongolia Tughrik)' => '₮',
            'د.إ MNT (Moroccan-dirham)' => 'د.إ',
            'MT MZN (Mozambique Metical)' => 'MT',
            '$ NAD (Namibia Dollar)' => '$',
            '₨ NPR (Nepal Rupee)' => '₨',
            'ƒ ANG (Netherlands Antilles Guilder)' => 'ƒ',
            '$ NZD (New Zealand Dollar)' => '$',
            'C$ NIO (Nicaragua Cordoba)' => 'C$',
            '₦ NGN (Nigeria Naira)' => '₦',
            'kr NOK (Norway Krone)' => 'kr',
            'ریال OMR (Oman Rial)' => 'ریال',
            '₨ PKR (Pakistan Rupee)' => '₨',
            'B/. PAB (Panama Balboa)' => 'B/.',
            'Gs PYG (Paraguay Guarani)' => 'Gs',
            'S/. PEN (Peru Sol)' => 'S/.',
            '₱ PHP (Philippines Peso)' => '₱',
            'zł PLN (Poland Zloty)' => 'zł',
            'ریال QAR (Qatar Riyal)' => 'ریال',
            'lei RON (Romania Leu)' => 'lei',
            '₽ RUB (Russia Ruble)' => '₽',
            '£ SHP (Saint Helena Pound)' => '£',
            'ریال SAR (Saudi Arabia Riyal)' => 'ریال',
            'Дин. RSD (Serbia Dinar)' => 'Дин.',
            '₨ SCR (Seychelles Rupee)' => '₨',
            '$ SGD (Singapore Dollar)' => '$',
            '$ SBD (Solomon Islands Dollar)' => '$',
            'S SOS (Somalia Shilling)' => 'S',
            '₩ KRW (South Korean Won)' => '₩',
            'R ZAR (South Africa Rand)' => 'R',
            '₨ LKR (Sri Lanka Rupee)' => '₨',
            'kr SEK (Sweden Krona)' => 'kr',
            'CHF CHF (Switzerland Franc)' => 'CHF',
            '$ SRD (Suriname Dollar)' => '$',
            '£ SYP (Syria Pound)' => '£',
            'NT$ TWD (Taiwan New Dollar)' => 'NT$',
            '฿ THB (Thailand Baht)' => '฿',
            'TT$ TTD (Trinidad and Tobago Dollar)' => 'TT$',
            '₺ TRY (Turkey Lira)' => '₺',
            '$ TVD (Tuvalu Dollar)' => '$',
            '₴ UAH (Ukraine Hryvnia)' => '₴',
            'د.إ AED (UAE-Dirham)' => 'د.إ',
            '£ GBP (United Kingdom Pound)' => '£',
            '$ USD (United States Dollar)' => '$',
            '$U UYU (Uruguay Peso)' => '$U',
            'лв UZS (Uzbekistan Som)' => 'лв',
            'Bs VEF (Venezuela Bolívar)' => 'Bs',
            '₫ VND (Viet Nam Dong)' => '₫',
            'ریال YER (Yemen Rial)' => 'ریال',
            'Z$ ZWD (Zimbabwe Dollar)' => 'Z$',
      );

      return $currencies;

}

// Add is_active column in rs_languages table
function add_is_active_column_to_rs_languages_table() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . 'rs_languages';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($table_rs_languages)."' AND COLUMN_NAME = 'is_active';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$table_rs_languages` ADD `is_active` tinyint(1) NOT NULL DEFAULT '1';");

            rp_update_is_active_based_on_enabled_languages();
            
      }
}

add_action('init', 'add_is_active_column_to_rs_languages_table', 999);


// Update is_active column of rs_languages table based on enabled languages

function rp_update_is_active_based_on_enabled_languages() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_languages = $rpQuery->prefix . 'rs_languages';

      $rp_enabled_languages = rp_get_enabled_languages_old();

      $rp_languages = rp_get_all_languages();

      foreach ($rp_languages as $row) {

            if(array_search($row->name, $rp_enabled_languages) !== FALSE) {

                  $rpQuery->query("UPDATE `$rs_languages` SET `is_active` = '1' WHERE `name` = '".rp_escape_sql($row->name)."'");

            } else {

                  $rpQuery->query("UPDATE `$rs_languages` SET `is_active` = '0' WHERE `name` = '".rp_escape_sql($row->name)."'");

            }

      }

}

function rp_fix_language_image( $link = '' ) {

      if( empty( $link ) ) {

            return WP_REPAIR_PLUGIN_URL . 'assets/images/circle-64.png';

      }
      
      /* Added in v1.5.0 for Backward Compatibility */
      if( strpos($link, 'assets') === 0 ) {
            return WP_REPAIR_PLUGIN_URL . $link;
      } else {
            return $link;
      }

}

function rp_update_default_language( $new = '' ) {

      /* Added in V1.5.0 */

      global $rpQuery;

      $localization = $rpQuery->prefix . "rs_localization";

      $result       = $rpQuery->get_row("SELECT * FROM $localization LIMIT 1;");

      if( !empty( $result ) ) {

            // update set_language column where localization_id matches $result->localization_id

            $rpQuery->query("UPDATE `$localization` SET `set_language` = '".rp_escape_sql($new)."' WHERE `localization_id` = '".rp_escape_sql($result->localization_id)."' LIMIT 1;");

      }

}

function rp_get_default_language() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $localization = $rpQuery->prefix . "rs_localization";

      $result       = $rpQuery->get_row("SELECT * FROM $localization LIMIT 1;");

      return trim($result->set_language);

}

function rp_get_language_row_by_id( $id = '' ) {

      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . "rs_languages";

      if( !empty( $id ) ) {

            $row = $rpQuery->get_row("SELECT * FROM `$table_rs_languages` WHERE `id` = '".rp_escape_sql($id)."' LIMIT 1;");

            if( !empty($row) ) {

                  return $row;

            }

      }

      return array();

}

function rp_get_default_language_id() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $default_language = rp_get_default_language();

      $table_rs_languages = $rpQuery->prefix . "rs_languages";

      if( !empty( $default_language ) ) {

            $row = $rpQuery->get_row("SELECT id FROM `$table_rs_languages` WHERE `name` = '".rp_escape_sql($default_language)."' LIMIT 1;");

            if( !empty($row) ) {

                  return $row->id;

            }

      }
      
      return 0;

}


function rp_delete_language() {

      rp_verify_csrf_token_for_ajax('localization');
      
      (new RepairPluginPro\RP_Languages())->rp_delete_language();

}

rp_ajax_for_admin('rp_delete_language', 'rp_delete_language');


function rp_update_language_image() {

      rp_verify_csrf_token_for_ajax('localization');
      
      (new RepairPluginPro\RP_Languages())->rp_update_language_image();

}

rp_ajax_for_admin('rp_update_language_image', 'rp_update_language_image');

function rp_update_languages() {

      rp_verify_csrf_token_for_ajax('localization');
      
      (new RepairPluginPro\RP_Languages())->rp_update_languages();

}

rp_ajax_for_admin('rp_update_languages', 'rp_update_languages');

/*
Create rs_translations table if not exists
*/
function rp_create_rs_translations_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_translations = $rpQuery->prefix . "rs_translations"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_translations (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `language` varchar(255) NOT NULL,

      `key` varchar(255) NOT NULL,

      `value` varchar(1000) NOT NULL,

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_translations_table', 999);

/*
Create rs_upsale_translations table if not exists
*/
function rp_create_rs_upsale_translations_table() {
      
      /* Added in V1.5.6 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_upsale_translations = $rpQuery->prefix . "rs_upsale_translations"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_upsale_translations (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `language` varchar(255) NOT NULL,

      `key` varchar(255) NOT NULL,

      `value` varchar(1000) NOT NULL,

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_upsale_translations_table', 999);

/*
Alter rs_translations value column to have varchar(10000) length
*/
function rp_alter_rs_translations_value_column() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_translations = $rpQuery->prefix . "rs_translations"; 

      if( rp_get_option('rp_modified_translations_value_col_length', FALSE) === FALSE ) {

            rp_add_option('rp_modified_translations_value_col_length', '1');

            $sql = "ALTER TABLE `$table_rs_translations` MODIFY COLUMN `value` varchar(10000) NOT NULL;";

            $rpQuery->query($sql);

      }

}

add_action('init', 'rp_alter_rs_translations_value_column', 999);

/*
Create rs_dr_translations table if not exists
*/

function rp_create_rs_dr_translations_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_dr_translations = $rpQuery->prefix . "rs_dr_translations"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_dr_translations (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `language` varchar(255) NOT NULL,

      `key` varchar(255) NOT NULL,

      `value` varchar(1000) NOT NULL,

      `uq_id` varchar(1000) NOT NULL,

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_dr_translations_table', 999);


/*
Create rs_r_translations table if not exists
*/

function rp_create_rs_r_translations_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_r_translations = $rpQuery->prefix . "rs_r_translations"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_r_translations (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `language` varchar(255) NOT NULL,

      `key` varchar(255) NOT NULL,

      `value` varchar(1000) NOT NULL,

      `uq_id` varchar(1000) NOT NULL,

      `m_id_fk` int(11) NOT NULL,

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_r_translations_table', 999);


/*
Create rs_rattr_translations table if not exists
*/

function rp_create_rs_rattr_translations_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_rattr_translations (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `language` varchar(255) NOT NULL,

      `key` varchar(255) NOT NULL,

      `value` varchar(1000) NOT NULL,

      `uq_id` varchar(1000) NOT NULL,

      `r_id_fk` int(11) NOT NULL,

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_rattr_translations_table', 999);


/*
Create rs_order_language table if not exists
*/

function rp_create_rs_order_language_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_order_language = $rpQuery->prefix . "rs_order_language"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_order_language (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `o_id_fk` int(11) NOT NULL,

      `language` varchar(255) NOT NULL,

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_order_language_table', 999);

/*
Insert in rs_background_jobs table
*/

function rp_insert_rs_background_jobs_table($callback, $data) {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_background_jobs = $rpQuery->prefix . "rs_background_jobs"; 

      $data = json_encode($data, JSON_UNESCAPED_UNICODE);

      $rpQuery->query("INSERT INTO $table_rs_background_jobs (`callback`, `data`) VALUES ('".rp_escape_sql($callback)."', '".rp_escape_sql($data)."')");

}

/*
Process first row of background jobs if job is registered like this:
rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());
*/

function rp_process_rs_background_jobs_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_background_jobs = $rpQuery->prefix . "rs_background_jobs"; 

      $row = $rpQuery->get_row("SELECT * FROM `$table_rs_background_jobs` WHERE `processed` = 0 ORDER BY `id` ASC LIMIT 1;");

      if($row) {

            // mark the job as processed
            $rpQuery->query("UPDATE `$table_rs_background_jobs` SET `processed` = 1 WHERE `id` = ".rp_escape_sql($row->id).";");

            $callback = $row->callback;

            $data = json_decode($row->data, true);

            if(function_exists($callback)) {

                  call_user_func_array($callback, $data);

            }

            if( isset($GLOBALS['rp_keep_job_as_pending']) && $GLOBALS['rp_keep_job_as_pending'] === true ) {

                  // keep the job as pending
                  $rpQuery->query("UPDATE `$table_rs_background_jobs` SET `processed` = 0 WHERE `id` = ".rp_escape_sql($row->id).";");

            }

            $row = $rpQuery->get_row("SELECT * FROM `$table_rs_background_jobs` WHERE `processed` = 0 ORDER BY `id` ASC LIMIT 1;");

            if(empty($row)) {

                  // all rows are processed so we can delete them all
                  $rpQuery->query("DELETE FROM `$table_rs_background_jobs` WHERE `processed` = 1;");

            }

      } else {

            // all rows are processed so we can delete them all
            $rpQuery->query("DELETE FROM `$table_rs_background_jobs` WHERE `processed` = 1;");

      }

}


/*
Create rs_background_jobs table if not exists
*/

function rp_create_rs_background_jobs_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

      $table_rs_background_jobs = $rpQuery->prefix . "rs_background_jobs"; 

      $charset_collate = $rpQuery->get_charset_collate();

      $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_background_jobs (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `callback` varchar(255) NOT NULL,

      `data` varchar(1000) NOT NULL,

      `processed` tinyint(1) NOT NULL DEFAULT '0',

      PRIMARY KEY (`id`))

      $charset_collate";

      dbDelta( $sql );

}

add_action('init', 'rp_create_rs_background_jobs_table', 999);


/*
Get language from rs_order_language table by order id
*/

function rp_get_language_by_order_id($order_id = 0) {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_order_language = $rpQuery->prefix . "rs_order_language";

      $sql = "SELECT * FROM `$table_rs_order_language` WHERE `o_id_fk` = '$order_id'";

      $result = $rpQuery->get_row($sql);

      if( !empty($result) ) {

            return $result->language;

      }

      return '';

}

/*
Save language in rs_order_language table
*/

function rp_save_order_related_language( $o_id = '', $language = '' ) {

      global $rpQuery;

      $table_rs_order_language = $rpQuery->prefix . "rs_order_language";

      $sql = "SELECT * FROM `$table_rs_order_language` WHERE `o_id_fk` = '$o_id'";

      $result = $rpQuery->get_row($sql);

      if( !empty( $result ) ) {

            // Update

            $rpQuery->query("UPDATE `$table_rs_order_language` SET `language` = '".rp_escape_sql($language)."' WHERE `o_id_fk` = '".rp_escape_sql($o_id)."'");

      } else {

            // Create

            $rpQuery->query("INSERT INTO `$table_rs_order_language` (`o_id_fk`, `language`) VALUES ('".rp_escape_sql($o_id)."', '".rp_escape_sql($language)."')");

      }

} 


/*
Use this function to insert multiple rows in a table
*/
function rp_insert_multiple_rows($table_name = '', $data = array(), $start_transaction = TRUE) {

      if( empty( $data ) ) {

            return FALSE;

      }

      /* Added in V1.5.0 */

      global $rpQuery;

      if( $start_transaction === TRUE ) {
            $rpQuery->query('START TRANSACTION');
      }

      $table_name = $rpQuery->prefix . $table_name;

      $keys = array_keys($data[0]);

      $keys_str = "`" . implode('`, `', $keys) . "`";

      $sql = "INSERT INTO `$table_name` ({$keys_str}) VALUES ";

      $values = array();

      foreach ($data as $row) {

            $row_values = array();

            foreach($keys as $key) {

                  if( !isset($row[$key]) ) {

                        $row[$key] = '';

                  }

                  $row_values[] = "'".rp_escape_sql($row[$key])."'";

            }

            $values[] = '('.implode(', ', $row_values).')';

      }

      $sql .= implode(', ', $values);

      $rpQuery->query($sql);

      if( $start_transaction === TRUE ) {
            $rpQuery->query('COMMIT');
      }

}


/*
Add Default Translation Rows First Time
*/

function rp_add_default_translation_rows() {
      
      if( rp_get_option('rp_added_default_translation_rows', FALSE) === '1' ) {

            return;

      }

      rp_add_option('rp_added_default_translation_rows', '1');

      global $rpQuery;

      $english_translations = rp_get_static_translations_of_language('English (United States)');

      $dutch_translations = rp_get_static_translations_of_language('Dutch (Netherlands)');

      // Check if rs_translations table is empty

      $table_rs_translations = $rpQuery->prefix . "rs_translations";

      $result = $rpQuery->get_row("SELECT * FROM $table_rs_translations LIMIT 1;");

      if( empty( $result ) ) {

            $rows = array();

            $added = array();

            foreach( $english_translations as $key => $value ) {

                  if( !in_array($key, $added) ) {

                        $rows[] = array(

                              'language' => 'English (United States)',
      
                              'key' => $key,
      
                              'value' => $value
      
                        );
      
                        $added[] = $key;

                  }

            }

            rp_insert_multiple_rows('rs_translations', $rows);

            $rows = array();

            $added = array();

            foreach( $dutch_translations as $key => $value ) {

                  if( !in_array($key, $added) ) {

                        $rows[] = array(

                              'language' => 'Dutch (Netherlands)',
      
                              'key' => $key,
      
                              'value' => $value
      
                        );
      
                        $added[] = $key;

                  }

            }

            rp_insert_multiple_rows('rs_translations', $rows);

      }

}

add_action('init', 'rp_add_default_translation_rows', 999);


/*
Get related translations in all languages by key
*/

function rp_get_related_translations() {

      (new RepairPluginPro\RP_Languages())->rp_get_related_translations();

}

rp_ajax_for_admin('rp_get_related_translations', 'rp_get_related_translations');


/*
Get related translations in all languages by key
*/

function rp_update_translations() {

      rp_verify_csrf_token_for_ajax(array(
            'styling',
            'planning_discount',
            'notification_offer'
      ));

      (new RepairPluginPro\RP_Languages())->rp_update_translations();

}

rp_ajax_for_admin('rp_update_translations', 'rp_update_translations');


/*
Get related repair translations in all languages by key
*/

function rp_get_r_translations() {

      (new RepairPluginPro\RP_Repair_Translations())->rp_get_r_translations();

}

rp_ajax_for_admin('rp_get_r_translations', 'rp_get_r_translations');


/*
Get related repair translations in all languages by key
*/

function rp_get_dr_translations() {

      (new RepairPluginPro\RP_Repair_Translations())->rp_get_dr_translations();

}

rp_ajax_for_admin('rp_get_dr_translations', 'rp_get_dr_translations');

/*
Get related upsale translations in all languages by key
*/

function rp_get_upsale_translations() {

      (new RepairPluginPro\RP_Upsale_Translations())->rp_get_upsale_translations();

}

rp_ajax_for_admin('rp_get_upsale_translations', 'rp_get_upsale_translations');


/*
Get related repair attr translations in all languages by key
*/

function rp_get_rattr_translations() {

      (new RepairPluginPro\RP_Repair_Translations())->rp_get_rattr_translations();

}

rp_ajax_for_admin('rp_get_rattr_translations', 'rp_get_rattr_translations');


/*
Get related default repair attr translations in all languages by key
*/

function rp_get_drattr_translations() {

      (new RepairPluginPro\RP_Repair_Translations())->rp_get_drattr_translations();

}

rp_ajax_for_admin('rp_get_drattr_translations', 'rp_get_drattr_translations');



/*
Get static translation of repair
*/

function rp_get_static_translations_of_repair($key = '', $uqid = '', $connection_id = '', $use_language = 'default') {

      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_r_translations";

      $table_rs_repair = $rpQuery->prefix . "rs_repair";

      $org_key = $key;

      if( $key === 'repair_name' ) {

            $key = 'r_name';

      }

      if( $key === 'repair_desc' ) {

            $key = 'r_discription';

      }

      if( $key === 'repair_badge' ) {

            $key = 'r_badge';

      }

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$key` FROM `$table_rs_repair` WHERE `r_uq_id` = '".rp_escape_sql($uqid)."' AND m_id_fk = '".rp_escape_sql($connection_id)."' LIMIT 1;");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$key};

      }

      if( $use_language === 'selected' ) {

            global $wp_repair_selected_language;

            $default_language = $wp_repair_selected_language;

      } else {

            global $rp_use_order_related_language;

            if( isset( $rp_use_order_related_language ) && !empty( $rp_use_order_related_language ) ) {

                  $default_language = $rp_use_order_related_language;

            } else {

                  $cache = RepairPluginPro\RP_Memory_Cache::get('rp_get_default_language');

                  if( empty( $cache ) ) {

                        $default_language = rp_get_default_language();

                        RepairPluginPro\RP_Memory_Cache::save('rp_get_default_language', $default_language);

                  } else {

                        $default_language = $cache;

                  }

            }

      }

      // select value from rs_r_translations where key = $key and uq_id = $uqid and m_id_fk = $connection_id

      // $result = $rpQuery->get_row("SELECT `value` FROM `$table_rs_r_translations` WHERE `key` = '".rp_escape_sql($org_key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND m_id_fk = '".rp_escape_sql($connection_id)."' AND `language` = '".rp_escape_sql($default_language)."' LIMIT 1;");



      $d_result = rp_get_static_translations_of_drepair($org_key, $uqid, $default_language);

      if( !empty($d_result) ) {

            $final_result = $d_result;

      } else {

            $result = rp_get_dynamic_translations_of_repair($connection_id, $default_language);

            if( is_array( $result ) && isset( $result[$uqid.$org_key] ) ) {

                  $final_result = $result[$uqid.$org_key];

            }

      }

      return $final_result ?? '';

}

/*
Get static translation of upsale
*/

function rp_get_static_translations_of_upsale_selected($key = '', $uqid = '', $use_language = 'default') {

      global $rpQuery;

      $rs_upsale_translations = $rpQuery->prefix . "rs_upsale_translations";

      $rs_upsales = $rpQuery->prefix . "rs_upsales";

      $nm_key = $key;

      $org_key = $key;

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$nm_key` FROM `$rs_upsales` WHERE `upsales_id` = '".rp_escape_sql($uqid)."' LIMIT 1;");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$nm_key};

      }

      if( $use_language === 'selected' ) {

            global $wp_repair_selected_language;

            $default_language = $wp_repair_selected_language;

      } else {

            global $rp_use_order_related_language;

            if( isset( $rp_use_order_related_language ) && !empty( $rp_use_order_related_language ) ) {

                  $default_language = $rp_use_order_related_language;

            } else {

                  $cache = RepairPluginPro\RP_Memory_Cache::get('rp_get_default_language');

                  if( empty( $cache ) ) {

                        $default_language = rp_get_default_language();

                        RepairPluginPro\RP_Memory_Cache::save('rp_get_default_language', $default_language);

                  } else {

                        $default_language = $cache;

                  }

            }

      }

      $d_result = rp_get_static_translations_of_upsale($org_key, $uqid, $default_language);

      if( !empty($d_result) ) {

            $final_result = $d_result;

      }

      return $final_result;

}

/*
Get static translation of repair by language
*/

function rp_get_static_translations_of_repair_by_language($key = '', $uqid = '', $connection_id = '', $language = '') {

      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_r_translations";

      $table_rs_repair = $rpQuery->prefix . "rs_repair";

      $org_key = $key;

      if( $key === 'repair_name' ) {

            $key = 'r_name';

      }

      if( $key === 'repair_desc' ) {

            $key = 'r_discription';

      }

      if( $key === 'repair_badge' ) {

            $key = 'r_badge';

      }

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$key` FROM `$table_rs_repair` WHERE `r_uq_id` = '".rp_escape_sql($uqid)."' AND m_id_fk = '".rp_escape_sql($connection_id)."' LIMIT 1;");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$key};

      }

      $default_language = $language;

      $d_result = rp_get_static_translations_of_drepair($org_key, $uqid, $default_language);

      if( !empty($d_result) ) {

            $final_result = $d_result;

      } else {

            $result = rp_get_dynamic_translations_of_repair($connection_id, $default_language);

            if( is_array( $result ) && isset( $result[$uqid.$org_key] ) ) {

                  $final_result = $result[$uqid.$org_key];

            }

      }

      return $final_result;

}


function rp_get_dynamic_translations_of_repair($connection_id = '', $default_language = '') {

      $cache_key = md5('rp_get_dynamic_translations_of_repair_' . $connection_id . $default_language);

      $cache = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if( $cache !== NULL && is_array( $cache ) ) {

            return $cache;

      }
      
      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_r_translations";

      $result = $rpQuery->get_results("SELECT * FROM `$table_rs_r_translations` WHERE m_id_fk = '".rp_escape_sql($connection_id)."' AND `language` = '".rp_escape_sql($default_language)."';");
      
      $prepared = array();

      if( !empty( $result ) ) {

            foreach( $result as $row ) {

                  $prepared[$row->uq_id.$row->key] = $row->value;

            }

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $prepared);

      return $prepared;

}

/*
Get static translation of default repair
*/

function rp_get_static_translations_of_drepair($key = '', $uqid = '', $lang = '') {

      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_dr_translations";

      $table_rs_repair = $rpQuery->prefix . "rs_default_repair";

      $org_key = $key;

      if( $key === 'repair_name' ) {

            $key = 'dr_name';

      }

      if( $key === 'repair_desc' ) {

            $key = 'dr_discription';

      }
      
      if( $key === 'repair_badge' ) {

            $key = 'dr_badge';

      }

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$key` FROM `$table_rs_repair` WHERE `dr_uq_id` = '".rp_escape_sql($uqid)."';");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$key};

      }
      
      if(!empty($lang)) {

            $default_language = $lang;

      } else {

            $cache = RepairPluginPro\RP_Memory_Cache::get('rp_get_default_language');

            if( empty( $cache ) ) {

                  $default_language = rp_get_default_language();

                  RepairPluginPro\RP_Memory_Cache::save('rp_get_default_language', $default_language);

            } else {

                  $default_language = $cache;

            }

      }

      // select value from rs_r_translations where key = $key and uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `value` FROM `$table_rs_r_translations` WHERE `key` = '".rp_escape_sql($org_key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($default_language)."';");

      if( !empty( $result ) ) {

            $final_result = $result->value;

      }

      return $final_result ?? '';

}

/*
Get static translation of upsale
*/

function rp_get_static_translations_of_upsale($key = '', $uqid = '', $lang = '') {

      global $rpQuery;

      $rs_upsale_translations = $rpQuery->prefix . "rs_upsale_translations";

      $rs_upsales = $rpQuery->prefix . "rs_upsales";

      $nm_key = $key;

      $key = $uqid.'_'.$key;

      $org_key = $key;

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$nm_key` FROM `$rs_upsales` WHERE `upsales_id` = '".rp_escape_sql($uqid)."';");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$nm_key};

      }
      
      if(!empty($lang)) {

            $default_language = $lang;

      } else {

            $cache = RepairPluginPro\RP_Memory_Cache::get('rp_get_default_language');

            if( empty( $cache ) ) {

                  $default_language = rp_get_default_language();

                  RepairPluginPro\RP_Memory_Cache::save('rp_get_default_language', $default_language);

            } else {

                  $default_language = $cache;

            }

      }

      // select value from rs_r_translations where key = $key and uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `value` FROM `$rs_upsale_translations` WHERE `key` = '".rp_escape_sql($org_key)."' AND `language` = '".rp_escape_sql($default_language)."';");

      if( !empty( $result ) ) {

            $final_result = $result->value;

      }

      return $final_result;

}


/*
Get static translation of attr
*/

function rp_get_static_translations_of_attr($key = '', $uqid = '', $connection_id = '', $use_language = 'default') {

      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_rattr_translations";

      $table_rs_repair = $rpQuery->prefix . "rs_repair_attr";

      $org_key = $key;

      if( $key === 'attr_name' ) {

            $key = 'a_name';

      }

      if( $key === 'attr_desc' ) {

            $key = 'a_desc';

      }

      if( $key === 'attr_badge' ) {

            $key = 'a_badge';

      }

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$key` FROM `$table_rs_repair` WHERE `a_uq_id` = '".rp_escape_sql($uqid)."' AND r_id_fk = '".rp_escape_sql($connection_id)."' LIMIT 1;");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$key};

      }

      if( $use_language === 'selected' ) {

            global $wp_repair_selected_language;

            $default_language = $wp_repair_selected_language;

      } else {

            global $rp_use_order_related_language;

            if( isset( $rp_use_order_related_language ) && !empty( $rp_use_order_related_language ) ) {

                  $default_language = $rp_use_order_related_language;

            } else {

                  $cache = RepairPluginPro\RP_Memory_Cache::get('rp_get_default_language');

                  if( empty( $cache ) ) {

                        $default_language = rp_get_default_language();

                        RepairPluginPro\RP_Memory_Cache::save('rp_get_default_language', $default_language);

                  } else {

                        $default_language = $cache;

                  }

            }

      }

      // select value from rs_r_translations where key = $key and uq_id = $uqid and m_id_fk = $connection_id

      // $result = $rpQuery->get_row("SELECT `value` FROM `$table_rs_r_translations` WHERE `key` = '".rp_escape_sql($org_key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND r_id_fk = '".rp_escape_sql($connection_id)."' AND `language` = '".rp_escape_sql($default_language)."' LIMIT 1;");


      $d_result = rp_get_static_translations_of_dattr($org_key, $uqid, $default_language);

      if( !empty( $d_result ) ) {

            $final_result = $d_result;

      } else {

            $result = rp_get_dynamic_translations_of_attr($connection_id, $default_language);

            if( is_array( $result ) && isset( $result[$uqid.$org_key] ) ) {

                  $final_result = $result[$uqid.$org_key];

            }

      }

      return $final_result ?? '';

}


/*
Get static translation of attr by language
*/

function rp_get_static_translations_of_attr_by_language($key = '', $uqid = '', $connection_id = '', $language = '') {

      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_rattr_translations";

      $table_rs_repair = $rpQuery->prefix . "rs_repair_attr";

      $org_key = $key;

      if( $key === 'attr_name' ) {

            $key = 'a_name';

      }

      if( $key === 'attr_desc' ) {

            $key = 'a_desc';

      }

      if( $key === 'attr_badge' ) {

            $key = 'a_badge';

      }

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$key` FROM `$table_rs_repair` WHERE `a_uq_id` = '".rp_escape_sql($uqid)."' AND r_id_fk = '".rp_escape_sql($connection_id)."' LIMIT 1;");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$key};

      }

      $default_language = $language;

      $d_result = rp_get_static_translations_of_dattr($org_key, $uqid, $default_language);

      if( !empty( $d_result ) ) {

            $final_result = $d_result;

      } else {

            $result = rp_get_dynamic_translations_of_attr($connection_id, $default_language);

            if( is_array( $result ) && isset( $result[$uqid.$org_key] ) ) {

                  $final_result = $result[$uqid.$org_key];

            }

      }

      return $final_result;

}


function rp_get_dynamic_translations_of_attr($connection_id = '', $default_language = '') {

      $cache_key = md5('rp_get_dynamic_translations_of_attr' . $connection_id . $default_language);

      $cache = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if( $cache !== NULL && is_array( $cache ) ) {

            return $cache;

      }
      
      global $rpQuery;

      $table_rs_rattr_translations = $rpQuery->prefix . "rs_rattr_translations";

      $result = $rpQuery->get_results("SELECT * FROM `$table_rs_rattr_translations` WHERE r_id_fk = '".rp_escape_sql($connection_id)."' AND `language` = '".rp_escape_sql($default_language)."';");
      
      $prepared = array();

      if( !empty( $result ) ) {

            foreach( $result as $row ) {

                  $prepared[$row->uq_id.$row->key] = $row->value;

            }

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $prepared);

      return $prepared;

}

/*
Get static translation of default attr
*/

function rp_get_static_translations_of_dattr($key = '', $uqid = '', $lang = '') {

      global $rpQuery;

      $table_rs_r_translations = $rpQuery->prefix . "rs_dr_translations";

      $table_rs_repair = $rpQuery->prefix . "rs_default_repair_attr";

      $org_key = $key;

      if( $key === 'attr_name' ) {

            $key = 'da_name';

      }

      if( $key === 'attr_desc' ) {

            $key = 'da_desc';

      }

      if( $key === 'attr_badge' ) {

            $key = 'da_badge';

      }

      // select $key from rs_repair where r_uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT `$key` FROM `$table_rs_repair` WHERE `da_uq_id` = '".rp_escape_sql($uqid)."';");

      $final_result = '';

      if( !empty( $result ) ) {

            $final_result = $result->{$key};

      }

      if(!empty($lang)) {
            $default_language = $lang;
      } else {
            $default_language = rp_get_default_language();
      }

      // select value from rs_r_translations where key = $key and uq_id = $uqid and m_id_fk = $connection_id

      $result = $rpQuery->get_row("SELECT * FROM `$table_rs_r_translations` WHERE `key` = '".rp_escape_sql($org_key)."' AND `uq_id` = '".rp_escape_sql($uqid)."' AND `language` = '".rp_escape_sql($default_language)."';");

      if( !empty( $result ) ) {

            $final_result = $result->value;

      }

      return $final_result ?? '';

}


/*
Update related repair translations in all languages by key
*/

function rp_update_r_translations() {
      
      rp_verify_csrf_token_for_ajax('manage_models');

      (new RepairPluginPro\RP_Repair_Translations())->rp_update_r_translations();

}

rp_ajax_for_admin('rp_update_r_translations', 'rp_update_r_translations');

/*
Update related attr translations in all languages by key
*/

function rp_update_rattr_translations() {

      rp_verify_csrf_token_for_ajax('manage_models');

      (new RepairPluginPro\RP_Repair_Translations())->rp_update_rattr_translations();

}

rp_ajax_for_admin('rp_update_rattr_translations', 'rp_update_rattr_translations');


/*
Update related default repair translations in all languages by key
*/

function rp_update_dr_translations() {

      rp_verify_csrf_token_for_ajax('default_repairs');

      (new RepairPluginPro\RP_Repair_Translations())->rp_update_dr_translations();

}

rp_ajax_for_admin('rp_update_dr_translations', 'rp_update_dr_translations');

/*
Update related upsale translations in all languages by key
*/

function rp_update_upsale_translations() {

      rp_verify_csrf_token_for_ajax('manage_upsales');

      (new RepairPluginPro\RP_Upsale_Translations())->rp_update_upsale_translations();

}

rp_ajax_for_admin('rp_update_upsale_translations', 'rp_update_upsale_translations');


/*
Update related default attr translations in all languages by key
*/

function rp_update_drattr_translations() {

      rp_verify_csrf_token_for_ajax('default_repairs');

      (new RepairPluginPro\RP_Repair_Translations())->rp_update_drattr_translations();

}

rp_ajax_for_admin('rp_update_drattr_translations', 'rp_update_drattr_translations');


/* Translate repairs and attributes */

function rp_translate_locked_repairs_and_attributes( $repairs = array(), $attrs = array() ) {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . "rs_repair";

      $rs_repair_attr = $rpQuery->prefix . "rs_repair_attr";

      if( !empty( $repairs ) ) {

            foreach($repairs as $index => $repair_row) {

                  if( isset($repair_row->repair_name) && !empty($repair_row->repair_name) ) {

                        $row = $rpQuery->get_row("SELECT * FROM `$rs_repair` WHERE `r_id` = '".rp_escape_sql($repair_row->r_id_fk)."';");

                        if( !empty( $row ) ) {

                              $translated = rp_get_static_translations_of_repair('repair_name', $row->r_uq_id, $row->m_id_fk);

                              if( !empty( $translated ) ) {

                                    $repair_row->repair_name = $translated;

                              }

                        }

                        $repairs[$index] = $repair_row;

                  } else {

                        unset($repair_row->repair_name);

                  }

                  if( isset($repair_row->r_name) ) {

                        $translated = rp_get_static_translations_of_repair('repair_name', $repair_row->r_uq_id, $repair_row->m_id_fk);

                        if( !empty( $translated ) ) {

                              $repair_row->r_name = $translated;

                        }

                        $repairs[$index] = $repair_row;

                  } else {

                        $repair_row->r_name = '';

                  }

            }

      }

      if( !empty( $attrs ) ) {

            foreach($attrs as $index => $attr_row) {

                  if( isset( $attr_row->repair_attr_name ) && !empty( $attr_row->repair_attr_name ) ) {

                        $row = $rpQuery->get_row("SELECT * FROM `$rs_repair_attr` WHERE `a_id` = '".rp_escape_sql($attr_row->ra_id_fk)."';");

                        if( !empty( $row ) ) {

                              $translated = rp_get_static_translations_of_attr('attr_name', $row->a_uq_id, $row->r_id_fk);

                              if( !empty( $translated ) ) {

                                    $attr_row->repair_attr_name = $translated;

                              }

                              $translated_attr = rp_get_static_translations_of_attr('attr_desc', $row->a_uq_id, $row->r_id_fk);

                              if( !empty( $translated_attr ) ) {

                                    $attr_row->repair_attr_desc = $translated_attr;

                              }

                              $translated_badge = rp_get_static_translations_of_attr('attr_badge', $row->a_uq_id, $row->r_id_fk);

                              if( !empty( $translated_badge ) ) {

                                    $attr_row->repair_attr_badge = $translated_badge;

                              }

                        }

                        $attrs[$index] = $attr_row;

                  } else {

                        unset($attr_row->repair_attr_name);

                  }

                  if( isset($attr_row->a_name) ) {

                        $translated = rp_get_static_translations_of_attr('attr_name', $attr_row->a_uq_id, $attr_row->r_id_fk);

                        if( !empty( $translated ) ) {

                              $attr_row->a_name = $translated;

                        }

                        $translated_attr = rp_get_static_translations_of_attr('attr_desc', $attr_row->a_uq_id, $attr_row->r_id_fk);

                        if( !empty( $translated_attr ) ) {

                              $attr_row->a_desc = $translated_attr;

                        }

                        $translated_badge = rp_get_static_translations_of_attr('attr_badge', $attr_row->a_uq_id, $attr_row->r_id_fk);

                        if( !empty( $translated_badge ) ) {

                              $attr_row->a_badge = $translated_badge;

                        }

                        $attrs[$index] = $attr_row;

                  } else {

                        $attr_row->a_name = '';

                  }

            }

      }

      return array( $repairs, $attrs );

}

/* Translate upsales */

function rp_translate_locked_upsales( $upsales = array() ) {

      global $rpQuery;

      $rs_upsales = $rpQuery->prefix . "rs_upsales";

      if( !empty( $upsales ) ) {

            foreach($upsales as $index => $upsale_row) {

                  if( isset($upsale_row->upsale_name) && !empty($upsale_row->upsale_name) ) {

                        $row = $rpQuery->get_row("SELECT * FROM `$rs_upsales` WHERE `upsales_id` = '".rp_escape_sql($upsale_row->upsales_id_fk)."';");

                        if( !empty( $row ) ) {

                              $translated = rp_get_static_translations_of_upsale_selected('us_name', $row->upsales_id);

                              if( !empty( $translated ) ) {

                                    $upsale_row->upsale_name = $translated;

                              }

                        }

                        $upsales[$index] = $upsale_row;

                  } else {

                        unset($upsale_row->upsale_name);

                  }

                  if( isset($upsale_row->us_name) ) {

                        $translated = rp_get_static_translations_of_upsale_selected('us_name', $upsale_row->upsales_id);

                        if( !empty( $translated ) ) {

                              $upsale_row->us_name = $translated;

                        }

                        $upsales[$index] = $upsale_row;

                  } else {

                        $upsale_row->us_name = '';

                  }

            }

      }

      return $upsales;

}

function rp_set_order_related_language( $order_id ) {

      global $rp_use_order_related_language;

      $lang = trim(rp_get_language_by_order_id($order_id));

      if( !empty( $lang ) ) {

            $rp_use_order_related_language = $lang;

            rp_set_new_plugin_language($lang);

      }

}

function rp_unset_order_related_language() {

      global $rp_use_order_related_language;

      if( isset( $rp_use_order_related_language ) ) {

            $rp_use_order_related_language = '';

            unset( $rp_use_order_related_language );

      }

      rp_reset_plugin_language();

}

function rp_update_language_name_everywhere( $old_language_name = '', $new_language_name = '' ) {

      global $rpQuery;

      $rs_dr_translations = $rpQuery->prefix . 'rs_dr_translations';

      $rs_rattr_translations = $rpQuery->prefix . 'rs_rattr_translations';

      $rs_r_translations = $rpQuery->prefix . 'rs_r_translations';

      $rs_translations = $rpQuery->prefix . 'rs_translations';
      
      $rpQuery->query("UPDATE `$rs_dr_translations` SET `language` = '".rp_escape_sql($new_language_name)."' WHERE `language` = '".rp_escape_sql($old_language_name)."';");

      $rpQuery->query("UPDATE `$rs_rattr_translations` SET `language` = '".rp_escape_sql($new_language_name)."' WHERE `language` = '".rp_escape_sql($old_language_name)."';");

      $rpQuery->query("UPDATE `$rs_r_translations` SET `language` = '".rp_escape_sql($new_language_name)."' WHERE `language` = '".rp_escape_sql($old_language_name)."';");

      $rpQuery->query("UPDATE `$rs_translations` SET `language` = '".rp_escape_sql($new_language_name)."' WHERE `language` = '".rp_escape_sql($old_language_name)."';");

}

function rp_delete_related_translations_from_all_tables($language = '') {

      if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

            rp_insert_rs_background_jobs_table(__FUNCTION__, func_get_args());

            return;

      }

      global $rpQuery;

      $rs_dr_translations = $rpQuery->prefix . 'rs_dr_translations';

      $rs_rattr_translations = $rpQuery->prefix . 'rs_rattr_translations';

      $rs_r_translations = $rpQuery->prefix . 'rs_r_translations';

      $rs_translations = $rpQuery->prefix . 'rs_translations';

      $rpQuery->query("START TRANSACTION");

      $rpQuery->query("DELETE FROM `$rs_dr_translations` WHERE `language` = '".rp_escape_sql($language)."';");

      $rpQuery->query("DELETE FROM `$rs_rattr_translations` WHERE `language` = '".rp_escape_sql($language)."' LIMIT 1000;");

      $rpQuery->query("DELETE FROM `$rs_r_translations` WHERE `language` = '".rp_escape_sql($language)."' LIMIT 1000;");

      $rpQuery->query("DELETE FROM `$rs_translations` WHERE `language` = '".rp_escape_sql($language)."';");

      $rpQuery->query("COMMIT");

      // check if any row exists in $rs_dr_translations

      $row1 = $rpQuery->get_row("SELECT * FROM `$rs_rattr_translations` WHERE `language` = '".rp_escape_sql($language)."' LIMIT 1;");

      $row2 = $rpQuery->get_row("SELECT * FROM `$rs_r_translations` WHERE `language` = '".rp_escape_sql($language)."' LIMIT 1;");

      if( !empty( $row1 ) || !empty( $row2 ) ) {

            $GLOBALS['rp_keep_job_as_pending'] = true;

      }

}

function rp_create_unique_language_id( $language_name = '' ) {

      /* Added in V1.5.0 */

      if( trim($language_name) === 'English (United States)' ) {

            return 'lang_dfb21ebmKgeS';

      }

      if( trim($language_name) === 'Dutch (Netherlands)' ) {

            return 'lang_5cb71ecQxMIZ';

      }

      // create random and unique id with language name as prefix

      $language_name = trim($language_name);

      // limit language name to 10 characters

      $language_name = md5($language_name);

      $language_name = substr($language_name, 0, 6);

      $language_name = $language_name . rp_generate_random_string(6);

      return 'lang_' . $language_name;

}

function rp_generate_random_string( $length = 10 ) {

      /* Added in V1.5.0 */

      $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';

      $charactersLength = strlen($characters);

      $randomString = '';

      for ($i = 0; $i < $length; $i++) {

            $randomString .= $characters[rand(0, $charactersLength - 1)];

      }

      return $randomString;

}

function rp_update_languages_uq_id_for_first_time() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . 'rs_languages';

      $languages = $rpQuery->get_results("SELECT * FROM `$table_rs_languages` WHERE `uq_id` = '' OR `uq_id` IS NULL");

      if( !empty($languages) ) {

            foreach ($languages as $language) {

                  $uq_id = rp_create_unique_language_id($language->name);

                  $rpQuery->query("UPDATE `$table_rs_languages` SET `uq_id` = '".rp_escape_sql($uq_id)."' WHERE `id` = '".rp_escape_sql($language->id)."';");

            }

      }

}

// Add uq_id column in rs_languages table
function add_uq_id_column_to_rs_languages_table() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $table_rs_languages = $rpQuery->prefix . 'rs_languages';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($table_rs_languages)."' AND COLUMN_NAME = 'uq_id';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$table_rs_languages` ADD `uq_id` varchar(255) NULL DEFAULT NULL;");

            rp_update_languages_uq_id_for_first_time();
            
      }
}

add_action('init', 'add_uq_id_column_to_rs_languages_table', 999);


// Get all dr_uq_id from rs_default_repair table

function rp_get_all_dr_uq_id_from_rs_default_repair_table() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $dr_uq_ids = $rpQuery->get_results("SELECT `dr_uq_id` FROM `$rs_default_repair` GROUP BY `dr_uq_id`");

      if( !empty($dr_uq_ids) ) {
            
            $dr_uq_ids = array_column($dr_uq_ids, 'dr_uq_id');

            return $dr_uq_ids;

      }

      return array();

}


// Get all dr_uq_id from rs_default_repair table and c_uq_id from rs_category table where dr_category = c_id

function rp_get_all_dr_uq_id_and_c_uq_id() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_category = $rpQuery->prefix . 'rs_category';

      $dr_uq_ids = $rpQuery->get_results("SELECT `dr_uq_id`, `c_uq_id` FROM `$rs_default_repair` AS `dr` LEFT JOIN `$rs_category` AS `c` ON `dr`.`dr_category` = `c`.`c_id`");

      if( !empty($dr_uq_ids) ) {
            
            $new_arr = array();
            
            foreach( $dr_uq_ids as $row ) {

                  $new_arr[] = $row->c_uq_id . '|' . $row->dr_uq_id;

            }

            return $new_arr;

      }

      return array();

}

// Get all da_uq_id from rs_default_repair_attr table and c_uq_id from rs_category table where dr_category = c_id

function rp_get_all_da_uq_id_and_c_uq_id() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $rs_category = $rpQuery->prefix . 'rs_category';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $da_uq_ids = $rpQuery->get_results("SELECT `da_uq_id`, `c_uq_id` FROM `$rs_default_repair_attr` AS `dr` LEFT JOIN `$rs_category` AS `c` ON `dr`.`dr_category` = `c`.`c_id` INNER JOIN `$rs_default_repair` AS `d` ON `dr`.`dr_id_fk` = `d`.`dr_id`"); // temp back

      if( !empty($da_uq_ids) ) {
            
            $new_arr = array();
            
            foreach( $da_uq_ids as $row ) {

                  $new_arr[] = $row->c_uq_id . '|' . $row->da_uq_id;

            }

            return $new_arr;

      }

      return array();

}


// Get all uq_id from rs_languages table

function rp_get_all_uq_id_from_rs_languages_table() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_languages = $rpQuery->prefix . 'rs_languages';

      $uq_ids = $rpQuery->get_results("SELECT `uq_id` FROM `$rs_languages` GROUP BY `uq_id`");

      if( !empty($uq_ids) ) {
            
            $uq_ids = array_column($uq_ids, 'uq_id');

            return $uq_ids;

      }

      return array();

}

// Get all da_uq_id from rs_default_repair_attr table

function rp_get_all_da_uq_id_from_rs_default_repair_attr_table() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $da_uq_ids = $rpQuery->get_results("SELECT `da_uq_id` FROM `$rs_default_repair_attr` GROUP BY `da_uq_id`");

      if( !empty($da_uq_ids) ) {
            
            $da_uq_ids = array_column($da_uq_ids, 'da_uq_id');

            return $da_uq_ids;

      }

      return array();

}


function rp_check_if_combined_index_exists( $table_name = '', $column_names = array(), $prefix = TRUE ) {

      global $rpQuery;

      if($prefix === TRUE) {

            $table_name = $rpQuery->prefix . $table_name;

      }

      // explode column names by (
      
      foreach ($column_names as $key => $value) {

            $value = explode('(', $value);

            $column_names[$key] = $value[0];

      }

      $index_name = implode(',', $column_names);

      // show indexes from table

      $found_index = $rpQuery->get_results("SELECT `table_name`, GROUP_CONCAT(column_name ORDER BY seq_in_index) AS `index_columns`
      FROM information_schema.statistics
      WHERE `table_schema` = '".DB_NAME."' AND `table_name` = '$table_name'
      GROUP BY `table_name`, `index_name`;");

      if( !empty( $found_index ) ) {

            foreach ($found_index as $key => $value) {

                  if($value->index_columns === $index_name) {

                        return TRUE;

                  }

            }

      }

      return FALSE;

}

function rp_add_combined_index( $table_name = '', $index_name = '', $column_names = array(), $prefix = TRUE ) {

      global $rpQuery;

      $table_name_org = $table_name;

      if($prefix === TRUE) {

            $table_name = $rpQuery->prefix . $table_name;

      }

      $option_key = 'rp_index_added_'.$table_name_org.'_'.$index_name;

      $has_created_index = rp_get_option($option_key, FALSE);

      if(empty($has_created_index)) {

            $found_index = rp_check_if_combined_index_exists($table_name_org, $column_names, TRUE);

            if($found_index === FALSE) {

                  rp_add_option($option_key, TRUE);

                  foreach ($column_names as $key => $value) {

                        $value = explode('(', $value);

                        $column_names[$key] = '`'.$value[0].'`';

                        if(isset($value[1])) {

                              // remove ) from the end

                              $value[1] = substr($value[1], 0, -1);

                              $column_names[$key] .= '('.$value[1].')';

                        }

                  }

                  $column_names = implode(',', $column_names);

                  $sql = "ALTER TABLE `$table_name` ADD INDEX($column_names);";

                  $rpQuery->query($sql);

                  return TRUE;
            }

      }

      return FALSE;

}


function rp_add_combined_index_for_repair_used_count() {

      rp_add_combined_index('rs_repair', 'r_uq_id_m_id_fk', array('r_uq_id', 'm_id_fk'), TRUE);

      rp_add_combined_index('rs_repair_attr', 'a_uq_id_r_id_fk', array('a_uq_id', 'r_id_fk'), TRUE);

      rp_add_combined_index('rs_r_translations', 'val_key_uqid_lang', array('key(48)', 'uq_id(48)', 'language(100)', 'value(255)'), TRUE);

      rp_add_combined_index('rs_r_translations', 'key_uqid_lang', array('key(48)', 'uq_id(48)', 'language(100)'), TRUE);

      rp_add_combined_index('rs_r_translations', 'mid_key_uqid_lang', array('key(48)', 'uq_id(48)', 'language(100)', 'm_id_fk'), TRUE);

      rp_add_combined_index('rs_rattr_translations', 'val_key_uqid_lang', array('key(48)', 'uq_id(48)', 'language(100)', 'value(255)'), TRUE);

      rp_add_combined_index('rs_rattr_translations', 'key_uqid_lang', array('key(48)', 'uq_id(48)', 'language(100)'), TRUE);

      rp_add_combined_index('rs_rattr_translations', 'mid_key_uqid_lang', array('key(48)', 'uq_id(48)', 'language(100)', 'r_id_fk'), TRUE);

}

function rp_get_category_by_c_uq_id( $c_uq_id = '' ) {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';

      $category_row = $rpQuery->get_row("SELECT * FROM `$rs_category` WHERE `c_uq_id` = '$c_uq_id' LIMIT 1;");

      return ($category_row ?? array());

}

function rp_get_category_id_by_c_uq_id( $c_uq_id = '' ) {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';

      $category_id = $rpQuery->get_var("SELECT `c_id` FROM `$rs_category` WHERE `c_uq_id` = '$c_uq_id' LIMIT 1;");

      return ($category_id ?? '');

}

function rp_add_or_update_option( $key = '', $value = NULL ) {

      if( rp_get_option($key, FALSE) === FALSE ) {

            rp_add_option($key, $value);

      } else {

            rp_update_option($key, $value);
            
      }

}

function rp_get_all_c_uq_ids_from_rs_category() {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . 'rs_category';

      $c_uq_ids = $rpQuery->get_results("SELECT `c_uq_id` FROM `$rs_category`;");

      if( !empty( $c_uq_ids ) ) {

            $c_uq_ids = array_column($c_uq_ids, 'c_uq_id');

            return $c_uq_ids;

      }

      return array();

}

function rp_get_cached_default_color_by_id( $dc_id = '' ) {

      $cache_key = 'rp_get_cached_default_color_by_id_'.$dc_id;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      global $rpQuery;

      $rs_default_color = $rpQuery->prefix . 'rs_default_color';

      $default_color = $rpQuery->get_row("SELECT * FROM `$rs_default_color` WHERE `dc_id` = '$dc_id' LIMIT 1;");

      $result = array();

      if( !empty( $default_color ) ) {

            $result = $default_color;

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $result);

      return $result;

}


function rp_get_cached_default_repair_by_id( $dr_id = '' ) {

      $cache_key = 'rp_get_cached_default_repair_by_id_'.$dr_id;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $default_repair = $rpQuery->get_row("SELECT * FROM `$rs_default_repair` WHERE `dr_id` = '$dr_id' LIMIT 1;");

      $result = array();

      if( !empty( $default_repair ) ) {

            $result = $default_repair;

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $result);

      return $result;

}

function rp_get_cached_default_repair_attr_by_id( $dr_id = '' ) {

      $cache_key = 'rp_get_cached_default_repair_attr_by_id_'.$dr_id;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $default_repair_attrs = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE `dr_id_fk` = '$dr_id';");

      $result = array();

      if( !empty( $default_repair_attrs ) ) {

            $result = $default_repair_attrs;

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $result);

      return $result;

}

function rp_process_colors_and_repair_for_model($m_id = '') {

      $m_id = trim($m_id);

      if( empty( $m_id ) ) {

            if( !isset( $_REQUEST['rp_process_background_job'] ) || empty( $_REQUEST['rp_process_background_job'] ) ) {

                  rp_insert_rs_background_jobs_table(__FUNCTION__, array(''));
      
                  return;
      
            }

      }

      global $rpQuery;

      $rs_model = $rpQuery->prefix . 'rs_model';

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $rs_model_color = $rpQuery->prefix . 'rs_model_color';

      // select one row from rs_model where repair_ids != '' or color_ids != ''

      if( !empty( $m_id ) ) {

            $model = $rpQuery->get_row("SELECT `m_id`, `device_type`, `repair_ids`, `color_ids` FROM `$rs_model` WHERE `repair_ids` != '' AND `m_id` = '".rp_escape_sql($m_id)."' LIMIT 1;");

      } else {

            $model = $rpQuery->get_row("SELECT `m_id`, `device_type`, `repair_ids`, `color_ids` FROM `$rs_model` WHERE `repair_ids` != '' OR `color_ids` != '' LIMIT 1;");

      }

      if( !empty( $model ) ) {

            // start transaction

            $rpQuery->query("START TRANSACTION");

            $m_id_fk = $model->m_id;

            if( !empty( trim($model->color_ids) ) ) {

                  $color_ids = explode(',', $model->color_ids);

                  if( !empty( $color_ids ) ) {

                        foreach( $color_ids as $c ) {

                              if( empty( trim($c) ) ) {

                                    continue;

                              }

                              $d_color = rp_get_cached_default_color_by_id( $c );

                              if( empty( $d_color ) ) {

                                    continue;

                              }

                              $color_row = array(
                                    'color_name' => $d_color->dc_name,
                                    'color_code' => $d_color->dc_code,
                                    'mc_uq_id'   => $d_color->dc_uq_id,
                                    'm_id_fk'    => $m_id_fk
                              );

                              // mc_uq_id and m_id_fk must not be in rs_model_color

                              $exists = $rpQuery->get_row("SELECT `mc_id` FROM `$rs_model_color` WHERE `mc_uq_id` = '".rp_escape_sql($d_color->dc_uq_id)."' AND `m_id_fk` = '".rp_escape_sql($m_id_fk)."' LIMIT 1;");

                              if( empty( $exists ) ) {

                                    $rpQuery->insert($rs_model_color, $color_row);

                              }

                        }

                  }

            }

            if( !empty( trim($model->repair_ids) ) ) {

                  $repair_ids = explode(',', $model->repair_ids);

                  if( !empty( $repair_ids ) ) {

                        foreach( $repair_ids as $r ) {

                              if( empty( trim($r) ) ) {

                                    continue;

                              }

                              $dr_repair = rp_get_cached_default_repair_by_id( $r );

                              if( empty( $dr_repair ) ) {

                                    continue;

                              }

                              $dr_newdata = array(
                                    'r_name' => $dr_repair->dr_name,
                                    'r_price' => $dr_repair->dr_price,
                                    'r_duration' => $dr_repair->dr_duration,
                                    'r_duration_measure' => $dr_repair->dr_duration_measure,
                                    'r_discription' => $dr_repair->dr_discription,
                                    'r_badge' => $dr_repair->dr_badge,
                                    'r_img' => $dr_repair->dr_img,
                                    'is_free' => $dr_repair->is_free,
                                    'is_active' => $dr_repair->is_active,
                                    'r_uq_id' => $dr_repair->dr_uq_id,
                                    'm_id_fk' => $m_id_fk
                              );

                              $dr_newdata = apply_filters('rp_processing_repair_for_manually_added_model', $dr_newdata, $dr_repair);

                              // r_uq_id and m_id_fk must not be in rs_repair

                              $exists = $rpQuery->get_row("SELECT `r_id` FROM `$rs_repair` WHERE `r_uq_id` = '$dr_repair->dr_uq_id' AND `m_id_fk` = '$m_id_fk' LIMIT 1;");

                              if( empty( $exists ) ) {

                                    $rpQuery->insert($rs_repair, $dr_newdata);

                                    $r_id_fk = $rpQuery->insert_id;

                                    $attrs = rp_get_cached_default_repair_attr_by_id($r);

                                    if( !empty( $attrs ) ) {

                                          foreach($attrs as $a){

                                                // delete existing rows that match a_uq_id and r_id_fk

                                                $rpQuery->query("DELETE FROM `$rs_repair_attr` WHERE `a_uq_id` = '".rp_escape_sql($a->da_uq_id)."' AND `r_id_fk` = '".rp_escape_sql($r_id_fk)."'");

								$ra_name   = $a->da_name;
                                                $ra_desc   = $a->da_desc;
                                                $ra_badge   = $a->da_badge;
                                                $ra_duration   = $a->da_duration;
                                                $ra_duration_measure   = $a->da_duration_measure;
								$ra_price  = $a->da_price;
								$a_uq_id   = $a->da_uq_id;
                                                $is_active = $a->is_active;

                                                if( apply_filters('rp_process_attr_margin_for_manually_added_model', FALSE) === TRUE ) {
                                                      $a_margin_col = ', a_margin';
                                                      $a_margin_val = ",'".rp_escape_sql($a->da_margin)."'";
                                                } else {
                                                      $a_margin_col = '';
                                                      $a_margin_val = '';
                                                }

								$rpQuery->query("INSERT INTO `$rs_repair_attr` (a_name, a_desc, a_badge, a_duration, a_duration_measure,a_price,r_id_fk,a_uq_id,is_active $a_margin_col) VALUES('".rp_escape_sql($ra_name)."','".rp_escape_sql($ra_desc)."','".rp_escape_sql($ra_badge)."','".rp_escape_sql($ra_duration)."','".rp_escape_sql($ra_duration_measure)."','".rp_escape_sql($ra_price)."','".rp_escape_sql($r_id_fk)."','".rp_escape_sql($a_uq_id)."','".rp_escape_sql($is_active)."' $a_margin_val)");
							}

                                    }

                              }

                        }

                  }

            }

            $rpQuery->query("UPDATE `$rs_model` SET `repair_ids` = '', `color_ids` = '' WHERE `m_id` = '".rp_escape_sql($m_id_fk)."' LIMIT 1;");

            $rpQuery->query("COMMIT");

      }

      // select one row from rs_model where repair_ids != '' or color_ids != ''

      if( empty( $m_id ) ) {

            $still_have = $rpQuery->get_row("SELECT `m_id`, `device_type`, `repair_ids`, `color_ids` FROM `$rs_model` WHERE `repair_ids` != '' OR `color_ids` != '' LIMIT 1;");
      
            if( !empty( $still_have ) ) {

                  $GLOBALS['rp_keep_job_as_pending'] = true;

            }

      }

}

function rp_trigger_translations_update_for_all_dr_repairs() {

      global $rpQuery;

      $rs_dr_translations = $rpQuery->prefix . 'rs_dr_translations';

      // select distinct uq_id from rs_dr_translations

      $dr_uq_ids = $rpQuery->get_results("SELECT DISTINCT `uq_id` FROM `$rs_dr_translations`;");

      $the_data = array();

      $the_attrs_data = array();

      if( !empty( $dr_uq_ids ) ) {

            foreach( $dr_uq_ids as $row ) {

                  $uq_id = $row->uq_id;

                  // select * from rs_dr_translations where uq_id = $uq_id

                  $allowed_keys = array('repair_name', 'repair_desc', 'repair_badge');

                  $translations = $rpQuery->get_results("SELECT * FROM `$rs_dr_translations` WHERE `uq_id` = '".rp_escape_sql($uq_id)."';");

                  if( !empty( $translations ) ) {

                        foreach( $translations as $dt_row ) {

                              if( !in_array($dt_row->key, $allowed_keys) ) {

                                    if( $dt_row->key === 'attr_name' || $dt_row->key === 'attr_desc' || $dt_row->key === 'attr_badge' ) {

                                          $the_attrs_data[] = array($dt_row->key, $dt_row->uq_id, $dt_row->language, $dt_row->value);

                                    }

                              } else {

                                    $the_data[] = array($dt_row->key, $dt_row->uq_id, $dt_row->language, $dt_row->value);

                              }

                        }

                  }

            }


      }

      foreach( $the_data as $the_row ) {

            rp_insert_rs_background_jobs_table('rp_create_matching_repair_translations_if_not_already', $the_row);

      }

      foreach( $the_attrs_data as $the_attr_row ) {

            rp_insert_rs_background_jobs_table('rp_create_matching_attr_translations_if_not_already', $the_attr_row);

      }
      
      return $the_data;

}


function download_languages_from_server_first_time() {
      global $rpQuery;
      if(isset($_POST['lang_ids'])) {
            $lang_ids = $_POST['lang_ids'] ?? array();
            if(empty($lang_ids)) {
                  return;
            }
            $baselang_class = new RepairPluginPro\Download_Languages($rpQuery);

            $alreadyDownloaded = rp_get_uq_id_of_all_rs_languages();

            foreach($lang_ids as $key => $lang_id) {
                  if(in_array($lang_id, $alreadyDownloaded)) {
                        unset($lang_ids[$key]);
                  }
            }

            $lang_ids = array_values($lang_ids);

            if( empty( $lang_ids ) ) {

                  return;

            }

            $_POST['lang_uq_ids'] = $lang_ids;

            $response = $baselang_class->fetch_repair_base(TRUE, FALSE);

            $baselang_class->save_handler_first_time($response);
      }
}

/*
Add INDEXES to most used columns to make
quries faster!
*/

function add_indexes_to_wp_rs_repair_table(){

      $indexes = array(

            'rs_repair|m_id_fk',

            'rs_repair|r_uq_id',

            'rs_repair_attr|r_id_fk',

            'rs_repair_attr|a_uq_id',

            'rs_model|device_type',

            'rs_model|brand_id_fk',

            'rs_model_color|m_id_fk',

            'rs_model_color|mc_uq_id',

            'rs_default_color|dc_uq_id',

            'rs_r_translations|language',

            'rs_r_translations|key',

            'rs_r_translations|uq_id',

            'rs_r_translations|m_id_fk',

            'rs_rattr_translations|language',

            'rs_rattr_translations|key',

            'rs_rattr_translations|uq_id',

            'rs_rattr_translations|r_id_fk',

            'rs_dr_translations|language',

            'rs_dr_translations|key',

            'rs_dr_translations|uq_id',

            'rs_translations|language',

            'rs_translations|key',

            'rp_add_combined_index_for_repair_used_count'

      );

      foreach ($indexes as $values) {

            if( $values === 'rp_add_combined_index_for_repair_used_count' ) {

                  rp_add_combined_index_for_repair_used_count();

                  break;

            }

            list($table_name, $column_name) = explode('|',$values);

            if(rp_add_index_to_column_if_not_exists($table_name, $column_name) === TRUE) {

                  break;

            }

      }

}
add_action('init', 'add_indexes_to_wp_rs_repair_table', 999);

function rp_add_first_time_languages_and_translations(){
      
      if( empty( $_REQUEST['action'] ?? '' ) && !wp_doing_ajax() ) {

            return;

      }

      global $rpQuery;

      $rs_translations = $rpQuery->prefix . 'rs_translations';

      $rs_languages = $rpQuery->prefix . 'rs_languages';

      if( !defined('WP_REPAIR_REQUIRE_LICENSE') ) {           

            if( rp_get_option('rp_download_lang_translations_first_time', FALSE) === FALSE ) {

                  rp_add_option('rp_download_lang_translations_first_time', '1');

                  // ----------- MAKING SURE IT'S FIRST TIME DOWNLOAD
                  // in rs_planning_and_discount table, make pickup_service and repair_on_location = 1
                  $rs_planning_and_discount = $rpQuery->prefix . 'rs_planning_and_discount';
                  
                  $rpQuery->query("UPDATE `$rs_planning_and_discount` SET `pickup_service` = 1, `repair_on_location` = 1");
                  // ------------ MAKING SURE IT'S FIRST TIME DOWNLOAD

                  if( !isset($_POST) ) {

                        $_POST = array();
      
                  }

                  $_POST['lang_ids'] = array('lang_5cb71ecQxMIZ', 'lang_dfb21ebmKgeS');

                  // empty rs_languages table

                  $rpQuery->query("TRUNCATE TABLE `$rs_languages`;");

                  // empty rs_translations table

                  $rpQuery->query("TRUNCATE TABLE `$rs_translations`;");

                  rp_assume_existing_text_as_translated();

                  rp_assume_existing_text_desc_as_translated();

                  download_languages_from_server_first_time();

                  // Latest translations already fetched so mark all groups as fetched

                  $new_translation_groups = apply_filters('rp_force_fetch_new_translation_groups', array());

                  if( !empty( $new_translation_groups ) ) {

                        foreach( $new_translation_groups as $group => $groupItems ) {

                              if( rp_get_option( $group, FALSE ) === FALSE ) {
      
                                    rp_add_option( $group, '1' );

                              }
      
                        }

                  }

            }

      }

}

add_action('init', 'rp_add_first_time_languages_and_translations', 999);


function rp_assume_existing_text_as_translated() {

      $key_value = json_decode('{"repair_105679":"Koptelefoonpoort","repair_112816":"GPS-antenne","repair_121308":"NFC-antenne","repair_139755":"Touchscreen (glas)","repair_163266":"Gyroscoop (draaisensor)","repair_197069":"Softwarereset","repair_198982":"Luidspreker","repair_223897":"Display (LCD)","repair_241218":"Geheugenkaartlezer","repair_265315":"GSM-antenne","repair_266233":"Bluetooth-antenne","repair_272218":"Waterschade","repair_279312":"Simkaartlezer","repair_325712":"Volumeknop(pen)","repair_351195":"Cameraglas","repair_388209":"Nabijheidssensor","repair_401401":"Schermmodule","repair_426379":"Gezichtsherkenningsmodule","repair_433241":"Oplaadpoort","repair_445208":"Oorspeaker","repair_520851":"Vingerafdrukscanner","repair_537674":"Aan-uitknop","repair_540484":"WiFi-antenne","repair_545344":"Batterij","repair_551559":"Stille Modusknop","repair_566600":"Onderzoeken","repair_587656":"Camera voorzijde","repair_680281":"Overzetten oude gegevens","repair_732129":"Draadloze oplaadmodule","repair_768329":"Achterpaneel","repair_806428":"Vergrendelingsreset","repair_839922":"Flitser","repair_860132":"Home-knop","repair_870919":"Microfoon (video)","repair_927037":"Moederbord","repair_929441":"Trilmotor","repair_945141":"Microfoon (telefoon)","repair_947962":"Behuizing","repair_993586":"Camera achterzijde"}', TRUE);

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_dr_translations = $rpQuery->prefix . 'rs_dr_translations';

      $languages = array('English (United States)', 'Dutch (Netherlands)');

      // get all default repair

      $default_repairs = $rpQuery->get_results("SELECT * FROM `$rs_default_repair`");

      $keys = array('repair_name');

      foreach($default_repairs as $drepair) {

            $dr_name = $drepair->dr_name;

            $uq_id = $drepair->dr_uq_id;
            
            if( isset($key_value[$uq_id]) ) {
                  // Is Downloaded Default Repair
                  
                  if( trim($dr_name) !== trim($key_value[$uq_id]) ) {
                        // Has Changed Default Repair Name

                        foreach($keys as $key) {

                              foreach($languages as $language) {

                                    $rpQuery->insert($rs_dr_translations, array(

                                          'language' => $language,

                                          'key' => $key,

                                          'uq_id' => $uq_id,

                                          'value' => $drepair->dr_name

                                    ));

                              }

                        }

                  }

            }

      }
      
}

function rp_assume_existing_text_desc_as_translated() {

      $key_value = json_decode('{"repair_105679":"Verstopping van de ingang en\/of geen verbinding met bekabelde headset mogelijk.","repair_112816":"Problemen met navigatie en\/of het verkrijgen locatiegegevens.","repair_121308":"Problemen met contactloos betalen, digitale toegangsbewijzen of legitimatie met DigiD-app.","repair_139755":"Scherm reageert niet\/slecht op aanrakingen en\/of is gebarsten.","repair_163266":"Bij kwartslag apparaat draait scherm niet mee.","repair_197069":"Problemen met updates besturingssysteem of wissen van gegevens (gebruikersinformatie, beeldmateriaal en apps).","repair_198982":"Weinig, slecht of geen geluid uit interne speakers.","repair_223897":"Het scherm toont ongewenste strepen, vegen en\/of kleuren.","repair_241218":"SD-kaart wordt niet herkend en\/of de pinnetjes zijn gebroken.","repair_265315":"Niet mogelijk om verbinding te maken met 3G- of 4G-netwerk.","repair_266233":"Verbindingsproblemen met draadloze speakers, headsets of andere randapparatuur.","repair_272218":"Defecten na contact met water of andere vloeistoffen.","repair_279312":"Problemen met herkennen van simkaart en\/of gebroken pinnetjes in de sleuf.","repair_325712":"Knop reageert niet en\/of kan niet ingedrukt worden.","repair_351195":"Vuil onder en\/of beschadigingen in het glas van de lens.","repair_388209":"Schermslot schakelt zichzelf niet in tijdens telefoongesprekken aan het oor. ","repair_401401":"Fysieke barsten en\/of ongewenste strepen\/vegen\/kleuren in het display.","repair_426379":"Ontgrendelen via iris- of gezicht is niet mogelijk.","repair_433241":"Opladen en\/of verbinding maken met computer niet mogelijk.","repair_445208":"Tijdens telefoongesprekken kun je de andere persoon niet of slecht horen.","repair_520851":"Problemen met ontgrendelen of legitimatie via vingerafdruk.","repair_537674":"De aan-uitknop reageert niet en\/of kan niet ingedrukt worden.","repair_540484":"Last van trage verbinding via WiFi, oorzaak ligt niet bij internetprovider.","repair_545344":"Apparaat valt onverwacht uit, is snel leeg en\/of moet vaak aan de lader.","repair_551559":"Schuifknop aan de zijkant werkt niet en\/of kan niet verschoven worden.","repair_566600":"Niet zeker wat er mis is? Geen probleem! Wij voeren een onderzoek uit.","repair_587656":"Selfie-camera werkt niet en\/of maakt slechte, onduidelijke of onscherpe beelden.","repair_680281":"Overdracht bestanden van oud naar nieuw toestel zonder verlies persoonlijke gegevens.","repair_732129":"Bekabeld opladen werkt normaal terwijl draadloos niet mogelijk is.","repair_768329":"Barsten in glasplaat aan de achterkant.","repair_806428":"Geen ontgrendeling mogelijk omdat toegangscode niet werkt, ontbreekt en\/of vergeten is.","repair_839922":"De camera flits kan niet meer ingeschakeld worden en\/of uw zaklamp werkt niet. ","repair_860132":"De knop (onder) reageert niet en\/of kan niet ingedrukt worden.","repair_870919":"Geen\/slechte audio van video-opnames en\/of geen gehoor bij videobellen of telefoongesprekken via de luidspreker.","repair_927037":"Vervangbare onderdelen lijken allemaal in orde, toestel toch defect. (Nader onderzoek vereist)","repair_929441":"Trilfunctie werkt niet bij notificaties.","repair_945141":"Anderen horen jou niet of slecht tijdens telefoongesprekken.","repair_947962":"Deuken of krassen aan de randen en\/of achterkant.","repair_993586":"Camera(\u2019s) aan de achterkant van het toestel werkt niet en\/of maakt slechte, onduidelijke of onscherpe beelden."}', TRUE);

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_dr_translations = $rpQuery->prefix . 'rs_dr_translations';

      $languages = array('English (United States)', 'Dutch (Netherlands)');

      // get all default repair

      $default_repairs = $rpQuery->get_results("SELECT * FROM `$rs_default_repair`");

      $keys = array('repair_desc');

      foreach($default_repairs as $drepair) {

            $dr_discription = $drepair->dr_discription;

            $uq_id = $drepair->dr_uq_id;
            
            if( isset($key_value[$uq_id]) ) {
                  // Is Downloaded Default Repair
                  
                  if( trim($dr_discription) !== trim($key_value[$uq_id]) ) {
                        // Has Changed Default Repair Name

                        foreach($keys as $key) {

                              foreach($languages as $language) {

                                    $rpQuery->insert($rs_dr_translations, array(

                                          'language' => $language,

                                          'key' => $key,

                                          'uq_id' => $uq_id,

                                          'value' => $drepair->dr_discription

                                    ));

                              }

                        }

                  }

            }

      }
      
}

function rp_get_search_box_redirect_link() {

      /* Added in V1.5.0 */

	$permalink = rp_get_option('rp_search_box_redirect_option', 'Find It Automatically');

	if( $permalink === 'Use My Custom Link' ) {

		$link = rp_get_option('rp_search_box_custom_redirect_link', '');

		if( !empty( $link ) && filter_var($link, FILTER_VALIDATE_URL) ) {
			
			return $link;

		}

	}

	return '';

}



// add 'Approved (Send device)' in noti_type column of rs_notification_types table

function add_approved_send_device_in_rs_notification_types() {
      
      /* Added in V1.4.5 */

      global $rpQuery;

      $noti_type = 'Booking Approved (Send device)';

      $rs_notification_types = $rpQuery->prefix . 'rs_notification_types';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notification_types` WHERE `noti_type` = '".rp_escape_sql($noti_type)."' LIMIT 1;");

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notification_types` (`noti_type`, `is_active`) VALUES ('".rp_escape_sql($noti_type)."', '1');");
            
      }
}

add_action('init', 'add_approved_send_device_in_rs_notification_types', 999);


// insert into all columns from rs_notifications_and_offers table if noti_type_id_fk 5 doesn't exist

function insert_approved_send_device_markup_in_notification_table() {
      
      /* Added in V1.4.5 */

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notifications_and_offers` WHERE `noti_type_id_fk` = '6' LIMIT 1;");

      $subject = 'Uw afspraak is goedgekeurd';

      $message = '<p>Beste $CustomerName$,</p> <p>Bedankt voor het maken van&nbsp;een afspraak bij $LocationName$ om jouw&nbsp;$Model$ te laten repareren.</p> <h3 style="font-family: sans-serif; font-size: 24px; border-radius: 8px; padding: 6px 12px; margin-bottom: 10px; background-color: #d5e4f4; color: #000000; border: 1px solid #b5c6b6;">De afspraak is op $AppointmentDate$, om $AppointmentTime$ uur.</h3> <p>&nbsp;</p> <p>$OrderSummary$</p> <p>&nbsp;</p> <p><strong>Uw gegevens<br></strong>Naam:&nbsp;$CustomerName$<br>Email:&nbsp;$CustomerEmail$<br>Tel:&nbsp;$CustomerPhone$</p> <p>&nbsp;</p> <p>Tijdens de reparatie kun je wachten bij ons in de winkel. We hebben supersnelle Wi-fi en heerlijke koffie. Komt het toch niet uit en wil je de afspraak annuleren?<br /><br />Klik dan op de volgende link: $cancel$.</p> <p>&nbsp;</p> <p>Met vriendelijke groet,</p> <p><strong>$LocationName$<br /></strong>$LocationAddress$<br />$LocationZipcodeCity$</p> <p>&nbsp;</p> <p>&nbsp;</p> <footer style="margin-top: 20px; font-size: 80%; color: #888; clear: both;"> <p>Je krijgt deze e-mail omdat er een afspraak is gemaakt met dit emailadres op de website van $LocationName$. Deze e-mail is een bevestiging van de gemaakte afspraak. Heb je deze afspraak niet zelf gemaakt, neem dan contact met ons op.</p> </footer>';

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notifications_and_offers` (`noti_subject`, `noti_message`, `noti_type_id_fk`, `is_active`, `date_time`) VALUES ('$subject', '$message', '6', '1', '".date('Y-m-d H:i:s')."');");
            
      }
}

add_action('init', 'insert_approved_send_device_markup_in_notification_table', 999);


/*
Add coupons_type column to rs_coupons table
*/

function add_coupons_type_column_to_rs_coupons() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_coupons = $rpQuery->prefix . 'rs_coupons';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_coupons)."' AND COLUMN_NAME = 'coupons_type';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_coupons` ADD `coupons_type` varchar(32) NOT NULL DEFAULT 'fixed';");
            
      }
}

add_action('init', 'add_coupons_type_column_to_rs_coupons', 999);

/*
Add coupon_amount column to rs_user_pdfs table
*/

function add_coupon_amount_column_to_rs_orders() {
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'coupon_amount';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `coupon_amount` varchar(32) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_coupon_amount_column_to_rs_orders', 999);

/*
Add dm_cost column to rs_orders table
*/

function add_dm_cost_column_to_rs_orders() {
      /* Added in V1.5.6 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'dm_cost';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `dm_cost` varchar(32) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_dm_cost_column_to_rs_orders', 999);



/*
Create RepairPlugin Special Timing Table
*/
function repair_plugin_special_timing_table(){
      /* Added in V1.5.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_special_timing = $rpQuery->prefix . "rs_special_timing"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_special_timing (

      `cs_id` int(11) NOT NULL AUTO_INCREMENT,

      `cs_day` varchar(255) NOT NULL,

      `cs_open_timing` varchar(255) NOT NULL,

      `cs_close_timing` varchar(255) NOT NULL,

      `cs_breaktime_from` varchar(255) NULL DEFAULT '',

      `cs_breaktime_to` varchar(255) NULL DEFAULT '',

      `is_closed` tinyint(4) NOT NULL DEFAULT '0',

      `location_id_fk` int(11) NOT NULL,

      `reminder_name` varchar(255) NOT NULL,

      `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

      PRIMARY KEY (`cs_id`))  

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_special_timing_table');


function rp_get_last_models_downloaded_date() {

      /* Added in V1.5.0 */

      global $rpQuery;

      $model_table = $rpQuery->prefix . 'rs_model';

      $latest_package = $rpQuery->get_row("SELECT * FROM $model_table ORDER BY updated_on DESC");

      if(!empty($latest_package)){

            $date = strtotime($latest_package->updated_on);

            $date = date('Y-m-d H:i:s', $date);
            
            return $date;

      }

      return FALSE;

}

function rp_update_option_for_existing_customers() {

      /* Added in V1.5.0 */

      $date = rp_get_last_models_downloaded_date();

      if( !empty( $date ) && rp_get_option('rp_download_models_last_update', FALSE) === FALSE ) {

            rp_add_option('rp_download_models_last_update', $date);

      }

}

add_action('init', 'rp_update_option_for_existing_customers', 999);


function rp_add_a_desc_column_to_rs_repair_attr() {

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair_attr)."' AND COLUMN_NAME = 'a_desc';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair_attr` ADD `a_desc` varchar(255) NULL DEFAULT '' AFTER `a_name`;");
            
      }

}

add_action('init', 'rp_add_a_desc_column_to_rs_repair_attr', 999);


function rp_add_da_desc_column_to_rs_default_repair_attr() {

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair_attr)."' AND COLUMN_NAME = 'da_desc';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair_attr` ADD `da_desc` varchar(255) NULL DEFAULT '' AFTER `da_name`;");
            
      }

}

add_action('init', 'rp_add_da_desc_column_to_rs_default_repair_attr', 999);


/*
Add reschedule_link column to rs_orders table
*/

function add_reschedule_link_column_to_rs_orders() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'reschedule_link';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `reschedule_link` varchar(1000) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_reschedule_link_column_to_rs_orders', 999);



// add 'Reminder' in noti_type column of rs_notification_types table

function add_reminder_in_rs_notification_types() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $noti_type = 'Reminder';

      $rs_notification_types = $rpQuery->prefix . 'rs_notification_types';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notification_types` WHERE `noti_type` = '".rp_escape_sql($noti_type)."' LIMIT 1;");

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notification_types` (`noti_type`, `is_active`) VALUES ('".rp_escape_sql($noti_type)."', '1');");
            
      }
}

add_action('init', 'add_reminder_in_rs_notification_types', 999);


// insert into all columns from rs_notifications_and_offers table if noti_type_id_fk 5 doesn't exist

function insert_reminder_markup_in_notification_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notifications_and_offers` WHERE `noti_type_id_fk` = '7' LIMIT 1;");

      $subject = 'Subject for Reminder Email';

      $message = 'This is message for reminder email';

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notifications_and_offers` (`noti_subject`, `noti_message`, `noti_type_id_fk`, `is_active`, `date_time`) VALUES ('$subject', '$message', '7', '1', '".date('Y-m-d H:i:s')."');");
            
      }
}

add_action('init', 'insert_reminder_markup_in_notification_table', 999);


// add 'Aftersales' in noti_type column of rs_notification_types table

function add_aftersales_in_rs_notification_types() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $noti_type = 'Aftersales';

      $rs_notification_types = $rpQuery->prefix . 'rs_notification_types';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notification_types` WHERE `noti_type` = '".rp_escape_sql($noti_type)."' LIMIT 1;");

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notification_types` (`noti_type`, `is_active`) VALUES ('".rp_escape_sql($noti_type)."', '1');");
            
      }
}

add_action('init', 'add_aftersales_in_rs_notification_types', 999);


// insert into all columns from rs_notifications_and_offers table if noti_type_id_fk 5 doesn't exist

function insert_aftersales_markup_in_notification_table() {
      
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notifications_and_offers` WHERE `noti_type_id_fk` = '8' LIMIT 1;");

      $subject = 'Subject for Aftersales Email';

      $message = 'This is message for aftersales email';

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notifications_and_offers` (`noti_subject`, `noti_message`, `noti_type_id_fk`, `is_active`, `date_time`) VALUES ('$subject', '$message', '8', '1', '".date('Y-m-d H:i:s')."');");
            
      }
}

add_action('init', 'insert_aftersales_markup_in_notification_table', 999);


function rp_get_all_reminder_emails() {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $all_orders = $rpQuery->get_results("SELECT * FROM `$rs_orders` WHERE `reminder` = '0' AND (`created_time` IS NOT NULL AND `created_time` != '');");

      $filtered_orders = array();

      $rp_x_hours_before_appointment = (int) rp_get_option('rp_x_hours_before_appointment', '2');

      foreach ($all_orders as $order) {

            $_delivery_method = str_replace(' ', '_', strtolower(trim($order->delivery_method)));

            if(empty( ($order->created_time ?? '') ) || $_delivery_method == 'ship_device' || $_delivery_method == 'pdf_offer' ) {

                  continue;

            }

            if( empty( trim( $order->delivery_date ) ) || empty( trim( $order->delivery_time ) ) ) {
                  
                  continue;

            }

            $delivery_date = $order->delivery_date;

            if( strpos($delivery_date, '-') !== 4 ) {
                  
                  // convert d-m-Y to Y-m-d
                  $delivery_date_arr = explode('-', $delivery_date);
                  $delivery_date = $delivery_date_arr[2] . '-' . $delivery_date_arr[1] . '-' . $delivery_date_arr[0];

            }

            if( strpos( $order->delivery_time, ' — ' ) !== FALSE ) {

                  $delivery_time = explode(' — ', $order->delivery_time)[0];

                  $delivery_time = date('H:i:s', strtotime($delivery_time));

            } else {
                  
                  $delivery_time = date('H:i:s', strtotime($order->delivery_time));

            }

            $appointment_date_time = strtotime($delivery_date . ' ' . $delivery_time); // 10:30

            $appointment_date_time_less_x_hours = $appointment_date_time - ($rp_x_hours_before_appointment * 3600); // 10:30 - 2 hours = 8:30

            $created_time = strtotime($order->created_time);

            $diff = $appointment_date_time - $created_time;

            $diff_in_hours = $diff / 3600;
            
            if( $diff_in_hours > $rp_x_hours_before_appointment ) {

                  // We will have opportunity to send reminder email
                  $current_time = strtotime(date('Y-m-d H:i:s'));

                  if( $current_time >= $appointment_date_time_less_x_hours && $current_time < $appointment_date_time ) {

                        $filtered_orders[] = $order->o_id;

                  }

            }

      }

      return $filtered_orders;

}

function rp_get_all_aftersales_emails() {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $all_orders = $rpQuery->get_results("SELECT * FROM `$rs_orders` WHERE `aftersales` = '0' AND (`created_time` IS NOT NULL AND `created_time` != '');");

      $filtered_orders = array();

      $rp_x_hours_after_appointment = (int) rp_get_option('rp_x_hours_after_appointment', '2');

      foreach ($all_orders as $order) {

            $_delivery_method = str_replace(' ', '_', strtolower(trim($order->delivery_method)));

            if(empty( ($order->created_time ?? '') ) || $_delivery_method == 'ship_device' || $_delivery_method == 'pdf_offer' ) {

                  continue;

            }

            if( empty( trim( $order->delivery_date ) ) || empty( trim( $order->delivery_time ) ) ) {
                  
                  continue;

            }

            $delivery_date = $order->delivery_date;

            if( strpos($delivery_date, '-') !== 4 ) {
                  
                  // convert d-m-Y to Y-m-d
                  $delivery_date_arr = explode('-', $delivery_date);
                  $delivery_date = $delivery_date_arr[2] . '-' . $delivery_date_arr[1] . '-' . $delivery_date_arr[0];

            }

            if( strpos( $order->delivery_time, ' — ' ) !== FALSE ) {

                  $delivery_time = explode(' — ', $order->delivery_time)[1];

                  $delivery_time = date('H:i:s', strtotime($delivery_time));

            } else {
                  
                  $delivery_time = date('H:i:s', strtotime($order->delivery_time));

            }

            $appointment_date_time = strtotime($delivery_date . ' ' . $delivery_time); // 10:30

            $appointment_date_time_plus_x_hours = $appointment_date_time + ($rp_x_hours_after_appointment * 3600); // 10:30 + 2 hours = 12:30
            
            $current_time = strtotime(date('Y-m-d H:i:s'));

            if( $current_time >= $appointment_date_time_plus_x_hours ) {

                  $filtered_orders[] = $order->o_id;

            }

      }

      return $filtered_orders;

}

function rp_send_all_reminder_emails() {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $filtered_orders = rp_get_all_reminder_emails();

      foreach ($filtered_orders as $order_id) {

            rp_send_appointment_status_update_email($order_id, FALSE);

            // update reminder status to 1
            $rpQuery->query("UPDATE `$rs_orders` SET `reminder` = '1' WHERE `o_id` = '$order_id';");

      }

}

function rp_send_all_aftersales_emails() {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $filtered_orders = rp_get_all_aftersales_emails();

      foreach ($filtered_orders as $order_id) {

            rp_send_appointment_status_update_email($order_id, FALSE);

            // update aftersales status to 1
            $rpQuery->query("UPDATE `$rs_orders` SET `aftersales` = '1' WHERE `o_id` = '$order_id';");

      }

}

function _rp_return_7($status = '', $order = NULL) {
      return 7;
}

function _rp_return_8($status = '', $order = NULL) {
      return 8;
}

function rp_send_scheduled_emails() {

      global $rp_cron_job;

      if( isset($rp_cron_job) && $rp_cron_job === 'rp' ) {

            _rp_set_default_tiemzone();

            // Reminder
            add_filter('rp_override_status_while_sending_email', '_rp_return_7', 2);

            rp_send_all_reminder_emails();

            remove_filter('rp_override_status_while_sending_email', '_rp_return_7');

            // Aftersales
            add_filter('rp_override_status_while_sending_email', '_rp_return_8', 2);

            rp_send_all_aftersales_emails();

            remove_filter('rp_override_status_while_sending_email', '_rp_return_8');

      }

      exit;

}

function _run_rp_send_scheduled_emails_through_cron() {

      if( isset( $_REQUEST['action'] ) && ( $_REQUEST['action'] ?? '' ) == 'rp_send_scheduled_emails' ) {

            global $rp_cron_job;

            if( isset($rp_cron_job) && $rp_cron_job === 'rp' ) {

                  rp_send_scheduled_emails();

            }

      }

}

add_action('init', '_run_rp_send_scheduled_emails_through_cron', 9999);


/*
Add created_time column to rs_orders table
*/

function add_created_time_column_to_rs_orders() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'created_time';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `created_time` varchar(64) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_created_time_column_to_rs_orders', 999);

/*
Add reminder column to rs_orders table
*/

function add_reminder_column_to_rs_orders() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'reminder';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `reminder` tinyint(1) NOT NULL DEFAULT '0';");
            
      }
}

add_action('init', 'add_reminder_column_to_rs_orders', 999);


/*
Add aftersales column to rs_orders table
*/

function add_aftersales_column_to_rs_orders() {
      /* Added in V1.5.0 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'aftersales';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `aftersales` tinyint(1) NOT NULL DEFAULT '0';");
            
      }
}

add_action('init', 'add_aftersales_column_to_rs_orders', 999);

function rp_update_general_info() {

      rp_verify_csrf_token_for_ajax('general_info');

      global $rpQuery;

      $company_information = $rpQuery->prefix . "rs_company_information";
      $company_location    = $rpQuery->prefix . "rs_company_location";
      $company_schedules   = $rpQuery->prefix . "rs_company_schedules";

      $c_results = $rpQuery->get_results("SELECT * FROM `$company_information`");
      $company_id = $c_results[0]->company_id;

      // Update General Information

      $c_name        = $_POST['c_name'];
      $c_email       = $_POST['c_email'];
      $c_phone       = $_POST['c_phone'];
      $c_reg_no      = $_POST['c_reg_no'];
      $tax_reg_no    = $_POST['tax_reg_no'];

      $rpQuery->query("UPDATE $company_information SET company_name = '".rp_escape_sql($c_name)."',company_email = '".rp_escape_sql($c_email)."',company_phone = '".rp_escape_sql($c_phone)."',company_reg_no = '".rp_escape_sql($c_reg_no)."',tax_reg_no = '".rp_escape_sql($tax_reg_no)."' WHERE company_id='".rp_escape_sql($company_id)."'");

      if( isset( $_POST['terms_and_conditions_page'] ) && !empty( $_POST['terms_and_conditions_page'] ) && filter_var( $_POST['terms_and_conditions_page'], FILTER_VALIDATE_URL ) !== FALSE ) {

		// check if have get_option rp_terms_and_conditions_page_link
		if( rp_get_option( 'rp_terms_and_conditions_page_link', FALSE ) !== FALSE ) {
			rp_update_option( 'rp_terms_and_conditions_page_link', $_POST['terms_and_conditions_page'] );
		} else {
			rp_add_option( 'rp_terms_and_conditions_page_link', $_POST['terms_and_conditions_page'] );
		}

	} else {

            // check if have get_option rp_terms_and_conditions_page_link
		if( rp_get_option( 'rp_terms_and_conditions_page_link', FALSE ) !== FALSE ) {
			rp_update_option( 'rp_terms_and_conditions_page_link', '' );
		}

      }

      if( rp_is_active_enhanced_locations_addon() === true ) {

            echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated'));

            exit;
            
      }

      // Update Locations Information

      // loc_ids: loc_ids,
      // location_names: location_names,
      // location_addresses: location_addresses,
      // location_cities: location_cities,
      // google_maps_links: google_maps_links,
      // location_emails: location_emails,

      $loc_ids = $_POST['loc_ids'] ?? array();

      foreach($loc_ids as $index => $loc_id) {

            $location_name = $_POST['location_names'][$index] ?? '';

            $location_address = $_POST['location_addresses'][$index] ?? '';

            $location_city = $_POST['location_cities'][$index] ?? '';

            $google_maps_link = $_POST['google_maps_links'][$index] ?? '';

            $loc_email = $_POST['location_emails'][$index] ?? '';

            if(!empty($loc_email) && filter_var($loc_email, FILTER_VALIDATE_EMAIL) !== FALSE) {

                  if( trim($loc_email) !== rp_get_settings_placeholder_email() && trim($loc_email) !== rp_get_settings_company_email() ) {

                        // Skip... So It will save the email.

                  } else {

                        $loc_email = '';

                  }

            } else {

                  $loc_email = '';

            }

            $rpQuery->query("UPDATE $company_location SET location_name='".rp_escape_sql($location_name)."',location_address='".rp_escape_sql($location_address)."',location_city_district='".rp_escape_sql($location_city)."',google_map_link='".rp_escape_sql($google_maps_link)."',location_email='".rp_escape_sql($loc_email)."' WHERE location_id='".rp_escape_sql($loc_id)."'");

            // cs_ids: cs_ids,
            // cs_days: cs_days,
            // cs_open_timings: cs_open_timings,
            // cs_close_timings: cs_close_timings,
            // cs_break_from_times: cs_break_from_times,
            // cs_break_to_times: cs_break_to_times,
            // cs_closed: cs_closed,

            $cs_id = $_POST['cs_ids'][$index] ?? array();

            $cs_day = $_POST['cs_days'][$index] ?? array();

            $cs_open_timing = $_POST['cs_open_timings'][$index] ?? array();

            $cs_close_timing = $_POST['cs_close_timings'][$index] ?? array();

            $cs_breaktime_from = $_POST['cs_break_from_times'][$index] ?? array();

            $cs_breaktime_to = $_POST['cs_break_to_times'][$index] ?? array();

            $is_closed = $_POST['cs_closed'][$index] ?? array();

            $somethingHasChanged = FALSE;
            
            foreach($cs_id as $i => $id){

                  if($is_closed[$i] == 'true'){
          
                      $closed = '1';
          
                  }else{
          
                      $closed = '0';                
          
                  }

                  // check if same entry exists
                  $check = $rpQuery->query("SELECT * FROM `$company_schedules` WHERE cs_day='".rp_escape_sql($cs_day[$i])."' AND cs_open_timing='".rp_escape_sql($cs_open_timing[$i])."' AND cs_close_timing='".rp_escape_sql($cs_close_timing[$i])."' AND cs_breaktime_from='".rp_escape_sql($cs_breaktime_from[$i])."' AND cs_breaktime_to='".rp_escape_sql($cs_breaktime_to[$i])."' AND is_closed='".rp_escape_sql($closed)."' AND location_id_fk='".rp_escape_sql($loc_id)."' AND cs_id = '".rp_escape_sql($id)."'");

                  if( empty( $check ) ) {

                        $somethingHasChanged = TRUE;

                  }
          
                  $rpQuery->query("UPDATE `$company_schedules` SET cs_day='".rp_escape_sql($cs_day[$i])."',cs_open_timing='".rp_escape_sql($cs_open_timing[$i])."',cs_close_timing='".rp_escape_sql($cs_close_timing[$i])."',cs_breaktime_from='".rp_escape_sql($cs_breaktime_from[$i])."',cs_breaktime_to='".rp_escape_sql($cs_breaktime_to[$i])."',is_closed='".rp_escape_sql($closed)."',location_id_fk='".rp_escape_sql($loc_id)."' WHERE cs_id='".rp_escape_sql($id)."'");
          
            }

            if( $somethingHasChanged === TRUE ) {

                  $schedule_group_id = 0;

                  $rpQuery->query("UPDATE $company_location SET schedule_group_id='".rp_escape_sql($schedule_group_id)."' WHERE location_id='".rp_escape_sql($loc_id)."'");

            }

      }
    
      echo json_encode(Array('status'=> true, 'message'  => 'Successfully Updated'));

      exit;

}

rp_ajax_for_public('rp_update_general_info', 'rp_update_general_info');


function rp_get_sorted_noti_types() {

      global $rpQuery;

      $notification_types = $rpQuery->prefix . "rs_notification_types";
      
      $n_types = $rpQuery->get_results("SELECT * FROM $notification_types");

      if( !empty($n_types) ) {

            $sort_by_id_like_this = array(1,2,3,6,9,10,4,5,7,8);
      
            // sot $n_types as $n_type->noti_type_id like $sort_by_id_like_this
            $sorted_n_types = array();
            foreach($sort_by_id_like_this as $sort_id) {
                  foreach($n_types as $n_type) {
                        if($n_type->noti_type_id == $sort_id) {
                              $sorted_n_types[] = $n_type;
                        }
                  }
            }
      
            $n_types = $sorted_n_types;
      
      }

      return $n_types;

}

function rp_get_all_noti_type_names() {

      $n_types = rp_get_sorted_noti_types();

      $pair = array();

      foreach($n_types as $n_type) {

            if($n_type->noti_type_id == 3) {

                  $name = 'Booking Approved (Come by our store)';

            } else {

                  $name = $n_type->noti_type;

            }

            // get all alphabets and spaces
            $key = preg_replace('/[^a-zA-Z\s]/', '', $name);

            // replace space with underscore
            $key = str_replace(' ', '_', $key);

            // make it lowercase
            $key = trim( strtolower($key) );

            $pair[$key.'_email_subject'] = '<span class="notification-badge">Email</span>' . $name;

            $pair[$key.'_email_template'] = '<span class="notification-badge">Email</span>' . $name;

      }

      return $pair;

}

function get_translation_key_by_noti_type_id($noti_type_id = 0) {

      $noti_type_id = (int) $noti_type_id;

      $n_types = rp_get_sorted_noti_types();

      foreach($n_types as $n_type) {

            if($n_type->noti_type_id == $noti_type_id) {

                  if($n_type->noti_type_id == 3) {

                        $name = 'Booking Approved (Come by our store)';

                  } else {

                        $name = $n_type->noti_type;

                  }

                  // get all alphabets and spaces
                  $key = preg_replace('/[^a-zA-Z\s]/', '', $name);

                  // replace space with underscore
                  $key = str_replace(' ', '_', $key);

                  // make it lowercase
                  $key = trim( strtolower($key) );

                  return $key;

            }

      }

      return false;

}

function rp_get_all_noti_key_value_translation() {

      global $rpQuery;

      $notification_offers = $rpQuery->prefix . "rs_notifications_and_offers";

      $n_types = rp_get_sorted_noti_types();

      $pair = array();

      foreach($n_types as $n_type) {

            $n_offer = $rpQuery->get_row("SELECT * FROM $notification_offers WHERE noti_type_id_fk = '".rp_escape_sql($n_type->noti_type_id)."'");

            if($n_type->noti_type_id == 3) {

                  $name = 'Booking Approved (Come by our store)';

            } else {

                  $name = $n_type->noti_type;

            }

            // get all alphabets and spaces
            $key = preg_replace('/[^a-zA-Z\s]/', '', $name);

            // replace space with underscore
            $key = str_replace(' ', '_', $key);

            // make it lowercase
            $key = trim( strtolower($key) );

            $pair[$key.'_email_subject'] = $n_offer->noti_subject;

            $pair[$key.'_email_template'] = $n_offer->noti_message;

      }

      return $pair;

}

add_filter('rp_get_where_translation_is_used', function($pair = array()) {

      $pair = array_merge($pair, rp_get_all_noti_type_names());

      return $pair;

}, 10, 1);


add_filter('rp_returning_static_translations_pair', function($pair = array(), $language = '') {

      $pair = array_merge($pair, rp_get_all_noti_key_value_translation());

      return $pair;

}, 10, 2);


function rp_get_all_models_for_upsales_page() {

      global $rpQuery;

      $rs_category = $rpQuery->prefix . "rs_category";

      $rs_brand = $rpQuery->prefix . "rs_brand";

      $rs_model = $rpQuery->prefix . "rs_model";

      $allCategories = $rpQuery->get_results("SELECT * FROM $rs_category ORDER BY `c_position` ASC");

      $result = array();

      if( !empty( $allCategories ) ) {

            foreach($allCategories as $category) {

                  if( !isset( $category->_brands ) ) {

                        $category->_brands = array();

                  }

                  $allBrands = $rpQuery->get_results("SELECT * FROM `$rs_brand` WHERE c_id_fk = '".rp_escape_sql($category->c_id)."' ORDER BY `b_position` ASC");

                  if( !empty( $allBrands ) ) {

                        foreach($allBrands as $brand) {

                              if( !isset( $brand->_models ) ) {

                                    $brand->_models = array();

                              }

                              $allModels = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE brand_id_fk = '".rp_escape_sql($brand->b_id)."' ORDER BY `m_position` ASC");

                              if( !empty( $allModels ) ) {

                                    foreach($allModels as $model) {

                                          $brand->_models[] = $model;

                                    }

                                    $category->_brands[] = $brand;

                              }

                        }

                  }

                  $result[] = $category;

            }

      }

      return $result;

}

function rp_remove_repair_related_attributes( $r_id_fk = '' ) {

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $rpQuery->query("DELETE FROM `$rs_repair_attr` WHERE `r_id_fk` = '".rp_escape_sql($r_id_fk)."'");

}

function rp_remove_model_related_repairs( $m_id_fk = '' ) {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $repairs = $rpQuery->get_results("SELECT * FROM `$rs_repair` WHERE `m_id_fk` = '".rp_escape_sql($m_id_fk)."'");

      if( !empty( $repairs ) ) {

            foreach( $repairs as $repair ) {

                  rp_remove_repair_related_attributes( $repair->r_id );

            }

            $rpQuery->query("DELETE FROM `$rs_repair` WHERE `m_id_fk` = '".rp_escape_sql($m_id_fk)."'");

      }

}

function rp_delete_brand_related_data( $brand_id_fk = '' ) {

      global $rpQuery;

      $rs_model = $rpQuery->prefix . 'rs_model';

      $models = $rpQuery->get_results("SELECT * FROM `$rs_model` WHERE `brand_id_fk` = '".rp_escape_sql($brand_id_fk)."'");  

      if( !empty( $models ) ) {

            foreach( $models as $model ) {

                  rp_remove_model_related_repairs( $model->m_id );

            }

            $rpQuery->query("DELETE FROM `$rs_model` WHERE `brand_id_fk` = '".rp_escape_sql($brand_id_fk)."'");

      }

}

function rp_remove_category_related_data( $c_id_fk = '' ) {

      global $rpQuery;

      $rs_brand = $rpQuery->prefix . 'rs_brand';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $brands = $rpQuery->get_results("SELECT * FROM `$rs_brand` WHERE `c_id_fk` = '".rp_escape_sql($c_id_fk)."'");      

      if( !empty( $brands ) ) {

            foreach( $brands as $brand ) {

                  rp_delete_brand_related_data($brand->b_id);

            }

            $rpQuery->query("DELETE FROM `$rs_brand` WHERE `c_id_fk` = '".rp_escape_sql($c_id_fk)."'"); 

      }

      $rpQuery->query("DELETE FROM `$rs_default_repair` WHERE `dr_category` = '".rp_escape_sql($c_id_fk)."'"); 

      $rpQuery->query("DELETE FROM `$rs_default_repair_attr` WHERE `dr_category` = '".rp_escape_sql($c_id_fk)."'"); 

}

/*
add appointment_number column to rs_orders table
*/

function add_appointment_number_column_to_existing_db() {
      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'appointment_number';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `appointment_number` varchar(255) NULL DEFAULT '';");
            
      }
}

add_action('init', 'add_appointment_number_column_to_existing_db', 999);


/*
Create RepairPlugin Appointment Number Table
*/
function repair_plugin_appointment_number_table(){
      /* Added in V1.5.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_appointment_number = $rpQuery->prefix . "rs_appointment_number"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_appointment_number (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `date` varchar(255) NOT NULL,

      `count` int(11) NOT NULL,

      PRIMARY KEY (`id`))  

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_appointment_number_table');

/*
Create RepairPlugin Outgoing Triggers Table
*/
function repair_plugin_outgoing_triggers_table(){
      /* Added in V1.5.7 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_outgoing_triggers = $rpQuery->prefix . "rs_outgoing_triggers"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_outgoing_triggers (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `trigger` varchar(255) NOT NULL,

      `url` varchar(1000) NOT NULL,

      PRIMARY KEY (`id`))  

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_outgoing_triggers_table');

function rp_get_all_outgoing_triggers() {

      global $rpQuery;
  
      $rs_outgoing_triggers = $rpQuery->prefix . "rs_outgoing_triggers";
  
      $sql = "SELECT * FROM `$rs_outgoing_triggers`";
  
      $results = $rpQuery->get_results($sql);
  
      if( !empty($results) ) {
  
          return $results;
  
      }
  
      return array();
  
}

function rp_available_trigger_options() {

      return array(

            'scheduled_appointment_created', // done

            'scheduled_appointment_updated', // done

            'scheduled_appointment_cancelled', // done
            
            'scheduled_appointment_deleted', // done

            'ship_device_appointment_created', // done

            'ship_device_appointment_updated', // done

            'ship_device_appointment_cancelled', // done
            
            'ship_device_appointment_deleted', // done

            'pdf_offer_created', // done

            'pdf_offer_updated', // done

            'pdf_offer_cancelled', // done
            
            'pdf_offer_deleted' // done

      );

}

function rp_get_outgoing_triggers_data_for_edit() {
  
      rp_verify_csrf_token_for_ajax('webhooks');
 
      $available_options = rp_available_trigger_options();

      global $rpQuery;
  
      $rs_outgoing_triggers = $rpQuery->prefix . "rs_outgoing_triggers";
  
      $id = (int) ($_POST['id'] ?? 0);
  
      if( empty( $id ) ) {
  
          // it is a create form
          echo json_encode(Array('status'=> true, 'data'  => array(
              'id' => 0,
              'trigger' => '',
              'url' => '',
              'available_options' => $available_options
          )));
  
          exit;
  
      } else {
  
          // verify if the location exists
          $sql = "SELECT * FROM $rs_outgoing_triggers WHERE `id` = $id";
  
          $results = $rpQuery->get_row($sql);
  
          if( empty($results) ) {
  
              echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Trigger ID'));
  
              exit;
  
          }

          $results = rp_sanitize_output($results);
          
          echo json_encode(Array('status'=> true, 'data'  => array(
              'id' => $results->id,
              'trigger' => $results->trigger,
              'url' => $results->url,
              'available_options' => $available_options
          )));
  
          exit;
  
      }
  
      exit;
  
}  

rp_ajax_for_admin('rp_get_outgoing_triggers_data_for_edit', 'rp_get_outgoing_triggers_data_for_edit');

function rp_create_or_edit_outgoing_trigger() {
  
      rp_verify_csrf_token_for_ajax('webhooks');
  
      global $rpQuery;
  
      $rs_outgoing_triggers = $rpQuery->prefix . "rs_outgoing_triggers";
  
      $required_fields = array( 'trigger', 'url' );
  
      // make sure all required fields are present
      foreach( $required_fields as $field ) {
  
          if( empty( $_POST[$field] ) ) {
  
              echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Please fill all required fields'));
  
              exit;
  
          }
  
      }
  
      $trigger = sanitize_text_field( trim( $_POST['trigger'] ?? '' ) );
  
      $url = sanitize_text_field( trim( $_POST['url'] ?? '' ) );
  
      // make sure selected schedule is valid
      $triggerOptions = rp_available_trigger_options();
  
      if( !in_array( $trigger, $triggerOptions ) ) {
  
          echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid trigger'));
  
          exit;
  
      }
      
      // make sure url is valid url

      if( !filter_var($url, FILTER_VALIDATE_URL) ) {
  
          echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid URL'));
  
          exit;
  
      }
  
      $trigger_id = (int) ($_POST['id'] ?? 0);
  
      if( $trigger_id == 0 ) {
  
          // create new location
          $sql = "INSERT INTO `$rs_outgoing_triggers` (`trigger`, `url`) VALUES (':trigger', ':url')";
  
          $rpQuery->query($sql, array(
              'trigger' => $trigger,
              'url' => $url
          ));
  
          $insert_id = $rpQuery->insert_id;

          $payload = rp_get_example_trigger_payload( $trigger );

          RepairPluginPro\RP_Trigger_Request::create_request_safe( $url, $payload );
  
      } else {
  
          // verify location id
          $verify = $rpQuery->get_row("SELECT * FROM $rs_outgoing_triggers WHERE `id` = $trigger_id");
  
          if( empty( $verify ) ) {
  
              echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Trigger ID'));
  
              exit;
  
          }

          $old_url = $verify->url;
  
          $sql = "UPDATE `$rs_outgoing_triggers` SET `trigger` = ':trigger', `url` = ':url' WHERE `id` = $trigger_id";
  
          $rpQuery->query($sql, array(
            'trigger' => $trigger,
            'url' => $url
          ));

          if( trim( $old_url ) !== trim( $url ) ) {

            $payload = rp_get_example_trigger_payload( $trigger );

            RepairPluginPro\RP_Trigger_Request::create_request_safe( $url, $payload );

          }
  
      }
  
      echo json_encode(Array('status'=> true, 'message'  => 'Successfully created / updated'));
  
      exit;
  
}

rp_ajax_for_admin('rp_create_or_edit_outgoing_trigger', 'rp_create_or_edit_outgoing_trigger');

function rp_delete_outgoing_trigger() {
  
      rp_verify_csrf_token_for_ajax('webhooks');
  
      global $rpQuery;
  
      $rs_outgoing_triggers = $rpQuery->prefix . "rs_outgoing_triggers";
  
      $trigger_id = (int) ($_POST['id'] ?? 0);
  
      if( $trigger_id == 0 ) {
  
          echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Trigger ID'));
  
          exit;
  
      }
  
      // verify location id
      $verify = $rpQuery->get_row("SELECT * FROM $rs_outgoing_triggers WHERE `id` = $trigger_id");
  
      if( empty( $verify ) ) {
  
          echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Trigger ID'));
  
          exit;
  
      }
  
      $rpQuery->query("DELETE FROM `$rs_outgoing_triggers` WHERE `id` = $trigger_id");
  
      echo json_encode(Array('status'=> true, 'message'  => 'Successfully deleted'));
  
      exit;
  
  
}

rp_ajax_for_admin('rp_delete_outgoing_trigger', 'rp_delete_outgoing_trigger');

function rp_get_payload_of_outgoing_trigger() {

      global $rpQuery;

      $rs_outgoing_triggers = $rpQuery->prefix . 'rs_outgoing_triggers';

      $id = (int) $_POST['id'] ?? '';

      if( empty( $id ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Trigger ID'));
  
            exit;

      }

      $sql = "SELECT * FROM `$rs_outgoing_triggers` WHERE `id` = {$id}";

      $result = $rpQuery->get_row( $sql );

      if( empty( $result ) ) {

            echo json_encode(Array('status'=> false, 'rp_validation_error'  => 'Invalid Trigger ID'));
  
            exit;

      }

      $payload = rp_get_example_trigger_payload( $result->trigger );

      echo json_encode(Array('status'=> true, 'payload'  => $payload));

      exit;

}

function rp_get_example_trigger_payload( $name_of_trigger = '' ) {

      $payload = array();

      $org_trigger = $name_of_trigger;

      // explode by _ and get last element
      $name_of_trigger = explode( '_', $name_of_trigger );

      $name_of_trigger = end( $name_of_trigger );

      $payload = RepairPluginPro\RP_Trigger_Examples::prepare( $name_of_trigger );

      $payload = apply_filters('rp_filter_outgoing_trigger_payload', $payload, $org_trigger, 0);

      return $payload;

}

function rp_filter_outgoing_trigger_payload( $payload = array(), $name_of_trigger = '', $o_id = 0 ) {

      if( empty( $payload ) ) {

            return $payload;

      }

      if( strpos( $name_of_trigger, 'ship_device_' ) !== FALSE || strpos( $name_of_trigger, 'pdf_offer_' ) !== FALSE ) {

            $unset_these = array(
                  'reschedule_link',
                  'event_start_datetime',
                  'event_end_datetime',
                  'appointment_date',
                  'appointment_time'
            );

            foreach( $unset_these as $value ) {

                  if( isset( $payload[ $value ] ) ) {

                        unset( $payload[ $value ] );

                  }

            }

      }

      if( strpos( $name_of_trigger, 'pdf_offer_' ) !== FALSE ) {
       
            if( !empty( $o_id ) ) {

                  global $rpQuery;

                  $rs_user_pdfs = $rpQuery->prefix . 'rs_user_pdfs';

                  $o_id = (int) $o_id;

                  $sql = "SELECT * FROM `$rs_user_pdfs` WHERE `o_id_fk` = {$o_id}";

                  $pdf_row = $rpQuery->get_row( $sql );

                  if( !empty( $pdf_row ) ) {

                        $payload['finalize_link'] = $pdf_row->finalize_link;
                        $payload['customer_street_address'] = $pdf_row->streetname_phone;
                        $payload['customer_city'] = $pdf_row->zipcode_city;
                        $payload['customer_zipcode'] = $pdf_row->zipcode_only;
                        $payload['customer_imei'] = $pdf_row->notes;

                  } else {

                        $payload['finalize_link'] = '';
                        $payload['customer_street_address'] = '';
                        $payload['customer_city'] = '';
                        $payload['customer_zipcode'] = '';
                        $payload['customer_imei'] = '';

                  }

            } else {

                  $payload['finalize_link'] = '';
                  $payload['customer_street_address'] = 'Streetname + Number';
                  $payload['customer_city'] = 'Amsterdam';
                  $payload['customer_zipcode'] = '1102 MX';
                  $payload['customer_imei'] = '9999999999999999';

            }

            $unset_these = array(
                  'customer_house_no',
                  'customer_notes'
            );

            foreach( $unset_these as $valuep ) {

                  if( array_key_exists( $valuep, $payload ) ) {

                        unset( $payload[ $valuep ] );

                  }

            }

            // loop through payload and make position of customer_imei after customer_zipcode

            $payload_copy = $payload;

            $payload = array();

            foreach( $payload_copy as $keyp => $valuep ) {

                  if( $keyp == 'customer_imei' ) {

                        continue;

                  }

                  $payload[ $keyp ] = $valuep;

                  if( $keyp == 'customer_zipcode' ) {

                        $payload['customer_imei'] = $payload_copy['customer_imei'];

                  }

            }

      }

      if( empty( $o_id ) ) {

            if( strpos( $name_of_trigger, 'pdf_offer_' ) !== FALSE ) {

                  $payload['delivery_method'] = 'PDF Offer';

                  // $payload['delivery_method_translated'] = 'PDF Offer';

                  if( $payload['appointment_status'] !== 'Deleted' ) {

                        $payload['appointment_status'] = 'Offered';

                  }

            }

            if( strpos( $name_of_trigger, 'ship_device_' ) !== FALSE ) {

                  $payload['delivery_method'] = 'Ship device';

                  // $payload['delivery_method_translated'] = 'Ship device';

            }

      }

      return $payload;

}

add_filter('rp_filter_outgoing_trigger_payload', 'rp_filter_outgoing_trigger_payload', 10, 3);

function rp_trigger_outgoing_webhook( $name_of_trigger = '', $o_id = 0 ) {

      global $rpQuery;

      $rs_outgoing_triggers = $rpQuery->prefix . 'rs_outgoing_triggers';

      $sql = "SELECT * FROM `$rs_outgoing_triggers` WHERE `trigger` = '".rp_escape_sql( $name_of_trigger )."'";

      $triggers = $rpQuery->get_results( $sql );

      if( !empty( $triggers ) ) {

            $payload = rp_get_order_trigger_payload( $name_of_trigger, $o_id );

            $payload_copy = $payload;

            $payload_copy['trigger_name'] = $name_of_trigger;

            if( !empty( $payload ) ) {

                  foreach( $triggers as $theTrigger ) {

                        $url = $theTrigger->url;
      
                        RepairPluginPro\RP_Trigger_Request::create_request_safe( $url, $payload );
      
                  }

            }
                  
      }

}

function rp_get_order_trigger_payload( $name_of_trigger = '', $o_id = 0 ) {

      $payload = array();

      $org_trigger = $name_of_trigger;

      // explode by _ and get last element
      $name_of_trigger = explode( '_', $name_of_trigger );

      $name_of_trigger = end( $name_of_trigger );

      $payload = RepairPluginPro\RP_Trigger_Payload::prepare( $o_id, $name_of_trigger );

      $payload = apply_filters('rp_filter_outgoing_trigger_payload', $payload, $org_trigger, $o_id);

      return $payload;

}

rp_ajax_for_admin('rp_get_payload_of_outgoing_trigger', 'rp_get_payload_of_outgoing_trigger');

function rp_save_session_data($session_id = '', $session_data = array()) {
      global $rpQuery;

      $table_rs_sessions = $rpQuery->prefix . 'rs_sessions';

      $session_data = json_encode($session_data);

      $session_expires = time() + 60 * 60 * 24;

      // if have session id related row then update it
      $session = $rpQuery->get_row("SELECT * FROM `$table_rs_sessions` WHERE `session_id` = '".rp_escape_sql($session_id)."'");

      if( !empty( $session ) ) {

            $rpQuery->query("UPDATE `$table_rs_sessions` SET `session_data` = '".rp_escape_sql($session_data)."', `session_expires` = '".rp_escape_sql($session_expires)."' WHERE `session_id` = '".rp_escape_sql($session_id)."'");

      } else {

            $rpQuery->query("INSERT INTO `$table_rs_sessions` (`session_id`, `session_data`, `session_expires`) VALUES ('".rp_escape_sql($session_id)."', '".rp_escape_sql($session_data)."', '".rp_escape_sql($session_expires)."')");

      }

}

function rp_delete_expired_sessions() {
      global $rpQuery;

      $table_rs_sessions = $rpQuery->prefix . 'rs_sessions';

      $rpQuery->query("DELETE FROM `$table_rs_sessions` WHERE `session_expires` < '".time()."'");

}

function rp_get_session_data($session_id = '') {

      rp_delete_expired_sessions();

      global $rpQuery;

      $table_rs_sessions = $rpQuery->prefix . 'rs_sessions';

      $session_data = $rpQuery->get_row("SELECT * FROM `$table_rs_sessions` WHERE `session_id` = '".rp_escape_sql($session_id)."'");

      if(!empty($session_data)) {
            $session_data = json_decode($session_data->session_data, true);
      }

      return $session_data;
}


// Create rp_sessions table

function repair_plugin_sessions_table(){
      /* Added in V1.5.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_sessions = $rpQuery->prefix . "rs_sessions"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_sessions (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `session_id` varchar(255) NOT NULL,

      `session_data` text NOT NULL,

      `session_expires` int(11) NOT NULL,

      PRIMARY KEY (`id`))  

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_sessions_table');

function rp_increment_appointment_number($created_date = '') {

      global $rpQuery;

      $rs_appointment_number = $rpQuery->prefix . 'rs_appointment_number';

      $parsed = date('Y-m-d', strtotime($created_date));

      $row = $rpQuery->get_row("SELECT * FROM `$rs_appointment_number` WHERE `date` = '$parsed'");

      if( !empty( $row ) ) {

            $new_count = ($row->count) + 1;

            $rpQuery->query("UPDATE `$rs_appointment_number` SET `count` = '{$new_count}' WHERE `id` = {$row->id}");

      } else {

            $rpQuery->query("INSERT INTO `$rs_appointment_number` (`date`, `count`) VALUES ('$parsed', 1)");

      }

}

function rp_get_new_appointment_number($created_date = '') {

      global $rpQuery;

      $rs_appointment_number = $rpQuery->prefix . 'rs_appointment_number';

      $parsed = date('Y-m-d', strtotime($created_date));

      $row = $rpQuery->get_row("SELECT * FROM `$rs_appointment_number` WHERE `date` = '$parsed'");

      if( !empty( $row ) ) {

            $old = $row->count;

            return date('ymdh', strtotime($created_date)) . ($old + 1);

      }

      return date('ymdh', strtotime($created_date)) . '1';

}

function rp_update_rs_orders_appointment_number() {

      if( rp_get_option('rp_update_rs_orders_appointment_number', FALSE) === FALSE ) {

            rp_add_option('rp_update_rs_orders_appointment_number', TRUE);

            global $rpQuery;

            $rs_orders = $rpQuery->prefix . 'rs_orders';

            $orders = $rpQuery->get_results("SELECT * FROM `$rs_orders` WHERE `appointment_number` = ''");

            if( !empty( $orders ) ) {

                  _rp_set_default_tiemzone();

                  foreach( $orders as $order_row ) {

                        $created_time = ($order_row->created_time ?? $order_row->o_date_time);

                        rp_set_new_appointment_number($order_row->o_id, $created_time);

                  }

            }

      }

}

add_action('init', 'rp_update_rs_orders_appointment_number', 999);

function rp_set_new_appointment_number($oid = '', $created_time = '') {

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $new_appointment_number = rp_get_new_appointment_number( $created_time );

      $rpQuery->query("UPDATE `$rs_orders` SET `appointment_number` = '$new_appointment_number' WHERE `o_id` = '{$oid}' LIMIT 1;");

      rp_increment_appointment_number( $created_time );

}

function rp_get_all_dr_uq_ids() {

      $cache_key = 'rp_get_all_dr_uq_ids';

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if(!empty($cached)) {

            return $cached;

      }

      $uq_ids = rp_get_all_dr_uq_id_from_rs_default_repair_table();

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $uq_ids);

      return $uq_ids;

}

function rp_is_default_repair( $r_uq_id = '' ) {

      return in_array($r_uq_id, rp_get_all_dr_uq_ids()) !== false;

}

function rp_get_default_repair_row_by_uq_id( $r_uq_id = '', $dr_category = '', $dr_row_old = null ) {

      if( !empty( $dr_row_old ) ) {

            return $dr_row_old;

      }

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      return $rpQuery->get_row("SELECT * FROM `$rs_default_repair` WHERE `dr_uq_id` = '".rp_escape_sql($r_uq_id)."' AND `dr_category` = '".rp_escape_sql($dr_category)."' LIMIT 1;");

}

function rp_get_attributes_of_default_repair( $dr_id = '', $create_attr_later = array() ) {

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $results = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE `dr_id_fk` = '".rp_escape_sql($dr_id)."';");
      
      if( !empty( $results ) ) {

            $new_results = array();

            foreach( $results as $attr_row ) {

                  if( !in_array($attr_row->da_id, $create_attr_later) ) {

                        $new_results[] = $attr_row;

                  }

            }

            return $new_results;

      }

      return $results;

}

function rp_exclude_just_adding_repairs( $attr_rows = array(), $create_attr_later = array() ) {

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      if( !empty( $attr_rows ) && !empty( $create_attr_later ) ) {

            $create_attr_later_str = join(', ', $create_attr_later);

            $just_adding_rows = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE `da_id` IN ({$create_attr_later_str});");

            if( !empty( $just_adding_rows ) ) {

                  $excludeUqIds = array();

                  foreach( $just_adding_rows as $justAddingRow ) {

                        $excludeUqIds[] = $justAddingRow->da_uq_id;

                  }

                  $new_results = array();

                  foreach( $attr_rows as $attr_row ) {

                        if( !in_array( $attr_row->a_uq_id, $excludeUqIds ) ) {

                              $new_results[] = $attr_row;

                        }

                  }
                  
                  return $new_results;

            }

      }

      return $attr_rows;

}

function rp_is_unchanged_default_repair_individually( $r_id = '', $dr_category = '', $dr_row_old = null, $create_attr_later = array() ) {

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $row = $rpQuery->get_row("SELECT * FROM `$rs_repair` WHERE `r_id` = '".rp_escape_sql($r_id)."' LIMIT 1;");

      if( !empty( $row ) ) {

            if( rp_is_default_repair( $row->r_uq_id ) !== false ) {

                  $dr_row = rp_get_default_repair_row_by_uq_id( $row->r_uq_id, $dr_category, $dr_row_old );

                  $dr_attr_rows = rp_get_attributes_of_default_repair( $dr_row->dr_id, $create_attr_later );

                  $attr_rows = $rpQuery->get_results("SELECT * FROM `$rs_repair_attr` WHERE `r_id_fk` = '".rp_escape_sql($r_id)."';");

                  $attr_rows = rp_exclude_just_adding_repairs( $attr_rows, $create_attr_later );
                  
                  if( empty( $dr_attr_rows ) && empty( $attr_rows ) ) {

                        if(isset($_POST['rmargin']) && !empty($_POST['rmargin'])) {
                              // Do not have attributes.. Means row matches... Just check the price & is_free
                              return ($row->is_free === $dr_row->is_free) && ($row->r_price === $dr_row->dr_price) && ($row->r_margin === $dr_row->dr_margin);
                        } else {
                              // Do not have attributes.. Means row matches... Just check the price & is_free
                              return ($row->is_free === $dr_row->is_free) && ($row->r_price === $dr_row->dr_price);
                        }

                  } else if( rp_attributes_exactly_matches_default( $attr_rows, $dr_attr_rows ) ) {

                        // Attributes exactly matches but might have price changed
                        return TRUE;

                  }  else {

                        return FALSE;

                  }

            }
            
      }      

      return FALSE;

}

function rp_attributes_exactly_matches_default( $attr_rows = array(), $dr_attr_rows = array() ) {

      if( empty( $attr_rows ) || empty( $dr_attr_rows ) || (count($attr_rows) !== count($dr_attr_rows)) ) {

            return FALSE;

      }

      $uq_ids = array();

      foreach( $dr_attr_rows as $dr_attr_row ) {

            $uq_ids[] = $dr_attr_row->da_uq_id;

      }

      foreach( $attr_rows as $attr_row ) {

            if( !in_array( $attr_row->a_uq_id, $uq_ids ) ) {

                  return FALSE;

            }

      }

      return TRUE;

}

function rp_is_fully_unchanged_default_repair_individually( $r_id = '', $mid = '' ) {

      $dr_category = rp_get_model_cached_category_by_mid($mid);

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $row = $rpQuery->get_row("SELECT * FROM `$rs_repair` WHERE `r_id` = '".rp_escape_sql($r_id)."' LIMIT 1;");

      if( !empty( $row ) ) {

            if( rp_is_default_repair( $row->r_uq_id ) !== false ) {

                  $dr_row = rp_get_default_repair_row_by_uq_id( $row->r_uq_id, $dr_category );

                  $dr_attr_rows = rp_get_attributes_of_default_repair( $dr_row->dr_id );

                  $attr_rows = $rpQuery->get_results("SELECT * FROM `$rs_repair_attr` WHERE `r_id_fk` = '".rp_escape_sql($r_id)."';");
                  
                  if( empty( $dr_attr_rows ) && empty( $attr_rows ) ) {

                        // Do not have attributes.. Means row matches... Just check the price & is_free
                        return apply_filters('rp_is_fully_unchanged_default_repair_individually',(($row->is_free === $dr_row->is_free) && 
                        ($row->r_price === $dr_row->dr_price) &&
                        ($row->r_duration === $dr_row->dr_duration) &&
                        ($row->r_duration_measure === $dr_row->dr_duration_measure)), $row, $dr_row);

                  } else if( rp_attributes_exactly_matches_default_with_price( $attr_rows, $dr_attr_rows ) ) {

                        // Attributes exactly matches but might have price changed
                        return apply_filters('rp_is_fully_unchanged_default_repair_individually',(($row->is_free === $dr_row->is_free) && 
                        ($row->r_price === $dr_row->dr_price) &&
                        ($row->r_duration === $dr_row->dr_duration) &&
                        ($row->r_duration_measure === $dr_row->dr_duration_measure)), $row, $dr_row);

                  }  else {

                        return FALSE;

                  }

            }
            
      }     
      
      return FALSE;

}


function rp_attributes_exactly_matches_default_with_price( $attr_rows = array(), $dr_attr_rows = array() ) {

      if( empty( $attr_rows ) || empty( $dr_attr_rows ) || (count($attr_rows) !== count($dr_attr_rows)) ) {

            return FALSE;

      }

      $uq_ids = array();
      $price = array();

      foreach( $dr_attr_rows as $dr_attr_row ) {

            $uq_ids[] = $dr_attr_row->da_uq_id;
            $price[] = $dr_attr_row->da_price;

      }

      $results = array();

      foreach( $attr_rows as $attr_row ) {

            if( !in_array( $attr_row->a_uq_id, $uq_ids ) ) {

                  return FALSE;

            } else {

                  $results[] = $price[array_search($attr_row->a_uq_id, $uq_ids)] === $attr_row->a_price;

            }

      }

      $final_result = array_search(FALSE, $results) === FALSE;

      return apply_filters('rp_attributes_exactly_matches_default_with_price', $final_result, $attr_rows, $dr_attr_rows);

}

function rp_get_model_cached_category_by_mid( $mid = '' ) {

      $cache_key = 'rp_get_model_cached_category_by_mid_'.$mid;

      $cached = RepairPluginPro\RP_Memory_Cache::get($cache_key);

      if( !empty( $cached ) ) {

            return $cached;

      }

      global $rpQuery;

      $rs_model = $rpQuery->prefix . 'rs_model';

      $model_row = $rpQuery->get_row("SELECT `device_type` FROM `$rs_model` WHERE `m_id` = '$mid' LIMIT 1;");

      $result = 0;

      if( !empty( $model_row ) ) {

            $result = $model_row->device_type;

      }

      RepairPluginPro\RP_Memory_Cache::save($cache_key, $result);

      return $result;

}

function rp_reset_default_repair_individually() {

      rp_verify_csrf_token_for_ajax('manage_models');

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $rid = $_POST['drid'] ?? '';

      $table_rows = array();

      $repair_row = $rpQuery->get_row("SELECT * FROM `$rs_repair` WHERE `r_id` = '".rp_escape_sql($rid)."' LIMIT 1;");

      if( !empty( $repair_row ) ) {

            $dr_category = rp_get_model_cached_category_by_mid( $repair_row->m_id_fk );

            // Remove repair related attributes
            $rpQuery->query("DELETE FROM `$rs_repair_attr` WHERE `r_id_fk` = '".rp_escape_sql($rid)."';");

            $uq_id = $repair_row->r_uq_id;

            $dr_row = $rpQuery->get_row("SELECT * FROM `$rs_default_repair` WHERE `dr_uq_id` = '".rp_escape_sql($uq_id)."' AND `dr_category` = '".rp_escape_sql($dr_category)."' LIMIT 1");

            if( !empty( $dr_row ) ) {

                  $also_reset_margin = apply_filters('rp_reset_default_repair_individually_margin', FALSE);
                  $reset_margin = '';
                  if( $also_reset_margin === TRUE ) {
                        $reset_margin = "`r_margin` = '".rp_escape_sql($dr_row->dr_margin)."',";
                  }

                  // Update individual repair row to match default repair
                  
                  $rpQuery->query("UPDATE `$rs_repair` SET 
                  `r_name` = '".rp_escape_sql($dr_row->dr_name)."', 
                  `r_discription` = '".rp_escape_sql($dr_row->dr_discription)."', 
                  `r_badge` = '".rp_escape_sql($dr_row->dr_badge)."', 
                  `is_free` = '".rp_escape_sql($dr_row->is_free)."', 
                  `r_price` = '".rp_escape_sql($dr_row->dr_price)."', 
                  `r_duration` = '".rp_escape_sql($dr_row->dr_duration)."', 
                  `r_duration_measure` = '".rp_escape_sql($dr_row->dr_duration_measure)."',
                  {$reset_margin}
                  `r_img` = '".rp_escape_sql($dr_row->dr_img)."' 
                  WHERE `r_id` = '".rp_escape_sql($repair_row->r_id)."';
                  ");

                  $default_attrs = $rpQuery->get_results("SELECT * FROM `$rs_default_repair_attr` WHERE `dr_id_fk` = '".rp_escape_sql($dr_row->dr_id)."'");

                  if( !empty( $default_attrs ) ) {

                        foreach( $default_attrs as $dAttr ) {

                              $r_id_fk = $repair_row->r_id;

                              $a_name = $dAttr->da_name;

                              $a_desc = $dAttr->da_desc;

                              $a_badge = $dAttr->da_badge;

                              $a_duration = $dAttr->da_duration;

                              $a_duration_measure = $dAttr->da_duration_measure;

                              $a_price = $dAttr->da_price;

                              $a_uq_id = $dAttr->da_uq_id;

                              $is_active = $dAttr->is_active;
                              
                              $ra_position = $dAttr->da_position;

                              if( $also_reset_margin === TRUE ) {

                                    $a_margin_reset = ', `a_margin`';
                                    $a_margin_reset_value = ", '".rp_escape_sql($dAttr->da_margin)."'";

                              } else {

                                    $a_margin_reset = '';
                                    $a_margin_reset_value = '';

                              }

                              $rpQuery->query("INSERT INTO `$rs_repair_attr` (`r_id_fk`, `a_name`, `a_desc`, `a_badge`, `a_duration`, `a_duration_measure`,`a_price`, `a_uq_id`, `is_active`, `ra_position` $a_margin_reset) 
                              VALUES ('".rp_escape_sql($r_id_fk)."', '".rp_escape_sql($a_name)."', '".rp_escape_sql($a_desc)."', '".rp_escape_sql($a_badge)."', '".rp_escape_sql($a_duration)."', '".rp_escape_sql($a_duration_measure)."', '".rp_escape_sql($a_price)."', '".rp_escape_sql($a_uq_id)."', '".rp_escape_sql($is_active)."', '".rp_escape_sql($ra_position)."' $a_margin_reset_value);
                              ");

                        }

                  }

            }

            $table_rows = rp_get_model_related_repairs_for_admin_page( $repair_row->m_id_fk );

      }

      echo json_encode(array(
            'status' => true,
            'table_rows' => $table_rows
      ));

      exit;

}

rp_ajax_for_admin('rp_reset_default_repair_individually', 'rp_reset_default_repair_individually');


// create 32 character unique id for csrf token

function rp_create_csrf_token($action = '') {

      RepairPluginPro\RP_Session_Handler::start();

      if( !isset($_SESSION['rp_csrf_tokens']) ) {

            $_SESSION['rp_csrf_tokens'] = array();

      }

      // loop through all $_SESSION['rp_csrf_tokens'] and remove expired tokens (older than 1 hour)
      foreach( $_SESSION['rp_csrf_tokens'] as $key => $value ) {

            if( time() - $value['time'] > 3600 ) {

                  unset( $_SESSION['rp_csrf_tokens'][$key] );

            }

      }

      if( !isset( $_SESSION['rp_csrf_tokens'][$action] ) ) {

            $token = bin2hex(random_bytes(32));
      
            $_SESSION['rp_csrf_tokens'][$action] = array(
                  'token' => $token,
                  'time' => time()
            );

      } else {

            $token = $_SESSION['rp_csrf_tokens'][$action]['token'];

            $_SESSION['rp_csrf_tokens'][$action]['time'] = time();

      }

      RepairPluginPro\RP_Session_Handler::close();

      return $token;

}

function rp_verify_csrf_token_for_simple_form($action = '') {

      if( !rp_verify_csrf_token($action) ) {

            RepairPluginPro\RP_Session_Handler::start();

            $_SESSION['rp_validation_error'] = 'Form submission failed due to security reasons, please try again and it should work fine.';

            RepairPluginPro\RP_Session_Handler::close();

            $referrer = $_SERVER['HTTP_REFERER'] ?? '';

            if( !empty( $referrer ) && strpos( $referrer, get_site_url() ) !== FALSE ) {

                  echo '<script>window.location.href = "'.$referrer.'";</script>';

            } else {

                  echo '<script>window.location.href = "'.get_site_url().'";</script>';

            }

            exit;

      }

}

function rp_verify_csrf_token_for_apple_calendar() {

      if( !rp_verify_csrf_token('appointments') && !rp_verify_csrf_token('frontend') ) {

            RepairPluginPro\RP_Session_Handler::start();

            $_SESSION['rp_validation_error'] = 'Form submission failed due to security reasons, please try again and it should work fine.';

            RepairPluginPro\RP_Session_Handler::close();

            $referrer = $_SERVER['HTTP_REFERER'] ?? '';

            if( !empty( $referrer ) && strpos( $referrer, get_site_url() ) !== FALSE ) {

                  echo '<script>window.location.href = "'.$referrer.'";</script>';

            } else {

                  echo '<script>window.location.href = "'.get_site_url().'";</script>';

            }

            exit;

      }

}

function rp_verify_csrf_token_for_ajax($action = '') {

      if( !is_array($action) ) {

            $action = array($action);
            
      }

      foreach( $action as $_action ) {

            if( rp_verify_csrf_token($_action) ) {

                  return true;

            }

      }

      echo json_encode(
            array(
                  'status'=> false, 
                  'rp_validation_error' => 'Form submission failed, please refresh the page and try again.'
            )
      );

      exit();

}

function rp_verify_csrf_token($action = '') {

      $token = $_POST['rp_csrf_token'] ?? '';

      if( empty($token) ) {

            return false;

      }

      RepairPluginPro\RP_Session_Handler::start();

      if( !isset($_SESSION['rp_csrf_tokens']) ) {

            // Try to re-read session from scratch
            // as some other plugins might have overwrite
            // $_SESSION variable till the time this function is called

            RepairPluginPro\RP_Session_Handler::$readed_cookie = false;

            RepairPluginPro\RP_Session_Handler::start();

            if( !isset($_SESSION['rp_csrf_tokens']) ) {

                  $_SESSION['rp_csrf_tokens'] = array();
      
            }

      }

      // loop through all $_SESSION['rp_csrf_tokens'] and remove expired tokens (older than 1 hour)
      foreach( $_SESSION['rp_csrf_tokens'] as $key => $value ) {

            if( time() - $value['time'] > 3600 ) {

                  unset( $_SESSION['rp_csrf_tokens'][$key] );

            }

      }

      if( !isset( $_SESSION['rp_csrf_tokens'][$action] ) ) {

            return false;

      } else {

            $_token = $_SESSION['rp_csrf_tokens'][$action]['token'];

            if( $_token === $token ) {

                  $_SESSION['rp_csrf_tokens'][$action]['time'] = time();

                  RepairPluginPro\RP_Session_Handler::close();

                  return true;

            } else {

                  return false;

            }

      }

}

function rp_is_recaptcha_correctly_configured() {
      $is_enabled = rp_is_enabled_recaptcha();
      $site_key = trim( rp_get_google_recaptcha_site_key() );
      $secret_key = trim( rp_get_google_recaptcha_secret_key() );

      if( $is_enabled == 1 && !empty( $site_key ) && !empty( $secret_key ) ) {

            return TRUE;

      }

      return FALSE;
}

function rp_google_recaptcha_token_form_for_ajax() {

      $is_enabled = rp_is_enabled_recaptcha();
      $need_to_hide_badge = rp_is_need_to_hide_recaptcha_badge();
      $site_key = trim( rp_get_google_recaptcha_site_key() );
      $secret_key = trim( rp_get_google_recaptcha_secret_key() );

      if( $is_enabled == 1 && !empty( $site_key ) && !empty( $secret_key ) ) {
            
            echo '<form method="POST" style="display:none;">';
            echo '<input type="hidden" name="rp-recaptcha-response" id="rp-recaptcha-response" value="">';
            echo '</form>';

            if( $need_to_hide_badge == 1 ) {
                  ?>
                  <style type="text/css">
                        .grecaptcha-badge { display: none !important; }
                  </style>
                  <?php
            }

            ?>
            <script src="https://www.google.com/recaptcha/api.js?render=<?php echo $site_key; ?>"></script>
            <script>
            grecaptcha.ready(function() {
                  grecaptcha.execute('<?php echo $site_key; ?>', {action: 'submit'}).then(function(token) {
                        document.getElementById('rp-recaptcha-response').value = token;
                        console.log(token);
                  });
            });
            </script>
            <?php

      }

}

function rp_csrf_token_form_for_ajax($action = '') {

      $token = rp_create_csrf_token($action);

      echo '<form method="POST" style="display:none;">';
      echo '<input type="hidden" name="rp_csrf_token" value="'.$token.'">';
      echo '</form>';

}

function rp_regenerate_front_end_csrf_token() {

      if( !isset( $_POST['regenerate'] ) ) {

            echo json_encode(array(
                  'status' => false,
                  'message' => 'Invalid request'
            ));
            exit();

      }

      $token = rp_create_csrf_token( 'frontend' );

      echo json_encode(array(
            'status' => true,
            'token' => $token
      ));
      exit();

}

rp_ajax_for_public('rp_regenerate_front_end_csrf_token', 'rp_regenerate_front_end_csrf_token');


function rp_csrf_token_input_field($action = '') {

      $token = rp_create_csrf_token($action);

      echo '<input type="hidden" name="rp_csrf_token" value="'.$token.'">';

}

function rp_load_session_data() {

      $page = $_REQUEST['page'] ?? '';
      
      $repairPluginPages = array(
            'wp_repair_appointments',
            'wp_repair_manage_categories',
            'wp_repair_manage_models',
            'wp_repair_manage_upsales',
            'wp_repair_download_libraries',
            'wp_repair_default_repairs',
            'wp_repair_settings'
      );

      if( in_array($page, $repairPluginPages) ) {

            // check if is ajax request (xmlhttprequest)
            if( strtolower( $_SERVER['HTTP_X_REQUESTED_WITH'] ?? '' ) !== 'xmlhttprequest' ) {

                  rp_fix_addon_switch_settings_on_backend();

            }

            RepairPluginPro\RP_Session_Handler::start();

      }

}

add_action('init', 'rp_load_session_data');

function rp_is_enabled_recaptcha() {

      return rp_get_option('rp_enable_recaptcha', 0);

}

function rp_is_need_to_hide_recaptcha_badge() {

      return rp_get_option('rp_hide_recaptcha_badge', 0);

}

function rp_get_google_recaptcha_site_key() {

      return rp_get_option('rp_recaptcha_site_key', '');

}

function rp_get_google_recaptcha_secret_key() {

      return rp_get_option('rp_recaptcha_secret_key', '');

}

function rp_verify_google_recaptcha() {

      if( !rp_is_recaptcha_correctly_configured() ) {
            
            return TRUE;

      }

      $recaptcha_url = 'https://www.google.com/recaptcha/api/siteverify';
      $recaptcha_secret = rp_get_google_recaptcha_secret_key();
      $recaptcha_response = trim($_POST['recaptcha_response'] ?? '');

      if( empty($recaptcha_response) ) {

            return FALSE;

      }

      $recaptcha = rp_fetch_image_from_base($recaptcha_url . '?secret=' . $recaptcha_secret . '&response=' . $recaptcha_response);

      if( empty($recaptcha) ) {

            return FALSE;

      }

      $recaptcha = json_decode($recaptcha);

      if (!empty($recaptcha) && isset($recaptcha->success) && $recaptcha->success == true && isset($recaptcha->score) &&  $recaptcha->score >= 0.5) {

            return TRUE;

      } else {

            return FALSE;

      }

}

function rp_get_recaptcha_info() {

      echo json_encode(array(
            'success' => true,
            'site_key' => rp_get_google_recaptcha_site_key(),
            'secret_key' => rp_get_google_recaptcha_secret_key(),
            'enable_protection' => rp_is_enabled_recaptcha(),
            'hide_recaptcha_badge' => rp_is_need_to_hide_recaptcha_badge()
      ));

      exit;

}

rp_ajax_for_admin('rp_get_recaptcha_info', 'rp_get_recaptcha_info');


function rp_save_recaptcha_info() {

      // data: {
      //       action:'rp_save_recaptcha_info',
      //       rp_enable_recaptcha: rp_enable_recaptcha,
      //       hide_recaptcha_badge: hide_recaptcha_badge,
      //       recaptcha_site_key: recaptcha_site_key,
      //       recaptcha_secret_key: recaptcha_secret_key
      // },

      rp_verify_csrf_token_for_ajax('styling');

      $rp_enabled_recaptcha = $_POST['rp_enable_recaptcha'] ?? 0;
      $rp_hide_recaptcha_badge = $_POST['hide_recaptcha_badge'] ?? 0;
      $rp_recaptcha_site_key = trim( $_POST['recaptcha_site_key'] ?? '' );
      $rp_recaptcha_secret_key = trim( $_POST['recaptcha_secret_key'] ?? '' );

      // make sure rp_enabled_recaptcha is either 1 or 0
      $rp_enabled_recaptcha = $rp_enabled_recaptcha == 1 ? 1 : 0;

      // make sure rp_hide_recaptcha_badge is either 1 or 0
      $rp_hide_recaptcha_badge = $rp_hide_recaptcha_badge == 1 ? 1 : 0;

      if( rp_get_option('rp_enable_recaptcha', FALSE) === FALSE ) {

            rp_add_option('rp_enable_recaptcha', $rp_enabled_recaptcha);

      } else {

            rp_update_option('rp_enable_recaptcha', $rp_enabled_recaptcha);

      }

      if( rp_get_option('rp_hide_recaptcha_badge', FALSE) === FALSE ) {

            rp_add_option('rp_hide_recaptcha_badge', $rp_hide_recaptcha_badge);

      } else {

            rp_update_option('rp_hide_recaptcha_badge', $rp_hide_recaptcha_badge);

      }

      if( rp_get_option('rp_recaptcha_site_key', FALSE) === FALSE ) {

            rp_add_option('rp_recaptcha_site_key', $rp_recaptcha_site_key);

      } else {

            rp_update_option('rp_recaptcha_site_key', $rp_recaptcha_site_key);

      }

      if( rp_get_option('rp_recaptcha_secret_key', FALSE) === FALSE ) {

            rp_add_option('rp_recaptcha_secret_key', $rp_recaptcha_secret_key);

      } else {

            rp_update_option('rp_recaptcha_secret_key', $rp_recaptcha_secret_key);

      }

      echo json_encode(array(
            'success' => true,
            'correctly_configured' => rp_is_recaptcha_correctly_configured()
      ));

      exit;

}

rp_ajax_for_admin('rp_save_recaptcha_info', 'rp_save_recaptcha_info');


function rp_add_new_delivery_methods_column() {

      /* Added in V1.5.1 */

      if(rp_get_option('rp_added_new_delivery_methods_column', FALSE) === FALSE) {

            rp_add_column_to_table_if_not_exists('rs_planning_and_discount', 'pickup_service', "tinyint(1) NOT NULL DEFAULT '0'");

            rp_add_column_to_table_if_not_exists('rs_planning_and_discount', 'repair_on_location', "tinyint(1) NOT NULL DEFAULT '0'");

            rp_add_option('rp_added_new_delivery_methods_column', '1');

      }

}

add_action('init', 'rp_add_new_delivery_methods_column', 999);

/*
Create RepairPlugin Special Timing Table
*/
function repair_plugin_delivery_timeslots_table(){
      /* Added in V1.5.0 */

    global $rpQuery;

    require_once( ABSPATH .'wp-admin/includes/upgrade.php' );

    $table_rs_delivery_timeslots = $rpQuery->prefix . "rs_delivery_timeslots"; 

    $charset_collate = $rpQuery->get_charset_collate();

    $sql[] = "CREATE TABLE IF NOT EXISTS $table_rs_delivery_timeslots (

      `id` int(11) NOT NULL AUTO_INCREMENT,

      `slot_start` varchar(255) NOT NULL,

      `slot_end` varchar(255) NOT NULL,

      `delivery_method` varchar(255) NOT NULL,

      `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,

      PRIMARY KEY (`id`))  

    $charset_collate";

    dbDelta( $sql );

}

add_action('init', 'repair_plugin_delivery_timeslots_table');

// add default rows in delivery timeslots table
function rp_add_default_delivery_timeslots_rows() {

      global $rpQuery;

      $rs_delivery_timeslots = $rpQuery->prefix . 'rs_delivery_timeslots';

      $is_empty_table = empty($rpQuery->get_row("SELECT * FROM `$rs_delivery_timeslots` LIMIT 1"));

      if( $is_empty_table === TRUE && rp_get_option('rp_added_default_delivery_timeslots_rows', FALSE) === FALSE ) {

            $delivery_methods = array('pickup_service', 'repair_on_location');

            $slots = array(
                  array(
                        'start' => '09:00',
                        'end' => '12:00'
                  ),
                  array(
                        'start' => '12:00',
                        'end' => '15:00'
                  ),
                  array(
                        'start' => '15:00',
                        'end' => '18:00'
                  )
            );

            foreach( $delivery_methods as $method ) {

                  foreach( $slots as $slot ) {

                        $rpQuery->insert($rs_delivery_timeslots, array(
                              'slot_start' => $slot['start'],
                              'slot_end' => $slot['end'],
                              'delivery_method' => $method
                        ));

                  }

            }

            // also setup default available on days if not already
            if( rp_get_option('rp_pickup_service_available_days', FALSE) === FALSE ) {

                  rp_add_option('rp_pickup_service_available_days', json_encode(array(
                        'monday' => 1,
                        'tuesday' => 1,
                        'wednesday' => 1,
                        'thursday' => 1,
                        'friday' => 1,
                        'saturday' => 1,
                        'sunday' => 0
                  )));

            }

            if( rp_get_option('rp_repair_on_location_available_days', FALSE) === FALSE ) {

                  rp_add_option('rp_repair_on_location_available_days', json_encode(array(
                        'monday' => 1,
                        'tuesday' => 1,
                        'wednesday' => 1,
                        'thursday' => 1,
                        'friday' => 1,
                        'saturday' => 1,
                        'sunday' => 0
                  )));

            }

            rp_add_option('rp_added_default_delivery_timeslots_rows', '1');

      }

}

add_action('init', 'rp_add_default_delivery_timeslots_rows', 999);

// Update rs_special_timing structure 
function rp_update_rs_special_timing_structure() {

      if( rp_get_option('rp_updated_rs_special_timing_structure', FALSE) === FALSE ) {

            global $rpQuery;

            // cs_day	
            // cs_open_timing	
            // cs_close_timing
            // is_closed
            // reminder_name
            // location_id_fk

            $rs_special_timing = $rpQuery->prefix . 'rs_special_timing';

            // get all from rs_special_timing
            $all_special_timings = $rpQuery->get_results("SELECT * FROM `$rs_special_timing`");

            // Loop through all special timings and find rows where cs_day, cs_open_timing, cs_close_timing, is_closed, reminder_name all are same
            
            $special_timing_only_unique = array();

            foreach($all_special_timings as $row) {

                  $cs_day = $row->cs_day;
                  $cs_open_timing = $row->cs_open_timing;
                  $cs_close_timing = $row->cs_close_timing;
                  $is_closed = $row->is_closed;
                  $reminder_name = $row->reminder_name;

                  $is_already_exists = FALSE;

                  foreach($special_timing_only_unique as $the_key => $unique_row) {

                        if( $unique_row->cs_day == $cs_day && $unique_row->cs_open_timing == $cs_open_timing && $unique_row->cs_close_timing == $cs_close_timing && $unique_row->is_closed == $is_closed && $unique_row->reminder_name == $reminder_name ) {

                              $is_already_exists = TRUE;

                              $special_timing_only_unique[$the_key]->location_id_fk .= ',' . $row->location_id_fk;

                        }

                  }

                  if( $is_already_exists === FALSE ) {

                        $special_timing_only_unique[] = $row;

                  }


            }

            // delete all rows from rs_special_timing

            $rpQuery->query("DELETE FROM `$rs_special_timing` WHERE 1 = 1");

            $sql = "ALTER TABLE `$rs_special_timing` MODIFY COLUMN `location_id_fk` varchar(255) NULL DEFAULT NULL;";

            $rpQuery->query($sql);

            // insert all new rows again in rs_special_timing
            foreach($special_timing_only_unique as $row) {

                  $rpQuery->insert($rs_special_timing, array(
                        'cs_day' => $row->cs_day,
                        'cs_open_timing' => $row->cs_open_timing,
                        'cs_close_timing' => $row->cs_close_timing,
                        'is_closed' => $row->is_closed,
                        'reminder_name' => $row->reminder_name,
                        'location_id_fk' => $row->location_id_fk
                  ));

            }

            rp_add_option('rp_updated_rs_special_timing_structure', '1');

      }

}

add_action('init', 'rp_update_rs_special_timing_structure', 999);

function rp_get_special_timing($cs_day = '', $loc_id = '') {

      global $rpQuery;

      $rs_special_timing = $rpQuery->prefix . 'rs_special_timing';

      // check if cs_day and location_id_fk already exists
      $all_rows = $rpQuery->get_results("SELECT * FROM `$rs_special_timing` WHERE cs_day = '".rp_escape_sql($cs_day)."';");

      if( empty( $all_rows ) ) {

            return array();
            
      }

      foreach($all_rows as $row) {

            $location_id_fk = $row->location_id_fk;

            $location_id_fk_array = explode(',', $location_id_fk);

            if( array_search($loc_id, $location_id_fk_array) !== FALSE ) {

                  return $row;

            }

      }

      return array();

}

function rp_check_if_special_timing_exists($cs_day = '', $loc_id = '') {

      return !empty( rp_get_special_timing( $cs_day, $loc_id ) );

}

function rp_get_delivery_timeslots( $method = '', $day = '' ) {

      global $rpQuery;

      $rs_delivery_timeslots = $rpQuery->prefix . 'rs_delivery_timeslots';

      $key = 'rp_'.$method.'_available_days';

      $available_days = rp_get_option($key, json_encode(array()));

      $available_days = json_decode($available_days, TRUE);

      $day = strtolower($day);

      if( isset( $available_days[ $day ] ) && (int) $available_days[ $day ] === 1 ) {

            $all_rows = $rpQuery->get_results("SELECT * FROM `$rs_delivery_timeslots` WHERE `delivery_method` = '".rp_escape_sql($method)."' ORDER BY `slot_start` ASC;");

            return $all_rows;


      }

      return array();

}

function rp_add_new_customer_info_column() {

      /* Added in V1.5.1 */

      if(rp_get_option('rp_added_new_customer_info_column', FALSE) === FALSE) {

            rp_add_column_to_table_if_not_exists('rs_customer', 'customer_house_no', 'varchar(255) NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_customer', 'customer_street_address', 'varchar(255) NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_customer', 'customer_city', 'varchar(255) NULL DEFAULT NULL');

            rp_add_column_to_table_if_not_exists('rs_customer', 'customer_zipcode', 'varchar(255) NULL DEFAULT NULL');

            rp_add_option('rp_added_new_customer_info_column', '1');

      }

}

add_action('init', 'rp_add_new_customer_info_column', 999);


// add 'Pickup Service' in noti_type column of rs_notification_types table

function add_pickup_service_in_rs_notification_types() {
      
      /* Added in V1.5.1 */

      global $rpQuery;

      $noti_type = 'Booking Approved (Pickup Service)';

      $rs_notification_types = $rpQuery->prefix . 'rs_notification_types';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notification_types` WHERE `noti_type` = '".rp_escape_sql($noti_type)."' LIMIT 1;");

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notification_types` (`noti_type`, `is_active`) VALUES ('".rp_escape_sql($noti_type)."', '1');");
            
      }
}

add_action('init', 'add_pickup_service_in_rs_notification_types', 999);


// insert into all columns from rs_notifications_and_offers table if noti_type_id_fk 5 doesn't exist

function insert_pickup_service_markup_in_notification_table() {
      
      /* Added in V1.5.1 */

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notifications_and_offers` WHERE `noti_type_id_fk` = '9' LIMIT 1;");

      $subject = 'Subject for Pickup Service Email';

      $message = 'This is message for pickup service email';

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notifications_and_offers` (`noti_subject`, `noti_message`, `noti_type_id_fk`, `is_active`, `date_time`) VALUES ('$subject', '$message', '9', '1', '".date('Y-m-d H:i:s')."');");
            
      }
}

add_action('init', 'insert_pickup_service_markup_in_notification_table', 999);


// add 'Repair on Location' in noti_type column of rs_notification_types table

function add_repair_on_location_in_rs_notification_types() {
      
      /* Added in V1.5.1 */

      global $rpQuery;

      $noti_type = 'Booking Approved (Repair on Location)';

      $rs_notification_types = $rpQuery->prefix . 'rs_notification_types';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notification_types` WHERE `noti_type` = '".rp_escape_sql($noti_type)."' LIMIT 1;");

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notification_types` (`noti_type`, `is_active`) VALUES ('".rp_escape_sql($noti_type)."', '1');");
            
      }
}

add_action('init', 'add_repair_on_location_in_rs_notification_types', 999);


// insert into all columns from rs_notifications_and_offers table if noti_type_id_fk 5 doesn't exist

function insert_on_location_markup_in_notification_table() {
      
      /* Added in V1.5.1 */

      global $rpQuery;

      $rs_notifications_and_offers = $rpQuery->prefix . 'rs_notifications_and_offers';

      $exists = $rpQuery->get_row("SELECT * FROM `$rs_notifications_and_offers` WHERE `noti_type_id_fk` = '10' LIMIT 1;");

      $subject = 'Subject for Repair on Location Email';

      $message = 'This is message for repair on location email';

      if(empty($exists)) {

            $rpQuery->query("INSERT INTO `$rs_notifications_and_offers` (`noti_subject`, `noti_message`, `noti_type_id_fk`, `is_active`, `date_time`) VALUES ('$subject', '$message', '10', '1', '".date('Y-m-d H:i:s')."');");
            
      }
}

add_action('init', 'insert_on_location_markup_in_notification_table', 999);


// if not exists, wp_repair_admin_duplicate_email_status_5,6,7,8,9,10 add to options table using get_option, rp_add_option 

function add_duplicate_email_status_arr_in_options_table() {
      
      /* Added in V1.5.1 */

      $notification_ids = array(
            '5',
            '6',
            '7',
            '8',
            '9',
            '10'
      );

      foreach( $notification_ids as $the_id ) {

            $exists = rp_get_option('wp_repair_admin_duplicate_email_status_'.$the_id, FALSE);

            if( $exists === FALSE ) {

                  rp_add_option('wp_repair_admin_duplicate_email_status_'.$the_id, '1');

            }

      }

}

add_action('init', 'add_duplicate_email_status_arr_in_options_table', 999);


function rp_add_new_translation_by_release( $new_translation_groups = array() ) {

      $new_translation_groups['rp_plugin_translations_v1_5_1'] = array(

            'user_location_zipcode',

            'user_location_house_number',

            'user_location_street_address',

            'user_location_city',

            'let_us_pickup_your_device',

            'pickup_service_description',

            'repair_on_your_location',

            'repair_on_your_location_description',

            'booking_approved_pickup_service_email_subject',

            'booking_approved_pickup_service_email_template',

            'booking_approved_repair_on_location_email_subject',

            'booking_approved_repair_on_location_email_template',

            'do_you_need_help',

            'invalid_license_code',

            'enter_your_license_key_here',

            'if_you_dont_have_license',

            'license_code',

            'license_email',

            'we_will_send_updated_news_of_this_product'

      );

      $new_translation_groups['rp_plugin_translations_v1_5_2'] = array(

            'device_mobile'

      );

      $new_translation_groups['rp_plugin_translations_v1_5_7'] = array(

            'combo_discount_percent_off'

      );

      return $new_translation_groups;

}

add_filter('rp_force_fetch_new_translation_groups', 'rp_add_new_translation_by_release', 10, 1);


function rp_download_new_translations_from_server( $new_translation_keys = array() ) {

      global $rpQuery;
      
      $baselang_class = new RepairPluginPro\Download_Languages($rpQuery);

      $lang_ids = rp_get_uq_id_of_all_rs_languages();

      if( empty( $lang_ids ) ) {

            // No languages available!
            return TRUE;

      }

      $_POST['lang_uq_ids'] = $lang_ids;

      $response = $baselang_class->fetch_repair_base(TRUE, FALSE);

      if( empty( $response ) ) {

            return FALSE;

      }

      return $baselang_class->save_handler_new_translations($response, $new_translation_keys) !== FALSE;

}

function rp_force_fetch_new_translations(){
      
      if( empty( $_REQUEST['action'] ?? '' ) && !wp_doing_ajax() ) {

            return;

      }

      if( !defined('WP_REPAIR_REQUIRE_LICENSE') ) {

            $new_translation_groups = apply_filters('rp_force_fetch_new_translation_groups', array());

            if( empty( $new_translation_groups ) ) {

                  return;

            }

            $uniqueGroups = array();

            $new_translation_keys = array();

            foreach( $new_translation_groups as $group => $groupItems ) {

                  if( array_search( $group, $uniqueGroups ) === FALSE ) {

                        $uniqueGroups[] = $group;

                  }

                  if( rp_get_option( $group, FALSE ) === FALSE ) {

                        foreach( $groupItems as $item ) {

                              if( array_search( trim($item), $new_translation_keys ) === FALSE ) {

                                    $new_translation_keys[] = trim($item);

                              }

                        }

                  }

            }

            if( !empty( $new_translation_keys ) ) {

                  if( rp_download_new_translations_from_server( $new_translation_keys ) !== FALSE ) {

                        // Downloaded correctly!
                        foreach($uniqueGroups as $group) {

                              if( rp_get_option( $group, FALSE ) === FALSE ) {

                                    rp_add_option( $group, '1' );

                              }

                        }

                  }

            }

      }

}

add_action('init', 'rp_force_fetch_new_translations', 999);

// Fix for those sites that already got messed by v1.5.2
function rp_remove_empty_sorting_values_from_options() {

      // delete from $wpdb->prefix . options where option_name like 'rp_default_listing_index_' and (option_value = '' or option_value = '0' or option_value IS NULL);

      if( rp_get_option('rp_remove_empty_sorting_values_from_options', FALSE) === FALSE ) {

            rp_add_option('rp_remove_empty_sorting_values_from_options', '1');

            global $rpQuery;

            $options_table = $rpQuery->prefix . 'options';

            $rpQuery->query("DELETE FROM `$options_table` WHERE `option_name` LIKE 'rp_default_listing_index_%' AND (`option_value` = '' OR `option_value` = '0' OR `option_value` IS NULL);");

      }

}

add_action('init', 'rp_remove_empty_sorting_values_from_options', 999);

function rp_round_to_even_number( $number = 0 ) {

      if( $number == 0 ) {

            return 0;

      }

      if( $number % 2 == 0 ) {

            return $number;

      }

      return $number + 1;

}

function returning_por_repairs( $por_repairs = array(), $order_id = 0 ) {
    
      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $order_repairs = $rpQuery->get_results("SELECT * FROM `$rs_order_repairs` WHERE `o_id_fk` = '{$order_id}';");

      if( !empty( $order_repairs ) ) {

            $r_id_fk = array();

            foreach( $order_repairs as $row ) {

                  $repair_attrs = $rpQuery->get_results("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '{$order_id}' AND `r_id_fk` = '".rp_escape_sql($row->r_id_fk)."';");

                  if( empty( $repair_attrs ) && ((float) $row->repair_price) <= 0 && property_exists( $row, 'pricing_method' ) && $row->pricing_method === 'price_on_request') {

                        $r_id_fk[] = $row->r_id_fk;

                  }

            }

            if( !empty( $r_id_fk ) ) {

                  $r_id_fk = join(',', $r_id_fk);

                  $price_on_request = $rpQuery->get_results("SELECT * FROM `$rs_repair` WHERE `r_id` IN ({$r_id_fk}) AND `is_free` != '1';");

                  $unique_tracking = array();

                  $filtered_items = array();

                  foreach( $por_repairs as $theRepairRow ) {

                        if( array_search( $theRepairRow->r_id, $unique_tracking ) === FALSE ) {

                              $unique_tracking[] = $theRepairRow->r_id;

                              $filtered_items[] = $theRepairRow;

                        }

                  }

                  if( !empty( $price_on_request ) ) {

                        // loop through the price on request items and add them to the list
                        foreach( $price_on_request as $por_row ) {

                              if( array_search( $por_row->r_id, $unique_tracking ) === FALSE ) {

                                    $unique_tracking[] = $por_row->r_id;

                                    $filtered_items[] = $por_row;

                              }

                        }

                  }

                  $por_repairs = $filtered_items;

            }

      }

      return $por_repairs;

}

add_filter('returning_por_repairs', 'returning_por_repairs', 10, 2);


function returning_por_repair_attrs( $por_repairs = array(), $order_id = 0 ) {

      /* Added in V1.4.4 */

      global $rpQuery;

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $order_repairs = $rpQuery->get_results("SELECT * FROM `$rs_order_repair_attr` WHERE `o_id_fk` = '{$order_id}';");

      if( !empty( $order_repairs ) ) {

            $ra_id_fk = array();

            foreach( $order_repairs as $row ) {

                  if( ((float) $row->repair_attr_price) <= 0 && property_exists( $row, 'pricing_method' ) && $row->pricing_method === 'price_on_request' ) {

                        $ra_id_fk[] = $row->ra_id_fk;

                  }

            }

            if( !empty( $ra_id_fk ) ) {

                  $ra_id_fk = join(',', $ra_id_fk);

                  $price_on_request = $rpQuery->get_results("SELECT * FROM `$rs_repair_attr` WHERE `a_id` IN ({$ra_id_fk});");

                  $unique_tracking = array();

                  $filtered_items = array();

                  foreach( $por_repairs as $theRepairRow ) {

                        if( array_search( $theRepairRow->a_id, $unique_tracking ) === FALSE ) {

                              $unique_tracking[] = $theRepairRow->a_id;

                              $filtered_items[] = $theRepairRow;

                        }

                  }

                  if( !empty( $price_on_request ) ) {

                        // loop through the price on request items and add them to the list
                        foreach( $price_on_request as $por_row ) {

                              if( array_search( $por_row->a_id, $unique_tracking ) === FALSE ) {

                                    $unique_tracking[] = $por_row->a_id;

                                    $filtered_items[] = $por_row;

                              }

                        }

                  }

                  $por_repairs = $filtered_items;

            }

      }

      return $por_repairs;

}

add_filter('returning_por_repair_attrs', 'returning_por_repair_attrs', 10, 2);


function rp_appointment_is_dynamic_price( $row = null ) {

      if( property_exists( $row, 'pricing_method' ) && $row->pricing_method === 'dynamic' && property_exists( $row, 'pricing_info' ) && !empty( $row->pricing_info ) ) {

            return TRUE;

      }

      return FALSE;

}

function rp_find_model_by_id( $m_id = 0 ) {

      global $rpQuery;

      $rs_model = $rpQuery->prefix . 'rs_model';

      $rs_brand = $rpQuery->prefix . 'rs_brand';

      $rs_category = $rpQuery->prefix . 'rs_category';

      $m_id = intval( $m_id );

      $sql = "SELECT $rs_model.*, $rs_brand.b_name, $rs_category.c_name FROM `$rs_model` 
      LEFT JOIN `$rs_brand` ON `$rs_brand`.`b_id` = `$rs_model`.`brand_id_fk`
      LEFT JOIN `$rs_category` ON `$rs_category`.`c_id` = `$rs_model`.`device_type`
      WHERE m_id = {$m_id}";

      $model = $rpQuery->get_row( $sql );

      if( !empty( $model ) ) {

            return $model;

      }

      return array();

}


function rp_dp_get_live_skus( $m_uq_id = '', $repair_uqid = '' ) {

      global $rpQuery;
  
      $rs_dp_linking = $rpQuery->prefix . 'rs_dp_linking';

      if( function_exists('rp_get_enabled_suppliers') ) {

            $enabled_suppliers = rp_get_enabled_suppliers();

            $additional_sql = "AND `supplier_id` IN (" . implode( ',', $enabled_suppliers ) . ")";

      } else {

            $additional_sql = '';

      }
  
      $m_uq_id = rp_escape_sql( $m_uq_id );
  
      $repair_uqid = rp_escape_sql( $repair_uqid );
  
      $sql = "SELECT `price`, `sku`, `supplier_id`, `supplier_name` FROM `$rs_dp_linking` WHERE `m_uq_id` = '{$m_uq_id}' AND `repair_uqid` = '{$repair_uqid}' AND `status` = 'published' AND `fetched_status` = 'OK' AND `on_stock` = 'Y'";
  
      $sql .= $additional_sql;

      $result = $rpQuery->get_results( $sql );
  
      if( !empty( $result ) ) {
  
            return $result;
  
      }
  
      return array();
  
}

function rp_get_more_details_of_sku() {

      global $rpQuery;

      $rs_order_repairs = $rpQuery->prefix . 'rs_order_repairs';

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_order_repair_attr = $rpQuery->prefix . 'rs_order_repair_attr';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $row_id = intval($_POST['row_id'] ?? 0);

      $row_type = $_POST['row_type'] ?? '';

      if( !empty( $row_id ) && !empty( $row_type ) ) {

            if( $row_type === 'attr' ) {

                  $id_col = 'ora_id';
                  $table = $rs_order_repair_attr;
                  $id_fk = 'ra_id_fk';
                  $uqid = 'a_uq_id';

            } else {

                  $id_col = 'or_id';
                  $table = $rs_order_repairs;
                  $id_fk = 'r_id_fk';
                  $uqid = 'r_uq_id';

            }

            $row = $rpQuery->get_row("SELECT * FROM `$table` WHERE `$id_col` = '{$row_id}' AND `pricing_method` = 'dynamic';");

            if( !empty( $row ) ) {

                  $o_id_fk = $row->o_id_fk;

                  if( $uqid === 'r_uq_id' ) {
                        $repair_uqid = ( ( rp_find_repair_row_by_id( $row->{$id_fk} )->{$uqid} ) ?? 0 );
                  } else {
                        $repair_uqid = ( ( rp_find_repair_attr_row_by_id( $row->{$id_fk} )->{$uqid} ) ?? 0 );
                  }

                  $pricing_info = $row->pricing_info;

                  // select * from rs_orders where o_id = $o_id_fk
                  $rs_orders = $rpQuery->prefix . 'rs_orders';

                  $order = $rpQuery->get_row("SELECT * FROM `$rs_orders` WHERE `o_id` = '{$o_id_fk}';");

                  if( !empty( $order ) ) {

                        $theModel = rp_find_model_by_id( $order->m_id_fk );

                        $m_uq_id = $theModel->m_uq_id;

                        $modelName = $theModel->c_name . '<span class="model-name-separator">></span>' . $theModel->b_name . '<span class="model-name-separator">></span>' . $theModel->m_name;

                        if( !empty( $m_uq_id ) ) {

                              $live_skus = rp_dp_get_live_skus( $m_uq_id, $repair_uqid );

                              if( empty( $live_skus ) ) {

                                    $live_skus = array();

                              }

                              $rp_dp_finished_cron_processing_time = '';
                              if( rp_get_option('rp_dp_finished_cron_processing', FALSE) !== FALSE ) {
                                    $rp_dp_finished_cron_processing = rp_get_option('rp_dp_finished_cron_processing', FALSE);
                                    if( !empty( $rp_dp_finished_cron_processing ) ) {
                                          $rp_dp_finished_cron_processing_time = '<span class="rp-dp-rounded-time-box">' .file_get_contents(WP_REPAIR_PLUGIN_PATH.'assets/images/history.svg') . trim( _rp_dp_human_readable_difference( date('Y-m-d H:i:s', $rp_dp_finished_cron_processing), date('Y-m-d H:i:s', time()) ) ) . ' ago</span>';
                                    }
                              }

                              $pricing_info = unserialize( $pricing_info );

                              if( isset( $pricing_info['calculated_at_time'] ) && !empty( $pricing_info['calculated_at_time'] ) ) {

                                    $pricing_info['calculated_at_time'] = '<span class="rp-dp-rounded-time-box">' . file_get_contents(WP_REPAIR_PLUGIN_PATH.'assets/images/history.svg') . trim( _rp_dp_human_readable_difference( date('Y-m-d H:i:s', $pricing_info['calculated_at_time']), date('Y-m-d H:i:s', time()) ) ) . ' ago</span>';

                              } else {

                                    $pricing_info['calculated_at_time'] = '';

                              }

                              echo json_encode( array(

                                    'status' => true,

                                    'pricing_info' => $pricing_info,

                                    'live_skus' => $live_skus,

                                    'model_name' => $modelName,

                                    'last_processing' => $rp_dp_finished_cron_processing_time

                              ) );

                              exit;

                        }

                  }

            } 
            
            echo json_encode( array(

                  'status'=> false, 

                  'rp_validation_error' => 'Something went wrong, please refresh the page and try agian.'

            ) );
            
      } else {

            echo json_encode( array(

                  'status'=> false, 

                  'rp_validation_error' => 'Invalid request'

            ) );

      }

      exit;

}

rp_ajax_for_admin('rp_get_more_details_of_sku', 'rp_get_more_details_of_sku');

function _rp_dp_human_readable_difference( $date1 = '', $date2 = '' ) {

    $diff = new \DateTime( $date1 );
    $now = new \DateTime( $date2 );
    $interval = $now->diff( $diff );

    // year, month, day, hour, minute, second
    $y = $interval->y;
    $m = $interval->m;
    $d = $interval->d;
    $h = $interval->h;
    $i = $interval->i;
    $s = $interval->s;

    $response = '';
    
    if( $y > 0 ) {
        $response .= $y . ' year';
        if( $y > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $m > 0 ) {
        $response .= $m . ' month';
        if( $m > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $d > 0 ) {
        $response .= $d . ' day';
        if( $d > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $h > 0 ) {
        $response .= $h . ' hour';
        if( $h > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $i > 0 ) {
        $response .= $i . ' minute';
        if( $i > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    if( $s > 0 ) {
        $response .= $s . ' second';
        if( $s > 1 ) {
            $response .= 's';
        }
        $response .= ' ';
        return $response;
    }

    return $response ?? 'Just now';

}

function _rp_manage_upsales_more_details( $upsales_id = 0 ) {

      global $rpQuery;

      ob_start();

      require WP_REPAIR_PLUGIN_PATH.'html/manage_upsales_more_details.php';

      $content = ob_get_contents();

      ob_end_clean();

      return $content;

}

function rp_manage_upsales_more_details() {

      global $rpQuery;

      $upsales_id = (int) $_POST['upsales_id'] ?? 0;

      $rs_upsales = $rpQuery->prefix . "rs_upsales";

      $up_sales = $rpQuery->get_row("SELECT * FROM $rs_upsales WHERE `upsales_id` = '".rp_escape_sql($upsales_id)."' ORDER BY upsales_position ASC");

      if( !empty( $up_sales ) ) {

            $more_details = _rp_manage_upsales_more_details( $upsales_id );

            echo json_encode(array(

                  'status' => true,

                  'more_details' => $more_details

            ));

            exit;

      }

      echo json_encode(array(

            'status' => false,

            'rp_validation_error' => 'Invalid Upsale ID'

      ));

      exit;

}

rp_ajax_for_admin('rp_manage_upsales_more_details', 'rp_manage_upsales_more_details');

function rp_svg_icon_only( $icon = '' ) {

      $icon_file = WP_REPAIR_PLUGIN_PATH.'assets/images/'. $icon . '.svg';

      if( file_exists( $icon_file ) ) {

            return rp_fetch_image_from_base( $icon_file );

      }

      return '';

}

function rp_svg_icons_loader( $icon = '', $sizes = '' ) {

      $fa_spinning = '';

      if( strpos( $icon, 'fa-spinner' ) !== false ) {

            $fa_spinning = ' fa-spinning';

      }

      $icon_file = WP_REPAIR_PLUGIN_PATH.'assets/images/'. $icon . '.svg';

      if( file_exists( $icon_file ) ) {

            return '<span class="rp-svg-icon '.$sizes.' '.$icon.$fa_spinning.'">'.rp_fetch_image_from_base( $icon_file ).'</span>';

      }

      return '';

}

/*
Add r_badge column to rs_repair table
*/

function add_r_badge_column_to_rs_repair() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair)."' AND COLUMN_NAME = 'r_badge';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair` ADD `r_badge` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_r_badge_column_to_rs_repair', 999);

/*
Add dr_badge column to rs_default_repair table
*/

function add_dr_badge_column_to_rs_default_repair() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair)."' AND COLUMN_NAME = 'dr_badge';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair` ADD `dr_badge` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_dr_badge_column_to_rs_default_repair', 999);

/*
Add a_badge column to rs_repair_attr table
*/

function add_a_badge_column_to_rs_repair_attr() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair_attr)."' AND COLUMN_NAME = 'a_badge';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair_attr` ADD `a_badge` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_a_badge_column_to_rs_repair_attr', 999);

/*
Add da_badge column to rs_default_repair_attr table
*/

function add_da_badge_column_to_rs_default_repair_attr() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair_attr)."' AND COLUMN_NAME = 'da_badge';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair_attr` ADD `da_badge` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_da_badge_column_to_rs_default_repair_attr', 999);

/*
Add a_duration_measure column to rs_repair_attr table
*/

function add_a_duration_measure_column_to_rs_repair_attr() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair_attr)."' AND COLUMN_NAME = 'a_duration_measure';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair_attr` ADD `a_duration_measure` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_a_duration_measure_column_to_rs_repair_attr', 999);

/*
Add da_duration_measure column to rs_default_repair_attr table
*/

function add_da_duration_measure_column_to_rs_default_repair_attr() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair_attr)."' AND COLUMN_NAME = 'da_duration_measure';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair_attr` ADD `da_duration_measure` varchar(255) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_da_duration_measure_column_to_rs_default_repair_attr', 999);

/*
Add a_duration column to rs_repair_attr table
*/

function add_a_duration_column_to_rs_repair_attr() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_repair_attr)."' AND COLUMN_NAME = 'a_duration';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_repair_attr` ADD `a_duration` int(11) NOT NULL DEFAULT 0;");
            
      }
}

add_action('init', 'add_a_duration_column_to_rs_repair_attr', 999);

/*
Add da_duration column to rs_default_repair_attr table
*/

function add_da_duration_column_to_rs_default_repair_attr() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_default_repair_attr)."' AND COLUMN_NAME = 'da_duration';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_default_repair_attr` ADD `da_duration` int(11) NOT NULL DEFAULT 0;");
            
      }
}

add_action('init', 'add_da_duration_column_to_rs_default_repair_attr', 999);

function rp_fix_attribute_duration_field_first_time() {

      if( rp_get_option('rp_fix_attribute_duration_field_first_time', FALSE) !== FALSE ) {

            return;

      }

      rp_add_option('rp_fix_attribute_duration_field_first_time', '1');

      // It will work only once

      global $rpQuery;

      $rs_default_repair = $rpQuery->prefix . 'rs_default_repair';

      $rs_repair = $rpQuery->prefix . 'rs_repair';

      $rs_default_repair_attr = $rpQuery->prefix . 'rs_default_repair_attr';

      $rs_repair_attr = $rpQuery->prefix . 'rs_repair_attr';

      // SELECT * FROM $rs_default_repair
      $default_repairs = $rpQuery->get_results("SELECT * FROM `$rs_default_repair`;");

      if( !empty( $default_repairs ) ) {

            foreach( $default_repairs as $row ) {

                  $duration = $row->dr_duration;

                  $duration_measure = $row->dr_duration_measure;

                  $rpQuery->query("UPDATE `$rs_default_repair_attr` SET `da_duration` = '{$duration}', `da_duration_measure` = '{$duration_measure}' WHERE `da_duration` = '0' AND (`da_duration_measure` = '' OR `da_duration_measure` IS NULL) AND `dr_id_fk` = '{$row->dr_id}';");

            }

      }

      // SELECT * FROM $rs_repair
      $repairs = $rpQuery->get_results("SELECT * FROM `$rs_repair`;");

      if( !empty( $repairs ) ) {

            foreach( $repairs as $row ) {

                  $duration = $row->r_duration;

                  $duration_measure = $row->r_duration_measure;

                  $rpQuery->query("UPDATE `$rs_repair_attr` SET `a_duration` = '{$duration}', `a_duration_measure` = '{$duration_measure}' WHERE `a_duration` = '0' AND (`a_duration_measure` = '' OR `a_duration_measure` IS NULL) AND `r_id_fk` = '{$row->r_id}';");

            }

      }

}

add_action('init', 'rp_fix_attribute_duration_field_first_time', 9999);

/*
Add combo_dtype column to rs_orders table
*/

function add_combo_dtype_column_to_rs_orders() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'combo_dtype';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `combo_dtype` varchar(32) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_combo_dtype_column_to_rs_orders', 999);

/*
Add combo_damount column to rs_orders table
*/

function add_combo_damount_column_to_rs_orders() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'combo_damount';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `combo_damount` varchar(32) NULL DEFAULT NULL;");
            
      }
}

add_action('init', 'add_combo_damount_column_to_rs_orders', 999);

/*
Add combo_dstatus column to rs_orders table
*/

function add_combo_dstatus_column_to_rs_orders() {
      /* Added in V1.5.7 */

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $exists = $rpQuery->query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".DB_NAME."' AND TABLE_NAME='".rp_escape_sql($rs_orders)."' AND COLUMN_NAME = 'combo_dstatus';");

      if(empty($exists)) {

            $rpQuery->query("ALTER TABLE `$rs_orders` ADD `combo_dstatus` int(11) NOT NULL DEFAULT 0;");
            
      }
}

add_action('init', 'add_combo_dstatus_column_to_rs_orders', 999);

function rp_fix_order_discount_fields_first_time() {

      if( rp_get_option('rp_fix_order_discount_fields_first_time', FALSE) !== FALSE ) {

            return;

      }

      rp_add_option('rp_fix_order_discount_fields_first_time', '1');

      // It will work only once

      global $rpQuery;

      $rs_orders = $rpQuery->prefix . 'rs_orders';

      $all_orders = $rpQuery->get_results("SELECT * FROM `$rs_orders`;");

      if( !empty( $all_orders ) ) {

            foreach( $all_orders as $row ) {

                  if( $row->combo_dstatus == '1' ) {

                        continue;

                  }

                  $discount = $row->combo_discount;

                  if( $discount != '' ) {

                        $discount = (float) $discount;

                        if( $discount == 0 || $discount == 0.00 ) {

                              $discount = 0;

                        }

                  } else {

                        $discount = 0;

                  }

                  if( $discount != 0 ) {

                        list( $rpCart, $calculations ) = rp_get_latest_order_calculations( $row->o_id );

                        $total_items = $rpCart->calculate_combo_discount_items();

                        $discount = round( ($discount / $total_items), 2 );

                        $rpQuery->query("UPDATE `$rs_orders` SET `combo_damount` = '{$discount}', `combo_dtype` = 'Fixed', `combo_dstatus` = '1' WHERE `o_id` = '{$row->o_id}';");

                  }

            }

      }

}

add_action('init', 'rp_fix_order_discount_fields_first_time', 9999);

// load dynamic pricing addon
require_once WP_REPAIR_PLUGIN_PATH . 'dynamic_pricing.php';

// load dynamic pricing addon
require_once WP_REPAIR_PLUGIN_PATH . 'enhanced_locations.php';

new Repairplugin();
new RP_DynamicPricingAddon();
new RP_EnhancedLocationsAddon();
